#include <qspinbox.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qfont.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qdialog.h>
#include <qfiledialog.h>
#include <qinputdialog.h>
#include <qtextstream.h>
#include <qgrid.h>
#include <qrect.h> 
#include <qframe.h> 
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qgroupbox.h>
#include <qhbox.h>
#include <qaccel.h>
#include <qfileinfo.h> 
#include <qdir.h>
#include <qlistbox.h>
#include <qpixmap.h> 
#include <qkeycode.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qtoolbar.h>
#include <qtoolbutton.h> 
#include <qprinter.h>
#include <qpaintdevicemetrics.h>
#include <qcolordialog.h>
#include <qfontdialog.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qiconset.h>
#include <qarray.h>
#include <qvbox.h>
#include <qtabdialog.h>
#include <qbuttongroup.h>
#include <qregexp.h> 
#include <qcombobox.h>
#include <qvaluelist.h>
#include <qpen.h>
#include <qcolor.h>
#include <qcursor.h>
#include <qpixmap.h>
#include <qpixmapcache.h> 
#include <qstylesheet.h> 
#include <qdatetime.h> 
#include <qwmatrix.h> 
#include <qlabel.h> 
#include <qimage.h>
#include <qmessagebox.h>
#include <qclipboard.h>
#include <qdragobject.h>
#include <qbitmap.h>

#include "graph.h"
#include "fit.h"
#include "ImageMarker.h"
#include "LegendMarker.h"

#include "parser/muParser.h"

#include <qwt_plot.h>
#include <qwt_painter.h>
#include <qwt_curve.h>
#include <qwt_scldraw.h>
#include <qwt_plot_canvas.h>
#include <qwt_plot_layout.h>
#include <qwt_plot_zoomer.h>
#include <qwt_autoscl.h>
#include <qwt_scale.h>
#include <qwt_marker.h>
#include <qwt_text.h>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>

#include <gsl/gsl_vector.h>
#include <gsl/gsl_blas.h>
#include <gsl/gsl_multifit_nlin.h>
#include <gsl/gsl_fit.h>
#include <gsl/gsl_sort.h>
#include <gsl/gsl_sort_vector.h>
#include <gsl/gsl_multifit.h>
#include <gsl/gsl_math.h>
#include <gsl/gsl_spline.h>
#include <gsl/gsl_interp.h>
#include <gsl/gsl_fft_real.h>
#include <gsl/gsl_fft_halfcomplex.h>
#include <gsl/gsl_vector.h>
#include <gsl/gsl_histogram.h>
#include <gsl/gsl_statistics.h>

extern "C" {	
size_t *ivector(long nl, long nh);
void free_ivector(size_t *v, long nl, long nh);

double *vector(long nl, long nh);
void free_vector(double *v, long nl, long nh);
	
double **matrix(long nrl, long nrh, long ncl, long nch);
void free_matrix(double **m, long nrl, long nrh, long ncl, long nch);

int expd3_fdf (const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J);
int expd3_df (const gsl_vector * x, void *params,
         gsl_matrix * J);
int expd3_f (const gsl_vector * x, void *params,
        gsl_vector * f);
	
int expd2_fdf (const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J);
int expd2_df (const gsl_vector * x, void *params,
         gsl_matrix * J);
int expd2_f (const gsl_vector * x, void *params,
        gsl_vector * f);

int expb_fdf (const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J);
int expb_df (const gsl_vector * x, void *params,
         gsl_matrix * J);
int expb_f (const gsl_vector * x, void *params,
        gsl_vector * f);

int expg_fdf (const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J);
int expg_df (const gsl_vector * x, void *params,
         gsl_matrix * J);
int expg_f (const gsl_vector * x, void *params,
        gsl_vector * f);

int gauss_fdf (const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J);
int gauss_df (const gsl_vector * x, void *params,
         gsl_matrix * J);
int gauss_f (const gsl_vector * x, void *params,
        gsl_vector * f);

int lorentz_fdf (const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J);
int lorentz_df (const gsl_vector * x, void *params,
         gsl_matrix * J);
int lorentz_f (const gsl_vector * x, void *params,
        gsl_vector * f);
	
int user_function(const gsl_vector * x, void *params, gsl_vector * f);
int user_function_jacob(const gsl_vector * x, void *params,gsl_matrix * J);
int user_function_fdf(const gsl_vector * x, void *params,gsl_vector * f, gsl_matrix * J);

void polint(double xa[], double ya[], int n, double x, double *y, double *dy);
int print_state (size_t iter, gsl_multifit_fdfsolver * s);//for debugging purposes
}

void Graph::updateHistogram(WksheetWindow* w, const QString& curveName, int curve, bool automatic, double binSize, double begin, double end)
{
int i;
long curveID = keys[curve];
QwtPlotCurve *cv= myPlot->curve (curveID);
if (!cv)
	return;

curveLayout cl=layout[curve];
if (cl.autoBinning && automatic)
	return;

int ycol=w->colIndex(curveName);
int r=w->tableRows();
	
QArray<double> Y(1);
int it=0;
for (i = 0;i<r;i++ )
	{
	QString yval=w->text(i,ycol);
	if (!yval.isEmpty())
		{
		it++;
		Y.resize(it);
		Y[it-1]=yval.toDouble();
		}
	}

if(it<2 || (it==2 && Y[0] == Y[1]))
	{//non valid histogram
	QArray<double> X(2); 
	Y.resize(2);
	for (i = 0;i<2;i++ )
		{
		Y[i] = 0;
		X[i] = 0;
		}

	myPlot->setCurveData(curveID, X, Y, 2);
	myPlot->replot(); 
	updateScale();
	return;
	}

//size_t n;
int n;
gsl_histogram * h; 

if (automatic)
	{
    n=10;
	h = gsl_histogram_alloc (n); 

	if (!h)  
		return;

	gsl_vector *v;
	v = gsl_vector_alloc (it);
	for (i = 0;i<it;i++ )
		gsl_vector_set (v, i, Y[i]);

	double min, max;
	gsl_vector_minmax (v, &min, &max);
	gsl_vector_free (v);

	double from=floor(min);
	double to=ceil(max);

	gsl_histogram_set_ranges_uniform (h, floor(min), ceil(max));

	cl.autoBinning=1;
	cl.binSize=(double)(to-from)/(double)n;
	cl.histBegin=from;
	cl.histEnd=to;
	}
else
	{
	n=int((end - begin)/binSize+1);
	h = gsl_histogram_alloc (n); 
	if (!h)  
		return;

	double *range;
	range=vector(0,n+1);

	for (i = 0;i<=n+1;i++ )
		range[i]=begin+i*binSize;

	gsl_histogram_set_ranges (h, range, n+1);

	free_vector(range,0,n+1);

	cl.autoBinning=0;
	cl.binSize=binSize;
	cl.histBegin=begin;
	cl.histEnd=end;
	}

layout[curve]=cl;

for (i = 0;i<it;i++ )
	gsl_histogram_increment (h, Y[i]);

QArray<double> X(n); //stock ranges (x) and bins (y)
Y.resize(n);

for (i = 0;i<n;i++ )
	{
	Y[i] = gsl_histogram_get (h, i);
	double lower, upper;
	gsl_histogram_get_range (h, i, &lower, &upper);
	X[i] = lower;
	}

myPlot->setCurveData(curveID, X, Y, n);
gsl_histogram_free (h);
	
for (i=0;i<4;i++)
		myPlot->setAxisAutoScale(i);

myPlot->replot(); 
updateScale();
}

void Graph::updateHistogram(WksheetWindow* w, const QString& curveName, int curve)
{
int i;
long curveID = keys[curve];
QwtPlotCurve *cv= myPlot->curve (curveID);
if (!cv)
	return;

int ycol=w->colIndex(curveName);
int r=w->tableRows();
	
QArray<double> Y(1);
int it=0;
for (i = 0;i<r;i++ )
	{
	QString yval=w->text(i,ycol);
	if (!yval.isEmpty())
		{
		it++;
		Y.resize(it);
		Y[it-1]=yval.toDouble();
		}
	}

if(it<2 || (it==2 && Y[0] == Y[1]))
	{//non valid histogram data
	QArray<double> X(2); 
	Y.resize(2);
	for (i = 0;i<2;i++ )
		{
		Y[i] = 0;
		X[i] = 0;
		}

	myPlot->setCurveData(curveID, X, Y, 2);
	myPlot->replot(); 
	updateScale();
	return;
	}

curveLayout cl=layout[curve];

bool automatic = cl.autoBinning;
double binSize = cl.binSize;
double begin = cl.histBegin;
double end = cl.histEnd;

int n;
gsl_histogram * h; 

if (automatic)
	{
    n=10;
	h = gsl_histogram_alloc (n); 
	if (!h)  
		return;

	gsl_vector *v;
	v = gsl_vector_alloc (it);
	for (i = 0;i<it;i++ )
		gsl_vector_set (v, i, Y[i]);

	double min, max;
	gsl_vector_minmax (v, &min, &max);
	gsl_vector_free (v);

	double from=floor(min);
	double to=ceil(max);

	gsl_histogram_set_ranges_uniform (h, floor(min), ceil(max));

	cl.binSize=(double)(to-from)/(double)n;
	cl.histBegin=from;
	cl.histEnd=to;
	layout[curve]=cl;
	}
else
	{
	n=int((end - begin)/binSize+1);
	h = gsl_histogram_alloc (n); 
	if (!h)  
		return;

	double *range;
	range=vector(0,n+1);

	for (i = 0;i<=n+1;i++ )
		range[i]=begin+i*binSize;

	gsl_histogram_set_ranges (h, range, n+1);
	free_vector(range,0,n+1);
	}

for (i = 0;i<it;i++ )
	gsl_histogram_increment (h, Y[i]);

QArray<double> X(n); //stock ranges (x) and bins (y)
Y.resize(n);
for (i = 0;i<n;i++ )
	{
	Y[i] = gsl_histogram_get (h, i);
	double lower, upper;
	gsl_histogram_get_range (h, i, &lower, &upper);
	X[i] = lower;
	}

myPlot->setCurveData(curveID, X, Y, n);
gsl_histogram_free (h);
myPlot->replot(); 
updateScale();
}

QString Graph::showHistogramStats(WksheetWindow* w, const QString& curveName, int curve)
{
int i;
long curveID = keys[curve];
QwtPlotCurve *cv= myPlot->curve (curveID);
if (!cv)
	return "";

int ycol=w->colIndex(curveName);
int r=w->tableRows();
	
QArray<double> Y(1);
int it=0;
for (i = 0;i<r;i++ )
	{
	QString yval=w->text(i,ycol);
	if (!yval.isEmpty())
		{
		it++;
		Y.resize(it);
		Y[it-1]=yval.toDouble();
		}
	}

if(it<2 || (it==2 && Y[0] == Y[1]))
	{
	//non valid histogram data
	QMessageBox::critical(0,tr("Error - QtiPlot"),"Your data is not valid. You need at least two different points for a histogram!");
	return "";
	}

curveLayout cl=layout[curve];

bool automatic = cl.autoBinning;
double binSize = cl.binSize;
double begin = cl.histBegin;
double end = cl.histEnd;

int n;
gsl_histogram * h; 

if (automatic)
	{
    n=10;
	h = gsl_histogram_alloc (n); 
	if (!h)  
		return "";

	gsl_vector *v;
	v = gsl_vector_alloc (it);
	for (i = 0;i<it;i++ )
		gsl_vector_set (v, i, Y[i]);

	double min, max;
	gsl_vector_minmax (v, &min, &max);
	gsl_vector_free (v);

	gsl_histogram_set_ranges_uniform (h, floor(min), ceil(max));
	}
else
	{
	n=int((end - begin)/binSize+1);
	h = gsl_histogram_alloc (n); 
	if (!h)  
		return "";

	double *range;
	range=vector(0,n+1);

	for (i = 0;i<=n+1;i++ )
		range[i]=begin+i*binSize;

	gsl_histogram_set_ranges (h, range, n+1);
	free_vector(range,0,n+1);
	}

for (i = 0;i<it;i++ )
	gsl_histogram_increment (h, Y[i]);

QArray<double> X(n); //stock ranges (x) and bins (y)
Y.resize(n);
for (i = 0;i<n;i++ )
	{
	Y[i] = gsl_histogram_get (h, i);
	double lower, upper;
	gsl_histogram_get_range (h, i, &lower, &upper);
	X[i] = lower;
	}

double h_sum=0.0; 
for (i = 0;i<n;i++ )
	h_sum+=Y[i];

double sum=0.0;
QString text="Bins\tCounts\tSum\tPercent\n";
for (i = 0;i<n;i++ )
	{
	sum+=Y[i];
	text+=QString::number(X[i])+"\t";
	text+=QString::number(Y[i])+"\t";
	text+=QString::number(sum)+"\t";
	text+=QString::number(sum/h_sum*100)+"\n";
	}

QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();
  
QString info=date+"\t"+"Histogram and Probabilities for: "+curveName+"\n";
info+="Mean = "+QString::number(gsl_histogram_mean (h))+"\t";
info+="Standard Deviation = "+QString::number(gsl_histogram_sigma (h))+"\n";
info+="Minimum = "+QString::number(gsl_histogram_min_val(h))+"\t";
info+="Maximum = "+QString::number(gsl_histogram_max_val(h))+"\t";
info+="Bins = "+QString::number(gsl_histogram_bins (h))+"\n";
info+="-------------------------------------------------------------\n";
gsl_histogram_free (h);

QString caption="Bins-"+curveName;
emit createHistogramTable(caption, n, 4, text);
return info;
}


void Graph::initHistogram(long curveID, const QArray<double>& Y, int it)
{
int i;
if(it<2 || (it==2 && Y[0] == Y[1]))
	{//non valid histogram data
	QArray<double> x(2),y(2); 
	for (i = 0;i<2;i++ )
		{
		y[i] = 0;
		x[i] = 0;
		}

	myPlot->setCurveData(curveID, x, y, 2);
	myPlot->replot(); 
	updateScale();
	return;
	}

int n=10;//default value
QArray<double> x(n),y(n); //stock ranges (x) and bins (y)
gsl_histogram * h = gsl_histogram_alloc (n); 
if (!h)  
	return;

gsl_vector *v;
v = gsl_vector_alloc (it);
for (i = 0;i<it;i++ )
	gsl_vector_set (v, i, Y[i]);

double min, max;
gsl_vector_minmax (v, &min, &max);
gsl_vector_free (v);

double from=floor(min);
double to=ceil(max);

gsl_histogram_set_ranges_uniform (h, floor(min), ceil(max));

for (i = 0;i<it;i++ )
	{
    gsl_histogram_increment (h, Y[i]);
	}

for (i = 0;i<n;i++ )
	{
	y[i] = gsl_histogram_get (h, i);
	double lower, upper;
	gsl_histogram_get_range (h, i, &lower, &upper);
	x [i] = lower;
	}

curveLayout cl=layout[cOnPlot-1];
cl.autoBinning=1;
cl.binSize=(double)(to-from)/(double)n;
cl.histBegin=(double)from;
cl.histEnd=(double)to;
layout[cOnPlot-1]=cl;

myPlot->setCurveData(curveID, x, y, n);
gsl_histogram_free (h);
}

void Graph::fft(long curveKey, const QString & FFT)
{
/* Vasilis code*/
	
QwtPlotCurve *curv=myPlot->curve(curveKey);
if (!curv)
	return;

size_t i,j,n;
if (rangeSelectorsEnabled)
	n=abs(endPoint-startPoint)+1;
else
	n=curv->dataSize();

double *y,*x, *result;
y=vector(0,n-1);
x=vector(0,n-1);
result=vector(0,n-1);

// This is the data to be analysed 
if (rangeSelectorsEnabled)
	{
	if (startPoint<endPoint)
  		{
  		for (i = 0; i < n; i++)
			{
			x[i] = curv->x(i+startPoint);
      			y[i] = curv->y(i+startPoint);
			}
  		}
 	else
  		{
  		for (i = 0; i < n; i++)
			{
			x[i]=curv->x(i+endPoint);
      			y[i]=curv->y(i+endPoint);
			}
  		}
	}
else
	{
	for (i = 0; i < n; i++)
		{
		x[i]=curv->x(i);
      		y[i]=curv->y(i);
		}
	}
gsl_fft_real_workspace *work;
work=gsl_fft_real_workspace_alloc(n);
if(FFT=="fft_for")
	{
	gsl_fft_real_wavetable * real;
	real=gsl_fft_real_wavetable_alloc(n);
	gsl_fft_real_transform(y,1,n,real,work);
	for(i=0;i<n;i++)
		result[i]=y[i];
	gsl_fft_real_wavetable_free(real);
	}
else if(FFT=="fft_inv")
	{
	gsl_fft_halfcomplex_wavetable *hc;
	hc=gsl_fft_halfcomplex_wavetable_alloc(n);
	gsl_fft_halfcomplex_inverse(y,1,n,hc,work);
	for(i=0;i<n;i++)
		result[i]=y[i];
	gsl_fft_halfcomplex_wavetable_free(hc);
	}
gsl_fft_real_workspace_free(work);

QString label;
if(FFT=="fft_for")
	label="FFT-Forward";
else if(FFT=="fft_inv")
	label="FFT-Inverse";

label+=QString::number(fitID);
QString caption=label;
label+="_2";
long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(red,1)); 

label=caption+"_1(X),"+label+"(Y)";
associations<<label;

cOnPlot++;
keys.resize(cOnPlot);
keys[cOnPlot-1]=curveID;
	
if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		mrk->setText(legendText());
	}

	myPlot->setCurveData(curveID,x,result,n);
	QString text="1\t2\n";
	for (j=0;j<n;j++)
	{
	text+=QString::number(x[j]);
	text+="\t";
	text+=QString::number(result[j]);
	text+="\n";
	}
	free_vector(x,0,n-1); free_vector(y,0,n-1); free_vector(result,0,n-1);
	
	emit createTable(caption,n,2,text);
	
	for (i=0;i<4;i++)
		{
		myPlot->setAxisAutoScale(i);
		myPlot->setAxisOptions (i,QwtAutoScale::Floating);
		}
	myPlot->replot();
	updateScale();

	curveLayout cl = initCurveLayout();
	layout.resize(cOnPlot);
	layout[cOnPlot-1]=cl;
}

void Graph::interpolation(long curveKey, const QString & Interp)
{
QwtPlotCurve *curv=myPlot->curve(curveKey);
if (!curv)
	return;

size_t i,n;
if (rangeSelectorsEnabled)
	n=abs(endPoint-startPoint)+1;
else
	n=curv->dataSize();

if (Interp=="interlinear" && (int)n<3)
	{
	QMessageBox::critical(this,"QtiPlot - Warning",
				"You need at least 3 points to interpolate! Operation aborted!");
	return;
	}
else if (Interp=="intercubic" && (int)n<4)
	{
	QMessageBox::critical(this,"QtiPlot - Warning",
				"You need at least 4 points to interpolate! Operation aborted!");
	return ;
	}
else if (Interp=="akima" && (int)n<5)
	{
	QMessageBox::critical(this,"QtiPlot - Warning",
				"You need at least 5 points to interpolate! Operation aborted!");
	return;
	}
	
double *x,*y;
x=vector(0,n-1);
y=vector(0,n-1);

// This is the data to be analysed 
if (rangeSelectorsEnabled)
	{
	if (startPoint<endPoint)
  		{
  		for (i = 0; i < n; i++)
    			{
	  			x[i]=curv->x(i+startPoint);
      			y[i] = curv->y(i+startPoint);
    			}
  		}
 	else
  		{
  		for (i = 0; i < n; i++)
    			{
	  			x[i]=curv->x(i+endPoint);
      			y[i]=curv->y(i+endPoint);
    			}
  		}
	}
else
	{
	for (i = 0; i < n; i++)
    			{
	  			x[i]=curv->x(i);
      			y[i]=curv->y(i);
    			}
	}
//unsigned int *p;
size_t *p;
p=ivector(0,n-1);

double *ytemp,*xtemp;
xtemp=vector(0,n-1);
ytemp=vector(0,n-1);
	// Sort the elements. The unfortune thing here is that we do not know whether they are
	// sorted or not
	// Use GSl libraries for that
	gsl_sort_index(p,x,1,n); // Find the permutation
	//Sort y
	ytemp=vector(0,n-1);
	for (i=0;i<n;i++)
		ytemp[i]=y[p[i]];
	for (i=0;i<n;i++)
		y[i]=ytemp[i];
	//sort x
	xtemp=vector(0,n-1);
	for (i=0;i<n;i++)
		xtemp[i]=x[p[i]];
	for (i=0;i<n;i++)
		x[i]=xtemp[i];	
	// free the auxilary vectors
	free_vector(ytemp,0,n-1);free_vector(xtemp,0,n-1); free_ivector(p,0,n-1);
	
// Do the interpolation, use GSL libraries for that
	gsl_interp_accel *acc= gsl_interp_accel_alloc ();
	const gsl_interp_type *method;
	if(Interp=="interlinear")
		method=gsl_interp_linear;
	else if(Interp=="intercubic")
		method=gsl_interp_cspline;
	else if(Interp=="akima")
		method=gsl_interp_akima;
		
	gsl_spline *interp = gsl_spline_alloc (method, n);
        
	gsl_spline_init (interp, x, y, n);

QString label;
if(Interp=="interlinear")
	label="LinearInt";
else if(Interp=="intercubic")
	label="CubicInt";
else if(Interp=="akima")
	label="AkimaInt";

label+=QString::number(fitID);
QString caption=label;
label+="_2";
long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(red,1)); 

label=caption+"_1(X),"+label+"(Y)";
associations<<label;

cOnPlot++;
keys.resize(cOnPlot);
keys[cOnPlot-1]=curveID;
	
if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		mrk->setText(legendText());
	}

	int n1,j,k,l;
	double step;
	if(n>1000) 
		{
		n1=n;
		k=0;
		}
	else if(n<1000)	
	{
	k=1000/n+1; //Plot roughly about 100 points
	n1=k*(n-1)+n; // So that between two old consecutive points we have k new points
	}
	
	free_vector(y,0,n-1);
	xtemp=vector(0,n1-1);y=vector(0,n1-1);
	l=1;
	xtemp[0]=x[0];
	y[0]=gsl_spline_eval (interp,x[0], acc);
	for (i=0;i<(n-1);i++)
		{
		step=(x[i+1]-x[i])/(k+1);
		for (j=1;j<(k+2);j++)
			{
			xtemp[l]=x[i]+j*step;
			y[l]=gsl_spline_eval (interp,xtemp[l], acc);
			l++;
			}
		}
	myPlot->setCurveData(curveID,xtemp,y,n1);
	QString text="1\t2\n";
	for (j=0;j<n1;j++)
	{
	text+=QString::number(xtemp[j]);
	text+="\t";
	text+=QString::number(y[j]);
	text+="\n";
	}
	free_vector(x,0,n-1); free_vector(y,0,n1-1); free_vector(xtemp,0,n1-1);
	gsl_spline_free (interp);
    	gsl_interp_accel_free (acc);
	
	emit createTable(caption,n1,2,text);
	
	for (i=0;i<4;i++)
		{
		myPlot->setAxisAutoScale(i);
		myPlot->setAxisOptions (i,QwtAutoScale::Floating);
		}
	myPlot->replot();
	updateScale();

	curveLayout cl = initCurveLayout();
	layout.resize(cOnPlot);
	layout[cOnPlot-1]=cl;
}

void Graph::diffCurve(long curveKey)
{
QwtPlotCurve *curv=myPlot->curve(curveKey);
if (!curv)
	return;

int i,n;
if (rangeSelectorsEnabled)
	n=abs(endPoint-startPoint)+1;
else
	n=curv->dataSize();
	
double *x,*y,*result;
x=vector(0,n-1);
y=vector(0,n-1);
result=vector(0,n-2);

// This is the data to be analysed 
if (rangeSelectorsEnabled)
	{
	if (startPoint<endPoint)
  		{
  		for (i = 0; i < n; i++)
    			{
	  			x[i]=curv->x(i+startPoint);
      			y[i] = curv->y(i+startPoint);
    			}
  		}
 	else
  		{
  		for (i = 0; i < n; i++)
    			{
	  			x[i]=curv->x(i+endPoint);
      			y[i]=curv->y(i+endPoint);
    			}
  		}
	}
else
	{
	for (i = 0; i < n; i++)
    			{
	  			x[i]=curv->x(i);
      			y[i]=curv->y(i);
    			}
	}
	
for (i = 1; i < n-1; i++)
	 result[i]=0.5*((y[i+1]-y[i])/(x[i+1]-x[i]) + (y[i]-y[i-1])/(x[i]-x[i-1]));
	
QString text="x\tderivative\n";
for (i = 1; i < n-1; i++)
    {
	text+=QString::number(x[i]);
	text+="\t";
	text+=QString::number(result[i]);
	text+="\n";
	}
	
emit createTable(curv->title()+"-Derivative",n-2,2,text);
free_vector(x,0,n-1);free_vector(y,0,n-1);free_vector(result,0,n-2);
}

QString Graph::integrateCurve(long curveKey, int order, int iter, double tol, double low, double up)
{
QwtPlotCurve *curv=myPlot->curve(curveKey);
if (!curv)
	return "";

int i, size=curv->dataSize();
int iStart=0, iEnd=size-1;

	for (i=0;i<size;i++)
		{
		if (curv->x(i)>=low)
			{
			iStart=i;
			break;
			}
		}
	for (i=iEnd;i>=0;i--)
		{
		if (curv->x(i)<=up)
			{
			iEnd=i;
			break;
			}
		}
	
const int n=abs(iEnd-iStart)+1;	
if (n<2)
	{
	QMessageBox::critical(this,"QtiPlot - Integration error",
				"You need at least 2 points to integrate! Integration aborted!");
	return "";
	}
	
double *x,*y;
x=vector(0,n-1);
y=vector(0,n-1);

gsl_vector *Y;
Y = gsl_vector_alloc (n);

bool noBijection = false;
double pr_x = 0;
if (low<up)
{
for (i = 0; i <n; i++)
    {
	x[i]=curv->x(i+iStart);
	if (x[i] == pr_x)
		{
		noBijection = true;
		break;
		}
		
	y[i] = curv->y(i+iStart);// This is the data to be fitted 
	gsl_vector_set (Y, i, y[i]);
	pr_x = x[i];
	}
}
else
{
for (i = 0; i <n; i++)
    {
	x[i]=curv->x(i+iEnd);
	if (x[i] == pr_x)
		{
		noBijection = true;
		break;
		}
	
	y[i] = curv->y(i+iEnd);// This is the data to be fitted 
	gsl_vector_set (Y, i, y[i]);
	pr_x = x[i];
	}
}

if (noBijection)
	{
	QMessageBox::critical(this,"QtiPlot - Integration error",
				"Several points have the same x value causing divisions by zero, integration aborted!");
	return "";
	}
	
//Find the x that has the maximum y
 int maxID=gsl_vector_max_index (Y);
  gsl_vector_free (Y);
	
// Sort the vectors, so that they can be interpolated
size_t *p;
p=ivector(0,n-1);

double *ytemp,*xtemp;
xtemp=vector(0,n-1);
ytemp=vector(0,n-1);
	// Use GSl libraries for that
gsl_sort_index(p,x,1,n); // Find the permutation
	//Sort y
ytemp=vector(0,n-1);
for (i=0;i<n;i++)
	ytemp[i]=y[p[i]];
for (i=0;i<n;i++)
	y[i]=ytemp[i];
	//sort x
xtemp=vector(0,n-1);
for (i=0;i<n;i++)
	xtemp[i]=x[p[i]];
for (i=0;i<n;i++)
	x[i]=xtemp[i];	
	// free the auxilary vectors
free_vector(ytemp,0,n-1);free_vector(xtemp,0,n-1); free_ivector(p,0,n-1);
	
// Do the interpolation, use GSL libraries for that
	gsl_interp_accel *acc= gsl_interp_accel_alloc ();
	const gsl_interp_type *method;
// The method for interpolation is chosen based on the number of points
if(n>3)
	method=gsl_interp_linear;
else if(n>4)
	method=gsl_interp_cspline;
else if(n>5)
	method=gsl_interp_akima;
	
// If we have enough points use GSL libraries for interpolation, else use the polint algorithm
gsl_spline *interp ;
	if(n>3) 
	{	
	interp = gsl_spline_alloc (method, n);
	gsl_spline_init (interp, x, y, n);
	}

// Now the funny part starts. Use Numerical Recipes
// This is Romberg Integration method
// This method uses the Nevilles' algorithm for interpollation; I
// do not thing I could use the GSl libraries for this
double yup,ylow;
double xx,tnm,sum,del,ss,dss,error,tsum;
if(n>3)
{
yup=gsl_spline_eval (interp,up, acc);
ylow=gsl_spline_eval (interp,low, acc);
}
else if (n<=3) 
{
polint(x,y,n,up,&yup,&dss);
polint(x,y,n,low,&ylow,&dss);
}

double *S, *h;
S=vector(0,iter);
h=vector(0,iter);
int j,it,l;
bool success=FALSE;
h[1]=1.0;
for(j=1;j<=iter;j++)
{	//Trapezoid Rule
	if(j==1)
		S[j]=0.5*(up-low)*(ylow+yup);
	else
	{
		for(it=1,l=1;l<j-1;l++)it<<=1;
		tnm=it;
		del=(up-low)/tnm;
		xx=low+0.5*del;
		for(sum=0.0,l=1;l<=it;l++) 
		{
		if(n>3)
		sum+=gsl_spline_eval (interp,xx, acc);
		else if(n<=3)
		{
		polint(x,y,n,xx,&tsum,&dss);
		sum+=tsum;
		}
		xx+=del;
		}
		S[j]=0.5*(S[j-1]+(up-low)*sum/tnm);
	
	}
	if(j>=order)
	{
		polint(&h[j-order],&S[j-order],order,0,&ss,&dss);
		/*if(fabs(dss)<=tol*fabs(ss))
		{
			error=tol;
			success=TRUE;
		}*/
		S[j]=ss;
	}
	h[j+1]=0.25*h[j];
	S[j+1]=S[j];
error=fabs(S[j]-S[j-1]);
if(error<=tol) success=TRUE;
if(success) break;
}
QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();

QString pref;
if(order==1) pref="st";
if(order==2) pref="nd";
if(order==3) pref="rd";
if(order>3) pref="th";
QString info=date+"\t"+this->caption()+"\nNumerical integration of: "+myPlot->curveTitle(curveKey)+" with a "+QString::number(order)+pref+" order method \n";
if(success)
info+="Number of iterations: "+QString::number(j)+"\n"; 
if(!success)
info+="Number of iterations: "+QString::number(j-1)+"\n"; 
info+="Tolerance (max="+QString::number(tol)+"): "+QString::number(error)+"\n";
info+="Points: "+QString::number(n)+" from x="+QString::number(low) +" to x="+QString::number(up)+"\n";
info+="Peak at x=";
info+=QString::number(x[maxID])+"\t";
info+="y=";
info+=QString::number(y[maxID])+"\n";
info+="Area=";
if(success)
info+=QString::number(S[j]);
if(!success)
info+=QString::number(S[j-1]);
info+="\n-------------------------------------------------------------\n";

if(n>3)
gsl_spline_free (interp);

gsl_interp_accel_free (acc);
free_vector(x,0,n-1);free_vector(y,0,n-1); free_vector(S,0,iter);free_vector(h,0,iter);
return info;
}

void polint(double xa[], double ya[], int n, double x, double *y, double *dy)
{
int i,m,ns=1;
double den,dif,dift,ho,hp,w;
double *c, *d;
dif=fabs(x-xa[1]);
c=vector(1,n);
d=vector(1,n);
for(i=1;i<=n;i++)
{
	if ((dift=fabs(x-xa[i]))<dif)
	{
		ns=i;
		dif=dift;
	}
	c[i]=ya[i];
	d[i]=ya[i];
}
*y=ya[ns--];
for(m=1;m<n;m++)
{	
	for(i=1;i<=n-m;i++)
	{
		ho=xa[i]-x;
		hp=xa[i+m]-x;
		w=c[i+1]-d[i];
		den=ho-hp;
		den=w/den;
		d[i]=hp*den;
		c[i]=ho*den;
	}
	*y+=(*dy=(2*ns<(n-m)?c[ns+1]:d[ns--]));
}
free_vector(d,1,n);
free_vector(c,1,n);
}


QString Graph::fitCurveLinear(long curveKey)
{
QwtPlotCurve *curv=myPlot->curve(curveKey);
if (!curv)
	return "";

int i,n;
if (rangeSelectorsEnabled)
	n=abs(endPoint-startPoint)+1;
else
	n=curv->dataSize();

double *x,*y;
x=vector(0,n-1);
y=vector(0,n-1);

// This is the data to be fitted 
 if (rangeSelectorsEnabled)
	{
	if (startPoint<endPoint)
  		{
  		for (i = 0; i < n; i++)
    			{
	  			x[i]=curv->x(i+startPoint);
      			y[i] = curv->y(i+startPoint);
    			}
  		}
 	else
  		{
  		for (i = 0; i < n; i++)
    			{
	  			x[i]=curv->x(i+endPoint);
      			y[i]=curv->y(i+endPoint);
    			}
  		}
	}
else
	{
	for (i = 0; i < n; i++)
    	{
	  	x[i]=curv->x(i);
      	y[i]=curv->y(i);
    	}
	}

	double c0, c1, cov00, cov01, cov11,sumsq;
	double sst, Rsquare;
	
	gsl_fit_linear (x, 1, y, 1, n,
                   &c0, &c1, &cov00, &cov01, &cov11,
                   &sumsq);
	sst=(n-1)*gsl_stats_variance(y,1,n);
	Rsquare=1-sumsq/sst;

	for (i=0;i<n;i++)
		y[i]=c0+c1*x[i];
		
	QString label="LinearFit"+QString::number(fitID)+"_2";
//	curvesList<<label;

	long curveID = myPlot->insertCurve(label);
	myPlot->setCurvePen(curveID, QPen(red,1)); 

	label="LinearFit"+QString::number(fitID)+"_1(X),"+label+"(Y)";
	associations<<label;

	cOnPlot++;
	keys.resize(cOnPlot);
	keys[cOnPlot-1]=curveID;
	
	if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		mrk->setText(legendText());
	}

	myPlot->setCurveData(curveID,x,y,n);
	for (i=0;i<4;i++)
		myPlot->setAxisAutoScale(i);
	myPlot->replot();
	updateScale();

	curveLayout cl = initCurveLayout();
	layout.resize(cOnPlot);
	layout[cOnPlot-1]=cl;

QString text="1\t2\n";
	for (i=0;i<n;i++)
	{
	text+=QString::number(x[i]);
	text+="\t";
	text+=QString::number(y[i]);
	text+="\n";
	}

QString caption="LinearFit"+QString::number(fitID);
emit createTable(caption,n,2,text);	
	
QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();

QString info=date+"\t"+this->caption()+"\nLinear regresion of "+myPlot->curveTitle(curveKey)+": y=Ax+B\n";
info+="From x="+QString::number(x[0]) +" to x="+QString::number(x[n-1])+"\n";
info+="A="+QString::number(c1);
info+="\tB="+QString::number(c0);
info+="\n-------------------------------------------------------------\n";
info+="sumsq="+QString::number(sumsq);
info+="\nRsquare="+QString::number(Rsquare);
info+="\n-------------------------------------------------------------\n";

free_vector(x,0,n-1);free_vector(y,0,n-1);
return info;
}

QString Graph::fitNonlinearCurve(const QString& curve,const QString& formula,
							  const QStringList& params,const QStringList& paramsInit,
							  double from,double to,int points, int solver, double tolerance)
{
QStringList curvesList = plotCurvesList();
int index=curvesList.findIndex(curve);
QwtPlotCurve *curv=myPlot->curve(keys[index]);
int n=curv->dataSize();
if (n<=0)
	{
	QMessageBox::critical(this,"QtiPlot - Warning",
				"The selected curve is empty! Please select another curve!");
	return "";
	}

int i,size=0;
for (i = 0; i < n; i++)
    {
	if (curv->x(i)>=from && curv->x(i) <=to)
		size++;
    }

double *y,*sigma,*X;
y=vector(0,size-1);
X=vector(0,size-1);
sigma=vector(0,size-1);
gsl_vector *XX = gsl_vector_alloc (size);
// This is the data to be fitted 
size=0;
for (i = 0; i <n; i++)
    {
	if (curv->x(i)>=from && curv->x(i) <=to)
		{
		X[size]=curv->x(i);
		y[size]=curv->y(i);
		sigma[size] =0.1;
		gsl_vector_set (XX,size,X[size]);
		size++;
		}
  	}

n=size;

QString names=params.join (",");
const char *parNames=names.ascii();

QStringList equation=QStringList::split("=",formula,FALSE);

const char *function=equation[1].ascii();
const size_t p = params.count();

double *par;
par=vector(0,p-1);
	
const gsl_multifit_fdfsolver_type *T;
gsl_multifit_fdfsolver *s;	
gsl_matrix *covar = gsl_matrix_alloc (p, p);

struct fit d = { n, p, X, y, sigma,function,parNames};
gsl_multifit_function_fdf f;

double *x_init;
x_init=vector(0,p-1);

for (i=0;i<(int)p;i++)
	x_init[i]=paramsInit[i].toDouble();

gsl_vector_view x = gsl_vector_view_array (x_init, p);

  f.f = &user_function;
  f.df = &user_function_jacob;
  f.fdf = &user_function_fdf;
  f.n = n;
  f.p = p;
  f.params = &d;

  if (solver)
  	T = gsl_multifit_fdfsolver_lmder;
  else
	 T = gsl_multifit_fdfsolver_lmsder;
  
  s = gsl_multifit_fdfsolver_alloc (T, n, p);
  gsl_multifit_fdfsolver_set (s, &f, &x.vector);

int status;
size_t iter = 0;

   do
    {
      iter++;
	  status = gsl_multifit_fdfsolver_iterate (s);

	  if (status)
        break;

      status = gsl_multifit_test_delta (s->dx, s->x,
                                        tolerance, tolerance);
    }
  while (status == GSL_CONTINUE && (int)iter < points);

gsl_multifit_covar (s->J, 0.0, covar);

for (i=0;i<(int)p;i++)
	par[i]=gsl_vector_get(s->x,i); //retrieve values for the parameters
  
QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();
  
QString info=date+"\t"+this->caption()+"\nNon-linear fit of "+curve+", using function: "+formula+"\n";
if (solver)
	info+="Unscaled Levenberg-Marquardt";
else
	info+="Scaled Levenberg-Marquardt";
info+=" algorithm with tolerance = "+QString::number(tolerance)+"\n";
info+="From x="+QString::number(X[0]) +" to x="+QString::number(X[n-1])+"\n";

QString info2, error;
for (i=0;i<(int)p;i++)
	{
	info2.sprintf(params[i]+"=%.5f +/- %.5f\n",par[i],sqrt(gsl_matrix_get(covar,i,i)));
	info+=info2;
	}

info+="Iterations = "+QString::number(iter)+"\n";

error.sprintf ("status = %s\n", gsl_strerror (status));
info+=error;
info+="-------------------------------------------------------------\n";

gsl_multifit_fdfsolver_free (s);
free_vector(sigma,0,n-1);
gsl_matrix_free (covar);

QString label="Fit"+QString::number(fitID)+"_2";
long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(red,1)); 
	
label="Fit"+QString::number(fitID)+"_1(X),"+label+"(Y)";
associations<<label;

	cOnPlot++;
	keys.resize(cOnPlot);
	keys[cOnPlot-1]=curveID;
	
	if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		mrk->setText(legendText());
	}

	MathUtils::Parser parser;
	double xvar;
	for (i=0;i<(int)p;i++)
		parser.AddVar(params[i].ascii(), &par[i]);

	parser.AddVar("x", &xvar);
	
	 if (equation[1].contains("pi")>0)
		parser.AddConst("pi", M_PI);
	 if (equation[1].contains("Pi")>0)
		parser.AddConst("Pi", M_PI);
	 if (equation[1].contains("PI")>0)
		parser.AddConst("PI", M_PI);
	 
	parser.SetFormula(function);

	int n1=n;
	if(n<100)
		n1=100; 
	
	double X0,XN, step;	
	
	int maxID=gsl_vector_max_index (XX);
	int minID=gsl_vector_min_index (XX);
  	gsl_vector_free (XX);
	
	X0=X[minID];
	XN=X[maxID];
	free_vector(X,0,n-1); free_vector(y,0,n-1);
	step=(XN-X0)/(n1-1);
	X=vector(0,n1-1); y=vector(0,n1-1);
	for (i=0;i<n1;i++)
		{
		X[i]=X0+i*step;
		xvar=X[i];
		y[i]=parser.Calc();
		}
	myPlot->setCurveData(curveID,X,y,n1);	
	QString text="1\t2\n";
	for (i=0;i<n1;i++)
	{
	text+=QString::number(X[i]);
	text+="\t";
	text+=QString::number(y[i]);
	text+="\n";
	}
	free_vector(X,0,n1-1); free_vector(y,0,n1-1);
	QString caption="Fit"+QString::number(fitID);
	emit createTable(caption,n1,2,text);
	
	free_vector(par,0,p-1);

	for (i=0;i<4;i++)
		myPlot->setAxisAutoScale(i);
	myPlot->replot();
	updateScale();

	curveLayout cl = initCurveLayout();
	layout.resize(cOnPlot);
	layout[cOnPlot-1]=cl;

return info;
}	

int user_function(const gsl_vector * x, void *params, gsl_vector * f)
{
  size_t n = ((struct fit *)params)->n;
  size_t p = ((struct fit *)params)->p;
  double *X = ((struct fit *)params)->X;
  double *y = ((struct fit *)params)->y;
  double *sigma = ((struct fit *) params)->sigma;
  const char *function = ((struct fit *) params)->function;
  QString names = (QString)((struct fit *) params)->names;
  QStringList parNames= QStringList::split(",",names,FALSE);

  MathUtils::Parser parser;
  double *parameters;
  parameters=vector(0,p-1);

try
	{
	double xvar;
	parser.AddVar("x", &xvar);

	size_t i;
	for (i=0;i<p;i++)
		{
		parser.AddVar(parNames[i].ascii(), &parameters[i]);
		parameters[i]=gsl_vector_get(x,i);
		}
		
	 if (((QString)function).contains("pi")>0)
		parser.AddConst("pi", M_PI);
	 if (((QString)function).contains("Pi")>0)
		parser.AddConst("Pi", M_PI);
	 if (((QString)function).contains("PI")>0)
		parser.AddConst("PI", M_PI);

	parser.SetFormula(function);
	 
	for (i = 0; i < n; i++)
		{
		xvar=X[i];
		double Yi = parser.Calc();
		gsl_vector_set (f, i, (Yi - y[i])/sigma[i]);

		}
	}
catch(MathUtils::ParserException &e)
	{
	QMessageBox::critical(0,"QtiPlot - user_function",e.what());
	}

  free_vector(parameters,0,p-1);

  return GSL_SUCCESS;
}

int user_function_jacob(const gsl_vector * x, void *params,
         gsl_matrix * J)
{
  size_t n = ((struct fit *)params)->n;
  size_t p = ((struct fit *)params)->p;
  double *X = ((struct fit *)params)->X;
  double *sigma = ((struct fit *) params)->sigma;
  const char *function = ((struct fit *) params)->function;
  QString names = (QString)((struct fit *) params)->names;
  QStringList parNames= QStringList::split(",",names,FALSE);

  MathUtils::Parser parser;

  double *parameters;
  parameters=vector(0,p-1);

  size_t i;
try
	{
	for (i=0;i<p;i++)
		{
		parser.AddVar(parNames[i].ascii(), &parameters[i]);
		parameters[i]=gsl_vector_get(x,i);
		}

	double xvar;
	parser.AddVar("x", &xvar);
	
	 if (((QString)function).contains("pi")>0)
		parser.AddConst("pi", M_PI);
	 if (((QString)function).contains("Pi")>0)
		parser.AddConst("Pi", M_PI);
	 if (((QString)function).contains("PI")>0)
		parser.AddConst("PI", M_PI);
	 
	parser.SetFormula(function);

	size_t j;
	for (i = 0; i < n; i++)
		{
        // Jacobian matrix J(i,j) = dfi / dxj
		xvar = X[i];
		double s = sigma[i];
      
		for (j=0;j<p;j++)
			{
			double diff=parser.Diff(&parameters[j],xvar);
			gsl_matrix_set (J, i, j, diff/s);
			}
		}
	}
catch(MathUtils::ParserException &e)
	{
	QMessageBox::critical(0,"QtiPlot - Input function error",e.what());
	}

  free_vector(parameters,0,p-1);
  return GSL_SUCCESS;
}

int user_function_fdf(const gsl_vector * x, void *params,
          gsl_vector * f, gsl_matrix * J)
{
user_function (x, params, f);
user_function_jacob (x, params, J);
return GSL_SUCCESS;
}

QString Graph::fitExpDecay(long curveKey)
{ 
QwtPlotCurve *curv=myPlot->curve(curveKey);
if (!curv)
	return "";
	
size_t n;
if (rangeSelectorsEnabled)
	n=abs(endPoint-startPoint)+1;
else
	n=curv->dataSize();

double *y,*sigma,*X;
y=vector(0,n-1);
X=vector(0,n-1);
sigma=vector(0,n-1);

gsl_vector *XX = gsl_vector_alloc (n);
// This is the data to be fitted 
int i;
if (rangeSelectorsEnabled)
{
if (startPoint < endPoint)
  	{
  	for (i = 0; i < (int)n; i++)
    	{		
	  	X[i]=curv->x(i+startPoint);
      	y[i] = curv->y(i+startPoint);
      	sigma[i] =0.1;	
		gsl_vector_set (XX,i,X[i]);	
    	}
  	}
else 
  	{
  	for (i = 0; i < (int)n; i++)
    	{
	  	X[i]=curv->x(i+endPoint);
      	y[i] = curv->y(i+endPoint);
      	sigma[i] =0.1;
		gsl_vector_set (XX,i,X[i]);
    	}
  	}
}
else
	{
  	for (i = 0; i < (int)n; i++)
    	{
	  	X[i]=curv->x(i);
      	y[i] = curv->y(i);
      	sigma[i] =0.1;
		gsl_vector_set (XX,i,X[i]);
    	}
  	}
	
const size_t p = 3;	
const gsl_multifit_fdfsolver_type *T;
gsl_multifit_fdfsolver *s;	
gsl_matrix *covar = gsl_matrix_alloc (p, p);
struct data d = { n, X, y, sigma};
gsl_multifit_function_fdf f;
double x_init[3] = { 1.0, 0.001, 0.0 };
gsl_vector_view x = gsl_vector_view_array (x_init, p);

  f.f = &expb_f;
  f.df = &expb_df;
  f.fdf = &expb_fdf;
  f.n = n;
  f.p = p;
  f.params = &d;

  T = gsl_multifit_fdfsolver_lmsder;
  s = gsl_multifit_fdfsolver_alloc (T, n, p);
  gsl_multifit_fdfsolver_set (s, &f, &x.vector);

int status;
size_t iter = 0;
   do
    {
      iter++;
	  status = gsl_multifit_fdfsolver_iterate (s);
      
	  if (status)
        break;

      status = gsl_multifit_test_delta (s->dx, s->x,
                                        1e-4, 1e-4);
    }
  while (status == GSL_CONTINUE && iter < 1000);

gsl_multifit_covar (s->J, 0.0, covar);

double A=gsl_vector_get(s->x,0);
double l=gsl_vector_get(s->x,1);
double B=gsl_vector_get(s->x,2);
  
QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();
  
QString info=date+"\t"+this->caption()+"\nExponential decay of "+myPlot->curveTitle(curveKey)+": y=Aexp(-lx)+B\n";
info+="From x="+QString::number(X[0]) +" to x="+QString::number(X[n-1])+"\n";
QString infoA,infoL,infoB,error;
infoA.sprintf("A = %.5f +/- %.5f\n",A,sqrt(gsl_matrix_get(covar,0,0)));
info+=infoA;
infoL.sprintf("l   = %.5f +/- %.5f\n",l,sqrt(gsl_matrix_get(covar,1,1)));
info+=infoL;
infoB.sprintf("B = %.5f +/- %.5f\n",B,  sqrt(gsl_matrix_get(covar,2,2)));
info+=infoB;
info+="Iterations ="+QString::number(iter)+"\n";
error.sprintf ("status = %s\n", gsl_strerror (status));
info+=error;
info+="-------------------------------------------------------------\n";

gsl_multifit_fdfsolver_free (s);
free_vector(sigma,0,n-1);
gsl_matrix_free (covar);

QString label="Fit"+QString::number(fitID)+"_2";
long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(red,1)); 
	
label="Fit"+QString::number(fitID)+"_1(X),"+label+"(Y)";
associations<<label;

	cOnPlot++;
	keys.resize(cOnPlot);
	keys[cOnPlot-1]=curveID;
	
	if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		mrk->setText(legendText());
	}

	int n1=n;
	if(n<100)
		n1=100;
	
	double X0,XN, step;
	
	int maxID=gsl_vector_max_index (XX);
	int minID=gsl_vector_min_index (XX);
  	gsl_vector_free (XX);
	
	X0=X[minID];
	XN=X[maxID];
	step=(XN-X0)/(n1-1);
	free_vector(X,0,n-1); free_vector(y,0,n-1);
	
	X=vector(0,n1-1);y=vector(0,n1-1);
	for (i=0;i<n1;i++)
		{
		X[i]=X0+i*step;
		y[i]=A*exp(-l*X[i])+B;
		}
	myPlot->setCurveData(curveID,X,y,n1);	
	QString text="1\t2\n";
	for (i=0;i<n1;i++)
	{
	text+=QString::number(X[i]);
	text+="\t";
	text+=QString::number(y[i]);
	text+="\n";
	}
	free_vector(X,0,n1-1); free_vector(y,0,n1-1);
	QString caption="Fit"+QString::number(fitID);
	emit createTable(caption,n1,2,text);
	
	for (i=0;i<4;i++)
		myPlot->setAxisAutoScale(i);
	myPlot->replot();
	updateScale();

	curveLayout cl=initCurveLayout();
	layout.resize(cOnPlot);
	layout[cOnPlot-1]=cl;
	
return info;
}

QString Graph::fitExpDecay(const QString& name, double damping, double amplitude, double yOffset)
{  
QStringList curvesList = plotCurvesList();
int index=curvesList.findIndex(name);
QwtPlotCurve *curv=myPlot->curve(keys[index]);
if (curv->dataSize() < 3)
	{
	QMessageBox::critical(this,"QtiPlot - Warning",
				"You need at least 3 points to fit! Operation aborted!");
	return "";
	}

int status;
size_t  n, iter = 0;
	
if (rangeSelectorsEnabled)
	n=abs(endPoint-startPoint)+1;
else
	n=curv->dataSize();

double *y,*sigma,*X;
y=vector(0,n-1);
X=vector(0,n-1);
sigma=vector(0,n-1);

gsl_vector *XX = gsl_vector_alloc (n);
// This is the data to be fitted 
int i;
if (rangeSelectorsEnabled)
{
if (startPoint < endPoint)
  	{
  	for (i = 0; i < (int)n; i++)
    	{		
	  	X[i]=curv->x(i+startPoint);
      	y[i] = curv->y(i+startPoint);
      	sigma[i] =0.1;	
		gsl_vector_set (XX,i,X[i]);	
    	}
  	}
else 
  	{
  	for (i = 0; i < (int)n; i++)
    	{
	  	X[i]=curv->x(i+endPoint);
      	y[i] = curv->y(i+endPoint);
      	sigma[i] =0.1;
		gsl_vector_set (XX,i,X[i]);
    	}
  	}
}
else
	{
  	for (i = 0; i < (int)n; i++)
    	{
	  	X[i]=curv->x(i);
      	y[i] = curv->y(i);
      	sigma[i] =0.1;
		gsl_vector_set (XX,i,X[i]);
    	}
  	}
	
const size_t p = 3;	
const gsl_multifit_fdfsolver_type *T;
gsl_multifit_fdfsolver *s;	
gsl_matrix *covar = gsl_matrix_alloc (p, p);
struct data d = { n, X, y, sigma};
gsl_multifit_function_fdf f;
double x_init[3] = { amplitude, damping, yOffset};
gsl_vector_view x = gsl_vector_view_array (x_init, p);

  f.f = &expb_f;
  f.df = &expb_df;
  f.fdf = &expb_fdf;
  f.n = n;
  f.p = p;
  f.params = &d;

  T = gsl_multifit_fdfsolver_lmsder;
  s = gsl_multifit_fdfsolver_alloc (T, n, p);
  gsl_multifit_fdfsolver_set (s, &f, &x.vector);

do
    {
      iter++;
	  status = gsl_multifit_fdfsolver_iterate (s);
      
	  if (status)
        break;

      status = gsl_multifit_test_delta (s->dx, s->x,
                                        1e-4, 1e-4);
    }
  while (status == GSL_CONTINUE && iter < 1000);

gsl_multifit_covar (s->J, 0.0, covar);

double A=gsl_vector_get(s->x,0);
double l=gsl_vector_get(s->x,1);
double B=gsl_vector_get(s->x,2);
  
QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();
  
QString info=date+"\t"+this->caption()+"\nExponential decay of " + name + ": y=Aexp(-lx)+B\n";
info+="From x="+QString::number(X[0]) +" to x="+QString::number(X[n-1])+"\n";
QString infoA,infoL,infoB,error;
infoA.sprintf("A = %.5f +/- %.5f\n",A,sqrt(gsl_matrix_get(covar,0,0)));
info+=infoA;
infoL.sprintf("l   = %.5f +/- %.5f\n",l,sqrt(gsl_matrix_get(covar,1,1)));
info+=infoL;
infoB.sprintf("B = %.5f +/- %.5f\n",B,  sqrt(gsl_matrix_get(covar,2,2)));
info+=infoB;
info+="Iterations ="+QString::number(iter)+"\n";
error.sprintf ("status = %s\n", gsl_strerror (status));
info+=error;
info+="-------------------------------------------------------------\n";

gsl_multifit_fdfsolver_free (s);
free_vector(sigma,0,n-1);
gsl_matrix_free (covar);

QString label="Fit"+QString::number(fitID)+"_2";
long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(red,1)); 
	
label="Fit"+QString::number(fitID)+"_1(X),"+label+"(Y)";
associations<<label;

	cOnPlot++;
	keys.resize(cOnPlot);
	keys[cOnPlot-1]=curveID;
	
	if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		mrk->setText(legendText());
	}

	int n1=n;
	if(n<100)
		n1=100;
	
	double X0,XN, step;
	
	int maxID=gsl_vector_max_index (XX);
	int minID=gsl_vector_min_index (XX);
  	gsl_vector_free (XX);
	
	X0=X[minID];
	XN=X[maxID];
	step=(XN-X0)/(n1-1);
	free_vector(X,0,n-1); free_vector(y,0,n-1);
	
	X=vector(0,n1-1);y=vector(0,n1-1);
	for (i=0;i<n1;i++)
		{
		X[i]=X0+i*step;
		y[i]=A*exp(-l*X[i])+B;
		}
	myPlot->setCurveData(curveID,X,y,n1);	
	QString text="1\t2\n";
	for (i=0;i<n1;i++)
	{
	text+=QString::number(X[i]);
	text+="\t";
	text+=QString::number(y[i]);
	text+="\n";
	}
	free_vector(X,0,n1-1); free_vector(y,0,n1-1);
	QString caption="Fit"+QString::number(fitID);
	emit createTable(caption,n1,2,text);
	
	for (i=0;i<4;i++)
		myPlot->setAxisAutoScale(i);
	myPlot->replot();
	updateScale();

	curveLayout cl=initCurveLayout();
	layout.resize(cOnPlot);
	layout[cOnPlot-1]=cl;
	
return info;
}

QString Graph::fitExpDecay2(const QString& name, double firstTime, double secondTime,
						double start, double yOffset)
{  
QStringList curvesList = plotCurvesList();
int index=curvesList.findIndex(name);
QwtPlotCurve *curv=myPlot->curve(keys[index]);
if (curv->dataSize() < 5)
	{
	QMessageBox::critical(this,"QtiPlot - Warning",
				"You need at least 5 points to fit! Operation aborted!");
	return "";
	}

const gsl_multifit_fdfsolver_type *T;
gsl_multifit_fdfsolver *s;

int status;
size_t i, n, iter = 0;
const size_t p = 5;
n=curv->dataSize();
	
int startIndex=0;
for (i = 0; i < n; i++)
    	{
		if (curv->x(i)>= start)
			{
	  		startIndex=i;
			break;	
			}
		}
		
n=n-startIndex;
		
gsl_matrix *covar = gsl_matrix_alloc (p, p);
  
  double *y,*sigma,*X;
  y=vector(0,n-1);
  X=vector(0,n-1);
  sigma=vector(0,n-1);
gsl_vector *XX = gsl_vector_alloc (n);	
    // This is the data to be fitted 
  	for (i = 0; i < n; i++)
    	{
	  	X[i]=curv->x(i+startIndex);
      	y[i] = curv->y(i+startIndex);
      	sigma[i] =0.1;
		gsl_vector_set (XX,i,X[i]);
    	}
	
  struct data d = { n, X, y, sigma};

  gsl_multifit_function_fdf f;

  double x_init[5] = { 1.0, 1.0/firstTime,1.0, 1.0/secondTime, yOffset};

  gsl_vector_view x = gsl_vector_view_array (x_init, p);

  f.f = &expd2_f;
  f.df = &expd2_df;
  f.fdf = &expd2_fdf;
  f.n = n;
  f.p = p;
  f.params = &d;

  T = gsl_multifit_fdfsolver_lmsder;
  s = gsl_multifit_fdfsolver_alloc (T, n, p);
  gsl_multifit_fdfsolver_set (s, &f, &x.vector);

   do
    {
      iter++;
	  status = gsl_multifit_fdfsolver_iterate (s);
      
	  if (status)
        break;

      status = gsl_multifit_test_delta (s->dx, s->x,
                                        1e-4, 1e-4);
    }
  while (status == GSL_CONTINUE && iter < 1000);

  gsl_multifit_covar (s->J, 0.0, covar);

QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();

double A1=gsl_vector_get(s->x,0);
double t1=1.0/gsl_vector_get(s->x,1);
double A2=gsl_vector_get(s->x,2);
double t2=1.0/gsl_vector_get(s->x,3);
double y0=gsl_vector_get(s->x,4);

QString info=date+"\t"+this->caption()+"\nExponential decay of "+ name +": y=A1*exp(-x/t1)+A2*exp(-x/t2)+y0\n";
info+="From x="+QString::number(X[0]) +" to x="+QString::number(X[n-1])+"\n";
QString infoA,error;
infoA.sprintf("A1 = %.5f +/- %.5f\n",A1,sqrt(gsl_matrix_get(covar,0,0)));
info+=infoA;
infoA.sprintf("t1   = %.5f +/- %.5f\n",t1,sqrt(gsl_matrix_get(covar,1,1)));
info+=infoA;
infoA.sprintf("A2 = %.5f +/- %.5f\n",A2,sqrt(gsl_matrix_get(covar,2,2)));
info+=infoA;
infoA.sprintf("t2   = %.5f +/- %.5f\n",t2,sqrt(gsl_matrix_get(covar,3,3)));
info+=infoA;
infoA.sprintf("y0 = %.5f +/- %.5f\n",y0,  sqrt(gsl_matrix_get(covar,4,4)));
info+=infoA;
info+="Iterations ="+QString::number(iter)+"\n";
error.sprintf ("status = %s\n", gsl_strerror (status));
info+=error;
info+="-------------------------------------------------------------\n";

gsl_multifit_fdfsolver_free (s);
free_vector(sigma,0,n-1);
gsl_matrix_free (covar);

QString label="Fit"+QString::number(fitID)+"_2";
long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(red,1)); 
	
	label="Fit"+QString::number(fitID)+"_1(X),"+label+"(Y)";
	associations<<label;

	cOnPlot++;
	keys.resize(cOnPlot);
	keys[cOnPlot-1]=curveID;
	
	if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		mrk->setText(legendText());
	}

	size_t n1=n;
	double X0,XN, step;
	if(n<100)	
		n1=100;
	
	int maxID=gsl_vector_max_index (XX);
	int minID=gsl_vector_min_index (XX);
  	gsl_vector_free (XX);
	
	X0=X[minID];
	XN=X[maxID];
	step=(XN-X0)/(n1-1);
	free_vector(X,0,n-1);free_vector(y,0,n-1);
	X=vector(0,n1-1);y=vector(0,n1-1);
	for (i=0;i<n1;i++)
		{
		X[i]=X0+i*step;
		y[i]=A1*exp(-X[i]/t1)+A2*exp(-X[i]/t2)+y0;
		}
	myPlot->setCurveData(curveID,X,y,n1);
	QString text="1\t2\n";
	for (i=0;i<n1;i++)
	{
	text+=QString::number(X[i]);
	text+="\t";
	text+=QString::number(y[i]);
	text+="\n";
	}
	free_vector(X,0,n1-1); free_vector(y,0,n1-1);
	QString caption="Fit"+QString::number(fitID);
	emit createTable(caption,n1,2,text);
	
	for (i=0;i<4;i++)
		myPlot->setAxisAutoScale(i);
	myPlot->replot();
	updateScale();

	curveLayout cl = initCurveLayout();
	layout.resize(cOnPlot);
	layout[cOnPlot-1]=cl;
	
return info;
}

QString Graph::fitExpDecay3(const QString& name, double firstTime, double secondTime,
						double thirdTime,double start, double yOffset)
{ 
QStringList curvesList = plotCurvesList();	
int index=curvesList.findIndex(name);
QwtPlotCurve *curv=myPlot->curve(keys[index]);
if (curv->dataSize()<=7)
	{
	QMessageBox::critical(this,"QtiPlot - Warning",
				"You need at least 7 points to fit! Operation aborted!");
	return "";
	}
	
const gsl_multifit_fdfsolver_type *T;
gsl_multifit_fdfsolver *s;

int status;
size_t i, n, iter = 0;
const size_t p = 7;
n=curv->dataSize();
	
int startIndex=0;
for (i = 0; i < n; i++)
    	{
		if (curv->x(i)>= start)
			{
	  		startIndex=i;
			break;	
			}
		}
		
n=n-startIndex;
		
gsl_matrix *covar = gsl_matrix_alloc (p, p);
  
  double *y,*sigma,*X;
  y=vector(0,n-1);
  X=vector(0,n-1);
  sigma=vector(0,n-1);
gsl_vector *XX = gsl_vector_alloc (n);	
    // This is the data to be fitted 
  	for (i = 0; i < n; i++)
    	{
	  	X[i]=curv->x(i+startIndex);
      	y[i] = curv->y(i+startIndex);
      	sigma[i] =0.1;
		gsl_vector_set (XX,i,X[i]);
    	}
	
  struct data d = { n, X, y, sigma};

  gsl_multifit_function_fdf f;

  double x_init[7] = { 1.0, 1.0/firstTime,1.0, 1.0/secondTime,1.0, 1.0/thirdTime,yOffset};

  gsl_vector_view x = gsl_vector_view_array (x_init, p);

  f.f = &expd3_f;
  f.df = &expd3_df;
  f.fdf = &expd3_fdf;
  f.n = n;
  f.p = p;
  f.params = &d;

  T = gsl_multifit_fdfsolver_lmsder;
  s = gsl_multifit_fdfsolver_alloc (T, n, p);
  gsl_multifit_fdfsolver_set (s, &f, &x.vector);

   do
    {
      iter++;
	  status = gsl_multifit_fdfsolver_iterate (s);
      
	  if (status)
        break;

      status = gsl_multifit_test_delta (s->dx, s->x,
                                        1e-4, 1e-4);
    }
  while (status == GSL_CONTINUE && iter < 1000);


  gsl_multifit_covar (s->J, 0.0, covar);

QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();

double A1=gsl_vector_get(s->x,0);
double t1=1.0/gsl_vector_get(s->x,1);
double A2=gsl_vector_get(s->x,2);
double t2=1.0/gsl_vector_get(s->x,3);
double A3=gsl_vector_get(s->x,4);
double t3=1.0/gsl_vector_get(s->x,5);
double y0=gsl_vector_get(s->x,6);

QString info=date+"\t"+this->caption()+"\nExponential decay of "+ name +": y=A1*exp(-x/t1)+A2*exp(-x/t2)+A3*exp(-x/t3)+y0\n";
info+="From x="+QString::number(X[0]) +" to x="+QString::number(X[n-1])+"\n";
QString infoA,error;
infoA.sprintf("A1 = %.5f +/- %.5f\n",A1,sqrt(gsl_matrix_get(covar,0,0)));
info+=infoA;
infoA.sprintf("t1   = %.5f +/- %.5f\n",t1,sqrt(gsl_matrix_get(covar,1,1)));
info+=infoA;
infoA.sprintf("A2 = %.5f +/- %.5f\n",A2,sqrt(gsl_matrix_get(covar,2,2)));
info+=infoA;
infoA.sprintf("t2   = %.5f +/- %.5f\n",t2,sqrt(gsl_matrix_get(covar,3,3)));
info+=infoA;
infoA.sprintf("A3 = %.5f +/- %.5f\n",A3,sqrt(gsl_matrix_get(covar,4,4)));
info+=infoA;
infoA.sprintf("t3   = %.5f +/- %.5f\n",t3,sqrt(gsl_matrix_get(covar,5,5)));
info+=infoA;
infoA.sprintf("y0 = %.5f +/- %.5f\n",y0,  sqrt(gsl_matrix_get(covar,6,6)));
info+=infoA;
info+="Iterations ="+QString::number(iter)+"\n";
error.sprintf ("status = %s\n", gsl_strerror (status));
info+=error;
info+="-------------------------------------------------------------\n";

gsl_multifit_fdfsolver_free (s);
free_vector(sigma,0,n-1);
gsl_matrix_free (covar);

QString label="Fit"+QString::number(fitID)+"_2";

long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(red,1)); 
	
	label="Fit"+QString::number(fitID)+"_1(X),"+label+"(Y)";
	associations<<label;

	cOnPlot++;
	keys.resize(cOnPlot);
	keys[cOnPlot-1]=curveID;
	
	if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		mrk->setText(legendText());
	}

	size_t n1=n;
	if(n<100)	
		n1=100;
	
	double XN,X0, step;

	int maxID=gsl_vector_max_index (XX);
	int minID=gsl_vector_min_index (XX);
  	gsl_vector_free (XX);
	
	X0=X[minID];
	XN=X[maxID];
	step=(XN-X0)/(n1-1);
	free_vector(X,0,n-1);free_vector(y,0,n-1);
	X=vector(0,n1-1);y=vector(0,n1-1);
	for (i=0;i<n1;i++)
		{
		X[i]=X0+i*step;
		y[i]=A1*exp(-X[i]/t1)+A2*exp(-X[i]/t2)+A3*exp(-X[i]/t3)+y0;
		}
	myPlot->setCurveData(curveID,X,y,n1);
	QString text="1\t2\n";
	for (i=0;i<n1;i++)
	{
	text+=QString::number(X[i]);
	text+="\t";
	text+=QString::number(y[i]);
	text+="\n";
	}
	free_vector(X,0,n1-1); free_vector(y,0,n1-1);
	QString caption="Fit"+QString::number(fitID);
	emit createTable(caption,n1,2,text);
	
	for (i=0;i<4;i++)
		myPlot->setAxisAutoScale(i);
	myPlot->replot();
	updateScale();

	curveLayout cl = initCurveLayout();
	layout.resize(cOnPlot);
	layout[cOnPlot-1]=cl;
return info;
}

QString Graph::fitExpGrowth(long curveKey)
{   
QwtPlotCurve *curv=myPlot->curve(curveKey);
if (!curv)
	return "";
	
size_t n;
if (rangeSelectorsEnabled)
	n=abs(endPoint-startPoint)+1;
else
	n=curv->dataSize();

double *y,*sigma,*X;
y=vector(0,n-1);
X=vector(0,n-1);
sigma=vector(0,n-1);
gsl_vector *XX = gsl_vector_alloc (n);
// This is the data to be fitted 
int i;
if (rangeSelectorsEnabled)
{
if (startPoint < endPoint)
  	{
  	for (i = 0; i < (int)n; i++)
    	{		
	  	X[i]=curv->x(i+startPoint);
      	y[i] = curv->y(i+startPoint);
      	sigma[i] =0.1;	
		gsl_vector_set (XX,i,X[i]);	
    	}
  	}
else 
  	{
  	for (i = 0; i < (int)n; i++)
    	{
	  	X[i]=curv->x(i+endPoint);
      	y[i] = curv->y(i+endPoint);
      	sigma[i] =0.1;
		gsl_vector_set (XX,i,X[i]);
    	}
  	}
}
else
	{
  	for (i = 0; i < (int)n; i++)
    	{
	  	X[i]=curv->x(i);
      	y[i] = curv->y(i);
      	sigma[i] =0.1;
		gsl_vector_set (XX,i,X[i]);
    	}
  	}

const gsl_multifit_fdfsolver_type *T;
gsl_multifit_fdfsolver *s;

const size_t p = 3;
gsl_matrix *covar = gsl_matrix_alloc (p, p);
struct data d = { n, X, y, sigma};
gsl_multifit_function_fdf f;

double x_init[3] = {1.0, 0.1, 0.0 };
gsl_vector_view x = gsl_vector_view_array (x_init, p);

  f.f = &expg_f;
  f.df = &expg_df;
  f.fdf = &expg_fdf;
  f.n = n;
  f.p = p;
  f.params = &d;

  T = gsl_multifit_fdfsolver_lmsder;
  s = gsl_multifit_fdfsolver_alloc (T, n, p);
  gsl_multifit_fdfsolver_set (s, &f, &x.vector);

int status;
size_t iter = 0;
do
    {
      iter++;
	  status = gsl_multifit_fdfsolver_iterate (s);
      
	  if (status)
        break;

      status = gsl_multifit_test_delta (s->dx, s->x,
                                        1e-4, 1e-4);
    }
while (status == GSL_CONTINUE && iter < 1000);

gsl_multifit_covar (s->J, 0.0, covar);

double A=gsl_vector_get(s->x,0);
double l=gsl_vector_get(s->x,1);

double B=gsl_vector_get(s->x,2);

QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();

QString info=date+"\t"+this->caption()+"\nExponential growth of "+myPlot->curveTitle(curveKey)+": y=Aexp(lx)+B\n";
info+="From x="+QString::number(X[0]) +" to x="+QString::number(X[n-1])+"\n";
QString infoA,infoL,infoB,error;
infoA.sprintf("A = %.5f +/- %.5f\n",A,sqrt(gsl_matrix_get(covar,0,0)));
info+=infoA;
infoL.sprintf("l   = %.5f +/- %.5f\n",l,sqrt(gsl_matrix_get(covar,1,1)));
info+=infoL;
infoB.sprintf("B = %.5f +/- %.5f\n",B,  sqrt(gsl_matrix_get(covar,2,2)));
info+=infoB;
info+="Iterations ="+QString::number(iter)+"\n";
error.sprintf ("status = %s\n", gsl_strerror (status));
info+=error;
info+="-------------------------------------------------------------\n";

gsl_multifit_fdfsolver_free (s);
free_vector(sigma,0,n-1);
gsl_matrix_free (covar);

QString label="Fit"+QString::number(fitID)+"_2";
long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(red,1)); 

label="Fit"+QString::number(fitID)+"_1(X),"+label+"(Y)";
associations<<label;

cOnPlot++;
keys.resize(cOnPlot);
keys[cOnPlot-1]=curveID;
	
if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		mrk->setText(legendText());
	}

	int n1=n;
	if(n<100)	
		n1=100;
	
	double X0,XN, step;

	int maxID=gsl_vector_max_index (XX);
	int minID=gsl_vector_min_index (XX);
  	gsl_vector_free (XX);
	
	X0=X[minID];
	XN=X[maxID];
	step=(XN-X0)/(n1-1);
	free_vector(X,0,n-1);free_vector(y,0,n-1);
	X=vector(0,n1-1);y=vector(0,n1-1);
	for (i=0;i<n1;i++)
		{
		X[i]=X0+i*step;
		y[i]=A*exp(l*X[i])+B;
		}
	myPlot->setCurveData(curveID,X,y,n1);
	QString text="1\t2\n";
	for (i=0;i<n1;i++)
	{
	text+=QString::number(X[i]);
	text+="\t";
	text+=QString::number(y[i]);
	text+="\n";
	}
	free_vector(X,0,n1-1); free_vector(y,0,n1-1);
	QString caption="Fit"+QString::number(fitID);
	emit createTable(caption,n1,2,text);
	
	for (i=0;i<4;i++)
		{
		myPlot->setAxisAutoScale(i);
		myPlot->setAxisOptions (i,QwtAutoScale::Floating);
		}
	myPlot->replot();
	updateScale();

	curveLayout cl=initCurveLayout();
	layout.resize(cOnPlot);
	layout[cOnPlot-1]=cl;
return info;
}

QString Graph::fitGauss(long curveKey)
{ 
QwtPlotCurve *curv=myPlot->curve(curveKey);
if (!curv)
	return "";
	
size_t n;
if (rangeSelectorsEnabled)
	n=abs(endPoint-startPoint)+1;
else
	n=curv->dataSize();

double *y,*sigma,*X;
y=vector(0,n-1);
X=vector(0,n-1);
sigma=vector(0,n-1);
gsl_vector *XX = gsl_vector_alloc (n);

// This is the data to be fitted 
int i;	
if (rangeSelectorsEnabled)
{
if (startPoint < endPoint)
  	{
  	for (i = 0; i < (int)n; i++)
    	{		
	  	X[i]=curv->x(i+startPoint);
      	y[i] = curv->y(i+startPoint);
      	sigma[i] =0.1;	
		gsl_vector_set (XX,i,X[i]);	
    	}
  	}
else 
  	{
  	for (i = 0; i < (int)n; i++)
    	{
	  	X[i]=curv->x(i+endPoint);
      	y[i] = curv->y(i+endPoint);
      	sigma[i] =0.1;
		gsl_vector_set (XX,i,X[i]);
    	}
  	}
}
else
	{
  	for (i = 0; i < (int)n; i++)
    	{
	  	X[i]=curv->x(i);
      	y[i] = curv->y(i);
      	sigma[i] =0.1;
		gsl_vector_set (XX,i,X[i]);
    	}
  	}

  gsl_vector *sort = gsl_vector_alloc (n);
  for (i = 0; i < (int)n; i++)
    	gsl_vector_set (sort,i,y[i]);

  const size_t maxID=gsl_vector_max_index (sort);
  gsl_vector_free (sort);
  
const gsl_multifit_fdfsolver_type *T;
gsl_multifit_fdfsolver *s;
const size_t p = 4;
gsl_matrix *covar = gsl_matrix_alloc (p, p);
struct data d = { n, X, y, sigma};
gsl_multifit_function_fdf f;

double x_init[4] = { 0.0, 1.0, X[maxID], 1.0};
gsl_vector_view x = gsl_vector_view_array (x_init, p);

  f.f = &gauss_f;
  f.df = &gauss_df;
  f.fdf = &gauss_fdf;
  f.n = n;
  f.p = p;
  f.params = &d;

  T = gsl_multifit_fdfsolver_lmsder;
  s = gsl_multifit_fdfsolver_alloc (T, n, p);
  gsl_multifit_fdfsolver_set (s, &f, &x.vector);

int status;
size_t iter = 0;
   do
    {
      iter++;
	  status = gsl_multifit_fdfsolver_iterate (s);

	  if (status)
        break;

      status = gsl_multifit_test_delta (s->dx,s->x,1e-5,1e-5);
    }
  while (status == GSL_CONTINUE && iter < 500);
  gsl_multifit_covar (s->J, 0.0, covar);

QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();

double B=gsl_vector_get(s->x,0);
double A=gsl_vector_get(s->x,1);
double C=gsl_vector_get(s->x,2);
double w=gsl_vector_get(s->x,3);

QString info=date+"\t"+this->caption()+"\nGauss fit of "+myPlot->curveTitle(curveKey)+": y=y0+Aexp[-(x-xc)*(x-xc)/(2*w*w)]\n";
info+="From x="+QString::number(X[0]) +" to x="+QString::number(X[n-1])+"\n";
QString infoA,infoC,infoW,infoB,error;
infoB.sprintf("y0 (offset) = %.5f +/- %.5f\n",B,sqrt(gsl_matrix_get(covar,0,0)));
info+=infoB;
infoA.sprintf("A (height)  = %.5f +/- %.5f\n",A,sqrt(gsl_matrix_get(covar,1,1)));
info+=infoA;
infoC.sprintf("xc (center) = %.5f +/- %.5f\n",C, sqrt(gsl_matrix_get(covar,2,2)));
info+=infoC;
infoW.sprintf("w (width) = %.5f +/- %.5f\n",w,  sqrt(gsl_matrix_get(covar,3,3)));
info+=infoW;
info+="Iterations ="+QString::number(iter)+"\n";
error.sprintf ("status = %s\n", gsl_strerror (status));
info+=error;
info+="-------------------------------------------------------------\n";

gsl_multifit_fdfsolver_free (s);
free_vector(sigma,0,n-1);
gsl_matrix_free (covar);

QString label="Fit"+QString::number(fitID)+"_2";
long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(red,1)); 

label="Fit"+QString::number(fitID)+"_1(X),"+label+"(Y)";
associations<<label;

	cOnPlot++;
	keys.resize(cOnPlot);
	keys[cOnPlot-1]=curveID;
	
	if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		mrk->setText(legendText());
	}

	int n1=n;
	if(n<100)	
		n1=100;

	double X0,XN, step;
	
	int imaxID=gsl_vector_max_index (XX);
	int iminID=gsl_vector_min_index (XX);
  	gsl_vector_free (XX);
	
	X0=X[iminID];
	XN=X[imaxID];
	step=(XN-X0)/(n1-1);
	free_vector(X,0,n-1);free_vector(y,0,n-1);
	X=vector(0,n1-1);y=vector(0,n1-1);
	for (i=0;i<n1;i++)
		{
		X[i]=X0+i*step;
		double diff=X[i]-C;
		y[i]=A*exp(-0.5*diff*diff/(w*w))+B;
		}
	myPlot->setCurveData(curveID,X,y,n1);
	QString text="1\t2\n";
	for (i=0;i<n1;i++)
	{
	text+=QString::number(X[i]);
	text+="\t";
	text+=QString::number(y[i]);
	text+="\n";
	}
	free_vector(X,0,n1-1); free_vector(y,0,n1-1);
	QString caption="Fit"+QString::number(fitID);
	emit createTable(caption,n1,2,text);
	
	for (i=0;i<4;i++)
		myPlot->setAxisAutoScale(i);
	myPlot->replot();
	updateScale();
	
	curveLayout cl = initCurveLayout();
	layout.resize(cOnPlot);
	layout[cOnPlot-1]=cl;
return info;
}

QString Graph::fitLorentz(long curveKey)
{ 
QwtPlotCurve *curv=myPlot->curve(curveKey);
if (!curv)
	return "";
	
size_t n;
if (rangeSelectorsEnabled)
	n=abs(endPoint-startPoint)+1;
else
	n=curv->dataSize();

double *y,*sigma,*X;
y=vector(0,n-1);
X=vector(0,n-1);
sigma=vector(0,n-1);
gsl_vector *XX = gsl_vector_alloc (n);
// This is the data to be fitted
int i;	
if (rangeSelectorsEnabled)
{
if (startPoint < endPoint)
  	{
  	for (i = 0; i < (int)n; i++)
    	{		
	  	X[i]=curv->x(i+startPoint);
      	y[i] = curv->y(i+startPoint);
      	sigma[i] =0.1;	
		gsl_vector_set (XX,i,X[i]);	
    	}
  	}
else 
  	{
  	for (i = 0; i < (int)n; i++)
    	{
	  	X[i]=curv->x(i+endPoint);
      	y[i] = curv->y(i+endPoint);
      	sigma[i] =0.1;
		gsl_vector_set (XX,i,X[i]);
    	}
  	}
}
else
	{
  	for (i = 0; i < (int)n; i++)
    	{
	  	X[i]=curv->x(i);
      	y[i] = curv->y(i);
      	sigma[i] =0.1;
		gsl_vector_set (XX,i,X[i]);
    	}
  	}
	
gsl_vector *sort = gsl_vector_alloc (n);
for (i = 0; i <(int)n; i++)
    gsl_vector_set (sort,i,y[i]);

const size_t maxID=gsl_vector_max_index (sort);
gsl_vector_free (sort);

const gsl_multifit_fdfsolver_type *T;
gsl_multifit_fdfsolver *s;
const size_t p = 4;
gsl_matrix *covar = gsl_matrix_alloc (p, p);
struct data d = { n, X, y, sigma};

double x_init[4] = { 0.0, 1.0, X[maxID], 1.0};
gsl_vector_view x = gsl_vector_view_array (x_init, p);

gsl_multifit_function_fdf f;
  f.f = &lorentz_f;
  f.df = &lorentz_df;
  f.fdf = &lorentz_fdf;
  f.n = n;
  f.p = p;
  f.params = &d;

  T = gsl_multifit_fdfsolver_lmsder;
  s = gsl_multifit_fdfsolver_alloc (T, n, p);
  gsl_multifit_fdfsolver_set (s, &f, &x.vector);

int status;
size_t iter = 0;
   do
    {
      iter++;
	  status = gsl_multifit_fdfsolver_iterate (s);

	  if (status)
        break;

      status = gsl_multifit_test_delta (s->dx,s->x,1e-5,1e-5);
    }
  while (status == GSL_CONTINUE && iter < 500);
  gsl_multifit_covar (s->J, 0.0, covar);

QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();

double B=gsl_vector_get(s->x,0);
double A=gsl_vector_get(s->x,1);
double C=gsl_vector_get(s->x,2);
double w=gsl_vector_get(s->x,3);

QString info=date+"\t"+this->caption()+"\nLorentz fit of "+myPlot->curveTitle(curveKey)+": y=y0+2A/pi*w/[4*(x-xc)*(x-xc)+w*w)]\n";
info+="From x="+QString::number(X[0]) +" to x="+QString::number(X[n-1])+"\n";
QString infoA,infoC,infoW,infoB,error;
infoB.sprintf("y0 (offset) = %.5f +/- %.5f\n",B,sqrt(gsl_matrix_get(covar,0,0)));
info+=infoB;
infoA.sprintf("A (area)  = %.5f +/- %.5f\n",2*atan(1.0)*A,sqrt(gsl_matrix_get(covar,1,1)));
info+=infoA;
infoC.sprintf("xc (center) = %.5f +/- %.5f\n",C, sqrt(gsl_matrix_get(covar,2,2)));
info+=infoC;
infoW.sprintf("w (width) = %.5f +/- %.5f\n",w,  sqrt(gsl_matrix_get(covar,3,3)));
info+=infoW;
info+="Iterations ="+QString::number(iter)+"\n";
error.sprintf ("status = %s\n", gsl_strerror (status));
info+=error;
info+="-------------------------------------------------------------\n";

gsl_multifit_fdfsolver_free (s);
free_vector(sigma,0,n-1);
gsl_matrix_free (covar);

QString label="Fit"+QString::number(fitID)+"_2";
long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(red,1)); 

label="Fit"+QString::number(fitID)+"_1(X),"+label+"(Y)";
associations<<label;

	cOnPlot++;
	keys.resize(cOnPlot);
	keys[cOnPlot-1]=curveID;
	
	if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		mrk->setText(legendText());
	}
	
	int n1=n;
	if(n<100)	
		n1=100;
	
	double X0,XN, step;
	
	int imaxID=gsl_vector_max_index (XX);
	int iminID=gsl_vector_min_index (XX);
  	gsl_vector_free (XX);
	
	X0=X[iminID];
	XN=X[imaxID];
	step=(XN-X0)/(n1-1);
	free_vector(X,0,n-1);free_vector(y,0,n-1);
	X=vector(0,n1-1);y=vector(0,n1-1);
	for (i=0;i<n1;i++)
		{
		X[i]=X[0]+i*step;
		double diff=X[i]-C;
		y[i]=A*w/(4*diff*diff+w*w)+B;
		}
	myPlot->setCurveData(curveID,X,y,n1);
	QString text="1\t2\n";
	for (i=0;i<n1;i++)
	{
	text+=QString::number(X[i]);
	text+="\t";
	text+=QString::number(y[i]);
	text+="\n";
	}
	free_vector(X,0,n1-1); free_vector(y,0,n1-1);
	QString caption="Fit"+QString::number(fitID);
	emit createTable(caption,n1,2,text);
	
	for (i=0;i<4;i++)
		myPlot->setAxisAutoScale(i);
	myPlot->replot();
	updateScale();

	curveLayout cl = initCurveLayout();
	layout.resize(cOnPlot);
	layout[cOnPlot-1]=cl;

return info;
}

QString Graph::fitPolynomial(const QString& name,int order,int points,
				double start, double end, bool showFormula)
{
int m=order+1;
QStringList curvesList = plotCurvesList();	
int index=curvesList.findIndex(name);
QwtPlotCurve *curv=myPlot->curve(keys[index]);
if (!curv)
	return "";

int i,j;
int size=curv->dataSize();
int iStart=0, iEnd=size-1;

for (i=0;i<size;i++)
	{
	if (curv->x(i)>=start)
		{
		iStart=i;
		break;
		}
	}
	
	for (i=iEnd;i>=0;i--)
	{
	if (curv->x(i)<=end)
		{
		iEnd=i;
		break;
		}
	}
	
const size_t n=iEnd-iStart+1;	
if ((int)n<m)
	{
	QMessageBox::critical(this,"QtiPlot - Warning",
				"You need at least "+QString::number(m)+" points to fit!");
	return "";
	}
	
double xi, yi, ei=exp(0), chisq;
gsl_matrix *X, *cov;
gsl_vector *y, *w, *c,*x;

X = gsl_matrix_alloc (n, m);
x = gsl_vector_alloc (n);
y = gsl_vector_alloc (n);
w = gsl_vector_alloc (n);

c = gsl_vector_alloc (m);
cov = gsl_matrix_alloc (m,m);

for (i = 0; i <(int)n; i++)
    {
		xi=curv->x(i+iStart);
		yi = curv->y(i+iStart);// This is the data to be fitted 
		
		for (j= 0; j < m; j++)
     		gsl_matrix_set (X, i, j, pow(xi,j));
    
		gsl_vector_set (x, i, xi);	
    	gsl_vector_set (y, i, yi);
    	gsl_vector_set (w, i, 1.0/(ei*ei));
    }
		
gsl_multifit_linear_workspace * work 
    = gsl_multifit_linear_alloc (n, m);
gsl_multifit_wlinear (X, w, y, c, cov,
                          &chisq, work);

QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();
QString date=dt.toString ("dd/MM/yyyy ");
date+=time.toString();

QString t=": y=";	
for (j= 0; j < m; j++)
		{
		t+="a"+QString::number(j);
		if (j>0)
			t+="*X";
		if (j>1)
			t+="^"+QString::number(j);
		t+="+";
		}
t=t.left(t.length()-1);

double cj;
double x0=gsl_vector_get(x,0);
double xn=gsl_vector_get(x,n-1);		
double step=double (xn-x0) / double (points-1); 
		
QString info=date+"\t"+this->caption()+"\nPolynomial fit of "+curv->title()+t;
info+="\nFrom x="+QString::number(x0) +" to x="+QString::number(xn)+"\n";
QString infoA;
for (i = 0; i < m; i++)
    {
	double ci=gsl_vector_get(c,i);
	infoA.sprintf("a%d = %.5f +/- %.5f\n",i,ci,sqrt(gsl_matrix_get(cov,i,i)));
	info+=infoA;
	}
info+="-------------------------------------------------------------\n";

gsl_multifit_linear_free (work);
gsl_matrix_free (X);
gsl_matrix_free (cov);
gsl_vector_free (y);
gsl_vector_free (x);
gsl_vector_free (w);
		
QString label="Fit"+QString::number(fitID)+"_2";
long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(red,1)); 

label="Fit"+QString::number(fitID)+"_1(X),"+label+"(Y)";
associations<<label;

cOnPlot++;
keys.resize(cOnPlot);
keys[cOnPlot-1]=curveID;

if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		mrk->setText(legendText());
	}

	QArray<double> a(points), b(points);
	
	QString text="1\t2\n";
	for (i=0;i<points;i++)
		{
		xi=x0+i*step;
		yi=0.0;
		for (j=0; j<m;j++)
			{
			cj=gsl_vector_get(c,j);
			yi+=cj*pow(xi,j);
			}
		a[i]=xi;
		text+=QString::number(xi)+"\t";
		b[i]=yi;
		text+=QString::number(yi)+"\n";
		}
		
	myPlot->setCurveData(curveID,a,b,points);
	
	QString caption="Fit"+QString::number(fitID);
	emit createTable(caption, points,2,text);

for (i=0;i<4;i++)
	myPlot->setAxisAutoScale(i);
myPlot->replot();
updateScale();
	
if (showFormula)
	{
	LegendMarker* aux= new LegendMarker(myPlot);
	QRect rect=QRect(10,10,10,10);
	if (legendMarkerID>=0)
		{
		LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
		QPoint p=mrk->rect().bottomLeft();
		rect.setTopLeft(QPoint(p.x(),p.y()+10));
		}
	aux->setOrigin(rect.topLeft());
	aux->setBackground(2);
		
	QString s;
	cj=gsl_vector_get(c,0);
	t=QString::number(cj);
		
	s.sprintf("%.5f",cj);
	if (s == "-0.00000" || s == "0.00000")
			t="";
	for (j= 1; j < m; j++)
		{
		cj=gsl_vector_get(c,j);
		s.sprintf("%.5f",cj);
		if (s != "-0.00000" && s != "0.00000")
			{
			if (cj>0 && !t.isEmpty())
				t+="+";
			if (s != "1.00000")
				t+=QString::number(cj);
			t+="X<sup>";
			if (j>1)
				t+=QString::number(j);
			t+="</sup>";
			}
		}
	aux->setText(t);	
	myPlot->insertMarker(aux);
	myPlot->replot();		
	}

curveLayout cl = initCurveLayout();
layout.resize(cOnPlot);
layout[cOnPlot-1]=cl;

gsl_vector_free (c);
return info;
}

void Graph::calculateLineProfile(const QPoint& start, const QPoint& end)
{	
ImageMarker* mrk=(ImageMarker*) myPlot->marker(selectedMarker);
if (!mrk)
	{
	QMessageBox::warning(0,tr("QtiPlot - Pixel selection warning"),  
						"Please select the start line point inside the image rectangle!");
	linesOnPlot--;
	myPlot->removeMarker(lines[linesOnPlot]);
	lines.resize(linesOnPlot);
	return;
	}

QRect rect=mrk->rect();
if (!rect.contains(start) || !rect.contains(end))
	{
	QMessageBox::warning(0,tr("QtiPlot - Pixel selection warning"),  
						"Please select the end line point inside the image rectangle!");
	linesOnPlot--;
	myPlot->removeMarker(lines[linesOnPlot]);
	lines.resize(linesOnPlot);		
	return;		
	}
	
QPoint o=mrk->getOrigin();
QPixmap pic=mrk->image();
QImage image=pic.convertToImage();
lineProfileOn=FALSE;
	
int x1=start.x()-o.x();
int x2=end.x()-o.x();
int y1=start.y()-o.y();
int y2=end.y()-o.y();
	
QSize realSize=pic.size();
QSize actualSize=mrk->size();
	
if (realSize != actualSize)
	{
	double ratioX= (double)realSize.width()/(double)actualSize.width();
	double ratioY= (double)realSize.height()/(double)actualSize.height();
	x1=int(x1*ratioX); 
	x2=int(x2*ratioX);
	y1=int(y1*ratioY); 
	y2=int(y2*ratioY); 	
	}

QString text="pixel\tx\ty\tintensity\n";
//uses the fast Bresenham's line-drawing algorithm
#define sgn(x) ((x<0)?-1:((x>0)?1:0)) 	
int i,dx,dy,sdx,sdy,dxabs,dyabs,x,y,px,py,n;

  dx=x2-x1;      //the horizontal distance of the line 
  dy=y2-y1;      //the vertical distance of the line 
  dxabs=abs(dx);
  dyabs=abs(dy);
  sdx=sgn(dx);
  sdy=sgn(dy);
  x=dyabs>>1;
  y=dxabs>>1;
  px=x1;
  py=y1;

if (dxabs>=dyabs) //the line is more horizontal than vertical 
  {
    for(i=0;i<dxabs;i++)
    {
      y+=dyabs;
      if (y>=dxabs)
      {
        y-=dxabs;
        py+=sdy;
      }
      px+=sdx;
	  
	n=dxabs;
	text+=QString::number(i)+"\t";
	text+=QString::number(px)+"\t";
	text+=QString::number(py)+"\t";
	 text+=QString::number(averageImagePixel(image, px, py, averagePixels, TRUE))+"\n"; 
    }
  }
  else // the line is more vertical than horizontal 
  {
    for(i=0;i<dyabs;i++)
    {
      x+=dxabs;
      if (x>=dyabs)
      {
        x-=dyabs;
        px+=sdx;
      }
      py+=sdy;
    
    n=dyabs;
	text+=QString::number(i)+"\t";	  
	text+=QString::number(px)+"\t";
	text+=QString::number(py)+"\t";
	text+=QString::number(averageImagePixel(image, px, py, averagePixels, FALSE))+"\n"; 
    }
  }
QString caption="table1";
emit createTablePlot(caption,n,4, text);
}

int Graph::averageImagePixel(const QImage& image, int px, int py, int average, bool moreHorizontal)
{
QRgb pixel;
int sum=0,start,i;
int middle=int(0.5*(average-1));
if (moreHorizontal)
	{
	start=py-middle;

	for(i=0;i<average;i++)
    	{
		pixel= image.pixel(px,start+i);	
		sum+=qGray(pixel);
		}
	}
else
	{
	start=px-middle;
	for(i=0;i<average;i++)
    	{
		pixel= image.pixel(start+i,py);	
		sum+=qGray(pixel);
		}
	}
return sum/average;
}

void Graph::calculateProfile(int average, bool ok)
{
lineProfileOn=ok;	
if (average % 2 == 0)
	averagePixels=average+1;
else
	averagePixels=average;
}
