#ifndef APPLICATION_H
#define APPLICATION_H

#include <qmainwindow.h>
#include <qlist.h>
#include <qlistview.h> 
#include <qwt_plot.h>

#include "graph.h"
#include "worksheet.h"
#include "graph3D.h"
#include "plot3DDialog.h"

class QAction;
class QActionGroup;
class QTextEdit;
class QToolBar;
class QToolButton;
class QPopupMenu;
class QWorkspace;
class QPopupMenu;
class QListView;
class QToolButton;
class QListBoxItem;
class QTextBrowser;
class QLineEdit;
class Graph;
class ScalePicker;
class Graph3D;
class plot3DDialog;
class MultiLayer;
class fitDialog;
	
class ApplicationWindow: public QMainWindow
{
    Q_OBJECT
public:
    ApplicationWindow();
	ApplicationWindow(const QString& filename);
	~ApplicationWindow();

	QTextEdit *results;
	QWorkspace* ws;
    QToolBar *fileTools, *plotTools, *tableTools, *plot3DTools, *displayBar;
    QPopupMenu *windowsMenu,*view,*graph,*file,*format,*calcul,*edit,*dataMenu,*recent;
	QPopupMenu *help,*type,*import,*plot2D,*plot3D, *specialPlot, *panels,*stat, *inter, *fft,*decay;
	QListView *lv;
	QToolButton *btnList,*btnResults, *fileSave, *btnUndo, *btnRedo, *btnText;
	QWidgetList *hiddenWindows;
	QTextBrowser *browser;
	QLineEdit *info;
	QWidget *lastModified;

	QActionGroup* dataTools;
	QAction *btnCursor, *btnSelect, *btnPicker, *btnRemovePoints, *btnMovePoints;
	QAction  *btnZoom, *btnPointer, *btnLine;
	
	QActionGroup* coord;
	QAction* Box;
    QAction* Frame;
    QAction* None;

	QActionGroup* grids;
    QAction* front;
    QAction* back;
    QAction* right;
    QAction* left;
    QAction* ceil;
    QAction* floor;

	QActionGroup* floorstyle;
    QAction* floordata;
    QAction* flooriso;
    QAction* floornone;

	QActionGroup* plotstyle;
    QAction* wireframe;
    QAction* hiddenline;
    QAction* polygon;
    QAction* filledmesh;
    QAction* pointstyle;
	QAction* barstyle;
	QAction *conestyle, *crossHairStyle;
	Graph *activeGraph, *selectorsActiveGraph, *lastCopiedLayer;
	fitDialog *fd;
	
public slots:
	void setSaveSettings(bool autoSaving, int min);
	void initAppStyle(const QString& s);
	void changeAppStyle(const QString& s);
	void changeAppFont(const QFont& f);
	void updateAppFonts();
	void setAppColors(const QColor& wc,const QColor& pc,const QColor& tpc);

	MultiLayer* plotMultiLayers(int c, int r, int style);
	MultiLayer* multilayerPlot(WksheetWindow* w,const QStringList& colList, int style);
	MultiLayer* multilayerPlot(const QString& caption);
	MultiLayer* multilayerPlot(const QStringList& colList); //for plot wizard
	void connectMultilayerPlot(MultiLayer *g);

	void plot2VerticalLayers();
	void plot2HorizontalLayers();
	void plot4Layers();
	void plotStackedLayers();
	void plotStackedHistograms();

	//3D data plots
	Graph3D* dataPlot3D(WksheetWindow* table,const QString& colName);
	Graph3D* dataPlotXYZ(WksheetWindow* table,const QString& zColName, int type);
		//when reading from .qti file
	Graph3D* dataPlot3D(const QString& caption,const QString& formula,
											 double xl, double xr, double yl, double yr, double zl, double zr);
	Graph3D* dataPlotXYZ(const QString& caption,const QString& formula,
 												double xl, double xr, double yl, double yr, double zl, double zr);
	/*!
	* used when plotting from the wizard
	*/
	Graph3D* dataPlotXYZ(const QString& formula);
	Graph3D* dataPlot3D(const QString& formula);

	//surface plots
	Graph3D* newPlot3D(const QString& formula, double xl, double xr,
					   double yl, double yr, double zl, double zr);

	Graph3D* newPlot3D(const QString& caption,const QString& formula, 
					   double xl, double xr,double yl, double yr, 
					   double zl, double zr);
	Graph3D* copySurfacePlot();
	Graph3D* surfacePlot(const QString& name);
	void connectSurfacePlot(Graph3D *plot);
	void newSurfacePlot();
	void addSurfacePlot();
	void remove3DPlot();
	void add3DData();
	void insertNew3DData(const QString& colName);

	//multilayer plots
	MultiLayer* plot(const QString& name);
	MultiLayer* copyGraph();
	void clone(Graph* g, Graph* g2);

	void addLayer();
	void deleteLayer();

	//user-defined functions
	void newFunctionPlot();
	void newFunctionPlot(QString& type,QStringList &formulas,QStringList &vars,QValueList<double> &ranges,QValueList<int> &points);

	void addFunctionCurve();
	void addFunctionCurve(QString&,QStringList &,QStringList &,QValueList<double> &,QValueList<int> &);
	void clearFunctionsList();
	void clearFitFunctionsList();
	void clearSurfaceFunctionsList();
	void clearLogInfo();
	void clearParamFunctionsList();
	void clearPolarFunctionsList();

	WksheetWindow* newWrksheet();
	WksheetWindow* newWrksheet(const QString& fname,const QString &sep, int lines);
	WksheetWindow* newWrksheet(const QString& caption,int r, int c);
	WksheetWindow* newWrksheet(const QString& caption, int r, int c,const QString& text);
	WksheetWindow* table(const QString& name);
	WksheetWindow* copyTable();
	void connectTable(WksheetWindow* w);
	void newWrksheetPlot(const QString& caption,int r, int c, const QString& text);
	void showHistogramTable(const QString& caption, int r, int c, const QString& text);
	void initTable(WksheetWindow* w, const QString& caption);
	void customTable(WksheetWindow* w);
	void customizeTables(const QColor& bgColor,const QColor& textColor,
										const QColor& headerColor,const QFont& textFont, const QFont& headerFont);
	
	void customGraph(Graph* g);
	void setGraphDefaultSettings(int ticks, int legendFrame, bool title, bool canvasFrame,
														const QFont& axesFnt,const QFont& numFnt, 
														const QFont& legendFnt, const QFont& titleFnt);
	
	void plot3DRibbon();
	void plot3DScatter();
	void plot3DTrajectory();
	void plot3DBars();

	void plotL();
	void plotP();
	void plotLP();
	void plotPie();
	void plotVerticalBars();
	void plotHorizontalBars();
	void plotArea();
	void plotSteps();
	void plotSpline();
	void plotVerticalDropLines();
	void plotHistogram();
	void plotVectXYXY();
	
	//image analysis
	void intensityTable();
	WksheetWindow* createIntensityTable(const QPixmap& pic);
	void pixelLineProfile();
	void loadImage();
	
	void plotFile(const QString& fn);
    void load();
	void loadMultiple();
    void exportGraph();
	void saveProjectAs();
	void saveProject();
	void readSettings();
	void saveSettings();
	void modifiedProject();
	void modifiedProject(QWidget *w);
    void print();
	void printHelp();
	void setImportOptions(const QString& sep, int lines);
    
	void showList();
	QStringList columnsList();
	QStringList columnsListXY();
	
	void undo();
	void redo();
	
	QWidget* copyWindow();
	QWidget* window(QListViewItem *it);
	void rename();
	void renameWindow();
	void maximizeWindow(QListViewItem * lbi);
	bool minimized(QWidget* window);
	void checkMinimizedWindows();
	void minimizeWindow();
	void maximizeWindow();
	bool maximized(QWidget* window);
	bool hidden(QWidget* window);
	void resizedWindow(QWidget* window);
	void closeActiveWindow();
	void closeWindow(QWidget* window);
	void hideWindow(QWidget* window);
	void hideWindow();
	void hideActiveWindow();
	void activateWindow();
	void printWindow();
	void updateTable(const QString& caption,int row,const QString& text);
    void about();
    void windowsMenuAboutToShow();
    void windowsMenuActivated( int id );
	void addCurve(Graph* g,const QString& name);
	void removeColFromGraphs(QString& name);
	QStringList dependingPlots(const QString& caption);
	void resizeActiveWindow();
	void resizeWindow();
	
	// list view in project explorer
	void updateListView(const QString& caption);
	void setListView(const QString& caption,const QString& view);
	void renameListViewItem(const QString& oldName,const QString& newName);
	void setListViewDate(const QString& caption,const QString& date);
	QString listViewDate(const QString& caption);
	void setListViewSize(const QString& caption,const QString& size);
	
	void updateCurvesDialogs(const QString& old, const QString& newName);
	void updateCurveInGraphs(const QString& name);
	
	void showWorksheet(const QString& curve);
	void addColToTable();
	void cutSelection();
	void copySelection();
	void copyMarker();
	void pasteSelection();
	void clearSelection();
	void clearCellFromTable(const QString& name,double value);
	void copyActiveLayer();
	
	void newProject();
	void open();
	bool open(const QString& fn);
	void openQtiFile_03(const QString& fn);
	void openQtiFile(const QString& fn);
	void openTable(ApplicationWindow* app, const QStringList &flist);
	void openSurfacePlot(ApplicationWindow* app, const QStringList &list);
	void openGraph(ApplicationWindow* app, MultiLayer *plot, const QStringList &list);

	void openRecentProject(int index);
	void init();
	void initMainMenu();
	void initToolBars();
	void initPlot3DToolBar();
	void disableMenus();
	
	void customToolBars(QWidget* w);
	void customMenu(QWidget* w);
	void windowActivated(QWidget *w);

	// plot tools 
	void newLegend();
	void drawLine();
	void addText();
	void disableAddText();
	void addImage();
	void zoom();
	void unzoom();
	void pickDataTool( QAction* action );

	//interpolation
	void interlinear();
	void akima();
	void intercubic();
	
	//fitting
	void fitLinear();
	void fitExpDecay();
	void fitExpDecay(const QString&, double, double, double);
	void fitExpDecay2(const QString&,double,double,double,double);
	void fitExpDecay3(const QString& name, double firstTime, double secondTime,
						double thirdTime, double start, double yOffset);
	void fitExpGrowth();
	void fitGauss();
	void fitLorentz();
	void fitPolynomial(const QString&,int,int, double, double, bool);
	void fitCurve(const QString& curve,const QString& formula,
				 const QStringList& params,const QStringList& paramsInit,
				 double from,double to,int points, int solver, double tolerance);
				 
	//calculus
	void integrate(int curve, int order, int iter, double tol, double low, double up);
	void differentiate();
	void fft_for();
	void fft_inv();
	void analysis(const QString& whichFit);
	void analyzeCurve(const QString& curve);
	void analyzeCurve(const QString& whichFit, long curveKey, int points);
	void showAnalysisDialog();

	void addErrorBars();
	void defineErrorBars(const QString& name,int type,const QString& percent,int direction);
	void defineErrorBars(const QString& curveName,const QString& errColumnName, int direction);
	void movePoints();
	void removePoints();

	// event handlers 
	void closeEvent( QCloseEvent*);
	void showEvent ( QShowEvent *);
	void hideEvent ( QHideEvent *);
	void timerEvent ( QTimerEvent *e);
	
	//dialogs
	void showRangeSelectors();	
	void showCursor();
	void showPicker();
	void showLayoutDialog();
	void showXAxesDialog();
	void showYAxesDialog();
	void showResults();
	void showTextDialog();
	void showLineDialog();
	void showTitleDialog();
	void showExportASCIIDialog();
	void showCurvesDialog();
	void showXAxisTitleDialog();
	void showYAxisTitleDialog();
	void showRightAxisTitleDialog();
	void showTopAxisTitleDialog();
	void showTableOptionsDialog();
	void showRowsDialog();
	void showRowMenu(int r);
	void showColMenu(int c);
	void showColumnValuesDialog();	
	void showGraphContextMenu();
	void showWindowContextMenu();
	void showWindowPopupMenu(QListViewItem *it, const QPoint &p, int);
	void showMoreWindows();
	void showImportDialog();
	void showMarkerPopupMenu();
	void showHelp();
	void showPieDialog();
	void showPlotWizard();
	void showFitPolynomDialog();
	void showIntDialog();
	void showExpDecayDialog();
	void showTwoExpDecayDialog();	
	void showExpDecay3Dialog();
	void showRowStatistics();
	void showColStatistics();
	void showFitDialog();
	plot3DDialog* showPlot3dDialog();
	void showImageDialog();
	void showPlotGeometryDialog();
	void showLayerDialog();
	void showConfigureDialog();
	
	void updateConfirmOptions(bool askTables, bool askPlots2D, bool askPlots3D);
	void updateHistogram(const QString& curveName, int curveIndex, 
										bool automatic, double binSize, double begin, double end);
	void showHistogramStats(const QString& curveName, int curveIndex);

	void updateVectorsLayout(int curve, int colorIndex, int width, int arrowLength, int arrowAngle, 
									 bool filled, const QString& xEndColName, const QString& yEndColName);

	void showAxis(int axis, int type, const QString& labelsColName, bool axisOn, int ticksType,
		bool labelsOn, const QColor& c, int format, int prec, int rotation);
	//plot3D tools
	void update3DToolBar();
	
	void setFramed3DPlot();
	void setBoxed3DPlot();
	void removeAxes3DPlot();
	void removeGrid3DPlot();
	void setHiddenLineGrid3DPlot();
	void setLineGrid3DPlot();
	void setPoints3DPlot();
	void setCrosses3DPlot();
	void setCones3DPlot();
	void setBars3DPlot();
	void setFilledMesh3DPlot();
	void setEmptyFloor3DPlot();
	void setFloorData3DPlot();
	void setFloorIso3DPlot();
	
	void setFloorGrid3DPlot(bool on);
	void setCeilGrid3DPlot(bool on);

	void setRightGrid3DPlot(bool on);
	void setLeftGrid3DPlot(bool on);
	void setFrontGrid3DPlot(bool on);
	void setBackGrid3DPlot(bool on);
	
	void pickPlotStyle( QAction* action );
	void pickCoordSystem( QAction* action);
	void pickFloorStyle( QAction* action);
	void custom3DActions(QWidget *w);
	void custom3DGrids(int grids);
	
signals:
	void windowClosed(const QString&);
	void modified();
	
private:
	double fileVersion;
	QColor workspaceColor, panelsColor, panelsTextColor;
	QString appStyle;
	bool confirmCloseTable, confirmClosePlot2D, confirmClosePlot3D;
	bool canvasFrameOn, titleOn, autoSave;
	int ticksStyle, legendFrameStyle, autoSaveTime;
	QFont appFont;
	QFont tableTextFont, tableHeaderFont, plotAxesFont, plotLegendFont, plotNumbersFont, plotTitleFont;
	QColor tableBkgdColor, tableTextColor, tableHeaderColor;
	QString projectname,separator, helpFilePath, analysisType, configFilePath, logInfo;
	int exploreID,logID,printID,asciiID,closeID,moveID,removeID;
	int exportID,plotID,axesID,titleID,undoID, redoID;
	int graphs,tables, fitNumber, ignoredLines, timerId, savingTimerId;
	bool listViewOn,drawLineOn, copiedLayer;
	QStringList plotWindows,tableWindows, recentProjects, plot3DWindows;
	bool cursorOn,textOn,resultsOn,pickerOn,saved,movePointsOn, removePointsOn, selectOn, zoomOn;

	QStringList functions; //user-defined functions;
	QStringList xFunctions, yFunctions, rFunctions,tetaFunctions; // user functions for parametric and polar plots
	QStringList fitFunctions; //user-defined fit functions;
	QStringList surfaceFunc; //user-defined surface functions;
	QWidget *aw, *wmax; //active window and maximized window when loading a .qti file
	int maxt,maxg,maxFit; //aux when opening .qti file
	Graph::MarkerType copiedMarkerType;
	
//variables used when user copy/paste markers
	QString auxMrkText;
	QFont auxMrkFont;
	QColor auxMrkColor;
	QPoint auxMrkStart,auxMrkEnd;
	Qt::PenStyle auxMrkStyle;
	QString auxMrkFileName;
	int auxMrkBkg,auxMrkWidth;
	bool startArrowOn, endArrowOn, fillArrowHead;
	int arrowHeadLength, arrowHeadAngle;
};

#endif
