#include "configDialog.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qfont.h>
#include <qfontdialog.h> 
#include <qcolordialog.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qspinbox.h>
#include <qstylefactory.h>

configDialog::configDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "configDialog" );
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( 310, 140 ) );
	setMaximumSize( QSize( 310, 140 ) );
    setMouseTracking( TRUE );
    setSizeGripEnabled( FALSE );

	generalDialog = new QTabWidget( this, "generalDialog" );
	
	plots = new QWidget( generalDialog, "plots" );
	
	QButtonGroup *GroupBox4 = new QButtonGroup( 2,QGroupBox::Horizontal,tr("Options"),plots,"GroupBox4" );
	new QLabel( tr( "Ticks" ), GroupBox4, "TextLabel489",0 ); 
	boxTicks = new QComboBox(GroupBox4,"ticks");
	boxTicks->insertItem(tr("In"));
	boxTicks->insertItem(tr("Out"));
	boxTicks->insertItem(tr("Both"));
	boxTicks->insertItem(tr("None"));
	
	new QLabel( tr( "Legend frame" ), GroupBox4, "TextLabel689",0 ); 
	boxLegend = new QComboBox(GroupBox4,"legend");
	boxLegend->insertItem( tr( "None" ) );
	boxLegend->insertItem( tr( "Rectangle" ) );
    boxLegend->insertItem( tr( "Shadow" ) );
    boxLegend->insertItem( tr( "White out" ) );
    boxLegend->insertItem( tr( "Black out" ) );
	
	new QLabel( tr( "Show title" ), GroupBox4, "TextLabel468",0 );   
    boxTitle= new QCheckBox("",GroupBox4,"title");
	new QLabel( tr( "Draw canvas frame" ), GroupBox4, "TextLabel4689",0 );  
	boxFrame= new QCheckBox("",GroupBox4,"frame");
	
	QButtonGroup *GroupBox5 = new QButtonGroup( 1,QGroupBox::Horizontal,tr("Fonts"),plots,"GroupBox5" );
	buttonTitleFont= new QPushButton(GroupBox5, "buttonTitleFont" );  
	buttonLegendFont= new QPushButton(GroupBox5, "buttonLegendFont" );	 
    buttonAxesFont= new QPushButton(GroupBox5, "buttonAxesFont" );   
	buttonNumbersFont= new QPushButton(GroupBox5, "buttonNumbersFont" );

	QHBoxLayout* hlayout2 = new QHBoxLayout(plots,5,5, "hlayout1");
    hlayout2->addWidget(GroupBox4);
	hlayout2->addWidget(GroupBox5);
	
	generalDialog->insertTab(plots, tr( "2D &Plots" ) );
	
	tables = new QWidget( generalDialog, "tables" );
	
	GroupBox1 = new QButtonGroup( 1,QGroupBox::Horizontal,tr("Colors"),tables,"GroupBox1" );
    buttonBackground= new QPushButton(GroupBox1, "buttonBackground" );
    buttonText= new QPushButton(GroupBox1, "buttonText" );
	buttonHeader= new QPushButton(GroupBox1, "buttonHeader" );
	
	GroupBox3 = new QButtonGroup( 1,QGroupBox::Horizontal,tr("Fonts"),tables,"GroupBox3" );
    buttonTextFont= new QPushButton(GroupBox3, "buttonTextFont" );
	buttonHeaderFont= new QPushButton(GroupBox3, "buttonHeaderFont" );
	
	QHBoxLayout* hlayout1 = new QHBoxLayout(tables,5,5, "hlayout1");
    hlayout1->addWidget(GroupBox1);
	hlayout1->addWidget(GroupBox3);
	
	generalDialog->insertTab(tables, tr( "&Tables" ) );
		
	confirm = new QWidget( generalDialog, "confirm" );
	
	QButtonGroup *GroupBox6 = new QButtonGroup( 1,QGroupBox::Horizontal,tr("Prompt on closing"),confirm,"GroupBox6" );
    boxTables= new QCheckBox(tr("Tables"),GroupBox6, "boxTables" );
    boxPlots2D= new QCheckBox(tr("2D Plots"),GroupBox6, "boxPlots2D" );
	boxPlots3D= new QCheckBox(tr("3D Plots"),GroupBox6, "boxPlots3D" );
	
	QHBoxLayout* hlayout3 = new QHBoxLayout(confirm,5,5, "hlayout3");
    hlayout3->addWidget(GroupBox6);
	
	generalDialog->insertTab(confirm, tr( "&Confirmations" ) );

	application = new QWidget( generalDialog, "application" );
	QButtonGroup *GroupBox7 = new QButtonGroup( 2,QGroupBox::Horizontal,tr("General"),application,"GroupBox7" );
    
	new QLabel( tr( "Style" ), GroupBox7, "TextLabel690",0 ); 
	boxStyle = new QComboBox(GroupBox7,"style");
	QStringList styles = QStyleFactory::keys();
	styles.sort();
	boxStyle->insertStringList(styles);
   
    new QLabel( tr( "Fonts" ), GroupBox7, "TextLabel699",0 ); 
	fontsBtn= new QPushButton(tr("Choose &font"),GroupBox7, "fontsBtn" );
	
	boxSave= new QCheckBox(tr("Save every"),GroupBox7, "boxSave" );
	boxMinutes = new QSpinBox(1,100,1,GroupBox7, "minutes");
	boxMinutes->setSuffix(" minutes");
	
	QButtonGroup *GroupBox8 = new QButtonGroup( 1,QGroupBox::Horizontal,tr("Colors"),application,"GroupBox7" );
 	btnWorkspace = new QPushButton(GroupBox8, "btnWorkspace" );
	btnPanels = new QPushButton(GroupBox8, "btnPanels" );
	btnPanelsText = new QPushButton(GroupBox8, "btnPanelsText" );
	
	QHBoxLayout* hlayout4 = new QHBoxLayout(application,5,5, "hlayout4");
    hlayout4->addWidget(GroupBox7);
	hlayout4->addWidget(GroupBox8);
	
	generalDialog->insertTab(application, tr( "&Application" ) );
	
	GroupBox2 = new QButtonGroup( 3,QGroupBox::Horizontal,tr(""),this,"GroupBox2" );
	GroupBox2->setFlat(TRUE);
	GroupBox2->setLineWidth(0);
	
	buttonApply = new QPushButton(GroupBox2, "buttonApply" );
	
	buttonOk = new QPushButton(GroupBox2, "buttonOk" );
    buttonOk->setDefault( TRUE );
   
    buttonCancel = new QPushButton(GroupBox2, "buttonCancel" );
	
	QVBoxLayout* hlayout = new QVBoxLayout(this,5,5, "hlayout");
    hlayout->addWidget(generalDialog);
	hlayout->addWidget(GroupBox2);

    languageChange();
   
    // signals and slots connections
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( buttonApply, SIGNAL( clicked() ), this, SLOT( update() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( buttonBackground, SIGNAL( clicked() ), this, SLOT( pickBgColor() ) );
	connect( buttonText, SIGNAL( clicked() ), this, SLOT( pickTextColor() ) );
	connect( buttonHeader, SIGNAL( clicked() ), this, SLOT( pickHeaderColor() ) );
	connect( buttonTextFont, SIGNAL( clicked() ), this, SLOT( pickTextFont() ) );
	connect( buttonHeaderFont, SIGNAL( clicked() ), this, SLOT( pickHeaderFont() ) );
	connect( buttonAxesFont, SIGNAL( clicked() ), this, SLOT( pickAxesFont() ) );
	connect( buttonNumbersFont, SIGNAL( clicked() ), this, SLOT( pickNumbersFont() ) );
	connect( buttonLegendFont, SIGNAL( clicked() ), this, SLOT( pickLegendFont() ) );
	connect( buttonTitleFont, SIGNAL( clicked() ), this, SLOT( pickTitleFont() ) );
	connect( fontsBtn, SIGNAL( clicked() ), this, SLOT( pickApplicationFont() ) );
	connect( boxSave, SIGNAL( toggled(bool) ), boxMinutes, SLOT( setEnabled(bool) ) );
	connect( btnWorkspace, SIGNAL( clicked() ), this, SLOT( pickWorkspaceColor() ) );
	connect( btnPanels, SIGNAL( clicked() ), this, SLOT( pickPanelsColor() ) );
	connect( btnPanelsText, SIGNAL( clicked() ), this, SLOT( pickPanelsTextColor() ) );
}

configDialog::~configDialog()
{
}

void configDialog::languageChange()
{
    setCaption( tr( "QtiPlot - Choose default settings" ) );
    buttonOk->setText( tr( "&OK" ) );
	buttonCancel->setText( tr( "&Cancel" ) );
	buttonApply->setText( tr( "&Apply" ) );
	buttonBackground->setText( tr( "&Background" ) );
	buttonText->setText( tr( "&Text" ) );
	buttonHeader->setText( tr( "&Labels" ) );
	buttonTextFont->setText( tr( "&Text Font" ) );
	buttonHeaderFont->setText( tr( "&Labels Font" ) );
	buttonAxesFont->setText( tr( "A&xes Legend" ) );
	buttonNumbersFont->setText( tr( "Axes &Numbers" ) );
	buttonLegendFont->setText( tr( "&Legend" ) );
	buttonTitleFont->setText( tr( "T&itle" ) );
	btnWorkspace->setText( tr( "&Workspace" ) );
	btnPanels->setText( tr( "Pa&nels" ) );
	btnPanelsText->setText( tr( "Panels Te&xt" ) );
}

void configDialog::accept()
{
update();
close();
}

void configDialog::update()
{
if (generalDialog->currentPage()==(QWidget*)tables)
	emit updateTables(bgColor, textColor, headerColor, textFont, headerFont);
else if (generalDialog->currentPage()==(QWidget*)plots)
	{
	emit setGraphDefaultSettings(ticksType(), boxLegend->currentItem(), 
					boxTitle->isChecked(), boxFrame->isChecked(), 
					axesFont, numbersFont, legendFont, titleFont);
	}
else if (generalDialog->currentPage()==(QWidget*)application)
	{
	emit changeAppFont(appFont);
	emit changeAppStyle(boxStyle->currentText());
	emit setSaveSettings(boxSave->isChecked(), boxMinutes->value());
	emit setAppColors(workspaceColor, panelsColor, panelsTextColor);
	}
else
	updateConfirmOptions(boxTables->isChecked(), boxPlots2D->isChecked(), boxPlots3D->isChecked());
}

QColor configDialog::pickBgColor()
{
QColor c = QColorDialog::getColor(bgColor, this );
if ( !c.isValid() )
		return bgColor;

bgColor = c;
return bgColor;
}

QColor configDialog::pickTextColor()
{
QColor c = QColorDialog::getColor(textColor, this );
if ( !c.isValid() )
		return textColor;

textColor = c;
return textColor;
}

QColor configDialog::pickHeaderColor()
{
QColor c = QColorDialog::getColor(headerColor, this );
if ( !c.isValid() )
		return headerColor;

headerColor = c;
return headerColor;
}

void configDialog::pickTextFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,textFont,this);
    if ( ok ) {
        textFont = font;
    } else {
     return;
    }
}

void configDialog::pickHeaderFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,headerFont,this);
    if ( ok ) {
        headerFont = font;
    } else {
     return;
    }
}

void configDialog::initColors(const QColor& bgCol, const QColor& textCol, const QColor& headerCol)
{
bgColor=bgCol;
textColor=textCol;
headerColor=headerCol;
}

void configDialog::initFonts(const QFont& textFnt, const QFont& headerFnt)
{
textFont=textFnt;
headerFont=headerFnt;	
}

void configDialog::initPlotOptions(int ticksStyle, int legendStyle, bool titleOn, bool frameOn)
{
switch (ticksStyle)
	{
	case -1:
		boxTicks->setCurrentItem(0);
	break;

	case 1:
		boxTicks->setCurrentItem(1);
	break;

	case 2:
		boxTicks->setCurrentItem(2);
	break;

	case 0:
		boxTicks->setCurrentItem(3);
	break;	
	}
boxLegend->setCurrentItem(legendStyle);
boxTitle->setChecked(titleOn);	

boxFrame->setChecked(frameOn);	
}

int configDialog::ticksType()
{
int ticks=-1;
switch (boxTicks->currentItem())
	{
	case 0:
	break;

	case 1:
		ticks=1;
	break;

	case 2:
		ticks=2;
	break;

	case 3:
		ticks=0;
	break;	
	}
return ticks;
}

void configDialog::pickLegendFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,legendFont,this);
    if ( ok ) {
        legendFont = font;
    } else {
     return;
    }
}

void configDialog::pickAxesFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,axesFont,this);
    if ( ok ) {
        axesFont = font;
    } else {
     return;
    }
}

void configDialog::pickNumbersFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,numbersFont,this);
    if ( ok ) {
        numbersFont = font;
    } else {
     return;
    }
}

void configDialog::pickTitleFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,titleFont,this);
    if ( ok ) 
		titleFont = font;
    else 
     	return;
}

void configDialog::pickApplicationFont()
{
bool ok;
QFont font = QFontDialog::getFont(&ok,appFont,this);
    if ( ok ) 
		appFont = font;
    else 
     	return;
}

void configDialog::initSaveSettings(bool autoSave, int min)
{
boxSave->setChecked(autoSave);
boxMinutes->setValue(min);
boxMinutes->setEnabled(autoSave);
}

void configDialog::initAppFonts(const QFont& fnt)
{
appFont = fnt;	
}

void configDialog::initAppStyle(const QString& s)
{
boxStyle->setCurrentText(s);
}

void configDialog::initGraphFonts(const QFont& axesFnt, const QFont& numbersFnt, 
																const QFont& legendFnt, const QFont& titleFnt)
{
axesFont=axesFnt;
numbersFont=numbersFnt;
legendFont=legendFnt;	
titleFont=titleFnt;
}

void configDialog::initConfirmOptions(bool askTables, bool askPlots2D, bool askPlots3D)
{
boxTables->setChecked(askTables);
boxPlots2D->setChecked(askPlots2D);
boxPlots3D->setChecked(askPlots3D);
}

QColor configDialog::pickPanelsTextColor()
{
QColor c = QColorDialog::getColor(panelsTextColor, this );
if ( !c.isValid() )
		return panelsTextColor;

panelsTextColor = c;
return panelsTextColor;
}

QColor configDialog::pickPanelsColor()
{
QColor c = QColorDialog::getColor(panelsColor, this );
if ( !c.isValid() )
		return panelsColor;

panelsColor = c;
return panelsColor;
}

QColor configDialog::pickWorkspaceColor()
{
QColor c = QColorDialog::getColor(workspaceColor, this );
if ( !c.isValid() )
		return workspaceColor;

workspaceColor = c;
return workspaceColor;
}

void configDialog::initAppColors(const QColor& wCol, const QColor& pCol, const QColor& tCol)
{
workspaceColor=wCol;
panelsColor=pCol;
panelsTextColor=tCol;	
}
