#ifndef configDialog_H
#define configDialog_H

#include <qvariant.h>
#include <qdialog.h>
#include <qhbox.h> 
#include <qlayout.h>

class QButtonGroup;
class QPushButton;
class QTabWidget;
class QWidget;
class QCheckBox;
class QComboBox;
class QSpinBox;
	
class configDialog : public QDialog
{
    Q_OBJECT

public:
    configDialog( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~configDialog();

    QPushButton* buttonOk, *buttonBackground, *buttonText, *buttonHeader;
	QPushButton* buttonCancel, *buttonApply;
	QPushButton* buttonTextFont, *buttonHeaderFont;
    QButtonGroup* GroupBox1, *GroupBox2, *GroupBox3;
    QTabWidget* generalDialog;
	QWidget *tables, *plots, *confirm, *application;
	QPushButton* buttonAxesFont, *buttonNumbersFont, *buttonLegendFont, *buttonTitleFont, *fontsBtn;
	QCheckBox* boxTitle, *boxFrame, *boxPlots3D, *boxPlots2D, *boxTables, *boxSave;
	QComboBox *boxLegend, *boxTicks, *boxStyle;
	QSpinBox *boxMinutes;
	QPushButton *btnWorkspace, *btnPanels, *btnPanelsText;

public slots:
    virtual void languageChange();
	void accept();
	void update();

	//table colors
	QColor pickBgColor();
	QColor pickTextColor();
	QColor pickHeaderColor();
	void initColors(const QColor& bgCol, const QColor& textCol, const QColor& headerCol);

	//table fonts
	void pickTextFont();
	void pickHeaderFont();
	void initFonts(const QFont& textFnt, const QFont& headerFnt);

	//graph fonts
	void pickAxesFont();
	void pickNumbersFont();
	void pickLegendFont();
	void pickTitleFont();
	void initGraphFonts(const QFont& axesFnt, const QFont& numbersFnt, 
										const QFont& legendFnt, const QFont& titleFnt);

	void initPlotOptions(int ticksStyle, int legendStyle, bool titleOn, bool frameOn);
	void initConfirmOptions(bool askTables, bool askPlots2D, bool askPlots3D);
	int ticksType();

	//application
	void pickApplicationFont();
	void initAppFonts(const QFont& fnt);
	void initAppStyle(const QString& s);
	void initSaveSettings(bool autoSave, int min);
	QColor pickPanelsTextColor();
	QColor pickPanelsColor();
	QColor pickWorkspaceColor();
	void initAppColors(const QColor& wCol, const QColor& pCol, const QColor& tCol);
	
signals:
	void updateTables(const QColor&,const QColor&,const QColor&,const QFont&, const QFont&);
	void setGraphDefaultSettings(int, int, bool, bool,const QFont&,const QFont&, 
															const QFont&, const QFont&);
	void updateConfirmOptions(bool, bool, bool);
	void changeAppFont(const QFont&);
	void changeAppStyle(const QString&);
	void setSaveSettings(bool, int);
	void setAppColors(const QColor&,const QColor&,const QColor&);

private:
	QColor bgColor, textColor, headerColor, workspaceColor, panelsColor, panelsTextColor;
	QFont textFont, headerFont, axesFont, numbersFont, legendFont, titleFont, appFont;
};

#endif // configDialog_H
