#include "curvesDialog.h"

#include <qlabel.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qmessagebox.h>
#include <qvbox.h>
#include <qpopupmenu.h>
#include <qaccel.h>

static const char * remove_xpm[] = {
"16 17 27 1",
" 	c None",
".	c #000000",
"+	c #BDD3BD",
"@	c #B5CBB5",
"#	c #B5C7B5",
"$	c #B5CFB5",
"%	c #BDCFB5",
"&	c #BDCFBD",
"*	c #7BAA7B",
"=	c #ADC7AD",
"-	c #B5CBAD",
";	c #ADC7A5",
">	c #6B8263",
",	c #292C29",
"'	c #ADCBAD",
")	c #B5C7AD",
"!	c #638E63",
"~	c #9CAE9C",
"{	c #84AE84",
"]	c #84B284",
"^	c #8CAE84",
"/	c #8CB284",
"(	c #527152",
"_	c #395131",
":	c #5A7952",
"<	c #4A6142",
"[	c #425539",
"        .       ",
"       ..       ",
"      .+.       ",
"     .@#.       ",
"    .@@#........",
"   .@@$$%+%&+@*.",
"  .@=@@-=====;>.",
" ,@')=='===)='!.",
".~{{{{{{]^{]/{(.",
" ._:::::::::::<.",
"  ._::::::::::<.",
"   ._::<______[.",
"    ._:<........",
"     ._<.       ",
"      .<.       ",
"       ..       ",
"        .       "};

static const char * add_xpm[] = {
"16 17 26 1",
" 	c None",
".	c #000000",
"+	c #8CA684",
"@	c #BAD4B8",
"#	c #73A26B",
"$	c #BAD4BA",
"%	c #ADCBAD",
"&	c #739A63",
"*	c #BDD7BD",
"=	c #BAD3B8",
"-	c #BAD3BA",
";	c #ADC7AD",
">	c #081008",
",	c #ADC7A5",
"'	c #A5C7A5",
")	c #7B9E73",
"!	c #A5BEA5",
"~	c #A5C3A5",
"{	c #6B9263",
"]	c #738E6B",
"^	c #6B8E63",
"/	c #425929",
"(	c #6B8663",
"_	c #5A7952",
":	c #63825A",
"<	c #526942",
"       .        ",
"       ..       ",
"       .+.      ",
"       .@#.     ",
"........$%&.    ",
".*******=-;&>   ",
".;%;;;,,;%;').  ",
".!;,',,;%;;%~{. ",
".]^{{{{{{{{{{^/.",
".(___________/. ",
".(__________/.  ",
".:<<<<<<___/.   ",
"........<_/.    ",
"       .</.     ",
"       ./.      ",
"       ..       ",
"       .        "};

curvesDialog::curvesDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "curvesDialog" );
	setMinimumSize(QSize(400,200));
    setCaption( tr( "QtiPlot - Add/Remove curves" ) );
	setFocus();
	
	QVBox  *box1=new QVBox (this, "box1"); 
	box1->setMargin(5);
	box1->setSpacing (5);
	
	TextLabel1 = new QLabel(box1, "TextLabel1" );
    TextLabel1->setText( tr( "Available data" ) );

    available = new QListBox( box1, "available" );
	available->setSelectionMode (QListBox::Multi);
	
	QVBox  *box2=new QVBox (this, "box2"); 
	box2->setMargin(5);
	box2->setSpacing (5);

    btnAdd = new QPushButton(box2, "btnAdd" );
    btnAdd->setPixmap( QPixmap(add_xpm) );
	btnAdd->setFixedWidth (35);
	btnAdd->setFixedHeight (30);
	
    btnRemove = new QPushButton(box2, "btnRemove" );
    btnRemove->setPixmap( QPixmap(remove_xpm) );
	btnRemove->setFixedWidth (35);
	btnRemove->setFixedHeight(30);
		
	QVBox  *box3=new QVBox (this, "box3"); 
	box3->setMargin(5);
	box3->setSpacing (5);
	
	TextLabel2 = new QLabel(box3, "TextLabel2" );
    TextLabel2->setText( tr( "Graph contents" ) );

    contents = new QListBox( box3, "contents" );
	contents->setSelectionMode (QListBox::Multi);

	QVBox  *box4=new QVBox (this, "box4"); 
	box4->setMargin(5);
	box4->setSpacing (5);

    btnOK = new QPushButton(box4, "btnOK" );
    btnOK->setText( tr( "OK" ) );
	btnOK->setDefault( TRUE );
	
    btnCancel = new QPushButton(box4, "btnCancel" );
    btnCancel->setText( tr( "Cancel" ) );
	
	QHBoxLayout* layout = new QHBoxLayout(this,5,5, "hlayout3");
    layout->addWidget(box1);
	layout->addWidget(box2);
	layout->addWidget(box3);
	layout->addWidget(box4);
	
connect(btnAdd, SIGNAL(clicked()),this, SLOT(addCurve()));
connect(btnRemove, SIGNAL(clicked()),this, SLOT(removeCurve()));
connect(btnOK, SIGNAL(clicked()),this, SLOT(close()));
connect(btnCancel, SIGNAL(clicked()),this, SLOT(close()));
connect(btnAdd, SIGNAL(clicked()),this, SLOT(enableRemoveBtn()));
connect(btnRemove, SIGNAL(clicked()),this, SLOT(enableRemoveBtn()));

connect(contents, SIGNAL(rightButtonClicked(QListBoxItem *, const QPoint &)), this, SLOT(deletePopupMenu(QListBoxItem *, const QPoint &)));
connect(available, SIGNAL(rightButtonClicked(QListBoxItem *, const QPoint &)), this, SLOT(addPopupMenu(QListBoxItem *, const QPoint &)));

QAccel *accel = new QAccel(this);
accel->connectItem( accel->insertItem( Key_Delete ),
                            this, SLOT(removeCurve()) );
}

QSize curvesDialog::sizeHint() const 
{
return QSize(400, 200 );
}

void curvesDialog::deletePopupMenu(QListBoxItem *it, const QPoint &point)
{
selectedCurve=contents->index (it);
	
QPopupMenu contextMenu(this);
contextMenu.insertItem("&Delete", this, SLOT(removeSelectedCurve()));
contextMenu.exec(point);
}

void curvesDialog::addPopupMenu(QListBoxItem *it, const QPoint &point)
{
selectedCurve=available->index (it);
	
QPopupMenu contextMenu(this);
contextMenu.insertItem("&Plot", this, SLOT(addSelectedCurve()));
contextMenu.exec(point);
}

void curvesDialog::insertCurvesToDialog(const QStringList& names)
{
available->clear();
int i,n=names.count();	
for (i=0;i<n;i++)
	available->insertItem(names[i],i);

if (n==0)
	btnAdd->setDisabled(true);
}

void curvesDialog::insertPlottedCurves(const QStringList& names)
{
contents->clear();
int i,n=names.count();
for (i=0;i<n;i++)
	contents->insertItem(names[i],i);

enableRemoveBtn();
}

void curvesDialog::addSelectedCurve()
{
QString text=available->text(selectedCurve);
if (!contents->findItem(text, Qt::ExactMatch))
		{
		emit curveToPlot(text);
		contents->insertItem(text,-1);
		}
}

void curvesDialog::addCurve()
{
for (int i=0;i<int(available->count());i++)
	{
	if (available->isSelected(i))
		{
		QString text=available->text(i);
		if (!contents->findItem(text, Qt::ExactMatch))
			{
			emit curveToPlot(text);
			contents->insertItem(text,-1);
			}
		}
	}
}

void curvesDialog::removeSelectedCurve()
{
emit curveToRemove(selectedCurve);
contents->removeItem (selectedCurve);
}

void curvesDialog::removeCurve()
{
int i;
QStringList texts;	
for (i=0;i<int(contents->count());i++)
	{
	if (contents->isSelected(i))
		texts<<contents->text(i);	
	}

for (i=0;i<int(texts.count());i++)
	{
	QListBoxItem *it=contents->findItem (texts[i],Qt::ExactMatch);
	
	int index=contents->index(it);		
	emit curveToRemove(index);
	contents->removeItem(index);	
	}	
}

void curvesDialog::clear()
{
contents->clear();
btnRemove->setDisabled (TRUE);
}

void curvesDialog::enableRemoveBtn()
{
if (contents->count()>0)
	btnRemove->setEnabled (TRUE);
else
	btnRemove->setDisabled (TRUE);
}

curvesDialog::~curvesDialog()
{
}
