#include "exportDialog.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>

/*
 *  Constructs a exportDialog as a child of 'parent', with the
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
exportDialog::exportDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "exportDialog" );
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( 310, 140 ) );
	setMaximumSize( QSize( 310, 140 ) );
    setMouseTracking( TRUE );
    setSizeGripEnabled( FALSE );
	
	GroupBox1 = new QButtonGroup( 2,QGroupBox::Horizontal,tr(""),this,"GroupBox1" );

    boxNames = new QCheckBox(GroupBox1, "boxNames" );
    boxNames->setChecked( TRUE );
	
	new QLabel( tr( "" ), GroupBox1, "TextLabel1",0 );

    boxSelection = new QCheckBox(GroupBox1, "boxSelection" );
    boxSelection->setChecked( FALSE );
	
	new QLabel( tr( "" ), GroupBox1, "TextLabel2",0 );

    new QLabel( tr( "Separator" ), GroupBox1, "TextLabel3",0 );
	
    boxSeparator = new QComboBox( FALSE,GroupBox1, "boxSeparator" );
	
	GroupBox2 = new QButtonGroup(1,QGroupBox::Horizontal,tr(""),this,"GroupBox2" );
	GroupBox2->setFlat (TRUE);
	GroupBox2->setLineWidth (0);
	
	buttonOk = new QPushButton(GroupBox2, "buttonOk" );
    buttonOk->setAutoDefault( TRUE );
    buttonOk->setDefault( TRUE );
   
    buttonCancel = new QPushButton(GroupBox2, "buttonCancel" );
    buttonCancel->setAutoDefault( TRUE );
	
	QHBoxLayout* hlayout = new QHBoxLayout(this,5,5, "hlayout");
    hlayout->addWidget(GroupBox1);
	hlayout->addWidget(GroupBox2);

    languageChange();
   
    // signals and slots connections
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
}

void exportDialog::languageChange()
{
    setCaption( tr( "QtiPlot - Export ASCII" ) );
    buttonOk->setText( tr( "&OK" ) );
	buttonCancel->setText( tr( "&Cancel" ) );
    boxNames->setText( tr( "Include Column Names" ) );
    boxSelection->setText( tr( "Export Selection" ) );
    boxSeparator->clear();
    boxSeparator->insertItem( tr( "TAB" ) );
    boxSeparator->insertItem( tr( "SPACE" ) );
	boxSeparator->insertItem( tr( ",TAB" ) );
	boxSeparator->insertItem( tr( ";TAB" ) );
	boxSeparator->insertItem( tr( ",SPACE" ) );
	boxSeparator->insertItem( tr( ";SPACE" ) );
    boxSeparator->insertItem( tr( ";" ) );
    boxSeparator->insertItem( tr( "," ) );
}

void exportDialog::accept()
{
emit options(separator(),boxNames->isChecked(),boxSelection->isChecked());
close();
}

QString exportDialog::separator()
{
QString sep;
switch (boxSeparator->currentItem())
	{
case 0:
	sep="\t";
break;

case 1:
	sep=" ";
break;

case 2:
	sep=",\t";
break;

case 3:
	sep=";\t";
break;

case 4:
	sep=", ";
break;

case 5:
	sep="; ";
break;

case 6:
	sep=";";
break;

case 7:
	sep=",";
break;
	}
return sep; 
}

exportDialog::~exportDialog()
{
}


