#include "fitDialog.h"
#include "parser/muParser.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qregexp.h> 
#include <qhbox.h> 

#include <stdio.h> 

#ifndef M_PI
#define M_PI	3.141592653589793238462643 
#endif

extern "C" {
double *vector(long nl, long nh);
void free_vector(double *v, long nl, long nh);}

fitDialog::fitDialog( QWidget* parent, const char* name, WFlags fl )
    : QWidget( parent, name, fl )
{
    if ( !name )
	setName( "fitDialog" );
	setCaption(tr("QtiPlot - Non-linear curve fit"));
	
	GroupBox1 = new QButtonGroup( 2,QGroupBox::Horizontal,tr(""),this,"GroupBox1" );
	
	new QLabel( tr("Curve"), GroupBox1, "TextLabel22",0 );
	boxCurve = new QComboBox(GroupBox1, "boxCurve" );

	new QLabel( tr("Function"), GroupBox1, "TextLabel2",0 );
	boxFunction = new QComboBox(GroupBox1, "boxOrder" );
	boxFunction->setEditable(TRUE);
	boxFunction->setCurrentText("f(x,a,b,...)=");

	new QLabel( tr("Initial guesses"), GroupBox1, "TextLabel23",0 );
	boxParams = new QLineEdit(GroupBox1, "boxParams" );
	boxParams->setText("a=1 b=1");
	
	QHBox *hbox=new QHBox(this,"hbox");
	hbox->setSpacing(5);
	
	QButtonGroup* GroupBox4 = new QButtonGroup(2,QGroupBox::Horizontal,tr( "" ),hbox, "GroupBox4" );
	
	new QLabel( tr("From x="), GroupBox4, "TextLabel3",0 );
	boxFrom = new QLineEdit(GroupBox4, "boxPoints" );
	
	new QLabel( tr("To x="), GroupBox4, "TextLabel5",0 );
	boxTo = new QLineEdit(GroupBox4, "boxOrder" );
	
	QButtonGroup *GroupBox3 = new QButtonGroup( 2,QGroupBox::Horizontal,tr(""),hbox,"GroupBox3" );
			
	new QLabel( tr("Iterations"), GroupBox3, "TextLabel4",0 );
	boxPoints = new QLineEdit(GroupBox3, "boxStart" );
	boxPoints->setText(tr("100"));
	
	new QLabel( tr("Tolerance"), GroupBox3, "TextLabel41",0 );
	boxTolerance = new QLineEdit(GroupBox3, "boxTolerance" );
	boxTolerance->setText(tr("1e-4"));
	
	new QLabel( tr("Algorithm"), GroupBox1, "TextLabel44",0 );
	boxSolver = new QComboBox(GroupBox1, "boxSolver" );
	boxSolver->insertItem("Scaled Levenberg-Marquardt");
	boxSolver->insertItem("Unscaled Levenberg-Marquardt");
	
	GroupBox2 = new QButtonGroup(3,QGroupBox::Horizontal,tr(""),this,"GroupBox2" );
	GroupBox2->setFlat (TRUE);
	GroupBox2->setLineWidth (0);

	buttonClear = new QPushButton(GroupBox2, "buttonClear" );
    buttonClear->setText( tr( "Clear &list" ) );
	
	buttonOk = new QPushButton(GroupBox2, "buttonOk" );
    buttonOk->setText( tr( "&Fit" ) );
	buttonOk->setAutoDefault( TRUE );
    buttonOk->setDefault( TRUE );
   
    buttonCancel = new QPushButton(GroupBox2, "buttonCancel" );
    buttonCancel->setText( tr( "&Cancel" ) );
	
	QVBoxLayout* hlayout = new QVBoxLayout(this,5,5, "hlayout");
    hlayout->addWidget(GroupBox1);
	hlayout->addWidget(hbox);
	hlayout->addWidget(GroupBox2);
   
    // signals and slots connections
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( close() ) );
	connect( buttonClear, SIGNAL( clicked() ), this, SLOT(clearList() ) );
	
	setFocusProxy(boxFunction);
}

void fitDialog::setLimits(const QStringList& limits)
{
boxFrom->setText (limits[0]);
boxTo->setText (limits[1]);
}

void fitDialog::setCurveNames(const QStringList& names)
{
boxCurve->insertStringList (names,-1);
}

void fitDialog::clearList()
{
boxFunction->clear();
emit clearFunctionsList();
}

void fitDialog::accept()
{
QString from=boxFrom->text();
QString to=boxTo->text();
QString points=boxPoints->text();
QString tolerance=boxTolerance->text();

QStringList equation=QStringList::split("=",boxFunction->currentText(),FALSE);
if (int(equation.count()) < 2)
	{
	QMessageBox::critical(0,"QtiPlot - Input function error",
				"Please enter a valid function!");
	boxFunction->setFocus();
	return;
	}	
	
if (int(equation.count()) > 2)
	{
	QMessageBox::critical(0,"QtiPlot - Input function error",
				"Please use exactly one '=' symbol!");
	boxFunction->setFocus();
	return;
	}

if (boxParams->text().isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter initial guesses for your parameters!");
	boxParams->setFocus();
	return;
	}

if (boxFunction->currentText().isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid non empty formula!");
	boxFunction->setFocus();
	return;
	}
	
if(from.isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid floating-point value for the start x limit!");
	boxFrom->setFocus();
	return;
	}
	
if(to.isEmpty() )
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid floating-point value for the end x limit!");
	boxTo->setFocus();
	return;
	}
	
if(points.isEmpty() || points.contains(QRegExp ("\\D"))>0)
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid positive integer (non floating-point) value for the number of points!");
	boxPoints->setFocus();
	return;
	} 

if(tolerance.isEmpty() )
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a value for the tolerance!");
	boxTolerance->setFocus();
	return;
	} 
	
double start,end,eps;

try
		{
		MathUtils::Parser parser;

		if (from.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (from.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (from.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula(from.ascii());
		start=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Start limit error",e.what());
		boxFrom->setFocus();
		return;
		}	

	try
		{
		MathUtils::Parser parser;

		if (to.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (to.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (to.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula(to.ascii());
		end=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - End limit error",e.what());
		boxTo->setFocus();
		return;
		}	

if (start>=end)
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter x limits that satisfy: from < end!");
	boxTo->setFocus();
	return;
	}
	
try
		{
		MathUtils::Parser parser;

		if (tolerance.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (tolerance.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (tolerance.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula(tolerance.ascii());
		eps=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Tolerance input error",e.what());
		boxTolerance->setFocus();
		return;
		}	
	
if (eps<0 || eps>=1)
	{
	QMessageBox::critical(0,"QtiPlot - Tolerance input error",
				"The tolerance value must be positive and less than 1!");
	boxTolerance->setFocus();
	return;
	}
	
	QRegExp autoSep= QRegExp("[,;\\s]", FALSE, FALSE);
	QStringList paramsList=QStringList::split(autoSep,boxParams->text(),false);
	
	int n=int(paramsList.count());
			
	QStringList parameters, initialValues;
	MathUtils::Parser parser;
	
	double x, *paramsInit;
	paramsInit=vector(0,n-1);

	bool error=FALSE;

	try
		{
		parser.AddVar("x", &x);	

		for (int i=0;i<n;i++)
			{
			QStringList p=QStringList::split("=",paramsList[i],FALSE);
			paramsInit[i]=p[1].toDouble();
			parser.AddVar(p[0].ascii(), &paramsInit[i]);
			parameters<<p[0];
			initialValues<<p[1];
			}
			
	 if (equation[1].contains("pi")>0)
		parser.AddConst("pi", M_PI);
	 if (equation[1].contains("Pi")>0)
		parser.AddConst("Pi", M_PI);
	 if (equation[1].contains("PI")>0)
		parser.AddConst("PI", M_PI);

		parser.SetFormula(equation[1].ascii());
		
		x=start;
		parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Input function error",e.what());
		boxFunction->setFocus();
		error=TRUE;	
		}

	free_vector(paramsInit,0,n-1);

	if (!error)
		{
		emit options(boxCurve->currentText(),boxFunction->currentText(),
					parameters,initialValues,start,end, points.toInt(), boxSolver->currentItem(), eps);
		}
}

void fitDialog::insertFunctionsList(const QStringList& list)
{
boxFunction->insertStringList (list, 1);
}

fitDialog::~fitDialog()
{
}
