#include "functionDialog.h"
#include "parser/muParser.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qmessagebox.h>
#include <qcombobox.h>
#include <qregexp.h> 
#include <qspinbox.h>

#ifndef M_PI
#define M_PI	3.141592653589793238462643 
#endif

fDialog::fDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : functionDialogui( parent, name, modal, fl )
{
    if ( !name )
	setName( "fDialog" );

	connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( buttonClear, SIGNAL( clicked() ), this, SLOT(clearList() ) );
	setFocusProxy(boxFunction);
}

void fDialog::clearList()
{
int type=boxType->currentItem();
switch (type)
	{
	case 0:
		boxFunction->clear();
		emit clearFunctionsList();
	break;
	
	case 1:
		boxXFunction->clear();
		boxYFunction->clear();
		emit clearParamFunctionsList();
	break;
	
	case 2:
		boxPolarTheta->clear();
		boxPolarRadius->clear();
		emit clearPolarFunctionsList();
	break;
	}
}

void fDialog::acceptFunction()
{
QString from=boxFrom->text();
QString to=boxTo->text();
QString points=boxPoints->text();
QRegExp nonDigit("\\D");

from=from.remove(".");
to=to.remove(".");

int pos=from.find("-",0);
if(pos==0)
	from=from.replace(pos,1,"");

pos=to.find("-",0);
if(pos==0)
	to=to.replace(pos,1,"");

double start, end;

if (boxFunction->currentText().isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid non empty formula!");
	boxFunction->setFocus();
	return;
	}
	
if (from.isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid floating-point value for the start x limit!");
	boxFrom->setFocus();
	return;
	}
	
if(to.isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid floating-point value for the end x limit!");
	boxTo->setFocus();
	return;
	}
	
if(points.isEmpty() || points.contains(nonDigit) >0)
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid integer value for the number of points!");
	boxPoints->setFocus();
	return;
	} 
	
if (from.contains(nonDigit) >0)
	{
	try
		{
		MathUtils::Parser parser;

		if (from.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (from.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (from.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula((boxFrom->text()).ascii());
		start=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Start limit error",e.what());
		boxFrom->setFocus();
		return;
		}	
	}
else
	start=boxFrom->text().toDouble();

if (to.contains(nonDigit) >0)
	{
	try
		{
		MathUtils::Parser parser;

		if (to.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (to.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (to.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula((boxTo->text()).ascii());
		end=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - End limit error",e.what());
		boxTo->setFocus();
		return;
		}	
	}
else
	end=boxTo->text().toDouble();

if (start>=end)
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter x limits that satisfy: from < end!");
	boxTo->setFocus();
	return;
	}

	double x;
	QString formula=boxFunction->currentText();
	bool error=FALSE;

	try
		{
		MathUtils::Parser parser;

		if (formula.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (formula.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (formula.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.AddVar("x", &x);	
		parser.SetFormula(formula.ascii());
		
		x=start;
		parser.Calc();
		x=end;
		parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Input function error",e.what());
		boxFunction->setFocus();
		error=TRUE;	
		}	

	// Collecting all the information
	QString type=QString("Function");
	QStringList formulas;
	QStringList variables;	
	QValueList<double> ranges;
	QValueList<int> varpoints;
	formulas+=formula;
	variables+="x";
	ranges+=start;
	ranges+=end;
	varpoints[0]=boxPoints->value();
	if (!error)
		{
		emit options(type,formulas,variables,ranges,varpoints);	 
		close();
		}		

}
void fDialog::acceptParametric()
{
QString from=boxParFrom->text();
QString to=boxParTo->text();
QString points=boxParPoints->text();
QRegExp nonDigit("\\D");

from=from.remove(".");
to=to.remove(".");

int pos=from.find("-",0);
if(pos==0)
	from=from.replace(pos,1,"");

pos=to.find("-",0);
if(pos==0)
	to=to.replace(pos,1,"");

double start, end;

if (boxParameter->text().isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error", "Please enter a parameter name!");
	boxParameter->setFocus();
	return;
	}

if (boxXFunction->currentText().isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid non empty formula!");
	boxXFunction->setFocus();
	return;
	}

if (boxYFunction->currentText().isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid non empty formula!");
	boxYFunction->setFocus();
	return;
	}
		
if (from.isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid floating-point value for the parameter start limit!");
	boxParFrom->setFocus();
	return;
	}
	
if(to.isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid floating-point value for the parameter end limit!");
	boxParTo->setFocus();
	return;
	}
	
if(points.isEmpty() || points.contains(nonDigit) >0)
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid integer value for the number of points!");
	boxParPoints->setFocus();
	return;
	} 
	
if (from.contains(nonDigit) >0)
	{
	try
		{
		MathUtils::Parser parser;

		if (from.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (from.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (from.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula((boxParFrom->text()).ascii());
		start=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Start limit error",e.what());
		boxParFrom->setFocus();
		return;
		}	
	}
else
	start=boxParFrom->text().toDouble();

if (to.contains(nonDigit) >0)
	{
	try
		{
		MathUtils::Parser parser;

		if (to.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (to.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (to.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula((boxParTo->text()).ascii());
		end=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - End limit error",e.what());
		boxParTo->setFocus();
		return;
		}	
	}
else
	end=boxParTo->text().toDouble();

if (start>=end)
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter parameter limits that satisfy: from < end!");
	boxParTo->setFocus();
	return;
	}

	double parameter;
	QString xformula=boxXFunction->currentText();
	QString yformula=boxYFunction->currentText();	
	bool error=FALSE;

	try
		{
		MathUtils::Parser parser;

		if (xformula.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (xformula.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (xformula.contains("PI")>0)
			parser.AddConst("PI", M_PI);
				
		parser.AddVar((boxParameter->text()).ascii(), &parameter);	
		parser.SetFormula(xformula.ascii());
		
		parameter=start;
		parser.Calc();
		parameter=end;
		parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Input function error",e.what());
		boxXFunction->setFocus();
		error=TRUE;	
		}	
	try
		{
		MathUtils::Parser parser;

		if (yformula.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (yformula.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (yformula.contains("PI")>0)
			parser.AddConst("PI", M_PI);
				
		parser.AddVar((boxParameter->text()).ascii(), &parameter);	
		parser.SetFormula(yformula.ascii());
		
		parameter=start;
		parser.Calc();
		parameter=end;
		parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Input function error",e.what());
		boxYFunction->setFocus();
		error=TRUE;	
		}
	// Collecting all the information
	QString type=QString("Parametric plot");
	QStringList formulas;
	QStringList variables;	
	QValueList<double> ranges;
	QValueList<int> varpoints;
	formulas+=xformula;
	formulas+=yformula;
	variables+=boxParameter->text();
	ranges+=start;
	ranges+=end;
	varpoints[0]=boxParPoints->value();
	if (!error)
		{
		emit options(type,formulas,variables,ranges,varpoints);	 
		close();
		}
}

void fDialog::acceptPolar()
{
QString from=boxPolarFrom->text();
QString to=boxPolarTo->text();
QString points=boxPolarPoints->text();
QRegExp nonDigit("\\D");

from=from.remove(".");
to=to.remove(".");

int pos=from.find("-",0);
if(pos==0)
	from=from.replace(pos,1,"");

pos=to.find("-",0);
if(pos==0)
	to=to.replace(pos,1,"");

double start, end;

if (boxPolarParameter->text().isEmpty()) 
	{
	QMessageBox::critical(0,"QtiPlot - Input error", "Please enter a parameter name!");
	boxPolarParameter->setFocus();
	return;
	}

if (boxPolarRadius->currentText().isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid non empty formula!");
	boxPolarRadius->setFocus();
	return;
	}

if (boxPolarTheta->currentText().isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid non empty formula!");
	boxPolarTheta->setFocus();
	return;
	}
		
if (from.isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid floating-point value for the parameter start limit!");
	boxPolarFrom->setFocus();
	return;
	}
	
if(to.isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid floating-point value for the parameter end limit!");
	boxPolarTo->setFocus();
	return;
	}
	
if(points.isEmpty() || points.contains(nonDigit) >0)
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid integer value for the number of points!");
	boxPolarPoints->setFocus();
	return;
	} 
	
if (from.contains(nonDigit) >0)
	{
	try
		{
		MathUtils::Parser parser;

		if (from.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (from.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (from.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula((boxPolarFrom->text()).ascii());
		start=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Start limit error",e.what());
		boxPolarFrom->setFocus();
		return;
		}	
	}
else
	start=boxPolarFrom->text().toDouble();

if (to.contains(nonDigit) >0)
	{
	try
		{
		MathUtils::Parser parser;

		if (to.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (to.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (to.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula((boxPolarTo->text()).ascii());
		end=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - End limit error",e.what());
		boxPolarTo->setFocus();
		return;
		}	
	}
else
	end=boxPolarTo->text().toDouble();

if (start>=end)
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter parameter limits that satisfy: from < end!");
	boxPolarTo->setFocus();
	return;
	}

	double parameter;
	QString rformula=boxPolarRadius->currentText();
	QString tformula=boxPolarTheta->currentText();	
	bool error=FALSE;

	try
		{
		MathUtils::Parser parser;

		if (rformula.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (rformula.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (rformula.contains("PI")>0)
			parser.AddConst("PI", M_PI);
				
		parser.AddVar((boxPolarParameter->text()).ascii(), &parameter);	
		parser.SetFormula(rformula.ascii());
		
		parameter=start;
		parser.Calc();
		parameter=end;
		parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Input function error",e.what());
		boxPolarRadius->setFocus();
		error=TRUE;	
		}	
	try
		{
		MathUtils::Parser parser;

		if (tformula.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (tformula.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (tformula.contains("PI")>0)
			parser.AddConst("PI", M_PI);
				
		parser.AddVar((boxPolarParameter->text()).ascii(), &parameter);	
		parser.SetFormula(tformula.ascii());
		
		parameter=start;
		parser.Calc();
		parameter=end;
		parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Input function error",e.what());
		boxPolarTheta->setFocus();
		error=TRUE;	
		}
	// Collecting all the information
	QString type=QString("Polar plot");
	QStringList formulas;
	QStringList variables;	
	QValueList<double> ranges;
	QValueList<int> varpoints;
	formulas+=rformula;
	formulas+=tformula;
	variables+=boxPolarParameter->text();
	ranges+=start;
	ranges+=end;
	varpoints[0]=boxPolarPoints->value();
	if (!error)
		{
		emit options(type,formulas,variables,ranges,varpoints);	 
		close();
		}
}

void fDialog::accept()
{
int type=boxType->currentItem();
switch (type)
	{
	case 0:
				acceptFunction();
	break;
	
	case 1:
				acceptParametric();
	break;
	
	case 2:
				acceptPolar();
	break;
	}
}

void fDialog::insertFunctionsList(const QStringList& list)
{
boxFunction->insertStringList (list, 1);
}

void fDialog::insertParamFunctionsList(const QStringList& xList, const QStringList& yList)
{
boxXFunction->insertStringList (xList, 1);
boxYFunction->insertStringList (yList, 1);
}

void fDialog::insertPolarFunctionsList(const QStringList& rList, const QStringList& tetaList)
{
boxPolarRadius->insertStringList (rList, 1);
boxPolarTheta->insertStringList (tetaList, 1);	
}

fDialog::~fDialog()
{
}
