#ifndef GENDIALOG_H
#define GENDIALOG_H

#include <qvariant.h>
#include <qdialog.h>
#include <qarray.h>
#include <qstring.h>
#include <qvaluelist.h>

class QCheckBox;
class QComboBox;
class QLabel;
class QLineEdit;
class QListBox;
class QListBoxItem;
class QPushButton;
class QRadioButton;
class QSpinBox;
class QTabWidget;
class QWidget;
class QStringList;
class QButtonGroup;

typedef struct{ 
  int majorOnX;
  int minorOnX;
  int majorOnY;
  int minorOnY;
  int majorStyle;
  int majorCol;
  int majorWidth;
  int minorStyle;
  int minorCol;
  int minorWidth;
  int xZeroOn;
  int yZeroOn;
}  gridOptions;

class genDialog : public QDialog
{ 
    Q_OBJECT

public:
    genDialog( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~genDialog();

	void initAxesPage();
	void initScalesPage();
	void initGridPage();
	void initFramePage();

    QPushButton* buttonApply;
    QPushButton* buttonOk;
    QPushButton* buttonCancel;
    QTabWidget* generalDialog;
    QWidget* scalesPage;
    QLineEdit* boxEnd;   
    QLineEdit* boxStart;
    QComboBox* boxScaleType;
    QLineEdit* boxMinorValue;
    QLineEdit* boxStep;
    QCheckBox* btnStep, *btnInvert;
    QLineEdit* boxMajorValue;
    QCheckBox* btnMinor;
    QListBox* axesList;
    QWidget* Widget22;
    QCheckBox* boxMajorGrid;
    QCheckBox* boxMinorGrid;
    QComboBox* boxTypeMajor;
    QComboBox* boxColorMinor;
    QComboBox* boxColorMajor;
    QSpinBox* boxWidthMajor;
    QComboBox* boxTypeMinor;
    QSpinBox* boxWidthMinor;
    QCheckBox* boxXLine;
    QCheckBox* boxYLine;
    QListBox* axesGridList;
    QWidget* tab, *frame;
    QListBox* axesTitlesList;
    QCheckBox* boxShowAxis, *boxShowLabels;
    QLineEdit* boxTitle, *boxEndAxis, *boxStartAxis, *boxAxisStep;
	QSpinBox *boxFrameWidth, *boxPrecision, *boxAngle;
    QPushButton* btnAxesFont;
	QCheckBox* boxFramed;
	QComboBox* boxAxisColor, *boxFrameColor, *boxTicksType, *boxFormat, *boxAxisType, *boxColName;
	QButtonGroup* GroupBox0;
	QButtonGroup* GroupBox1;
	QButtonGroup* GroupBox2;
	QButtonGroup* GroupBox3;
	QButtonGroup *GroupBox5, *GroupBox6;
	QLabel *label1, *label2, *label3;

public slots:
	QStringList scaleLimits(int axis, double start, double end, double step, 
																const QString& majors, const QString&minors);
	QStringList secondaryScaleLimits(int axis);
	void setAxesType(const QValueList<int>& list);
	void setAxisType(int axis);
	void updateAxisType(int axis);
	void setAxisTitles(QStringList t);
	void updateTitleBox(int axis);
	bool updatePlot();
	void updateScale(int axis);
	void updateSecondaryScales(int axis);
	void updateLineBoxes(int axis);
	void setScaleLimits(const QStringList& limits);
	void setSecondaryScales(const QStringList& limits);
	void stepEnabled();
	void stepDisabled();
	void majorGridEnabled(bool on);
	void minorGridEnabled(bool on);
	gridOptions getGridOptions();
	void putGridOptions(gridOptions gr);
	void setGridOptions();
	void tabPageChanged(QWidget *w);
	void accept();
	void customAxisFont();
	void showAxis();
	void updateShowBox(int axis);
	void setEnabledAxes(QArray<bool> ok);
	void drawFrame(bool framed);
	void setFramed(bool ok);
	void setFrameColor(const QColor& c);
	void setFrameWidth(int w);
	QColor frameColor();
	QColor axisColor();
	void setAxisColor(const QColor& c);
	void updateAxisColor(int);
	void setAxesColors(const QStringList& colors);
	int mapToQwtAxisId();
	void setEnabledTickLabels(const QStringList& labelsOn);
	void updateTickLabelsList(bool);
	void setTicksType(const QValueList<int>& list);
	void setTicksType(int);
	void updateColorsList(int index);
	void setCurrentScale(int axis);
	void initAxisFonts(const QFont& xB, const QFont& yL, const QFont& xT, const QFont& yR );
	int ticksType();
	void updateTicksType(int);
	void updateGrid(int);
	void updateFrame(int);
	void setLabelsFormat(const QStringList& list);
	void setLabelsFormat(int);
	void updateLabelsFormat(int);
	void initLabelsRotation(int xAngle, int yAngle);
	void insertColList(const QStringList& cols);
	void setAxesLabelsColumns(const QStringList& list);
	void showAxisFormatOptions(int format);

signals:
	void updateAxisTitle(int,const QString&);
	void updatePlotLimits(int,const QStringList&);
	void updateSecondaryLimits(int,const QStringList&);
	void changeGridOptions(const gridOptions &);
	void changeAxisFont(int, const QFont &);
	void showAxis(int, int, const QString&, bool, int, bool,const QColor&, int, int, int);	
	void drawFrame(bool,int,const QColor&);

protected:
	QStringList titles,scales,secondaryScales,axesColors, tickLabelsOn, labelsFormat, axesLabelsColumns;
	QValueList<int> ticks;
	QValueList <int> axesType;
	gridOptions grid;
	bool xAxisOn,yAxisOn,topAxisOn,rightAxisOn;
	QFont xBottomFont, yLeftFont, xTopFont, yRightFont;
	int xBottomLabelsRotation, xTopLabelsRotation;
};

#endif // GENDIALOG_H
