#include <qspinbox.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qfont.h>
#include <qmenubar.h>
#include <qpopupmenu.h>
#include <qdialog.h>
#include <qfiledialog.h>
#include <qinputdialog.h>
#include <qtextstream.h>
#include <qgrid.h>
#include <qrect.h> 
#include <qframe.h> 
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qgroupbox.h>
#include <qhbox.h>
#include <qaccel.h>
#include <qfileinfo.h> 
#include <qdir.h>
#include <qlistbox.h>
#include <qpixmap.h> 
#include <qkeycode.h>
#include <qlayout.h>
#include <qpainter.h>
#include <qtoolbar.h>
#include <qtoolbutton.h> 
#include <qprinter.h>
#include <qpaintdevicemetrics.h>
#include <qcolordialog.h>
#include <qfontdialog.h>
#include <qradiobutton.h>
#include <qcheckbox.h>
#include <qiconset.h>
#include <qarray.h>
#include <qvbox.h>
#include <qtabdialog.h>
#include <qbuttongroup.h>
#include <qregexp.h> 
#include <qcombobox.h>
#include <qvaluelist.h>
#include <qpen.h>
#include <qcolor.h>
#include <qcursor.h>
#include <qpixmap.h>
#include <qpixmapcache.h> 
#include <qstylesheet.h> 
#include <qdatetime.h> 
#include <qwmatrix.h> 
#include <qlabel.h> 
#include <qimage.h>
#include <qmessagebox.h>
#include <qclipboard.h>
#include <qdragobject.h>
#include <qbitmap.h>
#include <qfontdatabase.h>

#include "graph.h"
#include "canvaspicker.h"
#include "ErrorBar.h"
#include "LegendMarker.h"
#include "LineMarker.h"
#include "cursors.h"
#include "scalePicker.h"
#include "pie.h"
#include "ImageMarker.h"
#include "BarCurve.h"
#include "Histogram.h"
#include "VectXYXYCurve.h"
#include "SuperscriptScale.h"

#include "parser/muParser.h"

#include <qwt_plot.h>
#include <qwt_painter.h>
#include <qwt_curve.h>
#include <qwt_scldraw.h>
#include <qwt_plot_canvas.h>
#include <qwt_plot_layout.h>
#include <qwt_plot_zoomer.h>
#include <qwt_autoscl.h>
#include <qwt_scale.h>
#include <qwt_marker.h>
#include <qwt_text.h>

#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <stddef.h>

class QwtNoLabelsScaleDraw: public QwtScaleDraw
{
public:
	QwtNoLabelsScaleDraw(){};
	~QwtNoLabelsScaleDraw(){};
		
	virtual void draw(QPainter *p)
	{
	QwtScaleDraw::draw(p);	
	};

	virtual QString label(double) const
	{
	return QString::null;
	};
};

class QwtTextScaleDraw: public QwtScaleDraw
{
public:
	QwtTextScaleDraw(const QStringList& list){labels = list;};
	~QwtTextScaleDraw(){};
		
	virtual void draw(QPainter *p)
	{
	QwtScaleDraw::draw(p);	
	};

	virtual QString label(double value) const
	{
	int index=qRound(value);
	if (floor(value) == value && index > 0 && index <= (int)labels.count())
		return labels[index - 1];
	else
		return "";
	};
private:
	QStringList labels;
};

Graph::Graph(QWidget* parent, const char* name, WFlags f)
        : QWidget(parent,name,f)
{
	if ( !name )
	setName( "graph" );
	
	cOnPlot=0;linesOnPlot=0;
	widthLine=1;mrkX=-1;mrkY=-1;fitID=0;
	selectedCol=0;selectedPoint=-1;
    selectedCurve =-1;selectedMarker=-1;selectedCursor=-1;
	startPoint=0;endPoint=-1;
	startID=-1; endID=-1;
	pieRay=100;
	
	lineProfileOn=FALSE;
	drawTextOn=FALSE;
	drawLineOn=FALSE;
	cursorEnabled=FALSE;
	movePointsEnabled=FALSE;
	removePointsEnabled=FALSE;
	pickerEnabled=FALSE;
	rangeSelectorsEnabled=FALSE;
	piePlot=FALSE;
	isTitleSelected=FALSE;
	functions=0;
	ignoreResize= false;
	
	int i;
	for (i = 0; i<16; i++)
		scales<<"";
	
	for (i=0;i<QwtPlot::axisCnt;i++)
		{
		tickLabelsOn<<"1";
		axisType << Numeric;
		axesLabelsColumns<<"";
		lblFormat<<Automatic;
		}

	myPlot = new Plot(this,"field");		
	cp= new CanvasPicker(this);

	for (i = 0;i<QwtPlot::axisCnt;i++)
		{		
		QwtScale *scale = (QwtScale *)myPlot->axis(i);
		if (scale)
			scale->setBaselineDist(0);
		}
		
LegendMarker *mrk = new LegendMarker(myPlot);
mrk->setOrigin(QPoint(10, 10));
legendMarkerID=myPlot->insertMarker(mrk);
		
titlePicker = new TitlePicker(myPlot);
scalePicker = new ScalePicker(myPlot);

d_zoomer= new QwtPlotZoomer(QwtPlot::xBottom, QwtPlot::yLeft,
        QwtPicker::DragSelection, QwtPicker::AlwaysOff, myPlot->canvas());
d_zoomer->setRubberBandPen(Qt::black);
zoom(FALSE);

QArray<curveLayout> layout(1);
		
xCanvasMap=myPlot->canvasMap (QwtPlot::xBottom);
yCanvasMap=myPlot->canvasMap (QwtPlot::yLeft);
		
setGeometry( QRect(0, 0, 520, 420 ) );
setFocusPolicy(QWidget::StrongFocus);
setFocusProxy(myPlot);
setMouseTracking(TRUE );

grid.majorOnX=0;
grid.majorOnY=0;
grid.minorOnX=0;
grid.minorOnY=0;
grid.majorCol=0;
grid.majorStyle=2;
grid.majorWidth=1;
grid.minorCol=0;
grid.minorStyle=2;
grid.minorWidth=1;
grid.xZeroOn=0;
grid.yZeroOn=0;
	
setGridOptions(grid);

images.resize(0);

connect (cp,SIGNAL(resizeGraph(const QPoint&)),this,SLOT(resizeGraph(const QPoint&)));
connect (cp,SIGNAL(moveGraph(const QPoint&)),this,SLOT(moveGraph(const QPoint&)));
connect (cp,SIGNAL(releasedGraph()),this,SLOT(releaseGraph()));
connect (cp,SIGNAL(newSizeGraph()),this,SLOT(newSizeGraph()));

connect (cp,SIGNAL(selectPlot()),this,SLOT(activateGraph()));
connect (cp,SIGNAL(drawTextOff()),this,SIGNAL(drawTextOff()));
connect (cp,SIGNAL(viewImageDialog()),this,SIGNAL(viewImageDialog()));
connect (cp,SIGNAL(viewTextDialog()),this,SIGNAL(viewTextDialog()));
connect (cp,SIGNAL(viewLineDialog()),this,SIGNAL(viewLineDialog()));
connect (cp,SIGNAL(showPlotDialog()),this,SIGNAL(showPlotDialog()));
connect (cp,SIGNAL(showPieDialog()),this,SIGNAL(showPieDialog()));
connect (cp,SIGNAL(showMarkerPopupMenu()),this,SIGNAL(showMarkerPopupMenu()));
connect (cp,SIGNAL(modified()),this,SIGNAL(modifiedGraph()));
connect (cp,SIGNAL(calculateProfile(const QPoint&, const QPoint&))
					,this,SLOT(calculateLineProfile(const QPoint&, const QPoint&)));

connect (titlePicker,SIGNAL(doubleClicked()),this,SIGNAL(viewTitleDialog()));
connect (titlePicker,SIGNAL(removeTitle()),this,SLOT(removeTitle()));
connect (titlePicker,SIGNAL(clicked()), this,SLOT(selectTitle()));
connect (titlePicker,SIGNAL(moveGraph(const QPoint&)),this,SLOT(moveGraph(const QPoint&)));
connect (titlePicker,SIGNAL(releasedGraph()),this,SLOT(releaseGraph()));
connect (titlePicker,SIGNAL(resizeGraph(const QPoint&)),this,SLOT(resizeGraph(const QPoint&)));
connect (titlePicker,SIGNAL(resizedGraph()),this,SLOT(newSizeGraph()));

connect (scalePicker,SIGNAL(clicked()),this,SLOT(activateGraph()));
connect (scalePicker,SIGNAL(clicked()),this,SLOT(deselectItems()));
connect (scalePicker,SIGNAL(xAxisDblClicked()),this,SIGNAL(xAxisDblClicked()));
connect (scalePicker,SIGNAL(yAxisDblClicked()),this,SIGNAL(yAxisDblClicked()));
connect (scalePicker,SIGNAL(xAxisTitleDblClicked()),this,SIGNAL(xAxisTitleDblClicked()));		
connect (scalePicker,SIGNAL(yAxisTitleDblClicked()),this,SIGNAL(yAxisTitleDblClicked()));
connect (scalePicker,SIGNAL(rightAxisTitleDblClicked()),this,SIGNAL(rightAxisTitleDblClicked()));
connect (scalePicker,SIGNAL(topAxisTitleDblClicked()),this,SIGNAL(topAxisTitleDblClicked()));
connect (scalePicker,SIGNAL(resizeGraph(const QPoint&)),this,SLOT(resizeGraph(const QPoint&)));
connect (scalePicker,SIGNAL(resizedGraph()),this,SLOT(newSizeGraph()));
connect (scalePicker,SIGNAL(moveGraph(const QPoint&)),this,SLOT(moveGraph(const QPoint&)));
connect (scalePicker,SIGNAL(releasedGraph()),this, SLOT(releaseGraph()));

connect (this,SIGNAL(modifiedGraph()),this,SLOT(modified()));
connect (d_zoomer,SIGNAL(zoomed (const QwtDoubleRect &)),this,SLOT(zoomed (const QwtDoubleRect &)));
}

void Graph::resizeGraph(const QPoint& pos)
{
emit resizeGraph(this,pos);
}

void Graph::modified()
{
emit modifiedGraph(this);	
}

void Graph::moveGraph(const QPoint& pos)
{
emit moveGraph(this, pos);
}

void Graph::releaseGraph()
{
emit releaseGraph(this);
}

void Graph::newSizeGraph()
{
emit newSizeGraph(this);
}

void Graph::activateGraph()
{
emit selectedGraph(this);
}

void Graph::deselectItems()
{
isTitleSelected=FALSE;
myPlot->replot();
myPlot->titleLabel ()->repaint();
}

long Graph::selectedMarkerKey()
{
return selectedMarker;
}

QwtPlotMarker* Graph::selectedMarkerPtr()
{
return myPlot->marker(selectedMarker);
}

void Graph::setSelectedMarker(long mrk)
{
selectedMarker=mrk;
}

void Graph::selectLineMarker(long markerID)
{
LineMarker* mrkL=(LineMarker*) myPlot->marker(markerID);	
if (!mrkL)
	return;

selectedMarker=markerID;	
copyCanvas(TRUE);
	
QwtPlotCanvas *canvas=myPlot->canvas ();
int w=canvas->width();
int h=canvas->height();
QPixmap pix(w,h,-1);
pix.fill(QColor(white));
QPixmapCache::find ("field",pix);
// draw highlight rectangle 
QPainter painter(canvas);
painter.drawPixmap(0,0,pix,0,0,-1,-1);
painter.save();
QPoint aux=mrkL->startPoint();
painter.translate(aux.x(),aux.y());
double t=mrkL->teta();
painter.rotate(-t);
painter.setRasterOp(Qt::NotXorROP);
painter.setPen(QPen(QColor(red),1,Qt::SolidLine));
w=mrkL->width();
int d=w+(int)floor(mrkL->headLength()*tan(M_PI*mrkL->headAngle()/180.0)+0.5);
painter.drawRect(-d, -d-w, int(mrkL->length()+2*d), 2*(d+w));				
painter.restore();	
}

void Graph::selectTextMarker(long markerID)
{
LegendMarker* mrk=(LegendMarker*)myPlot->marker(markerID);
if (!mrk)
	return;

selectedMarker=markerID;		
copyCanvas(TRUE);
					
QwtPlotCanvas *canvas=myPlot->canvas ();
int w=canvas->width();
int h=canvas->height();

QPixmap pix(w,h,-1);
pix.fill( QColor(white));
QPixmapCache::find ("field",pix);

QPainter painter(canvas);
painter.drawPixmap(0,0,pix,0,0,-1,-1);
painter.setPen(QPen(red,2,Qt::SolidLine));
painter.setRasterOp(Qt::NotXorROP);
painter.drawRect (mrk->rect());	
}

void Graph::selectImageMarker(long markerID)
{
ImageMarker* mrkI=(ImageMarker*)myPlot->marker(markerID);
if (!mrkI)
		return;	

selectedMarker=markerID;
copyCanvas(TRUE);
					
QwtPlotCanvas *canvas=myPlot->canvas ();
int w=canvas->width();
int h=canvas->height();
QPixmap pix(w,h,-1);
pix.fill( QColor(white));
QPixmapCache::find ("field",pix);

QPainter painter(canvas);
painter.drawPixmap(0,0,pix,0,0,-1,-1);
painter.setPen(QPen(red,2,Qt::SolidLine));
painter.setRasterOp(Qt::NotXorROP);
painter.drawRect (mrkI->rect());	
}

void Graph::selectNextMarker()
{
QwtArray<long> mrkKeys=myPlot->markerKeys();
int n=mrkKeys.size();
if (n==0)
	return;

long min_key=mrkKeys[0], max_key=mrkKeys[0];
for (int i = 0; i<n; i++ )
	{
	if (mrkKeys[i] >= max_key)
		max_key=mrkKeys[i];
	if (mrkKeys[i] <= min_key)
		min_key=mrkKeys[i];
	}
	
QwtArray<long> texts=textMarkerKeys();
if (selectedMarker >= 0)
	{
	long key = selectedMarker+1;
	if ( key > max_key )
		key=min_key;
	while(key <= max_key)
		{
		if (texts.contains(key))
			{
			selectTextMarker(key);
			break;
			}
		else if (lines.contains(key))
			{
			selectLineMarker(key);
			break;
			}
		else  if (images.contains(key))
			{
			selectImageMarker(key);
			break;
			}
		else
			key++;
		}
	}
else
	{
	long key=min_key;
	while(selectedMarker < 0)
		{
		if (texts.contains(key))
			selectTextMarker(key);
		else if (lines.contains(key))
			selectLineMarker(key);
		else  if (images.contains(key))
			selectImageMarker(key);
		key++;
		}
	}
}

QwtPlot* Graph::plotWidget()
{
return (QwtPlot*)myPlot;	
}

int Graph::colX(int id)
{
return xCols[id];
}

void Graph::replot()
{
myPlot->replot();
}
	
void Graph::movedPicker(const QPoint &pos)
{
    QString info;
    info.sprintf("x=%g; y=%g",
        myPlot->invTransform(QwtPlot::xBottom, pos.x()),
        myPlot->invTransform(QwtPlot::yLeft, pos.y()));
	emit cursorInfo(info);
}

bool Graph::pickerActivated()
{
return pickerEnabled;
}

void Graph::initFonts(const QFont &scaleTitleFnt, const QFont &numbersFnt)
{
for (int i = 0;i<QwtPlot::axisCnt;i++)
	{
	myPlot->setAxisFont (i,numbersFnt);
	myPlot->setAxisTitleFont (i,scaleTitleFnt);
	}
}

void Graph::setAxisFont(int axis,const QFont &fnt)
{
myPlot->setAxisFont (axis,fnt);
myPlot->replot();
emit modifiedGraph();
}

QFont Graph::axisFont(int axis)
{
return myPlot->axisFont (axis);
}

void Graph::enableAxes(const QStringList& list)
{
int i;
for (i = 0;i<QwtPlot::axisCnt;i++)
	{
	bool ok=list[i+1].toInt();
	myPlot->enableAxis(i,ok);
	}
	
for (i = 0;i<QwtPlot::axisCnt;i++)
	{
	QwtScale *scale = (QwtScale *)myPlot->axis(i);
	if (scale)
		scale->setBaselineDist(0);
	}
scalePicker->refresh();
}

void Graph::enableAxes(QArray<bool> axesOn)
{
int i;
for (i = 0;i<QwtPlot::axisCnt;i++)
	myPlot->enableAxis(i,axesOn[i]);
	
for (i = 0;i<QwtPlot::axisCnt;i++)
	{
	QwtScale *scale = (QwtScale *)myPlot->axis(i);
	if (scale)
		scale->setBaselineDist(0);
	}
scalePicker->refresh();
}

QArray<bool> Graph::enabledAxes()
{
QArray<bool> axesOn(4);
for (int i = 0;i<QwtPlot::axisCnt;i++)
		{
		axesOn[i]=myPlot->axisEnabled (i);
		}
return axesOn;
}

QValueList<int> Graph::axesType()
{
return axisType;
}

QStringList Graph::axesLabelsDatasheets()
{
return axesLabelsColumns;
}


QValueList<int> Graph::ticksType()
{
return myPlot->getTicksType();
}

QStringList Graph::labelsFormat()
{
QStringList format;
for (int i= 0; i<8; i++)
	format<<"0";

int prec, fieldwidth;
char f;
for (int j= 0; j<4; j++)
	{
	if (myPlot->axisEnabled (j))
		{
		myPlot->axisLabelFormat (j, f, prec, fieldwidth);		
		
		format[2*j]=QString::number(lblFormat[j]);
		format[2*j+1]=QString::number(prec);
		}
	else
		{
		format[2*j]="0";
		format[2*j+1]="4";
		}
	}
return format;
}

void Graph::setLabelsFormat(const QStringList& list)
{
for (int axis= 0; axis<4; axis++)
		{
		int format=list[2*axis+1].toInt();
		int prec=list[2*axis+2].toInt();
			
		lblFormat[axis] = format;

		QwtScaleDraw *sd_old= (QwtScaleDraw *)myPlot->axisScaleDraw (axis);
		const QwtScaleDiv div=sd_old->scaleDiv ();
			
		if (format == Superscripts)
			{
			QwtScale *scale = (QwtScale *)myPlot->axis(axis);
			QColor color = QColor();
			if (scale)	
				{
				QPalette pal=scale->palette();
				color =pal.color(QPalette::Active, QColorGroup::Foreground);
				}
				
			QwtSupersciptsScaleDraw* sd= new QwtSupersciptsScaleDraw(axisFont(axis), color);
			myPlot->setAxisLabelFormat (axis, 'e', prec, 0);
			sd->setScale(div);
			myPlot->setAxisScaleDraw (axis,sd);	
			}
		else
			{				
			QwtScaleDraw *sd= new QwtScaleDraw();
				
			if (format == Automatic)
				myPlot->setAxisLabelFormat (axis, 'g', prec, 0);
			else if (format == Scientific )
				myPlot->setAxisLabelFormat (axis, 'e', prec, 0);
			else if (format == Decimal)
				myPlot->setAxisLabelFormat (axis, 'f', prec, 0);

			sd->setScale(div);
			myPlot->setAxisScaleDraw (axis,sd);	
			}
		}
}

void Graph::setAxesType(const QStringList& tl)
{
for (int i = 0; i < 4; i++)
	{
	axisType[i] = tl[i+1].toInt();
	}
}

QString Graph::saveAxesLabelsType()
{
QString s="AxisType\t";
for (int i=0; i<4; i++)
	s+=QString::number(axisType[i])+"\t";

s+="\nAxesTickLabelsCol\t";
s+=axesLabelsColumns.join("\t");
return s+"\n";
}

QString Graph::saveTicksType()
{
QValueList<int> ticksTypeList=myPlot->getTicksType();
QString s="EnabledTicks\t";
for (int i=0;i<4;i++)
	s+=QString::number(ticksTypeList[i])+"\t";
return s+"\n";
}

QStringList Graph::enabledTickLabels()
{
return tickLabelsOn;
}

QString Graph::saveEnabledTickLabels()
{
QString s="EnabledTickLabels\t";
for (int i=0;i<4;i++)
	s+=tickLabelsOn[i]+"\t";
return s+"\n";
}

QString Graph::saveLabelsFormat()
{
QString s="LabelsFormat\t";
QStringList format=labelsFormat();
s+=format.join("\t");
return s+"\n";
}

QString Graph::saveLabelsRotation()
{
QString s="LabelsRotation\t";
s+=QString::number(labelsRotation(QwtPlot::xBottom))+"\t";
s+=QString::number(labelsRotation(QwtPlot::xTop))+"\n";
return s;
}

void Graph::setEnabledTickLabels(const QStringList& labelsOn)
{
if (tickLabelsOn == labelsOn)
	return;

for (int i=0;i<4;i++)
	{
	if (labelsOn[i]=="0")
		myPlot->setAxisScaleDraw (i, new QwtNoLabelsScaleDraw());
	else
		{
		QwtScaleDraw *sd= new QwtScaleDraw();
		QwtScaleDraw *sd_old= (QwtScaleDraw *)myPlot->axisScaleDraw (i);	
	
		const QwtScaleDiv div=sd_old->scaleDiv ();
		sd->setScale(div);
		myPlot->setAxisScaleDraw (i,sd);		
		}
	}
tickLabelsOn=labelsOn;
}

void Graph::setTicksType(const QValueList<int>& list)
{
if (myPlot->getTicksType() == list)
	return;

for (int i=0;i<4;i++)
	{
	QwtScaleDraw *sd= (QwtScaleDraw *)myPlot->axisScaleDraw (i);
	if (list[i]==Plot::None || list[i]==Plot::In)
		sd->setTickLength(0,0,0);
	else if (list[i]==Plot::Out)
		sd->setTickLength(4,6,8);
	else if (list[i] == Plot::Both)
		sd->setTickLength (0,0,8); 
	
	myPlot->setTicksType(i,list[i]);
	}
}

void Graph::setTicksType(const QStringList& list)
{
for (int i=1;i<5;i++)
	{
	int type=list[i].toInt();
	int axis=i-1;
	QwtScaleDraw *sd= (QwtScaleDraw *)myPlot->axisScaleDraw (axis);
	if (type==Plot::None || type==Plot::In)
		sd->setTickLength(0,0,0);
	else if (type==Plot::Out)
		sd->setTickLength(4,6,8);
	else if (type == Plot::Both)
		sd->setTickLength (0,0,8); 
	
	myPlot->setTicksType(axis,type);
	}
}

void Graph::showAxis(int axis, int type, const QString& labelsColName, WksheetWindow *table, bool axisOn, int ticksType, bool labelsOn, 
					 const QColor& c, int format, int prec, int rotation)
{		
QValueList<int> ticksTypeList = myPlot->getTicksType();
	
char f;
int pr,fw;
myPlot->axisLabelFormat (axis, f, pr, fw);

if (myPlot->axisEnabled (axis) == axisOn &&
	tickLabelsOn[axis] == QString::number(labelsOn) &&
	ticksTypeList[axis] == ticksType &&
	axesColors()[axis] == c.name() &&
	prec == pr && format == lblFormat[axis] &&
	labelsRotation(axis) == rotation &&
	axisType[axis] == type &&
	axesLabelsColumns[axis] == labelsColName)
 return;

myPlot->enableAxis (axis,axisOn);

axisType[axis] = type;
tickLabelsOn[axis]=QString::number(labelsOn);

if (!labelsOn)
	myPlot->setAxisScaleDraw (axis, new QwtNoLabelsScaleDraw());
else
	{
	setAxisLabelRotation(axis, rotation);
		
	if (type == Numeric)
		{
		lblFormat[axis] = format;
			
		QwtScaleDraw *sd_old= (QwtScaleDraw *)myPlot->axisScaleDraw (axis);	
		const QwtScaleDiv div=sd_old->scaleDiv ();
			
		if (format == Superscripts)
			{
			QwtSupersciptsScaleDraw* sd= new QwtSupersciptsScaleDraw(myPlot->axisFont (axis), c);
			myPlot->setAxisLabelFormat (axis, 'e', prec, fw);
			sd->setScale(div);
			myPlot->setAxisScaleDraw (axis,sd);
			}
		else
			{				
			QwtScaleDraw *sd= new QwtScaleDraw();
				
			if (format == Automatic)
				myPlot->setAxisLabelFormat (axis, 'g', prec, fw);
			else if (format == Scientific )
				myPlot->setAxisLabelFormat (axis, 'e', prec, fw);
			else if (format == Decimal)
				myPlot->setAxisLabelFormat (axis, 'f', prec, fw);

			sd->setScale(div);
			myPlot->setAxisScaleDraw (axis,sd);
			}		
		}
	else 
		setAxisTickLabels(axis, type, labelsColName, table);
	}
	
QwtScale *scale = (QwtScale *)myPlot->axis(axis);
if (scale)
	{
	scale->setBaselineDist(0);	
	QPalette pal =scale->palette();
	if (pal.color(QPalette::Active, QColorGroup::Foreground) != c)
		{
		pal.setColor(QColorGroup::Foreground,c);
		scale->setPalette(pal);
		}
 	}
	
if (axis==QwtPlot::xTop && myPlot->axisEnabled(QwtPlot::xBottom))
	{
	const QwtScaleDiv *sd=myPlot->axisScale(QwtPlot::xBottom);	
	myPlot->setAxisScale (axis,sd->lBound (), sd->hBound (), sd->majStep());
		
	int majTicks=scales[3].toInt();
	int minTicks=scales[4].toInt();
	myPlot->setAxisMaxMajor(axis,majTicks);
	myPlot->setAxisMaxMinor(axis,minTicks);
	}
	
if (axis==QwtPlot::yRight && myPlot->axisEnabled(QwtPlot::yLeft))
	{
	const QwtScaleDiv *sd=myPlot->axisScale(QwtPlot::yLeft);	
	myPlot->setAxisScale (axis,sd->lBound (), sd->hBound (), sd->majStep());
	
	int majTicks=scales[10].toInt();
	int minTicks=scales[11].toInt();
	myPlot->setAxisMaxMajor(axis,majTicks);
	myPlot->setAxisMaxMinor(axis,minTicks);
	}
scalePicker->refresh();	

QwtScaleDraw *sclDraw= (QwtScaleDraw *)myPlot->axisScaleDraw (axis);	
if (ticksType == Plot::None || ticksType == Plot::In)
	sclDraw->setTickLength (0,0,0);
else if (ticksType == Plot::Out)
	sclDraw->setTickLength (4,6,8);
else if (ticksType == Plot::Both)
	sclDraw->setTickLength (0,0,8); 

//setAxisLabelRotation(axis, rotation);
myPlot->setTicksType(axis, ticksType);
myPlot->replot();	
emit modifiedGraph();
}


void Graph::setAxisTickLabels(int axis, int type, const QString& labelsColName, WksheetWindow *table)
{
if (type == Numeric || tickLabelsOn[axis].toInt() == 0)
	return;

QStringList list;
if (type == Txt)
	{
	if (!table)
		return;
	
	axesLabelsColumns[axis] = labelsColName;
	int r = table->tableRows();
	int col = table->colIndex(labelsColName);

	for (int i=0; i < r; i++)
		list<<table->text(i, col);
	}
else if (type == Day)
	{
	const QwtScaleDiv *sd = myPlot->axisScale (axis);
	int maj = sd->majCnt();
	int min = sd->minCnt();
	int k=0;
	QString day;
	for (int i=0; i<maj; i++)
		{
		while(k < min)
			{
			if (sd->minMark(k) < sd->majMark(i))
				{
				day = QDate::shortDayName (int(sd->minMark(k))%8);
				if (list.isEmpty () || list.last() != day)
					list<<day;
				k++;
				}
			else
				break;
			}
		
		day = QDate::shortDayName (int(sd->majMark (i))%8);
		if (list.isEmpty () || list.last() != day)
			list<<day;
		}

	while(k < min)
		{
		day = QDate::shortDayName (int(sd->minMark(k))%8);
		if (list.last() != day)
			list<<day;
		k++;
		}
	}
else if (type == Month)
	{
	const QwtScaleDiv *sd = myPlot->axisScale (axis);
	int maj = sd->majCnt();
	int min = sd->minCnt();
	int k=0;
	QString day;
	for (int i=0; i<maj; i++)
		{
		while(k < min)
			{
			if (sd->minMark(k) < sd->majMark(i))
				{
				day = QDate::shortMonthName (int(sd->minMark(k))%13);
				if (list.isEmpty () || list.last() != day)
					list<<day;
				k++;
				}
			else
				break;
			}
		
		day = QDate::shortMonthName (int(sd->majMark (i))%13);
		if (list.isEmpty () || list.last() != day)
			list<<day;
		}

	while(k < min)
		{
		day = QDate::shortMonthName (int(sd->minMark(k))%13);
		if (list.last() != day)
			list<<day;
		k++;
		}
	}
	
myPlot->setAxisScaleDraw (axis, new QwtTextScaleDraw(list));
axisType[axis] = type;
}

void Graph::setAxisLabelRotation(int axis, int rotation)
{
if (axis==QwtPlot::xBottom)
	{
	if (rotation > 0)
		myPlot->setAxisLabelAlignment(axis, Qt::AlignRight|Qt::AlignVCenter);
	else if (rotation < 0)
		myPlot->setAxisLabelAlignment(axis, Qt::AlignLeft|Qt::AlignVCenter);
	else if (rotation == 0)
		myPlot->setAxisLabelAlignment(axis, Qt::AlignHCenter|Qt::AlignBottom);
	}
else if (axis==QwtPlot::xTop)
	{
	if (rotation > 0)
		myPlot->setAxisLabelAlignment(axis, Qt::AlignLeft|Qt::AlignVCenter);
	else if (rotation < 0)
		myPlot->setAxisLabelAlignment(axis, Qt::AlignRight|Qt::AlignVCenter);
	else if (rotation == 0)
		myPlot->setAxisLabelAlignment(axis, Qt::AlignHCenter|Qt::AlignTop);
	}
myPlot->setAxisLabelRotation (axis, double(rotation));
}

int Graph::labelsRotation(int axis)
{
QwtScaleDraw *sclDraw= (QwtScaleDraw *)myPlot->axisScaleDraw (axis);
return (int)sclDraw->labelRotation();	
}

void Graph::setYAxisTitleFont(const QFont &fnt)
{
myPlot->setAxisTitleFont (QwtPlot::yLeft,fnt);
myPlot->replot();
emit modifiedGraph();
}

void Graph::setXAxisTitleFont(const QFont &fnt)
{
myPlot->setAxisTitleFont (QwtPlot::xBottom,fnt);
myPlot->replot();
emit modifiedGraph();
}

void Graph::setRightAxisTitleFont(const QFont &fnt)
{
myPlot->setAxisTitleFont (QwtPlot::yRight,fnt);
myPlot->replot();
emit modifiedGraph();
}

void Graph::setTopAxisTitleFont(const QFont &fnt)
{
myPlot->setAxisTitleFont (QwtPlot::xTop,fnt);
myPlot->replot();
emit modifiedGraph();
}

void Graph::setAxisTitleFont(int axis,const QFont &fnt)
{
myPlot->setAxisTitleFont (axis,fnt);
myPlot->replot();
emit modifiedGraph(); 
}

QFont Graph::axisTitleFont(int axis)
{
return myPlot->axisTitleFont (axis); 
}

QColor Graph::axisTitleColor(int axis)
{
QColor c;
QwtScale *scale = (QwtScale *)myPlot->axis(axis);
if (scale)
	c=scale->titleColor();
return c; 
}

void Graph::setCurveLegend(const QString& text,int key)
{
myPlot->setCurveTitle((long)key,text);
}

void Graph::setAxesColors(const QStringList& colors)
{
for (int i=0;i<4;i++)
	{
	QwtScale *scale = (QwtScale *)myPlot->axis(i);
	if (scale)
		{
		QPalette pal =scale->palette();
		pal.setColor(QColorGroup::Foreground,QColor(colors[i]));
		scale->setPalette(pal);
 		}
	}
}

QString Graph::saveAxesColors()
{
QString s="AxesColors\t";
QStringList colors;
QPalette pal;
int i;
for (i=0;i<4;i++)
colors<<(Qt::black).name();
		
for (i=0;i<4;i++)
	{
	QwtScale *scale = (QwtScale *)myPlot->axis(i);
	if (scale)	
		{
		pal=scale->palette();
		colors[i]=pal.color(QPalette::Active, QColorGroup::Foreground).name();
		}
	}
s+=colors.join ("\t")+"\n";
return s;
}

QStringList Graph::axesColors()
{
QStringList colors;
QPalette pal;
int i;
for (i=0;i<4;i++)
colors<<(Qt::black).name();
		
for (i=0;i<4;i++)
	{
	QwtScale *scale = (QwtScale *)myPlot->axis(i);
	if (scale)	
		{
		pal=scale->palette();
		colors[i]=pal.color(QPalette::Active, QColorGroup::Foreground).name();
		}
	}
return colors;
}

QFont Graph::titleFont()
{
return myPlot->titleFont();
}

QColor Graph::titleColor()
{
QLabel *title=myPlot->titleLabel();
QPalette pal =title->palette();
return pal.color(QPalette::Active, QColorGroup::Foreground);
}

void Graph::setTitleColor(const QColor & c)
{
QLabel *title=myPlot->titleLabel();
QPalette pal =title->palette();
pal.setColor(QColorGroup::Foreground,c);
if (title)
	title->setPalette(pal);
myPlot->replot();
emit modifiedGraph(); 
}

void Graph::setTitleAlignment(int align)
{
QLabel *title=myPlot->titleLabel();
title->setAlignment(align);
myPlot->replot();
emit modifiedGraph(); 
}

void Graph::setTitleFont(const QFont &fnt)
{
myPlot->setTitleFont(fnt);
myPlot->replot();
emit modifiedGraph(); 
}

QString Graph::title()
{
return myPlot->title();
}

int Graph::titleAlignment()
{
QLabel *title=myPlot->titleLabel();
return title->alignment();
}

void Graph::addCurve(const QString& name)
{
emit addCurveToGraph (this, name);
}

void Graph::setAutoScale()
{	
int scaleType=scales[6].toInt();

bool inverted = false;
if (scales[7] == "1")
	inverted = true;

int a = QwtPlot::xBottom;	
if (scaleType == 1)
	{
	if (inverted)
		myPlot->setAxisOptions(a, QwtAutoScale::Logarithmic | QwtAutoScale::Inverted);
	else
		myPlot->setAxisOptions(a, QwtAutoScale::Logarithmic);
	}
else if (scaleType == 0)
	{
	if (inverted)
		myPlot->setAxisOptions(a, QwtAutoScale::None | QwtAutoScale::Inverted);
	else
		myPlot->setAxisOptions(a, QwtAutoScale::None);
	}

scaleType=scales[14].toInt();
	
if (scales[15] == "1")
	inverted = true;
else
	inverted = false;	

a = QwtPlot::yLeft;	
if (scaleType == 1)
	{
	if (inverted)
		myPlot->setAxisOptions(a, QwtAutoScale::Logarithmic | QwtAutoScale::Inverted);
	else
		myPlot->setAxisOptions(a, QwtAutoScale::Logarithmic);
	}
else if (scaleType == 0)
	{
	if (inverted)
		myPlot->setAxisOptions(a, QwtAutoScale::None | QwtAutoScale::Inverted);
	else
		myPlot->setAxisOptions(a, QwtAutoScale::None);
	}
	
myPlot->setAxisAutoScale (QwtPlot::xBottom);
myPlot->setAxisAutoScale (QwtPlot::yLeft);
myPlot->replot();
updateScale();	
emit modifiedGraph();
}

void Graph::setYAxisTitle(const QString& text)
{
myPlot->setAxisTitle(0,text);
myPlot->replot();
emit modifiedGraph();
}

void Graph::setXAxisTitle(const QString& text)
{
myPlot->setAxisTitle(2,text);
myPlot->replot(); 
emit modifiedGraph();
}

void Graph::setRightAxisTitle(const QString& text)
{
myPlot->setAxisTitle(1,text);
myPlot->replot(); 
emit modifiedGraph();
}

void Graph::setTopAxisTitle(const QString& text)
{
myPlot->setAxisTitle(3,text);
myPlot->replot(); 
emit modifiedGraph();
}

int Graph::axisTitleAlignment (int axis)
{
return myPlot->axisTitleAlignment (axis);
}

void Graph::setAxesTitlesAlignment(const QStringList& align) 
{
for (int i=0;i<4;i++)	
	myPlot->setAxisTitleAlignment(i,align[i+1].toInt());	
}

void Graph::setXAxisTitleAlignment(int align) 
{
myPlot->setAxisTitleAlignment(QwtPlot::xBottom,align);
myPlot->replot(); 
emit modifiedGraph();	
}

void Graph::setYAxisTitleAlignment(int align) 
{
myPlot->setAxisTitleAlignment(QwtPlot::yLeft,align);
myPlot->replot(); 
emit modifiedGraph();	
}

void Graph::setTopAxisTitleAlignment(int align) 
{
myPlot->setAxisTitleAlignment(QwtPlot::xTop,align);
myPlot->replot(); 
emit modifiedGraph();	
}

void Graph::setRightAxisTitleAlignment(int align) 
{
myPlot->setAxisTitleAlignment(QwtPlot::yRight,align);
myPlot->replot(); 
emit modifiedGraph();	
}

void Graph::setAxisTitle(int axis, const QString& text)
{
int a;
switch (axis)
	{
	case 0:
		a=2;
	break;

	case 1:
		a=0;
	break;

	case 2:
		a=3;
	break;

	case 3:
		a=1;
	break;
	}

myPlot->setAxisTitle(a,text);
myPlot->replot();
emit modifiedGraph(); 
}

void Graph::setGridOptions(const gridOptions& options)
{
grid=options;
	
QColor minColor=MyDialog::color(grid.minorCol);
QColor majColor=MyDialog::color(grid.majorCol);

Qt::PenStyle majStyle;
Qt::PenStyle minStyle;

switch (grid.majorStyle)
	{
	case 0:
		majStyle=Qt::SolidLine;
	break;
	case 1:
		majStyle=Qt::DashLine;
	break;
	case 2:
		majStyle=Qt::DotLine;
	break;
	case 3:
		majStyle=Qt::DashDotLine;
	break;
	case 4:
		majStyle=Qt::DashDotDotLine;
	break;
	}

switch (grid.minorStyle)
	{
	case 0:
		minStyle=Qt::SolidLine;
	break;
	case 1:
		minStyle=Qt::DashLine;
	break;
	case 2:
		minStyle=Qt::DotLine;
	break;
	case 3:
		minStyle=Qt::DashDotLine;
	break;
	case 4:
		minStyle=Qt::DashDotDotLine;
	break;
	}

QPen majPen=QPen (majColor,grid.majorWidth,majStyle);
myPlot->setGridMajPen (majPen);

QPen minPen=QPen (minColor,grid.minorWidth,minStyle);
myPlot->setGridMinPen(minPen);

if (grid.majorOnX) myPlot->enableGridX (TRUE);
else if (grid.majorOnX==0) myPlot->enableGridX (FALSE);

if (grid.minorOnX) myPlot->enableGridXMin (TRUE);
else if (grid.minorOnX==0) myPlot->enableGridXMin (FALSE);

if (grid.majorOnY) myPlot->enableGridY (TRUE);
else if (grid.majorOnY==0) myPlot->enableGridY (FALSE);

if (grid.minorOnY) myPlot->enableGridYMin (TRUE);
else myPlot->enableGridYMin (FALSE);

if (mrkX<0 && grid.xZeroOn)
	{
	mrkX=myPlot->insertMarker();
	myPlot->setMarkerLineStyle(mrkX, QwtMarker::VLine);
	myPlot->setMarkerPos(mrkX, 0.0,0.0);
	myPlot->setMarkerPen(mrkX, QPen(black, 2,SolidLine));
	}
else if (mrkX>=0 && !grid.xZeroOn)
	{
	myPlot->removeMarker(mrkX);
	mrkX=-1;
	}
	
if (mrkY<0 && grid.yZeroOn)
	{
	myPlot->removeMarker(mrkY);
	mrkY=myPlot->insertMarker();
	myPlot->setMarkerLineStyle(mrkY, QwtMarker::HLine);
	myPlot->setMarkerPos(mrkY, 0.0,0.0);
	myPlot->setMarkerPen(mrkY, QPen(black, 2,SolidLine));
	}
else if (mrkY>=0 && !grid.yZeroOn)
	{
	myPlot->removeMarker(mrkY);
	mrkY=-1;		
	}
	
myPlot->replot();
}

gridOptions Graph::getGridOptions()
{
return grid;
}

QStringList Graph::scalesTitles()
{
QStringList scaleTitles;
int i,axis;
for (i=0;i<QwtPlot::axisCnt;i++)
	{
	switch (i)
	{
	case 0:
	axis=2;
	scaleTitles<<myPlot->axisTitle(axis);
	break;

	case 1:
	axis=0;
	scaleTitles<<myPlot->axisTitle(axis);
	break;

	case 2:
	axis=3;
	scaleTitles<<myPlot->axisTitle(axis);
	break;

	case 3:
	axis=1;
	scaleTitles<<myPlot->axisTitle(axis);
	break;
	}
}
return scaleTitles;
}

QStringList Graph::plotLimits()
{
return scales;
}

QStringList Graph::xBottomAxisLimits()
{
QStringList l;
l<<scales[0];
l<<scales[1];
return l;
}

QStringList Graph::secondaryPlotLimits()
{
QStringList limits;
QwtScaleDiv *scale;
bool topOn=myPlot->axisEnabled (QwtPlot::xTop);
bool rightOn=myPlot->axisEnabled (QwtPlot::yRight);
	
if (topOn && !rightOn)
	{
	scale=(QwtScaleDiv *)myPlot->axisScale(QwtPlot::xTop);
	limits<<QString::number(scale->lBound());	
	limits<<QString::number(scale->hBound());
	limits<<QString::number(scale->majStep());

	scale=(QwtScaleDiv *)myPlot->axisScale(QwtPlot::yLeft);
	limits<<QString::number(scale->lBound());	
	limits<<QString::number(scale->hBound());
	limits<<QString::number(scale->majStep());			
	}
else if (!topOn && rightOn)
	{
	scale=(QwtScaleDiv *)myPlot->axisScale(QwtPlot::xBottom);
	limits<<QString::number(scale->lBound());	
	limits<<QString::number(scale->hBound());
	limits<<QString::number(scale->majStep());
		
	scale=(QwtScaleDiv *)myPlot->axisScale(QwtPlot::yRight);
	limits<<QString::number(scale->lBound());	
	limits<<QString::number(scale->hBound());
	limits<<QString::number(scale->majStep());	
	}

else if (topOn && rightOn)
	{
	scale=(QwtScaleDiv *)myPlot->axisScale(QwtPlot::xTop);
	limits<<QString::number(scale->lBound());	
	limits<<QString::number(scale->hBound());
	limits<<QString::number(scale->majStep());			
	
	scale= (QwtScaleDiv *)myPlot->axisScale(QwtPlot::yRight);
	limits<<QString::number(scale->lBound());	
	limits<<QString::number(scale->hBound());
	limits<<QString::number(scale->majStep());	
	}
else
	{
	scale=(QwtScaleDiv *)myPlot->axisScale(QwtPlot::xBottom);
	limits<<QString::number(scale->lBound());	
	limits<<QString::number(scale->hBound());
	limits<<QString::number(scale->majStep());			
	
	scale= (QwtScaleDiv *)myPlot->axisScale(QwtPlot::yLeft);
	limits<<QString::number(scale->lBound());	
	limits<<QString::number(scale->hBound());
	limits<<QString::number(scale->majStep());	
	}
	
return limits;	
}

void Graph::setSecondaryPlotLimits(const QStringList& limits)
{
bool ok;
double start, end, step;	

start=limits[0].toDouble(&ok);
end=limits[1].toDouble(&ok);
step=limits[2].toDouble(&ok);
		
if (myPlot->axisEnabled(QwtPlot::xBottom))
	{
	int majTicks=scales[3].toInt();
	int minTicks=scales[4].toInt();
	myPlot->setAxisMaxMajor(QwtPlot::xTop,majTicks);
	myPlot->setAxisMaxMinor(QwtPlot::xTop,minTicks);	
	}
myPlot->setAxisScale(QwtPlot::xTop,start,end,step);

//yRight axis	
start=limits[3].toDouble(&ok);
end=limits[4].toDouble(&ok);
step=limits[5].toDouble(&ok);
		
if (myPlot->axisEnabled(QwtPlot::yLeft))
	{
	int majTicks=scales[10].toInt();
	int minTicks=scales[11].toInt();
	myPlot->setAxisMaxMajor(QwtPlot::yRight,majTicks);
	myPlot->setAxisMaxMinor(QwtPlot::yRight,minTicks);	
	}
myPlot->setAxisScale(QwtPlot::yRight,start,end,step);
myPlot->replot();
}

void Graph::setSecondaryPlotLimits(int axis,const QStringList& limits)
{	
bool ok;
int a=-1;
int majTicks, minTicks;
	
if (axis==2) 
	{
	a=3;
	majTicks=scales[3].toInt();
	minTicks=scales[4].toInt();
	}
else if (axis==3)
	{
	a=1;
	majTicks=scales[10].toInt();
	minTicks=scales[11].toInt();
	}
else
	return;

double start=limits[0].toDouble(&ok);
double end=limits[1].toDouble(&ok);
double step=limits[2].toDouble(&ok);

QwtScaleDiv *scale=(QwtScaleDiv *)myPlot->axisScale(a);
if (start == scale->lBound() && end == scale->hBound() && step == scale->majStep())
	return;

myPlot->setAxisScale(a,start,end,step);
myPlot->setAxisMaxMajor(a,majTicks);
myPlot->setAxisMaxMinor(a,minTicks);

myPlot->replot();

emit modifiedGraph();
}

QString Graph::saveSecondaryPlotLimits()
{
QString limits="SecondaryAxes\t";

QwtScaleDiv *scale;
if (myPlot->axisEnabled (QwtPlot::xTop) && myPlot->axisEnabled (QwtPlot::yRight))
	{	
	scale= (QwtScaleDiv *)myPlot->axisScale(QwtPlot::xTop);
	limits+=QString::number(scale->lBound())+"\t";	
	limits+=QString::number(scale->hBound())+"\t";
	limits+=QString::number(scale->majStep())+"\t";
		
	scale= (QwtScaleDiv *)myPlot->axisScale(QwtPlot::yRight);
	limits+=QString::number(scale->lBound())+"\t";	
	limits+=QString::number(scale->hBound())+"\t";
	limits+=QString::number(scale->majStep())+"\n";
	}
else if (myPlot->axisEnabled (QwtPlot::xTop) && ! myPlot->axisEnabled (QwtPlot::yRight))
	{	
	scale= (QwtScaleDiv *)myPlot->axisScale(QwtPlot::xTop);
	limits+=QString::number(scale->lBound())+"\t";	
	limits+=QString::number(scale->hBound())+"\t";
	limits+=QString::number(scale->majStep())+"\t";
		
	scale= (QwtScaleDiv *)myPlot->axisScale(QwtPlot::yLeft);
	limits+=QString::number(scale->lBound())+"\t";	
	limits+=QString::number(scale->hBound())+"\t";
	limits+=QString::number(scale->majStep())+"\n";
	}
else if (myPlot->axisEnabled (QwtPlot::yRight) && !myPlot->axisEnabled (QwtPlot::xTop))
	{
	scale= (QwtScaleDiv *)myPlot->axisScale(QwtPlot::xBottom);
	limits+=QString::number(scale->lBound())+"\t";	
	limits+=QString::number(scale->hBound())+"\t";
	limits+=QString::number(scale->majStep())+"\t";
				
	scale= (QwtScaleDiv *)myPlot->axisScale(QwtPlot::yRight);
	limits+=QString::number(scale->lBound())+"\t";	
	limits+=QString::number(scale->hBound())+"\t";
	limits+=QString::number(scale->majStep())+"\n";
	}
else
	limits="";

return limits;	
}

void Graph::setPlotLimits(int axis,const QStringList& limits)
{
int a;
if (axis==0) a=2;
else a=0;

scales=limits;

double start=scales[8*axis+0].toDouble();
double end=scales[8*axis+1].toDouble();
double step=scales[8*axis+2].toDouble();
int majTicks=scales[8*axis+3].toInt();
int minTicks=scales[8*axis+4].toInt();
int stepOn=scales[8*axis+5].toInt();
int scaleType=scales[8*axis+6].toInt();

bool inverted = false;
if (scales[8*axis+7] == "1")
	inverted = true;

if (scaleType == 1)
	{
	if (inverted)
		myPlot->setAxisOptions(a, QwtAutoScale::Logarithmic | QwtAutoScale::Inverted);
	else
		myPlot->setAxisOptions(a, QwtAutoScale::Logarithmic);
	
	if (stepOn == 1)	
		myPlot->setAxisScale(a,start,end,step);
	else
		{
		myPlot->setAxisScale(a, start, end, 0.0);
		myPlot->setAxisMaxMajor(a, majTicks);
		myPlot->setAxisMaxMinor(a, minTicks);
		}
	}
else if (scaleType == 0)
	{
	if (inverted)
		myPlot->setAxisOptions(a, QwtAutoScale::None | QwtAutoScale::Inverted);
	else
		myPlot->setAxisOptions(a, QwtAutoScale::None);

	if (stepOn==1)	
		myPlot->setAxisScale(a, start, end, step);
	else
		{
		myPlot->setAxisMaxMajor(a, majTicks);
		myPlot->setAxisMaxMinor(a, minTicks);
		myPlot->setAxisScale(a, start, end, 0.0);
		}
	}
	
//control minor and major ticks of the secondary axes
	
if (a==QwtPlot::xBottom && myPlot->axisEnabled(QwtPlot::xTop))
	{
	myPlot->setAxisMaxMajor(QwtPlot::xTop,majTicks);
	myPlot->setAxisMaxMinor(QwtPlot::xTop,minTicks);	
	}
	
if (a==QwtPlot::yLeft && myPlot->axisEnabled(QwtPlot::yRight))
	{
	myPlot->setAxisMaxMajor(QwtPlot::yRight,majTicks);
	myPlot->setAxisMaxMinor(QwtPlot::yRight,minTicks);	
	}
	
myPlot->replot();
emit modifiedGraph();
}

void Graph::definePlotLimits(QStringList s)
{
scales=s;
bool ok;
int axis,a, majTicks,minTicks,stepOn,scaleType;
double start,end,step;

for (axis=0;axis<2;axis++)
{
start=scales[8*axis+0].toDouble(&ok);
end=scales[8*axis+1].toDouble(&ok);
step=scales[8*axis+2].toDouble(&ok);
majTicks=scales[8*axis+3].toInt(&ok);
minTicks=scales[8*axis+4].toInt(&ok);
stepOn=scales[8*axis+5].toInt(&ok);
scaleType=scales[8*axis+6].toInt(&ok);
	
bool inverted = false;
if (scales[8*axis+7] == "1")
	inverted = true;

if (axis==0) a=2;
else a=0;
	
if (scaleType == 1)
	{
	if (inverted)
		myPlot->setAxisOptions(a, QwtAutoScale::Logarithmic | QwtAutoScale::Inverted);
	else
		myPlot->setAxisOptions(a, QwtAutoScale::Logarithmic);
	
	if (stepOn == 1)	
		myPlot->setAxisScale(a,start,end,step);
	else
		{
		myPlot->setAxisScale(a, start, end, 0.0);
		myPlot->setAxisMaxMajor(a, majTicks);
		myPlot->setAxisMaxMinor(a, minTicks);
		}
	}
else if (scaleType == 0)
	{
	if (inverted)
		myPlot->setAxisOptions(a, QwtAutoScale::None | QwtAutoScale::Inverted);
	else
		myPlot->setAxisOptions(a, QwtAutoScale::None);

	if (stepOn==1)	
		myPlot->setAxisScale(a, start, end, step);
	else
		{
		myPlot->setAxisMaxMajor(a, majTicks);
		myPlot->setAxisMaxMinor(a, minTicks);
		myPlot->setAxisScale(a, start, end, 0.0);
		}
	}
	}
	
myPlot->replot();
}

void Graph::copyCanvas(bool on)
{
int w=myPlot->canvas()->width();
int h=myPlot->canvas()->height();
	if (on)
	{
	QPixmap pix(w,h,-1);
	pix=pix.grabWidget(myPlot->canvas(),0,0,-1,-1 );
	QPixmapCache::insert ("field",pix);
	}
}

void Graph::shiftCurveSelector(bool up)
{
const QwtArray<long> keys = myPlot->curveKeys();

int index = 0;
if ( selectedCurve >= 0 )
    {
    for ( uint i = 0; i < keys.count() - 1; i++ )
        {
        if ( selectedCurve == keys[(int)i] )
            {
            index = i + (up ? 1 : -1);
            break;
            }
        }
    }
	
index = (keys.count() + index) % keys.count();
if ( selectedCurve != keys[index] )
        selectedCurve = keys[index];

QwtPlotCurve *curve = myPlot->curve(selectedCurve);
if ( !curve || curve->dataSize()<=0)
        return;


	startPoint=0;
	endPoint=curve->dataSize()-1;
		
	myPlot->setMarkerPos (startID,curve->x(startPoint),curve->y(startPoint));
	myPlot->setMarkerPos (endID,curve->x(endPoint),curve->y(endPoint));
	myPlot->replot();
	
	QString info;		
	if ( selectedCursor == startID )
		{
		info="L <=> ";
		selectedPoint=startPoint;
		}
	else if ( selectedCursor == endID )
		{
		info="R <=> ";
		selectedPoint=endPoint;
		}
		
	info+=curve->title(); 
	info+="[";
	info+=QString::number(selectedPoint);
	info+="]: x=";
	info+=QString::number(curve->x(selectedPoint));
	info+="; y=";
	info+=QString::number(curve->y(selectedPoint));
	
	emit cursorInfo(info);
	
	QPainter painter(myPlot->canvas());
	painter.setClipping(TRUE);
	painter.setClipRect(myPlot->canvas()->contentsRect());
	painter.setRasterOp(Qt::NotROP);

	curve->draw(&painter,
       myPlot->canvasMap(curve->xAxis()), myPlot->canvasMap(curve->yAxis()),
			selectedPoint,selectedPoint);
}

void Graph::shiftRangeSelector(bool shift)
{		
QwtPlotCurve *curve = myPlot->curve(selectedCurve);
if ( !curve || curve->dataSize()<=0)
        return;

QString info;	
if (shift)
	{
	QwtSymbol symbol=myPlot->markerSymbol (selectedCursor);
	if ( selectedCursor == startID )
		{
		symbol.setPen(QPen(black,2,Qt::SolidLine));
		myPlot->setMarkerSymbol (selectedCursor,symbol);
			
    	selectedCursor = endID;
		symbol=myPlot->markerSymbol (selectedCursor);
		symbol.setPen(QPen(red,2,Qt::SolidLine));
		myPlot->setMarkerSymbol (selectedCursor,symbol);
			
		selectedPoint = endPoint;
		/*endPoint=startPoint;
		startPoint=selectedPoint;*/
			
		info="R <=> ";
		}
	else if ( selectedCursor == endID )
		{
		symbol.setPen(QPen(black,2,Qt::SolidLine));
		myPlot->setMarkerSymbol (selectedCursor,symbol);
		selectedCursor = startID;
			
		symbol=myPlot->markerSymbol (selectedCursor);
		symbol.setPen(QPen(red,2,Qt::SolidLine));
		myPlot->setMarkerSymbol (selectedCursor,symbol);
			
    	selectedPoint = startPoint;
		/*startPoint=endPoint;
		endPoint=selectedPoint;*/
			
		info="L <=> ";
		}
	myPlot->replot();
		
	info+=curve->title(); 
	info+="[";
	info+=QString::number(selectedPoint+1);
	info+="]: x=";
	info+=QString::number(curve->x(selectedPoint));
	info+="; y=";
	info+=QString::number(curve->y(selectedPoint));
	
	emit cursorInfo(info);

	QPainter painter(myPlot->canvas());
	painter.setClipping(TRUE);
	painter.setClipRect(myPlot->canvas()->contentsRect());
	painter.setRasterOp(Qt::NotROP);

	curve->draw(&painter,
       myPlot->canvasMap(curve->xAxis()), myPlot->canvasMap(curve->yAxis()),
			selectedPoint,selectedPoint);
	}
}

void Graph::moveRangeSelector(bool up)
{	
QwtPlotCurve *curve = myPlot->curve(selectedCurve);
if ( !curve )
        return;

int points=curve->dataSize();
if (points == 0)
	{
	QMessageBox::critical(this,"QtiPlot - Warning",
				"All the curves on this plot are empty!");
	disableRangeSelectors();
	return;
	}

int index = selectedPoint + (up ? 1 : -1);
index = (index + points) % points;
	
QString info;
if ( index != selectedPoint )
    selectedPoint = index;

myPlot->setMarkerPos (selectedCursor,curve->x(selectedPoint),curve->y(selectedPoint));
myPlot->replot();

if ( selectedCursor == startID )
	{
    startPoint = selectedPoint;
	info="L <=> ";
	}
else if ( selectedCursor == endID )
	{
    endPoint = selectedPoint;
	info="R <=> ";
	}

info+=curve->title();
info+="[";
info+=QString::number(index+1);
info+="]: x=";
info+=QString::number(curve->x(index));
info+="; y=";
info+=QString::number(curve->y(index));
	
emit cursorInfo(info);

QPainter painter(myPlot->canvas());
painter.setClipping(TRUE);
painter.setClipRect(myPlot->canvas()->contentsRect());
painter.setRasterOp(Qt::NotROP);

curve->draw(&painter,
       myPlot->canvasMap(curve->xAxis()), myPlot->canvasMap(curve->yAxis()),
			selectedPoint,selectedPoint);	
}

//places the active range selector at the selected point
void Graph::moveRangeSelector()
{	
QwtPlotCurve *curve = myPlot->curve(selectedCurve);
if ( !curve )
        return;

int points=curve->dataSize();
if (points == 0)
	{
	QMessageBox::critical(this,"QtiPlot - Warning",
				"All the curves on this plot are empty!");
	disableRangeSelectors();
	return;
	}

int index = selectedPoint;
index = (index + points) % points;
	
QString info;
if ( index != selectedPoint )
    selectedPoint = index;

myPlot->setMarkerPos (selectedCursor,curve->x(selectedPoint),curve->y(selectedPoint));
myPlot->replot();

if ( selectedCursor == startID )
	{
    startPoint = selectedPoint;
	info="L <=> ";
	}
else if ( selectedCursor == endID )
	{
    endPoint = selectedPoint;
	info="R <=> ";
	}

info+=curve->title(); 
info+="[";
info+=QString::number(index+1);
info+="]: x=";
info+=QString::number(curve->x(index));
info+="; y=";
info+=QString::number(curve->y(index));
	
emit cursorInfo(info);

QPainter painter(myPlot->canvas());
painter.setClipping(TRUE);
painter.setClipRect(myPlot->canvas()->contentsRect());
painter.setRasterOp(Qt::NotROP);

curve->draw(&painter,
       myPlot->canvasMap(curve->xAxis()), myPlot->canvasMap(curve->yAxis()),
			selectedPoint,selectedPoint);	
}

bool Graph::selectorsEnabled()
{
return rangeSelectorsEnabled;
}

bool Graph::enableRangeSelectors(bool on)
{
rangeSelectorsEnabled=on;

long curveID;
QwtPlotCurve *curve=0;
bool success=FALSE;	
for (int i=cOnPlot-1;i>=0;i--)
	{
	curveID=keys[i];
	curve=myPlot->curve(curveID);
	if (curve && curve->dataSize()>0)
		{
		success=TRUE;
		break;
		}			
	}
	
if (!success)
	{
	QMessageBox::critical(this,"QtiPlot - Warning",
				"All the curves on this plot are empty!");
		
	disableRangeSelectors();
	return FALSE;
	}

startPoint=0;

if ( on )
	{	
	QCursor cursor=QCursor (QPixmap(vizor_xpm),-1,-1);
	myPlot->canvas()->setCursor(cursor);

	endPoint=curve->dataSize()-1;
	selectedCurve=curveID;
	selectedPoint=0;
	myPlot->canvas()->setFocus();
	
	int d=15;		
	QwtSymbol symbol=myPlot->curveSymbol(selectedCurve);
	if (symbol.style() != QwtSymbol::None)
		{
		QSize sz=symbol.size();
		d+=QMAX(sz.width(),sz.height());
		}
		
	startID=myPlot->insertMarker (QString::null,QwtPlot::xBottom,QwtPlot::yLeft);
	myPlot->setMarkerSymbol(startID,QwtSymbol(QwtSymbol::Rect,
					QBrush(Qt::NoBrush), QPen(red,2), QSize(d,d)));	
	myPlot->setMarkerPos (startID,curve->x(0),curve->y(0));
	selectedCursor=startID;
		
	endID=myPlot->insertMarker (QString::null,QwtPlot::xBottom,QwtPlot::yLeft);
	myPlot->setMarkerSymbol(endID,QwtSymbol(QwtSymbol::Rect,
					QBrush(Qt::NoBrush), QPen(black,2), QSize(d,d)));	
	myPlot->setMarkerPos (endID,curve->x(endPoint),curve->y(endPoint));
	myPlot->replot();
		
	QPainter painter(myPlot->canvas());
    
    painter.setClipping(TRUE);
    painter.setClipRect(myPlot->canvas()->contentsRect());
	painter.setRasterOp(Qt::NotROP);

    curve->draw(&painter,
        myPlot->canvasMap(curve->xAxis()), myPlot->canvasMap(curve->yAxis()),0,0);
	}
else
	disableRangeSelectors();	

return TRUE;
}

void  Graph::disableRangeSelectors()
{
myPlot->removeMarker(startID);
myPlot->removeMarker(endID);
myPlot->replot();
startID=-1;
endID=-1;
startPoint=0;
endPoint=-1;
selectedCurve=-1;
selectedPoint=-1;
selectedCursor=-1;	
rangeSelectorsEnabled=FALSE;	
myPlot->canvas()->setCursor(Qt::arrowCursor);
}

void Graph::enableCursor(bool on)
{
cursorEnabled=on;
}

bool Graph::enabledCursor()
{
return cursorEnabled;
}

void Graph::showPlotPicker(bool on)
{
pickerEnabled=on;
QCursor cursor=QCursor (QPixmap(cursor_xpm),-1,-1);
if (on)
	myPlot->canvas()->setCursor(cursor);
else 
	myPlot->canvas()->setCursor(Qt::arrowCursor);
}

void Graph::insertPlottedList(const QStringList& names)
{
for (int i=0; i<cOnPlot; i++)
	myPlot->setCurveTitle(keys[i], names[i]);
}

QStringList Graph::plotCurvesList()
{	
QStringList cList;
for (int i=0; i<cOnPlot; i++)
	cList<<myPlot->curveTitle(keys[i]);
return cList;
}

QStringList Graph::plotAssociations()
{
return associations;
}

void Graph::setPlotAssociations(const QStringList& newList)
{
associations=newList;
}

QStringList Graph::errorBarsAssociations()
{
return errorBars;
}

void Graph::setErrorBarsAssociations(const QStringList& newList)
{
errorBars=newList;
}

void Graph::copyImage()
{
QPixmap p=QPixmap::grabWidget(myPlot,2,2,myPlot->width()-4,myPlot->height()-4);		
QImage image= p.convertToImage();		
QApplication::clipboard()->setData( new QImageDrag (image,myPlot,0) );	
}

void Graph::saveTransparentImage(const QString& fileName,const QString& fileType)
{
//doesn't work for the moment
QPixmap pic;
QPainter paint;
paint.begin(&pic);
	
PrintFilter  filter(myPlot); 
//filter.setOptions(QwtPlotPrintFilter::PrintAll 
  //              & ~QwtPlotPrintFilter::PrintCanvasBackground);
	
myPlot->print(&paint, myPlot->rect(), filter);

paint.end();
QImage image = pic.convertToImage();
image.save(fileName,fileType,100);
}

void Graph::saveTransparentImage(QPixmap p,const QString& fileName,const QString& type)
{
QImage image;
image = p.convertToImage();
QRgb frame_pixel = image.pixel(0, 0);

QBitmap mask(p.size());
mask.fill(Qt::color1);
QPainter pic(&mask);
pic.setPen(Qt::color0);

for (int y=0; y<image.height(); y++)
	{
	for ( int x=0; x<image.width(); x++ )
		{
		QRgb rgb = image.pixel(x, y);
		if (rgb == frame_pixel) // we want the frame transparent
			{
			pic.drawPoint( x, y );
			}
		}
	}
pic.end();
p.setMask(mask);
p.save(fileName,type);
}

void Graph::exportGraph()
{	
int i;	
QStringList list=QImage::outputFormatList ();
QString filter="*.eps;;",selectedFilter,aux;

#ifdef Q_OS_WIN
  filter="";
#endif
	
for (i=0;i<(int)list.count();i++)
	{
	aux="*."+(list[i]).lower()+";;";
	filter+=aux;
	}
	
QString fname = QFileDialog::getSaveFileName( QString::null,filter,0,"file dialog",
	tr("Choose a filename to save under"),&selectedFilter,TRUE);
if ( !fname.isEmpty() ) 
	{ 		
		QFileInfo fi(fname);
		QString baseName = fi.fileName();
	
		if (baseName.contains(".")==0)
			fname.append(selectedFilter.remove("*"));		
		
		if ( QFile::exists(fname) &&
        	QMessageBox::question(
            0,
           tr("QtiPlot - Overwrite File?"),
           tr("A file called: <p><b>%1</b><p>already exists. "
                "Do you want to overwrite it?")
                .arg(fname),
            tr("&Yes"), tr("&No"),
            QString::null, 0, 1 ) )
        			return ;
			else
			{
			QFile f(fname);
			if ( !f.open( IO_WriteOnly ) ) 
			{
			QMessageBox::critical(0, tr("QtiPlot - Export Error"),tr("Could not write to file: <br><h4>"+fname+ "</h4><p>Please verify that you have the right to write to this location!").arg(fname));
   			 return;
			}
			
			if (selectedFilter.contains(".eps")>0)
				{
				exportToEPS(fname);
				return;
				}
				
			QPixmap p=QPixmap::grabWidget(myPlot,0,0,myPlot->width(),myPlot->height());
			QImage image;
			image = p.convertToImage();
				
			/*if (selectedFilter.contains(".png")>0)
				{
				//saveTransparentImage(p,fname,"PNG");
				saveTransparentImage(fname,"PNG");
				return;
				}
			else if (selectedFilter.contains(".xpm")>0)
				{
				saveTransparentImage(fname,"XPM");
				//saveTransparentImage(p,fname,"XPM");
				return;
				}*/
			
			for (i=0;i<(int)list.count();i++)
				{
				if (selectedFilter.contains("."+(list[i]).lower())>0)
					image.save(fname,list[i],100);	
				}
			}		
    }
}

void Graph::exportToEPS(const QString& fname)
{	
	QPrinter printer;
	printer.setResolution(84);
	printer.setPageSize ( QPrinter::A5 );
	printer.setOrientation(QPrinter::Landscape);
	printer.setColorMode (QPrinter::Color);
	printer.setFullPage(TRUE);
	printer.setOutputToFile (TRUE);
	printer.setOutputFileName(fname);
	
	PrintFilter  filter(myPlot); 
	filter.setOptions(QwtPlotPrintFilter::PrintAll 
                & ~QwtPlotPrintFilter::PrintCanvasBackground);
	
	QPaintDeviceMetrics mpr(&printer);	
   // export should preserve plot aspect ratio, if possible
    double aspect = double(myPlot->width())/double(myPlot->height());
	int width = int(aspect*mpr.height());
	int x=qRound(abs(mpr.width()-width)*0.5);
	QRect rect(x, 0, width, mpr.height());
    if (width > mpr.width())
		{
		rect.setLeft(0);
		rect.setWidth(mpr.width());
		}
	QPainter paint(&printer);
	myPlot->print(&paint, rect, filter);
}

void Graph::print()
{
	QPrinter printer;
	printer.setResolution(84); //empirical value, it works...
	printer.setOrientation(QPrinter::Landscape);
	printer.setColorMode (QPrinter::Color);
	printer.setFullPage(TRUE);

	if (printer.setup())
    	{
		QPainter paint(&printer);
		QPaintDeviceMetrics metrics( paint.device() );
		int dpiy = metrics.logicalDpiY();
		int margin = (int) ( (2/2.54)*dpiy ); // 2 cm margins
		QRect body( margin, margin, metrics.width() - 2*margin, metrics.height() - 2*margin );

		PrintFilter  filter(myPlot); 	
    	filter.setOptions(QwtPlotPrintFilter::PrintAll 
                & ~QwtPlotPrintFilter::PrintCanvasBackground);
		
    	myPlot->print(&paint, body, filter);
    	}
}

void Graph::movePoints(bool enabled)
{
movePointsEnabled=enabled;
if (enabled)
	myPlot->canvas()->setCursor(Qt::pointingHandCursor);
else 
	myPlot->canvas()->setCursor(Qt::arrowCursor);
}

bool Graph::movePointsActivated()
{
return movePointsEnabled;
}

void Graph::removePoints(bool enabled)
{
removePointsEnabled=enabled;
QCursor cursor=QCursor (QPixmap(vizor_xpm),-1,-1);

if (enabled)
	myPlot->canvas()->setCursor(cursor);
else 
	myPlot->canvas()->setCursor(Qt::arrowCursor);
}

bool Graph::removePointActivated()
{
return removePointsEnabled;
}

void Graph::selectPoint(const QPoint &pos)
{
    int dist, point;
    double xVal, yVal;

    const int curve = myPlot->closestCurve(
        pos.x(), pos.y(), dist, xVal, yVal, point);

    if ( curve >= 0 && dist < 10 ) // 10 pixels tolerance
		{
		const QwtPlotCurve *curv = myPlot->curve(curve);
		if (!curv)
			return;
		
		if (rangeSelectorsEnabled && selectedCurve != curve)
			{			
			if ( selectedCursor == startID )
				{
				startPoint=point;
				int n=curv->dataSize()-1;				
				endPoint=n;
				if (point ==n)
					endPoint=0;		
				}
			else if ( selectedCursor == endID )
				{
				startPoint=0;
				endPoint=point;
				if (point ==0)
					startPoint=curv->dataSize()-1;		
				}
				
			myPlot->setMarkerPos (startID,curv->x(startPoint),curv->y(startPoint));
			myPlot->setMarkerPos (endID,curv->x(endPoint),curv->y(endPoint));
			}
			
        selectedCurve = curve;
        selectedPoint = point;
		highlightPoint(TRUE);

		QString info;
		info=curv->title();
		info+="[";
		info+=QString::number(point+1);
		info+="]: x=";
		info+=QString::number(curv->x(point));
		info+="; y=";
		info+=QString::number(curv->y(point));
	
		emit cursorInfo(info);
		}
	else if (!rangeSelectorsEnabled)// deselect
		{
        selectedCurve = -1;
        selectedPoint = -1;
		highlightPoint(FALSE);
		}
}

int Graph::selectedCurveID()
{
return selectedCurve;	
}

void Graph::selectCurve(const QPoint &pos)
{
    int dist, point;
    double xVal, yVal;

    const int curve = myPlot->closestCurve(
        pos.x(), pos.y(), dist, xVal, yVal, point);

    if ( curve >= 0 && dist < 10 ) // 10 pixels tolerance
    {
        selectedCurve = curve;
        selectedPoint = point;
        showCursor(TRUE);

		const QwtPlotCurve *curv = myPlot->curve(selectedCurve);
		QString info;
		info=curv->title();
		info+="[";
		info+=QString::number(point+1);
		info+="]: x=";
		info+=QString::number(curv->x(point));
		info+="; y=";
		info+=QString::number(curv->y(point));
	
		emit cursorInfo(info);
    }
    else // deselect
    {
        showCursor(FALSE);
        selectedCurve = -1;
        selectedPoint = -1;
    }
}

bool Graph::markerSelected()
{
bool selected=FALSE;
if (selectedMarker>=0)
	selected=TRUE;
return  selected;
}

void Graph::removeMarker()
{
if (selectedMarker>=0)
	{
	myPlot->removeMarker(selectedMarker);
	myPlot->replot();
	emit modifiedGraph();
		
	if (selectedMarker==legendMarkerID)
		legendMarkerID=-1;

	if (lines.contains(selectedMarker)>0)
		{
		int i,index=lines.find(selectedMarker,0);
		for (i=index;i<linesOnPlot;i++)
			lines[i]=lines[i+1];
		linesOnPlot--;
		lines.resize(linesOnPlot);
		}

	else if (images.contains(selectedMarker)>0)
		{
		int i,index=images.find(selectedMarker,0);
		int imagesOnPlot=images.size();

		for (i=index;i<imagesOnPlot;i++)
			images[i]=images[i+1];
		imagesOnPlot--;
		images.resize(imagesOnPlot);
		}
		
	selectedMarker=-1;
	}
}

void Graph::cutMarker()
{
copyMarker();
removeMarker();
}

bool Graph::arrowMarkerSelected()
{
bool arrow=FALSE;
if (lines.contains(selectedMarker)>0)
	arrow=TRUE;
return arrow;
}

bool Graph::imageMarkerSelected()
{
bool image=FALSE;
if (images.contains(selectedMarker)>0)
	image=TRUE;
return image;
}

void Graph::copyMarker()
{
if (selectedMarker<0)
	{
	selectedMarkerType=None;
	return ;
	}

if (lines.contains(selectedMarker)>0)
		{
		LineMarker* mrkL=(LineMarker*) myPlot->marker(selectedMarker);
		auxMrkStart=mrkL->startPoint();
		auxMrkEnd=mrkL->endPoint();
		selectedMarkerType=Arrow;
		}
else if (images.contains(selectedMarker)>0)
		{
		ImageMarker* mrkI=(ImageMarker*) myPlot->marker(selectedMarker);
		auxMrkStart=mrkI->getOrigin();
		QRect rect=mrkI->rect();
		auxMrkEnd=rect.bottomRight();
		auxMrkFileName=mrkI->getFileName();
		selectedMarkerType=Image;
		}
else
	selectedMarkerType=Text;
}

void Graph::pasteMarker()
{
if (selectedMarkerType==Arrow)
	{
	LineMarker* mrkL=new LineMarker(myPlot);
	mrkL->setColor(auxMrkColor);
	mrkL->setWidth(auxMrkWidth);
	mrkL->setStyle(auxMrkStyle);
	mrkL->setStartPoint(QPoint(auxMrkStart.x()+10,auxMrkStart.y()+10));
	mrkL->setEndPoint(QPoint(auxMrkEnd.x()+10,auxMrkEnd.y()+10));
	mrkL->setStartArrow(startArrowOn);
	mrkL->setEndArrow(endArrowOn);
	mrkL->setHeadLength(auxArrowHeadLength);
	mrkL->setHeadAngle(auxArrowHeadAngle);
	mrkL->fillArrowHead(auxFilledArrowHead);
	long mrkID=myPlot->insertMarker(mrkL);
	linesOnPlot++;
	lines.resize(linesOnPlot);
	lines[linesOnPlot-1]=mrkID;
	myPlot->replot();
	
	selectedMarker=-1;
	}
else if (selectedMarkerType==Image)
	{
	QString fn=auxMrkFileName;
	QRect rect=QRect(auxMrkStart,auxMrkEnd);

	QPixmap photo;
	QFileInfo fi(fn);
	QString baseName = fi.fileName();
	int pos=baseName.find(".",0);
	QString type=baseName.right(baseName.length()-pos-1);
		
	if (type.upper()=="JPG")
			photo.load(fn,"JPEG",QPixmap::Auto);
	else
			photo.load(fn,type.upper(),QPixmap::Auto);	

	ImageMarker* mrk= new ImageMarker(photo, myPlot);
	mrk->setFileName(fn);

	QPoint o=myPlot->canvas()->mapFromGlobal(QCursor::pos());
	if (!myPlot->canvas()->contentsRect().contains(o))
		o=QPoint(auxMrkStart.x()+20,auxMrkStart.y()+20);

	mrk->setOrigin(o);
	mrk->setSize(rect.size());
	long mrkID=myPlot->insertMarker(mrk);
	myPlot->replot();
	
	int imagesOnPlot=images.size();
	imagesOnPlot++;
	images.resize(imagesOnPlot);
	images[imagesOnPlot-1]=mrkID;
	
	selectedMarker=-1;
	}
else
	{	
	LegendMarker* mrk=new LegendMarker(myPlot);
	QPoint o=myPlot->canvas()->mapFromGlobal(QCursor::pos());

	if (!myPlot->canvas()->contentsRect().contains(o))
		o=QPoint(auxMrkStart.x()+20,auxMrkStart.y()+20);
	
	mrk->setOrigin(o);
	mrk->setAngle(auxMrkAngle);
	mrk->setBackground(auxMrkBkg);
	mrk->setFont(auxMrkFont);
	mrk->setText(auxMrkText);
	mrk->setTextColor(auxMrkColor);
	myPlot->insertMarker(mrk);
	myPlot->replot();
	
	selectedMarker=-1;
	}
}

void Graph::setCopiedMarkerEnds(const QPoint& start, const QPoint& end)
{
auxMrkStart=start;
auxMrkEnd=end;
}

void Graph::setCopiedTextOptions(int bkg, const QString& text, const QFont& font, const QColor& color)
{
auxMrkBkg=bkg;
auxMrkText=text;
auxMrkFont=font;
auxMrkColor=color;
}

void Graph::setCopiedArrowOptions(int width, Qt::PenStyle style, const QColor& color,
															bool start, bool end, int headLength, 
															int headAngle, bool filledHead)
{
auxMrkWidth=width;	
auxMrkStyle=style;
auxMrkColor=color;
startArrowOn=start;
endArrowOn=end;
auxArrowHeadLength=headLength;
auxArrowHeadAngle=headAngle;
auxFilledArrowHead=filledHead;
}

bool Graph::titleSelected()
{
return isTitleSelected;
}

void Graph::setTitleSelected(bool on)
{
isTitleSelected=on;
}

void Graph::selectTitle()
{
if (!isTitleSelected)
	emit selectedGraph(this);

myPlot->replot();
isTitleSelected=TRUE;

QLabel *title=myPlot->titleLabel();
QPainter paint (title);
paint.setPen(QPen(Qt::black,1,Qt::DotLine));
paint.setRasterOp(Qt::NotROP);	
paint.drawRect (title->rect());
paint.end();
}

void Graph::setTitle(const QString& t)
{
QRect rect=myPlot->rect();
QString oldTitle=myPlot->title();
myPlot->setTitle (t);	
if (oldTitle.isEmpty())
	{
	myPlot->resize(rect.width(),rect.height()+1);
	myPlot->resize(rect.width(),rect.height()-1);
	}
emit modifiedGraph(); 
}

void Graph::removeTitle()
{
if (isTitleSelected)
	{
	QwtPlotLayout *plotLayout=myPlot->plotLayout();
	QRect tRect=plotLayout->titleRect ();
	int height=myPlot->height();
	int width=myPlot->width();	
	myPlot->setTitle("");
		
	int h=height-tRect.height();
	myPlot->resize(width,h);
	resize(width,h);
		
	emit modifiedGraph(); 
	}
}

void Graph::initTitle( bool on)
{
if (on)
	myPlot->setTitle("Title");
}

void Graph::initTitleFont( const QFont& fnt)
{
myPlot->setTitleFont(fnt);
}

void Graph::removeLegend()
{
if (legendMarkerID>=0)
	{
	myPlot->removeMarker(legendMarkerID);
	legendMarkerID=-1;
	}
}

void Graph::updateImageMarker(int x, int y, int width, int height)
{
ImageMarker* mrk=(ImageMarker*) myPlot->marker(selectedMarker);
mrk->setOrigin(QPoint(x,y));
mrk->setSize(QSize(width,height));
myPlot->replot();
emit modifiedGraph();
}

void Graph::updateLineMarker(const QColor& c,int w,Qt::PenStyle style,bool endArrow, bool startArrow)
{
LineMarker* mrkL=(LineMarker*) myPlot->marker(selectedMarker);
mrkL->setColor(c);
mrkL->setWidth(w);
mrkL->setStyle(style);
mrkL->setEndArrow(endArrow);
mrkL->setStartArrow(startArrow);

myPlot->replot();
emit modifiedGraph();
}

void Graph::setArrowHeadGeometry(int length, int angle, bool filled)
{
LineMarker* mrkL=(LineMarker*) myPlot->marker(selectedMarker);
if (!mrkL)
	return;

if (mrkL->headLength() != length)
	mrkL->setHeadLength( length );

if (mrkL->headAngle() != angle)
	mrkL->setHeadAngle( angle );

if (mrkL->filledArrowHead() != filled)
	mrkL->fillArrowHead( filled );

myPlot->replot();
emit modifiedGraph();
}

void Graph::updateLineMarkerGeometry(const QPoint& sp,const QPoint& ep)
{
LineMarker* mrkL=(LineMarker*) myPlot->marker(selectedMarker);
if (!mrkL)
	return;

QPoint startPoint=mrkL->startPoint();
QPoint endPoint=mrkL->endPoint();

if (startPoint != sp)
	mrkL->setStartPoint(sp);

if (endPoint != ep)
	mrkL->setEndPoint(ep);

myPlot->replot();
emit modifiedGraph();
}

void Graph::updateTextMarker(const QString& text,int angle, int bkg,const QFont& fnt,const QColor& c)
{
LegendMarker* mrkL=(LegendMarker*) myPlot->marker(selectedMarker);
mrkL->setText(text);
mrkL->setAngle(angle);
mrkL->setTextColor(c);
mrkL->setFont(fnt);
mrkL->setBackground(bkg);

myPlot->replot();
emit modifiedGraph();
}

void Graph::setLegendText(const QString& text)
{
if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		mrk->setText(text);	
	
	myPlot->replot();
	}
}

QString Graph::getLegendText()
{
QString legend;
if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		legend=mrk->getText();	
	}
return legend;
}

QString Graph::legendText()
{
int i;
QString text="";
for (i=0; i<cOnPlot; i++)
	{
	const QwtPlotCurve *curve=myPlot->curve(keys[i]);
	QString ass=associations[i];
	if (curve && ass.contains("(xErr)")<=0 && ass.contains("(yErr)")<=0)
		{
		text+="\\c{";
		text+=QString::number(i+1);
		text+="}";
		text+=curve->title(); //curvesList[id];
		text+="\n";
		}
	}

 /*QStringList curvesList = plotCurvesList();
 QwtPlotCurveIterator itc = myPlot->curveIterator();
 for (QwtPlotCurve *c = itc.toFirst(); c != 0; c = ++itc )
	{
	QString title = c->title();
	int i=curvesList.findIndex(title);
	QString ass=associations[i];
	if (ass.contains("(xErr)")<=0 && ass.contains("(yErr)")<=0)
		{
		text+="\\c{";
		text+=QString::number(i+1);
		text+="}";
		text+=title; 
		text+="\n";
		}
	}*/
	
return text;
}

QString Graph::pieLegendText()
{
QString text="";
	
const QwtPlotCurve *curve=myPlot->curve(keys[0]);
if (curve)
	{
	for (int i=0;i<int(curve->dataSize());i++)
		{
		text+="\\p{";
		text+=QString::number(i+1);
		text+="} ";
		text+=QString::number(i+1);
		text+="\n";
		}
	}
return text;
}

void Graph::moveBy(int dx, int dy)
{
    if ( dx == 0 && dy == 0 )
        return;

    const QwtPlotCurve *curve = myPlot->curve(selectedCurve);
    if ( !curve )
        return;

    const int x = myPlot->transform(curve->xAxis(),
        curve->x(selectedPoint)) + dx;
    const int y = myPlot->transform(curve->yAxis(),
        curve->y(selectedPoint)) + dy;

    move(QPoint(x, y));
}

void Graph::removePoint()
{
    QwtPlotCurve *curve = myPlot->curve(selectedCurve);
    if ( !curve )
        return;

	int id=keys.find(selectedCurve,0);
	QString name=associations[id];
	if (name.contains("=")>0)
		{
		QMessageBox::critical(0,"QtiPlot - Remove point error",
				"This function is not available for function curves!");
		return;	
		}

	if (name.contains("(yErr)",TRUE)>0 ||
		name.contains("(xErr)",TRUE)>0)	
		{
		int pos1=name.find(",",0);
		pos1=name.find(",",pos1+1);
		int pos2=name.find("(",pos1);
		name=name.mid(pos1+1,pos2-pos1-1);
		QwtErrorPlotCurve *err= (QwtErrorPlotCurve *) myPlot->curve(selectedCurve);
		double val=err->errors()[selectedPoint];
		emit clearCell(name,val);
		}
	else
		{
		int pos1=name.find(",",0);
		int pos2=name.find("(",pos1);
		name=name.mid(pos1+1,pos2-pos1-1);
		emit clearCell(name,curve->y(selectedPoint));
		}		
		
	this->setFocus();
	selectedPoint=-1;
	selectedCurve=-1;
}

// Move the selected point
void Graph::move(const QPoint &pos)
{
    QwtPlotCurve *curve = myPlot->curve(selectedCurve);
    if ( !curve )
        return;
	
	int id=keys.find(selectedCurve,0);
	if (associations[id].contains("=")>0)
		{
		QMessageBox::critical(0,"QtiPlot - Move point error",
				"This function is not available for function curves!");
		return;	
		}

    QwtArray<double> xData(curve->dataSize());
    QwtArray<double> yData(curve->dataSize());

    for ( int i = 0; i < curve->dataSize(); i++ )
    {
        if ( i == selectedPoint )
        {
            xData[i] = myPlot->invTransform(curve->xAxis(), pos.x());
            yData[i] = myPlot->invTransform(curve->yAxis(), pos.y());
        }
        else
        {
            xData[i] = curve->x(i);
            yData[i] = curve->y(i);
        }
    }
    curve->setData(xData, yData);
    myPlot->replot();

	QString text=QString::number(xData[selectedPoint])+"\t"+QString::number(yData[selectedPoint]);
	emit updateTable(associations[id],selectedPoint,text);

	QString info;
    info.sprintf("x=%g; y=%g",
        xData[selectedPoint],
        yData[selectedPoint]);
	emit cursorInfo(info);
}

// Hightlight the selected point
void Graph::highlightPoint(bool showIt)
{
    QwtPlotCurve *curve = myPlot->curve(selectedCurve);
    if ( !curve )
        return;

    QPainter painter(myPlot->canvas());
    
    painter.setClipping(TRUE);
    painter.setClipRect(myPlot->canvas()->contentsRect());

    if ( showIt )
        painter.setRasterOp(Qt::NotROP);

    curve->draw(&painter,
        myPlot->canvasMap(curve->xAxis()), myPlot->canvasMap(curve->yAxis()),
        selectedPoint,selectedPoint);
}

void Graph::showCursor(bool showIt)
{
    QwtPlotCurve *curve = myPlot->curve(selectedCurve);
    if ( !curve )
        return;

    QPainter painter(myPlot->canvas());

	int x=myPlot->transform(QwtPlot::xBottom,curve->x(selectedPoint));
	int y=myPlot->transform(QwtPlot::yLeft,curve->y(selectedPoint));

	myPlot->replot();
	if ( showIt )
		{
		painter.setRasterOp(Qt::NotXorROP);
		QwtMarker mrkCross;
		mrkCross.setLinePen (QPen(red,1));
		mrkCross.setLineStyle(QwtMarker::Cross);
		mrkCross.setSymbol(QwtSymbol(QwtSymbol::Rect,
					QBrush(Qt::NoBrush), QPen(black,1), QSize(25,25)));
		mrkCross.draw(&painter,x,y,myPlot->canvas()->contentsRect());				
		}
}

// Select the next/previous neighbour of the selected point
void Graph::shiftPointCursor(bool up)
{
    const QwtPlotCurve *curve = myPlot->curve(selectedCurve);
    if ( !curve )
        return;

    int index = selectedPoint + (up ? 1 : -1);
    index = (index + curve->dataSize()) % curve->dataSize();
	
	QString info;
	info=curve->title(); 
	info+="[";
	info+=QString::number(index+1);
	info+="]: x=";
	info+=QString::number(curve->x(index));
	info+="; y=";
	info+=QString::number(curve->y(index));
	
	emit cursorInfo(info);

    if ( index != selectedPoint )
    {
        showCursor(FALSE);
        selectedPoint = index;
        showCursor(TRUE);
    }
}

// Select the next/previous curve 
void Graph::shiftCurveCursor(bool up)
{
    const QwtArray<long> keys = myPlot->curveKeys();

    int index = 0;
    if ( selectedCurve >= 0 )
    {
        for ( uint i = 0; i < keys.count() - 1; i++ )
        {
            if ( selectedCurve == keys[(int)i] )
            {
                index = i + (up ? 1 : -1);
                break;
            }
        }
    }
    index = (keys.count() + index) % keys.count();

    if ( selectedCurve != keys[index] )
    {
        showCursor(FALSE);
        selectedCurve = keys[index];
        showCursor(TRUE);
    }

	const QwtPlotCurve *curve = myPlot->curve(selectedCurve);
    if ( !curve )
        return;

	QString info;
	info=curve->title(); //curvesList[id];
	info+="[";
	info+=QString::number(selectedPoint+1);
	info+="]: x=";
	info+=QString::number(curve->x(selectedPoint));
	info+="; y=";
	info+=QString::number(curve->y(selectedPoint));
	emit cursorInfo(info);
}

void Graph::clearPlot()
{
	myPlot->clear();
	myPlot->replot();
}

void Graph::updateCurveData(WksheetWindow* w, const QString& yColName, int curve)
{
QString as=associations[curve];	
if (as.contains("(yErr)",TRUE)>0 || as.contains("(xErr)",TRUE)>0)
	{
	updateErrorBarsData(w, curve);
	return;
	}
else
	{		
	if (piePlot)
		{		
		updatePieCurveData(w, yColName, curve);
		return;
		}
	
	int plotType = layout[curve].plotType;
	if (plotType == Histogram)
		{
		updateHistogram(w, yColName, curve);
		return;
		}
	else if (plotType == VectXYXY)
		{
		updateVectXYXY(w, curve);
		return;
		}
	else
		updateData(w, yColName, curve);
	}
}

void Graph::updateData(WksheetWindow* w, const QString& yColName, int curve)
{
long curveID = keys[curve];
QStringList cols = QStringList::split(",", associations[curve], false);
QString xcName = cols[0].remove("(X)");
QString ycName = cols[1].remove("(Y)");

int xcol=w->colIndex(xcName);
int ycol=w->colIndex(ycName);

QArray<double> X,Y;
int i, it=0;
int r = w->tableRows();

int xColType = w->checkColType(xcol);
int yColType = w->checkColType(ycol);

QStringList xLabels, yLabels;// store text labels
for (i = 0; i<r; i++ )
	{
	QString xval=w->text(i,xcol);
	QString yval=w->text(i,ycol);
	if (!xval.isEmpty() && !yval.isEmpty())
		{
		it++;
		X.resize(it);
		Y.resize(it);
		if (xColType == WksheetWindow::Text)
			{
			xLabels<<xval;
			X[it-1]=(double)it;
			}
		else
			X[it-1]=xval.toDouble();

		if (yColType == WksheetWindow::Text)
			{
			yLabels<<yval;
			Y[it-1]=(double)it;
			}
		else
			Y[it-1]=yval.toDouble();
		}
	}

//update curve data
if (layout[curve].plotType == HorizontalBars)
	myPlot->setCurveData(curveID, Y, X, it);
else
	myPlot->setCurveData(curveID, X, Y, it);

if (xColType == WksheetWindow::Text)
	{
	if (layout[curve].plotType == HorizontalBars)
		{
		axisType[QwtPlot::yLeft] = Txt;
		axesLabelsColumns[QwtPlot::yLeft] = xcName;
		myPlot->setAxisScaleDraw (QwtPlot::yLeft, new QwtTextScaleDraw(xLabels));
		}
	else
		{
		axisType[QwtPlot::xBottom] = Txt;
		axesLabelsColumns[QwtPlot::xBottom] = xcName;
		myPlot->setAxisScaleDraw (QwtPlot::xBottom, new QwtTextScaleDraw(xLabels));
		}
	}
	
if (yColType == WksheetWindow::Text)
	{
	axisType[QwtPlot::yLeft] = Txt;
	axesLabelsColumns[QwtPlot::yLeft] = ycName;
	myPlot->setAxisScaleDraw (QwtPlot::yLeft, new QwtTextScaleDraw(yLabels));
	}

// move error bars
for (i=0;i<cOnPlot;i++)
	{
	QString as = associations[i];
	if (as.contains(yColName,TRUE)>0 && (as.contains("(yErr)",TRUE)>0 || as.contains("(xErr)",TRUE)>0))
		{
		myPlot->setCurveData(keys[i], X, Y, it);
		}
	}

if (!removePointsEnabled && !movePointsEnabled)
	{
	for (i=0;i<QwtPlot::axisCnt; i++)
		myPlot->setAxisAutoScale(i);
	}			
	
myPlot->replot(); 
updateScale();	
}

void Graph::updateErrorBarsData(WksheetWindow* w, int curve)
{
QwtErrorPlotCurve *errors=(QwtErrorPlotCurve *)myPlot->curve(keys[curve]);
if (!errors)
	return;

QStringList asl = QStringList::split(",",associations[curve], FALSE);
QString errColName = asl[2].remove("(yErr)").remove("(xErr)");
int i;
int errcol = w->colIndex(errColName);
int size = errors->dataSize();
QArray<double> err(size);
for (i = 0; i<size; i++ )
	{
	QString errval=w->text(i,errcol);
	err[i]=errval.toDouble();
	}	

QArray<double> err_old = errors->errors();
if (err_old == err)
	return;
	
errors->setErrors(err);	

if (!removePointsEnabled && !movePointsEnabled)
	{
	for (i=0;i<QwtPlot::axisCnt; i++)
		myPlot->setAxisAutoScale(i);
	}

myPlot->replot(); 
updateScale();
}

void Graph::updatePieCurveData(WksheetWindow* w, const QString& yColName, int curve)
{
QArray<double> X;
int it=0;
int ycol=w->colIndex(yColName);

for (int i = 0; i<w->tableRows(); i++ )
		{
		QString xval=w->text(i, ycol);
		if (!xval.isEmpty())
			{
			it++;
			X.resize(it);
			X[it-1]=xval.toDouble();
			}
		}
		
myPlot->setCurveData(keys[curve], X, X, it);
myPlot->replot();	
}

void Graph::updateVectXYXY(WksheetWindow* w, int curve)
{
long curveID = keys[curve];
VectXYXYCurve *vect=(VectXYXYCurve *)myPlot->curve(curveID);
if (!vect)
	return;

QStringList cols=QStringList::split(",", associations[curve], false);
int xcol=w->colIndex(cols[0].remove("(X)"));
int ycol=w->colIndex(cols[1].remove("(Y)"));
int endXCol=w->colIndex(cols[2].remove("(X)"));
int endYCol=w->colIndex(cols[3].remove("(Y)"));
	
QArray<double> X(1), Y(1), endX(1), endY(1);
	
int i, it=0;
for (i = 0; i< w->tableRows(); i++ )
		{
		QString xval=w->text(i, xcol);
		QString yval=w->text(i, ycol);
		QString xend=w->text(i, endXCol);
		QString yend=w->text(i, endYCol);
		if (!xval.isEmpty() && !yval.isEmpty() && !xend.isEmpty() && !yend.isEmpty())
			{
			it++;
			X.resize(it); Y.resize(it); endX.resize(it); endY.resize(it);
			X[it-1]=xval.toDouble();
			Y[it-1]=yval.toDouble();
			endX[it-1]=xend.toDouble();
			endY[it-1]=yend.toDouble();
			}
		}
		
myPlot->setCurveData(curveID, X, Y, it);
vect->setVectorEnd(endX, endY);

if (!removePointsEnabled && !movePointsEnabled)
	{
	for (i=0;i<QwtPlot::axisCnt; i++)
		myPlot->setAxisAutoScale(i);
	}

myPlot->replot(); 
updateScale();
}

QString Graph::saveEnabledAxes()
{
QString list="EnabledAxes\t";
for (int i = 0;i<QwtPlot::axisCnt;i++)
	list+=QString::number(myPlot->axisEnabled (i))+"\t";

list+="\n";
return list;
}

bool Graph::framed()
{
bool frameOn=FALSE;

QwtPlotCanvas* canvas=(QwtPlotCanvas*) myPlot->canvas();  
if (canvas->lineWidth()>0)
	frameOn=TRUE;

return frameOn;
}

QColor Graph::canvasFrameColor()
{
QwtPlotCanvas* canvas=(QwtPlotCanvas*) myPlot->canvas();  
QPalette pal =canvas->palette();
return pal.color(QPalette::Active, QColorGroup::Foreground);
}

int Graph::canvasFrameWidth()
{
QwtPlotCanvas* canvas=(QwtPlotCanvas*) myPlot->canvas();  
return canvas->lineWidth();
}

void Graph::drawCanvasFrame(const QStringList& frame)
{
QwtPlotCanvas* canvas=(QwtPlotCanvas*) myPlot->canvas();	
canvas->setLineWidth((frame[1]).toInt());

QPalette pal =canvas->palette();
pal.setColor(QColorGroup::Foreground,QColor(frame[2]));
canvas->setPalette(pal); 
	
for (int i=0; i<QwtPlot::axisCnt; i++)
	{
	QwtScaleDraw *sclDraw= (QwtScaleDraw *)myPlot->axisScaleDraw (i);
	sclDraw->setOptions(0);
	}
}

void Graph::drawCanvasFrame(bool frameOn, int width, const QColor& color)
{
QwtPlotCanvas* canvas=(QwtPlotCanvas*) myPlot->canvas();  
QPalette pal = canvas->palette();
	
if (frameOn && canvas->lineWidth() == width && 
	pal.color(QPalette::Active, QColorGroup::Foreground) == color)
	return;
	
if (frameOn)
	{
	canvas->setLineWidth(width);
	pal.setColor(QColorGroup::Foreground,color);
	canvas->setPalette(pal);
	}
else
	{
	canvas->setLineWidth(0);
	pal.setColor(QColorGroup::Foreground,QColor(black));
	canvas->setPalette(pal);
	}

for (int i=0; i<QwtPlot::axisCnt; i++)
		{
		QwtScale *scale=(QwtScale*) myPlot->axis(i);
		if (scale)
			{
			QwtScaleDraw *sclDraw= (QwtScaleDraw *)myPlot->axisScaleDraw (i);
			if (frameOn)
				sclDraw->setOptions(0);
			else
				sclDraw->setOptions(1);
			
			scale->repaint();
			}
		}
			
emit modifiedGraph(this);	
}

void Graph::drawCanvasFrame(bool frameOn)
{
if (frameOn)

	{
	QwtPlotCanvas* canvas=(QwtPlotCanvas*) myPlot->canvas();	
	canvas->setLineWidth(1);
	
	for (int i=0; i<QwtPlot::axisCnt; i++)
		{
		QwtScaleDraw *sclDraw= (QwtScaleDraw *)myPlot->axisScaleDraw (i);
		sclDraw->setOptions(0);
		}
	}
}

QString Graph::saveCanvasFrame()
{
QString s="CanvasFrame\t";
QwtPlotCanvas* canvas=(QwtPlotCanvas*) myPlot->canvas();
int w=canvas->lineWidth();
if (w>0)
	{
	s+=QString::number(w)+"\t";
	s+=canvasFrameColor().name()+"\n";
	}
else s="";
return s;
}

QString Graph::saveFonts()
{
int i;
QString s;
QStringList list,axesList;
QFont f;
list<<"TitleFont";
f=myPlot->titleFont();
list<<f.family();
list<<QString::number(f.pointSize());
list<<QString::number(f.weight());
list<<QString::number(f.italic());
list<<QString::number(f.underline());
list<<QString::number(f.strikeOut());
s=list.join ("\t")+"\n";

for (i=0;i<myPlot->axisCnt;i++)
	{
	f=myPlot->axisTitleFont(i);
	list[0]="ScaleFont"+QString::number(i);
	list[1]=f.family();
	list[2]=QString::number(f.pointSize());
	list[3]=QString::number(f.weight());
	list[4]=QString::number(f.italic());
	list[5]=QString::number(f.underline());
	list[6]=QString::number(f.strikeOut());
	s+=list.join ("\t")+"\n";
	}

for (i=0;i<myPlot->axisCnt;i++)
	{
	f=myPlot->axisFont(i);
	list[0]="AxisFont"+QString::number(i);
	list[1]=f.family();
	list[2]=QString::number(f.pointSize());
	list[3]=QString::number(f.weight());
	list[4]=QString::number(f.italic());
	list[5]=QString::number(f.underline());
	list[6]=QString::number(f.strikeOut());
	s+=list.join ("\t")+"\n";
	}
return s;
}

QString Graph::saveScale()
{
QString s;
s=scales.join ("\t");
return s+"\n";
}

QString Graph::saveErrorBars()
{
QString all;
int i,j;
for (i=0;i<cOnPlot;i++)
{
	QString s="";
	if (associations[i].contains("(yErr)")>0 || associations[i].contains("(xErr)")>0)
	{
		long curveID = keys[i];
		QwtErrorPlotCurve *errors=(QwtErrorPlotCurve *)myPlot->curve(curveID);
		if (errors)
		{
		s+="ErrorBars\t";
		s+=QString::number(errors->direction())+"\t";
		QStringList cvs=QStringList::split(",",associations[i],FALSE);
		for (j=0;j<(int)cvs.count();j++)
			{
			int pos=cvs[j].find("(",0);
			s+=cvs[j].left(pos)+"\t";
			}
		s+=QString::number(errors->width())+"\t";
		s+=QString::number(errors->capLength())+"\t";
		s+=errors->color().name()+"\t";
		s+=QString::number(errors->throughSymbol())+"\t";
		s+=QString::number(errors->plusSide())+"\t";
		s+=QString::number(errors->minusSide())+"\n";
		all+=s;
		}				
	}
}
return all;
}

void Graph::setXAxisTitleColor(const QColor& c)
{
QwtScale *scale = (QwtScale *)myPlot->axis(QwtPlot::xBottom);
if (scale)
	{
	scale->setTitleColor (c);
	emit modifiedGraph();
	}
}

void Graph::setYAxisTitleColor(const QColor& c)
{
QwtScale *scale = (QwtScale *)myPlot->axis(QwtPlot::yLeft);
if (scale)
	{
	scale->setTitleColor (c);
	emit modifiedGraph();
	}
}

void Graph::setRightAxisTitleColor(const QColor& c)
{
QwtScale *scale = (QwtScale *)myPlot->axis(QwtPlot::yRight);
if (scale)
	{
	scale->setTitleColor (c);
	emit modifiedGraph();
	}
}

void Graph::setTopAxisTitleColor(const QColor& c)
{
QwtScale *scale = (QwtScale *)myPlot->axis(QwtPlot::xTop);
if (scale)
	{
	scale->setTitleColor (c);
	emit modifiedGraph();
	}
}

void Graph::setAxesTitleColor(QStringList l)
{
for (int i=0;i<int(l.count()-1);i++)
	{
	QwtScale *scale = (QwtScale *)myPlot->axis(i);
	if (scale)
		scale->setTitleColor (QColor(l[i+1]));
	}
}

QString Graph::saveAxesTitleColors()
{
QString s="AxesTitleColors\t";
for (int i=0;i<4;i++)
	{
	QwtScale *scale = (QwtScale *)myPlot->axis(i);
	QColor c;
	if (scale)
		c=scale->titleColor();
	else
		c=QColor(Qt::black);
	
	s+=c.name()+"\t";	
	}	
return s+"\n";
}

QString Graph::saveTitle()
{
QString s="PlotTitle\t";
s+=title().replace("\n", "<br>")+"\t";

QLabel *title=myPlot->titleLabel();
QPalette pal =title->palette();
QColor color=pal.color(QPalette::Active, QColorGroup::Foreground);
s+=color.name()+"\t";
s+=QString::number(title->alignment())+"\n";
return s;
}

QString Graph::saveScaleTitles()
{
int a;
QString s="";
for (int i=0;i<4;i++)
	{
	switch (i)
	{
	case 0:
		a=2;
	break;

	case 1:
		a=0;
	break;

	case 2:
		a=3;
	break;

	case 3:
		a=1;
	break;
	}
	s+=(myPlot->axisTitle(a)).replace("\n", "<br>")+"\t";
	}
return s+"\n";
}

QString Graph::saveAxesTitleAlignement()
{
QString s="AxesTitleAlignment\t";
QStringList axes;
int i;
for (i=0;i<4;i++)
	axes<<QString::number(Qt::AlignHCenter);

for (i=0;i<4;i++)
	{

	if (myPlot->axisEnabled(i))
		axes[i]=QString::number(myPlot->axisTitleAlignment (i));
	}
	
s+=axes.join("\t")+"\n";
return s;
}

QString Graph::savePieCurveLayout()
{
QString s="PieCurve\t";
//s+=curvesList[0]+"\t";
	
QwtPieCurve *pieCurve=(QwtPieCurve*)myPlot->curve(keys[0]);
s+=pieCurve->title()+"\t";
QPen pen=pieCurve->pen();
	
s+=QString::number(pen.width())+"\t";
s+=pen.color().name()+"\t";
	
Qt::PenStyle style=pen.style();
if (style==Qt::SolidLine)
	s+="SolidLine\t";
else if (style==Qt::DashLine)
	s+="DashLine\t";
else if (style==Qt::DotLine)
	s+="DotLine\t";	
else if (style==Qt::DashDotLine)
	s+="DashDotLine\t";		
else if (style==Qt::DashDotDotLine)
	s+="DashDotDotLine\t";

Qt::BrushStyle pattern=pieCurve->pattern();
int index;
if (pattern == Qt::SolidPattern)
	 index=0;
else if (pattern == Qt::HorPattern)
	 index=1;
else if (pattern == Qt::VerPattern)
	 index=2;
else if (pattern == Qt::CrossPattern)
	 index=3;
else if (pattern == Qt::BDiagPattern)
	 index=4;
else if (pattern == Qt::FDiagPattern)
	 index=5;
else if (pattern == Qt::DiagCrossPattern)
	 index=6;
else if (pattern == Qt::Dense1Pattern)
	 index=7;
else if (pattern == Qt::Dense2Pattern)
	 index=8;
else if (pattern == Qt::Dense3Pattern)
	 index=9;
else if (pattern == Qt::Dense4Pattern)
	 index=10;
else if (pattern == Qt::Dense5Pattern)
	 index=11;
else if (pattern == Qt::Dense6Pattern)
	 index=12;
else if (pattern == Qt::Dense7Pattern)
	 index=13;

s+=QString::number(index)+"\t";
s+=QString::number(pieRay)+"\t";
s+=QString::number(pieCurve->first())+"\n";

return s;
}

QString Graph::saveCurveLayout(int index)
{
QString s;
curveLayout cl=layout[index];

s+=QString::number(cl.plotType)+"\t";
s+=QString::number(cl.connectType)+"\t";
s+=QString::number(cl.lCol)+"\t";
s+=QString::number(cl.lStyle)+"\t";
s+=QString::number(cl.lWidth)+"\t";
s+=QString::number(cl.sSize)+"\t";
s+=QString::number(cl.sType)+"\t";
s+=QString::number(cl.symCol)+"\t";
s+=QString::number(cl.fillCol)+"\t";
s+=QString::number(cl.filledArea)+"\t";
s+=QString::number(cl.aCol)+"\t";
s+=QString::number(cl.aStyle)+"\t";

if(cl.plotType == Graph::VerticalBars || cl.plotType == Graph::HorizontalBars || cl.plotType == Graph::Histogram)
	s+=QString::number(cl.barGap)+"\t";

if(cl.plotType == Graph::Histogram)
	{
	s+=QString::number(cl.autoBinning)+"\t";
	s+=QString::number(cl.binSize)+"\t";
	s+=QString::number(cl.histBegin)+"\t";
	s+=QString::number(cl.histEnd)+"\n";
	}
else if(cl.plotType == Graph::VectXYXY)
	{
	s+=QString::number(cl.vectColor)+"\t";
	s+=QString::number(cl.vectWidth)+"\t";
	s+=QString::number(cl.headLength)+"\t";
	s+=QString::number(cl.headAngle)+"\t";
	s+=QString::number(cl.filledHead)+"\t";
		
	QStringList colsList=QStringList::split(",", associations[index], false);
	s+=colsList[2].remove("(X)")+"\t";
	s+=colsList[3].remove("(Y)")+"\n";
	}
else
	s+="\n";

return s;
}

QString Graph::saveCurves()
{
QString s;
if (piePlot)
	s+=savePieCurveLayout();
else
	{	
	for (int j=0;j<cOnPlot;j++)
		{
		if (associations[j].contains("(yErr)",TRUE)==0 
			&& associations[j].contains("(xErr)",TRUE)==0)
			{
			QwtPlotCurve *curve=myPlot->curve(keys[j]);
			QString name = curve->title();
				
			if (name.contains("_",TRUE)==0)
				{
				s+="FunctionCurve\t"+name+"\t";
				s+=QString::number(curve->dataSize())+"\t";
				s+=QString::number(curve->minXValue())+"\t";
				s+=QString::number(curve->x(1)-curve->x(0))+"\t";					
				}
			else
				s+="curve\t"+QString::number(colX(j))+"\t"+name+"\t";
			
			s+=saveCurveLayout(j);
			}
		}
	}
return s;
}

QString Graph::saveGridOptions()
{
QString s="grid\t";
s+=QString::number(grid.majorOnX)+"\t";
s+=QString::number(grid.minorOnX)+"\t";
s+=QString::number(grid.majorOnY)+"\t";
s+=QString::number(grid.minorOnY)+"\t";
s+=QString::number(grid.majorCol)+"\t";
s+=QString::number(grid.majorStyle)+"\t";
s+=QString::number(grid.majorWidth)+"\t";
s+=QString::number(grid.minorCol)+"\t";
s+=QString::number(grid.minorStyle)+"\t";
s+=QString::number(grid.minorWidth)+"\t";
s+=QString::number(grid.xZeroOn)+"\t";
s+=QString::number(grid.yZeroOn)+"\n";
	
return s;
}

bool Graph::legendOn()
{
bool on=FALSE;
if (legendMarkerID>=0)	
	on=TRUE;
return on;
}

LegendMarker* Graph::legend()
{
if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	return mrk;	
	}
else 
	return 0;
}

void Graph::newLegend(const QFont& fnt, int frameStyle)
{
LegendMarker* mrk= new LegendMarker(myPlot);
mrk->setOrigin(QPoint(10, 10));
if (piePlot)
	mrk->setText(pieLegendText());
else
	mrk->setText(legendText());

mrk->setFont(fnt);
mrk->setBackground(frameStyle);
legendMarkerID=myPlot->insertMarker(mrk);

emit modifiedGraph();
myPlot->replot();	
}

QSize Graph::newLegend(const QString& text)
{
LegendMarker* mrk= new LegendMarker(myPlot);
mrk->setOrigin(QPoint(10,10));
mrk->setText(text);
selectedMarker=myPlot->insertMarker(mrk);
myPlot->replot();
QRect rect=mrk->rect();
return rect.size();
}

void Graph::insertLegend(QStringList list)
{
bool ok;
QStringList fList=list;
QString text;
LegendMarker* mrk= new LegendMarker(myPlot);

mrk->setTextColor(QColor(fList[9]));

QPoint o=QPoint(fList[1].toInt(&ok),fList[2].toInt(&ok));
mrk->setOrigin(o);

QFont fnt=QFont (fList[3],fList[4].toInt(&ok),fList[5].toInt(&ok),fList[6].toInt(&ok));
fnt.setUnderline(fList[7].toInt(&ok));
fnt.setStrikeOut(fList[8].toInt(&ok));
mrk->setFont(fnt);

int bkg=fList[10].toInt(&ok);
mrk->setBackground(bkg);

int ang=fList[11].toInt(&ok);
mrk->setAngle(ang);

int i,n=(int)fList.count();
for (i=0;i<n-12;i++)
	text+=fList[12+i]+"\n";

mrk->setText(text);
legendMarkerID=myPlot->insertMarker(mrk);
myPlot->replot();
}

void Graph::insertTextMarker(QStringList list)
{
bool ok;
QStringList fList=list;
QString text;
LegendMarker* mrk= new LegendMarker(myPlot);

mrk->setTextColor(QColor(fList[9]));

QPoint o=QPoint(fList[1].toInt(&ok),fList[2].toInt(&ok));
mrk->setOrigin(o);

QFont fnt=QFont (fList[3],fList[4].toInt(&ok),fList[5].toInt(&ok),fList[6].toInt(&ok));
fnt.setUnderline(fList[7].toInt(&ok));
fnt.setStrikeOut(fList[8].toInt(&ok));
mrk->setFont(fnt);

int bkg=fList[10].toInt(&ok);
mrk->setBackground(bkg);

int ang=fList[11].toInt(&ok);
mrk->setAngle(ang);

int i,n=(int)fList.count();
for (i=0;i<n-12;i++)
	text+=fList[12+i]+"\n";

mrk->setText(text);
myPlot->insertMarker(mrk);
myPlot->replot();	
}

void Graph::insertLineMarker(QStringList list)
{
LineMarker* mrk= new LineMarker(myPlot);
	
int x=list[1].toInt();
int y=list[2].toInt();
mrk->setStartPoint(QPoint(x,y));
x=list[3].toInt();
y=list[4].toInt();
mrk->setEndPoint(QPoint(x,y));
	
int width=list[5].toInt();
mrk->setWidth(width);

mrk->setColor(QColor(list[6]));

Qt::PenStyle style;
if (list[7]=="SolidLine")
	style=Qt::SolidLine;
else if (list[7]=="DashLine")
	style=Qt::DashLine;
else if (list[7]=="DotLine")
	style=Qt::DotLine;	
else if (list[7]=="DashDotLine")
	style=Qt::DashDotLine;		
else if (list[7]=="DashDotDotLine")
	style=Qt::DashDotDotLine;
mrk->setStyle(style);

bool on=FALSE;
if (list[8]=="1") on=TRUE;
mrk->setEndArrow(on);

on=FALSE;
if (list[9]=="1") on=TRUE;
mrk->setStartArrow(on);

if (list.count()>10)
	{
	int headLength=list[10].toInt();
	mrk->setHeadLength(headLength);
		
	int headAngle=list[11].toInt();
	mrk->setHeadAngle(headAngle);
		
	bool fill=FALSE;
	if (list[12]=="1") 
		fill=TRUE;
	mrk->fillArrowHead(fill);
	}
	
long mrkID=myPlot->insertMarker(mrk);
linesOnPlot++;
lines.resize(linesOnPlot);
lines[linesOnPlot-1]=mrkID;
myPlot->replot();	
}

void Graph::insertLineMarker(LineMarker* mrk)
{
LineMarker* aux= new LineMarker(myPlot);
aux->setStartPoint(mrk->startPoint());
//QMessageBox::about(0,"",QString:: number(mrk->startPoint().x()));
//QMessageBox::about(0,"",QString:: number(aux->startPoint().x()));
	
aux->setEndPoint(mrk->endPoint());
aux->setWidth(mrk->width());
aux->setColor(mrk->color());
aux->setStyle(mrk->style());
aux->setEndArrow(mrk->getEndArrow());
aux->setStartArrow(mrk->getStartArrow());
aux->setHeadLength(mrk->headLength());
aux->setHeadAngle(mrk->headAngle());
aux->fillArrowHead(mrk->filledArrowHead());

long mrkID=myPlot->insertMarker(aux);
	
linesOnPlot++;
lines.resize(linesOnPlot);
lines[linesOnPlot-1]=mrkID;	
}

LineMarker* Graph::lineMarker(long id)
{
return (LineMarker*)myPlot->marker(id);	
}

ImageMarker* Graph::imageMarker(long id)
{
return (ImageMarker*)myPlot->marker(id);	
}

QwtArray<long> Graph::imageMarkerKeys()
{
return images;
}

QwtArray<long> Graph::lineMarkerKeys()
{
return lines;
}

LegendMarker* Graph::textMarker(long id)
{
QwtArray<long> txtMrkKeys=textMarkerKeys();
return (LegendMarker*)myPlot->marker(id);	
}

void Graph::insertTextMarker(LegendMarker* mrk)
{
LegendMarker* aux= new LegendMarker(myPlot);
aux->setTextColor(mrk->getTextColor());
aux->setOrigin(mrk->rect().topLeft ());
aux->setFont(mrk->getFont());
aux->setBackground(mrk->getBkgType());
aux->setAngle(mrk->getAngle());
aux->setText(mrk->getText());	
selectedMarker=myPlot->insertMarker(aux);
}

QwtArray<long> Graph::textMarkerKeys()
{
QwtArray<long> mrkKeys=myPlot->markerKeys();
QwtArray<long> txtMrkKeys(0);

int i,k=0,n=mrkKeys.size();
for (i=0;i<n;i++)
	{
	if (mrkKeys[i]!=mrkX && mrkKeys[i]!=mrkY &&
		mrkKeys[i]!=startID && mrkKeys[i]!=endID && lines.contains(mrkKeys[i])<=0
		&& images.contains(mrkKeys[i])<=0)
		{
		k++;
		txtMrkKeys.resize(k);
		txtMrkKeys[k-1]=mrkKeys[i];
		}
	}
return txtMrkKeys;
}

QString Graph::saveMarkers()
{
QString s;
QFont f;

QwtArray<long> texts=textMarkerKeys();
int i,t=texts.size(),l=lines.size(),im=images.size();
for (i=0;i<im;i++)
	{	
		ImageMarker* mrkI=(ImageMarker*) myPlot->marker(images[i]);
		s+="ImageMarker\t";
		s+=mrkI->getFileName()+"\t";

		QPoint aux=mrkI->getOrigin();
		s+=QString::number(aux.x())+"\t";
		s+=QString::number(aux.y())+"\t";
	
		QSize size=mrkI->size();
		s+=QString::number(size.width())+"\t";
		s+=QString::number(size.height())+"\n";
	}
	
	for (i=0;i<l;i++)
	{	
		LineMarker* mrkL=(LineMarker*) myPlot->marker(lines[i]);
		s+="lineMarker\t";
		QPoint aux=mrkL->startPoint();
		s+=QString::number(aux.x())+"\t";
		s+=QString::number(aux.y())+"\t";
		aux=mrkL->endPoint();
		s+=QString::number(aux.x())+"\t";
		s+=QString::number(aux.y())+"\t";
		
		s+=QString::number(mrkL->width())+"\t";
		QColor c=mrkL->color();
		s+=c.name()+"\t";

		Qt::PenStyle style=mrkL->style();
		if (style==Qt::SolidLine)
			s+="SolidLine\t";
		else if (style==Qt::DashLine)
			s+="DashLine\t";
		else if (style==Qt::DotLine)
			s+="DotLine\t";	
		else if (style==Qt::DashDotLine)
			s+="DashDotLine\t";		
		else if (style==Qt::DashDotDotLine)
			s+="DashDotDotLine\t";

		s+=QString::number(mrkL->getEndArrow())+"\t";
		s+=QString::number(mrkL->getStartArrow())+"\t";
		s+=QString::number(mrkL->headLength())+"\t";
		s+=QString::number(mrkL->headAngle())+"\t";
		s+=QString::number(mrkL->filledArrowHead())+"\n";
	}

	for (i=0;i<t;i++)
	{	
	if(texts[i]!=legendMarkerID)
		{
		LegendMarker* mrk=(LegendMarker*) myPlot->marker(texts[i]);
		s+="textMarker\t";
		QRect mRect=mrk->rect();
		s+=QString::number(mRect.x())+"\t";
		s+=QString::number(mRect.y())+"\t";

		f=mrk->getFont();
		s+=f.family()+"\t";
		s+=QString::number(f.pointSize())+"\t";
		s+=QString::number(f.weight())+"\t";
		s+=QString::number(f.italic())+"\t";
		s+=QString::number(f.underline())+"\t";
		s+=QString::number(f.strikeOut())+"\t";

		QColor c=mrk->getTextColor();
		s+=c.name()+"\t";

		int bkg=mrk->getBkgType();
		s+=QString::number(bkg)+"\t";

		int ang=mrk->getAngle();
		s+=QString::number(ang)+"\t";

		QStringList textList=QStringList::split ("\n",mrk->getText(),FALSE);
		s+=textList.join ("\t");
		s+="\n";
		}
	}

return s;
}

QString Graph::saveLegend()
{
QString s="";
if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	s+="Legend\t";
	QRect mRect=mrk->rect();
	s+=QString::number(mRect.x())+"\t";
	s+=QString::number(mRect.y())+"\t";

	QFont f=mrk->getFont();
	s+=f.family()+"\t";
	s+=QString::number(f.pointSize())+"\t";
	s+=QString::number(f.weight())+"\t";
	s+=QString::number(f.italic())+"\t";
	s+=QString::number(f.underline())+"\t";
	s+=QString::number(f.strikeOut())+"\t";

	QColor c=mrk->getTextColor();
	s+=c.name()+"\t";

	int bkg=mrk->getBkgType();
	s+=QString::number(bkg)+"\t";

	int ang=mrk->getAngle();
	s+=QString::number(ang)+"\t";

	QStringList textList=QStringList::split ("\n",mrk->getText(),FALSE);
	s+=textList.join ("\t");
	s+="\n";
	}
return s;		
}

QArray<curveLayout> Graph::allCurvesLayout()
{
return layout;
}

double Graph::curveXValue(int curve, int point)
{
QwtPlotCurve *cv = (QwtPlotCurve *)myPlot->curve(keys[curve]);
return cv->x(point);
}

double Graph::minXValue(int curve)
{
QwtPlotCurve *cv = (QwtPlotCurve *)myPlot->curve(keys[curve]);
return cv->minXValue();
}

double Graph::maxXValue(int curve)
{
QwtPlotCurve *cv = (QwtPlotCurve *)myPlot->curve(keys[curve]);
return cv->maxXValue();
}

double Graph::selectedXStartValue()
{
QwtPlotCurve *cv = (QwtPlotCurve *)myPlot->curve(selectedCurve);
return cv->x(startPoint);
}

double Graph::selectedXEndValue()
{
QwtPlotCurve *cv = (QwtPlotCurve *)myPlot->curve(selectedCurve);
return cv->x(endPoint);
}

int Graph::curveIndex(long key)
{
return keys.find (key, 0 );
}

long Graph::curveKey(int curve)
{
return keys[curve];
}

int Graph::curveDataSize(int curve)
{
QwtPlotCurve *cv = (QwtPlotCurve *)myPlot->curve(keys[curve]);
return cv->dataSize();
}

int Graph::selectedPoints(long curveKey)
{
int points;
QwtPlotCurve *cv = (QwtPlotCurve *)myPlot->curve(curveKey);

if (endPoint>0)
	points=abs(endPoint-startPoint)+1;
else
	points=cv->dataSize();
return points;
}

int Graph::curves()
{
return cOnPlot;
}

curveLayout Graph::initCurveLayout()
{
	curveLayout cl;
	cl.plotType=Graph::LineSymbols;
	cl.connectType=1;
	cl.lStyle=0;
	cl.lWidth=1;
	cl.sSize=7;
	cl.sType=0;
	cl.filledArea=0;
	cl.aCol=0;
	cl.aStyle=0;
	cl.lCol=0;
	cl.symCol=0;
	cl.fillCol=0;	
	cl.vectColor=0;
	cl.vectWidth=1;
	cl.headLength=4;
	cl.headAngle=45;
	cl.filledHead=1;
	return cl;
}

void Graph::updateCurveLayout(int curve,const curveLayout& cL)
{
long curveID = keys[curve];
layout[curve]=cL;

QColor lColor=MyDialog::color(cL.lCol);
QColor sColor=MyDialog::color(cL.symCol);
QColor fillColor=MyDialog::color(cL.fillCol);
QColor aColor=MyDialog::color(cL.aCol);

QwtSymbol::Style  style;
Qt::PenStyle linePen;
QBrush brush=QBrush(aColor, QBrush::NoBrush);

if (cL.filledArea)
{
Qt::BrushStyle brushStyle;
switch (cL.aStyle)
	{
	case 0:
		brushStyle=Qt::SolidPattern;
	break;
	case 1:
		brushStyle=Qt::HorPattern;
	break;
	case 2:
		brushStyle=Qt::VerPattern;
	break;
	case 3:
		brushStyle=Qt::CrossPattern;
	break;
	case 4:
		brushStyle=Qt::BDiagPattern;
	break;
	case 5:
		brushStyle=Qt::FDiagPattern;
	break;
	case 6:
		brushStyle=Qt::DiagCrossPattern;
	break;

	case 7:
		brushStyle=Qt::Dense1Pattern;
	break;
	case 8:
		brushStyle=Qt::Dense2Pattern;
	break;
	case 9:
		brushStyle=Qt::Dense3Pattern;
	break;
	case 10:
		brushStyle=Qt::Dense4Pattern;
	break;
	case 11:
		brushStyle=Qt::Dense5Pattern;
	break;
	case 12:
		brushStyle=Qt::Dense6Pattern;
	break;
	case 13:
		brushStyle=Qt::Dense7Pattern;
	break;
	}

brush.setStyle(brushStyle); 
}

switch (cL.lStyle)
	{
	case 0:
		linePen=Qt::SolidLine;
	break;
	case 1:
		linePen=Qt::DashLine;
	break;
	case 2:
		linePen=Qt::DotLine;
	break;
	case 3:
		linePen=Qt::DashDotLine;
	break;
	case 4:
		linePen=Qt::DashDotDotLine;
	break;
	}

switch (cL.sType)
	{
	case 0:
		style=QwtSymbol::Ellipse;
	break;
	case 1:
		style=QwtSymbol::Rect;
	break;
	case 2:
		style=QwtSymbol::Diamond;
	break;
	case 3:
		style=QwtSymbol::Triangle;
	break;
	case 4:
		style=QwtSymbol::DTriangle;
	break;
	case 5:
		style=QwtSymbol::LTriangle;
	break;
	case 6:
		style=QwtSymbol::RTriangle;
	break;
	case 7:
		style=QwtSymbol::Cross;
	break;
	case 8:
		style=QwtSymbol::XCross;
	break;
	}

QwtBarCurve *bars=0;
QwtHistogram *histogram=0;
VectXYXYCurve *vect=0;
switch (cL.plotType)
	{
	case Line:
	myPlot->setCurveSymbol(curveID, QwtSymbol(QwtSymbol::None,
        QBrush(fillColor), QPen(sColor), QSize(cL.sSize,cL.sSize)));
	myPlot->setCurvePen(curveID,QPen(lColor,cL.lWidth,linePen));
	myPlot->setCurveStyle(curveID,cL.connectType,0); 
	myPlot->setCurveBrush (curveID,brush); 
	break;

	case Scatter:
	myPlot->setCurveSymbol(curveID, QwtSymbol(style,
    QBrush(fillColor), QPen(sColor,cL.lWidth,Qt::SolidLine), QSize(cL.sSize,cL.sSize)));
	myPlot->setCurveStyle(curveID,0,0); 
	break;

	case LineSymbols:
	myPlot->setCurveSymbol(curveID, QwtSymbol(style,
        QBrush(fillColor), QPen(sColor,cL.lWidth,Qt::SolidLine), QSize(cL.sSize,cL.sSize)));
	myPlot->setCurvePen(curveID,QPen(lColor,cL.lWidth,linePen));
	myPlot->setCurveStyle(curveID,cL.connectType,0);
	myPlot->setCurveBrush (curveID,brush);
	break;
	
	case VerticalBars:
	bars=(QwtBarCurve *)myPlot->curve(curveID);
	if (bars)
		{
		bars->setPen(QPen(lColor,cL.lWidth,linePen));
		bars->setBrush (brush);
		bars->setGap(cL.barGap);
		}
	break;

	case HorizontalBars:
	bars=(QwtBarCurve *)myPlot->curve(curveID);
	if (bars)
		{
		bars->setPen(QPen(lColor,cL.lWidth,linePen));
		bars->setBrush (brush);
		bars->setGap(cL.barGap);
		}
	break;
	
	case Area:
	myPlot->setCurveSymbol(curveID, QwtSymbol(QwtSymbol::None,
        QBrush(fillColor), QPen(sColor), QSize(cL.sSize,cL.sSize)));
	myPlot->setCurvePen(curveID,QPen(lColor,cL.lWidth,linePen));
	myPlot->setCurveStyle(curveID,cL.connectType,0);
	myPlot->setCurveBrush (curveID,brush);
	break;
	
	case VerticalDropLines:
	myPlot->setCurveSymbol(curveID, QwtSymbol(style,
        QBrush(fillColor), QPen(sColor,cL.lWidth,Qt::SolidLine), QSize(cL.sSize,cL.sSize)));
	myPlot->setCurvePen(curveID,QPen(lColor,cL.lWidth,linePen));
	myPlot->setCurveStyle(curveID,cL.connectType,0);
	myPlot->setCurveBrush (curveID,brush);
	break;
	
	case Spline:
	myPlot->setCurveSymbol(curveID, QwtSymbol(style,
        QBrush(fillColor), QPen(sColor,cL.lWidth,Qt::SolidLine), QSize(cL.sSize,cL.sSize)));
	myPlot->setCurvePen(curveID,QPen(lColor,cL.lWidth,linePen));
	myPlot->setCurveStyle(curveID,cL.connectType,0);
	myPlot->setCurveBrush (curveID,brush);
	break;
	
	case Steps:
	myPlot->setCurveSymbol(curveID, QwtSymbol(QwtSymbol::None,
        QBrush(fillColor), QPen(sColor), QSize(cL.sSize,cL.sSize)));
	myPlot->setCurvePen(curveID,QPen(lColor,cL.lWidth,linePen));
	myPlot->setCurveStyle(curveID,cL.connectType,0); 
	myPlot->setCurveBrush (curveID,brush); 
	break;

	case Histogram:
	histogram=(QwtHistogram *)myPlot->curve(curveID);
	if (histogram)
		{
		histogram->setPen(QPen(lColor,cL.lWidth,linePen));
		histogram->setBrush (brush);
		histogram->setGap(cL.barGap);
		}
	break;
		
	case VectXYXY:
	vect=(VectXYXYCurve *)myPlot->curve(curveID);
	if (vect)
		{
		myPlot->setCurveStyle(curveID,cL.connectType,0); 
		myPlot->setCurvePen(curveID,QPen(lColor,cL.lWidth,linePen));
		myPlot->setCurveBrush (curveID,brush); 
			
		vect->setColor(MyDialog::color(cL.vectColor));
		vect->setWidth(cL.vectWidth);
		vect->setHeadLength(cL.headLength);
		vect->setHeadAngle(cL.headAngle);
		vect->fillArrowHead(cL.filledHead);
		}
	break;
	}

QString yColName=myPlot->curveTitle(curveID);
for (int i=0;i<cOnPlot;i++)
{
if (associations[i].contains(yColName,TRUE)>0 && 
	(associations[i].contains("(yErr)",TRUE)>0 ||  associations[i].contains("(xErr)",TRUE)>0) 
	&& i!=curve )
	{
	curveID=keys[i];
	QwtErrorPlotCurve *errors=(QwtErrorPlotCurve *)myPlot->curve(curveID);
	if (errors)
		errors->setSymbolSize(QSize(cL.sSize,cL.sSize));		
	}
}

myPlot->replot(); 
emit modifiedGraph(); 
}

void Graph::updateErrorBars(int curve,bool xErr,int width,int cap,const QColor& c,
							bool plus,bool minus,bool through)
{
long curveID = keys[curve];

QwtErrorPlotCurve *errors=(QwtErrorPlotCurve *)myPlot->curve(curveID);
if (errors)
	{
	errors->setWidth(width);
	errors->setCapLength(cap);
	errors->setColor(c);

	errors->setXErrors(xErr);
	errors->drawThroughSymbol(through);
	errors->drawPlusSide(plus);
	errors->drawMinusSide(minus);
	myPlot->replot();

	for (int i=0;i<(int)errorBars.count();i++)
		{
		if (errorBars[i]==associations[curve])
			{
			QString errors="";
			errors+=QString::number(xErr)+"\t";
			errors+=QString::number(width)+"\t";
			errors+=QString::number(cap)+"\t";
			errors+=c.name()+"\t";
			errors+=QString::number(through)+"\t";
			errors+=QString::number(plus)+"\t";
			errors+=QString::number(minus);

			errorBarsLayout[i]=errors;			
			}
		}

	if (xErr) 
		associations[curve]=associations[curve].replace("(yErr)","(xErr)",TRUE); 
	else
		associations[curve]=associations[curve].replace("(xErr)","(yErr)",TRUE); 
	}
}

void Graph::addErrorBars(WksheetWindow*w,bool xErrors,const QString& xColName,
						 const QString& yColName,WksheetWindow*errTable, const QString& errColName,
						 int width, int cap, const QColor& color,
						 bool through, bool minus,bool plus)
{
//curvesList<<errColName;
int xcol=w->colIndex(xColName);
int ycol=w->colIndex(yColName);
int errcol=errTable->colIndex(errColName);
int r=w->tableRows();

QArray<double> X, Y, err;
int i, it=0;
int xColType = w->checkColType(xcol);
int yColType = w->checkColType(ycol);
QStringList xLabels, yLabels;// store text labels
for (i = 0; i<r; i++ )
	{
	QString xval=w->text(i,xcol);
	QString yval=w->text(i,ycol);
	QString errval=errTable->text(i,errcol);
	if (!xval.isEmpty() && !yval.isEmpty() && !errval.isEmpty())
		{
		it++;
		X.resize(it);
		Y.resize(it);
		err.resize(it);
		if (xColType == WksheetWindow::Text)
			{
			xLabels<<xval;
			X[it-1]=(double)it;
			}
		else
			X[it-1]=xval.toDouble();

		if (yColType == WksheetWindow::Text)
			{
			yLabels<<yval;
			Y[it-1]=(double)it;
			}
		else
			Y[it-1]=yval.toDouble();

		err[it-1]=errval.toDouble();
		}
	}	

QSize size;
for (i=0;i<cOnPlot;i++)
	{
	if (associations[i].contains(yColName,TRUE)>0 && errorBars.contains(yColName)<=0)
		{
		long curveID = keys[i];
		QwtSymbol symb=myPlot->curveSymbol(curveID);
		size=symb.size();
		}			
	}

QwtErrorPlotCurve *errors=new QwtErrorPlotCurve(myPlot,0);
errors->setData(X,Y,it);
errors->setErrors(err);
errors->setXErrors(xErrors);
errors->setCapLength(cap);
errors->setSymbolSize(size);
errors->setColor(color);
errors->setWidth(width);
errors->drawPlusSide(plus);
errors->drawMinusSide(minus);
errors->drawThroughSymbol(through);

long errorsID=myPlot->insertCurve(errors);
myPlot->setCurveTitle(errorsID, errColName);	
	
cOnPlot++;
keys.resize(cOnPlot);
keys[cOnPlot-1]=errorsID;

QString errLabel=xColName+"(X),"+yColName+"(Y),"+errColName;
if (xErrors)
	errLabel+="(xErr)";
else
	errLabel+="(yErr)";

associations<<errLabel;
errorBars<<errLabel;

QString curve="";
curve+=QString::number(xErrors)+"\t";
curve+=QString::number(width)+"\t";
curve+=QString::number(cap)+"\t";
curve+=color.name()+"\t";
curve+=QString::number(through)+"\t";
curve+=QString::number(plus)+"\t";
curve+=QString::number(minus);

errorBarsLayout<<curve;
updateScale();
}

void Graph::addErrorBars(WksheetWindow* w,const QString& name,int type, const QString& errCol)
{
QString errColLabel=w->caption()+"_"+errCol;
//curvesList<<errColLabel;

int xcol=w->colX();
int r=w->tableRows();
QStringList labels,cols=w->colNames();

int pos=name.find("_",TRUE);
QString label=name.right(name.length()-pos-1);

int ycol,errcol,i,col=w->tableCols();
for (i = 0;i<col;i++ )
	{	
	if (cols[i]==label) ycol=i;
	if (cols[i]==errCol) errcol=i;
	}

QSize size;
for (i=0;i<cOnPlot;i++)
		{
		QString name=w->colName(ycol);
		if (associations[i].contains(name,TRUE)>0 && associations[i].contains("(yErr)",TRUE)<=0)
			{
			long curveID = keys[i];
			QwtSymbol symb=myPlot->curveSymbol(curveID);
			size=symb.size();
			}			
		}

QArray<double> X(1),Y(1), err(1);
int it=0;
	
int xColType = w->checkColType(xcol);
int yColType = w->checkColType(ycol);
QStringList xLabels, yLabels;// store text labels
for (i = 0; i<r; i++ )
	{
	QString xval=w->text(i,xcol);
	QString yval=w->text(i,ycol);
	QString errval=w->text(i,errcol);
	if (!xval.isEmpty() && !yval.isEmpty() && !errval.isEmpty())
		{
		it++;
		X.resize(it);
		Y.resize(it);
		err.resize(it);
		if (xColType == WksheetWindow::Text)
			{
			xLabels<<xval;
			X[it-1]=(double)it;
			}
		else
			X[it-1]=xval.toDouble();

		if (yColType == WksheetWindow::Text)
			{
			yLabels<<yval;
			Y[it-1]=(double)it;
			}
		else
			Y[it-1]=yval.toDouble();

		err[it-1]=errval.toDouble();
		}
	}


QwtErrorPlotCurve *errors=new QwtErrorPlotCurve(myPlot,0);
errors->setData(X,Y,it);
errors->setErrors(err);
errors->setCapLength(5);
errors->setSymbolSize(size);

QString errType="(yErr)";
bool xErr=FALSE;
if (type==0) 
	{
	xErr=TRUE;
	errType="(xErr)";
	}
errors->setXErrors(xErr);

long errorsID=myPlot->insertCurve(errors);
myPlot->setCurveTitle(errorsID, errColLabel);
	
cOnPlot++;
keys.resize(cOnPlot);
keys[cOnPlot-1]=errorsID;

QString errLabel=w->colName(xcol)+"(X),"+name+"(Y),"+errColLabel+errType;
associations<<errLabel;
errorBars<<errLabel;

for (i=0;i<QwtPlot::axisCnt;i++)
myPlot->setAxisAutoScale(i);
myPlot->replot();

updateScale();

QString curve="";
curve+=QString::number(xErr)+"\t";
curve+=QString::number(2)+"\t";
curve+=QString::number(5)+"\t";

QColor color=QColor(black);
curve+=color.name()+"\t";
curve+=QString::number(0)+"\t";
curve+=QString::number(1)+"\t";
curve+=QString::number(1);

errorBarsLayout<<curve;
}

void Graph::addErrorBars(WksheetWindow* w,const QString& curveName, 
												WksheetWindow* errorTable, const QString& errColName, int type)
{
//curvesList<<errColName;

int xcol=w->colX();
int r=w->tableRows();
QStringList labels,cols=w->colNames();

int ycol=w->colIndex(curveName);
int errcol=errorTable->colIndex(errColName);

QSize size;
int i;
for (i=0;i<cOnPlot;i++)
		{
		QString name=w->colName(ycol);
		if (associations[i].contains(name,TRUE)>0 && associations[i].contains("(yErr)",TRUE)<=0)
			{
			long curveID = keys[i];
			QwtSymbol symb=myPlot->curveSymbol(curveID);
			size=symb.size();
			}			
		}

QArray<double> X(1),Y(1), err(1);
int it=0;
	
int xColType = w->checkColType(xcol);
int yColType = w->checkColType(ycol);
QStringList xLabels, yLabels;// store text labels
for (i = 0; i<r; i++ )
	{
	QString xval=w->text(i,xcol);
	QString yval=w->text(i,ycol);
	QString errval=errorTable->text(i,errcol);
	if (!xval.isEmpty() && !yval.isEmpty() && !errval.isEmpty())
		{
		it++;
		X.resize(it);
		Y.resize(it);
		err.resize(it);
		if (xColType == WksheetWindow::Text)
			{
			xLabels<<xval;
			X[it-1]=(double)it;
			}
		else
			X[it-1]=xval.toDouble();

		if (yColType == WksheetWindow::Text)
			{
			yLabels<<yval;
			Y[it-1]=(double)it;
			}
		else
			Y[it-1]=yval.toDouble();

		err[it-1]=errval.toDouble();
		}
	}	

QwtErrorPlotCurve *errors=new QwtErrorPlotCurve(myPlot,0);
errors->setData(X,Y,it);
errors->setErrors(err);
errors->setCapLength(5);
errors->setSymbolSize(size);

QString errType="(yErr)";
bool xErr=FALSE;
if (type==0) 
	{
	xErr=TRUE;
	errType="(xErr)";
	}
errors->setXErrors(xErr);

long errorsID=myPlot->insertCurve(errors);
myPlot->setCurveTitle(errorsID, errColName);
		
cOnPlot++;
keys.resize(cOnPlot);
keys[cOnPlot-1]=errorsID;

QString errLabel=w->colName(xcol)+"(X),"+curveName+"(Y),"+errColName+errType;
associations<<errLabel;
errorBars<<errLabel;

for (i=0;i<QwtPlot::axisCnt;i++)
myPlot->setAxisAutoScale(i);
myPlot->replot();

updateScale();

QString curve="";
curve+=QString::number(xErr)+"\t";
curve+=QString::number(2)+"\t";
curve+=QString::number(5)+"\t";

QColor color=QColor(black);
curve+=color.name()+"\t";
curve+=QString::number(0)+"\t";
curve+=QString::number(1)+"\t";
curve+=QString::number(1);

errorBarsLayout<<curve;
}

QStringList Graph::erorsLayout()
{
return errorBarsLayout;
}

int Graph::pieSize()
{
return pieRay;
}

Qt::BrushStyle Graph::pieBrushStyle()
{
Qt::BrushStyle style=Qt::SolidPattern;
if (piePlot)
	{
	QwtPieCurve *pieCurve = (QwtPieCurve *)myPlot->curve(keys[0]);
	if (pieCurve->pattern() != style)
		style=pieCurve->pattern();
	}	
return 	style;
}

int Graph::pieFirstColor()
{
int first=0;
if (piePlot)
	{
	QwtPieCurve *pieCurve = (QwtPieCurve *)myPlot->curve(keys[0]);
	if (pieCurve->first() != first)
		first=pieCurve->first();
	}	
return 	first;
}

QPen Graph::pieCurvePen()
{
QPen pen=QPen(QColor(black),1,Qt::SolidLine);
if (piePlot)
	{
	QwtPieCurve *pieCurve = (QwtPieCurve *)myPlot->curve(keys[0]);
	if (pieCurve->pen() != pen)
		pen=pieCurve->pen();
	}	
return 	pen;
}

void Graph::updatePie(const QPen& pen, const Qt::BrushStyle &brushStyle, int size, int firstColor)
{
if (cOnPlot>0)
	{
	QwtPieCurve *pieCurve=(QwtPieCurve*)myPlot->curve(keys[0]);
	if (pieCurve)
		{
		pieCurve->setPen(pen);
		pieCurve->setRay(size);
		pieCurve->setBrushStyle(brushStyle);
		pieCurve->setFirstColor(firstColor);
		myPlot->replot();
	
		pieRay=size;
		emit modifiedGraph();
		}
	}
}

QwtPieCurve* Graph::pieCurve()
{
return (QwtPieCurve*)myPlot->curve(keys[0]);
}

void Graph::plotPie(QwtPieCurve* curve)
{
int data=curve->dataSize();
QArray<double> X(data),Y(data);
for (int i=0;i<data;i++)
	{
	X[i]=curve->x(i);
	Y[i]=curve->y(i);
	}

QwtPieCurve *pieCurve=new QwtPieCurve(myPlot,0);
pieCurve->setData(X,Y,data);
QString name=curve->title();

long pieID=myPlot->insertCurve(pieCurve);
pieCurve->setPen(curve->pen());
pieCurve->setTitle(name);
pieRay=curve->ray();
pieCurve->setRay(pieRay);
pieCurve->setBrushStyle(curve->pattern());
pieCurve->setFirstColor(curve->first());
piePlot=TRUE;
	
cOnPlot++;

keys.resize(cOnPlot);
keys[cOnPlot-1]=pieID;
//curvesList<<name;
associations<<name;
}


void Graph::plotPie(WksheetWindow* w,const QString& name,const QPen& pen, int brush, int size, int firstColor)
{
//curvesList<<name;
associations<<name;
int r=w->tableRows();
QStringList labels,cols=w->colNames();

int pos=name.find("_",TRUE);
QString label=name.right(name.length()-pos-1);

int ycol,i,col=w->tableCols();
for (i = 0;i<col;i++ )
	{	
	if (cols[i]==label) ycol=i;
	}

int xcol=w->colX();
QArray<double> X(1),Y(1);
int it=0;
	
for (i = 0;i<r;i++ )
	{
	QString xval=w->text(i,xcol);
	QString yval=w->text(i,ycol);
	if (!xval.isEmpty() && !yval.isEmpty())
		{
		it++;
		X.resize(it);
		Y.resize(it);
		Y[it-1]=yval.toDouble();
		X[it-1]=xval.toDouble();
		}
	}

QwtPieCurve *pieCurve=new QwtPieCurve(myPlot,0);
pieCurve->setData(X,Y,it);
long pieID=myPlot->insertCurve(pieCurve);
pieCurve->setTitle(name);
pieCurve->setPen(pen);
pieCurve->setRay(size);
pieCurve->setFirstColor(firstColor);
	
Qt::BrushStyle brushStyle;
switch (brush)
	{
	case 0:
		brushStyle=Qt::SolidPattern;
	break;
	case 1:
		brushStyle=Qt::HorPattern;
	break;
	case 2:
		brushStyle=Qt::VerPattern;
	break;
	case 3:
		brushStyle=Qt::CrossPattern;
	break;
	case 4:

		brushStyle=Qt::BDiagPattern;
	break;
	case 5:
		brushStyle=Qt::FDiagPattern;
	break;
	case 6:
		brushStyle=Qt::DiagCrossPattern;
	break;

	case 7:
		brushStyle=Qt::Dense1Pattern;
	break;
	case 8:
		brushStyle=Qt::Dense2Pattern;
	break;
	case 9:
		brushStyle=Qt::Dense3Pattern;
	break;
	case 10:
		brushStyle=Qt::Dense4Pattern;
	break;
	case 11:
		brushStyle=Qt::Dense5Pattern;
	break;
	case 12:
		brushStyle=Qt::Dense6Pattern;
	break;
	case 13:
		brushStyle=Qt::Dense7Pattern;
	break;
	}
	
pieCurve->setBrushStyle(brushStyle);
piePlot=TRUE;
pieRay=size;
	
cOnPlot++;
keys.resize(cOnPlot);
keys[cOnPlot-1]=pieID;
}

void Graph::plotPie(WksheetWindow* w,const QString& name)
{
//curvesList<<name;
associations<<name;
int r=w->tableRows();
QStringList labels,cols=w->colNames();

int pos=name.find("_",TRUE);
QString label=name.right(name.length()-pos-1);

int ycol,i,col=w->tableCols();
for (i = 0;i<col;i++ )
	{	
	if (cols[i] ==label) 
		ycol=i;
	}

int xcol=w->colX();
QArray<double> X(1),Y(1);
int it=0;
double sum=0.0;
	
for (i = 0;i<r;i++ )
	{
	QString xval=w->text(i,xcol);
	QString yval=w->text(i,ycol);
	if (!xval.isEmpty() && !yval.isEmpty())
		{
		it++;
		X.resize(it);
		Y.resize(it);
		Y[it-1]=yval.toDouble();
		X[it-1]=xval.toDouble();
		sum+=Y[it-1];
		}
	}

QwtPieCurve *pieCurve=new QwtPieCurve(myPlot,0);
pieCurve->setData(X,Y,it);
long pieID=myPlot->insertCurve(pieCurve);
pieCurve->setTitle(name);
	
QwtPlotLayout *pLayout=myPlot->plotLayout();
const QRect rect=pLayout->canvasRect();
	
const int ray = 125;
int xc=int(rect.width()*0.5+10);
int yc=int(rect.y()+rect.height()*0.5);
	
double PI=4*atan(1);
double angle = 90;	
	
for (i = 0;i<it;i++ )
	{
	const double value = Y[i]/sum*360;
	double alabel= (angle-value*0.5)*PI/180.0;	
		
	const int x=int(xc+ray*cos(alabel));
	const int y=int(yc-ray*sin(alabel));
		
	LegendMarker* aux= new LegendMarker(myPlot);
	aux->setOrigin(QPoint(x,y));
	aux->setBackground(0);
	aux->setText(QString::number(Y[i]/sum*100,'g',2)+"%");	
	myPlot->insertMarker(aux);
		
	angle -= value;
	}

piePlot=TRUE;
	
cOnPlot++;
keys.resize(cOnPlot);
keys[cOnPlot-1]=pieID;
	
if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		{
		QString text="";
		for (i=0;i<it;i++)
			{
			text+="\\p{";
			text+=QString::number(i+1);
			text+="} ";
			text+=QString::number(i+1);
			text+="\n";
			}
		mrk->setText(text);
		}		
	}
	
for (i=0;i<QwtPlot::axisCnt;i++)
	myPlot->enableAxis(i,FALSE);

myPlot->setTitle("");

QwtPlotCanvas* canvas=(QwtPlotCanvas*) myPlot->canvas();  
canvas->setLineWidth(1);
QPalette pal =canvas->palette();
pal.setColor(QColorGroup::Foreground,QColor(black));
canvas->setPalette(pal);

scalePicker->refresh();
myPlot->replot();
updateScale();
}

bool Graph::isPiePlot()
{
return piePlot;	
}

void Graph::plot(WksheetWindow* w,const QString& name, int style)
{
//provided for convenience
int xcol=w->colX();
plot(w, xcol, name, style);
}

void Graph::plot(WksheetWindow* w, int xcol, const QString& name, int style)
{
//curvesList<<name;
associations<<w->colName(xcol)+"(X),"+name+"(Y)";

int r=w->tableRows();
QStringList labels,cols=w->colNames();
int ycol=w->colIndex(name);
	
QArray<double> X(1),Y(1);
int i, it=0;
	
int xColType = w->checkColType(xcol);
int yColType = w->checkColType(ycol);
QStringList xLabels, yLabels;// store text labels

for (i = 0; i<r; i++ )
	{
	QString xval=w->text(i,xcol);
	QString yval=w->text(i,ycol);
	if (!xval.isEmpty() && !yval.isEmpty())
		{
		it++;
		X.resize(it);
		Y.resize(it);
		if (xColType == WksheetWindow::Text)
			{
			if (xLabels.contains(xval) == 0)
				xLabels<<xval;
			X[it-1]=(double)(xLabels.findIndex(xval)+1);
			}
		else
			X[it-1]=xval.toDouble();

		if (yColType == WksheetWindow::Text)
			{
			yLabels<<yval;
			Y[it-1]=(double)it;
			}
		else
			Y[it-1]=yval.toDouble();
		}
	}

if (xColType == WksheetWindow::Text)
	{
	if (style == HorizontalBars)
		{
		axesLabelsColumns[QwtPlot::yLeft] = w->colName(xcol);
		axesLabelsColumns[QwtPlot::yRight] = w->colName(xcol);
		axisType[QwtPlot::yLeft] = Txt;
		myPlot->setAxisScaleDraw (QwtPlot::yLeft, new QwtTextScaleDraw(xLabels));
		}
	else
		{
		axesLabelsColumns[QwtPlot::xBottom] = w->colName(xcol);
		axesLabelsColumns[QwtPlot::xTop] = w->colName(xcol);
		axisType[QwtPlot::xBottom] = Txt;
		myPlot->setAxisScaleDraw (QwtPlot::xBottom, new QwtTextScaleDraw(xLabels));
		}
	}
	
if (yColType == WksheetWindow::Text)
	{
	axesLabelsColumns[QwtPlot::yLeft] = name;
	axesLabelsColumns[QwtPlot::yRight] = name;
	axisType[QwtPlot::yLeft] = Txt;
	myPlot->setAxisScaleDraw (QwtPlot::yLeft, new QwtTextScaleDraw(yLabels));
	}

cOnPlot++;
curveLayout cl;
cl.plotType=style;
layout.resize(cOnPlot);
layout[cOnPlot-1]=cl;

long curveID=-1;
if (style == VerticalBars)
	{
	QwtBarCurve *bars=new QwtBarCurve(QwtBarCurve::Vertical, myPlot,0);
	if (!bars)
		return;

	curveID = myPlot->insertCurve(bars);
	myPlot->setCurveStyle(curveID, QwtCurve::UserCurve);
	myPlot->setCurveTitle(curveID,name);
	}
else if (style == HorizontalBars)
	{
	QwtBarCurve *bars=new QwtBarCurve(QwtBarCurve::Horizontal, myPlot,0);
	if (!bars)
		return;

	curveID = myPlot->insertCurve(bars);
	myPlot->setCurveStyle(curveID, QwtCurve::UserCurve);
	myPlot->setCurveTitle(curveID,name);
	}
else if (style == Histogram)
	{
	QwtHistogram *histogram=new QwtHistogram(myPlot,0);
	if (!histogram)
		return;

	curveID = myPlot->insertCurve(histogram);
	myPlot->setCurveStyle(curveID, QwtCurve::UserCurve);
	myPlot->setCurveTitle(curveID,name);
	}
else
	curveID = myPlot->insertCurve(name);

myPlot->setCurvePen(curveID, QPen(black,widthLine)); 

if (style == Histogram)
	initHistogram(curveID, Y, it);
else if (style == HorizontalBars)
	myPlot->setCurveData(curveID, Y, X, it);
else
	myPlot->setCurveData(curveID, X, Y, it);

keys.resize(cOnPlot);
keys[cOnPlot-1]=curveID;

xCols.resize(cOnPlot);
xCols[cOnPlot-1]=xcol;

for (i=0;i<4;i++)
	myPlot->setAxisAutoScale(i);
myPlot->replot();

if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		{
		QString text=mrk->getText()+"\\c{";
		text+=QString::number(cOnPlot)+"}";
		text+=name+"\n";
		mrk->setText(text);
		}
	myPlot->replot();
	}

updateScale();
d_zoomer->setZoomBase();
}

void Graph::plotVectXYXY(WksheetWindow* w, const QStringList& colList)
{
int r=w->tableRows();

int xcol, ycol, endXCol, endYCol;
if (colList.count()==2)
	{
	xcol=w->colX();
	ycol=w->colY();
	endXCol=w->colIndex(colList[0]);
	endYCol=w->colIndex(colList[1]);
	}
else if (colList.count()==4)
	{
	xcol=w->colIndex(colList[0]);
	ycol=w->colIndex(colList[1]);
	endXCol=w->colIndex(colList[2]);
	endYCol=w->colIndex(colList[3]);
	}
	
QArray<double> X(1), Y(1), X2(1), Y2(1);
int i,it=0;
for (i = 0;i<r;i++ )
	{
	QString xval=w->text(i,xcol);
	QString yval=w->text(i,ycol);
	QString xend=w->text(i,endXCol);
	QString yend=w->text(i,endYCol);
	if (!xval.isEmpty() && !yval.isEmpty() && !xend.isEmpty() && !yend.isEmpty())
		{
		it++;
		X.resize(it); Y.resize(it); X2.resize(it); Y2.resize(it);
		Y[it-1]=yval.toDouble();
		X[it-1]=xval.toDouble();
		Y2[it-1]=yend.toDouble();
		X2[it-1]=xend.toDouble();
		}
	}
	
QString label;
if (colList.count()==2)
	{
	label=w->colName(ycol);
	associations<<w->colName(xcol)+"(X),"+w->colName(ycol)+"(Y)"+
									colList[0]+"(X),"+colList[1]+"(Y)";
	}
else if (colList.count()==4)
	{
	label=colList[1];
	associations<<colList[0]+"(X),"+colList[1]+"(Y),"+
							colList[2]+"(X),"+colList[3]+"(Y)";
	}

cOnPlot++;
curveLayout cl=initCurveLayout();
cl.plotType=Graph::VectXYXY;
cl.connectType=0;
cl.lStyle=0;

layout.resize(cOnPlot);
layout[cOnPlot-1]=cl;

VectXYXYCurve *v=new VectXYXYCurve(myPlot, 0);
if (!v)
	return;

v->setVectorEnd(X2, Y2);
long curveID = myPlot->insertCurve(v);
myPlot->setCurveStyle(curveID, 0,0); 
myPlot->setCurveTitle(curveID, label);
myPlot->setCurveData(curveID, X, Y, it);

keys.resize(cOnPlot);
keys[cOnPlot-1]=curveID;
xCols.resize(cOnPlot);
xCols[cOnPlot-1]=xcol;

for (i=0;i<4;i++)
	myPlot->setAxisAutoScale(i);
myPlot->replot();

if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		{
		QString text=mrk->getText()+"\\c{";
		text+=QString::number(cOnPlot)+"}";
		text+=label+"\n";
		mrk->setText(text);
		}
	myPlot->replot();
	}
updateScale();
d_zoomer->setZoomBase();
}

void Graph::updateVectorsLayout(WksheetWindow *w, int curve, int colorIndex, int width, 
						int arrowLength, int arrowAngle, bool filled, 
						const QString& xEndColName, const QString& yEndColName)
{
long curveID = keys[curve];
VectXYXYCurve *vect=(VectXYXYCurve *)myPlot->curve(curveID);
if (!vect)
	return;

QColor color=MyDialog::color(colorIndex);
QStringList cols=QStringList::split(",", associations[curve], false);
if (vect->width() == width && vect->color() == color &&
	vect->headLength() == arrowLength && vect->headAngle() == arrowAngle &&
	vect->filledArrowHead() == filled && cols[2] == xEndColName && cols[3] == yEndColName)
return;

curveLayout cl=layout[curve];
vect->setColor(color);
cl.vectColor=colorIndex;
vect->setWidth(width);
cl.vectWidth=width;
vect->setHeadLength(arrowLength);
cl.headLength=arrowLength;
vect->setHeadAngle(arrowAngle);
cl.headAngle=arrowAngle;
vect->fillArrowHead(filled);
if (filled)
	cl.filledHead=1;
else
	cl.filledHead=0;
layout[curve]=cl;

if (cols[2] != xEndColName || cols[3] != yEndColName)
	{
	int r=w->tableRows();
	QString aux = cols[0];
	int xcol=w->colIndex(aux.remove("(X)"));
	aux = cols[1];
	int ycol=w->colIndex(aux.remove("(Y)"));
	int endXCol=w->colIndex(xEndColName);
	int endYCol=w->colIndex(yEndColName);
	int size=vect->dataSize();
	QArray<double> X(size), Y(size);
	int i,it=0;
	for (i = 0;i<r;i++ )
		{
		QString xval=w->text(i,xcol);
		QString yval=w->text(i,ycol);
		QString xend=w->text(i,endXCol);
		QString yend=w->text(i,endYCol);
		if (!xval.isEmpty() && !yval.isEmpty() && !xend.isEmpty() && !yend.isEmpty())
			{
			it++;
			X[it-1]=xend.toDouble();
			Y[it-1]=yend.toDouble();
			}
		}
	vect->setVectorEnd(X,Y);
	cols[2]=xEndColName+"(X)";
	cols[3]=yEndColName+"(Y)";
	associations[curve]=cols.join(",");
	}
myPlot->replot();
}

void Graph::updateScale()
{
const QwtScaleDiv *scDiv=myPlot->axisScale(QwtPlot::xBottom);
double step=scDiv->majStep();
scales[0]=QString::number(scDiv->lBound());
scales[1]=QString::number(scDiv->hBound());
scales[2]=QString::number(step);
scales[3]=QString::number(myPlot->axisMaxMajor(2));
scales[4]=QString::number(myPlot->axisMaxMinor(2));
	
scDiv=myPlot->axisScale(QwtPlot::yLeft);
step=scDiv->majStep();
scales[8]=QString::number(scDiv->lBound());
scales[9]=QString::number(scDiv->hBound());
scales[10]=QString::number(step);
scales[11]=QString::number(myPlot->axisMaxMajor(0));
scales[12]=QString::number(myPlot->axisMaxMinor(0));
	
//updating scale maps
xCanvasMap=myPlot->canvasMap (QwtPlot::xBottom);
yCanvasMap=myPlot->canvasMap (QwtPlot::yLeft);
}

void Graph::updateBarsGap(int curve, int gapPercent)
{
long curveID = keys[curve];
curveLayout cL=layout[curve];
int style=cL.plotType;
if (style == VerticalBars || style == HorizontalBars)
	{
	QwtBarCurve *bars=(QwtBarCurve *)myPlot->curve(curveID);
	if (!bars || bars->gap() == gapPercent)
		return;

	bars->setGap(gapPercent);
	}
else if (style == Histogram)
	{
	QwtHistogram *histogram= (QwtHistogram*) myPlot->curve(curveID);
	if (!histogram)
		return;

	histogram->setGap(gapPercent);
	}

myPlot->replot();
emit modifiedGraph();

cL.barGap=gapPercent;
layout[curve]=cL;
}


void Graph::removePie()
{	
//QStringList::Iterator it=curvesList.at(0);
//curvesList.remove(it);
	
if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		{
		mrk->setText("");
		}
	}
	
myPlot->removeCurve(keys[0]);
for (int i=0;i<QwtPlot::axisCnt;i++)
	myPlot->setAxisAutoScale(i);	
myPlot->replot();

cOnPlot--;
keys.resize(cOnPlot);
piePlot=FALSE;		
	
emit modifiedGraph();
}

/*
*provided for convenience
*/
void Graph::removeCurve(const QString& s)
{	
int index = associations.findIndex(s);
removeCurve(index);
}

void Graph::removeCurve(int index)
{	
int i;
//QStringList::Iterator it=curvesList.at(index);
//QString name=curvesList[index];
//curvesList.remove(it);
	
bool isErrorBar= false;
if (associations[index].contains("(yErr)",TRUE)>0 ||  associations[index].contains("(xErr)",TRUE)>0 )
	{
	isErrorBar = true;
	}

for (i=0; i<(int)errorBars.count(); i++)
	{
	if (errorBars[i]==associations[index])
		{
		QStringList::Iterator it=errorBars.at(i);
		errorBars.remove(it);
			
		it=errorBarsLayout.at(i);
		errorBarsLayout.remove(it);
		}
	}	
	
QStringList::Iterator it=associations.at(index);
associations.remove(it);

myPlot->removeCurve(keys[index]);

for (i=0;i<QwtPlot::axisCnt;i++)
	myPlot->setAxisAutoScale(i);
updateScale();

if (piePlot)
	{
	cOnPlot--;
	keys.resize(cOnPlot);
	piePlot=FALSE;		
	}
else
	{	
	for (i=index;i<cOnPlot;i++)
		{
		keys[i]=keys[i+1];
		layout[i]=layout[i+1];
		xCols[i]=xCols[i+1];
		}
		
	cOnPlot--;
	keys.resize(cOnPlot);
	layout.resize(cOnPlot);
	xCols.resize(cOnPlot);
		
	if (rangeSelectorsEnabled)
		{
		if (cOnPlot>0)
			shiftCurveSelector(TRUE);
		else
			{
			myPlot->removeMarker(startID);
			myPlot->removeMarker(endID);
			startID=-1;
			endID=-1;
			endPoint=-1;
			selectedCurve=-1;
			selectedPoint=-1;
			selectedCursor=-1;	
			}			
		}
	}

if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk && !isErrorBar)//for error bar curves there is no legend item
		{
		if (piePlot)
			mrk->setText("");
		else
			removeLegendItem(mrk, index);
		}
	}

emit modifiedGraph();
myPlot->replot();
}

void Graph::removeLegendItem(LegendMarker* mrk, int index)
{
QString text=mrk->getText();
QStringList items=QStringList::split( "\n", text, FALSE);

if (index >= (int) items.count())
	return;

QStringList::Iterator it=items.at(index);
items.remove(it);

for (int i=0; i< (int)items.count(); i++)
	{
	QString item = (items[i]).stripWhiteSpace();
	if (item.startsWith("\\c{", true))
		{
		int pos =	item.find("}", 0);
		item.remove(0, pos);
		item.prepend("\\c{"+QString::number(i+1));
		}
	items[i]=item;
	}
text=items.join ( "\n" ) + "\n";
mrk->setText(text);
}

void Graph::contextMenuEvent(QContextMenuEvent *e)
{
if (selectedMarker>=0)
	return;

emit showContextMenu();
e->accept();
}

bool Graph::zoomOn()
{
return d_zoomer->isEnabled();
}

void Graph::zoomed (const QwtDoubleRect &rect)
{
d_zoomer->setZoomBase (rect);
	
const QwtScaleDiv *scDiv=myPlot->axisScale(2);
double step=scDiv->majStep();
scales[0]=QString::number(scDiv->lBound(),'g',3);
scales[1]=QString::number(scDiv->hBound(),'g',3);
scales[2]=QString::number(step);
scales[3]=QString::number(myPlot->axisMaxMajor(2));
scales[4]=QString::number(myPlot->axisMaxMinor(2));

scDiv=myPlot->axisScale(0);
step=scDiv->majStep();
scales[7]=QString::number(scDiv->lBound(),'g',3);
scales[8]=QString::number(scDiv->hBound(),'g',3);
scales[9]=QString::number(step);
scales[10]=QString::number(myPlot->axisMaxMajor(0));
scales[11]=QString::number(myPlot->axisMaxMinor(0));
	
emit modifiedGraph();
}
	
void Graph::zoom(bool on)
{
d_zoomer->setEnabled(on);

int prec, fw;
char f;
for (int j= 0; j<QwtPlot::axisCnt; j++)
		{
		myPlot->axisLabelFormat(j,f,prec,fw);
		if (prec<4)
			myPlot->setAxisLabelFormat (j,f,4,fw);			
		}

QCursor cursor=QCursor (QPixmap(lens_xpm),-1,-1);
if (on)
	myPlot->canvas()->setCursor(cursor);
else 
	myPlot->canvas()->setCursor(Qt::arrowCursor);
}

void Graph::drawText(bool on)
{
QCursor c=QCursor(IbeamCursor);
if (on)
myPlot->canvas()->setCursor(c);
else
myPlot->canvas()->setCursor(Qt::arrowCursor);

drawTextOn=on;
}

void Graph::insertImageMarker(ImageMarker* mrk)
{
QPixmap photo=mrk->image();
ImageMarker* mrk2= new ImageMarker(photo,myPlot);
mrk2->setFileName(mrk->getFileName());
mrk2->setOrigin(mrk->getOrigin());	
mrk2->setSize(mrk->size());	
long mrk2ID=myPlot->insertMarker(mrk2);
myPlot->replot();
	
int imagesOnPlot=images.size();
imagesOnPlot++;
images.resize(imagesOnPlot);
images[imagesOnPlot-1]=mrk2ID;
}

void Graph::insertImageMarker(const QPixmap& photo, const QString& fileName)
{
ImageMarker* mrk= new ImageMarker(photo, myPlot);
mrk->setFileName(fileName);
mrk->setOrigin(QPoint(0,0));
	
QSize picSize=photo.size();
int w=myPlot->canvas()->width();
if (picSize.width()>w)
	picSize.setWidth(w);
int h=myPlot->canvas()->height();
if (picSize.height()>h)
	picSize.setHeight(h);

mrk->setSize(picSize);
long mrkID=myPlot->insertMarker(mrk);
myPlot->replot();
	
int imagesOnPlot=images.size();
imagesOnPlot++;
images.resize(imagesOnPlot);
images[imagesOnPlot-1]=mrkID;
emit modifiedGraph();
}

void Graph::insertImageMarker(const QStringList& options)
{
QString fn=options[1];
QFile f(fn);
if (!f.exists())
	{ 
	QMessageBox::warning(0,tr("QtiPlot - File open error"), 
				   tr("Image file:" 
				   "<p><b>"+fn+"</b>" 
				   "<p>does not exist anymore!"));
	}
else
	{
	QPixmap photo;
	QFileInfo fi(fn);
	QString baseName = fi.fileName();
	int pos=baseName.find(".",0);
		
	if (fn.contains(".jpg")>0)
		photo.load(fn,"JPEG",QPixmap::Auto);
	else
		{
		QString type=baseName.right(baseName.length()-pos-1);
		photo.load(fn,type.upper(),QPixmap::Auto);
		}			

	int x=options[2].toInt();
	int y=options[3].toInt();

	int w=options[4].toInt();
	int h=options[5].toInt();

	ImageMarker* mrk= new ImageMarker(photo,myPlot);
	mrk->setFileName(fn);
	mrk->setOrigin(QPoint(x,y));
	mrk->setSize(QSize(w,h));
	long mrkID=myPlot->insertMarker(mrk);
	myPlot->replot();

	int imagesOnPlot=images.size();
	imagesOnPlot++;
	images.resize(imagesOnPlot);
	images[imagesOnPlot-1]=mrkID;
	}
}

bool Graph::drawTextActive()
{
return drawTextOn;
}

void Graph::drawLine(bool on)
{
drawLineOn=on;
}

bool Graph::drawLineActive()
{
return drawLineOn;
}

void Graph::setFitID(int id)
{
	fitID=id;
}

void Graph::insertOldFunctionCurve(const QString& formula, double from, double step, int points)
{
functions++;
//curvesList<<formula;
associations << formula;
	
int i;
int pos=formula.find("=",0);
QString function=formula;
function=function.mid(pos+1,function.length()-pos-1);
	
MathUtils::Parser parser;
double x;
parser.AddVar("x", &x);	

if (formula.contains("pi")>0)
		parser.AddConst("pi", M_PI);
if (formula.contains("Pi")>0)
		parser.AddConst("Pi", M_PI);
if (formula.contains("PI")>0)
		parser.AddConst("PI", M_PI);
	
parser.SetFormula(function.ascii());

QArray<double> X(points),Y(points);
X[0]=from;x=from;Y[0]=parser.Calc();
for (i = 1;i<points;i++ )
	{
	x+=step;
	X[i]=x;
	Y[i]=parser.Calc();
	}
	
long curveID = myPlot->insertCurve(formula);
myPlot->setCurvePen(curveID, QPen(black,widthLine)); 
myPlot->setCurveData(curveID, X, Y, points);

cOnPlot++;
keys.resize(cOnPlot);
keys[cOnPlot-1]=curveID;
	
xCols.resize(cOnPlot);
xCols[cOnPlot-1]=0;//dummy value

curveLayout cl=initCurveLayout();

layout.resize(cOnPlot);
layout[cOnPlot-1]=cl;

for (i=0;i<QwtPlot::axisCnt;i++)
	myPlot->setAxisAutoScale(i);
myPlot->replot();

updateScale();
scales[5]="1";
scales[6]="0";
scales[12]="1";
scales[13]="0";

d_zoomer->setZoomBase();
}

void Graph::addFunctionCurve(QString& type,QStringList &formulas,QStringList &vars,QValueList<double> &ranges,QValueList<int> &points)
{
int i,ndata;
bool error=FALSE;
QArray<double> X(1),Y(1);
QString label,swap;
double step;

if (type=="Function")
{
	ndata=points[0];
	X.resize(points[0]);
	Y.resize(points[0]);
	step=(ranges[1]-ranges[0])/(double) (points[0]-1);
	MathUtils::Parser parser;
	double x;
	bool error=FALSE;

	try
    	{
		if (formulas[0].contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (formulas[0].contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (formulas[0].contains("PI")>0)
			parser.AddConst("PI", M_PI);
		
		parser.AddVar(vars[0].ascii(), &x);	
		parser.SetFormula(formulas[0].ascii());
			
		X[0]=ranges[0];x=ranges[0];Y[0]=parser.Calc();
		for (i = 1;i<ndata;i++ )
			{
			x+=step;
			X[i]=x;
			Y[i]=parser.Calc();
			}
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Input function error", e.what());
		error=TRUE;	
    	}
		label="f"+QString::number(functions)+"(x)="+formulas[0]+",x,"+QString::number(ranges[0],'g',6)+","+QString::number(ranges[1],'g',6);
}	

if (error)
	return;
	
if (type=="Parametric plot" || type=="Polar plot")
	{
	X.resize(points[0]);
	Y.resize(points[0]);
	step=(ranges[1]-ranges[0])/(double) (points[0]-1);	
	MathUtils::Parser xparser;
	MathUtils::Parser yparser;
	double par;
	ndata=points[0];
	if (type=="Parametric plot")
	{
		label="X"+QString::number(functions)+"("+vars[0]+")="+formulas[0]+",Y"+QString::number(functions)+"("+vars[0]+")="+formulas[1]+","+vars[0]+","+QString::number(ranges[0],'g',6)+","+QString::number(ranges[1],'g',6);
	}
	if (type=="Polar plot")
	{
		label="R"+QString::number(functions)+"("+vars[0]+")="+formulas[0]+",Theta"+QString::number(functions)+"("+vars[0]+")="+formulas[1]+","+vars[0]+","+QString::number(ranges[0],'g',6)+","+QString::number(ranges[1],'g',6);
		swap=formulas[0];
		formulas[0]="("+swap+")*cos("+formulas[1]+")";
		formulas[1]="("+swap+")*sin("+formulas[1]+")";
	}
	try
	    	{
		if (formulas[0].contains("pi")>0)
			xparser.AddConst("pi", M_PI);
		if (formulas[0].contains("Pi")>0)
			xparser.AddConst("Pi", M_PI);
		if (formulas[0].contains("PI")>0)
			xparser.AddConst("PI", M_PI);
		if (formulas[1].contains("pi")>0)
			yparser.AddConst("pi", M_PI);
		if (formulas[1].contains("Pi")>0)
			yparser.AddConst("Pi", M_PI);
		if (formulas[1].contains("PI")>0)
			yparser.AddConst("PI", M_PI);
		xparser.AddVar(vars[0].ascii(), &par);	
		yparser.AddVar(vars[0].ascii(), &par);	
		xparser.SetFormula(formulas[0].ascii());
		yparser.SetFormula(formulas[1].ascii());
		par=ranges[0];
		for (i = 0;i<points[0];i++ )
			{
			X[i]=xparser.Calc();
			Y[i]=yparser.Calc();
			par+=step;
			}
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Input function error",e.what());
		error=TRUE;	
    		}
	}		

if (error)
	return;

functions++;
//curvesList<<label;
associations << label;
	
QColor c=QColor(black);
long curveID = myPlot->insertCurve(label);
myPlot->setCurvePen(curveID, QPen(black,widthLine)); 
myPlot->setCurveData(curveID, X, Y, ndata);

cOnPlot++;
keys.resize(cOnPlot);
keys[cOnPlot-1]=curveID;

xCols.resize(cOnPlot);
xCols[cOnPlot-1]=0;
	
curveLayout cl = initCurveLayout();
cl.connectType=0;

layout.resize(cOnPlot);
layout[cOnPlot-1]=cl;
	
if (legendMarkerID>=0)
	{
	LegendMarker* mrk=(LegendMarker*) myPlot->marker(legendMarkerID);
	if (mrk)
		{
		QString text=mrk->getText();
		text+="\\c{";
		text+=QString::number(cOnPlot)+"}";
		text+=label+"\n";
		mrk->setText(text);
		}
	}
	
for (i=0;i<QwtPlot::axisCnt;i++)
	myPlot->setAxisAutoScale(i);
myPlot->replot();

updateScale();
d_zoomer->setZoomBase();
emit modifiedGraph();
}

void Graph::insertFunctionCurve(const QString& formula, double from, double step, int points)
{
QStringList curve;
QString type;
QStringList formulas;
QStringList variables;	
QValueList<double> ranges;
QValueList<int> varpoints;

if (formula.contains(",") == 0)		//version < 0.4.3
	insertOldFunctionCurve(formula,from, step, points);
else
	{//version >= 0.4.3
	curve=QStringList::split (",",formula,TRUE);
	if (curve[0][0]=='f')
		{
		type="Function";
		formulas+=curve[0].section('=',1,1);
		variables+=curve[1];
		ranges+=curve[2].toDouble();
		ranges+=curve[3].toDouble();
		varpoints[0]=points;
		}
	else if (curve[0][0]=='X')
		{
		type="Parametric plot";
		formulas+=curve[0].section('=',1,1);
		formulas+=curve[1].section('=',1,1);	
		variables+=curve[2];
		ranges+=curve[3].toDouble();
		ranges+=curve[4].toDouble();
		varpoints[0]=points;
		}
	else if (curve[0][0]=='R')
		{
		type="Polar plot";
		formulas+=curve[0].section('=',1,1);
		formulas+=curve[1].section('=',1,1);	
		variables+=curve[2];
		ranges+=curve[3].toDouble();
		ranges+=curve[4].toDouble();
		varpoints[0]=points;
		}
	addFunctionCurve(type,formulas,variables,ranges,varpoints);	 
	}
}

void Graph::createWorksheet(const QString& name)
{
for (int i=0; i<cOnPlot; i++)	
	{
	QwtPlotCurve *cv = (QwtPlotCurve *)myPlot->curve(keys[i]);
	if (cv->title() == name)
		{
		int size= cv->dataSize();
		QString text="1\t2\n";
		for (int j=0;j<size;j++)	
				{
				text+=QString::number(cv->x(j))+"\t";
				text+=QString::number(cv->y(j))+"\n";
				}
		emit createTable("table1",size,2, text);
		return;
		}
	}
}

bool Graph::lineProfile()
{
return lineProfileOn;	
}

QString Graph::saveToString()
{
QString s="<graph>\n";
			
s+="ggeometry\t";
QPoint p=this->pos();
s+=QString::number(p.x())+"\t";
s+=QString::number(p.y())+"\t";
s+=QString::number(this->frameGeometry().width())+"\t";
s+=QString::number(this->frameGeometry().height())+"\n";
s+=saveTitle();
s+=saveGridOptions();
s+=saveEnabledAxes();
s+="AxesTitles\t"+saveScaleTitles();
s+=saveAxesTitleColors();
s+=saveAxesTitleAlignement();
s+=saveFonts();
s+=saveTicksType();
s+=saveEnabledTickLabels();
s+=saveAxesColors();
s+=saveCanvasFrame();
s+=saveLabelsRotation();
s+=saveCurves();			
s+=saveErrorBars();
s+="scale\t"+saveScale();
s+=saveSecondaryPlotLimits();
s+=saveLabelsFormat();
s+=saveAxesLabelsType();

if (legendMarkerID>0)
	s+=saveLegend();
s+=saveMarkers();

s+="</graph>\n";
return s;
}

void Graph::showIntensityTable()
{	
ImageMarker* mrk=(ImageMarker*) myPlot->marker(selectedMarker);
if (!mrk)
	return;

QPixmap pic=mrk->image();
emit createIntensityTable(pic);
}

void Graph::resizeMarkers ()
{
if (ignoreResize)
	return;

QArray<long> texts=textMarkerKeys();

QwtDiMap xmap=myPlot->canvasMap (QwtPlot::xBottom);
QwtDiMap ymap=myPlot->canvasMap (QwtPlot::yLeft);

int i=0;
for (i=0;i<(int)lines.size();i++)
	{			
	LineMarker* mrkL=(LineMarker*)myPlot->marker(lines[i]);
	QPoint start=mrkL->startPoint();
	QPoint end=mrkL->endPoint();
		
	double r0_x=xCanvasMap.invTransform(start.x());
	double r0_y=yCanvasMap.invTransform(start.y());	
	double r1_x=xCanvasMap.invTransform(end.x());
	double r1_y=yCanvasMap.invTransform(end.y());
			
	int x0=xmap.transform(r0_x);
	int x1=xmap.transform(r1_x);	
	int y0=ymap.transform(r0_y);
	int y1=ymap.transform(r1_y);

	mrkL->setStartPoint(QPoint(x0,y0));
	mrkL->setEndPoint(QPoint(x1,y1));
	}
	
for (i=0;i<(int)texts.size();i++)
	{
	LegendMarker* mrkT = (LegendMarker*) myPlot->marker(texts[i]);
	QPoint o=mrkT->rect().topLeft();
	double d_ox=xCanvasMap.invTransform(o.x());
	double d_oy=yCanvasMap.invTransform(o.y());
		
	int ox=xmap.transform(d_ox);	
	int oy=ymap.transform(d_oy);
	mrkT->setOrigin(QPoint(ox,oy));		
	}
	
for (i=0;i<(int)images.size();i++)
	{
	ImageMarker* mrk = (ImageMarker*) myPlot->marker(images[i]);
	QPoint o=mrk->getOrigin();
	double d_ix=xCanvasMap.invTransform(o.x());
	double d_iy=yCanvasMap.invTransform(o.y());
	int ix=xmap.transform(d_ix);	
	int iy=ymap.transform(d_iy);	
	mrk->setOrigin(QPoint(ix,iy));
	}

xCanvasMap=xmap;
yCanvasMap=ymap;
}

Graph::~Graph()
{
delete d_zoomer;
delete titlePicker;
delete scalePicker;	
delete cp;
delete myPlot;
}

Plot::Plot(QWidget *parent, const char *name)
		: QwtPlot(parent,name)
{
setGeometry(QRect(0,0,500,400));
setAutoLegend(FALSE); // We don't want a legend
setAxisTitle(0,"Y Axis Title");
setAxisTitle(2,"X Axis Title");	
	
//custom scale
for (int j= 0; j<QwtPlot::axisCnt; j++)
		ticksType<<Out;
	
QwtPlotLayout *pLayout=plotLayout();
pLayout->setCanvasMargin(0);
	
QwtPlotCanvas* plCanvas=canvas();
plCanvas->setFocusPolicy(QWidget::StrongFocus);
plCanvas->setFocusIndicator(QwtPlotCanvas::ItemFocusIndicator);
plCanvas->setFocus();
plCanvas->setFrameShadow( QwtPlot::Plain);
plCanvas->setCursor(Qt::arrowCursor);
plCanvas->setLineWidth(0);
		
setFocusPolicy(QWidget::StrongFocus);
setFocusProxy(plCanvas);
		
minTickLength=5;
majTickLength=9;		
}

void Plot::printCanvas(QPainter *painter, const QRect &canvasRect,
    const QwtArray<QwtDiMap> &map, const QwtPlotPrintFilter &pfilter) const
{
	const QwtPlotCanvas* plotCanvas=canvas();	
	int w=plotCanvas->lineWidth();

	if (w>0)
    {
		QPalette pal =plotCanvas->palette();
		QColor color=pal.color(QPalette::Active, QColorGroup::Foreground);
		
		painter->save();
		painter->setPen (QPen(color,w,Qt::SolidLine));
		
		QRect rect=canvasRect;
		
		for (int i=0;i<4;i++)
			{
			switch (i)
				{
				case QwtPlot::xBottom :		
					if (ticksType[i] == Plot::In)
						rect.setHeight(canvasRect.height()-1);
				break;
					
				case QwtPlot::xTop :		
				break;
					
				case QwtPlot::yRight :		
					if (ticksType[i] == Plot::In)
						rect.setWidth(canvasRect.width()-1);
					else if (ticksType[i] == Plot::Out)
						rect.setWidth(canvasRect.width()+1);
				break;
					
				case QwtPlot::yLeft:		
				break;
				}
			}
		
		QwtPainter::drawRect(painter, rect.x(), rect.y(), rect.width(), rect.height());
		painter->restore();
    }
	
    painter->setClipping(TRUE);
    QwtPainter::setClipRect(painter, QRect(canvasRect.x(), canvasRect.y(),
        canvasRect.width() - 1, canvasRect.height() - 1));

    drawCanvasItems(painter, canvasRect, map, pfilter);
}

void Plot::drawCanvasItems (QPainter *painter, const QRect &rect, 
							const QwtArray< QwtDiMap > &map, const QwtPlotPrintFilter &pfilter) const
{
QwtPlot::drawCanvasItems(painter, rect, map, pfilter);

for (int i=0; i<QwtPlot::axisCnt; i++)
 	{
	if (ticksType[i] == In)	
		drawInwardTicks(painter, rect, map[i], i);
	else if (ticksType[i] == Both)	
		drawInwardMinorTicks(painter, rect, map[i], i);
 	}
}

void Plot::drawInwardTicks(QPainter *painter, const QRect &rect, 
							const QwtDiMap &map, int axis) const
{
	QwtScale *scale=(QwtScale *) QwtPlot::axis (axis);
	if (!scale)
		return;
	
	int x1=rect.left();
	int x2=rect.right();	
	int y1=rect.top();
	int y2=rect.bottom();
	
	QPalette pal=scale->palette();
	QColor color=pal.color(QPalette::Active, QColorGroup::Foreground);
		
    painter->save();	
    painter->setPen(QPen(color,1,QPainter::SolidLine));
		
	QwtScaleDiv *scDiv=(QwtScaleDiv *)axisScale(axis);
	int minTicks=scDiv->minCnt();
	int majTicks= scDiv->majCnt();

int j, x, y, low,high;
switch (axis)
	{
	case QwtPlot::yLeft:
	x=x1;
	low=y1+majTickLength;
	high=y2-majTickLength;
    for (j = 0; j < minTicks; j++)
        {
            y = map.transform(scDiv->minMark(j));
			if (y>low && y< high)
            	QwtPainter::drawLine(painter, x, y, x+minTickLength, y);
        }		
	 for (j = 0; j < majTicks; j++)
        {
            y = map.transform(scDiv->majMark(j));
			if (y>low && y< high)
            	QwtPainter::drawLine(painter, x, y, x+majTickLength, y);
        }
	break;
		
	case QwtPlot::yRight:
		{
		x=x2;
		low=y1+majTickLength;
		high=y2-majTickLength;
     	for (j = 0; j < minTicks; j++)
       	 	{
            y = map.transform(scDiv->minMark(j));
			if (y>low && y< high)
            	QwtPainter::drawLine(painter, x+1, y, x-minTickLength, y);
        	}
		 for (j = 0; j <majTicks; j++)
        	{
            y = map.transform(scDiv->majMark(j));
			if (y>low && y< high)
            	QwtPainter::drawLine(painter, x+1, y, x-majTickLength, y);
        	}
		}
	  break;
		
	case QwtPlot::xBottom:
		y=y2;
		low=x1+majTickLength;
		high=x2-majTickLength;
     	for (j = 0; j < minTicks; j++)
        	{
            x = map.transform(scDiv->minMark(j));
			if (x>low && x<high)
            	QwtPainter::drawLine(painter, x, y+1, x, y-minTickLength);
       		 }	
	 	for (j = 0; j < majTicks; j++)
        	{
            x = map.transform(scDiv->majMark(j));
			if (x>low && x<high)
            	QwtPainter::drawLine(painter, x, y+1, x, y-majTickLength);
        	}
		break;
	
	case QwtPlot::xTop:
		y=y1;
		low=x1+majTickLength;
		high=x2-majTickLength;
    	 for (j = 0; j < minTicks; j++)
       		{
             x = map.transform(scDiv->minMark(j));
			if (x>low && x<high)
            	QwtPainter::drawLine(painter, x, y, x, y+minTickLength);
       	    }		
	 	for (j = 0; j <majTicks; j++)
        	{
            x = map.transform(scDiv->majMark(j));
			if (x>low && x<high)
            	QwtPainter::drawLine(painter, x, y, x, y+majTickLength);
        	}
	break;
	}
painter->restore();
}

void Plot::drawInwardMinorTicks(QPainter *painter, const QRect &rect, 
							const QwtDiMap &map, int axis) const
{
	QwtScale *scale=(QwtScale *) QwtPlot::axis (axis);
	if (!scale)
		return;
	
	int x1=rect.left();
	int x2=rect.right();	
	int y1=rect.top();
	int y2=rect.bottom();
	
	QPalette pal=scale->palette();
	QColor color=pal.color(QPalette::Active, QColorGroup::Foreground);
		
    painter->save();	
    painter->setPen(QPen(color,1,QPainter::SolidLine));
		
	QwtScaleDiv *scDiv=(QwtScaleDiv *)axisScale(axis);
	int minTicks=scDiv->minCnt();

int j, x, y, low,high;
switch (axis)
	{
	case QwtPlot::yLeft:
	x=x1;
	low=y1+majTickLength;
	high=y2-majTickLength;
    for (j = 0; j < minTicks; j++)
        {
            y = map.transform(scDiv->minMark(j));
			if (y>low && y< high)
            	QwtPainter::drawLine(painter, x, y, x+minTickLength, y);
        }		
	break;
		
	case QwtPlot::yRight:
		{
		x=x2;
		low=y1+majTickLength;
		high=y2-majTickLength;
     	for (j = 0; j < minTicks; j++)
       	 	{
            y = map.transform(scDiv->minMark(j));
			if (y>low && y< high)
            	QwtPainter::drawLine(painter, x+1, y, x-minTickLength, y);
        	}
		}
	  break;
		
	case QwtPlot::xBottom:
		y=y2;
		low=x1+majTickLength;
		high=x2-majTickLength;
     	for (j = 0; j < minTicks; j++)
        	{
            x = map.transform(scDiv->minMark(j));
			if (x>low && x<high)
            	QwtPainter::drawLine(painter, x, y+1, x, y-minTickLength);
       		 }	
		break;
	
	case QwtPlot::xTop:
		y=y1;
		low=x1+majTickLength;
		high=x2-majTickLength;
    	 for (j = 0; j < minTicks; j++)
       		{
             x = map.transform(scDiv->minMark(j));
			if (x>low && x<high)
            	QwtPainter::drawLine(painter, x, y, x, y+minTickLength);
       	    }		
	break;
	}
painter->restore();
}

void Plot::setTicksType(int axis, int type)
{
ticksType[axis]=type;
}
