#ifndef GRAPH3D_H
#define GRAPH3D_H

#include <qwidget.h>
#include <qobject.h>
#include <qfont.h>

#include <qwt3d_surfaceplot.h>
#include <qwt3d_function.h> 

#include "worksheet.h"
#include "bars.h"

using namespace Qwt3D;

class UserFunction;

class Graph3D: public QWidget
{
	Q_OBJECT

public:
    Graph3D (const QString& s,double xl,double xr,double yl,double yr,
				double zl,double zr, QWidget* parent, const char* name, WFlags f);

	Graph3D (WksheetWindow* table,const QString& colName, 
			QWidget* parent, const char* name, WFlags f);

/*!
* used when readin a .qti file
*/
	Graph3D (WksheetWindow* table,const QString& xColName, const QString& yColName,
					double xl, double xr, double yl, double yr, double zl, double zr,
					QWidget* parent, const char* name, WFlags f);
/*!
* used when creating a ribbon plot from the plot wizard
*/
	Graph3D (WksheetWindow* table, const QString& xColName, const QString& yColName,
						QWidget* parent, const char* name, WFlags f);

	Graph3D (WksheetWindow* table, int xCol, int yCol, int zCol, 
						int type, QWidget* parent, const char* name, WFlags f);

/*!
* used when readin a .qti file
*/
	Graph3D (WksheetWindow* table, int xCol, int yCol, int zCol, 
						double xl, double xr, double yl, double yr, double zl, double zr,
						QWidget* parent, const char* name, WFlags f);

	~Graph3D();

	enum PlotType{Scatter=0, Trajectory = 1, Bars = 2};
	enum PointStyle{None=0, Dots=1, VerticalBars=2, HairCross=3, Cones=4};

	Qwt3D::SurfacePlot* sp;
	UserFunction *func;
	
public slots:
	void initPlot();
	void initCoord();
	void addFunction(const QString& s,double xl,double xr,double yl,
						  double yr,double zl,double zr);
	void insertNewFunction(const QString& s,double xl,double xr,double yl,
						  double yr,double zl,double zr);
	void insertNewData(WksheetWindow* table, const QString& colName);
	void addData(WksheetWindow* table,const QString& colName);
/*!
* used when creating a ribbon plot from the plot wizard
*/
	void addData(WksheetWindow* table,const QString& xColName,const QString& yColName);
	void addData(WksheetWindow* table,const QString& xColName,const QString& yColName,
							double xl, double xr, double yl, double yr, double zl, double zr);

	void addData(WksheetWindow* table, int xCol,int yCol,int zCol, int type);
	void addData(WksheetWindow* table, int xCol,int yCol,int zCol, 
							double xl, double xr, double yl, double yr, double zl, double zr);

	void removePlot();

	void updateData(WksheetWindow* table);
	void updateDataXY(WksheetWindow* table, int xCol, int yCol);
	void updateDataXYZ(WksheetWindow* table, int xCol, int yCol, int zCol);
	void setName(const QString& s);

	//user function
	UserFunction* userFunction();
	QString formula();

	//event handlers
	bool eventFilter(QObject *object, QEvent *e);
	void closeEvent( QCloseEvent *e );
	void resizeEvent ( QResizeEvent *);
	void contextMenuEvent(QContextMenuEvent *e);
	
	//axes
	void setFramed();
	void setBoxed();
	void setNoAxes();

	//mesh
	void setNoGrid();
	void setHiddenLineGrid();
	void setLineGrid();
	void setFilledMesh();
	void setPointsMesh();
	void setBarsPlot();
	void setFloorData();
	void setFloorIsolines();
	void setEmptyFloor();

	//grid
	int grids();
	void setGrid(Qwt3D::SIDE s, bool b);
	void setGrid(int grids);

	void setLeftGrid(bool b);
	void setRightGrid(bool b);
	void setCeilGrid(bool b);
	void setFloorGrid(bool b);
	void setFrontGrid(bool b);
	void setBackGrid(bool b);

	void setStyle(Qwt3D::COORDSTYLE coord,Qwt3D::FLOORSTYLE floor,
							Qwt3D::PLOTSTYLE plot, Graph3D::PointStyle point);
	void setStyle(const QStringList& st);
	void resetNonEmptyStyle();

	void setRotation(double  xVal,double  yVal,double  zVal);
	void setScale(double  xVal,double  yVal,double  zVal);
	void setShift(double  xVal,double  yVal,double  zVal);
	void updateScaling(double  xVal,double  yVal,double  zVal);
	
	double xRotation(){return sp->xRotation();};
	double yRotation(){return sp->yRotation();};
	double zRotation(){return sp->zRotation();};
	
	double xScale(){return sp->xScale();};
	double yScale(){return sp->yScale();};
	double zScale(){return sp->zScale();};
	
	double xShift(){return sp->xShift();};
	double yShift(){return sp->yShift();};
	double zShift(){return sp->zShift();};
	
	double zoom(){return sp->zoom();};
	void setZoom(double  val);
	void updateZoom(double  val);
	
	Qwt3D::PLOTSTYLE plotStyle();
	Qwt3D::FLOORSTYLE floorStyle();
	Qwt3D::COORDSTYLE coordStyle();

	void print();
	void copyImage();
	void saveImage();
	QString birthDate();
	QString saveToString(const QString& geometry);
	QString sizeToString();

	void zoomChanged(double);
	void rotationChanged(double, double, double);
	void scaleChanged(double, double, double);
	void shiftChanged(double, double, double);

	//mesh
	void setMeshLineWidth(double lw);
	double meshLineWidth(){return sp->meshLineWidth();};

	//colors
	void setDataColors(const QColor& cMax, const QColor& cMin);

	void updateColors(const QColor& meshColor,const QColor& axesColor,const QColor& numColor,
						   const QColor& labelColor,const QColor& bgColor,const QColor& gridColor);
	void setTransparency(double t);
	double transparency(){return alpha;};
		
	QColor minDataColor();
	QColor maxDataColor();
	QColor meshColor(){return meshCol;};
	QColor axesColor(){return axesCol;};
	QColor labelColor(){return labelsCol;};
	QColor numColor(){return numCol;};
	QColor bgColor(){return bgCol;};
	QColor gridColor(){return gridCol;};
	void setColors(const QStringList& colors);
	void setColors(const QColor& meshColor,const QColor& axesColor,const QColor& numColor,
						   const QColor& labelColor,const QColor& bgColor,const QColor& gridColor);

	//title
	void updateTitle(const QString& s,const QColor& color,const QFont& font);
	QFont titleFont(){return titleFnt;};
	QString plotTitle(){return title;};
	QColor titleColor(){return titleCol;};
	void setTitle(const QStringList& list);
	void setTitle(const QString& s,const QColor& color,const QFont& font);

	//resolution
	void setResolution(int r);
	int resolution(){return sp->resolution();};
	
	//legend
	void showColorLegend(bool show);
	bool isLegendOn(){return legendOn;};
	
	//axes
	QStringList axesLabels(){return labels;};
	void updateLabel(int axis,const QString& label, const QFont& f);
	void setAxesLabels(const QStringList& list);
	void resetAxesLabels();

	QFont xAxisLabelFont();
	QFont yAxisLabelFont();
	QFont zAxisLabelFont();

	void setXAxisLabelFont(const QFont& fnt);
	void setYAxisLabelFont(const QFont& fnt);
	void setZAxisLabelFont(const QFont& fnt);
	
	void setXAxisLabelFont(const QStringList& list);
	void setYAxisLabelFont(const QStringList& list);
	void setZAxisLabelFont(const QStringList& list);

	QFont numbersFont();
	void setNumbersFont(const QFont& font);
	void setNumbersFont(const QStringList& list);

	double xStart();
	double xStop();
	double yStart();
	double yStop();
	double zStart();
	double zStop();
	QStringList scaleLimits();
	void updateScale(int axis,const QStringList& options);
	void updateScales(double xl, double xr, double yl, double yr, double zl, double zr);
	void updateScales(double xl, double xr, double yl, double yr, 
				  		double zl, double zr, int xcol, int ycol);
	void updateScales(double xl, double xr, double yl, double yr, 
				  		double zl, double zr, int xCol, int yCol, int zCol);

	QStringList scaleTicks();
	void setTicks(const QStringList& options);
	
	void updateTickLength(int, double majorLength, double minorLength);
	void adjustLabels(int val);
	int labelsDistance(){return labelsDist;};

	QStringList axisTickLengths();
	void setTickLengths(const QStringList& list);

	void setOptions(bool legend, int r, int dist);
	void setOptions(const QStringList& list);
	void update();

	Qwt3D::Triple** allocateData(int columns, int rows);
	void deleteData(Qwt3D::Triple **data, int columns);

	// bars
	double barsRadius();
	void setBarsRadius(double rad);
	void updateBars(double rad);
	
	// scatter plots
	double pointsSize(){return pointSize;};
	bool smoothPoints(){return smooth;};
	void updatePoints(double size, bool sm);
	
	bool smoothCrossHair(){return crossHairSmooth;};
	bool boxedCrossHair(){return crossHairBoxed;};
	double crossHairRadius(){return crossHairRad;};
	double crossHairLinewidth(){return crossHairLineWidth;};
	void updateCross(double rad, double linewidth, bool smooth, bool boxed);
	void setCrossOptions(double rad, double linewidth, bool smooth, bool boxed);
	void setCrossMesh();
	
	double coneRadius(){return conesRad;};
	int coneQuality(){return conesQuality;};
	void updateCones(double rad, int quality);
	void setConesOptions(double rad, int quality);
	void setConesMesh();
	
	PointStyle pointType(){return pointStyle;};
	void setPointOptions(double size, bool s);
	
	void showWorksheet();
	void askOnCloseEvent(bool ask){askOnClose = ask; };

signals:   
	void showWorksheet(const QString& caption);
	void showContextMenu();
	void showOptionsDialog();
	void closedGraph(QWidget*);
	void hiddenPlot(QWidget*);
	void resizedPlot(QWidget*);
	void modified();
	void custom3DActions(QWidget*);
	
private:
	QString birthdate,title;
	QStringList labels;
	QFont titleFnt;
	bool legendOn;
	QMemArray<int> scaleType;
	QColor axesCol,labelsCol,titleCol,meshCol,bgCol,numCol,gridCol;
	QColor fromColor, toColor;//custom data colors
	int labelsDist;
	bool askOnClose; 
	Qwt3D::StandardColor* col_;
	double barsRad, alpha, pointSize, crossHairRad, crossHairLineWidth, conesRad;
	bool smooth; // draw 3D points with smoothed angles
	bool crossHairSmooth, crossHairBoxed;
    int conesQuality;
	PointStyle pointStyle;
	WksheetWindow *worksheet;
	Qwt3D::PLOTSTYLE style_;
};

class UserFunction : public Function
{
public:

    UserFunction(const QString& s, SurfacePlot& pw);
	~UserFunction();
    double operator()(double x, double y);
	QString function(){return formula;};

private:
	  QString formula;
};

#endif // Plot3D_H
