#include "importDialog.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qlabel.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qinputdialog.h>
#include <qmessagebox.h>
#include <qregexp.h>
#include <qspinbox.h>

importDialog::importDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "importDialog" );
    setSizePolicy( QSizePolicy( (QSizePolicy::SizeType)0, (QSizePolicy::SizeType)0, 0, 0, sizePolicy().hasHeightForWidth() ) );
    setMinimumSize( QSize( 310, 140 ) );
	setMaximumSize( QSize( 310, 140 ) );
    setMouseTracking( TRUE );
    setSizeGripEnabled( FALSE );
	
	GroupBox1 = new QButtonGroup( 2,QGroupBox::Horizontal,tr(""),this,"GroupBox1" );

	new QLabel( tr( "Separator" ), GroupBox1, "TextLabel1",0 );
    boxSeparator = new QComboBox(GroupBox1, "boxSeparator" );
	boxSeparator->setEditable ( TRUE );
	
	new QLabel( tr( "Ignore first" ), GroupBox1, "TextLabel2",0 );
    boxLines = new QSpinBox(0,1000,1,GroupBox1, "boxLines" );
	boxLines->setSuffix(" lines");
	
	GroupBox2 = new QButtonGroup( 2,QGroupBox::Horizontal,tr(""),this,"GroupBox2" );
	GroupBox2->setFlat(TRUE);
	GroupBox2->setLineWidth(0);
	
	buttonOk = new QPushButton(GroupBox2, "buttonOk" );
    buttonOk->setAutoDefault( TRUE );
    buttonOk->setDefault( TRUE );
   
    buttonCancel = new QPushButton(GroupBox2, "buttonCancel" );
    buttonCancel->setAutoDefault( TRUE );
	
	QVBoxLayout* hlayout = new QVBoxLayout(this,5,5, "hlayout");
    hlayout->addWidget(GroupBox1);
	hlayout->addWidget(GroupBox2);

    languageChange();
   
    // signals and slots connections
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
}

importDialog::~importDialog()
{
}

void importDialog::languageChange()
{
    setCaption( tr( "QtiPlot - Import Options" ) );
    buttonOk->setText( tr( "&OK" ) );
	buttonCancel->setText( tr( "&Cancel" ) );
	
	boxSeparator->clear();
	boxSeparator->insertItem( tr( "Auto" ) );
    boxSeparator->insertItem( tr( ":" ) );
    boxSeparator->insertItem( tr( ";" ) );
    boxSeparator->insertItem( tr( "," ) );
	boxSeparator->insertItem( tr( "Define" ) );
}

QString importDialog::separator()
{
QString sep;
switch (boxSeparator->currentItem())
	{
	case 0:
		sep="auto";
	break;

	case 1:
		sep=":";
	break;

	case 2:
		sep=";";
	break;

	case 3:
		sep=",";
	break;
	
	case 4:
	bool ok;
		again:
		sep=QInputDialog::getText ("Import options","Separator",QLineEdit::Normal,QString::null,&ok,this,0 );
	
		if(!ok)
		 	{
			sep="auto";
			break;
			}
		else if (sep.contains(QRegExp("[0-9]"))!=0)
			{
			QMessageBox::warning(0,"QtiPlot - Import options error","The separator must not contain digit characters");
			goto again;
			}
		else
			sep=sep.simplifyWhiteSpace(); // We don't want extra white spaces!
	break;
	}	
return sep; 
}

void importDialog::setLines(int lines)
{
boxLines->setValue(lines);
}

void importDialog::setSeparator(const QString& sep)
{
if (sep=="auto")
	boxSeparator->setCurrentItem(0);
else if (sep==":")
	boxSeparator->setCurrentItem(1);
else if (sep==";")
	boxSeparator->setCurrentItem(2);
else if (sep==",")
	boxSeparator->setCurrentItem(3);
else
	boxSeparator->setCurrentText(sep);
}

void importDialog::accept()
{
emit options(separator(), boxLines->value());
close();
}
