#include "plotDialog.h"
#include "colors.h"
#include "graph.h"

#include <qaccel.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlistbox.h>
#include <qpushbutton.h>
#include <qspinbox.h>
#include <qtabwidget.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qgroupbox.h>
#include <qbuttongroup.h>
#include <qhbox.h>
#include <qvbox.h>
#include <qmessagebox.h>
#include <qlineedit.h>
#include <qregexp.h>

#include "parser/muParser.h"

MyDialog::MyDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "MyDialog" );
    setCaption( tr( "QtiPlot - Custom plot" ) );

	QVBox *box1 = new QVBox(this);
	box1->setSpacing (5);

    listBox = new QListBox( box1, "listBox" );
   	
	QHBox *hbox1 = new QHBox ( box1);
	hbox1->setSpacing (5);
    new QLabel(tr( "Plot type" ),hbox1,"TextLabel111",0 );
	
	boxPlotType = new QComboBox( FALSE, hbox1, "boxPlotType" );
    boxPlotType->insertItem( tr( "Line" ) );
    boxPlotType->insertItem( tr( "Scatter" ) );
    boxPlotType->insertItem( tr( "Line + Symbol" ) ); 
	boxPlotType->insertItem( tr( "Vertical Bars" ) );
	boxPlotType->insertItem( tr( "Horizontal Bars" ) );
	boxPlotType->insertItem( tr( "Histogram" ) );
	boxPlotType->insertItem( tr( "Vector XYXY" ) );
	boxPlotType->insertItem( tr( "Error Bars" ) );
	
	QVBox  *box2=new QVBox (this); 
	box2->setSpacing (5);
    privateTabWidget = new QTabWidget(box2, "privateTabWidget" );

	initLinePage();
	initSymbolsPage();
	initHistogramPage();
	initErrorsPage();
	initSpacingPage();
	initVectPage();
	
	QHBox *hbox2 = new QHBox ( box2);
	hbox2->setSpacing (5);
	
	btnWorksheet = new QPushButton(hbox2, "btnWorksheet" );
    btnWorksheet->setText( tr( "&Worksheet" ) );
    btnWorksheet->setAutoDefault( TRUE );

	buttonApply = new QPushButton(hbox2, "buttonApply" );
    buttonApply->setText( tr( "&Apply" ) );
    buttonApply->setAutoDefault( TRUE );

    buttonOk = new QPushButton(hbox2, "buttonOk" );
    buttonOk->setText( tr( "&OK" ) );
    buttonOk->setAutoDefault( TRUE );
    buttonOk->setDefault( TRUE );

    buttonCancel = new QPushButton(hbox2, "buttonCancel" );
    buttonCancel->setText( tr( "&Cancel" ) );
    buttonCancel->setAutoDefault( TRUE );
	
	QHBoxLayout* hlayout3 = new QHBoxLayout(this,5,5, "hlayout3");
    hlayout3->addWidget(box1);
	hlayout3->addWidget(box2);

    // signals and slots connections
    connect( buttonOk, SIGNAL(clicked()), this, SLOT(quit() ) );
    connect( buttonCancel, SIGNAL(clicked()), this, SLOT(close()));
	connect( buttonApply, SIGNAL(clicked() ), this, SLOT(acceptParams() ) );
	connect( btnWorksheet, SIGNAL(clicked()), this, SLOT(showWorksheet()));

	connect(listBox, SIGNAL(highlighted(int)), this, SLOT(updateTabWindow(int)));
	connect(listBox, SIGNAL(rightButtonClicked(QListBoxItem *, const QPoint &)), this, SLOT(showPopupMenu(QListBoxItem *, const QPoint &)));
	connect(boxPlotType, SIGNAL(activated(int)), this, SLOT(changePlotType(int)));

	QAccel *accel = new QAccel(this);
	accel->connectItem( accel->insertItem( Key_Delete ),
                            this, SLOT(removeSelectedCurve()) );
}

void MyDialog::changePlotType(int plotType)
{
int index=listBox->currentItem();
QString text=listBox->currentText();
const curveLayout cl_old= layout[index];

if ((cl_old.plotType == Graph::Histogram && plotType != 5) ||
	(cl_old.plotType == Graph::VerticalBars && plotType != 3) ||
	(cl_old.plotType == Graph::HorizontalBars && plotType != 4) ||
	(cl_old.plotType == Graph::VectXYXY && plotType != 6) ||
	((cl_old.plotType <= 2 || cl_old.plotType == Graph::Area) && plotType >=3) || 
	(text.contains("(yErr)",TRUE)>0 || text.contains("(xErr)",TRUE)>0))
	{
	QMessageBox::critical(0,tr("QtiPlot - Error"),"This operation is not allowed! Please choose another plot style!");
	setPlotType(index);
	return;
	}

QWidget *w=privateTabWidget->currentPage();
if ( w == errorsPage || plotType >=3)
	return;

clearTabWidget();
switch (plotType)
	{
	case 0:
	privateTabWidget->addTab (linePage, "Line");
	privateTabWidget->showPage(linePage);
	break;

	case 1:
	privateTabWidget->addTab (symbolPage, "Symbol");
	privateTabWidget->showPage(symbolPage);
	break;

	case 2:
	privateTabWidget->addTab (linePage, "Line");
	privateTabWidget->addTab (symbolPage, "Symbol");
	privateTabWidget->showPage(symbolPage);
	break;
	}
acceptParams();
}

void MyDialog::initLinePage()
{
    linePage = new QWidget( privateTabWidget, "linePage" );

	GroupBox3 = new QButtonGroup(2,QGroupBox::Horizontal,tr( "" ),linePage, "GroupBox3" );

	new QLabel( tr( "Connect" ),GroupBox3, "TextLabel1",0 );  

    boxConnect = new QComboBox( FALSE, GroupBox3, "boxConnect" );
	boxConnect->insertItem("No line");
    boxConnect->insertItem("Lines");
    boxConnect->insertItem("Sticks");
    boxConnect->insertItem("Steps");
    boxConnect->insertItem("Dots");
	boxConnect->insertItem("Spline");
	
	new QLabel(tr( "Style" ) , GroupBox3, "TextLabel2",0 );

    boxLineStyle = new QComboBox( FALSE, GroupBox3, "boxLineStyle" );
    boxLineStyle->insertItem("_____");
	boxLineStyle->insertItem("_ _ _");
	boxLineStyle->insertItem(".....");
	boxLineStyle->insertItem("_._._");
	boxLineStyle->insertItem("_.._..");

    new QLabel(tr( "Width" ), GroupBox3, "TextLabel3",0 );  

    boxLineWidth = new QSpinBox( GroupBox3, "boxLineWidth" );
    boxLineWidth->setMinValue( 1 );
	boxLineWidth->setValue( 1 );

    new QLabel( tr( "Color" ), GroupBox3, "TextLabel4",0 );

    boxLineColor = new QComboBox( FALSE, GroupBox3, "boxLineColor" );
    boxLineColor->insertItem( QPixmap(black_xpm), tr( "black" ) );
    boxLineColor->insertItem( QPixmap(red_xpm), tr( "red" ) );
    boxLineColor->insertItem( QPixmap(green_xpm), tr( "green" ) );
    boxLineColor->insertItem( QPixmap(blue_xpm), tr( "blue" ) );
	boxLineColor->insertItem( QPixmap(cyan_xpm), tr( "cyan" ) );
	boxLineColor->insertItem( QPixmap(magenta_xpm), tr( "magenta" ) );
    boxLineColor->insertItem( QPixmap(yellow_xpm), tr( "yellow" ) );
	boxLineColor->insertItem( QPixmap(darkYellow_xpm), tr( "dark yellow" ) );
	boxLineColor->insertItem( QPixmap(darkBlue_xpm), tr( "navy" ) );
	boxLineColor->insertItem( QPixmap(darkMagenta_xpm), tr( "purple" ) );
	boxLineColor->insertItem( QPixmap(darkRed_xpm), tr( "wine" ) );
	boxLineColor->insertItem( QPixmap(darkGreen_xpm), tr( "olive" ) );
	boxLineColor->insertItem( QPixmap(darkCyan_xpm), tr( "dark cyan" ) );
	boxLineColor->insertItem(QPixmap(white_xpm),tr( "white" ) );
	boxLineColor->insertItem( QPixmap(lightGray_xpm), tr( "light gray" ) );
	boxLineColor->insertItem( QPixmap(darkGray_xpm), tr( "dark gray" ) );
	
	GroupBox4 = new QButtonGroup(2,QGroupBox::Horizontal,tr( "" ),linePage, "GroupBox4" );

	boxFill = new QCheckBox( GroupBox4, "boxFill" );
		
	new QLabel( "Fill area under curve",GroupBox4, "TextLabel4_15",0 );
	
	TextLabel4_3=new QLabel(  tr( "Fill color" ),GroupBox4, "TextLabel4_3",0 );  

    boxAreaColor = new QComboBox( FALSE,GroupBox4, "boxAreaColor" );
    boxAreaColor->insertItem( QPixmap(black_xpm), tr( "black" ) );
    boxAreaColor->insertItem( QPixmap(red_xpm), tr( "red" ) );
    boxAreaColor->insertItem( QPixmap(green_xpm), tr( "green" ) );
    boxAreaColor->insertItem( QPixmap(blue_xpm), tr( "blue" ) );
	boxAreaColor->insertItem( QPixmap(cyan_xpm), tr( "cyan" ) );
	boxAreaColor->insertItem( QPixmap(magenta_xpm), tr( "magenta" ) );
    boxAreaColor->insertItem( QPixmap(yellow_xpm), tr( "yellow" ) );
	boxAreaColor->insertItem( QPixmap(darkYellow_xpm), tr( "dark yellow" ) );
	boxAreaColor->insertItem( QPixmap(darkBlue_xpm), tr( "navy" ) );
	boxAreaColor->insertItem( QPixmap(darkMagenta_xpm), tr( "purple" ) );
	boxAreaColor->insertItem( QPixmap(darkRed_xpm), tr( "wine" ) );
	boxAreaColor->insertItem( QPixmap(darkGreen_xpm), tr( "olive" ) );
	boxAreaColor->insertItem( QPixmap(darkCyan_xpm), tr( "dark cyan" ) );
	boxAreaColor->insertItem(QPixmap(white_xpm),tr( "white" ) );
	boxAreaColor->insertItem( QPixmap(lightGray_xpm), tr( "light gray" ) );
	boxAreaColor->insertItem( QPixmap(darkGray_xpm), tr( "dark gray" ) );

	TextLabel4_4=new QLabel(tr( "Pattern" ), GroupBox4, "TextLabel4_4",0);

	boxPattern = new QComboBox( FALSE, GroupBox4, "boxPattern" );
    boxPattern->insertItem(tr( "Solid" ) );
    boxPattern->insertItem(tr( "Horizontal" ) );
    boxPattern->insertItem(tr( "Vertical" ) );
    boxPattern->insertItem(tr( "Cross" ) );
    boxPattern->insertItem(tr( "BDiagonal" ) );
	boxPattern->insertItem(tr( "FDiagonal" ) );
	boxPattern->insertItem(tr( "DiagCross" ) );

	boxPattern->insertItem(tr( "Dense1" ) );
    boxPattern->insertItem(tr( "Dense2" ) );
    boxPattern->insertItem(tr( "Dense3" ) );
    boxPattern->insertItem(tr( "Dense4" ) );
    boxPattern->insertItem(tr( "Dense5" ) );
	boxPattern->insertItem(tr( "Dense6" ) );
	boxPattern->insertItem(tr( "Dense7" ) );	
	
	QHBoxLayout* hlayout2 = new QHBoxLayout(linePage,5,5, "hlayout2");
    hlayout2->addWidget(GroupBox3);
	hlayout2->addWidget(GroupBox4);

	privateTabWidget->insertTab( linePage, tr( "Line" ) );

    //line page connections
	connect(boxLineColor, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxConnect, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxLineStyle, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxAreaColor, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxPattern, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxFill, SIGNAL(toggled(bool)), this, SLOT(showAreaColor(bool)));
	connect(boxFill, SIGNAL(clicked()), this, SLOT(acceptParams()));
}

void MyDialog::initSymbolsPage()
{	
    symbolPage = new QWidget( privateTabWidget, "symbolPage" );

	GroupBox0 = new QButtonGroup(2,QGroupBox::Horizontal,tr( "" ),symbolPage, "GroupBox0" );

    new QLabel(tr( "Pen Color" ), GroupBox0, "TextLabel4_2",0 ); 

    boxSymbolColor = new QComboBox( FALSE, GroupBox0, "boxSymbolColor" );
	boxSymbolColor->insertItem( QPixmap(black_xpm), tr( "black" ) );
    boxSymbolColor->insertItem( QPixmap(red_xpm), tr( "red" ) );
    boxSymbolColor->insertItem( QPixmap(green_xpm), tr( "green" ) );
    boxSymbolColor->insertItem( QPixmap(blue_xpm), tr( "blue" ) );
	boxSymbolColor->insertItem( QPixmap(cyan_xpm), tr( "cyan" ) );
	boxSymbolColor->insertItem( QPixmap(magenta_xpm), tr( "magenta" ) );
    boxSymbolColor->insertItem( QPixmap(yellow_xpm), tr( "yellow" ) );
	boxSymbolColor->insertItem( QPixmap(darkYellow_xpm), tr( "dark yellow" ) );
	boxSymbolColor->insertItem( QPixmap(darkBlue_xpm), tr( "navy" ) );
	boxSymbolColor->insertItem( QPixmap(darkMagenta_xpm), tr( "purple" ) );
	boxSymbolColor->insertItem( QPixmap(darkRed_xpm), tr( "wine" ) );
	boxSymbolColor->insertItem( QPixmap(darkGreen_xpm), tr( "olive" ) );
	boxSymbolColor->insertItem( QPixmap(darkCyan_xpm), tr( "dark cyan" ) );
	boxSymbolColor->insertItem(QPixmap(white_xpm),tr( "white" ) );
	boxSymbolColor->insertItem( QPixmap(lightGray_xpm), tr( "light gray" ) );
	boxSymbolColor->insertItem( QPixmap(darkGray_xpm), tr( "dark gray" ) );
	
	new QLabel( tr( "Fill Color" ), GroupBox0, "TextLabel5_2",0 );

	boxFillColor = new QComboBox( FALSE, GroupBox0, "boxFillColor" );
	boxFillColor->insertItem( QPixmap(black_xpm), tr( "black" ) );
    boxFillColor->insertItem( QPixmap(red_xpm), tr( "red" ) );
    boxFillColor->insertItem( QPixmap(green_xpm), tr( "green" ) );
    boxFillColor->insertItem( QPixmap(blue_xpm), tr( "blue" ) );
	boxFillColor->insertItem( QPixmap(cyan_xpm), tr( "cyan" ) );
	boxFillColor->insertItem( QPixmap(magenta_xpm), tr( "magenta" ) );
    boxFillColor->insertItem( QPixmap(yellow_xpm), tr( "yellow" ) );
	boxFillColor->insertItem( QPixmap(darkYellow_xpm), tr( "dark yellow" ) );
	boxFillColor->insertItem( QPixmap(darkBlue_xpm), tr( "navy" ) );
	boxFillColor->insertItem( QPixmap(darkMagenta_xpm), tr( "purple" ) );
	boxFillColor->insertItem( QPixmap(darkRed_xpm), tr( "wine" ) );
	boxFillColor->insertItem( QPixmap(darkGreen_xpm), tr( "olive" ) );
	boxFillColor->insertItem( QPixmap(darkCyan_xpm), tr( "dark cyan" ) );
	boxFillColor->insertItem(QPixmap(white_xpm),tr( "white" ) );
	boxFillColor->insertItem( QPixmap(lightGray_xpm), tr( "light gray" ) );
	boxFillColor->insertItem( QPixmap(darkGray_xpm), tr( "dark gray" ) );
	
	new QLabel(tr( "Style" ), GroupBox0, "TextLabel2_2",0 );

    boxSymbolStyle = new QComboBox( FALSE, GroupBox0, "boxSymbolStyle" );
	boxSymbolStyle->insertItem("Ellipse" );
    boxSymbolStyle->insertItem("Rectangle");
    boxSymbolStyle->insertItem("Diamond");
    boxSymbolStyle->insertItem("Up Triangle");
    boxSymbolStyle->insertItem("Down Triangle");
    boxSymbolStyle->insertItem("Left Triangle");
    boxSymbolStyle->insertItem("Right Triangle");
    boxSymbolStyle->insertItem("Cross");
    boxSymbolStyle->insertItem("Diagonal Cross");

    new QLabel(tr( "Size" ), GroupBox0, "TextLabel3_2",0 );
	
	boxSymbolSize = new QSpinBox( GroupBox0, "boxSymbolSize" );
    boxSymbolSize->setMinValue( 1 );
	boxSymbolSize->setValue(5);
	
	QHBoxLayout* hlayout0 = new QHBoxLayout(symbolPage,5,25, "hlayout");
    hlayout0->addWidget(GroupBox0);

	privateTabWidget->insertTab(symbolPage, tr( "Symbol" ) );

    //symbol page connections
	connect(boxSymbolColor, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxSymbolStyle, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
	connect(boxFillColor, SIGNAL(activated(int)), this, SLOT(changeCurveLayout(int)));
}

void MyDialog::initErrorsPage()
{
errorsPage = new QWidget(privateTabWidget, "errorsPage" );
	
GroupBox2 = new QButtonGroup(1,QGroupBox::Horizontal,tr( "Direction" ),errorsPage, "GroupBox2" );

plusBox = new QCheckBox( GroupBox2, "plusBox" );
plusBox->setText( tr( "Plus" ) );
plusBox->setChecked( TRUE );

    minusBox = new QCheckBox( GroupBox2, "minusBox" );
    minusBox->setText( tr( "Minus" ) );
    minusBox->setChecked( TRUE );

    xBox = new QCheckBox( GroupBox2, "xBox" ); 
    xBox->setText( tr( "&X Error Bar" ) );

    GroupBox1 = new QButtonGroup( 2,QGroupBox::Horizontal,tr( "Style" ),errorsPage, "GroupBox1" );
   
    new QLabel(tr( "Color" ), GroupBox1, "TextLabel3_3",0 );

    colorBox = new QComboBox( FALSE, GroupBox1, "colorBox" );
	colorBox->insertItem( QPixmap(black_xpm), tr( "black" ) );
    colorBox->insertItem( QPixmap(red_xpm), tr( "red" ) );
    colorBox->insertItem( QPixmap(green_xpm), tr( "green" ) );
    colorBox->insertItem( QPixmap(blue_xpm), tr( "blue" ) );
	colorBox->insertItem( QPixmap(cyan_xpm), tr( "cyan" ) );
	colorBox->insertItem( QPixmap(magenta_xpm), tr( "magenta" ) );
    colorBox->insertItem( QPixmap(yellow_xpm), tr( "yellow" ) );
	colorBox->insertItem( QPixmap(darkYellow_xpm), tr( "dark yellow" ) );
	colorBox->insertItem( QPixmap(darkBlue_xpm), tr( "navy" ) );
	colorBox->insertItem( QPixmap(darkMagenta_xpm), tr( "purple" ) );
	colorBox->insertItem( QPixmap(darkRed_xpm), tr( "wine" ) );
	colorBox->insertItem( QPixmap(darkGreen_xpm), tr( "olive" ) );
	colorBox->insertItem( QPixmap(darkCyan_xpm), tr( "dark cyan" ) );
	colorBox->insertItem(QPixmap(white_xpm),tr( "white" ) );
	colorBox->insertItem( QPixmap(lightGray_xpm), tr( "light gray" ) );
	colorBox->insertItem( QPixmap(darkGray_xpm), tr( "dark gray" ) );
	
	new QLabel(tr( "Line Width" ), GroupBox1, "TextLabel3_3_2",0 );

    widthBox = new QComboBox( FALSE, GroupBox1, "widthBox" );
	widthBox->insertItem( tr( "1" ) );
    widthBox->insertItem( tr( "2" ) );
    widthBox->insertItem( tr( "3" ) );
    widthBox->insertItem( tr( "4" ) );
    widthBox->insertItem( tr( "5" ) );
	widthBox->setEditable (TRUE);
	widthBox->setCurrentItem(1);
	
	new QLabel(tr( "Cap Width" ) , GroupBox1, "TextLabel3_3_2_2",0 );

    capBox = new QComboBox( FALSE, GroupBox1, "capBox" );
	capBox->insertItem( tr( "8" ) );
    capBox->insertItem( tr( "10" ) );
    capBox->insertItem( tr( "12" ) );
    capBox->insertItem( tr( "16" ) );
    capBox->insertItem( tr( "20" ) );
	capBox->setEditable (TRUE);
	capBox->setCurrentItem(0);

    throughBox = new QCheckBox( GroupBox1, "throughBox" );
    throughBox->setText( tr( "Through Symbol" ) );
	
	QHBoxLayout* hlayout = new QHBoxLayout(errorsPage,5,5, "hlayout");
    hlayout->addWidget(GroupBox2);
	hlayout->addWidget(GroupBox1);

	privateTabWidget->insertTab( errorsPage, tr( "Error Bars" ) );

   	connect(colorBox, SIGNAL(activated(int)), this, SLOT(changeErrorBarsColor(int)));
	connect(capBox, SIGNAL(activated(int)), this, SLOT(changeErrorBarsCap(int)));
	connect(widthBox, SIGNAL(activated(int)), this, SLOT(changeErrorBarsWidth(int)));
	connect(xBox, SIGNAL(clicked()), this, SLOT(changeErrorBarsType()));
	connect(plusBox, SIGNAL(clicked()), this, SLOT(changeErrorBarsPlus()));
	connect(minusBox, SIGNAL(clicked()), this, SLOT(changeErrorBarsMinus()));
	connect(throughBox, SIGNAL(clicked()), this, SLOT(changeErrorBarsThrough()));
}

void MyDialog::initHistogramPage()
{
histogramPage = new QWidget(privateTabWidget, "histogramPage" );
	
QHBox *GroupBoxH1 = new QHBox ( histogramPage);
GroupBoxH1->setSpacing (5);
	
automaticBox = new QCheckBox( GroupBoxH1, "automaticBox" );
automaticBox->setText( tr( "Automatic Binning" ) );

buttonStatistics = new QPushButton(GroupBoxH1, "buttonStatistics" );
buttonStatistics->setText( tr( "&Show statistics" ) );

GroupBoxH = new QButtonGroup(2,QGroupBox::Horizontal,tr( "" ),histogramPage, "GroupBoxH" );
   
new QLabel(tr( "Bin Size" ), GroupBoxH, "TextLabel3_3",0 );
binSizeBox = new QLineEdit(GroupBoxH);	

new QLabel(tr( "Begin" ), GroupBoxH, "TextLabel3_3_2",0 );
histogramBeginBox = new QLineEdit(GroupBoxH);	

new QLabel(tr( "End" ) , GroupBoxH, "TextLabel3_3_2_2",0 );
histogramEndBox = new QLineEdit(GroupBoxH);
	
QVBoxLayout* hlayout = new QVBoxLayout(histogramPage,5,5, "hlayout");
hlayout->addWidget(GroupBoxH1);
hlayout->addWidget(GroupBoxH);

connect(automaticBox, SIGNAL(clicked()), this, SLOT(setAutomaticBinning()));
connect(buttonStatistics, SIGNAL(clicked()), this, SLOT(showStatistics() ) );

privateTabWidget->insertTab( histogramPage, tr( "Histogram Data" ) );
}

void MyDialog::initSpacingPage()
{
spacingPage = new QWidget(privateTabWidget, "histogramPage" );

QLabel *textGap=new QLabel(tr( "Gap Between Bars (in %)" ) , spacingPage, "TextLabel_gap",0 );
gapBox=new QSpinBox(0,100,1,spacingPage, "gapBox");

QHBoxLayout* hlayout = new QHBoxLayout(spacingPage,35,25, "hlayout");
hlayout->addWidget(textGap);
hlayout->addWidget(gapBox);

privateTabWidget->insertTab( spacingPage, tr( "Spacing" ) );
}

void MyDialog::initVectPage()
{
vectPage = new QWidget(privateTabWidget, "vectPage" );
QVBox *box1 = new QVBox(vectPage);

QButtonGroup *GroupBox1 = new QButtonGroup( 2,QGroupBox::Horizontal,tr( "" ),box1, "GroupBox1" );
	
new QLabel(tr( "Color" ), GroupBox1, "TextLabel3_3",0 );

    vectColorBox = new QComboBox( FALSE, GroupBox1, "vectColorBox" );
	vectColorBox->insertItem( QPixmap(black_xpm), tr( "black" ) );
    vectColorBox->insertItem( QPixmap(red_xpm), tr( "red" ) );
    vectColorBox->insertItem( QPixmap(green_xpm), tr( "green" ) );
    vectColorBox->insertItem( QPixmap(blue_xpm), tr( "blue" ) );
	vectColorBox->insertItem( QPixmap(cyan_xpm), tr( "cyan" ) );
	vectColorBox->insertItem( QPixmap(magenta_xpm), tr( "magenta" ) );
    vectColorBox->insertItem( QPixmap(yellow_xpm), tr( "yellow" ) );
	vectColorBox->insertItem( QPixmap(darkYellow_xpm), tr( "dark yellow" ) );
	vectColorBox->insertItem( QPixmap(darkBlue_xpm), tr( "navy" ) );
	vectColorBox->insertItem( QPixmap(darkMagenta_xpm), tr( "purple" ) );
	vectColorBox->insertItem( QPixmap(darkRed_xpm), tr( "wine" ) );
	vectColorBox->insertItem( QPixmap(darkGreen_xpm), tr( "olive" ) );
	vectColorBox->insertItem( QPixmap(darkCyan_xpm), tr( "dark cyan" ) );
	vectColorBox->insertItem(QPixmap(white_xpm),tr( "white" ) );
	vectColorBox->insertItem( QPixmap(lightGray_xpm), tr( "light gray" ) );
	vectColorBox->insertItem( QPixmap(darkGray_xpm), tr( "dark gray" ) );


	new QLabel(tr( "Line Width" ), GroupBox1, "TextLabel3_3_2",0 );
    vectWidthBox = new QSpinBox( 1,100,1, GroupBox1, "vectWidthBox" );

GroupBox2 = new QButtonGroup( 2,QGroupBox::Horizontal,tr( "Arrowheads" ),box1, "GroupBox2" );

new QLabel(tr( "Length" ), GroupBox2, "TextLabel3_3",0 );
headLengthBox = new QSpinBox( 0,100,1, GroupBox2, "headLengthBox" );
	
new QLabel(tr( "Angle" ), GroupBox2, "TextLabel3_3_2",0 );
headAngleBox = new QSpinBox( 0,85,5, GroupBox2, "headAngleBox" );

filledHeadBox = new QCheckBox(GroupBox2, "filledBox" ); 
filledHeadBox->setText( tr( "&Filled" ) );

QVBox *box2 = new QVBox(vectPage);
GroupBox3 = new QButtonGroup( 2,QGroupBox::Horizontal,tr( "End Point" ),box2, "GroupBox2" );

new QLabel(tr( "X End" ), GroupBox3, "TextLabel3_3",0 );
xEndBox = new QComboBox( FALSE, GroupBox3, "headLengthBox" );
	
new QLabel(tr( "Y End" ), GroupBox3, "TextLabel3_3_2",0 );
yEndBox = new  QComboBox( FALSE, GroupBox3, "headAngleBox" );

QHBoxLayout* hlayout = new QHBoxLayout(vectPage,5,5, "hlayout");
hlayout->addWidget(box1);
hlayout->addWidget(box2);

privateTabWidget->insertTab( vectPage, tr( "Vector" ) );
}

void MyDialog::showStatistics()
{
QString text=listBox->currentText();
QStringList t=QStringList::split(": ", text, false);
QStringList list=QStringList::split (",", t[1], FALSE );
text=t[0] + "_" + list[1].remove("(Y)");
	
emit showHistogramStats(text, listBox->currentItem());
close();
}

void MyDialog::changeCurveLayout(int)
{
if (privateTabWidget->currentPage()==linePage || privateTabWidget->currentPage()==symbolPage)
	{		
	int index=listBox->currentItem();
	curveLayout cl=getLayout();
	layout[index]=cl;
	emit setLayout(index,cl);
	}
}

void MyDialog::showPopupMenu(QListBoxItem *it, const QPoint &point)
{
selectedCurve=listBox->index (it);
	
QPopupMenu contextMenu(this);
contextMenu.insertItem("&Delete", this, SLOT(removeCurve()));	
contextMenu.exec(point);
}

void MyDialog::removeCurve()
{
emit curveToRemove(selectedCurve);
listBox->removeItem (selectedCurve);
}

void MyDialog::removeSelectedCurve()
{
int curve=listBox->currentItem ();
emit curveToRemove(curve);
listBox->removeItem (curve);
}

void MyDialog::changeErrorBarsPlus()
{
QString text=listBox->currentText();
for (int i=0;i<(int)errorBars.count();i++)
	{
	if (errorBars[i]==text)
		{
		QStringList curve=QStringList::split ("\t",errorBarsLayout[i],FALSE);
		curve[5]=QString::number(plusBox->isChecked());
		QString s=curve.join("\t");
		errorBarsLayout[i]=s;
		break;
		}
	}
	
int j=colorBox->currentItem();

emit updateErrorBars(listBox->currentItem(),xBox->isChecked(),widthBox->currentText().toInt(),
					 capBox->currentText().toInt(),color(j),plusBox->isChecked(),minusBox->isChecked(),
					 throughBox->isChecked());
}

void MyDialog::changeErrorBarsMinus()
{
QString text=listBox->currentText();
for (int i=0;i<(int)errorBars.count();i++)
	{
	if (errorBars[i]==text)
		{
		QStringList curve=QStringList::split ("\t",errorBarsLayout[i],FALSE);
		curve[6]=QString::number(minusBox->isChecked());
		QString s=curve.join("\t");
		errorBarsLayout[i]=s;
		break;
		}
	}
	
int j=colorBox->currentItem();

emit updateErrorBars(listBox->currentItem(),xBox->isChecked(),widthBox->currentText().toInt(),
					 capBox->currentText().toInt(),color(j),plusBox->isChecked(),minusBox->isChecked(),
					 throughBox->isChecked());
}

void MyDialog::changeErrorBarsThrough()
{
QString text=listBox->currentText();
for (int i=0;i<(int)errorBars.count();i++)
	{
	if (errorBars[i]==text)
		{
		QStringList curve=QStringList::split ("\t",errorBarsLayout[i],FALSE);
		curve[4]=QString::number(throughBox->isChecked());
		QString s=curve.join("\t");
		errorBarsLayout[i]=s;
		break;
		}
	}
	
int j=colorBox->currentItem();

emit updateErrorBars(listBox->currentItem(),xBox->isChecked(),widthBox->currentText().toInt(),
					 capBox->currentText().toInt(),color(j),plusBox->isChecked(),minusBox->isChecked(),
					 throughBox->isChecked());
}

void MyDialog::changeErrorBarsType()
{
QString text=listBox->currentText();
int item=listBox->currentItem();
	
QColor color;
for (int i=0;i<(int)errorBars.count();i++)
	{
	if (errorBars[i]==text)
		{
		QStringList curve=QStringList::split ("\t",errorBarsLayout[i],FALSE);
		curve[0]=QString::number(xBox->isChecked());
		color=QColor(curve[3]);
		QString s=curve.join("\t");
		errorBarsLayout[i]=s;
		break;
		}
	}

emit updateErrorBars(item, xBox->isChecked(), widthBox->currentText().toInt(),
					 capBox->currentText().toInt(), color, plusBox->isChecked(), minusBox->isChecked(),
					 throughBox->isChecked());
}

void MyDialog::changeErrorBarsColor(int i)
{
QColor color=this->color(i);
QString text=listBox->currentText();
for (int j=0;j<(int)errorBars.count();j++)
	{
	if (errorBars[j]==text)
		{
		QStringList curve=QStringList::split ("\t",errorBarsLayout[j],FALSE);

		curve[3]=color.name();
		QString s=curve.join("\t");
		errorBarsLayout[j]=s;
		break;
		}
	}
	
emit updateErrorBars(listBox->currentItem(),xBox->isChecked(),widthBox->currentText().toInt(),
					 capBox->currentText().toInt(),color,plusBox->isChecked(),minusBox->isChecked(),
					 throughBox->isChecked());
}

void MyDialog::changeErrorBarsWidth(int)
{
int j=colorBox->currentItem();
	
QString text=listBox->currentText();
for (int i=0;i<(int)errorBars.count();i++)
	{
	if (errorBars[i]==text)
		{
		QStringList curve=QStringList::split ("\t",errorBarsLayout[i],FALSE);

		curve[1]=widthBox->currentText();
		QString s=curve.join("\t");
		errorBarsLayout[i]=s;
		break;
		}
	}
	
emit updateErrorBars(listBox->currentItem(),xBox->isChecked(),widthBox->currentText().toInt(),
					 capBox->currentText().toInt(),color(j),plusBox->isChecked(),minusBox->isChecked(),
					 throughBox->isChecked());
}

void MyDialog::changeErrorBarsCap(int)
{
QString text=listBox->currentText();
for (int i=0;i<(int)errorBars.count();i++)
	{
	if (errorBars[i]==text)
		{
		QStringList curve=QStringList::split ("\t",errorBarsLayout[i],FALSE);

		curve[2]=capBox->currentText();
		QString s=curve.join("\t");
		errorBarsLayout[i]=s;
		break;
		}
	}
	
int j=colorBox->currentItem();
emit updateErrorBars(listBox->currentItem(),xBox->isChecked(),widthBox->currentText().toInt(),
					 capBox->currentText().toInt(),color(j),plusBox->isChecked(),minusBox->isChecked(),
					 throughBox->isChecked());
}

void MyDialog::showAreaColor(bool show)
{
if (show) 
	{
	boxAreaColor->setEnabled(TRUE);
	TextLabel4_3->setEnabled(TRUE);
	TextLabel4_4->setEnabled(TRUE);
	boxPattern->setEnabled(TRUE);
	}
else 
	{
	boxAreaColor->setDisabled(TRUE);
	TextLabel4_3->setDisabled(TRUE);
	TextLabel4_4->setDisabled(TRUE);
	boxPattern->setDisabled(TRUE);
	}
}

void MyDialog::updateTabWindow(int curveIndex)
{
clearTabWidget();
setPlotType(curveIndex);
QString text=listBox->currentText();
if (text.contains("(yErr)",TRUE)>0 || text.contains("(xErr)",TRUE)>0)
	{
	privateTabWidget->addTab (errorsPage, "Error Bars");
	privateTabWidget->showPage(errorsPage);
	}
else
	{
	QString text=boxPlotType->currentText();
	if (text == "Line")
		{
		boxConnect->setEnabled(true);
		privateTabWidget->addTab (linePage, "Line");
		privateTabWidget->showPage(linePage);
		}
	else if (text == "Scatter")
		{
		boxConnect->setEnabled(true);
		privateTabWidget->addTab (symbolPage, "Symbol");
		privateTabWidget->showPage(symbolPage);
		}
	else if (text == "Line + Symbol")
		{
		boxConnect->setEnabled(true);
		privateTabWidget->addTab (linePage, "Line");
		privateTabWidget->addTab (symbolPage, "Symbol");
		privateTabWidget->showPage(symbolPage);
		}
	else if (text == "Vertical Bars" || text == "Horizontal Bars" || text == "Histogram")
		{
		boxConnect->setEnabled(false);
		privateTabWidget->addTab (linePage, "Pattern");
		privateTabWidget->addTab (spacingPage, "Spacing");

		if (text == "Histogram")
			{
			privateTabWidget->addTab (histogramPage, "Histogram Data");	
			privateTabWidget->showPage(histogramPage);
			}
		else
			privateTabWidget->showPage(linePage);
		}
	else if (text == "Vector XYXY")
		{
		boxConnect->setEnabled(true);
		privateTabWidget->addTab (linePage, "Line");
		privateTabWidget->addTab (vectPage, "Vector");
		privateTabWidget->showPage(vectPage);
		}
	}

setCurveParams(curveIndex);
}

void MyDialog::clearTabWidget()
{
privateTabWidget->removePage(linePage);
privateTabWidget->removePage(symbolPage);
privateTabWidget->removePage(errorsPage);
privateTabWidget->removePage(histogramPage);
privateTabWidget->removePage(spacingPage);
privateTabWidget->removePage(vectPage);
}

void MyDialog::quit()
{
if (acceptParams())
	close();
}

void MyDialog::showWorksheet()
{
int index=listBox->currentItem();
QString text=listBox->text(index);
if (text.contains("=")>0)
	emit createWorksheet(text);
else
	{
	QStringList t=QStringList::split(": ", text, false);
	emit worksheet(t[0]+"_");
	}
close();
}

curveLayout MyDialog::curveParams(int index)
{
// return the layout parameters of the curve at index
curveLayout cL=layout[index];
return cL;
}

void MyDialog::setAllCurvesLayout(const QArray<curveLayout>& l)
{
layout=l;
}

void MyDialog::setPlotType(int index)
{
int size=listBox->count();
if (size>0)
	{
	QString text = listBox->currentText();
	if (text.contains("(yErr)",TRUE)>0 || text.contains("(xErr)",TRUE)>0)
		{
		boxPlotType->setCurrentText("Error Bars");
		boxPlotType->setCurrentItem(7);
		return;
		}

	curveLayout cl=layout[index];
	if (cl.plotType == Graph::Line || cl.plotType == Graph::Steps || cl.plotType == Graph::Area)
		boxPlotType->setCurrentItem(0);
	else if (cl.plotType == Graph::Scatter)
		boxPlotType->setCurrentItem(1);
	else if (cl.plotType == Graph::LineSymbols || cl.plotType == Graph::VerticalDropLines || cl.plotType == Graph::Spline)
		boxPlotType->setCurrentItem(2);
	else if (cl.plotType == Graph::VerticalBars)
		boxPlotType->setCurrentItem(3);
	else if (cl.plotType == Graph::HorizontalBars)
		boxPlotType->setCurrentItem(4);
	else if (cl.plotType == Graph::Histogram)
		boxPlotType->setCurrentItem(5);
	else if (cl.plotType == Graph::VectXYXY)
		boxPlotType->setCurrentItem(6);
	}
}

void MyDialog::setCurveParams(int index)
{
//connected to the listBox highlighted signal and 
//tells the dialog object to show the current curve parameters
int size=listBox->count();
QString text=listBox->currentText();
if (size>0)
{
if (text.contains("(yErr)",TRUE)==0 && text.contains("(xErr)",TRUE)==0)
	{
	curveLayout cl=layout[index];

	//line page
	boxConnect->setCurrentItem(cl.connectType);
	boxLineStyle->setCurrentItem(cl.lStyle);
	boxLineWidth->setValue(cl.lWidth);
		
	boxFill->setChecked(cl.filledArea);
	boxAreaColor->setCurrentItem(cl.aCol);
	boxAreaColor->setEnabled(cl.filledArea);
	boxPattern->setCurrentItem(cl.aStyle);
	boxPattern->setEnabled(cl.filledArea);
	
	//symbol page
	boxSymbolSize->setValue(cl.sSize);
	boxSymbolStyle->setCurrentItem(cl.sType);
	boxSymbolColor->setCurrentItem(cl.symCol);
	boxFillColor->setCurrentItem(cl.fillCol);
	boxLineColor->setCurrentItem(cl.lCol);
		
	//spacing page
	if (cl.plotType == Graph::VerticalBars || cl.plotType == Graph::HorizontalBars || 
		cl.plotType == Graph::Histogram)
		gapBox->setValue(cl.barGap);

	//Histogram page
	if (cl.plotType == Graph::Histogram)
		{
		automaticBox->setChecked(cl.autoBinning);
		binSizeBox->setText(QString::number(cl.binSize));
		histogramBeginBox->setText(QString::number(cl.histBegin));
		histogramEndBox->setText(QString::number(cl.histEnd));
		setAutomaticBinning();
		}
	
	//Vector page		
	if (cl.plotType == Graph::VectXYXY)
		{
		 vectColorBox->setCurrentItem(cl.vectColor);
		 vectWidthBox->setValue(cl.vectWidth);
		 headLengthBox->setValue(cl.headLength);
		 headAngleBox->setValue(cl.headAngle);
		 filledHeadBox->setChecked(cl.filledHead);
		 updateEndPointColumns(text);
		}
	}
	else
	{
	for (int i=0;i<(int)errorBars.count();i++)
		{
		if (errorBars[i]==text)
			{
			QStringList curve=QStringList::split ("\t",errorBarsLayout[i],FALSE);
			bool ok,plus=TRUE,minus=TRUE,through=TRUE,xErrors=FALSE;
	
			if (curve[0]=="1") xErrors=TRUE;
			xBox->setChecked(xErrors);

			widthBox->setEditText(curve[1]);
			capBox->setEditText(curve[2]);
		
			setErrorBarsColor(QColor(curve[3]));

			int aux=curve[4].toInt(&ok);
			if (aux==0) through=FALSE;
			throughBox->setChecked(through);

			aux=curve[5].toInt(&ok);
			if (aux==0) plus=FALSE;
			plusBox->setChecked(plus);

			aux=curve[6].toInt(&ok);
			if (aux==0) minus=FALSE;
			minusBox->setChecked(minus);
			}
		}
	}
}
}

void MyDialog::updateEndPointColumns(const QString& text)
{
QStringList cols=QStringList::split(",", text, false);
QStringList aux=QStringList::split(":", cols[0], false);
QString table=aux[0];	
QStringList list;

for (int i=0; i<(int)columnNames.count(); i++)	
	{
	QString s=columnNames[i];
	if (s.contains(table)>0)
			list<<s;	
	}
xEndBox->clear();
xEndBox->insertStringList(list);
xEndBox->setCurrentText(table + "_" + cols[2].remove("(X)"));
	
yEndBox->clear();
yEndBox->insertStringList(list);	
yEndBox->setCurrentText(table + "_" + cols[3].remove("(Y)"));	
}

curveLayout MyDialog::getLayout()
{
curveLayout cl;
cl.connectType=boxConnect->currentItem();
cl.lStyle=boxLineStyle->currentItem();
cl.lWidth=boxLineWidth->value();

if (boxFill->isChecked()) 
	cl.filledArea=1;
else 
	cl.filledArea=0;

cl.aCol=boxAreaColor->currentItem();
cl.aStyle=boxPattern->currentItem();

int style = boxPlotType->currentItem();
if (style == 6)
	cl.plotType=Graph::VectXYXY;
else if (style == 5)
	cl.plotType=Graph::Histogram;
else if (style == 4)
	cl.plotType=Graph::HorizontalBars;
else
	cl.plotType=style;

cl.sSize=boxSymbolSize->value();
cl.sType=boxSymbolStyle->currentItem();
cl.symCol=boxSymbolColor->currentItem();
cl.fillCol=boxFillColor->currentItem();
cl.lCol=boxLineColor->currentItem();

cl.barGap=gapBox->value();

if (automaticBox->isChecked())
	cl.autoBinning=1;
else
	cl.autoBinning=0;

cl.binSize=binSizeBox->text().toDouble();
cl.histBegin=histogramBeginBox->text().toDouble();
cl.histEnd=histogramEndBox->text().toDouble();

//vect page
if (cl.plotType==Graph::VectXYXY)
	{
	cl.vectColor=vectColorBox->currentItem();
	cl.vectWidth=vectWidthBox->value();
	cl.headLength=headLengthBox->value();
	cl.headAngle=headAngleBox->value();
	cl.filledHead=filledHeadBox->isChecked();
	}
return cl;
}

bool MyDialog::acceptParams()
{
int index=listBox->currentItem();

if (privateTabWidget->currentPage()==linePage || privateTabWidget->currentPage()==symbolPage)
	{// define actual curve layout and save it to global array "layout"
	curveLayout cl=getLayout();
	emit setLayout(index,cl);
	layout[index]=cl;
	}

if (privateTabWidget->currentPage()==histogramPage)
	{
	QString text=listBox->currentText();
	QStringList t=QStringList::split(": ", text, false);
	QStringList list=QStringList::split (",", t[1], FALSE );
	text=t[0] + "_" + list[1].remove("(Y)");
	bool accept=validInput();
	if (accept)
		{
		emit updateHistogram(text, listBox->currentItem(), automaticBox->isChecked(),binSizeBox->text().toDouble(), 
						 histogramBeginBox->text().toDouble(), histogramEndBox->text().toDouble());
		
		curveLayout cl=getLayout();
		layout[index]=cl;
		}
	return accept;
	}

if (privateTabWidget->currentPage()==spacingPage)
	{
	emit updateBarsGap(listBox->currentItem(),gapBox->value());
	return true;
	}
	
if (privateTabWidget->currentPage()==vectPage)
	{
	int index=listBox->currentItem();
	emit updateVectorsLayout(index, vectColorBox->currentItem(), vectWidthBox->value(), 
									 headLengthBox->value(), headAngleBox->value(), filledHeadBox->isChecked(),
									 xEndBox->currentText(), yEndBox->currentText());
	
	QString text=listBox->currentText();
	QStringList t=QStringList::split(": ", text, false);
	QString table = t[0];
		
	QStringList cols=QStringList::split(",", t[1], false);
	QString xEndCol=xEndBox->currentText().remove(table + "_") + "(X)";
	QString yEndCol=yEndBox->currentText().remove(table + "_") + "(Y)";
	if (cols[2] != xEndCol || cols[3] != yEndCol)
		{
		cols[2] = xEndCol;
		cols[3] = yEndCol;
		text=table + ": " + cols.join(",");
		listBox->changeItem (text, index);
		}
		
	curveLayout cl=getLayout();
	layout[index]=cl;
	return true;
	}
	
if (privateTabWidget->currentPage() == errorsPage)
	{
	int j=colorBox->currentItem();
	emit updateErrorBars(listBox->currentItem(), xBox->isChecked(), widthBox->currentText().toInt(),
					 capBox->currentText().toInt(), color(j), plusBox->isChecked(), minusBox->isChecked(),
					 throughBox->isChecked());
	}
return true;
}

QColor MyDialog::color(int item)
{
QColor c;
switch (item)
	{
case 0:
	c=QColor(black);
break;

case 1:
	c=QColor(red);
break;

case 2:
	c=QColor(green);
break;

case 3:
	c=QColor(blue);
break;

case 4:
	c=QColor(cyan);
break;

case 5:
	c=QColor(magenta);
break;

case 6:
	c=QColor(yellow);
break;

case 7:
	c=QColor(darkYellow);
break;

case 8:
	c=QColor(darkBlue);
break;

case 9:
	c=QColor(darkMagenta);
break;

case 10:
	c=QColor(darkRed);
break;

case 11:
	c=QColor(darkGreen);
break;

case 12:
	c=QColor(darkCyan);
break;

case 13:
	c=QColor(white);
break;

case 14:
	c=QColor(lightGray);
break;

case 15:
	c=QColor(darkGray);
break;
	}
return c;
}

void MyDialog::setErrorBarsColor(QColor c)
{
if (c==QColor(black)) colorBox->setCurrentItem(0);
else if (c==QColor(red)) colorBox->setCurrentItem(1);
else if (c==QColor(green)) colorBox->setCurrentItem(2);
else if (c==QColor(blue)) colorBox->setCurrentItem(3);
else if (c==QColor(cyan)) colorBox->setCurrentItem(4);
else if (c==QColor(magenta)) colorBox->setCurrentItem(5);
else if (c==QColor(yellow)) colorBox->setCurrentItem(6);
else if (c==QColor(darkYellow)) colorBox->setCurrentItem(7);
else if (c==QColor(darkBlue)) colorBox->setCurrentItem(8);
else if (c==QColor(darkMagenta)) colorBox->setCurrentItem(9);
else if (c==QColor(darkRed)) colorBox->setCurrentItem(10);
else if (c==QColor(darkGreen)) colorBox->setCurrentItem(11);
else if (c==QColor(darkCyan)) colorBox->setCurrentItem(12);
else if (c==QColor(white)) colorBox->setCurrentItem(13);
else if (c==QColor(lightGray)) colorBox->setCurrentItem(14);
else if (c==QColor(darkGray)) colorBox->setCurrentItem(15);
}

void MyDialog::insertCurvesList(const QStringList& names)
{
listBox->clear();
QStringList newNames;
int i;
for (i=0;i<(int)names.count();i++)
	{
	QString s=names[i];
	int pos=s.find("_",0);
	if (pos>0)
		{
		QString table=s.left(pos);	
		QString cols=s.right(s.length()-pos-1);			
		newNames<<table+": "+cols.remove(table+"_",TRUE);
		}
	else
		newNames<<s;
	}
listBox->insertStringList(newNames,0);

for (i=0;i<(int)names.count();i++)
	{
	if (names[i].contains("(yErr)",TRUE)>0 || names[i].contains("(xErr)",TRUE)>0)
		errorBars<<newNames[i];
	}
}

void MyDialog::setErrorBarsLayout(QStringList err)
{
errorBarsLayout=err;
}

void MyDialog::setSelectedCurve(int index)
{
listBox->setSelected (index,TRUE);
}

void MyDialog::setAutomaticBinning()
{
GroupBoxH->setEnabled(!automaticBox->isChecked());
}

bool MyDialog::validInput()
{
QString from=histogramBeginBox->text();
QString to=histogramEndBox->text();
QString step=binSizeBox->text();
QRegExp nonDigit("\\D");
		
if (histogramBeginBox->text().isEmpty())
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid start limit!");
	histogramBeginBox->setFocus();
	return FALSE;
	}
		
if (histogramEndBox->text().isEmpty())
		{
		QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid end limit!");
		histogramEndBox->setFocus();
		return FALSE;
		}
		
if (binSizeBox->text().isEmpty())
		{
		QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter a valid bin size value!");
		binSizeBox->setFocus();
		return FALSE;
		}	
		
from=from.remove(".");
to=to.remove(".");
step=step.remove(".");

int pos=from.find("-",0);
if(pos==0)
		from=from.replace(pos,1,"");

pos=to.find("-",0);
if(pos==0)
	to=to.replace(pos,1,"");
	
double start,end, stp;
bool error=FALSE;	
if (from.contains(nonDigit) >0)
		{
		try
			{
			MathUtils::Parser parser;

			if (from.contains("pi")>0)
				parser.AddConst("pi", M_PI);
			if (from.contains("Pi")>0)
				parser.AddConst("Pi", M_PI);
			if (from.contains("PI")>0)
				parser.AddConst("PI", M_PI);
	
			parser.SetFormula((histogramBeginBox->text()).ascii());
			start=parser.Calc();
			}
		catch(MathUtils::ParserException &e)
			{
			QMessageBox::critical(0,"QtiPlot - Start limit error",e.what());
			histogramBeginBox->setFocus();
			error=TRUE;
			return FALSE;
			}	
		}
		else
			start=histogramBeginBox->text().toDouble();
		
		if (to.contains(nonDigit) >0)
			{
			try
				{
				MathUtils::Parser parser;

				if (to.contains("pi")>0)
						parser.AddConst("pi", M_PI);
				if (to.contains("Pi")>0)
						parser.AddConst("Pi", M_PI);
				if (to.contains("PI")>0)
						parser.AddConst("PI", M_PI);
	
				parser.SetFormula((histogramEndBox->text()).ascii());
				end=parser.Calc();
				}
			catch(MathUtils::ParserException &e)
				{
				QMessageBox::critical(0,"QtiPlot - End limit error",e.what());
				histogramEndBox->setFocus();
				error=TRUE;
				return FALSE;
				}	
			}
		else
			end=histogramEndBox->text().toDouble();

		if (start>=end)
			{
			QMessageBox::critical(0,"QtiPlot - Input error",
					"Please enter limits that satisfy: begin < end!");
			histogramEndBox->setFocus();
			return FALSE;
			}
			
		if (step.contains(nonDigit) >0)
		{
		try
			{
			MathUtils::Parser parser;

			if (step.contains("pi")>0)
				parser.AddConst("pi", M_PI);
			if (step.contains("Pi")>0)
				parser.AddConst("Pi", M_PI);
			if (step.contains("PI")>0)
				parser.AddConst("PI", M_PI);
	
			parser.SetFormula((binSizeBox->text()).ascii());
			stp=parser.Calc();
			}
		catch(MathUtils::ParserException &e)
			{
			QMessageBox::critical(0,"QtiPlot - Bin size input error",e.what());
			binSizeBox->setFocus();
			error=TRUE;
			return FALSE;
			}	
		}
		else
			stp=binSizeBox->text().toDouble();
		
		if (stp <=0)
			{
			QMessageBox::critical(0,"QtiPlot - Bin size input error","Please enter a positive bin size value!");
			binSizeBox->setFocus();
			return FALSE;	
			}

return TRUE;
}

MyDialog::~MyDialog()
{
}
