#include "surfaceDialog.h"
#include "parser/muParser.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qlayout.h>
#include <qbuttongroup.h>
#include <qlineedit.h>
#include <qcombobox.h>
#include <qmessagebox.h>
#include <qregexp.h> 

#ifndef M_PI
#define M_PI	3.141592653589793238462643 
#endif

sDialog::sDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "sDialog" );
	setCaption(tr("QtiPlot - Define surface plot"));
    setMinimumSize( QSize( 310, 140 ) );
	setMaximumSize( QSize( 310, 140 ) );
    setMouseTracking( TRUE );
    setSizeGripEnabled( FALSE );
	
	GroupBox1 = new QButtonGroup( 2,QGroupBox::Horizontal,tr(""),this,"GroupBox1" );
	GroupBox1->setFlat (TRUE);
	GroupBox1->setLineWidth (0);

	new QLabel( tr("f(x,y)="), GroupBox1, "TextLabel2",0 );
	boxFunction = new QComboBox(GroupBox1, "boxFunction" );
	boxFunction->setFixedWidth(250);
	boxFunction->setEditable(TRUE);

	GroupBox5 = new QButtonGroup(3,QGroupBox::Horizontal,tr(""),this,"GroupBox5" );
	GroupBox5->setFlat (TRUE);
	GroupBox5->setLineWidth (0);

	GroupBox3 = new QButtonGroup(1,QGroupBox::Horizontal,tr("X - axis"),GroupBox5,"GroupBox3" );
	
	new QLabel( tr("From"), GroupBox3, "TextLabel3",0 );
	boxXFrom = new QLineEdit(GroupBox3, "boxPoints" );
	boxXFrom->setText(tr("-1"));

	new QLabel( tr("To"), GroupBox3, "TextLabel5",0 );
	boxXTo = new QLineEdit(GroupBox3, "boxOrder" );
	boxXTo->setText(tr("1"));

	GroupBox4 = new QButtonGroup(1,QGroupBox::Horizontal,tr("Y - axis"),GroupBox5,"GroupBox4" );
	
	new QLabel( tr("From"), GroupBox4, "TextLabel33",0 );
	boxYFrom = new QLineEdit(GroupBox4, "boxPoints" );
	boxYFrom->setText(tr("-1"));

	new QLabel( tr("To"), GroupBox4, "TextLabel5",0 );
	boxYTo = new QLineEdit(GroupBox4, "boxYto" );
	boxYTo->setText(tr("1"));

	GroupBox6 = new QButtonGroup(1,QGroupBox::Horizontal,tr("Z - axis"),GroupBox5,"GroupBox4" );
	
	new QLabel( tr("From"), GroupBox6, "TextLabel35",0 );
	boxZFrom = new QLineEdit(GroupBox6, "boxZFrom" );
	boxZFrom->setText(tr("-1"));

	new QLabel( tr("To"), GroupBox6, "TextLabel5",0 );
	boxZTo = new QLineEdit(GroupBox6, "boxZto" );
	boxZTo->setText(tr("1"));

	GroupBox2 = new QButtonGroup(3,QGroupBox::Horizontal,tr(""),this,"GroupBox2" );
	GroupBox2->setFlat (TRUE);
	GroupBox2->setLineWidth (0);

	buttonClear = new QPushButton(GroupBox2, "buttonClear" );
    buttonClear->setText( tr( "Clear &list" ) );
	
	buttonOk = new QPushButton(GroupBox2, "buttonOk" );
    buttonOk->setText( tr( "&OK" ) );
    buttonOk->setDefault( TRUE );
   
    buttonCancel = new QPushButton(GroupBox2, "buttonCancel" );
    buttonCancel->setText( tr( "&Cancel" ) );
	
	QVBoxLayout* hlayout = new QVBoxLayout(this,5,5, "hlayout");
    hlayout->addWidget(GroupBox1);
	hlayout->addWidget(GroupBox5);
	hlayout->addWidget(GroupBox2);
   
    // signals and slots connections
	connect( buttonClear, SIGNAL( clicked() ), this, SLOT(clearList() ) );
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );

	setFocusProxy(boxFunction);
}

void sDialog::clearList()
{
boxFunction->clear();
emit clearFunctionsList();
}

void sDialog::accept()
{
QString Xfrom=boxXFrom->text();
QString Xto=boxXTo->text();
QString Yfrom=boxYFrom->text();
QString Yto=boxYTo->text();
QString Zfrom=boxZFrom->text();
QString Zto=boxZTo->text();
	
QRegExp nonDigit("\\D");

Xfrom=Xfrom.remove(".");
Xto=Xto.remove(".");
Yfrom=Yfrom.remove(".");
Yto=Yto.remove(".");
Zfrom=Zfrom.remove(".");
Zto=Zto.remove(".");

int pos=Xfrom.find("-",0);
if(pos==0)
	Xfrom=Xfrom.replace(pos,1,"");

pos=Xto.find("-",0);
if(pos==0)
	Xto=Xto.replace(pos,1,"");

pos=Yfrom.find("-",0);
if(pos==0)
	Yfrom=Yfrom.replace(pos,1,"");

pos=Yto.find("-",0);
if(pos==0)
	Yto=Yto.replace(pos,1,"");

pos=Zfrom.find("-",0);
if(pos==0)
	Zfrom=Zfrom.replace(pos,1,"");

pos=Zto.find("-",0);
if(pos==0)
	Zto=Zto.replace(pos,1,"");

if (boxFunction->currentText().isEmpty()
	|| Xfrom.isEmpty() 
	|| Xto.isEmpty() 
    || Yfrom.isEmpty()
	|| Yto.isEmpty() 
	|| Zfrom.isEmpty()
	|| Zto.isEmpty())
{
QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter valid non-empty input parameters!");
return;
}

double fromX, toX, fromY,toY, fromZ,toZ;
if (Xfrom.contains(nonDigit) >0)
	{
	try
		{
		MathUtils::Parser parser;

		if (Xfrom.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (Xfrom.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (Xfrom.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula((boxXFrom->text()).ascii());
		fromX=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Start limit error",e.what());
		boxXFrom->setFocus();
		return;
		}	
	}
else
	fromX=boxXFrom->text().toDouble();

if (Xto.contains(nonDigit) >0)
	{
	try
		{
		MathUtils::Parser parser;

		if (Xto.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (Xto.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (Xto.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula((boxXTo->text()).ascii());
		toX=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - End limit error",e.what());
		boxXTo->setFocus();
		return;
		}	
	}
else
	toX=boxXTo->text().toDouble();

if (Yfrom.contains(nonDigit) >0)
	{
	try
		{
		MathUtils::Parser parser;

		if (Yfrom.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (Yfrom.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (Yfrom.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula((boxYFrom->text()).ascii());
		fromY=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Start limit error",e.what());
		boxYFrom->setFocus();
		return;
		}	
	}
else
	fromY=boxYFrom->text().toDouble();

if (Yto.contains(nonDigit) >0)
	{
	try
		{
		MathUtils::Parser parser;

		if (Yto.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (Yto.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (Yto.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula((boxYTo->text()).ascii());
		toY=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - End limit error",e.what());
		boxYTo->setFocus();
		return;
		}	
	}
else
	toY=boxYTo->text().toDouble();

if (Zfrom.contains(nonDigit) >0)
	{
	try
		{
		MathUtils::Parser parser;

		if (Zfrom.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (Zfrom.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (Zfrom.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula((boxZFrom->text()).ascii());
		fromZ=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Start limit error",e.what());
		boxZFrom->setFocus();
		return;
		}	
	}
else
	fromZ=boxZFrom->text().toDouble();

if (Zto.contains(nonDigit) >0)
	{
	try
		{
		MathUtils::Parser parser;

		if (Zto.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (Zto.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (Zto.contains("PI")>0)
			parser.AddConst("PI", M_PI);
	
		parser.SetFormula((boxZTo->text()).ascii());
		toZ=parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - End limit error",e.what());
		boxZTo->setFocus();
		return;
		}	
	}
else
	toZ=boxZTo->text().toDouble();

if (fromX >= toX)
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter X limits that satisfy: from < end!");
	boxXTo->setFocus();
	return;
	}
	
if (fromY >= toY)
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter Y limits that satisfy: from < end!");
	boxYTo->setFocus();
	return;
	}
	
if (fromZ >= toZ)
	{
	QMessageBox::critical(0,"QtiPlot - Input error",
				"Please enter Z limits that satisfy: from < end!");
	boxZTo->setFocus();
	return;
	}

	MathUtils::Parser parser;
	double x,y;
    
	QString formula=boxFunction->currentText();
	bool error=FALSE;
	try
		{
		parser.AddVar("x", &x);	
		parser.AddVar("y", &y);
			
		if (formula.contains("pi")>0)
			parser.AddConst("pi", M_PI);
		if (formula.contains("Pi")>0)
			parser.AddConst("Pi", M_PI);
		if (formula.contains("PI")>0)
			parser.AddConst("PI", M_PI);
		
		parser.SetFormula(formula.ascii());
		
		x=fromX;y=fromY;
		parser.Calc();
		x=toX; y=toY;
		parser.Calc();
		}
	catch(MathUtils::ParserException &e)
		{
		QMessageBox::critical(0,"QtiPlot - Input function error",e.what());
		boxFunction->setFocus();
		error=TRUE;	
		}
	
	if (!error)
		{
		emit options(boxFunction->currentText(),fromX, toX, fromY, toY, fromZ, toZ);
		emit custom3DToolBar();
		close();
		}
}

void sDialog::insertFunctionsList(const QStringList& list)
{
boxFunction->insertStringList (list, 1);
}

sDialog::~sDialog()
{
}
