#include "tableDialog.h"

#include <qvariant.h>
#include <qpushbutton.h>
#include <qlineedit.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qbuttongroup.h>
#include <qcombobox.h>
#include <qlayout.h>
#include <qregexp.h>
#include <qdatetime.h> 

tableDialog::tableDialog( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "tableDialog" );
    setFixedSize( QSize( 430, 300 ) );
    setSizeGripEnabled( FALSE );

    GroupBox1 = new QButtonGroup(3,QGroupBox::Horizontal,tr(""),this,"GroupBox1" );
	GroupBox1->setLineWidth (0);
	
	new QLabel(tr( "Column name" ),GroupBox1, "TextLabel1",0 );
    colName = new QLineEdit( GroupBox1, "colName" );
	
	buttonOk = new QPushButton( GroupBox1, "buttonOk" );
    buttonOk->setAutoDefault( TRUE );
    buttonOk->setDefault( TRUE );

    enumerateAllBox = new QCheckBox( GroupBox1, "enumerateAllBox" );
	enumerateAllBox->setText("");
	
	new QLabel( tr( "Enumerate all to the right" ), GroupBox1, "TextLabelVide",0 );
	
	buttonCancel = new QPushButton( GroupBox1, "buttonCancel" );
    buttonCancel->setAutoDefault( TRUE );
	
	GroupBox2 = new QButtonGroup(3,QGroupBox::Horizontal,tr("Options"),this,"GroupBox2" );

    new QLabel( tr( "Plot designation" ), GroupBox2, "TextLabel2_2",0 );
	columnsBox = new QLineEdit(GroupBox2, "columnsBox" );
	columnsBox->setReadOnly(TRUE);
	new QLabel(tr( "" ), GroupBox2, "TextLabelVide1",0 );


	/*new QLabel( tr( "Display" ), GroupBox2, "TextLabel2_3",0 );
	displayBox = new QComboBox(GroupBox2, "displayBox" );
	displayBox->insertItem(tr("Numeric"));
	displayBox->insertItem(tr("Date"));
	displayBox->insertItem(tr("Time"));
	displayBox->insertItem(tr("Month"));
	displayBox->insertItem(tr("Day of Week"));
	
	new QLabel(tr( "" ), GroupBox2, "TextLabelVide3",0 );*/

    new QLabel(tr( "Format" ), GroupBox2, "TextLabel2",0 );
	formatBox = new QComboBox( FALSE, GroupBox2, "formatBox" );
	new QLabel(tr( "" ), GroupBox2, "TextLabelVide2",0 );
	
	labelNumeric = new QLabel(tr( "Numeric display" ),GroupBox2, "TextLabel3",0);
	numericBox = new QComboBox( FALSE, GroupBox2, "numericBox" );

    precisionBox = new QLineEdit( GroupBox2, "precisionBox" );
    precisionBox->setMaximumSize( QSize( 100, 32767 ) );
	precisionBox->setText("3");
	precisionBox->hide();

	applyToRightCols = new QCheckBox( GroupBox2, "applyToRightCols" );
	applyToRightCols->setText("");
	new QLabel(tr( "Apply to all columns to the right" ), GroupBox2, "TextLabelVide3",0 );	
	
	GroupBox3 = new QButtonGroup(3,QGroupBox::Horizontal,tr(""),this,"GroupBox3" );
	GroupBox3->setLineWidth (0);
	
	new QLabel(tr( "Column width" ),GroupBox3, "TextLabel1_2",0 );

    colWidth = new QLineEdit(GroupBox3, "colWidth" );

    applyToAllBox = new QCheckBox(GroupBox3, "applyToAllBox" );
	
	QVBoxLayout* hlayout = new QVBoxLayout(this,5,5, "hlayout");
    hlayout->addWidget(GroupBox1);
	hlayout->addWidget(GroupBox2);
	hlayout->addWidget(GroupBox3);

    languageChange();
    setFocusProxy (colName);

   // signals and slots connections
    connect( buttonOk, SIGNAL(clicked()), this, SLOT(accept()));
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect(numericBox, SIGNAL(activated(int)), this, SLOT(showPrecisionBox(int)) );
	connect(numericBox, SIGNAL(activated(int)), this, SLOT(updateNumeric(int)) );
	connect(precisionBox, SIGNAL(textChanged(const QString & )), this, SLOT(updatePrecision(const QString&)) );
	connect(applyToRightCols, SIGNAL(clicked()), this, SLOT(updateFormat()) );
	connect(formatBox, SIGNAL(activated(int)), this, SLOT(updateFormat(int)) );
	//connect(displayBox, SIGNAL(activated(int)), this, SLOT(updateDisplay(int)) );
}

void tableDialog::accept()
{
QString name=colName->text();
	
name.remove("[X]");
name.remove("[Y]");

QRegExp underscore("_");
emit nameChanged(name.replace(underscore,"-"));
emit enumRightCols(enumerateAllBox->isChecked());
emit changeWidth(colWidth->text(),applyToAllBox->isChecked());
close();
}

void tableDialog::closeEvent( QCloseEvent* ce )
{
emit closedTableDialog();
ce->accept();
}

void tableDialog::setPlotDesignation(int i)
{
	if (i==0)
		columnsBox->setText("X (abscissae)");
	else if (i==1)
		columnsBox->setText("Y (ordinates)");
	else 
		columnsBox->setText("Data (not X nor Y flags are set)");
}

void tableDialog::showPrecisionBox(int item)
{
switch(item)   
        {
            case 0:
            {
                precisionBox->hide();
                break;
            }
            case 1:
            {
                precisionBox->show();
                break;
            }
		}
}

void tableDialog::updatePrecision(const QString& text)
{
emit changeFormat(formatBox->currentItem(),numericBox->currentItem(),text,applyToRightCols->isChecked());
}

void tableDialog::updateNumeric(int numeric)
{
emit changeFormat(formatBox->currentItem(),numeric,precisionBox->text(),
		applyToRightCols->isChecked());
}

void tableDialog::updateFormat(int format)
{
emit changeFormat(format,numericBox->currentItem(),precisionBox->text(),
			applyToRightCols->isChecked());
}

void tableDialog::updateFormat()
{
emit changeFormat(formatBox->currentItem(),numericBox->currentItem(),
				precisionBox->text(),applyToRightCols->isChecked());
}

void tableDialog::setColWidth(const QString& w)
{
colWidth->setText(w);  
}

void tableDialog::setColName(const QString& text)
{
colName->setText(text);
colName->selectAll();  
}


tableDialog::~tableDialog()
{
}


/*void tableDialog::updateDisplay(int item)
{
if (item == 0)
	{
	formatBox->clear();
    formatBox->insertItem( tr( "Decimal: 1000" ) );
    formatBox->insertItem( tr( "Scientific: 1E3" ) );

	labelNumeric->show();
	numericBox->show();
	}
else
	{
	formatBox->clear();
	QDate date=QDate::currentDate ();

	switch (item)
		{
		case 1:
			formatBox->insertItem(date.toString ("dd/MM/yyyy"));
			formatBox->insertItem(date.toString (Qt::TextDate ));
		break;

		case 2:
			formatBox->insertItem(tr("HH:mm"));
			formatBox->insertItem(tr("HH") );
			formatBox->insertItem(tr("HH:mm:ss") );
			formatBox->insertItem(tr("HH:mm:ss.###") );
			formatBox->insertItem(tr("HH pm") );
			formatBox->insertItem(tr("HH:mm pm") );
			formatBox->insertItem(tr("mm:ss") );
			formatBox->insertItem(tr("mm:ss.###") );
			formatBox->insertItem(tr("HHmm") );
			formatBox->insertItem(tr("HHmmss") );
		break;

		case 3:
			formatBox->insertItem(QDate::shortMonthName (date.month()));
			formatBox->insertItem(QDate::longMonthName (date.month()));
		break;

		case 4:
			formatBox->insertItem(QDate::shortDayName (date.dayOfWeek ()) );
			formatBox->insertItem(QDate::longDayName (date.dayOfWeek ()));
		break;
		}

	precisionBox->hide();
	labelNumeric->hide();
	numericBox->hide();
	}
}*/

void tableDialog::languageChange()
{
    setCaption( tr( "QtiPlot - Table options" ) );
    buttonCancel->setText( tr( "&Cancel" ) );  
    buttonOk->setText( tr( "&OK" ) );
    
    numericBox->clear();
    numericBox->insertItem( tr( "Default" ) );
    numericBox->insertItem( tr( "Set Decimal Places=" ) );
    numericBox->insertItem( tr( "Significant Digits=" ) );
	
    formatBox->clear();
    formatBox->insertItem( tr( "Decimal: 1000" ) );
    formatBox->insertItem( tr( "Scientific: 1E3" ) );
	
    applyToAllBox->setText( tr( "Apply to all" ) );
}
