#include "textDialog.h"
#include "colors.h"
#include "txt_icons.h"

#include <qlabel.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qfont.h>
#include <qfontdialog.h>
#include <qcombobox.h>
#include <qbuttongroup.h>
#include <qmessagebox.h>

textDialog::textDialog( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "textDialog" );
    resize(330, 300 ); 
    setCaption( tr( "QtiPlot - Text Dialog" ) );
    setSizeGripEnabled( FALSE );
	
	GroupBox1 = new QButtonGroup(3,QGroupBox::Horizontal,tr( "" ),this, "GroupBox1" );
    
	TextLabel2 = new QLabel(GroupBox1, "TextLabel2" ); 
    TextLabel2->setText( tr( "Title Color" ) );

	boxColor = new QComboBox( FALSE, GroupBox1, "boxColor" );
	boxColor->insertItem( QPixmap(black_xpm), tr( "black" ) );
    boxColor->insertItem( QPixmap(red_xpm), tr( "red" ) );
    boxColor->insertItem( QPixmap(green_xpm), tr( "green" ) );
    boxColor->insertItem( QPixmap(blue_xpm), tr( "blue" ) );
	boxColor->insertItem( QPixmap(cyan_xpm), tr( "cyan" ) );
	boxColor->insertItem( QPixmap(magenta_xpm), tr( "magenta" ) );
    boxColor->insertItem( QPixmap(yellow_xpm), tr( "yellow" ) );
	boxColor->insertItem( QPixmap(darkYellow_xpm), tr( "dark yellow" ) );
	boxColor->insertItem( QPixmap(darkBlue_xpm), tr( "navy" ) );
	boxColor->insertItem( QPixmap(darkMagenta_xpm), tr( "purple" ) );
	boxColor->insertItem( QPixmap(darkRed_xpm), tr( "wine" ) );
	boxColor->insertItem( QPixmap(darkGreen_xpm), tr( "olive" ) );
	boxColor->insertItem( QPixmap(darkCyan_xpm), tr( "dark cyan" ) );
	boxColor->insertItem(QPixmap(white_xpm),tr( "white" ) );
	boxColor->insertItem( QPixmap(lightGray_xpm), tr( "light gray" ) );
	boxColor->insertItem( QPixmap(darkGray_xpm), tr( "dark gray" ) );
	
	buttonOk = new QPushButton( GroupBox1, "buttonOk" );
    buttonOk->setText( tr( "&OK" ) );
    buttonOk->setAutoDefault( TRUE );
    buttonOk->setDefault( TRUE );
	
	TextLabel1 = new QLabel( GroupBox1, "TextLabel1" );
    TextLabel1->setText( tr( "Title Font" ) );

    buttonFont = new QPushButton( GroupBox1, "buttonFont" );
    buttonFont->setText( tr( "&Font" ) );
    buttonFont->setAutoDefault( TRUE );
	
	buttonApply = new QPushButton( GroupBox1, "buttonApply" );
    buttonApply->setText( tr( "&Apply" ) );
    buttonApply->setAutoDefault( TRUE );
	
	new QLabel(tr( "Alignement" ),GroupBox1, "TextLabel1_22",0);
	
	alignementBox = new QComboBox( FALSE, GroupBox1, "alignementBox" );
    alignementBox->insertItem( tr( "Center" ) );
	alignementBox->insertItem( tr( "Left" ) );
    alignementBox->insertItem( tr( "Right" ) );
	
    buttonCancel = new QPushButton( GroupBox1, "buttonCancel" );
    buttonCancel->setText( tr( "&Cancel" ) );
    buttonCancel->setAutoDefault( TRUE );
	
	GroupBox2= new QButtonGroup(6,QGroupBox::Horizontal,tr( "" ),this, "GroupBox2" );
	
	buttonIndice = new QPushButton( GroupBox2, "buttonIndice" ); 
    buttonIndice->setPixmap (QPixmap(index_xpm));

    buttonExp = new QPushButton( GroupBox2, "buttonExp" );
    buttonExp->setPixmap (QPixmap(exp_xpm));

    buttonSym = new QPushButton( GroupBox2, "buttonSym" ); 
    buttonSym->setPixmap (QPixmap(symbol_xpm));
	
    buttonB = new QPushButton( GroupBox2, "buttonB" ); 
    buttonB->setPixmap (QPixmap(bold_xpm));

    buttonI = new QPushButton( GroupBox2, "buttonI" );
    buttonI->setPixmap (QPixmap(italic_xpm));

    buttonU = new QPushButton( GroupBox2, "buttonU" );
    buttonU->setPixmap (QPixmap(underline_xpm));
	
	LineEdit = new QMultiLineEdit( this, "LineEdit" );
	LineEdit->setFocus();
	
	QVBoxLayout* vlayout = new QVBoxLayout(this,5,5, "vlayout");
    vlayout->addWidget(GroupBox1);
	vlayout->addWidget(GroupBox2);
	vlayout->addWidget(LineEdit);

    // signals and slots connections
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( buttonApply, SIGNAL( clicked() ), this, SLOT( apply() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( buttonFont, SIGNAL( clicked() ), this, SLOT(customFont() ) );
	
	connect( buttonExp, SIGNAL(clicked()), this, SLOT(addExp()));
	connect( buttonIndice, SIGNAL(clicked()), this, SLOT(addIndex()));
	connect( buttonSym, SIGNAL(clicked()), this, SLOT(addSymbol()));
	connect( buttonU, SIGNAL(clicked()), this, SLOT(addUnderline()));
	connect( buttonI, SIGNAL(clicked()), this, SLOT(addItalic()));
	connect( buttonB, SIGNAL(clicked()), this, SLOT(addBold()));
}

bool textDialog::rejected()
{
if (result()==0)
	{
	reject();
	return TRUE;
	}	
else 
return FALSE;
}

void textDialog::apply()
{	
emit changeAlignment(alignment());
emit changeText(LineEdit->text());
emit changeColor(color());
}

void textDialog::accept()
{
apply();
close();
}

int textDialog::alignment()
{
int align=-1;
switch (alignementBox->currentItem())
	{
	case 0:
		align=Qt::AlignHCenter;
	break;

	case 1:
		align=Qt::AlignLeft;
	break;
	
	case 2:
		align=Qt::AlignRight;
	break;
	}
return align;
}

void textDialog::setAlignment(int align)
{	
if (align==Qt::AlignHCenter)
	alignementBox->setCurrentItem(0);
else if (align==Qt::AlignLeft)
	alignementBox->setCurrentItem(1);
else if (align==Qt::AlignRight)
	alignementBox->setCurrentItem(2);
else
	return;
}

void textDialog::customFont()
{
bool okF;
QFont fnt = QFontDialog::getFont( &okF,f,this);
if (okF)
f=fnt;
emit changeFont (fnt);
}

QFont textDialog::font()
{
return f;
}

void textDialog::setFont(const QFont& fnt)
{
f=fnt;
}

void textDialog::setText(const QString & t)
{
LineEdit->insertAt (t, 0, 0, TRUE );
}

QColor textDialog::color()
{
QColor c;
switch (boxColor->currentItem())
	{
case 0:
	c=QColor(black);
break;

case 1:
	c=QColor(red);
break;

case 2:
	c=QColor(green);
break;

case 3:
	c=QColor(blue);
break;

case 4:
	c=QColor(cyan);
break;

case 5:
	c=QColor(magenta);
break;

case 6:
	c=QColor(yellow);
break;

case 7:
	c=QColor(darkYellow);
break;

case 8:
	c=QColor(darkBlue);
break;

case 9:
	c=QColor(darkMagenta);
break;

case 10:
	c=QColor(darkRed);
break;

case 11:
	c=QColor(darkGreen);
break;

case 12:
	c=QColor(darkCyan);
break;

case 13:
	c=QColor(white);
break;

case 14:
	c=QColor(lightGray);
break;

case 15:
	c=QColor(darkGray);
break;
	}
return c;
}

void textDialog::setColor(QColor c)
{
if (c==QColor(black)) boxColor->setCurrentItem(0);
else if (c==QColor(red)) boxColor->setCurrentItem(1);
else if (c==QColor(green)) boxColor->setCurrentItem(2);
else if (c==QColor(blue)) boxColor->setCurrentItem(3);
else if (c==QColor(cyan)) boxColor->setCurrentItem(4);
else if (c==QColor(magenta)) boxColor->setCurrentItem(5);
else if (c==QColor(yellow)) boxColor->setCurrentItem(6);
else if (c==QColor(darkYellow)) boxColor->setCurrentItem(7);
else if (c==QColor(darkBlue)) boxColor->setCurrentItem(8);
else if (c==QColor(darkMagenta)) boxColor->setCurrentItem(9);
else if (c==QColor(darkRed)) boxColor->setCurrentItem(10);
else if (c==QColor(darkGreen)) boxColor->setCurrentItem(11);
else if (c==QColor(darkCyan)) boxColor->setCurrentItem(12);
else if (c==QColor(white)) boxColor->setCurrentItem(13);
else if (c==QColor(lightGray)) boxColor->setCurrentItem(14);
else if (c==QColor(darkGray)) boxColor->setCurrentItem(15);
}

QString textDialog::getText()
{
return LineEdit->text();
}

void textDialog::addUnderline()
{
int line=0, col=0;
if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<u>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</u>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insert("<u></u>");
	LineEdit->setCursorPosition (line,col+3,FALSE);
	}
}

void textDialog::addItalic()
{
int line=0, col=0;

if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<i>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</i>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insert("<i></i>");
	LineEdit->setCursorPosition (line,col+3,FALSE);
	}
}

void textDialog::addBold()
{
int line=0, col=0;
	
if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<b>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</b>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insert("<b></b>");
	LineEdit->setCursorPosition (line,col+3,FALSE);
	}
}

void textDialog::addIndex()
{
int line=0, col=0;
LineEdit->getCursorPosition (&line,&col);
	
if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<sub>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</sub>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->insert("<sub></sub>");
	LineEdit->setCursorPosition (line,col+5,FALSE);
	}
}

void textDialog::addExp()
{
int line, col;
LineEdit->getCursorPosition (&line,&col);

if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<sup>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</sup>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->insert("<sup></sup>");
	LineEdit->setCursorPosition (line,col+5,FALSE);
	}
}

void textDialog::addSymbol()
{
int line, col;
	
if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<font face=symbol>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</font>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insert("<font face=symbol></font>");
	LineEdit->setCursorPosition (line,col+18,FALSE);
	}
}

/*  
 *  Destroys the object and frees any allocated resources
 */
textDialog::~textDialog()
{
    // no need to delete child widgets, Qt does it all for us
}
