#include "txtDlg.h"
#include "colors.h"
#include "txt_icons.h"

#include <qcombobox.h>
#include <qlabel.h>
#include <qmultilineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qvariant.h>
#include <qbuttongroup.h>
#include <qfont.h>
#include <qfontdialog.h>

static const char * lineSymbol_xpm[] = {
"16 16 4 1",
" 	c None",
".	c #8C2727",
"+	c #272787",
"@	c #FFFFFF",
"                ",
"                ",
"                ",
"      ...       ",
"  ............  ",
"      ...       ",
"                ",
"                ",
"                ",
"      +++       ",
"  +++++@++++++  ",
"      +++       ",
"                ",
"                ",
"                ",
"                "};

textDlg::textDlg( QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name )
	setName( "textDlg" );
	setFixedWidth(400);
	setFixedHeight(600);
    setCaption( tr( "QtiPlot - Text options" ) );
    setSizeGripEnabled( FALSE );
	
	GroupBox1 = new QButtonGroup(3,QGroupBox::Horizontal,tr( "" ),this, "GroupBox1" );

	new QLabel(tr( "Color" ), GroupBox1, "TextLabel2",0 );
	
    colorBox = new QComboBox( FALSE, GroupBox1, "colorBox" );
	colorBox->insertItem( QPixmap(black_xpm), tr( "black" ) );
    colorBox->insertItem( QPixmap(red_xpm), tr( "red" ) );
    colorBox->insertItem( QPixmap(green_xpm), tr( "green" ) );
    colorBox->insertItem( QPixmap(blue_xpm), tr( "blue" ) );
	colorBox->insertItem( QPixmap(cyan_xpm), tr( "cyan" ) );
	colorBox->insertItem( QPixmap(magenta_xpm), tr( "magenta" ) );
    colorBox->insertItem( QPixmap(yellow_xpm), tr( "yellow" ) );
	colorBox->insertItem( QPixmap(darkYellow_xpm), tr( "dark yellow" ) );
	colorBox->insertItem( QPixmap(darkBlue_xpm), tr( "navy" ) );
	colorBox->insertItem( QPixmap(darkMagenta_xpm), tr( "purple" ) );
	colorBox->insertItem( QPixmap(darkRed_xpm), tr( "wine" ) );
	colorBox->insertItem( QPixmap(darkGreen_xpm), tr( "olive" ) );
	colorBox->insertItem( QPixmap(darkCyan_xpm), tr( "dark cyan" ) );
	colorBox->insertItem(QPixmap(white_xpm),tr( "white" ) );
	colorBox->insertItem( QPixmap(lightGray_xpm), tr( "light gray" ) );
	colorBox->insertItem( QPixmap(darkGray_xpm), tr( "dark gray" ) );
	
	buttonOk = new QPushButton( GroupBox1, "buttonOk" );
    buttonOk->setText( tr( "&OK" ) );
    buttonOk->setAutoDefault( TRUE );
    buttonOk->setDefault( TRUE );

	new QLabel(tr( "Background" ), GroupBox1, "TextLabel1",0 );

    backgroundBox = new QComboBox( FALSE, GroupBox1, "backgroundBox" );
    backgroundBox->insertItem( tr( "None" ) );
	backgroundBox->insertItem( tr( "Rectangle" ) );
    backgroundBox->insertItem( tr( "Shadow" ) );
    backgroundBox->insertItem( tr( "White out" ) );
    backgroundBox->insertItem( tr( "Black out" ) );

	buttonApply = new QPushButton( GroupBox1, "buttonApply" );
    buttonApply->setText( tr( "&Apply" ) );
    buttonApply->setAutoDefault( TRUE );
    buttonApply->setDefault( TRUE );
	
	new QLabel(tr( "Text font" ),GroupBox1, "TextLabel1_21",0);

	buttonFont = new QPushButton( GroupBox1, "buttonFont" );
    buttonFont->setText( tr( "&Font" ) );
    buttonFont->setAutoDefault( TRUE );
	
	buttonCancel = new QPushButton( GroupBox1, "buttonCancel" );
    buttonCancel->setText( tr( "&Cancel" ) );
    buttonCancel->setAutoDefault( TRUE );
	
	QLabel* rotate=new QLabel(tr( "Rotate (deg.)" ),GroupBox1, "TextLabel1_2",0);
	rotate->hide();
	
    rotateBox = new QComboBox( FALSE, GroupBox1, "rotateBox" );
    rotateBox->insertItem( tr( "0" ) );
    rotateBox->insertItem( tr( "45" ) );
    rotateBox->insertItem( tr( "90" ) );
    rotateBox->insertItem( tr( "135" ) );
    rotateBox->insertItem( tr( "180" ) );
    rotateBox->insertItem( tr( "225" ) );
    rotateBox->insertItem( tr( "270" ) );
    rotateBox->insertItem( tr( "315" ) );
	rotateBox->setEditable (TRUE);
	rotateBox->setCurrentItem(0);
	rotateBox->hide();
	
	GroupBox2= new QButtonGroup(7,QGroupBox::Horizontal,tr( "" ),this, "GroupBox2" );

	buttonCurve = new QPushButton( GroupBox2, "buttonCurve" ); 
    buttonCurve->setPixmap (QPixmap(lineSymbol_xpm));

    buttonIndice = new QPushButton( GroupBox2, "buttonIndice" ); 
    buttonIndice->setPixmap (QPixmap(index_xpm));

    buttonExp = new QPushButton( GroupBox2, "buttonExp" );
    buttonExp->setPixmap (QPixmap(exp_xpm));

    buttonSym = new QPushButton( GroupBox2, "buttonSym" ); 
    buttonSym->setPixmap (QPixmap(symbol_xpm));

    buttonB = new QPushButton( GroupBox2, "buttonB" ); 
    buttonB->setPixmap (QPixmap(bold_xpm));

    buttonI = new QPushButton( GroupBox2, "buttonI" );
    buttonI->setPixmap (QPixmap(italic_xpm));

    buttonU = new QPushButton( GroupBox2, "buttonU" );
    buttonU->setPixmap (QPixmap(underline_xpm));

    LineEdit = new QMultiLineEdit( this, "LineEdit" );
    LineEdit->setFont(QFont("Arial",12,QFont::Normal,FALSE));

	setFocusPolicy(QWidget::StrongFocus);
	setFocusProxy(LineEdit);
	
	QVBoxLayout* vlayout = new QVBoxLayout(this,5,5, "vlayout");
    vlayout->addWidget(GroupBox1);
	vlayout->addWidget(GroupBox2);
	vlayout->addWidget(LineEdit);

    // signals and slots connections
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( accept() ) );
	connect( buttonApply, SIGNAL( clicked() ), this, SLOT( apply() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
	connect( buttonFont, SIGNAL( clicked() ), this, SLOT(customFont() ) );
	connect( buttonExp, SIGNAL( clicked() ), this, SLOT(addExp() ) );
	connect( buttonIndice, SIGNAL( clicked() ), this, SLOT(addIndex() ) );
	connect( buttonSym, SIGNAL( clicked() ), this, SLOT(addSymbol() ) );
	connect( buttonU, SIGNAL( clicked() ), this, SLOT(addUnderline() ) );
	connect( buttonI, SIGNAL( clicked() ), this, SLOT(addItalic() ) );
	connect( buttonB, SIGNAL( clicked() ), this, SLOT(addBold() ) );
	connect( buttonCurve, SIGNAL( clicked() ), this, SLOT(addCurve() ) );
}

void textDlg::addCurve()
{
int line=0, col=0;
LineEdit->getCursorPosition (&line,&col);
LineEdit->insert("\\c{}");
LineEdit->setCursorPosition (line,col+3,FALSE);
}

void textDlg::addUnderline()
{
int line=0, col=0;
if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<u>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</u>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insert("<u></u>");
	LineEdit->setCursorPosition (line,col+3,FALSE);
	}
}

void textDlg::addItalic()
{
int line=0, col=0;

if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<i>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</i>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insert("<i></i>");
	LineEdit->setCursorPosition (line,col+3,FALSE);
	}
}

void textDlg::addBold()
{
int line=0, col=0;
	
if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<b>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</b>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insert("<b></b>");
	LineEdit->setCursorPosition (line,col+3,FALSE);
	}
}

void textDlg::addIndex()
{
int line=0, col=0;
LineEdit->getCursorPosition (&line,&col);
	
if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<sub>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</sub>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->insert("<sub></sub>");
	LineEdit->setCursorPosition (line,col+5,FALSE);
	}
}

void textDlg::addExp()
{
int line, col;
LineEdit->getCursorPosition (&line,&col);

if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<sup>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</sup>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->insert("<sup></sup>");
	LineEdit->setCursorPosition (line,col+5,FALSE);
	}
}

void textDlg::addSymbol()
{
int line, col;
	
if (LineEdit->hasMarkedText())
	{	
	QString markedText=LineEdit->markedText ();
	LineEdit->insert("<font face=symbol>"+markedText);
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insertAt ("</font>",line,col,FALSE);
	LineEdit->setCursorPosition (line,col,FALSE);
	}
else
	{	
	LineEdit->getCursorPosition (&line,&col);
	LineEdit->insert("<font face=symbol></font>");
	LineEdit->setCursorPosition (line,col+18,FALSE);
	}
}

int textDlg::backgroundType()
{
return backgroundBox->currentItem();
}

void textDlg::apply()
{
emit values(getText(),0,backgroundType(),f,color());
}

void textDlg::accept()
{
emit values(getText(),0,backgroundType(),f,color());
close();
}

void textDlg::setBackgroundType(int bkg)
{
backgroundBox->setCurrentItem(bkg);
}

void textDlg::customFont()
{
bool okF;
QFont fnt = QFontDialog::getFont( &okF,f,this);
if (okF)
f=fnt;
}

QFont textDlg::font()
{
return f;
}

void textDlg::setFont(const QFont& fnt)
{
f=fnt;
}

void textDlg::setAngle(int angle)
{
rotateBox->setEditText(QString::number(angle));
}

void textDlg::setText(const QString & t)
{
//LineEdit->setText(t);
LineEdit->insertAt (t, 0, 0, TRUE );
}

QColor textDlg::color()
{
QColor c;
switch (colorBox->currentItem())
	{
case 0:
	c=QColor(black);
break;

case 1:
	c=QColor(red);
break;

case 2:
	c=QColor(green);
break;

case 3:
	c=QColor(blue);
break;

case 4:
	c=QColor(cyan);
break;

case 5:
	c=QColor(magenta);
break;

case 6:
	c=QColor(yellow);
break;

case 7:
	c=QColor(darkYellow);
break;

case 8:
	c=QColor(darkBlue);
break;

case 9:
	c=QColor(darkMagenta);

break;

case 10:
	c=QColor(darkRed);
break;

case 11:
	c=QColor(darkGreen);
break;

case 12:
	c=QColor(darkCyan);
break;

case 13:
	c=QColor(white);
break;

case 14:
	c=QColor(lightGray);
break;

case 15:
	c=QColor(darkGray);
break;
	}
return c;
}

void textDlg::setColor(QColor c)
{
if (c==QColor(black)) colorBox->setCurrentItem(0);
else if (c==QColor(red)) colorBox->setCurrentItem(1);
else if (c==QColor(green)) colorBox->setCurrentItem(2);
else if (c==QColor(blue)) colorBox->setCurrentItem(3);
else if (c==QColor(cyan)) colorBox->setCurrentItem(4);
else if (c==QColor(magenta)) colorBox->setCurrentItem(5);
else if (c==QColor(yellow)) colorBox->setCurrentItem(6);
else if (c==QColor(darkYellow)) colorBox->setCurrentItem(7);
else if (c==QColor(darkBlue)) colorBox->setCurrentItem(8);
else if (c==QColor(darkMagenta)) colorBox->setCurrentItem(9);
else if (c==QColor(darkRed)) colorBox->setCurrentItem(10);
else if (c==QColor(darkGreen)) colorBox->setCurrentItem(11);
else if (c==QColor(darkCyan)) colorBox->setCurrentItem(12);
else if (c==QColor(white)) colorBox->setCurrentItem(13);
else if (c==QColor(lightGray)) colorBox->setCurrentItem(14);
else if (c==QColor(darkGray)) colorBox->setCurrentItem(15);
}


QString textDlg::getText()
{
return LineEdit->text();
}

/*  
 *  Destroys the object and frees any allocated resources
 */
textDlg::~textDlg()
{
    // no need to delete child widgets, Qt does it all for us
}
