#include "worksheet.h"
#include "sortDialog.h"

#include <qpopupmenu.h>
#include <qfiledialog.h>
#include <qinputdialog.h>
#include <qheader.h>
#include <qmessagebox.h>
#include <qarray.h>
#include <qfileinfo.h>
#include <qfile.h>
#include <qregexp.h>
#include <qtextstream.h>
#include <qclipboard.h> 
#include <qapplication.h> 
#include <qcursor.h>
#include <qdatetime.h>
#include <qprinter.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qdragobject.h>
#include <qlayout.h>
#include <qaccel.h>
#include "parser/muParser.h"

#include <stdlib.h>
#include <math.h>
#include <stdio.h>

#include <gsl/gsl_vector.h>
#include <gsl/gsl_sort.h>
#include <gsl/gsl_sort_vector.h>
#include <gsl/gsl_statistics.h>

extern "C" {
double **matrix(long nrl, long nrh, long ncl, long nch);
void free_matrix(double **m, long nrl, long nrh, long ncl, long nch);
double *vector(long nl, long nh);
void free_vector(double *v, long nl, long nh);
size_t *ivector(long nl, long nh);
void free_ivector(size_t *v, long nl, long nh);
}

WksheetWindow::WksheetWindow(const QString &fname,const QString &sep, int ignoredLines,
																		QWidget* parent, const char* name,WFlags f)
        : QWidget(parent,name,f)
{
QApplication::setOverrideCursor(waitCursor);
importASCII(fname, sep, ignoredLines);
QApplication::restoreOverrideCursor();
}

WksheetWindow::WksheetWindow(int r, int c, QWidget* parent, const char* name, WFlags f)
        : QWidget(parent,name,f)
{
init(r,c);	
}

void WksheetWindow::init(int rows, int cols)
{
selectedCol=0;clickedRow=0;xCol=0;yCol=1;
LeftButton=FALSE;
	
QDate dt = QDate::currentDate();
QTime time=QTime::currentTime();

birthdate=dt.toString ("dd/MM/yyyy ");
birthdate+=time.toString();

worksheet= new QTable (rows,cols,this,"table");
worksheet->setFocusPolicy(QWidget::StrongFocus);
worksheet->setFocus();
	
QVBoxLayout* hlayout = new QVBoxLayout(this,0,0, "hlayout1");
hlayout->addWidget(worksheet);

for (int i=0;i<cols;i++)
	commandes<<"col(1)";

QHeader* head=(QHeader*)worksheet->horizontalHeader();
head->installEventFilter(this);
head->setMouseTracking(TRUE);
head->setLabel(0,"1[X]");
head->setLabel(1,"2[Y]");

int w, h;
if (cols>5)
	w=5*(worksheet->horizontalHeader())->sectionSize(0);
else
	w=cols*(worksheet->horizontalHeader())->sectionSize(0);

if (rows>11)
	h=11*(worksheet->verticalHeader())->sectionSize(0);
else
	h=(rows+1)*(worksheet->verticalHeader())->sectionSize(0);
setGeometry(50,50,w+55,h);

QAccel *accel = new QAccel(this);
accel->connectItem( accel->insertItem( Key_Tab ),
                            this, SLOT(moveCurrentCell()) );
accel->connectItem( accel->insertItem( CTRL+Key_A ),
                            this, SLOT(selectAllTable()) );

connect(worksheet->verticalHeader(), SIGNAL(clicked(int)),this, SIGNAL(rowMenu(int)));
connect(worksheet, SIGNAL(valueChanged(int,int)),this, SLOT(cellEdited(int,int)));
}

int WksheetWindow::checkColType(int col)
{
int rows=worksheet->numRows();
for (int i=0;i<rows;i++)
	{
	QString text = worksheet->text(i,col).lower();
	text.remove("e").remove(",").remove(".").remove("-").remove("+");
	if (text.contains(QRegExp("\\D")) > 0)
		{
		return Text;
		}
	}
return Numeric;
}

void WksheetWindow::setBackgroundColor(const QColor& col)
{
worksheet->setPaletteBackgroundColor ( col );
}

void WksheetWindow::setTextColor(const QColor& col)
{
worksheet->setPaletteForegroundColor (col);
}

void WksheetWindow::setTextFont(const QFont& fnt)
{
worksheet->setFont (fnt);
QFontMetrics fm(fnt);
int lm = fm.width( QString::number(10*worksheet->numRows()));
worksheet->setLeftMargin( lm );
}

void WksheetWindow::setHeaderColor(const QColor& col)
{
worksheet->horizontalHeader()->setPaletteForegroundColor (col);
}

void WksheetWindow::setHeaderFont(const QFont& fnt)
{
worksheet->horizontalHeader()->setFont (fnt);
}

void WksheetWindow::print()
{
QPrinter printer;
printer.setColorMode (QPrinter::GrayScale);
if (printer.setup()) 
	{
        printer.setFullPage( TRUE );
        QPainter p;
        if ( !p.begin(&printer ) )
            return; // paint on printer
        QPaintDeviceMetrics metrics( p.device() );
        int dpiy = metrics.logicalDpiY();
        int margin = (int) ( (2/2.54)*dpiy ); // 2 cm margins
		
		int rows=worksheet->numRows();
		int cols=worksheet->numCols();
		int height=margin;
		int right=margin;
		int i,width=0;
		
		// print header
		QStringList header=colNames();	
		p.setFont(QFont("Arial",12, QFont::Normal, FALSE));
		QRect br=p.boundingRect(br,Qt::AlignCenter,	header[0],-1,0);
		p.drawLine(right,height,right,height+br.height());
		QRect tr(br);	
		
		for (i=0;i<cols;i++)
			width+=worksheet->columnWidth (i);
		
		if (width >= metrics.width()-2*margin )
			printer.setOrientation(QPrinter::Landscape);
		
		for (i=0;i<cols;i++)
		{	
			int width=worksheet->columnWidth (i);
			tr.setTopLeft(QPoint(right,height));
			tr.setWidth(width);	
			tr.setHeight(br.height());
			p.fillRect (tr,QBrush(QColor(lightGray)));
			p.drawText(tr,Qt::AlignCenter,header[i],-1);
			right+=width;
			p.drawLine(right,height,right,height+tr.height());
			
			if (right >= metrics.width()-2*margin )
				break;
		}
		p.drawLine(margin,height,right,height);
		height+=tr.height();	
		p.drawLine(margin,height,right,height);		
		
		// print table values
		for (i=0;i<rows;i++)
		{
			right=margin;
			QString text=worksheet->text(i,0)+"\t";
			tr=p.boundingRect(tr,Qt::AlignCenter,text,-1,0);
			p.drawLine(right,height,right,height+tr.height());
			for (int j=0;j<cols;j++)
			{
			int width=worksheet->columnWidth (j);
			text=worksheet->text(i,j)+"\t";
			tr=p.boundingRect(tr,Qt::AlignCenter,text,-1,0);
			br.setTopLeft(QPoint(right,height));	
			br.setWidth(width);	
			br.setHeight(tr.height());
			p.drawText(br,Qt::AlignCenter,text,-1);
			right+=width;
			p.drawLine(right,height,right,height+tr.height());
				
			if (right >= metrics.width()-2*margin )
				break;
			}
			height+=br.height();
			p.drawLine(margin,height,right,height);	
			
			if (height >= metrics.height()-margin )
				{
            	printer.newPage();
				height=margin;
				p.drawLine(margin,height,right,height);
				}
		}	
    }
}

void WksheetWindow::cellEdited(int,int col)
{
QString name=colName(col);
emit modifiedData(name);
emit modifiedTable(this);
}

int WksheetWindow::colX()
{
return xCol;
}

int WksheetWindow::colY()
{
return yCol;
}

void WksheetWindow::setXCol()
{
if (selectedCol == xCol)
	{
	QMessageBox::warning(0,tr("QtiPlot - Column setting error"),
	tr("The column: <b>%1</b><p>already has the <b>X flag</b> set.")
                .arg(worksheet->horizontalHeader()->label(xCol).remove("[X]")));
	return;
	}
	
if (selectedCol == yCol)
	{
	QMessageBox::critical(0,tr("QtiPlot - Column setting error"),
	tr("The column: <b>%1</b><p>has the <b>Y flag</b> set."
                "<p>Please select another column as the X column!")
                .arg(worksheet->horizontalHeader()->label(yCol).remove("[Y]")));
	return;
	}

QHeader *head=worksheet->horizontalHeader();
QString xColName=(head->label(xCol)).remove("[X]");
head->setLabel(xCol,xColName);
xCol=selectedCol;
xColName=head->label(xCol)+"[X]";
worksheet->horizontalHeader()->setLabel(xCol,xColName);
emit modifiedTable(this);
}

void WksheetWindow::setXCol(int i)
{
xCol=i;
}

void WksheetWindow::setYCol()
{
if (selectedCol == yCol)
	{
	QMessageBox::warning(0,tr("QtiPlot - Column setting error"),
	tr("The column: <b>%1</b><p>already has the <b>Y flag</b> set.")
                .arg(worksheet->horizontalHeader()->label(yCol).remove("[Y]")));
	return;
	}
	
if (selectedCol == xCol)
	{
	QMessageBox::critical(0,tr("QtiPlot - Column setting error"),
	tr("The column: <b>%1</b><p>has the <b>X flag</b> set."
                "<p>Please select another column as the Y column!")
                .arg(worksheet->horizontalHeader()->label(xCol).remove("[X]")));
	return;
	}

QHeader *head=worksheet->horizontalHeader();
QString yColName=(head->label(yCol)).remove("[Y]");
head->setLabel(yCol,yColName);
yCol=selectedCol;
yColName=head->label(yCol)+"[Y]";
worksheet->horizontalHeader()->setLabel(yCol,yColName);
emit modifiedTable(this);
}

void WksheetWindow::setYCol(int i)
{
yCol=i;
}

QStringList WksheetWindow::columnWidths()
{
QStringList widths;
for (int i=0;i<worksheet->numCols();i++)
	widths<<QString::number(worksheet->columnWidth(i));
	
return widths;
}

void WksheetWindow::setColWidths(const QStringList& widths)
{
for (int i=0;i<(int)widths.count();i++)
	worksheet->setColumnWidth (i, widths[i].toInt() );
}

QString WksheetWindow::saveColumnWidths()
{
QString s="ColWidth\t";
for (int i=0;i<worksheet->numCols();i++)
	s+=QString::number(worksheet->columnWidth (i))+"\t";
	
return s+"\n";
}

QString WksheetWindow::saveCommandes()
{
QString s="com\t";
s+=commandes.join ("\t")+"\n";
return s;
}

QStringList WksheetWindow::getCommandes()
{
return commandes;
}

void WksheetWindow::setCommandes(const QStringList& com)
{
commandes=com;
}

void WksheetWindow::setCommandes(const QString& com)
{
QStringList list=QStringList::split ("\t",com,FALSE);
for (int i=0;i<int(list.count());i++)
	{
	commandes[i]=list[i+1];
	}
}

QString WksheetWindow::birthDate()
{
return birthdate;
}

void WksheetWindow::enumerateRightCols(bool checked)
{	
if (!checked)
	return;

QString label=worksheet->horizontalHeader()->label(selectedCol);
if (selectedCol == xCol)
	label.remove("[X]");
else if (selectedCol == yCol)
	label.remove("[Y]");

QStringList oldLabels=colNames();
QString caption=this->caption()+"_";
int n=1;
for (int i=selectedCol;i<(int)worksheet->numCols();i++)
	{
	QString newLabel=label+QString::number(n);
	setColName(i,newLabel);
	emit changedColHeader(caption+oldLabels[i],colName(i));
	n++;
	}	
emit modifiedTable(this);
}

void WksheetWindow::changeColWidth(const QString& width, bool allCols)
{
int i,w=width.toInt();
int cols=worksheet->numCols();
	
if (allCols)
	{
	for (i=0;i<cols;i++)
		worksheet->setColumnWidth (i,w);
	
	emit modifiedTable(this);
	}
else
	{
	if (worksheet->columnWidth(selectedCol) == w)
		return;
	
	worksheet->setColumnWidth (selectedCol,w);
	emit modifiedTable(this);
	}
}

void WksheetWindow::changeColName(const QString& text)
{
QString caption=this->caption();
QString oldName=colName(selectedCol);
QString newName=caption+"_"+text;

if (oldName == newName)
	return;

if (caption == text)
	{
	QMessageBox::critical(0,tr("QtiPlot - Error"),
	"The column name must be different from the table caption : <b>"+caption+"</b></b>!<p>Please choose another name!");
	return;	
	}

QStringList labels=colNames();
if (labels.contains(text)>0)
	{
	QMessageBox::critical(0,tr("QtiPlot - Error"),
	"There is allready a column called : <b>"+text+"</b> in table <b>"+caption+"</b>!<p>Please choose another name!");
	return;	
	}
	
setColName(selectedCol, text);	
emit changedColHeader(oldName, newName);
emit modifiedTable(this);
}

void WksheetWindow::setColName(int col,const QString& text)
{
if (col == xCol)
	worksheet->horizontalHeader()->setLabel(col,text+"[X]",-1);
else if (col == yCol)
	worksheet->horizontalHeader()->setLabel(col,text+"[Y]",-1);
else
	worksheet->horizontalHeader()->setLabel(col,text,-1);
}

QStringList WksheetWindow::selectedColumns()
{
int i;
QStringList names;
for (i=0;i<worksheet->numCols();i++)
	{
	if(worksheet->isColumnSelected (i,TRUE))
		{
		if (i==xCol)
			names<<this->caption()+"_"+(worksheet->horizontalHeader()->label(i)).remove("[X]");
		else if (i==yCol)
			names<<this->caption()+"_"+(worksheet->horizontalHeader()->label(i)).remove("[Y]");
		else
			names<<this->caption()+"_"+worksheet->horizontalHeader()->label(i);
		}
	}
return names;
}

int WksheetWindow::firstSelectedColumn()
{
for (int i=0;i<worksheet->numCols();i++)
	{
	if(worksheet->isColumnSelected (i,TRUE))
		return i;
	}
	
return -1;
}

int WksheetWindow::selectedRows()
{
int r=0;
for (int i=0;i<worksheet->numRows();i++)
	{
	if(worksheet->isRowSelected (i,TRUE))
		r++;
	}
return r;
}

QStringList WksheetWindow::colNames()
{
int i;
int cols=worksheet->numCols();
QStringList names;
for (i=0;i<cols;i++)
		{
		if (i==xCol)
			names<<(worksheet->horizontalHeader()->label(i)).remove("[X]");
		else if (i==yCol)
			names<<(worksheet->horizontalHeader()->label(i)).remove("[Y]");
		else
			names<<worksheet->horizontalHeader()->label(i);
		}
return names;
}

QString WksheetWindow::colName(int col)
{
QString name=this->caption()+"_"+worksheet->horizontalHeader()->label(col);
if (col==xCol)
	return name.remove("[X]");
else if (col==yCol)
	return name.remove("[Y]");
else
	return name;
}

int WksheetWindow::colIndex(const QString& name)
{
int cols=worksheet->numCols();
int index=-1;
int pos=name.find("_",FALSE);
QString label=name.right(name.length()-pos-1);
for (int i=0;i<cols;i++)
	{
	if ((worksheet->horizontalHeader()->label(i)).remove("[X]")==label || 
		(worksheet->horizontalHeader()->label(i)).remove("[Y]")==label)
		{
		return i;
		}
	}
return index;
}

QArray<double> WksheetWindow::col(int ycol)
{
int i;
int rows=worksheet->numRows();
int cols=worksheet->numCols();
QArray<double> Y(rows);

if (ycol<=cols)
	{
	for (i=0;i<rows;i++)
		Y[i]=worksheet->text(i,ycol).toDouble();
	}
return Y;
}

void WksheetWindow::insertCol()
{
worksheet->clearSelection();
	
int i;
int cols=worksheet->numCols();
QRegExp letters=QRegExp ("[a-zA-Z]");
int index,max=0;
for (i=0;i<cols;i++)
	{
	QString label=worksheet->horizontalHeader()->label(i);
	if (i == xCol)
		label.remove("[X]");
	else if (i == yCol)
		label.remove("[Y]");
	
	if (label.contains(letters)==0)
		{
		index=label.toInt();
		if (index>max) max=index;
		}		
	}
cols++;
	
QStringList::Iterator it=commandes.at(selectedCol);
commandes.insert(it,"col(1)");
	
QStringList labels,com2;
for (i=selectedCol+1;i<cols;i++)
	labels<<worksheet->horizontalHeader()->label(i-1);
worksheet->insertColumns (selectedCol+1,1);

for (i=selectedCol+1;i<cols;i++)
	worksheet->horizontalHeader()->setLabel(i,labels[i-selectedCol-1],-1);
worksheet->horizontalHeader()->setLabel(selectedCol+1,QString::number(max+1),-1);

emit modifiedTable(this);
}

void WksheetWindow::insertRow()
{
worksheet->insertRows(clickedRow,1 );
emit modifiedTable(this);
}

void WksheetWindow::addCol()
{
worksheet->clearSelection();
int cols=worksheet->numCols();
QRegExp letters=QRegExp ("[a-zA-Z]");
int index,max=0,i;
for (i=0;i<cols;i++)
	{
	QString label=worksheet->horizontalHeader()->label(i);
	if (i == xCol)
		label.remove("[X]");
	else if (i == yCol)
		label.remove("[Y]");

	if (label.contains(letters)==0)
		{
		index=label.toInt();
		if (index>max) max=index;
		}		
	}
worksheet->insertColumns(cols,1);
worksheet->horizontalHeader()->setLabel (cols,QString::number(max+1),-1);
worksheet->ensureCellVisible ( 0, cols );
	
emit modifiedTable(this);
commandes<<"col(1)";
}

void WksheetWindow::clearCol()
{
int rows=worksheet->numRows();
for (int i=0;i<rows;i++)
	{
	worksheet->clearCell(i,selectedCol);
	worksheet->updateCell(i,selectedCol); 
	}
QString name=colName(selectedCol);
emit modifiedData(name);
emit modifiedTable(this);
}

void WksheetWindow::clearCell(int row, int col)
{
worksheet->clearCell(row,col);
worksheet->updateCell(row,col); 

QString name=colName(col);
emit modifiedData(name);
emit modifiedTable(this);
}

int WksheetWindow::atRow(int col, double value)
{
int row=-1;
int rows=worksheet->numRows();
QString s=QString::number(value);
for (int i=0;i<rows;i++)
	{
	if (worksheet->text(i,col)==s)
		{
		row=i;
		break;			
		}
	}
return row;
}

void WksheetWindow::deleteRow()
{
int cols=worksheet->numCols();
worksheet->removeRow(clickedRow);
for (int i=0;i<cols;i++)
	{
	QString name=colName(i);
	emit modifiedData(name);
	}
emit modifiedTable(this);
}

void WksheetWindow::cutSelection()
{
copySelection();
clearSelection();
}

bool WksheetWindow::singleCellSelected()
{
bool single=TRUE;
QTableSelection sel=worksheet->selection(worksheet->currentSelection());
int top=sel.topRow();
int bottom=sel.bottomRow();
int left=sel.leftCol();
int right=sel.rightCol();
	
if (top != bottom || left != right)
	single=FALSE;
return single;
}

void WksheetWindow::selectAllTable()
{	
worksheet->addSelection (QTableSelection( 0, 0, worksheet->numRows(), worksheet->numCols() ));
}

void WksheetWindow::deselect()
{	
worksheet->clearSelection();
}

void WksheetWindow::clearSelection()
{	
QStringList list=selectedColumns();
int i,n=int(list.count());

if (n>0)
{
for (i=0;i<n;i++)
	{
	QString name=list[i];
	int id=colIndex(name);
	selectedCol=id;
	clearCol();
	}
}
else
{
QTableSelection sel=worksheet->selection(worksheet->currentSelection());
int top=sel.topRow();
int bottom=sel.bottomRow();
int left=sel.leftCol();
int right=sel.rightCol();
	
if (sel.isEmpty ())
	{
	int row=worksheet->currentRow ();
	int col=worksheet->currentColumn ();
	worksheet->clearCell(row,col);
	worksheet->updateCell(row,col);
		
	QString name=colName(col);
	emit modifiedData(name);		
	}
else
	{	
	for (i=top;i<=bottom;i++)
		{
		for (int j=left;j<=right;j++)
			{
			worksheet->clearCell(i,j);
			worksheet->updateCell(i,j);
			}
		}

	for (i=left;i<=right;i++)
		{
		QString name=colName(i);
		emit modifiedData(name);
		}
	}
}
emit modifiedTable(this);
}

void WksheetWindow::copySelection()
{
QString text;
int i,j;
int rows=worksheet->numRows();
int cols=worksheet->numCols();

QArray<int> selection(1);
int c=0;	
for (i=0;i<cols;i++)
	{
	if (worksheet->isColumnSelected(i,TRUE))
		{
		c++;
		selection.resize(c);
		selection[c-1]=i;			
		}
	}
if (c>0)
	{	
	for (i=0;i<=rows;i++)
		{
		for (j=0;j<c-1;j++)
			text+=worksheet->text(i,selection[j])+"\t";
		text+=worksheet->text(i,selection[c-1])+"\n";
		}	
	}
else
	{
	QTableSelection sel=worksheet->selection(worksheet->currentSelection());
	int top=sel.topRow();
	int bottom=sel.bottomRow();
	int left=sel.leftCol();
	int right=sel.rightCol();
	for (i=top;i<=bottom;i++)
		{
		for (j=left;j<right;j++)
			text+=worksheet->text(i,j)+"\t";
		text+=worksheet->text(i,right)+"\n";
		}
	}		
	
// Copy text into the clipboard
QApplication::clipboard()->setData(new QTextDrag(text,worksheet,0));
}

void WksheetWindow::pasteSelection()
{
// Copy text from the clipboard (paste)
QString text;		
if ( QTextDrag::decode(QApplication::clipboard()->data(), text) && ! text.isNull())
{	
QStringList rowTexts=QStringList::split ("\n",text,TRUE);
int r=int(rowTexts.count());//-1;
QStringList cellTexts=QStringList::split ("\t",rowTexts[0],TRUE);
int c=int(cellTexts.count());
	
int i,j,top,bottom,right,left, firstCol=firstSelectedColumn();
	
QTableSelection sel=worksheet->selection(worksheet->currentSelection());
if (!sel.isEmpty())
	{
	top=sel.topRow();
	bottom=sel.bottomRow();
	left=sel.leftCol();
	right=sel.rightCol();
	}
else if (firstCol>=0)
	{
	top=0;
	bottom=r;	
	left=firstCol;
	right=firstCol+c;
	
	// If the selected columns is more than the existed ones, add more columns
	int tableCols=worksheet->numCols();
	if (right>tableCols)
		{
		int delta=right>tableCols;
		for(i=0;i<delta;i++)
		addCol();
		}
		//right=tableCols-1;
	}
else
	{
	top=0;
	bottom=r;	
	left=0;
	right=c;
	
	int tableCols=worksheet->numCols();
	if (right>tableCols)
		{
		int delta=right>tableCols;
		for(i=0;i<delta;i++)
		addCol();
		}
		//right=tableCols-1;
	}
	
int col=right-left+1;

QRegExp digit = QRegExp("\\d", FALSE, FALSE);
QStringList allCells;
bool HEADER=FALSE;
for (i=0;i<r;i++)
	{
	cellTexts=QStringList::split ("\t",rowTexts[i],TRUE);
	//If the first line has characters, use it to name the columns
	if (i==0)
		{
		for(j=0;j<c;j++)
		if(cellTexts[j].find(digit,0)!=0) HEADER=TRUE;
		if(HEADER)		
			{
			QString s;
			s="Do you want to change the name of the columns to<p>";
			for(j=0;j<c;j++)
			s+="<b>'"+cellTexts[j]+"'</b>"+"   ";
			switch( QMessageBox::information(0,"QtiPlot",s,"Yes", "No",0,0) ) 
				{	
				case 0:
					for(j=0;j<c;j++)
					worksheet->horizontalHeader()->setLabel(j,cellTexts[j],-1);
					break;
				case 1:
					break;
				}
			}
		}
	if(i>0 || (i==0 && !HEADER))
		{
		for (j=0;j<c;j++)					
			allCells<<cellTexts[j];
		}
	}
if(HEADER) r=r-1; //Since the first row was used to name the columns

if (r>(bottom-top+1) || c>col)
	{
	switch( QMessageBox::information(0,"QtiPlot",
		   "The text in the clipboard is larger than your current selection!\
	        \nDo you want to adjust it to current selection?",
				      "Yes", "Cancel",
				      0,0) ) 
		{
    	case 0:	
			for (i=top;i<=bottom;i++)
				{
				for (j=left;j<=right;j++)					
					worksheet->setText(i,j,allCells[(i-top)*c+j-left]);
				}
			for (i=left;i<=right;i++)
				{
				QString name=colName(i);
				emit modifiedData(name);
				}
		break;

    	case 1:
		break;
		}
	}
else
	{
	bool newRow=FALSE;
	int delta;
	if(r>worksheet->numRows())
		{
		delta=r-worksheet->numRows();
		worksheet->insertRows(worksheet->numRows(),r-worksheet->numRows());
		newRow=TRUE;
		emit modifiedTable(this);
		}
		
	for (i=top;i<top+r;i++)
		{
		for (j=left;j<left+c;j++)					
			worksheet->setText(i,j,allCells[(i-top)*c+(j-left)]);
		}
			
	for (i=left;i<left+c;i++)
		{
		QString name=colName(i);
		emit modifiedData(name);
		}	
	if(newRow)
		{
		for(i=worksheet->numRows()-delta+1;i<worksheet->numRows();i++)
			{
			if(isEmptyRow(i))
				{
				setSelectedRow(i);
				deleteRow();
				}
			}
		}
	}
}
emit modifiedTable(this);
}

void WksheetWindow::removeCol()
{
QStringList list=selectedColumns();
removeCol(list);
}

void WksheetWindow::removeCol(const QStringList& list)
{
QHeader *header=worksheet->horizontalHeader();
QString xColName=colName(xCol);	
QString yColName=colName(yCol);	
	
int i;
for (i=0;i<int(list.count());i++)
	{
	QString name=list[i];
	int id=colIndex(name);
	worksheet->removeColumn(id);

	QStringList::Iterator it=commandes.at(id);
	commandes.remove (it);
	emit removedCol(name);
	}

int cols=worksheet->numCols();	

if (list.contains(xColName) == 0)
	xCol=colIndex(xColName);	
else 
	{
	if (yCol > 1)
		xCol=0;
	else if (yCol <= 1)
		{		
		if (cols >=2)
				xCol=1;
		else
				xCol=0;
		}
		
	xColName=header->label(xCol).remove("[X]").remove("[Y]");
	setColName(xCol,xColName);
	}
	
if (list.contains(yColName) == 0)
	yCol=colIndex(yColName);	
else 
	{
	if (xCol != 1 && cols>=2)
		yCol=1;
	else if (xCol == 1 && cols>=2)	
		yCol=0;
	else
		yCol=-1;
		
	yColName=header->label(yCol).remove("[Y]");
	setColName(yCol,yColName);
	}
	
emit modifiedTable(this);
}

void WksheetWindow::normalizeSelection()
{
QStringList s=selectedColumns();
	
for (int i=0; i<(int)s.count(); i++)
	{
	selectedCol=colIndex(s[i]);
	normalizeCol();
	}
	
emit modifiedTable(this);	
}

void WksheetWindow::normalizeTable()
{
for (int i=0; i<worksheet->numCols(); i++)
	{
	selectedCol = i;
	normalizeCol();
	}
	
emit modifiedTable(this);	
}

void WksheetWindow::normalizeCol()
{
double max=worksheet->text(0,selectedCol).toDouble();
double aux=0.0;
int i;
QString text;
int rows=worksheet->numRows();
for (i=0; i<rows; i++)
	{
	text=worksheet->text(i,selectedCol);
	aux=text.toDouble();
	if (!text.isEmpty() && fabs(aux)>fabs(max)) 
		max=aux;
	}
	
if (max == 1.0)
	return;
	
for (i=0;i<rows;i++)
	{
	text=worksheet->text(i,selectedCol);
	aux=text.toDouble();
	if ( !text.isEmpty() )
		worksheet->setText(i,selectedCol,QString::number(aux/max));
	}

QString name=colName(selectedCol);
emit modifiedData(name);
}

void WksheetWindow::sortManyColDialog()
{
QStringList s=selectedColumns();
sortDialog *sortd=new sortDialog(this,"sortDialog",TRUE,WStyle_Tool|WDestructiveClose);
connect (sortd,SIGNAL(options(const QString&, const QString&, const QString&)),
		 this,SLOT(sortManyCol(const QString&, const QString&, const QString&)));
sortd->insertColumnsList(s);
sortd->showNormal();
sortd->setActiveWindow();
}

void WksheetWindow::sortTableDialog()
{
sortDialog *sortd=new sortDialog(this,"sortDialog",TRUE,WStyle_Tool|WDestructiveClose);
connect (sortd,SIGNAL(options(const QString&, const QString&, const QString&)),
		 this,SLOT(sortTable(const QString&, const QString&, const QString&)));
sortd->insertColumnsList(colNames());
sortd->showNormal();
sortd->setActiveWindow();
}

void WksheetWindow::sortTable(const QString& type, const QString& order, 
																	const QString& leadCol)
{
sortManyCol(colNames(), type, order, leadCol);
}

void WksheetWindow::sortManyCol(const QString& type, const QString& order, 
																	const QString& leadCol)
{
sortManyCol(selectedColumns(), type, order, leadCol);
}

void WksheetWindow::sortManyCol(const QStringList&s, const QString& type, const QString& order, 
																	const QString& leadCol)
{
int cols=s.count();
if(type=="Separately")
	{
	for(int i=0;i<cols;i++)
		{
		selectedCol=colIndex(s[i]);
		if(order=="Ascending")
			sortColAsc();
		else
			sortColDesc();
		}
	}
else if (type == "Together")
	{
	int i,j, leadcol=colIndex(leadCol);
	int rows=worksheet->numRows();
	QArray<double> r(rows), rtemp(rows);
	// Find the permutation index for the lead col
	size_t *p;
	p=ivector(0,rows-1);
	for (j = 0; j <rows; j++)
			r[j]=worksheet->text(j,leadcol).toDouble();

	gsl_sort_index(p,r,1,rows);
	// Since we have the permutation index, sort all the columns
	for(i=0;i<cols;i++)
		{
		int scol=colIndex(s[i]);
		if (!isEmptyColumn(scol))
			{
			for (j = 0; j <rows; j++)
	 			r[j]=worksheet->text(j,scol).toDouble();

			for (j=0;j<rows;j++)
				{
				int aux=p[j];
				rtemp[j]=r[aux];
				}
			for (j=0;j<rows;j++)
				r[j]=rtemp[j];
		
			if(order=="Ascending")
				{
				for (j=0;j<rows;j++)
					worksheet->setText(j,scol,QString::number(r[j])); 
				}
			else
				{
				for (j=0;j<rows;j++)
					worksheet->setText(j,scol,QString::number(r[rows-j-1])); 
				}
			emit modifiedData(colName(scol));
			}
		}
	free_ivector(p,0,rows-1);
	}
emit modifiedTable(this);	
}

void WksheetWindow::sortColAsc()
{
//changed from version 0.5.9
int rows=worksheet->numRows();
QArray<int> aux(rows);
QArray<double> r(rows);
int i,n=0;
for (i = 0; i <rows; i++)
	{
	QString text=worksheet->text(i,selectedCol);
	if (!text.isEmpty())
		{
		n++;
		aux[i]=i;
		r[i]=text.toDouble();
		}
	else
		{
		aux[i]=-1;
		r[i]=0.0;
		}
	}

if (!n)
	return;

gsl_vector * v = gsl_vector_alloc (n);
int index;
n=0;
for (i = 0; i <rows; i++)
	{
	index=aux[i];
	if (index>=0)
		{
   		gsl_vector_set (v, n, r[index]);
		n++;
		}	
	}

gsl_sort_vector (v);
n=0;
for (i=0;i<rows;i++)
	{
	index=aux[i];
	if (index>=0)
		{
		worksheet->setText(i,selectedCol,QString::number(gsl_vector_get (v, n))); 
		n++;
		}
	}
gsl_vector_free (v);
QString name=colName(selectedCol);
emit modifiedData(name);
emit modifiedTable(this);
}

void WksheetWindow::sortColDesc()
{//changed from version 0.5.9
int rows=worksheet->numRows();
QArray<int> aux(rows);
QArray<double> r(rows);
int i,n=0;
for (i = 0; i <rows; i++)
	{
	QString text=worksheet->text(i,selectedCol);
	if (!text.isEmpty())
		{
		n++;
		aux[i]=i;
		r[i]=text.toDouble();
		}
	else
		{
		aux[i]=-1;
		r[i]=0.0;
		}
	}

if (!n)
	return;

gsl_vector * v = gsl_vector_alloc (n);
int index;
n=0;
for (i = 0; i <rows; i++)
	{
	index=aux[i];
	if (index>=0)
		{
   		gsl_vector_set (v, n, r[index]);
		n++;
		}	
	}

gsl_sort_vector (v);

for (i=0;i<rows;i++)
	{
	index=aux[i];
	if (index>=0)
		{
		n--;
		worksheet->setText(i,selectedCol,QString::number(gsl_vector_get (v, n))); 
		}
	}
gsl_vector_free (v);
QString name=colName(selectedCol);
emit modifiedData(name);
emit modifiedTable(this);
}

int WksheetWindow::tableRows()
{
return worksheet->numRows();
}

int WksheetWindow::tableCols()
{
return worksheet->numCols();
}

bool WksheetWindow::isEmptyRow(int row)
{
bool empty=TRUE;
int cols=worksheet->numCols();
for (int i=0;i<cols;i++)
	{
	QString text=worksheet->text(row,i);
	if (!text.isEmpty())
		{
		empty=FALSE;
		break;
		}
	}	
return empty;
}

bool WksheetWindow::isEmptyColumn(int col)
{
bool empty=TRUE;
int rows=worksheet->numRows();
for (int i=0;i<rows;i++)
	{
	QString text=worksheet->text(i,col);
	if (!text.isEmpty())
		{
		empty=FALSE;
		break;
		}
	}	
return empty;
}

QString WksheetWindow::saveText()
{
QString text;
int cols=worksheet->numCols();
int rows=worksheet->numRows();
	
for (int i=0;i<rows;i++)
	{
	if (!isEmptyRow(i))
		{
		text+=QString::number(i)+"\t";
		for (int j=0;j<cols-1;j++)
			text+=worksheet->text(i,j)+"\t";
		
		text+=worksheet->text(i,cols-1)+"\n";
		}
	}	
return text;
}

int WksheetWindow::nonEmptyRows()
{
int r=0; 	
for (int i=0;i<worksheet->numRows();i++)
	{
	if (!isEmptyRow(i))
		r++;
	}	
return r;
}

QString WksheetWindow::text(int row, int col)
{
return worksheet->text(row,col).replace(",", ".");
}

void WksheetWindow::setText (int row, int col, const QString & text )
{
worksheet->setText (row,col,text); 
}

void WksheetWindow::freeMatrix()
{
int rows=worksheet->numRows();
int cols=worksheet->numCols();
free_matrix(wMatrix,0,rows-1,0,cols-1);
}

void WksheetWindow::setMatrix()
{
int rows=worksheet->numRows();
int cols=worksheet->numCols();
wMatrix=matrix(0,rows-1,0,cols-1);
int i,j;
bool ok;
	for (i=0;i<rows;i++)
	{
	for (j=0;j<cols;j++)
		wMatrix[i][j]=worksheet->text(i,j).toDouble(&ok); 
	}
}

void WksheetWindow::setNumericFormat(int f,int precType,const QString& prec,bool applyToAll)
{
int rows=worksheet->numRows();
int cols=worksheet->numCols();
QString t=prec;
bool ok;
int p=t.toInt(&ok);
int i,j;

if (applyToAll)
{
for (i=0;i<rows;i++)
	{
	for (j=selectedCol;j<cols;j++)
		{
		if (f==0) 
			{
			if (precType)
			t=t.setNum(wMatrix[i][j],'f',p);
			else 
			t=t.setNum(wMatrix[i][j],'f');

			worksheet->setText(i,j,t);
			}
		else if (f==1)
			{
			if (precType)
			t=t.setNum(wMatrix[i][j],'e',p);
			else 
			t=t.setNum(wMatrix[i][j],'e');

			worksheet->setText(i,j,t);
			}
		}
	}
}
else
{
for (i=0;i<rows;i++)
	{
	if (f==0) 
		{
		if (precType)
			t=t.setNum(wMatrix[i][selectedCol],'f',p);
		else 
			t=t.setNum(wMatrix[i][selectedCol],'f');

		worksheet->setText(i,selectedCol,t);
		}
	else if (f==1)
		{
		if (precType)
			t=t.setNum(wMatrix[i][selectedCol],'e',p);
		else 
			t=t.setNum(wMatrix[i][selectedCol],'e');

		worksheet->setText(i,selectedCol,t);
		}
	}
}
emit modifiedTable(this);
}

void WksheetWindow::setRandomValues()
{	
QApplication::setOverrideCursor(waitCursor);

int i;
double max=0.0;
int rows=worksheet->numRows();
QStringList list=selectedColumns();
QArray<double> r(rows);

for (int j=0;j<(int) list.count(); j++)
	{
	QString name=list[j];
	selectedCol=colIndex(name);

	srand(rand());

	for (i=0;i<rows;i++)
		{
		r[i]=rand();
		if (max<r[i]) max=r[i];
		}

	for (i=0;i<rows;i++) 
		{
		r[i]/=max;
		worksheet->setText(i,selectedCol,QString::number(r[i]));	
		}

	emit modifiedData(name);
	}

emit modifiedTable(this);
QApplication::restoreOverrideCursor();
}

void WksheetWindow::setHeader(QStringList header)
{
QHeader *head=worksheet->horizontalHeader();
for (int i=0;i<(int)worksheet->numCols();i++) 
	head->setLabel(i,header[i],-1);

QString xColName=head->label(xCol);
if (xColName.contains("[X]", TRUE) == 0)
	head->setLabel(xCol, xColName+"[X]");

QString yColName=head->label(yCol);
if (yColName.contains("[Y]", TRUE) == 0)
	head->setLabel(yCol, yColName+"[Y]");
}

void WksheetWindow::setAscValues()
{
QApplication::setOverrideCursor(waitCursor);

int rows=worksheet->numRows();
QStringList list=selectedColumns();

for (int j=0;j<(int) list.count(); j++)
	{
	QString name=list[j];
	selectedCol=colIndex(name);

	for (int i=0;i<rows;i++) 
		worksheet->setText(i,selectedCol,QString::number(i+1));

	emit modifiedData(name);
	}

emit modifiedTable(this);
QApplication::restoreOverrideCursor();
}

QString WksheetWindow::parseComand(int line,const QString& comand, 
														const QStringList& colLabels, const QStringList& rowIndexes)
{
QString s,aux=comand;
QStringList labels=colLabels;
double result;

int m=(int)colLabels.count();
QMemArray<double> vars(m); 

bool error=FALSE;
MathUtils::Parser parser, rparser;
try
    {
	for (int i=0; i<m; i++)
		{	
		parser.AddVar(colLabels[i].ascii(), &vars[i]);	
			
		int row=line;
		if (rowIndexes[i] != "i")
			{
			double l=(double)(line + 1);
			rparser.ClearVar();
			rparser.AddVar("i", &l);
			rparser.SetFormula(rowIndexes[i].ascii());
	        row=qRound(rparser.Calc()) - 1;
			if (row  < 0 && row >= worksheet->numRows())
				return "";
			}
		
		QStringList list=QStringList::split("_", labels[i], FALSE);	
		int col=list[1].toInt();
		s=worksheet->text(row, col);
		if (!s.isEmpty())
				vars[i]=s.toDouble();
		else 
				return "";
		}

	if (aux.contains("pi")>0)
		parser.AddConst("pi", M_PI);
	if (aux.contains("Pi")>0)
		parser.AddConst("Pi", M_PI);
	if (aux.contains("PI")>0)
		parser.AddConst("PI", M_PI);

	double index = (double)(line+1);
	parser.AddVar("i", &index);
	parser.SetFormula(aux.ascii());
	result=parser.Calc();
	}
catch(MathUtils::ParserException &e)
	{
	QMessageBox::critical(0,"QtiPlot - Input function error",e.what());
	error=TRUE;	
    }

if (error)
	return "";
else
	return QString::number(result);
}

void WksheetWindow::setColValues(const QString& text, const QString& com, 
										const QStringList& colLabels, const QStringList& rowIndexes,
										int startRow, int endRow)
{
int i;
int rows=worksheet->numRows();

if (endRow>rows) 
		{
		rows=endRow;
		worksheet->setNumRows(rows);
		}

for (i=startRow-1;i<endRow;i++) 
		worksheet->setText(i,selectedCol,parseComand(i, com, colLabels, rowIndexes));
	
commandes[selectedCol]=text;

QString name=colName(selectedCol);
emit modifiedData(name);
emit modifiedTable(this);
}

void WksheetWindow::setSelectedRow(int row)
{
clickedRow=row;
}

void WksheetWindow::setSelectedCol(int col)
{
selectedCol=col;
}

void WksheetWindow::plotL()
{
if (worksheet->numCols()<2)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need at least two columns for this operation!");
	return;
	}

QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this, s,Graph::Line);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void WksheetWindow::plotP()
{
if (worksheet->numCols()<2)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need at least two columns for this operation!");
	return;
	}

QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this, s,Graph::Scatter);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void WksheetWindow::plotLP()
{
if (worksheet->numCols()<2)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need at least two columns for this operation!");
	return;
	}

QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this, s, Graph::LineSymbols);
else
	QMessageBox::warning(0,"QtiPlot - Plot Error","Please select a column to plot!");
}

void WksheetWindow::plotVB()
{
if (worksheet->numCols()<2)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need at least two columns for this operation!");
	return;
	}

QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this, s,Graph::VerticalBars);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void WksheetWindow::plotHB()
{
if (worksheet->numCols()<2)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need at least two columns for this operation!");
	return;
	}

QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this, s,Graph::HorizontalBars);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void WksheetWindow::plotArea()
{
if (worksheet->numCols()<2)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need at least two columns for this operation!");
	return;
	}

QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this, s,Graph::Area);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void WksheetWindow::plotPie()
{
QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this,s, Graph::Pie);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void WksheetWindow::plotVerticalDropLines()
{
QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this,s,Graph::VerticalDropLines);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void WksheetWindow::plotSpline()
{
QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this,s,Graph::Spline);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void WksheetWindow::plotSteps()
{
QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this,s,Graph::Steps);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void WksheetWindow::plotHistogram()
{
QStringList s=selectedColumns();
if (int(s.count())>0)
	emit plotCol(this,s,Graph::Histogram);
else
	QMessageBox::warning(0,"QtiPlot","Please select a column to plot!");
}

void WksheetWindow::plotVectXYXY()
{
if (worksheet->numCols()<2)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need at least two columns for this operation!");
	return;
	}

QStringList s=selectedColumns();
int cols=int(s.count());
if (cols == 2 || cols == 4)
	emit plotCol(this, s,Graph::VectXYXY);
else
	QMessageBox::warning(0,"QtiPlot","Please select two or four columns for this operation!");
}

void WksheetWindow::plot3DRibbon()
{
if (worksheet->numCols()<2)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need at least two columns for this operation!");
	return;
	}
	
if (xCol<0)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need to define a X column first!");
	return;
	}
	
if (selectedCol>=0)
	{
	QString s=this->caption()+"_"+worksheet->horizontalHeader()->label(selectedCol);
	emit plot3DRibbon(this,s);
	}
}

void WksheetWindow::plot3DBars()
{
if (!validate3DPlot())
	return;
if (selectedCol>=0)
	{
	QString s=this->caption()+"_"+worksheet->horizontalHeader()->label(selectedCol);
	emit plotXYZ(this,s,2);
	}
}

void WksheetWindow::plot3DScatter()
{
if (!validate3DPlot())
	return;
if (selectedCol>=0)
	{
	QString s=this->caption()+"_"+worksheet->horizontalHeader()->label(selectedCol);
	emit plotXYZ(this, s, 0);
	}
}

void WksheetWindow::plot3DTrajectory()
{
if (!validate3DPlot())
	return;

if (selectedCol>=0)
	{
	QString s=this->caption()+"_"+worksheet->horizontalHeader()->label(selectedCol);
	emit plotXYZ(this,s,1);
	}
}

bool WksheetWindow::validate3DPlot()
{
if (worksheet->numCols()<2)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need at least two columns for this operation!");
	return false;
	}
if (xCol<0)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need to define a X column first!");
	return false;
	}
if (yCol<0)
	{
	QMessageBox::critical(0,"QtiPlot - Plot Error","You need to define a Y column first!");
	return false;
	}
return true;
}

int WksheetWindow::selectedColumn()
{
return selectedCol;
}

QString WksheetWindow::selectedColName()
{
QString name=worksheet->horizontalHeader()->label(selectedCol);
if (selectedCol == xCol)
	return name.remove("[X]");
else if (selectedCol == yCol)
	return name.remove("[Y]");
else
	return name;
}

QString WksheetWindow::selectedColComand()
{
return commandes[selectedCol];
}

QString WksheetWindow::selectedColWidth()
{
int w=worksheet->columnWidth (selectedCol);
return QString::number(w);
}

void WksheetWindow::importASCII(const QString &fname,const QString &sep, int ignoredLines)
{
QFile f(fname);
QTextStream t( &f );        // use a text stream

int rows = 1;
int cols = 0;

int i, dataStart = ignoredLines;
QRegExp autoSep= QRegExp("[^eE.\\d]*\\s", FALSE, FALSE);
QString s;
QStringList line;

if ( f.open(IO_ReadOnly) )
	{
    for (i=0; i<ignoredLines; i++)
		t.readLine();
		
	bool ok = false;
	while ( !t.atEnd() ) 
		{
		s = t.readLine(); 

		if (sep == "auto")
			line = QStringList::split(autoSep, s, false );
		else
			line = QStringList::split(sep, s, false );

		(line[0]).toDouble(&ok);
		if (ok)
			{
			cols = (int)line.count();
			break;
			}
		dataStart++;
		}

	while ( !t.atEnd() ) 
		{
		t.readLine(); 
		rows++;
		}

	init (rows, cols);

	f.reset();
	for (i=0; i<dataStart; i++)
		s = t.readLine();
		
	if ((dataStart - ignoredLines)>0)
		{
		// Use line before first data valid line to name the columns
		if (sep == "auto")
			{
			QRegExp headerSep= QRegExp("[:;,]*\\s", FALSE, FALSE);
			line=QStringList::split (headerSep,s,TRUE);
			}
		else
			line = QStringList::split(sep, s, true );	
	
		setHeader(line);
		}
		
	int row = 0;
	while ( !t.atEnd() ) 
		{
		s = t.readLine(); 
		if (sep == "auto")
			line = QStringList::split(autoSep, s, true );
		else
			line = QStringList::split(sep, s, true );

		for (int j=0; j<cols; j++)
			worksheet->setText(row, j, line[j]);
		
		row++;
		}
	f.close();
	}
}

void WksheetWindow::exportAscii(const QString& separator,bool withLabels,bool exportSelection)
{
QString selectedFilter;
	
QString fname = QFileDialog::getSaveFileName( QString::null,"*.txt;;*.dat;;*.DAT", 0,"file dialog",
	tr("Choose a filename to save under"),&selectedFilter,TRUE);
	
if (!fname.isEmpty() ) 
	{ // the user gave a file name	
	QFileInfo fi(fname);
	QString baseName = fi.fileName();
		
	if (baseName.contains(".")==0)
			fname.append(selectedFilter.remove("*"));	
	
	if ( QFile::exists(fname) &&
        	QMessageBox::question(
            0,
            tr("QtiPlot - Overwrite File?"),
            tr("A file called: <p><b>%1</b><p>already exists. "
                "Do you want to overwrite it?")
                .arg(fname),
            tr("&Yes"), tr("&No"),
            QString::null, 0, 1 ) )
        			return ;
	else
	{
	QFile f(fname);
	if ( !f.open( IO_WriteOnly ) ) 
		{
		QMessageBox::critical(0, tr("QtiPlot - ASCII Export Error"),tr("Could not write to file: <br><h4>"+fname+ "</h4><p>Please verify that you have the right to write to this location!").arg(fname));
   		 return;
		}
		
	QString text;
	int i,j,k,l=0,s=0;
	int rows=worksheet->numRows();
	int cols=worksheet->numCols();

	if (withLabels)
		{
		QStringList header=colNames();
			if (exportSelection)
			{
			for (i=0;i<rows;i++)
				{
				if (worksheet->isRowSelected(i,FALSE)) 
					{
					l=i;
					break;
					}
				}

			for (j=0;j<cols;j++)
				{
				if (worksheet->isSelected(l,j))
					{
					text+="'"+header[j]+"'";
					for (i=j+1;i<cols;i++)
						{
						if (worksheet->isSelected(l,i)) s++;
						}
					if (s>0) text+=separator;
					else text+="\n";
					s=0;
					}
				}
			}
			else
			{
			for (j=0;j<cols-1;j++) text+="'"+header[j]+"'"+separator;
			text+="'"+header[cols-1]+"'"+"\n";		
			} 
	}// end labels

	if (exportSelection)
		{
		for (k=0;k<rows;k++)
			{
			for (j=0;j<cols;j++)
				{
				if (worksheet->isSelected(k,j))
					{
					text+=worksheet->text(k,j);
					for (i=j+1;i<cols;i++)
						{
					if (worksheet->isSelected(k,i)) s++;
						}
					if (s>0) text+=separator;
					else text+="\n";
					s=0;
					}
				}
			}
		}
	else
		{
		for (k=0;k<rows;k++)
			{
			for (j=0;j<cols-1;j++)
				{
				text+=worksheet->text(k,j)+separator;
				}
			text+=worksheet->text(k,cols-1)+"\n";
			}
		}
	QTextStream t( &f );
    t << text;
    f.close();
	}
}
}

void WksheetWindow::resizeEvent(QResizeEvent *)
{
emit resizedTable(this);
}

void WksheetWindow::contextMenuEvent(QContextMenuEvent *e)
{
emit showContextMenu();
e->accept();
}

void WksheetWindow::closeEvent( QCloseEvent *e )
{
   /* bool empty=FALSE;
    for(int i=0;i<worksheet->numCols();i++)
    	{
	if(i==0 || (i>0 && empty))
    	empty=isEmptyColumn(i);
	}
    if (!empty)*/
	if (askOnClose)
    {
    switch( QMessageBox::information(0,tr("QtiPlot"),
				      tr("Do you want to hide or delete <p><b>'"
					   +this->caption()+"'</b> ?"),
				      "Delete", "Hide", "Cancel",
				      0,2) ) 
	{
    case 0:	
	e->accept();
	emit closedTable(this);
	break;

    case 1:
	e->ignore();
	emit hiddenTable(this);
	break;

	case 2:
	e->ignore();
	break;
    } 
    }
    else 
    {
    e->accept();
    emit closedTable(this);
    }
}

void WksheetWindow::moveCurrentCell()
{
int cols=worksheet->numCols();
int row=worksheet->currentRow();
int col=worksheet->currentColumn();
worksheet->clearSelection (TRUE);
	
if (col+1<cols)
	{
	worksheet->setCurrentCell (row,col+1);
	worksheet->selectCells(row,col+1,row,col+1);
	}
else
	{
	worksheet->setCurrentCell (row+1,0);
	worksheet->selectCells(row+1,0,row+1,0);	
	}
}

bool WksheetWindow::eventFilter(QObject *object, QEvent *e)
{
if ( object != (QObject *)worksheet->horizontalHeader())
	return FALSE;

QHeader *header = worksheet->horizontalHeader();
int offset =header->offset();
switch(e->type())
    {
        case QEvent::MouseButtonDblClick:
			{
            const QMouseEvent *me = (const QMouseEvent *)e;
			selectedCol = header->sectionAt (me->pos().x()+offset);
			emit optionsDialog();			
			return TRUE; 
			}

		case QEvent::MouseButtonPress:
			{
			const QMouseEvent *me = (const QMouseEvent *)e;
			if (me->button() == QMouseEvent::RightButton)	
				{
				const QMouseEvent *me = (const QMouseEvent *)e;
				selectedCol=header->sectionAt (me->pos().x() + offset);
				emit colMenu(selectedCol);
				}
			
			if (me->button() == QMouseEvent::LeftButton)	
				{
				LeftButton=TRUE;
				const QMouseEvent *me = (const QMouseEvent *)e;
				
				if (((const QMouseEvent *)e)->state ()==Qt::ControlButton)
					{
					int current=worksheet->currentSelection();
					QTableSelection sel=worksheet->selection(current);
					int	top=sel.topRow();
					int bottom=sel.bottomRow();
					int left=sel.leftCol();
					int right=sel.rightCol();					
				
					if (top != bottom && left != right) 							
						worksheet->removeSelection(sel);
				
					int r=worksheet->currentRow();
					if (worksheet->isRowSelected(r,TRUE))
						worksheet->removeSelection(QTableSelection (r,0,r,worksheet->numCols()-1));
						
					}
				else
					worksheet->clearSelection (TRUE);
				
				selectedCol=header->sectionAt (me->pos().x()+offset);
				lastSelectedCol=selectedCol;
				worksheet->selectColumn (selectedCol);
				}		
			return TRUE; 
			}
		
		case QEvent::MouseMove:
			{
			if(LeftButton)
				{
				const QMouseEvent *me = (const QMouseEvent *)e;
				selectedCol=header->sectionAt (me->pos().x() + offset);

				if(selectedCol != lastSelectedCol)
					{// This means that we are in the next column
					if(worksheet->isColumnSelected(selectedCol,TRUE))
						{//Since this column is selected, deselect it
						worksheet->removeSelection(QTableSelection (0,lastSelectedCol,worksheet->numRows()-1,lastSelectedCol));
						}
					else
						worksheet->selectColumn (selectedCol);
					}
				lastSelectedCol=selectedCol;
				}
			return TRUE;
			}
			
		case QEvent::MouseButtonRelease:
			{
			LeftButton=FALSE;
			return TRUE;
			}
			
			default:
				;
    }
return QObject::eventFilter(object, e);
}

void WksheetWindow::showColStatistics()
{
QString caption=this->caption()+"-statistics";
	
gsl_vector *y;
int rows=worksheet->numRows();
QStringList columns=selectedColumns();	
int n=(int)columns.count();
int colsNonEmpty=n;

QString text="Col Name\t";
text+="Rows\t";
text+="Mean\t";
text+="Standard Dev\t";
text+="Variance\t";	
text+="Sum\t";
text+="xMax\t";	
text+="Max\t";
text+="xMin\t";	
text+="Min\t";
text+="N\n";
	
for (int i=0;i<n;i++)
	{
	int index=colIndex(columns[i]);
	text+=columns[i]+"\t";
	text+="[1:"+QString::number(rows)+"]\t";
	
	int j,m=0;		
	for (j=0;j<rows;j++)
		{
		if (!worksheet->text(j,index).isEmpty())
			m++;	
		}

	if (m != 0)
		{
		double *dat;
		dat=vector(0,m-1);
		y = gsl_vector_alloc (m);
		
		for (j=0;j<m;j++)
		{
		double val=worksheet->text(j,index).toDouble();
		gsl_vector_set (y,j,val);
		dat[j]=val;			
		}
	
		double mean=gsl_stats_mean (dat,1,m);	
		text+=QString::number(mean)+"\t";
		text+=QString::number(gsl_stats_sd (dat,1,m))+"\t";
		text+=QString::number(gsl_stats_variance (dat,1,m))+"\t";
		text+=QString::number(mean*m)+"\t";

		text+=QString::number(gsl_vector_max_index (y)+1)+"\t";
		text+=QString::number(gsl_vector_max (y))+"\t";
		
		text+=QString::number(gsl_vector_min_index (y)+1)+"\t";
		text+=QString::number(gsl_vector_min (y))+"\t";	
		text+=QString::number(m)+"\n";	
		
		gsl_vector_free (y);
		free_vector(dat,0,m-1);
		}
	else
		colsNonEmpty--;
	}
	
emit createTable(caption,colsNonEmpty,11,text);
}

void WksheetWindow::showRowStatistics()
{
QString caption=this->caption()+"-statistics";
int rows=nonEmptyRows();
int cols=worksheet->numCols();
int r=0;
	
QString text="Row\t";
text+="Cols\t";
text+="Mean\t";
text+="Standard Dev\t";
text+="Variance\t";	
text+="Sum\t";
text+="iMax\t";	
text+="Max\t";
text+="iMin\t";	
text+="Min\t";
text+="N\n";

gsl_vector *y;	
for (int i=0;i<rows;i++)
	{
	if (worksheet->isRowSelected(i,TRUE))
		{
		r++;
		text+=QString::number(i+1)+"\t";
		text+="[1:"+QString::number(cols)+"]\t";
	
		int j,m=0;		
		for (j=0;j<cols;j++)
			{
			if (!worksheet->text(i,j).isEmpty())
				m++;	
			}

		if (m != 0)
			{
			double *dat;
			dat=vector(0,m-1);
			y = gsl_vector_alloc (m);
		
			for (j=0;j<m;j++)
				{
				double val=worksheet->text(i,j).toDouble();
				gsl_vector_set (y,j,val);
				dat[j]=val;			
				}
	
			double mean=gsl_stats_mean (dat,1,m);	
			text+=QString::number(mean)+"\t";
			text+=QString::number(gsl_stats_sd (dat,1,m))+"\t";
			text+=QString::number(gsl_stats_variance (dat,1,m))+"\t";
			text+=QString::number(mean*m)+"\t";

			text+=QString::number(gsl_vector_max_index (y)+1)+"\t";
			text+=QString::number(gsl_vector_max (y))+"\t";
		
			text+=QString::number(gsl_vector_min_index (y)+1)+"\t";
			text+=QString::number(gsl_vector_min (y))+"\t";	
			text+=QString::number(m)+"\n";	
		
			gsl_vector_free (y);
			free_vector(dat,0,m-1);
			}
		}
	}
	
emit createTable(caption,r,11,text);
}

QString WksheetWindow::sizeToString()
{
QString s=makeSpecifications();
int size=s.length()*sizeof(QString);

if (size>1024)
	return QString::number(s.length()*sizeof(QString)/1024)+" kB";
else
	return QString::number(s.length()*sizeof(QString))+" B";
}


QString WksheetWindow::makeSpecifications()
{
int cols=worksheet->numCols();
	
QString s="<table>\n";
s+=this->caption()+"\t";
s+=QString::number(worksheet->numRows())+"\t";
s+=QString::number(cols)+"\t";
s+=birthdate+"\t";	
s+=QString::number(xCol)+"\t";
s+=":\t";
s+=QString::number(yCol)+"\n";
s+="tgeometry\t";

QPoint	p =this->pos();// store position
s+=QString::number(p.x())+"\t";
s+=QString::number(p.y())+"\t";
s+=QString::number(this->frameGeometry().width())+"\t";
s+=QString::number(this->frameGeometry().height())+"\n";
s+="header\t";
	
for (int j=0;j<cols;j++)
	s+=worksheet->horizontalHeader()->label(j)+"\t";
s+="\n";
s+=saveColumnWidths();
s+=saveCommandes();
s+=saveText();
s+="</table>\n";
return s;
}

QString WksheetWindow::getSpecifications()
{
return specifications;
}

void WksheetWindow::setSpecifications(const QString& s)
{
if (specifications == s)
	return;

specifications=s;
}

void WksheetWindow::setNewSpecifications()
{
newSpecifications=makeSpecifications();
}

QString WksheetWindow::getNewSpecifications()
{
return newSpecifications;
}

QString WksheetWindow::oldCaption()
{
QTextStream ts( &specifications, IO_ReadOnly );
ts.readLine();
QString s=ts.readLine();
int pos=s.find("\t",0);
return s.left(pos);
}

QString WksheetWindow::newCaption()
{
QTextStream ts( &newSpecifications, IO_ReadOnly );
ts.readLine();
QString s=ts.readLine();
int pos=s.find("\t",0);
return s.left(pos);
}

void WksheetWindow::restore(const QString& spec)
{
int cols=worksheet->numCols();
int rows=worksheet->numRows();
int i,j;
	
QStringList l=QStringList::split ("\n",spec,FALSE);
QString s=l[1];
QStringList list=QStringList::split ("\t",s,TRUE);
	
QString oldCaption=this->caption();
QString newCaption=list[0];
if ( oldCaption != newCaption)
			this->setCaption(newCaption);
		
int r=list[1].toInt();
if (rows != r)
		worksheet->setNumRows (r);
		
int c=list[2].toInt();
if (cols != c)
		worksheet->setNumCols (c);

xCol=list[4].toInt();
if ((int)list.count()>=7)
	yCol=list[6].toInt();

for (i=0;i<rows;i++)
	{
	for (j=0;j<cols;j++)
			worksheet->clearCell(i,j);
	}

for (i=2;i<(int)l.count()-1;i++)
	{
	 s=l[i];
	list=QStringList::split ("\t",s,TRUE);
		
	if (s.contains ("header\t",TRUE)>0)
		{
		QStringList header;
		for (j=0;j<int(list.count());j++)
			header<<list[j+1];
		
		QStringList oldHeader= colNames();
		if ( oldHeader != header)
			{
			setHeader(header);
			header.gres("[X]","");
			header.gres("[Y]","");
			if (cols == c)
				{
				for (j=0;j<worksheet->numCols();j++)
					emit changedColHeader(oldCaption+"_"+oldHeader[j],newCaption+"_"+header[j]);
				}		
			}
		}			
	else if (s.contains ("com",TRUE)>0)
		{
		QStringList comm;
		for (j=0;j<int(list.count());j++)
			comm<<list[j+1];
		if (comm != commandes)
			commandes = comm;
		}		
	else if (s.contains ("ColWidth",TRUE)>0)
		{
		QStringList widths;
		for (j=0;j<int(list.count());j++)
			widths<<list[j+1];
		if (columnWidths() != widths)
			setColWidths(widths);
		}	
		
	else //set table values
		{
		for (j=0;j<worksheet->numCols();j++)
			worksheet->setText(list[0].toInt(),j,list[j+1]);	
		}
	} // end for
		
for (j=0;j<worksheet->numCols();j++)
	{
	QString name=colName(j);
	emit modifiedData(name);		
	}
}

void WksheetWindow::setNumRows(int rows)
{
worksheet->setNumRows(rows);
}

void WksheetWindow::updateNumRows(int rows)
{
if (worksheet->numRows() == rows)
	return;

if (worksheet->numRows() > rows)
	{
	QString text="The table <b>"+this->caption()+"</b> has more than "+QString::number(rows)+ " rows!";
	text+="<p>Do you really want to continue?";
	int i=0;
	switch( QMessageBox::information(0,"QtiPlot", text,"Yes", "Cancel", 0, 1 ) ) 
		{
    	case 0:
 				worksheet->setNumRows(rows);
				emit modifiedTable(this);
				for (i=0;i<worksheet->numCols();i++)
						{
						QString name=colName(i);
						emit modifiedData(name);		
						}
		break;
		
   	    case 1:
        	return;
		break;
    	}
	}
else
	{
	worksheet->setNumRows(rows);
	emit modifiedTable(this);	
	}
}

void WksheetWindow::setNumCols(int cols)
{
worksheet->setNumCols(cols);
}

void WksheetWindow::updateNumCols(int cols)
{
if (worksheet->numCols() == cols)
	return;

if (worksheet->numCols() > cols)
	{
	QString text="The table <b>"+this->caption()+"</b> has more than "+QString::number(cols)+ " columns!";
	text+="<p>Do you really want to continue?";
	int i=0;
	QStringList list;
	switch( QMessageBox::information(0,"QtiPlot", text,"Yes", "Cancel", 0, 1 ) ) 
		{
    	case 0:
				for (i=cols;i<worksheet->numCols();i++)
						{
						QString name=colName(i);
						list<<name;		
						}
				removeCol(list);
		break;
		
   	    case 1:
        	return;
		break;
    	}
	}
else
	{
	worksheet->setNumCols(cols);
	emit modifiedTable(this);
	}
}

WksheetWindow::~WksheetWindow()
{
}
