#ifndef WORKSHEET_H
#define WORKSHEET_H

#include <qwidget.h>
#include <qlist.h>
#include <qtable.h>
#include <qheader.h>
#include <qpopupmenu.h>

#include "graph.h"
	
class WksheetWindow: public QWidget
{
    Q_OBJECT

public:

	enum ColType{Numeric = 0, Text = 1};

   	WksheetWindow(const QString &fname,const QString &sep, int ignoredLines, QWidget* parent=0, const char* name=0, WFlags f=0);
	WksheetWindow(int r,int c, QWidget* parent=0, const char* name=0, WFlags f=0);
	~WksheetWindow();

public slots:

	QString sizeToString();
	int tableRows();
	void setNumRows(int rows);
	void updateNumRows(int rows);
	void setNumCols(int cols);
	void updateNumCols(int cols);

	int tableCols();
	QString text(int row, int col);
	QStringList colNames();
	QString colName(int col);
	int colIndex(const QString& name);
	void setColName(int col,const QString& text);
	void setHeader(QStringList header);
	void setText(int row,int col,const QString & text);
	void setColValues(const QString& text, const QString& com, 
										const QStringList& colLabels, const QStringList& rowIndexes,
										int startRow, int endRow);
	void setRandomValues();
	void setAscValues();

	QStringList getCommandes();
    void setCommandes(const QStringList& com);
	void setCommandes(const QString& com);
	void cellEdited(int,int col);
	void moveCurrentCell();
	void clearCell(int row, int col);
	QString saveText();
	bool isEmptyRow(int row);
	bool isEmptyColumn(int col);
	int nonEmptyRows();
	
	void plotL();
	void plotLP();
	void plotP();
	void plotVB();
	void plotHB();
	void plotArea();
	void plotPie();	
	void plotVerticalDropLines();
	void plotSpline();
	void plotSteps();
	void plotHistogram();
	void plotVectXYXY();
	
	//3D plots
	void plot3DRibbon();
	void plot3DScatter();
	void plot3DTrajectory();
	void plot3DBars();
	bool validate3DPlot();
	
	void print();
	
	// event handlers 
	void closeEvent( QCloseEvent *);
	void resizeEvent(QResizeEvent *);
	bool eventFilter(QObject *object, QEvent *e);
	void contextMenuEvent(QContextMenuEvent *e);
	
	// set col values
	QString parseComand(int line, const QString& comand, 
								const QStringList& colLabels, const QStringList& rowIndexes);
	
	// column operations 
	void removeCol();
	void removeCol(const QStringList& list);
	void clearCol();
	void insertCol();
	void addCol();	
	void sortTableDialog();
	void sortTable(const QString& type, const QString& order, 
																	const QString& leadCol);
	void sortColAsc();
	void sortColDesc();
	void sortManyCol(const QString& type, const QString& order, 
																	const QString& leadCol);
	void sortManyCol(const QStringList&s, const QString& type, const QString& order, 
																	const QString& leadCol);

	void sortManyColDialog();
	
	//normalization
	void normalizeCol();
	void normalizeSelection();
	void normalizeTable();

	QArray<double> col(int ycol);
	int colX();
	void setXCol();
	void setXCol(int i);
	
	int colY();
	void setYCol();
	void setYCol(int i);

	int atRow(int col, double value);
	void showColStatistics();
	void showRowStatistics(); 
	
	// row operations 
	void deleteRow();
	void insertRow();
	void setSelectedRow(int row);

	// selection operations 
	void cutSelection();
	void copySelection();
	void clearSelection();
	void pasteSelection();
	void selectAllTable();
	void deselect();
	bool singleCellSelected();

	void init(int rows, int cols);
	QStringList selectedColumns();
	void changeColName(const QString& text);
	void enumerateRightCols(bool checked);
	void changeColWidth(const QString& width, bool allCols);
	QString selectedColWidth();
	
	QStringList columnWidths();
	void setColWidths(const QStringList& widths);
	
	QString selectedColName();
	int selectedColumn();
	int firstSelectedColumn();
	void setSelectedCol(int col);
	QString selectedColComand();
	int selectedRows();
	
	void setMatrix();
	void freeMatrix();

	void setNumericFormat(int f, int precType, const QString& prec,bool applyToAll);
	QString birthDate();
	
	void exportAscii(const QString& separator,bool withLabels,bool exportSelection);
	void importASCII(const QString &fname,const QString &sep, int ignoredLines);

	//saving
	QString saveCommandes();
	QString saveColumnWidths();
	
	//restoring
	QString makeSpecifications();
	void setSpecifications(const QString& s);
	QString getSpecifications();
	void restore(const QString& spec);
	QString getNewSpecifications();
	void setNewSpecifications();
	
	/*!
	*used for restoring the table old caption stored in specifications string
	*/ 
	QString oldCaption();
	
	/*!
	*used for restoring the table caption stored in new specifications string
	*/ 
	QString newCaption();

	void setBackgroundColor(const QColor& col);
	void setTextColor(const QColor& col);
	void setHeaderColor(const QColor& col);
	void setTextFont(const QFont& fnt);
	void setHeaderFont(const QFont& fnt);
	
	void askOnCloseEvent(bool ask){askOnClose = ask; };
	
	int verticalHeaderWidth(){return worksheet->verticalHeader()->width();};
	int checkColType(int col);
			
signals:
	void plot3DRibbon(WksheetWindow *,const QString&);
	void plotXYZ(WksheetWindow *,const QString&, int);

	void plotCol(WksheetWindow *,const QStringList&, int);
	void closedTable(QWidget*);
	void changedColHeader(const QString&, const QString&);
	void removedCol(QString&);
	void modifiedData(const QString&);
	void modifiedTable(QWidget*);
	void optionsDialog();
	void colValuesDialog();
	void resizedTable(QWidget*);
	void hiddenTable(QWidget*);
	void rowMenu(int);
	void colMenu(int);
	void showContextMenu();
	void createTable(const QString&,int,int,const QString&);
	
private:
	QTable *worksheet;
	QString birthdate, specifications, newSpecifications;
	QStringList commandes;
	int selectedCol, lastSelectedCol, clickedRow,xCol,yCol;
	double **wMatrix;
	bool LeftButton;
	bool askOnClose;
};

#endif
