#include <qworkspace.h>
#include <qwidget.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qmovie.h>
#include <qmultilineedit.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qprinter.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qaccel.h>
#include <qtextstream.h>
#include <qpainter.h>
#include <qpaintdevicemetrics.h>
#include <qobjectlist.h>
#include <qvbox.h>
#include <qsplitter.h>
#include <qlistbox.h>
#include <qlist.h>
#include <qstringlist.h>
#include <qstring.h>
#include <qsize.h>
#include <qpoint.h>
#include <qnamespace.h>
#include <qinputdialog.h>
#include <qregexp.h>
#include <qtextview.h>
#include <qdatetime.h>
#include <qlistview.h>
#include <qstylesheet.h>
#include <qwidgetstack.h>
#include <qcursor.h>
#include <qtextbrowser.h>
#include <qevent.h>
#include <qaction.h>
#include <qprogressbar.h>
#include <qpixmapcache.h>
#include <qsettings.h>
#include <qstylefactory.h>

#include <qwt_plot.h>

#include "application.h"
#include "pixmaps.h"
#include "curvesDialog.h"
#include "plotDialog.h"
#include "genDialog.h"
#include "txtDlg.h"
#include "lineDlg.h"
#include "textDialog.h"
#include "exportDialog.h"
#include "tableDialog.h"
#include "valuesDialog.h"
#include "errDlg.h"
#include "LegendMarker.h"
#include "LineMarker.h"
#include "ImageMarker.h"
#include "importDialog.h"
#include "graph.h"
#include "pieDialog.h"
#include "plotWizard.h"
#include "polynomFitDialog.h"
#include "expDecayDialog.h"
#include "functionDialog.h"
#include "fitDialog.h"
#include "surfaceDialog.h"
#include "graph3D.h"
#include "plot3DDialog.h"
#include "imageDialog.h"
#include "multilayer.h"
#include "layerDialog.h"
#include "analysisDialog.h"
#include "intDialog.h"
#include "configDialog.h"
#include "rowsDialog.h"

#include <gsl/gsl_vector.h>

extern "C" {
double *vector(long nl, long nh);
void free_vector(double *v, long nl, long nh);
int *intvector(long nl, long nh);
void free_intvector(int *v, long nl, long nh);
}

ApplicationWindow::ApplicationWindow()
    : QMainWindow( 0, "main window", WDestructiveClose )
{
init();
}

ApplicationWindow::ApplicationWindow(const QString& filename)
    : QMainWindow( 0, "main window", WDestructiveClose )
{
QFileInfo f(filename);
if (!f.exists ())
	{
	QMessageBox::critical(this,"QtiPlot - File openning error",
				"The file: <p> <b>"+filename+"</b> <p> doesn't exist!");
	return;
	}

open (filename);
}

void ApplicationWindow::init()
{
QPixmapCache::setCacheLimit(20*QPixmapCache::cacheLimit ());

graphs=0; tables=0; fitNumber=0;
projectname="untitled";
separator="auto";
ignoredLines=0;
lastModified=0;
activeGraph=0;
selectorsActiveGraph=0;
copiedLayer=FALSE;
copiedMarkerType=Graph::None;
aw=0, wmax=0;
lastCopiedLayer=0;
logInfo="";
fd=0;
savingTimerId=0;
	
tableBkgdColor=QColor(white);
tableTextColor=QColor(black);
tableHeaderColor=QColor(black);
	
appFont = QApplication::font();
QString family = appFont.family();
int pointSize = appFont.pointSize();

tableTextFont=appFont;
tableHeaderFont=appFont;
plotAxesFont=QFont(family, pointSize, QFont::Bold, FALSE );
plotNumbersFont=QFont(family, pointSize - 2);
plotLegendFont=appFont;
plotTitleFont=QFont(family, pointSize + 2, QFont::Bold,FALSE);

workspaceColor=QColor(darkGray);
panelsColor=QColor(white);
panelsTextColor=QColor(black);

readSettings();

initAppStyle(appStyle);
this->setFont(appFont);
qApp->setFont(appFont);
initMainMenu();
initToolBars();
initPlot3DToolBar();

//set recent project list
int i;
for (i=0;i<(int)recentProjects.count();i++)
	recent->insertItem("&"+QString::number(i+1)+" "+recentProjects[i]);

QVBox* vb = new QVBox( this );
vb->setFrameStyle( QFrame::StyledPanel | QFrame::Sunken );

QSplitter *split = new QSplitter(vb);
split->setOrientation(QSplitter::Vertical );
split->setOpaqueResize( TRUE );

results=new QTextEdit(split,"results");
results->setReadOnly (TRUE);
results->setPaper ( QBrush (panelsColor, SolidPattern ) );
results->setPaletteForegroundColor (panelsTextColor);
results->hide();

ws = new QWorkspace( split );
ws->setScrollBarsEnabled (TRUE);
ws->setPaletteBackgroundColor (workspaceColor);

lv = new QListView( split,0 );
lv->addColumn ("Name",-1 );
lv->addColumn ("Type",-1 );
lv->addColumn ("View",-1 );
lv->addColumn ("Size",-1 );
lv->addColumn ("Created",-1);

int w=this->width()/5;
for (i=0;i<5;i++)
	lv->setColumnWidth(i,w);

lv->setMinimumHeight(80);
lv->setPaletteForegroundColor (panelsTextColor);
lv->setPaletteBackgroundColor (panelsColor);
lv->hide();
setCentralWidget( vb );

hiddenWindows=new QWidgetList();
hiddenWindows->setAutoDelete( TRUE );

QAccel *accel = new QAccel(this);
accel->connectItem( accel->insertItem( Key_F5 ),
                            ws, SLOT(activateNextWindow()) );
accel->connectItem( accel->insertItem( Key_F6 ),
                            ws, SLOT(activatePrevWindow()) );

connect(this, SIGNAL(modified()),this, SLOT(modifiedProject()));
connect(this, SIGNAL(windowClosed(const QString&)),this, SLOT(updateListView(const QString&)));
connect(ws, SIGNAL(windowActivated (QWidget*)),this, SLOT(windowActivated(QWidget*)));
connect(lv, SIGNAL(doubleClicked(QListViewItem *)),this, SLOT(maximizeWindow(QListViewItem *)));
connect(lv, SIGNAL(rightButtonClicked(QListViewItem *, const QPoint &, int)), this, SLOT(showWindowPopupMenu(QListViewItem *, const QPoint &, int)));
connect(recent, SIGNAL(activated(int)),this, SLOT(openRecentProject(int)));
}

void ApplicationWindow::initToolBars()
{
	setIcon(QPixmap(logo_xpm));
    QPixmap openIcon, saveIcon;

    fileTools = new QToolBar( this, "file operations" );
    addToolBar( fileTools, tr( "File" ), Top, TRUE );

	new QToolButton(QPixmap(new_xpm),"New project",
					   QString::null, this, SLOT(newProject()),
						fileTools,"");
	new QToolButton(QPixmap(table_xpm),"New worksheet",
					   QString::null, this, SLOT(newWrksheet()),
						fileTools,"");
	new QToolButton(QPixmap(import_xpm),"Import data file",
					   QString::null, this, SLOT(load()),
						fileTools,"");
	new QToolButton(QPixmap(multiload_xpm),"Import multiple data files",
					   QString::null, this, SLOT(loadMultiple()),
						fileTools,"");
	new QToolButton(QPixmap(duplicate_xpm),"Duplicate window",
					   QString::null, this, SLOT(copyWindow()),
						fileTools,"");
	fileTools->addSeparator ();

	openIcon = QPixmap(fileopen_xpm );

	new QToolButton( openIcon, "Open project", QString::null,
			   this, SLOT(open()), fileTools, "save file" );
	saveIcon = QPixmap(filesave_xpm);
	fileSave= new QToolButton( saveIcon, "Save project", QString::null,
			   this, SLOT(saveProject()), fileTools, "save file" );
	fileSave->setEnabled(FALSE);
	saved=TRUE;

	fileTools->addSeparator ();

	new QToolButton(QPixmap(newF_xpm),"New function plot",
					   QString::null, this, SLOT(newFunctionPlot()),
						fileTools,"");
	new QToolButton(QPixmap(newFxy_xpm),"New Surface 3D Plot",
						QString::null,this, SLOT(newSurfacePlot()),fileTools,"");

	fileTools->addSeparator ();

	btnUndo= new QToolButton(QPixmap(undo_xpm), "Undo", QString::null,
			   this, SLOT(undo()), fileTools, "undo" );
	btnUndo->setEnabled(FALSE);

	btnRedo= new QToolButton(QPixmap(redo_xpm), "Redo", QString::null,
			   this, SLOT(redo()), fileTools, "redo" );
	btnRedo->setEnabled(FALSE);

	fileTools->addSeparator();

	btnList = new QToolButton(QPixmap(folder_xpm),"Show project explorer",
					   QString::null, this, SLOT(showList()),fileTools,"");
	btnList->setToggleButton (TRUE);
	listViewOn=FALSE;

	btnResults = new QToolButton(QPixmap(start_xpm),"Show calculus results",
					   QString::null, this, SLOT(showResults()),
						fileTools,"");
	btnResults->setToggleButton (TRUE);
	resultsOn=FALSE;

	plotTools = new QToolBar( this, "file operations" );
    addToolBar( plotTools, tr( "Plot" ), Top,FALSE );

	new QToolButton(QPixmap(newLayer_xpm),"Add Layer",
					   QString::null, this, SLOT(addLayer()),
						plotTools,"");

	new QToolButton(QPixmap(arrangeLayers_xpm),"Arrange/Custom Layers",
					   QString::null, this, SLOT(showLayerDialog()),
						plotTools,"");

	plotTools->addSeparator();

	new QToolButton(QPixmap(curves_xpm),"Add curve to graph",
					   QString::null, this, SLOT(showCurvesDialog()),
						plotTools,"");

	new QToolButton(QPixmap(errors_xpm),"Add error bars",
					   QString::null, this, SLOT(addErrorBars()),
						plotTools,"");

	new QToolButton(QPixmap(fx_xpm),"Add function curve",
					   QString::null, this, SLOT(addFunctionCurve()),
						plotTools,"");

	new QToolButton(QPixmap(legend_xpm),"Add new legend",
					   QString::null, this, SLOT(newLegend()),
						plotTools,"");

	plotTools->addSeparator ();

	new QToolButton(QPixmap(unzoom_xpm),"Best fit",
					   QString::null, this, SLOT(unzoom()),
						plotTools,"");

	dataTools = new QActionGroup( this, "dataTools" );
    dataTools->setExclusive( TRUE );
    dataTools->setUsesDropDown( FALSE );

	btnPointer = new QAction( dataTools, "btnPointer" );
    btnPointer->setToggleAction( TRUE );
    btnPointer->setIconSet(QPixmap(pointer_xpm) );
	btnPointer->setToolTip( tr( "Pointer" ) );
	btnPointer->setOn(TRUE);
	btnPointer->addTo(plotTools);

	btnZoom = new QAction( dataTools, "btnZoom" );
    btnZoom->setToggleAction( TRUE );
    btnZoom->setIconSet(QPixmap(zoom_xpm) );
	btnZoom->setToolTip(tr("Zoom"));
	btnZoom->addTo(plotTools);

    btnCursor = new QAction( dataTools, "btnCursor" );
    btnCursor->setToggleAction( TRUE );
    btnCursor->setIconSet(QPixmap(select_xpm) );
	btnCursor->setToolTip(tr("Show data display"));
	btnCursor->addTo(plotTools);

	btnSelect = new QAction( dataTools, "btnSelect" );
    btnSelect->setToggleAction( TRUE );
    btnSelect->setIconSet(QPixmap(cursors_xpm) );
	btnSelect->setToolTip(tr("Select data range"));
	btnSelect->addTo(plotTools);

	btnPicker = new QAction( dataTools, "btnPicker" );
    btnPicker->setToggleAction( TRUE );
    btnPicker->setIconSet(QPixmap(cursor_16) );
	btnPicker->setToolTip(tr("Show data cursor"));
	btnPicker->addTo(plotTools);

	btnMovePoints = new QAction( dataTools, "btnMovePoints" );
    btnMovePoints->setToggleAction( TRUE );
    btnMovePoints->setIconSet(QPixmap(hand_xpm) );
	btnMovePoints->setToolTip(tr("Move data points"));
	btnMovePoints->addTo(plotTools);

	btnRemovePoints = new QAction( dataTools, "btnRemovePoints" );
    btnRemovePoints->setToggleAction( TRUE );
    btnRemovePoints->setIconSet(QPixmap(gomme_xpm) );
	btnRemovePoints->setToolTip(tr("Remove data points"));
	btnRemovePoints->addTo(plotTools);

	connect( dataTools, SIGNAL( selected( QAction* ) ), this, SLOT( pickDataTool( QAction* ) ) );

	plotTools->addSeparator ();

	btnText=new QToolButton(QPixmap(text_xpm), "Add text", QString::null,
			   this, SLOT(addText()), plotTools, "btnText" );
	btnText->setToggleButton (TRUE);

	btnLine = new QAction( dataTools, "btnLine" );
    btnLine->setToggleAction( TRUE );
    btnLine->setIconSet(QPixmap(arrow_xpm) );
	btnLine->setToolTip(tr("Draw line"));
	btnLine->addTo(plotTools);

	selectOn=FALSE;
	cursorOn=FALSE;
	pickerOn =FALSE;
	removePointsOn=FALSE;
	movePointsOn=FALSE;
	zoomOn=FALSE;
	textOn=FALSE;
	drawLineOn=FALSE;

	new QToolButton(QPixmap(monalisa_xpm),"Add Image",
					   QString::null, this, SLOT(addImage()),
						plotTools,"");

	plotTools->addSeparator ();
	new QToolButton(QPixmap(fileprint_xpm ), "Print graph", QString::null,
			   this, SLOT(print()), plotTools, "print file" );

	tableTools = new QToolBar( this, "table operations" );
    addToolBar( tableTools, tr( "Table" ), Top,FALSE );

new QToolButton(QPixmap(lPlot_xpm),"Plot with line",QString::null,this, SLOT(plotL()), tableTools,"");
new QToolButton(QPixmap(pPlot_xpm),"Plot with symbols",QString::null,this, SLOT(plotP()), tableTools,"");
new QToolButton(QPixmap(lpPlot_xpm),"Plot with line + symbols",QString::null,this, SLOT(plotLP()), tableTools,"");
new QToolButton(QPixmap(vertBars_xpm),"Plot with vertical bars",QString::null,this, SLOT(plotVerticalBars()), tableTools,"");
new QToolButton(QPixmap(hBars_xpm),"Plot with horizontal bars",QString::null,this, SLOT(plotHorizontalBars()), tableTools,"");

new QToolButton(QPixmap(area_xpm),"Plot area",QString::null,this, SLOT(plotArea()), tableTools,"");
new QToolButton(QPixmap(pie_xpm),"Plot pie",QString::null,this, SLOT(plotPie()), tableTools,"");
new QToolButton(QPixmap(histogram_xpm),"Histogram",QString::null,this, SLOT(plotHistogram()), tableTools,"");
new QToolButton(QPixmap(vectXYXY_xpm),"Vectors XYXY",QString::null,this, SLOT(plotVectXYXY()), tableTools,"");

	tableTools->addSeparator ();

new QToolButton(QPixmap(ribbon_xpm),"Plot 3D Ribbon",QString::null,this, SLOT(plot3DRibbon()), tableTools,"");
new QToolButton(QPixmap(bars_xpm),"Plot 3D Bars",QString::null,this, SLOT(plot3DBars()), tableTools,"");
new QToolButton(QPixmap(scatter_xpm),"Plot 3D Scatter",QString::null,this, SLOT(plot3DScatter()), tableTools,"");
new QToolButton(QPixmap(trajectory_xpm),"Plot 3D Trajectory",QString::null,this, SLOT(plot3DTrajectory()), tableTools,"");

	tableTools->addSeparator ();

	new QToolButton(QPixmap(addCol_xpm),"Add column",
					   QString::null, this, SLOT(addColToTable()),
						tableTools,"");
	new QToolButton(QPixmap(col_stat_xpm),"Selected columns statistics",
					   QString::null, this, SLOT(showColStatistics()),
						tableTools,"");
    new QToolButton(QPixmap(stat_rows_xpm),"Selected rows statistics",
					   QString::null, this, SLOT(showRowStatistics()),
						tableTools,"");
	tableTools->addSeparator ();

	new QToolButton(QPixmap(cut_xpm),"Cut selection",
					   QString::null, this, SLOT(cutSelection()),
						tableTools,"");

	new QToolButton(QPixmap(copy_xpm),"Copy selection",
					   QString::null, this, SLOT(copySelection()),
						tableTools,"");

	new QToolButton(QPixmap(paste_xpm),"Paste selection",
					   QString::null, this, SLOT(pasteSelection()),
						tableTools,"");

	plotTools->hide();
	tableTools->hide();
	
	displayBar = new QToolBar( this, "display operations" );
	info=new QLineEdit(displayBar,"test");
	info->setReadOnly(TRUE);

    QPalette pal;
    QColorGroup cg;
    cg.setColor( QColorGroup::Text, QColor( 79, 236, 0) );
    cg.setColor( QColorGroup::Background, QColor( 192, 192, 192) );
    pal.setActive( cg );
    cg.setColor( QColorGroup::Text, QColor( 79, 236, 0) );
    cg.setColor( QColorGroup::Background, QColor( 192, 192, 192) );
    pal.setInactive( cg );
    cg.setColor( QColorGroup::Text, QColor( 79, 236, 0) );
    cg.setColor( QColorGroup::Background, QColor( 192, 192, 192) );
    pal.setDisabled( cg );
    info->setPalette( pal );
    info->setFont(QFont("Times New Roman",14,QFont::Bold,FALSE) );

	displayBar->setStretchableWidget(info);

    addToolBar(displayBar, tr( "Data Display" ), Top, FALSE );
	displayBar->setCloseMode(QDockWindow::Undocked);
	setDockEnabled(displayBar, DockLeft, false);
	setDockEnabled(displayBar, DockRight, false);
	displayBar->hide();
}

void ApplicationWindow::initMainMenu()
{
    file = new QPopupMenu( this );
	file->setFont(appFont);
    menuBar()->insertItem( "&File", file );
	
	
	type = new QPopupMenu(this);
	type->setFont(appFont);
	type->insertItem(QPixmap(new_xpm),"New &Project", this, SLOT(newProject()), CTRL+Key_N );
	type->insertItem(QPixmap(table_xpm),"New &Table", this, SLOT(newWrksheet()), CTRL+Key_T);
	type->insertItem(QPixmap(newF_xpm),"New &Function Plot", this, SLOT(newFunctionPlot()), CTRL+Key_F);
	type->insertItem(QPixmap(newFxy_xpm),"New &Surface 3D Plot", this, SLOT(newSurfacePlot()), CTRL+Key_Z);

	file->insertItem( "&New",type);
    file->insertItem(QPixmap(fileopen_xpm ), "&Open",
			   this, SLOT(open()), CTRL+Key_O );
	file->insertItem("Open image &file",
			   this, SLOT(loadImage()), CTRL+Key_I );
	recent = new QPopupMenu(this);
	recent->setFont(appFont);
	file->insertItem("&Recent projects", recent);
	file->insertSeparator();

    file->insertItem(QPixmap(filesave_xpm), "&Save Project",
			   this, SLOT(saveProject()), CTRL+Key_S );
    file->insertItem( "Save Project &as...", this, SLOT(saveProjectAs()) );
    file->insertSeparator();
	exportID=file->insertItem("&Export Graph",
			   this, SLOT(exportGraph()), CTRL+Key_G);
	printID=file->insertItem(QPixmap(fileprint_xpm), "&Print",
			   this, SLOT(print()), CTRL+Key_P );

	file->insertSeparator();
	asciiID= file->insertItem("E&xport ASCII",
			   this, SLOT(showExportASCIIDialog()));

	import = new QPopupMenu(this);
	import->setFont(appFont);
	import->insertItem(QPixmap(import_xpm),"&Single file...", this, SLOT(load()) );
	import->insertItem(QPixmap(multiload_xpm),"&Multiple files...", this, SLOT(loadMultiple()));
	import->insertSeparator();
	import->insertItem("Set import &options", this, SLOT(showImportDialog()));
	file->insertItem( "&Import ASCII",import);
	

    file->insertSeparator();
	file->insertItem(QPixmap(quit_xpm), "&Quit",
				qApp, SLOT( closeAllWindows() ), CTRL+Key_Q );

	edit = new QPopupMenu(this);
	edit->setFont(appFont);
	undoID = edit->insertItem(QPixmap(undo_xpm),"&Undo", this, SLOT(undo()), CTRL+Key_U );
	redoID = edit->insertItem(QPixmap(redo_xpm),"&Redo", this, SLOT(redo()), CTRL+Key_R );
	edit->insertSeparator();
	edit->insertItem(QPixmap(cut_xpm),"Cu&t selection", this, SLOT(cutSelection()), CTRL+Key_X );
	edit->insertItem(QPixmap(copy_xpm), "&Copy selection", this, SLOT(copySelection()), CTRL+Key_C);
	edit->insertItem(QPixmap(paste_xpm), "&Paste selection", this, SLOT(pasteSelection()), CTRL+Key_V);
	edit->insertItem(QPixmap(erase_xpm),"&Delete selection", this, SLOT(clearSelection()),Key_Delete);
	edit->insertSeparator();
	edit->insertItem("Clear &log informations", this, SLOT(clearLogInfo()));

	menuBar()->insertItem( "&Edit", edit);

	view = new QPopupMenu(this);
	view->setFont(appFont);
	view->setCheckable(TRUE);
	view->insertItem(QPixmap(wizard_xpm),"Plot &wizard", this, SLOT(showPlotWizard()),CTRL+ALT+Key_W);

	exploreID=view->insertItem(QPixmap(folder_xpm),"Project &explorer");
	view->connectItem(exploreID, this, SLOT(showList()));
	view->setAccel(CTRL+Key_E,exploreID);
	view->setItemChecked(exploreID,FALSE);

	logID=view->insertItem(QPixmap(start_xpm),"Results &Log");
	view->connectItem(logID, this, SLOT(showResults()));
	view->setItemChecked(logID,FALSE);
	view->insertItem("&Preferences...", this, SLOT(showConfigureDialog()));

	menuBar()->insertItem("&View", view);

	graph = new QPopupMenu(this);
	graph->setFont(appFont);
	graph->setCheckable(TRUE);
	graph->insertItem(QPixmap(curves_xpm),"&Add/Remove curve", this, SLOT(showCurvesDialog()));
	graph->insertItem(QPixmap(errors_xpm),"Add &Error Bars", this, SLOT(addErrorBars()),CTRL+Key_B);
	graph->insertItem(QPixmap(fx_xpm),"Add &function curve",this, SLOT(addFunctionCurve()),CTRL+ALT+Key_F);
	graph->insertItem(QPixmap(unzoom_xpm),"&Rescale to show all", this, SLOT(unzoom()),CTRL+Key_R);
	graph->insertItem(QPixmap(legend_xpm),"New &Legend", this, SLOT(newLegend()),CTRL+Key_L);
	graph->insertSeparator();
	graph->insertItem(QPixmap(monalisa_xpm),"Add &Image", this, SLOT(addImage()));
	graph->insertItem(QPixmap(text_xpm),"Add &Text", this, SLOT(addText()));
	graph->insertItem(QPixmap(arrow_xpm),"&Draw arrow/line", this, SLOT(drawLine()));

	graph->insertSeparator();//layers section
	graph->insertItem(QPixmap(newLayer_xpm),"Add La&yer", this, SLOT(addLayer()));
	graph->insertItem(QPixmap(erase_xpm),"Delete Layer", this, SLOT(deleteLayer()));
	graph->insertItem(QPixmap(arrangeLayers_xpm),"Arran&ge/Custom Layers", this, SLOT(showLayerDialog()));
	menuBar()->insertItem( "&Graph", graph);

	dataMenu = new QPopupMenu(this);
	dataMenu->setFont(appFont);
	dataMenu->setCheckable(TRUE);

	plot2D = new QPopupMenu(this);
	plot2D->setFont(appFont);
	specialPlot = new QPopupMenu(this);
	specialPlot->setFont(appFont);
	panels = new QPopupMenu(this);
	panels->setFont(appFont);
	stat = new QPopupMenu(this);
	stat->setFont(appFont);
	
	plot2D->insertItem(QPixmap(lPlot_xpm),"&Line",this, SLOT(plotL()));
	plot2D->insertItem(QPixmap(pPlot_xpm),"&Scatter",this, SLOT(plotP()));
	plot2D->insertItem(QPixmap(lpPlot_xpm),"Line + S&ymbol",this, SLOT(plotLP()));

	specialPlot->insertItem(QPixmap(dropLines_xpm),"Vertical &drop lines", this,SLOT(plotVerticalDropLines()));
	specialPlot->insertItem(QPixmap(spline_xpm),"&Spline", this,SLOT(plotSpline()));
	specialPlot->insertItem(QPixmap(steps_xpm),"&Vertical Steps", this,SLOT(plotSteps()));
	plot2D->insertItem("Special Line/Symb&ol", specialPlot);
	plot2D->insertSeparator();
	plot2D->insertItem(QPixmap(vertBars_xpm),"&Columns",this, SLOT(plotVerticalBars()));
	plot2D->insertItem(QPixmap(hBars_xpm),"&Rows",this, SLOT(plotHorizontalBars()));
	plot2D->insertItem(QPixmap(area_xpm),"&Area",this, SLOT(plotArea()));
	plot2D->insertItem(QPixmap(pie_xpm),"&Pie",this, SLOT(plotPie()));
	plot2D->insertItem(QPixmap(vectXYXY_xpm),"Vectors &XYXY", this, SLOT(plotVectXYXY()));
	plot2D->insertSeparator();

	stat->insertItem(QPixmap(histogram_xpm),"&Histogram",this,SLOT(plotHistogram()));
	stat->insertItem(QPixmap(stacked_hist_xpm),"&Stacked Histograms",this,SLOT(plotStackedHistograms()));
	plot2D->insertItem("Statistical &Graphs",stat);
	
	panels->insertItem(QPixmap(panel_v2_xpm),"&Vertical 2 Layers",this, SLOT(plot2VerticalLayers()));
	panels->insertItem(QPixmap(panel_h2_xpm),"&Horizontal 2 Layers",this, SLOT(plot2HorizontalLayers()));
	panels->insertItem(QPixmap(panel_4_xpm),"&4 Layers",this, SLOT(plot4Layers()));
	panels->insertItem(QPixmap(stacked_xpm),"&Stacked Layers",this, SLOT(plotStackedLayers()));
	plot2D->insertItem("Pa&nel",panels);

	dataMenu->insertItem("&Plot 2D", plot2D);

	plot3D = new QPopupMenu(this);
	plot3D->setFont(appFont);
	plot3D->insertItem(QPixmap(ribbon_xpm),"&Ribbon",this, SLOT(plot3DRibbon()));
	plot3D->insertItem(QPixmap(bars_xpm),"&Bars",this, SLOT(plot3DBars()));
	plot3D->insertItem(QPixmap(scatter_xpm),"&Scatter",this, SLOT(plot3DScatter()));
	plot3D->insertItem(QPixmap(trajectory_xpm),"&Trajectory",this, SLOT(plot3DTrajectory()));
	dataMenu->insertItem("Plot &3D", plot3D);
	dataMenu->insertSeparator();

	moveID=dataMenu->insertItem(QPixmap(hand_xpm),"&Move data points", this, SLOT(movePoints()),CTRL+Key_M);
	movePointsOn=FALSE;
	removeID=dataMenu->insertItem(QPixmap(gomme_xpm),"Remove &bad data points", this, SLOT(removePoints()),ALT+Key_R);
	menuBar()->insertItem( "&Data", dataMenu);

	calcul = new QPopupMenu( this );
	calcul->setFont(appFont);
	calcul->insertItem(QPixmap(col_stat_xpm),"Statistics on &Columns",this, SLOT(showColStatistics()));
	calcul->insertItem(QPixmap(stat_rows_xpm),"Statistics on &Rows",this, SLOT(showRowStatistics()));
	calcul->insertSeparator();
	calcul->insertItem( "&Integrate ...", this, SLOT(showIntDialog()));

	inter = new QPopupMenu(this);
	inter->setFont(appFont);
	inter->insertItem( "Li&near", this, SLOT(interlinear()));
	inter->insertItem( "Cu&bic spline", this, SLOT(intercubic()));
	inter->insertItem( "Non-&rounded Akime spline", this, SLOT(akima()));
	calcul->insertItem( "Inter&polate",inter);

	calcul->insertSeparator();
	fft = new QPopupMenu(this);
	fft->setFont(appFont);
	fft->insertItem("&Forward",this,SLOT(fft_for()));
	fft->insertItem("&Inverse",this,SLOT(fft_inv()));
	calcul->insertItem("&FFT",fft);
	calcul->insertSeparator();

	calcul->insertItem( "&Differentiate", this, SLOT(differentiate()));
	calcul->insertSeparator();
    calcul->insertItem( "Fit &Linear", this, SLOT(fitLinear()));
	calcul->insertItem( "Fit &Polynomial ...", this, SLOT(showFitPolynomDialog()));
	calcul->insertSeparator();

	decay = new QPopupMenu(this);
	decay->setFont(appFont);
	//decay->insertItem( "&First Order ", this, SLOT(fitExpDecay()));
	decay->insertItem( "&First Order ...", this, SLOT(showExpDecayDialog()));
	decay->insertItem( "&Second Order ...", this, SLOT(showTwoExpDecayDialog()));
	decay->insertItem( "&Third Order ...", this, SLOT(showExpDecay3Dialog()));

	calcul->insertItem( "Fit E&xponential Decay", decay);
	calcul->insertItem( "Fit Exponential Gro&wth", this, SLOT(fitExpGrowth()));
	calcul->insertItem( "Fit &Gaussian",this, SLOT(fitGauss()));
	calcul->insertItem( "Fit Lorent&zian",this, SLOT(fitLorentz()));
	calcul->insertSeparator();
	calcul->insertItem( "&Non-linear Curve Fit ...",this, SLOT(showFitDialog()),CTRL+Key_Y);
	menuBar()->insertItem( "&Analysis", calcul);

	format = new QPopupMenu(this);
	format->setFont(appFont);
	plotID=format->insertItem("&Plot ...", this, SLOT(showLayoutDialog()));
	axesID=format->insertItem("&Axes ...", this, SLOT(showXAxesDialog()));
	titleID=format->insertItem("&Title ...", this, SLOT(showTitleDialog()));
	format->insertSeparator();
	format->insertItem("&Column options ...", this, SLOT(showTableOptionsDialog()));
	format->insertItem("&Set column values ...", this, SLOT(showColumnValuesDialog()));
	format->insertItem("&Rows and columns ...", this, SLOT(showRowsDialog()));
	menuBar()->insertItem( "For&mat", format);

	windowsMenu = new QPopupMenu( this );
	windowsMenu->setFont(appFont);
    windowsMenu->setCheckable( TRUE );
    connect( windowsMenu, SIGNAL( aboutToShow() ),
	     this, SLOT( windowsMenuAboutToShow() ) );
    menuBar()->insertItem( "&Windows", windowsMenu );

    menuBar()->insertSeparator();
    help = new QPopupMenu( this );
	help->setFont(appFont);
    menuBar()->insertItem( "&Help", help );

    help->insertItem(QPixmap(about_xpm), "&About", this, SLOT(about()), Key_F1);
	help->insertItem("&Help", this, SLOT(showHelp()), CTRL+Key_H);

	disableMenus();
}

void ApplicationWindow::customMenu(QWidget* w)
{
int i;
if(w)
{
if (plotWindows.contains(w->caption())>0)
	{
	file->setItemEnabled (printID,TRUE);
	file->setItemEnabled (exportID,TRUE);
	file->setItemEnabled (asciiID,FALSE);
	file->setItemEnabled (closeID,TRUE);

	for (i=0;i<3;i++)
		format->setItemEnabled (format->idAt(i),TRUE);
	for (i=3;i<(int)format->count();i++)
		format->setItemEnabled (format->idAt(i),FALSE);

	for (i=0;i<2;i++)
		calcul->setItemEnabled (calcul->idAt(i),FALSE);
	for (i=2;i<(int)calcul->count();i++)
		calcul->setItemEnabled (calcul->idAt(i),TRUE);
	for (i=0;i<(int)graph->count();i++)
		graph->setItemEnabled (graph->idAt(i),TRUE);
	for (i=0;i<2;i++)
		dataMenu->setItemEnabled (dataMenu->idAt(i),FALSE);
	for (i=2;i<(int)dataMenu->count();i++)
		dataMenu->setItemEnabled (dataMenu->idAt(i),TRUE);
	for (i=2;i<(int)edit->count();i++)
		edit->setItemEnabled (edit->idAt(i),TRUE);
	}
else if (plot3DWindows.contains(w->caption())>0)
	{
	disableMenus();

	file->setItemEnabled (printID,TRUE);
	file->setItemEnabled (exportID,TRUE);
	file->setItemEnabled (closeID,TRUE);

	format->setItemEnabled (plotID,TRUE);


	if (((Graph3D*)w)->coordStyle() == Qwt3D::NOCOORD)
		format->setItemEnabled (axesID,FALSE);
	else
		format->setItemEnabled (axesID,TRUE);

	format->setItemEnabled (titleID,TRUE);

	for (i=3;i<(int)format->count();i++)
		format->setItemEnabled (format->idAt(i),FALSE);
	}
else if (tableWindows.contains(w->caption())>0)
	{
	file->setItemEnabled (printID,TRUE);
	file->setItemEnabled (asciiID,TRUE);
	file->setItemEnabled (exportID,FALSE);
	file->setItemEnabled (closeID,TRUE);

	for (i=0;i<3;i++)
		format->setItemEnabled (format->idAt(i),FALSE);
	for (i=3;i<(int)format->count();i++)
		format->setItemEnabled (format->idAt(i),TRUE);

	for (i=0;i<2;i++)
		calcul->setItemEnabled (calcul->idAt(i),TRUE);
	for (i=2;i<(int)calcul->count();i++)
		calcul->setItemEnabled (calcul->idAt(i),FALSE);

	for (i=0;i<(int)graph->count();i++)
		graph->setItemEnabled (graph->idAt(i),FALSE);
	for (i=0;i<2;i++)
		dataMenu->setItemEnabled (dataMenu->idAt(i),TRUE);
	for (i=2;i<(int)dataMenu->count();i++)
		dataMenu->setItemEnabled (dataMenu->idAt(i),FALSE);
	for (i=2;i<(int)edit->count();i++)
		edit->setItemEnabled (edit->idAt(i),TRUE);
	}
	else
		disableMenus();
}
else
	disableMenus();
}

void ApplicationWindow::disableMenus()
{
	int i;

	file->setItemEnabled (printID,FALSE);
	file->setItemEnabled (asciiID,FALSE);
	file->setItemEnabled (exportID,FALSE);
	file->setItemEnabled (closeID,FALSE);

	edit->setItemEnabled (undoID,FALSE);
	edit->setItemEnabled (redoID,FALSE);

	for (i=0;i<(int)format->count();i++)
		format->setItemEnabled (format->idAt(i),FALSE);

	for (i=0;i<(int)calcul->count();i++)
		calcul->setItemEnabled (calcul->idAt(i),FALSE);
	for (i=0;i<(int)graph->count();i++)
		graph->setItemEnabled (graph->idAt(i),FALSE);
	for (i=0;i<(int)edit->count()-1;i++)
		edit->setItemEnabled (edit->idAt(i),FALSE);
	for (i=0;i<(int)dataMenu->count();i++)
		dataMenu->setItemEnabled (dataMenu->idAt(i),FALSE);
}

void ApplicationWindow::customToolBars(QWidget* w)
{
if (w)
{
	if ((int)plot3DWindows.count()<=0)
		plot3DTools->hide();
	if ((int)plotWindows.count()<=0)
		plotTools->hide();
	if ((int)tableWindows.count()<=0)
		tableTools->hide();

    if (plotWindows.contains(w->caption())>0)
	{
	if (plotTools->isHidden())
		plotTools->show();
	
	plotTools->setEnabled (TRUE);
	plot3DTools->setEnabled (FALSE);
	tableTools->setEnabled(FALSE);
	}
	else if (tableWindows.contains(w->caption())>0)
	{
	if (tableTools->isHidden())
		tableTools->show();
	
	plotTools->setEnabled (FALSE);
	plot3DTools->setEnabled (FALSE);
	tableTools->setEnabled (TRUE);
	}
	else if (plot3DWindows.contains(w->caption())>0)
	{
	plotTools->setEnabled (FALSE);
	plot3DTools->setEnabled (TRUE);
	tableTools->setEnabled (FALSE);

	custom3DActions(w);
	}
}
else
	{
	plot3DTools->hide();
	plotTools->hide();
	tableTools->hide();
		
	plotTools->setEnabled (FALSE);
	tableTools->setEnabled (FALSE);
	plot3DTools->setEnabled (FALSE);
	}
}

void ApplicationWindow::showList()
{
listViewOn = !listViewOn;
btnList->setOn(listViewOn);
view->setItemChecked(exploreID,listViewOn);

if (listViewOn)
lv->show();
else
lv->hide();
emit modified();
}

void ApplicationWindow::maximizeWindow(QListViewItem * lbi)
{
int i;
QString caption=lbi->text(0);

if (lbi->text(2)=="Hidden")
{
for (i=0;i<(int)hiddenWindows->count();i++)
	{
	QWidget *w=hiddenWindows->at(i);
	if (w && w->caption()==caption)
		{
		w->showMaximized();
		w->setFocus();
		hiddenWindows->take(hiddenWindows->find (w));
		lbi->setText(2,"Maximized");
		break;
		}
	}
}
else
{
QWidgetList windows =ws->windowList();
for (i=0;i<(int)windows.count();i++)
	{
	if (windows.at(i)->caption()==caption)
		{
		windows.at(i)->showMaximized();
		windows.at(i)->setFocus();
		break;
		}
	}
}

emit modified();
}

void ApplicationWindow::plot3DRibbon()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	{
	if(int(w->selectedColumns().count())==1)
		w->plot3DRibbon();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","You must select exactly one column for plotting!");
	}
}

void ApplicationWindow::plot3DBars()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	{
	if(int(w->selectedColumns().count())==1)
		w->plot3DBars();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","You must select exactly one column for plotting!");
	}
}

void ApplicationWindow::plot3DScatter()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	{
	if(int(w->selectedColumns().count())==1)
		w->plot3DScatter();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","You must select exactly one column for plotting!");
	}
}

void ApplicationWindow::plot3DTrajectory()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	{
	if(int(w->selectedColumns().count())==1)
		w->plot3DTrajectory();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","You must select exactly one column for plotting!");
	}
}

MultiLayer* ApplicationWindow::plotMultiLayers(int c, int r, int style)
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (!w || !tableWindows.contains(w->caption()))
	return 0;

QStringList list=w->selectedColumns();
if((int)list.count() <1)
	{
	QMessageBox::warning(0,"QtiPlot - Plot error","Please select a column to plot!");
	return 0;
	}

// excluding the x column of the table from the plot list
QString xColName=w->colName(w->colX());
QStringList::Iterator it=list.find(xColName);
if (it != list.end ())
	list.remove(it);

int curves= (int)list.count();

if (r<0)
	r=curves;

MultiLayer* g = new MultiLayer(ws,0,WDestructiveClose);
connectMultilayerPlot(g);
g->askOnCloseEvent(confirmClosePlot2D);

graphs++;
QString label="graph"+QString::number(graphs);
while(plotWindows.contains(label)>0 || plot3DWindows.contains(label)>0
	  || tableWindows.contains(label)>0)
	{
	graphs++;
	label="graph"+QString::number(graphs);
	}

QString date=g->birthDate();

int i;
int layers=c*r;

if (curves<layers)
	{
	for (i=0; i<curves;i++)
		{
		activeGraph=g->addLayer(w, list[i], style);

		if (activeGraph)
			{
			customGraph(activeGraph);
			if (style == Graph::VerticalBars || style == Graph::Histogram)
				{
				QValueList<int> ticksList;
				int ticksStyle= Plot::Out;
				ticksList<<ticksStyle<<ticksStyle<<ticksStyle<<ticksStyle;
				activeGraph->setTicksType(ticksList);
				}
			}
		}

	for (i=curves; i<layers;i++)
		{
		activeGraph=g->addLayerToOrigin();
		if (activeGraph)
			customGraph(activeGraph);
		}
	}
else
	{
	for (i=0; i<layers;i++)
		{
		activeGraph=g->addLayer(w,list[i], style);
		if (activeGraph)
			{
			customGraph(activeGraph);
			if (style == Graph::VerticalBars || style == Graph::Histogram)
				{
				QValueList<int> ticksList;
				int ticksStyle= Plot::Out;
				ticksList<<ticksStyle<<ticksStyle<<ticksStyle<<ticksStyle;
				activeGraph->setTicksType(ticksList);
				}
			}
		}
	}

g->setGeometry(0, 0, 500, 420);
g->showNormal();
g->arrangeLayers(c, r, 5, 5);

QListViewItem *element=new QListViewItem (lv);
element->setPixmap (0,QPixmap(graph_xpm));
element->setText (0,label);
element->setText (1,"Plot");
element->setText (2,"Normal");
element->setText (3,g->sizeToString());
element->setText (4,date);

plotWindows<<label;

g->setCaption(label);
g->setIcon( QPixmap(graph_xpm));
g->setFocus();

customToolBars(g);
customMenu(g);
emit modified();
return g;
}

void ApplicationWindow::plotVerticalBars()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	{
	if(int(w->selectedColumns().count())>0)
		w->plotVB();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","Please select a column to plot!");
	}
}

void ApplicationWindow::plotHorizontalBars()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	{
	if(	int(w->selectedColumns().count())>0)
		w->plotHB();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","Please select a column to plot!");
	}
}

void ApplicationWindow::plotHistogram()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	{
	if(	int(w->selectedColumns().count())>0)
		w->plotHistogram();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","Please select a column to plot!");
	}
}

void ApplicationWindow::plotArea()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	{
	if(int(w->selectedColumns().count())>0)
		w->plotArea();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","Please select a column to plot!");
	}
}

void ApplicationWindow::plotPie()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	{
	if(int(w->selectedColumns().count())==1)
		w->plotPie();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","You must select exactly one column for plotting!");
	}
}

void ApplicationWindow::plotL()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	{
	if(	int(w->selectedColumns().count())>0)
		w->plotL();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","Please select a column to plot!");
	}
}

void ApplicationWindow::plotP()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	if(	int(w->selectedColumns().count())>0)
		w->plotP();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","Please select a column to plot!");
}

void ApplicationWindow::plotLP()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	if(	int(w->selectedColumns().count())>0)
		w->plotLP();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","Please select a column to plot!");
}

void ApplicationWindow::plotVerticalDropLines()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	if(int(w->selectedColumns().count())>0)
		w->plotVerticalDropLines();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","Please select a column to plot!");
}

void ApplicationWindow::plotSpline()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	if(	int(w->selectedColumns().count())>0)
		w->plotSpline();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","Please select a column to plot!");
}

void ApplicationWindow::plotSteps()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	if(	int(w->selectedColumns().count())>0)
		w->plotSteps();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","Please select a column to plot!");
}

void ApplicationWindow::plotVectXYXY()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if (w &&  tableWindows.contains(w->caption())>0)
	{
	int cols = int(w->selectedColumns().count());
	if( cols == 2 || cols == 4)
		w->plotVectXYXY();
	else
		QMessageBox::warning(0,"QtiPlot - Plot error","Please select two or four columns for this type of plot!");
	}
}

void ApplicationWindow::updateTable(const QString& caption,int row,const QString& text)
{
int aux=caption.find("_",0);
QString name=caption.left(aux);
WksheetWindow* w=table(name);

QStringList cvs=QStringList::split(",",caption,FALSE);

int pos=cvs[0].find("(",0);
QString colName=cvs[0].left(pos);
int xCol=w->colIndex(colName);

pos=cvs[1].find("(",0);
colName=cvs[1].left(pos);
int yCol=w->colIndex(colName);

QStringList values=QStringList::split ("\t",text,FALSE);
w->setText(row,xCol,values[0]);
w->setText(row,yCol,values[1]);

updateCurveInGraphs(colName);
emit modified();
}

void ApplicationWindow::clearCellFromTable(const QString& name,double value)
{
int aux=name.find("_",0);
QString caption=name.left(aux);
WksheetWindow* w=table(caption);
if (w)
	{
	int col=w->colIndex(name);
	int row=w->atRow(col, value);
	w->clearCell(row,col);
	}
}

void ApplicationWindow::updateListView(const QString& caption)
{
QListViewItem *it=lv->findItem (caption,0, Qt::ExactMatch | Qt::CaseSensitive );
if (it)
	lv->takeItem(it);
}

void ApplicationWindow::renameListViewItem(const QString& oldName,const QString& newName)
{
QListViewItem *it=lv->findItem (oldName,0, Qt::ExactMatch | Qt::CaseSensitive );
if (it)
	it->setText(0,newName);
}

void ApplicationWindow::setListViewDate(const QString& caption,const QString& date)
{
QListViewItem *it=lv->findItem ( caption,0, Qt::ExactMatch | Qt::CaseSensitive );
if (it)
	it->setText(4,date);
}

void ApplicationWindow::setListView(const QString& caption,const QString& view)
{
QListViewItem *it=lv->findItem ( caption,0, Qt::ExactMatch | Qt::CaseSensitive );
if (it)
	it->setText(2,view);
}

void ApplicationWindow::setListViewSize(const QString& caption,const QString& size)
{
QListViewItem *it=lv->findItem ( caption,0, Qt::ExactMatch | Qt::CaseSensitive );
if (it)
	it->setText(3,size);
}

QString ApplicationWindow::listViewDate(const QString& caption)
{
QListViewItem *it=lv->findItem (caption,0,ExactMatch | CaseSensitive );
return it->text(4);
}

void ApplicationWindow::updateCurvesDialogs(const QString& old, const QString& newName)
{
QWidgetList windows = ws->windowList();
int c=int(windows.count());
QStringList onPlot;

for (int i=0;i<c;i++)
	{
	if (plotWindows.contains(windows.at(i)->caption())>0)
		{
		MultiLayer *plot=(MultiLayer*)windows.at(i);
		if (!plot)
			return;

		QWidgetList *graphsList=plot->graphPtrs();
		for (int j=0;j<(int)graphsList->count();j++)
			{
			Graph *g=(Graph*)graphsList->at(j);

			//update plotted curves list
			onPlot=g->plotCurvesList();
			onPlot.gres (old,newName,TRUE);
			g->insertPlottedList(onPlot);

			//update plot associations
			onPlot=g->plotAssociations();
			onPlot.gres (old,newName,TRUE);
			g->setPlotAssociations(onPlot);

			//update error bars associations
			onPlot=g->errorBarsAssociations();
			onPlot.gres (old,newName,TRUE);
			g->setErrorBarsAssociations(onPlot);

			//update legend
			QString legend=g->getLegendText();
			onPlot=QStringList::split ("\n",legend,FALSE );
			onPlot.gres (old,newName,TRUE);
			legend=onPlot.join("\n");
			g->setLegendText(legend);
			}
		}
	else if (plot3DWindows.contains(windows.at(i)->caption())>0)
		{
		Graph3D* g = (Graph3D*)windows.at(i);
		QString name=g->formula();
		if (name.contains(old,TRUE)>0)
			{
			name.replace(old,newName);
			g->setName(name);
			}
		}
	}
}

void ApplicationWindow::remove3DPlot()
{
Graph3D* g = (Graph3D*)ws->activeWindow();

if ( g && plot3DWindows.contains(g->caption())>0)
	{
	g->removePlot();
	plot3DTools->setDisabled(true);
	}
}

void ApplicationWindow::update3DToolBar()
{
plot3DTools->setEnabled(true);
filledmesh->setOn(true);
}

void ApplicationWindow::add3DData()
{
if (tableWindows.count() <= 0)
	{
	QMessageBox::warning(this,"QtiPlot - Warning",
				tr("<h4>There are no tables available in this project.</h4>"
					  "<p><h4>Please create a table and try again!</h4>"));
	return;
	}

analysisDialog *ad=new analysisDialog(0,"Column :", "analysisDialog",TRUE, WDestructiveClose);
connect (ad,SIGNAL(options(const QString&)), this, SLOT(insertNew3DData(const QString&)));

ad->setCaption(tr("QtiPlot - Choose data set"));
ad->setCurveNames(columnsList());
ad->showNormal();
ad->setActiveWindow();
}

void ApplicationWindow::insertNew3DData(const QString& colName)
{
Graph3D* g = (Graph3D*)ws->activeWindow();

if ( g && plot3DWindows.contains(g->caption())>0)
	{
	WksheetWindow* w=table(colName);
	g->insertNewData(w,colName);
	update3DToolBar();
	}
}

void ApplicationWindow::addSurfacePlot()
{
Graph3D* g = (Graph3D*)ws->activeWindow();

if ( g && plot3DWindows.contains(g->caption())>0)
	{
	sDialog* sd= new sDialog(this,"fDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (sd,SIGNAL(options(const QString&,double,double,double,double,double,double)),
		g,SLOT(insertNewFunction(const QString&,double,double,double,double,double,double)));
	connect (sd,SIGNAL(clearFunctionsList()),this,SLOT(clearSurfaceFunctionsList()));
	connect (sd,SIGNAL(custom3DToolBar()),this,SLOT(update3DToolBar()));

	sd->insertFunctionsList(surfaceFunc);
	sd->show();
	sd->setActiveWindow();
	}
}

void ApplicationWindow::newSurfacePlot()
{
sDialog* sd= new sDialog(this,"fDialog",TRUE,WStyle_Tool|WDestructiveClose);
connect (sd,SIGNAL(options(const QString&,double,double,double,double,double,double)),
		this,SLOT(newPlot3D(const QString&,double,double,double,double,double,double)));
connect (sd,SIGNAL(clearFunctionsList()),this,SLOT(clearSurfaceFunctionsList()));

sd->insertFunctionsList(surfaceFunc);
sd->show();
sd->setActiveWindow();
}

Graph3D* ApplicationWindow::newPlot3D(const QString& formula, double xl, double xr,
									  double yl, double yr, double zl, double zr)
{
	graphs++;
	QString label="graph"+QString::number(graphs);
	while(plotWindows.contains(label)>0 || plot3DWindows.contains(label)>0
		 || tableWindows.contains(label)>0)
		{
		graphs++;
		label="graph"+QString::number(graphs);
		}

	Graph3D *plot=new Graph3D(formula,xl,xr,yl,yr,zl,zr,ws,0,WDestructiveClose);
	connect (plot,SIGNAL(showContextMenu()),this,SLOT(showWindowContextMenu()));
	connect (plot,SIGNAL(showOptionsDialog()),this,SLOT(showPlot3dDialog()));
	connect (plot,SIGNAL(closedGraph(QWidget*)),this, SLOT(closeWindow(QWidget*)));
	connect (plot,SIGNAL(hiddenPlot(QWidget*)),this, SLOT(hideWindow(QWidget*)));
	connect (plot,SIGNAL(resizedPlot(QWidget*)),this, SLOT(resizedWindow(QWidget*)));
	connect (plot,SIGNAL(modified()),this, SIGNAL(modified()));

	plot->askOnCloseEvent(confirmClosePlot3D);
	plot->resize(500,400);
	plot->setCaption(label);
	plot->setIcon( QPixmap(grid_poly_xpm));
	plot->show();
	plot->setFocus();

	QListViewItem *element=new QListViewItem (lv);
	element->setPixmap (0,QPixmap(grid_poly_xpm));
	element->setText (0,label);
	element->setText (1,"Function Plot 3D");
	element->setText (2,"Normal");
	element->setText (3,plot->sizeToString());
	element->setText (4,plot->birthDate());

	plot3DWindows<<label;

	if (surfaceFunc.contains(formula)<=0)
		surfaceFunc<<formula;

	if (!plot3DTools->isVisible())
		plot3DTools->show();

	if (!plot3DTools->isEnabled())
			plot3DTools->setEnabled(TRUE);

	customMenu((QWidget*)plot);
	customToolBars((QWidget*)plot);

emit modified();
return plot;
}

Graph3D* ApplicationWindow::newPlot3D(const QString& caption,const QString& formula,
									  double xl, double xr,double yl, double yr,
									  double zl, double zr)
{
	Graph3D *plot=new Graph3D(formula,xl,xr,yl,yr,zl,zr,ws,0,WDestructiveClose);
	connect (plot,SIGNAL(showContextMenu()),this,SLOT(showWindowContextMenu()));
	connect (plot,SIGNAL(showOptionsDialog()),this,SLOT(showPlot3dDialog()));
	connect (plot,SIGNAL(closedGraph(QWidget*)),this, SLOT(closeWindow(QWidget*)));
	connect (plot,SIGNAL(hiddenPlot(QWidget*)),this, SLOT(hideWindow(QWidget*)));
	connect (plot,SIGNAL(resizedPlot(QWidget*)),this, SLOT(resizedWindow(QWidget*)));
	connect (plot,SIGNAL(modified()),this, SIGNAL(modified()));

	QString label=caption;
	while(plotWindows.contains(label)>0 || plot3DWindows.contains(label)>0
			|| tableWindows.contains(label)>0)
		{
		graphs++;
		label="graph"+QString::number(graphs);
		}

	plot->askOnCloseEvent(confirmClosePlot3D);
	plot->setCaption(label);
	plot->setIcon( QPixmap(grid_poly_xpm));
	plot->show();
	plot->setFocus();

	QListViewItem *element=new QListViewItem (lv);
	element->setPixmap (0,QPixmap(grid_poly_xpm));
	element->setText (0,label);
	element->setText (1,"Function Plot 3D");
	element->setText (2,"Normal");
	element->setText (4,plot->birthDate());

	plot3DWindows<<label;

	if (!plot3DTools->isVisible())
		plot3DTools->show();

	if (!plot3DTools->isEnabled())
			plot3DTools->setEnabled(TRUE);

	customMenu((QWidget*)plot);
	customToolBars((QWidget*)plot);

return plot;
}

Graph3D* ApplicationWindow::dataPlot3D(WksheetWindow* table,const QString& colName)
{
	QApplication::setOverrideCursor(waitCursor);

	graphs++;
	QString label="graph"+QString::number(graphs);
	while(plotWindows.contains(label)>0 || plot3DWindows.contains(label)>0
		 || tableWindows.contains(label)>0)
		{
		graphs++;
		label="graph"+QString::number(graphs);
		}

	Graph3D *plot=new Graph3D(table,colName,ws,0,WDestructiveClose);
	connectSurfacePlot(plot);

	plot->resize(500,400);
	plot->setCaption(label);
	plot->setIcon( QPixmap(trajectory_xpm));
	plot->show();
	plot->setFocus();

	QListViewItem *element=new QListViewItem (lv);
	element->setPixmap (0,QPixmap(trajectory_xpm));
	element->setText (0,label);
	element->setText (1,"Data Plot 3D");
	element->setText (2,"Normal");
	element->setText (3,plot->sizeToString());
	element->setText (4,plot->birthDate());

	plot3DWindows<<label;

	if (!plot3DTools->isVisible())
		plot3DTools->show();

	if (!plot3DTools->isEnabled())
			plot3DTools->setEnabled(TRUE);

	customMenu((QWidget*)plot);
	customToolBars((QWidget*)plot);

emit modified();
QApplication::restoreOverrideCursor();
return plot;
}

Graph3D* ApplicationWindow::dataPlot3D(const QString& caption,const QString& formula,
																			double xl, double xr, double yl, double yr, double zl, double zr)
{
	int pos=formula.find("_",0);
	QString wCaption=formula.left(pos);

	WksheetWindow* w=table(wCaption);
	if (!w)
		return 0;

	int posX=formula.find("(",pos);
	QString xCol=formula.mid(pos+1,posX-pos-1);

	pos=formula.find(",",posX);
	posX=formula.find("(",pos);
	QString yCol=formula.mid(pos+1,posX-pos-1);

	Graph3D *plot=new Graph3D(w,xCol,yCol,xl,xr,yl,yr,zl,zr,ws,0,WDestructiveClose);
	connectSurfacePlot(plot);

	QString label=caption;
	while(plotWindows.contains(label)>0 || plot3DWindows.contains(label)>0
			|| tableWindows.contains(label)>0)
		{
		graphs++;
		label="graph"+QString::number(graphs);
		}

	plot->setCaption(label);
	plot->setIcon( QPixmap(trajectory_xpm));
	plot->show();
	plot->setFocus();

	QListViewItem *element=new QListViewItem (lv);
	element->setPixmap (0,QPixmap(trajectory_xpm));
	element->setText (0,label);
	element->setText (1,"Data Plot 3D");
	element->setText (2,"Normal");
	element->setText (4,plot->birthDate());

	plot3DWindows<<label;

	if (!plot3DTools->isVisible())
		plot3DTools->show();

	if (!plot3DTools->isEnabled())
			plot3DTools->setEnabled(TRUE);

	customMenu((QWidget*)plot);
	customToolBars((QWidget*)plot);

return plot;
}


Graph3D* ApplicationWindow::dataPlot3D(const QString& formula)
{
	QApplication::setOverrideCursor(waitCursor);

	int pos=formula.find(":",0);
	QString caption=formula.left(pos)+"_";
	WksheetWindow *w=(WksheetWindow *)table(caption);
	if (!w)
		{
		QApplication::restoreOverrideCursor();
		return 0;
		}

	int posX=formula.find("(",pos);
	QString xColName=caption+formula.mid(pos+2,posX-pos-2);

	posX=formula.find(",",posX);
	int posY=formula.find("(",posX);
	QString yColName=caption+formula.mid(posX+2,posY-posX-2);

	QString label="graph"+QString::number(++graphs);
	while(plotWindows.contains(label)>0 || plot3DWindows.contains(label)>0
		 || tableWindows.contains(label)>0){
		label="graph"+QString::number(++graphs);
		}

	Graph3D *plot=new Graph3D(w, xColName, yColName,ws,0,WDestructiveClose);
	connectSurfacePlot(plot);

	plot->resize(500,400);
	plot->setCaption(label);
	plot->setIcon( QPixmap(trajectory_xpm));
	plot->show();
	plot->setFocus();

	QListViewItem *element=new QListViewItem (lv);
	element->setPixmap (0,QPixmap(trajectory_xpm));
	element->setText (0,label);
	element->setText (1,"Data Plot 3D");
	element->setText (2,"Normal");
	element->setText (3,plot->sizeToString());
	element->setText (4,plot->birthDate());

	plot3DWindows<<label;

	if (!plot3DTools->isVisible())
		plot3DTools->show();

	if (!plot3DTools->isEnabled())
			plot3DTools->setEnabled(TRUE);

	customMenu((QWidget*)plot);
	customToolBars((QWidget*)plot);

emit modified();
QApplication::restoreOverrideCursor();
return plot;
}

Graph3D* ApplicationWindow::dataPlotXYZ(WksheetWindow* table,const QString& zColName, int type)
{
	QApplication::setOverrideCursor(waitCursor);
	graphs++;
	QString label="graph"+QString::number(graphs);
	while(plotWindows.contains(label)>0 || plot3DWindows.contains(label)>0
		 || tableWindows.contains(label)>0)
		{
		graphs++;
		label="graph"+QString::number(graphs);
		}

	int xCol=table->colX();
	int yCol=table->colY();
	int zCol=table->colIndex(zColName);

	Graph3D *plot=new Graph3D(table,xCol,yCol,zCol,type,ws,0,WDestructiveClose);
	connectSurfacePlot(plot);

	plot->resize(500,400);
	plot->setCaption(label);
	plot->setIcon( QPixmap(trajectory_xpm));
	plot->show();
	plot->setFocus();

	QListViewItem *element=new QListViewItem (lv);
	element->setPixmap (0,QPixmap(trajectory_xpm));
	element->setText (0,label);
	element->setText (1,"Data Plot 3D");
	element->setText (2,"Normal");
	element->setText (3,plot->sizeToString());
	element->setText (4,plot->birthDate());

	plot3DWindows<<label;

	if (!plot3DTools->isVisible())
		plot3DTools->show();

	if (!plot3DTools->isEnabled())
			plot3DTools->setEnabled(TRUE);

	customMenu((QWidget*)plot);
	customToolBars((QWidget*)plot);

emit modified();
QApplication::restoreOverrideCursor();
return plot;
}

Graph3D* ApplicationWindow::dataPlotXYZ(const QString& caption,const QString& formula,
 												double xl, double xr, double yl, double yr, double zl, double zr)
{
	int pos=formula.find("_",0);
	QString wCaption=formula.left(pos);

	WksheetWindow* w=table(wCaption);
	if (!w)
		return 0;

	int posX=formula.find("(",pos);
	QString xColName=formula.mid(pos+1,posX-pos-1);

	pos=formula.find(",",posX);

	posX=formula.find("(",pos);
	QString yColName=formula.mid(pos+1,posX-pos-1);

	pos=formula.find(",",posX);
	posX=formula.find("(",pos);
	QString zColName=formula.mid(pos+1,posX-pos-1);

	int xCol=w->colIndex(xColName);
	int yCol=w->colIndex(yColName);
	int zCol=w->colIndex(zColName);

Graph3D *plot=new Graph3D(w,xCol,yCol,zCol,xl,xr,yl,yr,zl,zr,ws,0,WDestructiveClose);
connectSurfacePlot(plot);

QString label=caption;
while(plotWindows.contains(label)>0 || plot3DWindows.contains(label)>0
			|| tableWindows.contains(label)>0)
	{
	graphs++;
	label="graph"+QString::number(graphs);
	}

plot->setCaption(label);
plot->setIcon( QPixmap(trajectory_xpm));
plot->show();
plot->setFocus();

QListViewItem *element=new QListViewItem (lv);
element->setPixmap (0,QPixmap(trajectory_xpm));
element->setText (0,label);
element->setText (1,"Data Plot 3D");
element->setText (2,"Normal");
element->setText (4,plot->birthDate());

plot3DWindows<<label;

if (!plot3DTools->isVisible())
	plot3DTools->show();

if (!plot3DTools->isEnabled())
	plot3DTools->setEnabled(TRUE);

customMenu((QWidget*)plot);
return plot;
}

Graph3D* ApplicationWindow::dataPlotXYZ(const QString& formula)
{
	QApplication::setOverrideCursor(waitCursor);

	int pos=formula.find(":",0);
	QString caption=formula.left(pos)+"_";
	WksheetWindow *w=(WksheetWindow *)table(caption);
	if (!w)
		{
		QApplication::restoreOverrideCursor();
		return 0;
		}

	int posX=formula.find("(",pos);
	QString xColName=caption+formula.mid(pos+2,posX-pos-2);

	posX=formula.find(",",posX);
	int posY=formula.find("(",posX);
	QString yColName=caption+formula.mid(posX+2,posY-posX-2);

	posY=formula.find(",",posY);
	int posZ=formula.find("(",posY);
	QString zColName=caption+formula.mid(posY+2,posZ-posY-2);

	int xCol=w->colIndex(xColName);
	int yCol=w->colIndex(yColName);
	int zCol=w->colIndex(zColName);

Graph3D *plot=new Graph3D(w,xCol,yCol,zCol,1,ws,0,WDestructiveClose);
connectSurfacePlot(plot);

QString label="graph"+QString::number(++graphs);
while(plotWindows.contains(label)>0 || plot3DWindows.contains(label)>0
			|| tableWindows.contains(label)>0){
	label="graph"+QString::number(++graphs);}

plot->resize(500,400);
plot->setCaption(label);
plot->setIcon( QPixmap(trajectory_xpm));
plot->show();
plot->setFocus();

QListViewItem *element=new QListViewItem (lv);
element->setPixmap (0,QPixmap(trajectory_xpm));
element->setText (0,label);
element->setText (1,"Data Plot 3D");
element->setText (2,"Normal");
element->setText (3,plot->sizeToString());
element->setText (4,plot->birthDate());

plot3DWindows<<label;

if (!plot3DTools->isVisible())

	plot3DTools->show();

if (!plot3DTools->isEnabled())
	plot3DTools->setEnabled(TRUE);

customMenu((QWidget*)plot);
customToolBars((QWidget*)plot);

QApplication::restoreOverrideCursor();
return plot;
}

void ApplicationWindow::loadImage()
{
QStringList list=QImage::inputFormatList ();
QString filter="Images (*.jpg *.JPG ",aux;
int i;
for (i=0;i<(int)list.count();i++)
	{
	aux="*."+(list[i]).lower()+" *."+list[i]+" ";
	filter+=aux;
	}
filter+=");;";
for (i=0;i<(int)list.count();i++)
	{
	aux="*."+(list[i]).lower()+" *."+list[i]+";;";
	filter+=aux;
	}

QString fn = QFileDialog::getOpenFileName(QString::null, filter, 0, 0,
			"QtiPlot - Load image from file", 0, TRUE);
if ( !fn.isEmpty() )
	{
	QApplication::setOverrideCursor(waitCursor);
	QPixmap photo;
	if ( fn.contains(".jpg")>0 || fn.contains(".JPG")>0)
			photo.load(fn,"JPEG",QPixmap::Auto);
	else
	{
	for (i=0;i<(int)list.count();i++)
		{
		if (fn.contains("."+(list[i]).lower())>0 || fn.contains("."+list[i])>0)
			{
			photo.load(fn,list[i],QPixmap::Auto);
			break;
			}
		}
	}

	MultiLayer *plot=multilayerPlot(fn);
	if (plot->height()-20>photo.height())
		plot->setGeometry(0,0, plot->width(), photo.height()+20);
	
	plot->showNormal();
	Graph *g=plot->loadLayer(0,0, plot->width(), plot->height()-20);

	g->setTitle("");
	QArray<bool> axesOn(4);
	for (int j=0;j<4;j++)
		axesOn[j]=FALSE;
	g->enableAxes(axesOn);
	g->removeLegend();
	g->insertImageMarker(photo,fn);
	plot->connectLayer(g);
	QApplication::restoreOverrideCursor();
	}
}

MultiLayer* ApplicationWindow::multilayerPlot(WksheetWindow* w,const QStringList& colList, int style)
{
QApplication::setOverrideCursor(waitCursor);

MultiLayer* g = new MultiLayer(ws,0,WDestructiveClose);
connectMultilayerPlot(g);
g->askOnCloseEvent(confirmClosePlot2D);

graphs++;
QString label="graph"+QString::number(graphs);
while(plotWindows.contains(label)>0 || plot3DWindows.contains(label)>0
	  || tableWindows.contains(label)>0)
	{
	graphs++;
	label="graph"+QString::number(graphs);
	}

QString date=g->birthDate();
activeGraph=g->insertFirstGraph(w,colList,style);
customGraph(activeGraph);

if (style == Graph::VerticalBars || style == Graph::HorizontalBars ||style == Graph::Histogram)
	{
	QValueList<int> ticksList;
	int ticksStyle= Plot::Out;
	ticksList<<ticksStyle<<ticksStyle<<ticksStyle<<ticksStyle;
	activeGraph->setTicksType(ticksList);
	}

if (style == Graph::HorizontalBars)
	{
	activeGraph->setAxisTitle(0, "Y Axis Title");
	activeGraph->setAxisTitle(1, "X Axis Title");
	}
	
QListViewItem *element=new QListViewItem (lv);
element->setPixmap (0,QPixmap(graph_xpm));
element->setText (0,label);
element->setText (1,"Plot");
element->setText (2,"Normal");
element->setText (3,g->sizeToString());
element->setText (4,date);

plotWindows<<label;

g->setGeometry(0,0,500,420);

g->setCaption(label);
g->setIcon( QPixmap(graph_xpm));
g->showNormal();
g->setFocus();

emit modified();
QApplication::restoreOverrideCursor();

return g;
}

MultiLayer* ApplicationWindow::multilayerPlot(const QString& caption)
{
MultiLayer* g = new MultiLayer(ws,0,WDestructiveClose);
connectMultilayerPlot(g);

QString label=caption;
while(plotWindows.contains(label)>0 || plot3DWindows.contains(label)>0
	  || tableWindows.contains(label)>0)
	{
	graphs++;
	label="graph"+QString::number(graphs);
	}
label=label.replace(QRegExp("_"),"-");

QString date=g->birthDate();

QListViewItem *element=new QListViewItem (lv);
element->setPixmap (0,QPixmap(graph_xpm));
element->setText (0,label);
element->setText (1,"Plot");
element->setText (2,"Normal");
element->setText (3,"");
element->setText (4,date);

plotWindows<<label;

g->setGeometry(0,0,500,420);
g->setIcon(QPixmap(graph_xpm));
g->setCaption(label);
g->setFocus();
return g;
}

MultiLayer* ApplicationWindow::multilayerPlot(const QStringList& colList)
{
QApplication::setOverrideCursor(waitCursor);
MultiLayer* g = new MultiLayer(ws,0,WDestructiveClose);
connectMultilayerPlot(g);
	
graphs++;
QString label="graph"+QString::number(graphs);
while(plotWindows.contains(label)>0 || plot3DWindows.contains(label)>0
	  || tableWindows.contains(label)>0)
	{
	graphs++;
	label="graph"+QString::number(graphs);
	}

QString date=g->birthDate();

QListViewItem *element=new QListViewItem (lv);
element->setPixmap (0,QPixmap(graph_xpm));
element->setText (0,label);
element->setText (1,"Plot");
element->setText (2,"Normal");
element->setText (4,date);

plotWindows<<label;

QString colHead;
QStringList labels;

curveLayout cl=Graph::initCurveLayout();
Graph *ag=g->loadLayer();
customGraph(ag);

for (int i=0;i<(int)colList.count();i++)
	{
	QString s=colList[i];
	int pos=s.find(":",0);
	QString caption=s.left(pos)+"_";
	WksheetWindow *w=(WksheetWindow *)table(caption);
	int xinit=w->colX();

	int posX=s.find("(",pos);
	QString xCol=caption+s.mid(pos+2,posX-pos-2);
	int x=w->colIndex(xCol);
	w->setXCol(x);

	posX=s.find(",",posX);
	int posY=s.find("(",posX);
	QString yCol=caption+s.mid(posX+2,posY-posX-2);

	if (s.contains("(yErr)",TRUE)>0)
		{
		posY=s.find(",",posY);
		int posErr=s.find("(",posY);
		QString errCol=s.mid(posY+2,posErr-posY-2);
		ag->addErrorBars(w,yCol,1,errCol);
		}
	else if (s.contains("(xErr)",TRUE)>0)
		{
		posY=s.find(",",posY);
		int posErr=s.find("(",posY);
		QString errCol=s.mid(posY+2,posErr-posY-2);
		ag->addErrorBars(w,yCol,0,errCol);
		}
	else
		ag->plot(w,yCol,2);

	w->setXCol(xinit);

	cl.lCol=i;
	cl.symCol=i;
	cl.fillCol=i;
	ag->updateCurveLayout(i,cl);
	}

g->setCaption(label);
g->setIcon( QPixmap(graph_xpm));
g->setGeometry( QRect( 0, 0, 500, 420) );
g->showNormal();
g->setFocus();

element->setText (3,g->sizeToString());
emit modified();
ag->newLegend(plotLegendFont, legendFrameStyle);
g->connectLayer(ag);

QApplication::restoreOverrideCursor();
return g;
}

void ApplicationWindow::customizeTables(const QColor& bgColor,const QColor& textColor,
																		const QColor& headerColor,const QFont& textFont,
																		const QFont& headerFont)
{
if (tableBkgdColor == bgColor && tableTextColor == textColor &&
	tableHeaderColor == headerColor && tableTextFont == textFont &&
	tableHeaderFont == headerFont)
return;

tableBkgdColor = bgColor;
tableTextColor = textColor;
tableHeaderColor = headerColor;
tableTextFont = textFont;
tableHeaderFont = headerFont;

QWidgetList windows = ws->windowList();
int i=0;
for (i = 0; i < int(windows.count());i++ )
	{
	if (tableWindows.contains(windows.at(i)->caption())>0)
		customTable((WksheetWindow*)windows.at(i));
	}

for (i = 0; i < int(hiddenWindows->count());i++ )
	{
	if (tableWindows.contains(hiddenWindows->at(i)->caption())>0)
			customTable((WksheetWindow*)hiddenWindows->at(i));
	}
}

void ApplicationWindow::customTable(WksheetWindow* w)
{
w->setBackgroundColor ( tableBkgdColor);
w->setTextColor (tableTextColor);
w->setHeaderColor (tableHeaderColor);
w->setTextFont(tableTextFont);
w->setHeaderFont(tableHeaderFont);
}

void ApplicationWindow::customGraph(Graph* g)
{
QValueList<int> ticksList;
ticksList<<ticksStyle<<ticksStyle<<ticksStyle<<ticksStyle;
g->setTicksType(ticksList);

LegendMarker* legend = g->legend();
if (legend)
	{
	legend->setBackground(legendFrameStyle);
	legend->setFont(plotLegendFont);
	}

g->initTitleFont(plotTitleFont);
g->initTitle(titleOn);
g->initFonts(plotAxesFont, plotNumbersFont);
g->drawCanvasFrame(canvasFrameOn);
}

void ApplicationWindow::newWrksheetPlot(const QString& caption, int r, int c, const QString& text)
{
WksheetWindow* w =newWrksheet(caption, r, c,text);
QStringList colList;
colList<<w->caption()+"_intensity";
MultiLayer* plot=multilayerPlot(w,colList,0);
Graph *g=(Graph*)plot->activeGraph();
if (g)
	{
	g->setTitle("");
	g->setXAxisTitle("pixels");
	g->setYAxisTitle("pixel intensity (a.u.)");
	}
}

/*
*used when importing an ASCII file
*/
WksheetWindow* ApplicationWindow::newWrksheet(const QString& fname,const QString &sep, int lines)
{
WksheetWindow* w = new WksheetWindow(fname,sep, lines, ws,0,WDestructiveClose);
	
QFileInfo fi(fname);
initTable(w, fi.baseName());
	
w->showNormal();
return w;
}

/*
*creates a new empty table
*/
WksheetWindow* ApplicationWindow::newWrksheet()
{
WksheetWindow* w = new WksheetWindow(30, 2, ws, 0, WDestructiveClose);

tables++;
QString caption="table"+QString::number(tables);
	
initTable(w, caption);
w->showNormal();
	
return w;
}

/*
*used when opening a project file
*/
WksheetWindow* ApplicationWindow::newWrksheet(const QString& caption, int r, int c)
{
WksheetWindow* w = new WksheetWindow(r,c,ws,0,WDestructiveClose);
initTable(w, caption);
return w;
}

/*
*used to return the result of an analysis operation
*/
WksheetWindow* ApplicationWindow::newWrksheet(const QString& caption, int r, int c,const QString& text)
{
WksheetWindow* w = new WksheetWindow(r,c,ws,0,WDestructiveClose);
	
QStringList rows=QStringList::split ("\n",text,FALSE);
QString rlist=rows[0];
QStringList list=QStringList::split ("\t",rlist,TRUE);
w->setHeader(list);

for (int i=0; i<r; i++)
	{
	rlist=rows[i+1];
	list=QStringList::split ("\t",rlist,TRUE);
	for (int j=0; j<c; j++)
		w->setText(i, j, list[j]);
	}
	
initTable(w, caption);
w->showNormal();
return w;
}

void ApplicationWindow::initTable(WksheetWindow* w, const QString& caption)
{
connectTable(w);
customTable(w);

QString name=caption;
name=name.replace ("_","-");

while(tableWindows.contains(name)>0 || plotWindows.contains(name)>0
	  || plot3DWindows.contains(name)>0)
	{
	tables++;
	name="table"+QString::number(tables);
	}

tableWindows<<name;
	
w->setCaption(name);
w->setIcon( QPixmap(worksheet_xpm) );
w->setSpecifications(w->makeSpecifications());
	
QListViewItem *element=new QListViewItem (lv);
element->setPixmap (0,QPixmap(worksheet_xpm));
element->setText (0, name);
element->setText (1, "Table");
element->setText (2, "Normal");
element->setText (3, w->sizeToString());
element->setText (4, w->birthDate());
	
emit modified();
}

void ApplicationWindow::showHistogramTable(const QString& caption, int r, int c, const QString& text)
{
WksheetWindow* w =newWrksheet(caption, r, c,text);
w->showMaximized();
}

void ApplicationWindow::showWorksheet(const QString& curve)
{
WksheetWindow* w=table(curve);
if (w)
	w->showMaximized();

emit modified();
}

QWidget* ApplicationWindow::window(QListViewItem *it)
{
if (!it)
	return 0;

QWidget*w=0;
int i;
QString caption=it->text(0);
QString hidden=it->text(2);

if (hidden=="Hidden")
{
for (i=0;i<(int)hiddenWindows->count();i++)
	{
	if (hiddenWindows->at(i)->caption()==caption)
		{
		w=hiddenWindows->at(i);
		break;
		}
	}
}
else
{
QWidgetList windows =ws->windowList();
for (i=0;i<(int)windows.count();i++)
	{
	if (windows.at(i)->caption()==caption)
		{
		w=windows.at(i);
		break;
		}
	}
}
return w;
}

Graph3D* ApplicationWindow::surfacePlot(const QString& name)
{
int i;
Graph3D* w=0;

QWidgetList windows = ws->windowList();
for (i = 0; i < int(windows.count());i++ )
	{
	if (plot3DWindows.contains(name)>0
		&& windows.at(i)->caption()==name)
		{
		w=(Graph3D*)windows.at(i);
		break;
		}
	}

for (i = 0; i < int(hiddenWindows->count());i++ )
	{
	if (plot3DWindows.contains(name)>0
		&& hiddenWindows->at(i)->caption()==name)
		{
		w=(Graph3D*)hiddenWindows->at(i);
		break;
		}
	}
return  w;
}

MultiLayer* ApplicationWindow::plot(const QString& name)
{
int i,pos=name.find("_",0);
QString caption=name.left(pos);
MultiLayer* w=0;

QWidgetList windows = ws->windowList();
for (i = 0; i < int(windows.count());i++ )
	{
	if (plotWindows.contains(caption)>0
		&& windows.at(i)->caption()==caption)
		{
		w=(MultiLayer*)windows.at(i);
		break;
		}
	}

for (i = 0; i < int(hiddenWindows->count());i++ )
	{
	if (plotWindows.contains(caption)>0
		&& hiddenWindows->at(i)->caption()==caption)
		{
		w=(MultiLayer*)hiddenWindows->at(i);
		break;
		}
	}
return  w;
}

WksheetWindow* ApplicationWindow::table(const QString& name)
{
int i,pos=name.find("_",0);
QString caption=name.left(pos);
WksheetWindow* w=0;

QWidgetList windows = ws->windowList();
for (i = 0; i < int(windows.count());i++ )
	{
	if (tableWindows.contains(caption)>0
		&& windows.at(i)->caption()==caption)
		{
		w=(WksheetWindow*)windows.at(i);
		return w;
		}
	}

for (i = 0; i < int(hiddenWindows->count());i++ )
	{
	if (tableWindows.contains(caption)>0
		&& hiddenWindows->at(i)->caption()==caption)

		return (WksheetWindow*)hiddenWindows->at(i);
	}
return  w;
}

void ApplicationWindow::windowActivated(QWidget *w)
{
customToolBars(w);
customMenu(w);
emit modified();
}

void ApplicationWindow::addCurve(Graph* g,const QString& name)
{
WksheetWindow* w=table(name);
if (w)
	{
	g->plot(w, name, Graph::LineSymbols);
	curveLayout cl=Graph::initCurveLayout();
		
	QArray<curveLayout> layout = g->allCurvesLayout();
	int cvs = g->curves();
	int color;
	int maxColor = 0;
	for (int i = 0; i<cvs; i++)
		{
		color = (layout[i]).lCol;
		if (color > maxColor)
			maxColor = color;
		}
		
	if (cvs < 12)
		color = maxColor+1;
	else
		{
		color = cvs%13;
		cl.sType=cvs/9;
		}
		
	cl.lCol=color;
	cl.symCol=color;
	cl.fillCol=color;	
	
	g->updateCurveLayout(cvs-1, cl);
	emit modified();
	}
}

void ApplicationWindow::addErrorBars()
{
 MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
		return;

Graph* g = (Graph*)plot->activeGraph();
if ( g )
	{
	if (g->isPiePlot())
		QMessageBox::warning(this,"QtiPlot - Warning",
				"This functionality is not available for pie plots!");
	else
		{
		activeGraph=g;
		errDialog* ed= new errDialog(this,"errDialog",TRUE,WStyle_Tool|WDestructiveClose);
		connect (ed,SIGNAL(options(const QString&,int,const QString&,int)),this,SLOT(defineErrorBars(const QString&,int,const QString&,int)));
		connect (ed,SIGNAL(options(const QString&,const QString&,int)),this,SLOT(defineErrorBars(const QString&,const QString&,int)));

		QStringList curvesOnPlot=activeGraph->plotCurvesList();

		ed->setCurveNames(curvesOnPlot);
		ed->setExistingColumns(columnsList());
		ed->showNormal();
		}
	}
}

void ApplicationWindow::defineErrorBars(const QString& name,int type,const QString& percent,int direction)
{
	WksheetWindow *w=table(name);
	if (!w) //user defined function
		{
		QMessageBox::critical(this,"QtiPlot - Error bars error",
				"This feature is not available for user defined function curves!");
		return;
		}

	w->addCol();
	int r=w->tableRows();
	int c=w->tableCols()-1;
	int id=w->colIndex(name);

	QArray<double> Y(r);
	Y=w->col(id);
	QString errName=QString::number(c+1);

	bool ok;
	double prc=percent.toDouble(&ok);

	int i;
	double moyenne=0.0;
	if (type==0)
		{
		for (i=0;i<r;i++)
			w->setText(i,c,QString::number(Y[i]*prc/100.0,'g',6));
		}
	else if (type==1)
		{
		double dev=0.0;
		for (i=0;i<r;i++)
			moyenne+=Y[i];
		moyenne/=r;
		for (i=0;i<r;i++)
			dev+=(Y[i]-moyenne)*(Y[i]-moyenne);
		dev=sqrt(dev/(r-1));
		for (i=0;i<r;i++)
			w->setText(i,c,QString::number(dev,'g',6));
		}

activeGraph->addErrorBars(w,name,direction,errName);
}

void ApplicationWindow::defineErrorBars(const QString& curveName,const QString& errColumnName, int direction)
{
WksheetWindow *w=table(curveName);
if (!w) //user defined function --> no worksheet available
	{
	QMessageBox::critical(this,"QtiPlot - Error bars error",
				"This feature is not available for user defined function curves!");
	return;
	}

WksheetWindow *errTable=table(errColumnName);
if (w->tableRows() != errTable->tableRows())
	{
	QMessageBox::critical(this,"QtiPlot - Rows number error",
				"The selected columns have different numbers of rows!");

	addErrorBars();
	return;
	}

int errCol=errTable->colIndex(errColumnName);
if (errTable->isEmptyColumn(errCol))
	{
	QMessageBox::critical(this,"QtiPlot - Rows number error",
				"The selected error column is empty!");
	addErrorBars();
	return;
	}

activeGraph->addErrorBars(w, curveName, errTable, errColumnName, direction);
emit modified();
}

void ApplicationWindow::removeColFromGraphs(QString& name)
{
QApplication::setOverrideCursor(waitCursor);

QWidgetList windows = ws->windowList();
int c=windows.count();
for (int i=0; i<c; i++)
	{
	if (plotWindows.contains(windows.at(i)->caption())>=1)
		{
		MultiLayer* plot = (MultiLayer*)windows.at(i);
		QWidgetList *graphsList=plot->graphPtrs();
		for (int j=0; j<(int)graphsList->count(); j++)
			{
			Graph* g=(Graph*)graphsList->at(j);				
			QStringList associations=g->plotAssociations();		
			for (int k=0; k<int(associations.count()); k++)
				{
				QString ass = associations[k];
				if (ass.contains(name, TRUE) >0)
					g->removeCurve(ass);
				}			
			}
		}
	 else if (plot3DWindows.contains(windows.at(i)->caption())>0)
		{
		Graph3D* g = (Graph3D*)windows.at(i);
		if ((g->formula()).contains(name,TRUE)>0)
			g->removePlot();
		}
	}

QApplication::restoreOverrideCursor();
}

void ApplicationWindow::updateCurveInGraphs(const QString& name)
{
QApplication::setOverrideCursor(waitCursor);
QWidgetList windows = ws->windowList();
int c=(int)windows.count();
WksheetWindow *W=table(name);
if (!W)
	return;

for (int i=0; i<c; i++)
	{
	QString caption=windows.at(i)->caption();
	if (plotWindows.contains(caption)>0)
		{
		MultiLayer* plot = (MultiLayer*)windows.at(i);
		QWidgetList *graphsList=plot->graphPtrs();
		for (int k=0; k<(int)graphsList->count(); k++)
			{
			Graph* g=(Graph*)graphsList->at(k);
			if (g)
				{
				QStringList associations=g->plotAssociations();
				int cvs = associations.count();
				if (cvs > 0)
					{
					for (int j=0; j<cvs; j++)
						{
						if (associations[j].contains(name, TRUE)>0)
							g->updateCurveData(W, name, j);
						}
					}
				 }
			}
		}
	else if (plot3DWindows.contains(caption)>0)
		{
		Graph3D* g = (Graph3D*)windows.at(i);
		if ((g->formula()).contains(name,TRUE)>0)
			g->updateData(W);
		}
	}
QApplication::restoreOverrideCursor();
}

void ApplicationWindow::showConfigureDialog()
{
configDialog* cd= new configDialog(this,"configDialog",TRUE,WStyle_Tool|WDestructiveClose);

connect (cd,SIGNAL(updateTables(const QColor&,const QColor&,const QColor&,const QFont&, const QFont&)),
	this,SLOT(customizeTables(const QColor&,const QColor&,const QColor&,const QFont&, const QFont&)));
connect (cd,SIGNAL(setGraphDefaultSettings(int, int, bool, bool,const QFont&,const QFont&, const QFont&, const QFont&)),
	this,SLOT(setGraphDefaultSettings(int, int, bool, bool,const QFont&,const QFont&, const QFont&, const QFont&)));
connect (cd,SIGNAL(updateConfirmOptions(bool, bool, bool)),
	this,SLOT(updateConfirmOptions(bool, bool, bool)));
connect (cd,SIGNAL(changeAppFont(const QFont&)),
	this,SLOT(changeAppFont(const QFont&)));
connect (cd,SIGNAL(changeAppStyle(const QString&)),
	this,SLOT(changeAppStyle(const QString&)));
connect (cd,SIGNAL(setSaveSettings(bool, int)),
	this,SLOT(setSaveSettings(bool, int)));
connect (cd,SIGNAL(setAppColors(const QColor&,const QColor&,const QColor&)),
	this,SLOT(setAppColors(const QColor&,const QColor&,const QColor&)));
	
cd->initAppColors(workspaceColor, panelsColor, panelsTextColor);
cd->initColors(tableBkgdColor, tableTextColor, tableHeaderColor);
cd->initFonts(tableTextFont, tableHeaderFont);
cd->initGraphFonts(plotAxesFont, plotNumbersFont, plotLegendFont, plotTitleFont);
cd->initPlotOptions(ticksStyle, legendFrameStyle, titleOn, canvasFrameOn);
cd->initConfirmOptions(confirmCloseTable, confirmClosePlot2D, confirmClosePlot3D);
cd->initAppFonts(appFont);
cd->initAppStyle(appStyle);
cd->initSaveSettings(autoSave, autoSaveTime);
cd->showNormal();
cd->setActiveWindow();
}

void ApplicationWindow::setSaveSettings(bool autoSaving, int min)
{
if (autoSave==autoSaving && autoSaveTime==min)
	return;

autoSave=autoSaving;
autoSaveTime=min;	
	
killTimer(savingTimerId);

if (autoSave)
	savingTimerId=startTimer(autoSaveTime*60000);
else
	savingTimerId=0;
}
	
void ApplicationWindow::initAppStyle(const QString& s)
{
if (QStyle *style = QStyleFactory::create(s))
    {
        qApp->setStyle(style);
		appStyle = s;
    }
}

void ApplicationWindow::changeAppStyle(const QString& s)
{
if (appStyle == s)
	return;

if (QStyle *style = QStyleFactory::create(s))
    {
        qApp->setStyle(style);
        appStyle = s;
    }
}

void ApplicationWindow::changeAppFont(const QFont& f)
{
if (appFont == f)
	return;

appFont=f;
updateAppFonts();
}

void ApplicationWindow::updateAppFonts()
{
qApp->setFont (appFont);
this->setFont(appFont);
windowsMenu->setFont(appFont);
view->setFont(appFont);
graph->setFont(appFont);
file->setFont(appFont);
format->setFont(appFont);
calcul->setFont(appFont);
edit->setFont(appFont);
dataMenu->setFont(appFont);
recent->setFont(appFont);
help->setFont(appFont);
type->setFont(appFont);
import->setFont(appFont);
plot2D->setFont(appFont);
plot3D->setFont(appFont);
specialPlot->setFont(appFont);
panels->setFont(appFont);
stat->setFont(appFont);
inter->setFont(appFont);
fft->setFont(appFont);
decay->setFont(appFont);
}

void ApplicationWindow::updateConfirmOptions(bool askTables, bool askPlots2D, bool askPlots3D)
{
QWidgetList windows = ws->windowList();
int i=0;

if (confirmCloseTable != askTables)
	{
	confirmCloseTable=askTables;
	for (i = 0; i < int(windows.count());i++ )
		{
		if (tableWindows.contains(windows.at(i)->caption())>0)
			((WksheetWindow*)windows.at(i))->askOnCloseEvent(confirmCloseTable);
		}
	}

if (confirmClosePlot2D != askPlots2D)
	{
	confirmClosePlot2D=askPlots2D;
	for (i = 0; i < int(windows.count());i++ )
		{
		if (plotWindows.contains(windows.at(i)->caption())>0)
			((MultiLayer*)windows.at(i))->askOnCloseEvent(confirmClosePlot2D);
		}
	}

if (confirmClosePlot3D != askPlots3D)
	{
	confirmClosePlot3D=askPlots3D;
	for (i = 0; i < int(windows.count());i++ )
		{
		if (plot3DWindows.contains(windows.at(i)->caption())>0)
			((Graph3D*)windows.at(i))->askOnCloseEvent(confirmClosePlot3D);
		}
	}
}

void ApplicationWindow::setGraphDefaultSettings(int ticks, int legendFrame, bool title, bool canvasFrame,
														const QFont& axesFnt,const QFont& numFnt,
														const QFont& legendFnt, const QFont& titleFnt)
{
ticksStyle=ticks;
legendFrameStyle=legendFrame;
titleOn=title;
canvasFrameOn=canvasFrame;
plotAxesFont=axesFnt;
plotNumbersFont=numFnt;
plotLegendFont=legendFnt;
plotTitleFont=titleFnt;
}

void ApplicationWindow::plotFile(const QString& fn)
{	
QApplication::setOverrideCursor(waitCursor);
ApplicationWindow *app= new ApplicationWindow();
app->setCaption("QtiPlot - "+fn);
app->showMaximized();
app->projectname=fn;

WksheetWindow* w =app->newWrksheet(fn,"auto", 0);	
w->selectAllTable();

app->multilayerPlot(w, w->selectedColumns(),Graph::LineSymbols);
w->deselect();
QApplication::restoreOverrideCursor();
}

void ApplicationWindow::showImportDialog()
{
importDialog* id= new importDialog(this,"importDialog",TRUE,WStyle_Tool|WDestructiveClose);
connect (id,SIGNAL(options(const QString&, int)),this,SLOT(setImportOptions(const QString&, int)));
id->setSeparator(separator);
id->setLines(ignoredLines);
id->showNormal();
id->setActiveWindow();
}

void ApplicationWindow::setImportOptions(const QString& sep, int lines)
{
separator=sep;
ignoredLines=lines;
}

void ApplicationWindow::load()
{
QString filter="All files *;;Text (*.TXT *.txt);;Data (*DAT *.dat);;";
QString fn = QFileDialog::getOpenFileName(QString::null, filter, 0, 0,
			"QtiPlot - Import ASCII File", 0, TRUE);
if ( !fn.isEmpty() )
	newWrksheet(fn,separator, ignoredLines);
}

void ApplicationWindow::loadMultiple()
{
QString filter="All files *;;Text (*.TXT *.txt);;Data (*DAT *.dat);;";
QStringList s( QFileDialog::getOpenFileNames(filter,QString::null, 0, 0,
			"QtiPlot - Import Multiple ASCII Files", 0, TRUE) );

if (s.count()==0)
	return;

QString fn  = s[0];
WksheetWindow *firstTable=newWrksheet(fn, separator, ignoredLines);
if (!firstTable)
	return;

int dx=firstTable->verticalHeaderWidth();
int dy=firstTable->parentWidget()->frameGeometry().height() - firstTable->height();
firstTable->parentWidget()->move(QPoint(0,0));

for (int i=1;i<(int)s.count();i++)
	{
	fn  = s[i];
	WksheetWindow *w=newWrksheet(fn, separator, ignoredLines);
	if (w)
		w->parentWidget()->move(QPoint(i*dx,i*dy));
    }
}

bool ApplicationWindow::open(const QString& fn)
{
if  (!fn.contains(".qti"))
	{
	plotFile(fn);
	return true;
	}
	
QFile f(fn);
QTextStream t( &f );
f.open(IO_ReadOnly);
QString s = t.readLine();
QStringList list=QStringList::split (QRegExp("\\s"),s,false);

QString fileType=list[0], version=list[1];
if (fileType != "QtiPlot")
	{
	QMessageBox::critical(this,"QtiPlot - File openning error",
				"The file: <p> <b>"+fn+"</b> <p> was not created using QtiPlot!");
	return FALSE;
	}

fileVersion = (version.right(3)).toDouble();
if (fileVersion < 4)
	openQtiFile_03(fn);
else
	openQtiFile(fn);

f.close();
return TRUE;
}

void ApplicationWindow::openQtiFile_03(const QString& fn)
{
	QFile f(fn);
	QTextStream t( &f );
	f.open(IO_ReadOnly);

	QApplication::setOverrideCursor(waitCursor);
	QString s,caption,rows,cols;
	QStringList header,values,geometry,curve,plotCaptions,fList;
	WksheetWindow *w=0;
	MultiLayer *g=0;
	Graph *ag=0;
	aw=0, wmax=0;

	int i,r=0,x,y,width,height, nonEmptyRows=0,curveID=0,xcol=0;
	int tb=0,maxt=0,gr=0,maxg=0,maxFit=0;
	bool ok;
	QRegExp digit = QRegExp("\\d", FALSE, FALSE);

	ApplicationWindow * app= new ApplicationWindow;
	app->setCaption("QtiPlot - "+fn);
	app->showMaximized();
	app->projectname=fn;

	//process tables information
	while ( !t.eof() )
		{
			s = t.readLine();
			QStringList list=QStringList::split ("\t",s,TRUE);
			if (s.contains ("listViewOn\t",TRUE)>0)
				{
				if (s.contains("1",TRUE)>0) app->listViewOn=FALSE;
				else app->listViewOn=TRUE;
				app->showList();
				}
			if (s.left(6)=="table\t")
				{
				bool ok;
				caption=list[1];
				rows=list[2];cols=list[3];

				w = app->newWrksheet(caption, rows.toInt(&ok),cols.toInt(&ok));
				w->setXCol(list[5].toInt(&ok));
				r=0;

				QString date=list[4];
				app->setListViewDate(caption,date);

				nonEmptyRows = 0;
				if (int(list.count()) == 7)
					nonEmptyRows = list[6].toInt();

				if (caption.contains ("table",TRUE)>0)
					{
					tb=caption.mid(5,caption.length()-5).toInt(&ok);
					if (tb>maxt) maxt=tb;
					}

				if (caption.contains ("Fit",TRUE)>0)
					{
					int pos=caption.find("Fit",0);
					tb=caption.right(caption.length()-pos-3).toInt(&ok);
					if (tb>maxFit) maxFit=tb;
					}
				}
			else if (s.left(7)=="header\t")
				{
				s=s.mid(7,s.length()-7);
				header=QStringList::split ("\t",s,FALSE );
				w->setHeader(header);
				}
			else if (s.contains ("tgeometry",TRUE)>0)
				{
				if (s.contains ("minimized",TRUE)>0)
					{
					w->showMinimized();
					app->setListView(w->caption(),"Minimized");
					}
				else if (s.contains ("maximized",TRUE)>0)
					{
					w->show();
					wmax=(QWidget*)w;
					}
				else
					{
					x=list[1].toInt();y=list[2].toInt();
					width=list[3].toInt();height=list[4].toInt();
					w->showNormal();
					w->parentWidget()->setGeometry(x,y,width,height);

					if (list[5]=="active")
						aw=(QWidget*)w;

					if (list[5]=="hidden")
						{
						app->hiddenWindows->append(w);
						w->hide();
						app->setListView(w->caption(),"Hidden");
						}
					else
						app->setListView(w->caption(),"Normal");
					}
				}

			else if (s.left(4)=="com\t")
				w->setCommandes(s);

			else if (s.contains ("ColWidth",TRUE)>0)
				{
				QStringList widths;
				for (int j=0;j<int(list.count());j++)
					widths<<list[j+1];
				w->setColWidths(widths);
				}

			// read table values
			else if (!s.isEmpty())
				{
				if (nonEmptyRows > 0)
					{
					int c=cols.toInt(&ok);
					if (r< nonEmptyRows )
						{
						for (i=0;i<c;i++)
							w->setText(r,i,list[i]);
						r++;
						}
					}
				else if (nonEmptyRows ==0 && (s.find(digit,0)==0 || s.find("-",0) == 0))
					{
					int c=cols.toInt(&ok);
					for (i=0;i<c;i++)
						w->setText(r,i,list[i]);
					r++;

					if (r<rows.toInt())
						{
						for (i=r;i<rows.toInt();i++)
							{
							for (int j=0;j<c;j++)
								w->setText(i,j,"");
							}
						}
					}
				}
			}//end while
	f.close();
	app->tables=maxt;
	app->fitNumber=maxFit;

	//process graph information
	f.open(IO_ReadOnly);
	while ( !t.eof() )
		{
		s = t.readLine();
		// 3D plots
		if (s.contains ("SurfacePlot",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			caption=fList[1];
			QString date=fList[2];

			QString sg = t.readLine();
			s = t.readLine();

			fList=QStringList::split ("\t",s,FALSE );
			Graph3D *plot=app->newPlot3D(caption, fList[1],fList[2].toDouble(),fList[3].toDouble(),
								fList[4].toDouble(),fList[5].toDouble(),
								fList[6].toDouble(),fList[7].toDouble());

			if (sg.contains ("minimized",TRUE)>0)
				{
				plot->showMinimized();
				app->setListView(caption,"Minimized");
				}
			else if (sg.contains ("maximized",TRUE)>0)
				{
				plot->showNormal();
				wmax=(QWidget*)plot;
				}
			else
				{
				geometry=QStringList::split ("\t",sg,FALSE );
				x=geometry[1].toInt();y=geometry[2].toInt();
				width=geometry[3].toInt();height=geometry[4].toInt();
				plot->showNormal();
				plot->parentWidget()->setGeometry(x,y,width,height);

				if (geometry[5]=="active")
					aw=(QWidget*)plot;

				if (geometry[5]=="hidden")
					{
					app->hiddenWindows->append(plot);
					plot->hide();
					app->setListView(caption,"Hidden");
					}
				else
					app->setListView(caption,"Normal");
				}

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setStyle(fList);

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setGrid(fList[1].toInt());

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );

			plot->setTitle(fList);

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setColors(fList);

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setAxesLabels(fList);

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setTicks(fList);

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setTickLengths(fList);

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setOptions(fList);

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setNumbersFont(fList);

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setXAxisLabelFont(fList);

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setYAxisLabelFont(fList);

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setZAxisLabelFont(fList);

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setRotation(fList[1].toDouble(),fList[2].toDouble(),fList[3].toDouble());

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setZoom(fList[1].toDouble());

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setScale(fList[1].toDouble(),fList[2].toDouble(),fList[3].toDouble());

			s = t.readLine();
			fList=QStringList::split ("\t",s,FALSE );
			plot->setShift(fList[1].toDouble(),fList[2].toDouble(),fList[3].toDouble());

			plot->update();

			app->setListViewDate(caption,date);
			app->setListViewSize(caption,plot->sizeToString());
			}
		else if (s.left(6)=="graph\t")
			{
				QStringList graph=QStringList::split ("\t",s,TRUE);
				caption=graph[1];
				plotCaptions<<caption;
				g=app->multilayerPlot(caption);
				ag=(Graph*) g->loadLayer();
				curveID=0;
				QString date=graph[3];
				app->setListViewDate(caption,date);

				if (caption.contains ("graph",TRUE)>0)
					{
					gr=caption.mid(5,caption.length()-5).toInt(&ok);
					if (gr>maxg) maxg=gr;
					}
			}
		else if (s.contains ("ggeometry",TRUE)>0)
			{
			if (s.contains ("minimized",TRUE)>0)
				{
				g->setGraphOrigin(QPoint(0,0));
				g->showMinimized();
				app->setListView(g->caption(),"Minimized");
				}
			else if (s.contains ("maximized",TRUE)>0)
				{
				g->setGraphOrigin(QPoint(0,0));
				g->showMaximized();
				app->setListView(g->caption(),"Maximized");
				wmax=(QWidget*)g;
				}
			else
				{
				geometry=QStringList::split ("\t",s,FALSE );
				x=geometry[1].toInt();y=geometry[2].toInt();
				width=geometry[3].toInt();height=geometry[4].toInt();
				g->setGraphGeometry(0,0,width,height);
				g->showNormal();
				g->parentWidget()->setGeometry(x,y,width,height);

				if (geometry[5]=="active")
					aw=(QWidget*)g;

				if (geometry[5]=="hidden")
					{
					app->hiddenWindows->append(g);
					g->hide();
					app->setListView(g->caption(),"Hidden");
					}
				else
					app->setListView(g->caption(),"Normal");
				}
			g->setCols(1);g->setRows(1);
			}
		else if (s.contains ("EnabledAxes",TRUE)>0)
			{
				fList=QStringList::split ("\t",s,TRUE);
				ag->enableAxes(fList);
			}
		else if (s.contains ("EnabledTicks",TRUE)>0)
			{
				fList=QStringList::split ("\t",s,TRUE);
				ag->setTicksType(fList);
			}
		else if (s.contains ("EnabledTickLabels",TRUE)>0)
			{
				fList=QStringList::split ("\t",s,TRUE);
				for (i=0;i<(int)fList.count();i++)
					fList[i]=fList[i+1];
				ag->setEnabledTickLabels(fList);
			}
		else if (s.contains ("AxesColors",TRUE)>0)
			{
				fList=QStringList::split ("\t",s,TRUE);
				for (i=0;i<(int)fList.count();i++)
					fList[i]=fList[i+1];
				ag->setAxesColors(fList);
			}
		else if (s.contains ("SecondaryAxes",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			for (i=0;i<(int)fList.count();i++)
				fList[i]=fList[i+1];
			ag->setSecondaryPlotLimits(fList);
			}
		else if (s.left(5)=="grid\t")
			{
				QStringList grid=QStringList::split ("\t",s,TRUE);
				gridOptions gr;
				bool ok;
				gr.majorOnX=grid[1].toInt(&ok);
				gr.minorOnX=grid[2].toInt(&ok);
				gr.majorOnY=grid[3].toInt(&ok);
				gr.minorOnY=grid[4].toInt(&ok);
				gr.majorCol=grid[5].toInt(&ok);
				gr.majorStyle=grid[6].toInt(&ok);
				gr.majorWidth=grid[7].toInt(&ok);
				gr.minorCol=grid[8].toInt(&ok);
				gr.minorStyle=grid[9].toInt(&ok);
				gr.minorWidth=grid[10].toInt(&ok);
				gr.xZeroOn=grid[11].toInt(&ok);
				gr.yZeroOn=grid[12].toInt(&ok);
				ag->setGridOptions(gr);
			}
		else if (s.contains ("PieCurve",TRUE)>0)
			{
			curve=QStringList::split ("\t",s,TRUE);
			QString name=curve[1];
			int pos=name.find("_");
			caption=name.left(pos);

			QWidgetList windows = app->ws->windowList();
			int c=windows.count();
			for (i=0;i<c;i++)
				{
				w=(WksheetWindow *)windows.at(i);
				if (w && plotCaptions.contains(w->caption())==0)
					{
					if (w->caption()==caption)
						{
						Qt::PenStyle style;
						if (curve[4]=="SolidLine")
								style=Qt::SolidLine;
						else if (curve[4]=="DashLine")
								style=Qt::DashLine;
						else if (curve[4]=="DotLine")
								style=Qt::DotLine;
						else if (curve[4]=="DashDotLine")
								style=Qt::DashDotLine;
						else if (curve[4]=="DashDotDotLine")
								style=Qt::DashDotDotLine;

						QPen pen=QPen(QColor(curve[3]),curve[2].toInt(),style);
						ag->plotPie(w,name,pen,curve[5].toInt(),curve[6].toInt(),curve[7].toInt());
						break;
						}
					}
				}
			}
		else if (s.left(6)=="curve\t")
			{
			curve=QStringList::split ("\t",s,TRUE);
			QString name=curve[2];
			int pos=name.find("_");
			caption=name.left(pos);
			bool ok;
			xcol=curve[1].toInt(&ok);

			QArray<curveLayout> layout;
			curveLayout cl;
			cl.plotType=curve[3].toInt();
			cl.connectType=curve[4].toInt();
			cl.lCol=curve[5].toInt();
			cl.lStyle=curve[6].toInt();
			cl.lWidth=curve[7].toInt();
			cl.sSize=curve[8].toInt();
			cl.sType=curve[9].toInt();
			cl.symCol=curve[10].toInt();
			cl.fillCol=curve[11].toInt();
			cl.filledArea=curve[12].toInt();
			cl.aCol=curve[13].toInt();
			cl.aStyle=curve[14].toInt();

			layout.resize(curveID+1);
			layout[curveID]=cl;

			QWidgetList windows = app->ws->windowList();
			int c=windows.count();
			for (i=0;i<c;i++)
				{
				w=(WksheetWindow *)windows.at(i);
				if (w && plotCaptions.contains(w->caption())==0)
					{
					if (w->caption()==caption)
						{
						ag->plot(w,xcol,name,2);
						ag->updateCurveLayout(curveID,cl);
						long key=ag->curveKey(curveID);
						ag->setCurveLegend(name,key);
						}
					}
				}
			curveID++;
			}
		else if (s.contains ("FunctionCurve",TRUE)>0)
			{
			curve=QStringList::split ("\t",s,TRUE);
			QString name=curve[1];

			QArray<curveLayout> layout;
			curveLayout cl;
			cl.plotType=curve[5].toInt();
			cl.connectType=curve[6].toInt();
			cl.lCol=curve[7].toInt();
			cl.lStyle=curve[8].toInt();
			cl.lWidth=curve[9].toInt();
			cl.sSize=curve[10].toInt();
			cl.sType=curve[11].toInt();
			cl.symCol=curve[12].toInt();
			cl.fillCol=curve[13].toInt();
			cl.filledArea=curve[14].toInt();
			cl.aCol=curve[15].toInt();
			cl.aStyle=curve[16].toInt();

			layout.resize(curveID+1);
			layout[curveID]=cl;

			ag->insertFunctionCurve(name, curve[3].toDouble(),curve[4].toDouble(),curve[2].toInt());
			ag->updateCurveLayout(curveID,cl);
			curveID++;
			}
		else if (s.contains ("ErrorBars",TRUE)>0)
			{
			curve=QStringList::split ("\t",s,FALSE);
			QString xColName=curve[2];
			QString yColName=curve[3];
			QString errColName=curve[4];

			bool ok,plus=TRUE,minus=TRUE,through=TRUE,xErrors=FALSE;
			int type=curve[1].toInt(&ok);
			if (type==0) xErrors=TRUE;

			int width=curve[5].toInt(&ok);
			int cap=curve[6].toInt(&ok);
			QColor color=QColor(curve[7]);

			int aux=curve[8].toInt(&ok);
			if (aux==0) through=FALSE;

			aux=curve[9].toInt(&ok);
			if (aux==0) plus=FALSE;

			aux=curve[10].toInt(&ok);
			if (aux==0) minus=FALSE;

			w=app->table(yColName);
			WksheetWindow *errTable=app->table(errColName);
			ag->addErrorBars(w,xErrors,xColName,yColName,errTable, errColName,
							width,cap,color,through,minus,plus);
			}
		else if (s.left(6)=="scale\t")
			{
			QStringList scale=QStringList::split ("\t",s,TRUE);
			for (i=0;i<(int)scale.count();i++)
				scale[i]=scale[i+1];
			ag->definePlotLimits(scale);
			}
		else if (s.contains ("PlotTitle",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->setTitle(fList[1]);
			ag->setTitleColor(QColor(fList[2]));
			ag->setTitleAlignment(fList[3].toInt());
			}
		else if (s.contains ("TitleFont",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			QFont fnt=QFont (fList[1],fList[2].toInt(&ok),fList[3].toInt(&ok),fList[4].toInt(&ok));
			fnt.setUnderline(fList[5].toInt(&ok));
			fnt.setStrikeOut(fList[6].toInt(&ok));
			ag->setTitleFont(fnt);
			}
		else if (s.contains ("AxesTitles",TRUE)>0)
			{
				QStringList legend=QStringList::split ("\t",s,TRUE);
				for (i=0;i<4;i++)
					ag->setAxisTitle(i,legend[i+1]);
			}
		else if (s.contains ("AxesTitleColors",TRUE)>0)
			{
				QStringList colors=QStringList::split ("\t",s,FALSE);
				ag->setAxesTitleColor(colors);
			}
		else if (s.contains ("AxesTitleAlignment",TRUE)>0)
			{
				QStringList align=QStringList::split ("\t",s,FALSE);
				ag->setAxesTitlesAlignment(align);
			}
		else if (s.contains ("ScaleFont",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			QFont fnt=QFont (fList[1],fList[2].toInt(&ok),fList[3].toInt(&ok),fList[4].toInt(&ok));
			fnt.setUnderline(fList[5].toInt(&ok));
			fnt.setStrikeOut(fList[6].toInt(&ok));

			int axis=(fList[0].right(1)).toInt(&ok);
			ag->setAxisTitleFont(axis,fnt);
			}
		else if (s.contains ("AxisFont",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			QFont fnt=QFont (fList[1],fList[2].toInt(&ok),fList[3].toInt(&ok),fList[4].toInt(&ok));
			fnt.setUnderline(fList[5].toInt(&ok));
			fnt.setStrikeOut(fList[6].toInt(&ok));

			int axis=(fList[0].right(1)).toInt(&ok);
			ag->setAxisFont(axis,fnt);

			// we initiate slot connections here to preserve from updating transparancy several times
			g->connectLayer(ag);
			}
		else if (s.contains ("CanvasFrame",TRUE)>0)
			{
			QStringList list=QStringList::split ("\t",s,TRUE);
			ag->drawCanvasFrame(list);
			}
		else if (s.contains ("Legend",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->insertLegend(fList);
			}
		else if (s.contains ("textMarker",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->insertTextMarker(fList);
			}
		else if (s.contains ("lineMarker",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->insertLineMarker(fList);
			}
		else if (s.contains ("ImageMarker",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->insertImageMarker(fList);
			}
		else if (s.contains ("FitID",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->setFitID(fList[1].toInt());
			}
		}
f.close();
app->graphs=maxg;

if (wmax)
	{
	wmax->showMaximized();
	app->setListView(wmax->caption(),"Maximized");
	app->customToolBars(wmax);
	}
else if (aw)
	{
	aw->setActiveWindow();
	app->customToolBars(aw);
	}

QFileInfo fi(f);
QString fileName = fi.absFilePath();
if (app->recentProjects.contains(fileName)<=0)
	{
	if (app->recentProjects.count()>9)
		{
		QStringList::Iterator it=app->recentProjects.begin();
		app->recentProjects.remove(it);
		}

	app->recentProjects<<fileName;
	recentProjects=app->recentProjects;
	int count=app->recentProjects.count();
	app->recent->clear();
	for (i=0;i<count;i++)
		app->recent->insertItem("&"+QString::number(i+1)+" "+app->recentProjects[i]);
	}

app->functions == functions;
app->fitFunctions == fitFunctions;
app->saveProject();
QApplication::restoreOverrideCursor();
}

void ApplicationWindow::openQtiFile(const QString& fn)
{
QApplication::setOverrideCursor(waitCursor);
ApplicationWindow *app= new ApplicationWindow();
app->setCaption("QtiPlot - "+fn);
app->showMaximized();
app->projectname=fn;

QFile f(fn);
QTextStream t( &f );
f.open(IO_ReadOnly);

maxt=0;maxg=0;maxFit=0;
QString s;
QStringList list;
QProgressBar *pbar=new QProgressBar(0,"QtiPlot - Open file",WType_TopLevel|WStyle_StaysOnTop);
pbar->setIcon(QPixmap(logo_xpm));
pbar->setCenterIndicator ( TRUE );
pbar->setPercentageVisible(TRUE);

QFileInfo fi(fn);
QString baseName = fi.fileName();

s = t.readLine(); s = t.readLine();
if (s.contains("1",TRUE)>0)
	app->listViewOn=FALSE;
else
	app->listViewOn=TRUE;

s = t.readLine();
list=QStringList::split("\t",s,FALSE);
int i, aux=0,widgets=list[1].toInt();
pbar->setTotalSteps(widgets);

QString title="Opening file "+baseName+" window  1/"+QString::number(widgets)+"  ";
pbar->setCaption(title);
pbar->setGeometry(250,340,10*title.length(),30);
pbar->show();

//process tables information
while ( !t.eof() )
		{
		s = t.readLine();
		list.clear();
		if  (s.left(7)=="<table>")
			{
			aux++;
			title="Opening file "+baseName+" window "+QString::number(aux)+"/"+QString::number(widgets);
			pbar->setCaption(title);

			while ( s!="</table>" )
				{
				s=t.readLine();
				list<<s;
				}
			openTable(app,list);
			pbar->setProgress(aux);
			}
		}
f.close();
app->tables=maxt;
app->fitNumber=maxFit;

//process the rest
f.open(IO_ReadOnly);
QString layoutInfo;

MultiLayer *plot=0;
while ( !t.eof() )
		{
		s=t.readLine();
		if  (s.left(12)=="<multiLayer>")
				{//process multilayers information
				aux++;
			    title="Opening file "+baseName+" window "+QString::number(aux)+"/"+QString::number(widgets);
				pbar->setCaption(title);

				s=t.readLine();
				QStringList graph=QStringList::split ("\t",s,TRUE);
				QString caption=graph[0];
				plot=app->multilayerPlot(caption);
				plot->setCols(graph[1].toInt());
				plot->setRows(graph[2].toInt());
				//plot->setGraphDefaultSize(QSize(graph[3].toInt(),graph[4].toInt()));
				if (fileVersion < 6.3)
					app->setListViewDate(caption,graph[5]);
				else
					app->setListViewDate(caption,graph[3]);
				
				if (caption.contains ("graph",TRUE)>0)
						{
						int gr=caption.mid(5,caption.length()-5).toInt();
						if (gr>maxg) maxg=gr;
						}
				
				layoutInfo=t.readLine();//stores geometry information
				if (layoutInfo.contains ("minimized",TRUE)>0)
							{
							plot->show();
							QStringList geometry=QStringList::split ("\t",layoutInfo,FALSE );
							if ((int)geometry.count()>2)
								{
								int x=geometry[2].toInt();
								int y=geometry[3].toInt();
								int width=geometry[4].toInt();
								int height=geometry[5].toInt();
								plot->parentWidget()->setGeometry(x,y,width,height);
								}
							plot->showMinimized();
							app->setListView(caption,"Minimized");
							}
				else if (layoutInfo.contains ("maximized",TRUE)>0)
							{
							plot->show();
							app->setListView(caption,"Maximized");
							app->wmax=(QWidget*)plot;
							}
				else
							{
							QStringList geometry=QStringList::split ("\t",layoutInfo,FALSE );
							int x=geometry[1].toInt();
							int y=geometry[2].toInt();
							int width=geometry[3].toInt();
							int height=geometry[4].toInt();
														
							plot->showNormal();
							plot->parentWidget()->setGeometry(x,y,width,height);
						
							if (geometry[5]=="active")
									app->aw=(QWidget*)plot;
							else if (geometry[5]=="hidden")
									{
									app->hiddenWindows->append(plot);
									plot->hide();
									app->setListView(caption,"Hidden");
									}
							else
									app->setListView(caption,"Normal");
								}
				while ( s!="</multiLayer>" )
						{//open layers
						s=t.readLine();

						if (s.left(7)=="<graph>")
								{
								list.clear();
								while ( s!="</graph>" )
										{
										s=t.readLine();
										list<<s;
										}
								openGraph(app,plot,list);
								}
						}
				plot->updateTransparency();
				pbar->setProgress(aux);
				}
				else if  (s.left(13)=="<SurfacePlot>")
				{//process 3D plots information
				list.clear();
				aux++;
			    title="Opening file "+baseName+" window "+QString::number(aux)+"/"+QString::number(widgets);
				pbar->setCaption(title);
				while ( s!="</SurfacePlot>" )
						{
						s=t.readLine();
						list<<s;
						}
				openSurfacePlot(app,list);
				pbar->setProgress(aux);
				}
				else if  (s.left(5)=="<log>")
				{//process analysis information
					while ( s != "</log>" )
						{
						s=t.readLine();
						app->logInfo+=s+"\n";
						}
				}
			}
f.close();
			
app->logInfo=app->logInfo.remove ("</log>\n", FALSE);
app->graphs=maxg;
			
QFileInfo fi2(f);
QString fileName = fi2.absFilePath();
if (app->recentProjects.contains(fileName)<=0)
	{
	if (app->recentProjects.count()>9)
		{
		QStringList::Iterator it=app->recentProjects.begin();
		app->recentProjects.remove(it);
		}
	app->recentProjects<<fileName;
	recentProjects=app->recentProjects;
	int count=app->recentProjects.count();
	app->recent->clear();
	for (i=0;i<count;i++)
		app->recent->insertItem("&"+QString::number(i+1)+" "+app->recentProjects[i]);
	}
app->showList();	
			
if (app->wmax)
	{
	app->setListView(app->wmax->caption(),"Maximized");
	app->wmax->showMaximized();
	}
	
if (app->aw)
	app->aw->setFocus();
	
QApplication::restoreOverrideCursor();
app->saveProject();
delete pbar;
}

void ApplicationWindow::open()
{
QString fn = QFileDialog::getOpenFileName(QString::null, "QtiPlot project (*.qti);; All (*);;", 0, 0,
			"QtiPlot - Open Project", 0, TRUE);
if (!fn.isEmpty())
{
	if (projectname != "untitled")
		{
		QFileInfo fi(projectname);
		QString pn = fi.absFilePath();
		if (fn == pn)
			{
			QMessageBox::warning(this,"QtiPlot - File openning error",
				"The file: <p> <b>"+fn+"</b> <p> is the current file!");
			return;
			}
		}
		
	if (fn.contains(".qti",TRUE)>0)
		{
		QFileInfo f(fn);
		if (!f.exists ())
			{
			QMessageBox::critical(this,"QtiPlot - File openning error",
				"The file: <p> <b>"+fn+"</b> <p> doesn't exist!");
			return;
			}

		saveSettings();
		if (open (fn))
			this->close();
		}
	else
		{
		QMessageBox::critical(this,"QtiPlot - File openning error",
				"The file: <p> <b>"+fn+"</b> <p> is not a QtiPlot project file!");
		return;
		}
	}
}

void ApplicationWindow::openRecentProject(int index)
{
QString fn = recent->text(index);
int pos = fn.find(" ",0);
fn=fn.right(fn.length()-pos-1);

QFile f(fn);
if (!f.exists())
	{
	QMessageBox::critical(this,tr("QtiPlot - File Open Error"),
			   tr("The file:<p> <b>"+fn+"</b> <p>does not exist anymore!"
			   "<p>It will be removed from the list."));

	recentProjects.remove(fn);
	recent->clear();
	for (int i=0;i<(int)recentProjects.count();i++)
		recent->insertItem("&"+QString::number(i+1)+" "+recentProjects[i]);

	return;
	}

if (projectname != "untitled")
	{
	QFileInfo fi(projectname);
	QString pn = fi.absFilePath();
	if (fn == pn)
		{
		QMessageBox::warning(this,"QtiPlot - File openning error",
				"The file: <p> <b>"+fn+"</b> <p> is the current file!");
		return;
		}
	}

if ( !fn.isEmpty())
	{
	saveSettings();
	if (open (fn))
		this->close();
	}
}

void ApplicationWindow::readSettings()
{
helpFilePath="/usr/share/doc/qtiplot/help.html";
#ifdef Q_OS_WIN // Windows systems
	helpFilePath=qApp->applicationDirPath();
#endif
	
QSettings settings;
settings.setPath("Ion Vasilief", "QtiPlot");
settings.beginGroup("/QtiPlot");
helpFilePath=settings.readEntry("/helpFilePath", helpFilePath);
recentProjects=settings.readListEntry("/recentProjects");
functions=settings.readListEntry("/functions");
fitFunctions=settings.readListEntry("/fitFunctions");
surfaceFunc=settings.readListEntry("/surfaceFunctions");
xFunctions=settings.readListEntry("/xFunctions");
yFunctions=settings.readListEntry("/yFunctions");
rFunctions=settings.readListEntry("/rFunctions");
tetaFunctions=settings.readListEntry("/tetaFunctions");
QStringList tableColors=settings.readListEntry("/tableColors");
QStringList tableFonts=settings.readListEntry("/tableFonts");
titleOn=settings.readBoolEntry ("/titleOn", true, 0);
canvasFrameOn=settings.readBoolEntry ("/canvasFrameOn", false, 0);
ticksStyle=settings.readNumEntry ("/ticksStyle", Plot::Out, 0);
legendFrameStyle=settings.readNumEntry ("/legendFrameStyle", LegendMarker::Line, 0);
QStringList graphFonts=settings.readListEntry("/graphFonts");
confirmCloseTable=settings.readBoolEntry ("/confirmCloseTable", true, 0);
confirmClosePlot2D=settings.readBoolEntry ("/confirmClosePlot2D", true, 0);
confirmClosePlot3D=settings.readBoolEntry ("/confirmClosePlot3D", true, 0);
QStringList applicationFont=settings.readListEntry("/appFont");
appStyle=settings.readEntry("/appStyle", "Windows");
autoSave=settings.readBoolEntry("/autoSave",true);
autoSaveTime=settings.readNumEntry ("/autoSaveTime",15);
QStringList appColors=settings.readListEntry("/appColors");
settings.endGroup();

if (appColors.size() == 3)
	{
	workspaceColor=QColor(appColors[0]);
	panelsColor=QColor(appColors[1]);
	panelsTextColor=QColor(appColors[2]);
	}
	
if (tableColors.size () == 3)
	{
	tableBkgdColor=QColor(tableColors[0]);
	tableTextColor=QColor(tableColors[1]);
	tableHeaderColor=QColor(tableColors[2]);
	}

if (tableFonts.size() == 8)
	{
	tableTextFont=QFont (tableFonts[0],tableFonts[1].toInt(),tableFonts[2].toInt(),tableFonts[3].toInt());
	tableHeaderFont=QFont (tableFonts[4],tableFonts[5].toInt(),tableFonts[6].toInt(),tableFonts[7].toInt());
	}

if (graphFonts.size() == 16)
	{
	plotAxesFont=QFont (graphFonts[0],graphFonts[1].toInt(),graphFonts[2].toInt(),graphFonts[3].toInt());
	plotNumbersFont=QFont (graphFonts[4],graphFonts[5].toInt(),graphFonts[6].toInt(),graphFonts[7].toInt());
	plotLegendFont=QFont (graphFonts[8],graphFonts[9].toInt(),graphFonts[10].toInt(),graphFonts[11].toInt());
	plotTitleFont=QFont (graphFonts[12],graphFonts[13].toInt(),graphFonts[14].toInt(),graphFonts[15].toInt());
	}

if (applicationFont.size() == 4)	
	appFont=QFont (applicationFont[0],applicationFont[1].toInt(),applicationFont[2].toInt(),applicationFont[3].toInt());
}

void ApplicationWindow::saveSettings()
{
QStringList tableColors, appColors;
tableColors<<tableBkgdColor.name()<<tableTextColor.name()<<tableHeaderColor.name();
appColors<<workspaceColor.name()<<panelsColor.name()<<panelsTextColor.name();

QStringList tableFonts;
tableFonts<<tableTextFont.family();
tableFonts<<QString::number(tableTextFont.pointSize());
tableFonts<<QString::number(tableTextFont.weight());
tableFonts<<QString::number(tableTextFont.italic());
tableFonts<<tableHeaderFont.family();
tableFonts<<QString::number(tableHeaderFont.pointSize());
tableFonts<<QString::number(tableHeaderFont.weight());
tableFonts<<QString::number(tableHeaderFont.italic());

QStringList graphFonts;
graphFonts<<plotAxesFont.family();
graphFonts<<QString::number(plotAxesFont.pointSize());
graphFonts<<QString::number(plotAxesFont.weight());
graphFonts<<QString::number(plotAxesFont.italic());
graphFonts<<plotNumbersFont.family();
graphFonts<<QString::number(plotNumbersFont.pointSize());
graphFonts<<QString::number(plotNumbersFont.weight());
graphFonts<<QString::number(plotNumbersFont.italic());
graphFonts<<plotLegendFont.family();
graphFonts<<QString::number(plotLegendFont.pointSize());
graphFonts<<QString::number(plotLegendFont.weight());
graphFonts<<QString::number(plotLegendFont.italic());
graphFonts<<plotTitleFont.family();
graphFonts<<QString::number(plotTitleFont.pointSize());
graphFonts<<QString::number(plotTitleFont.weight());
graphFonts<<QString::number(plotTitleFont.italic());

QStringList applicationFont;
applicationFont<<appFont.family();
applicationFont<<QString::number(appFont.pointSize());
applicationFont<<QString::number(appFont.weight());
applicationFont<<QString::number(appFont.italic());

QSettings settings;
settings.setPath("Ion Vasilief", "QtiPlot");
settings.beginGroup("/QtiPlot");
settings.writeEntry("/helpFilePath", helpFilePath);
settings.writeEntry("/recentProjects", recentProjects);
settings.writeEntry("/functions", functions);
settings.writeEntry("/fitFunctions", fitFunctions);
settings.writeEntry("/surfaceFunctions", surfaceFunc);
settings.writeEntry("/xFunctions", xFunctions);
settings.writeEntry("/yFunctions", yFunctions);
settings.writeEntry("/rFunctions", rFunctions);
settings.writeEntry("/tetaFunctions", tetaFunctions);
settings.writeEntry("/tableColors", tableColors);
settings.writeEntry("/tableFonts", tableFonts);
settings.writeEntry("/titleOn", titleOn);
settings.writeEntry("/canvasFrameOn", canvasFrameOn);
settings.writeEntry("/ticksStyle", ticksStyle);
settings.writeEntry("/legendFrameStyle", legendFrameStyle);
settings.writeEntry("/graphFonts", graphFonts);
settings.writeEntry("/confirmCloseTable", confirmCloseTable);
settings.writeEntry("/confirmClosePlot2D", confirmClosePlot2D);
settings.writeEntry("/confirmClosePlot3D", confirmClosePlot3D);
settings.writeEntry("/appFont", applicationFont);
settings.writeEntry("/appStyle", appStyle);
settings.writeEntry("/autoSave", autoSave);
settings.writeEntry("/autoSaveTime", autoSaveTime);
settings.writeEntry("/appColors", appColors);
settings.endGroup();
}

void ApplicationWindow::exportGraph()
{
QWidget *w=ws->activeWindow();
if (!w)
	return;

if(plotWindows.contains(w->caption())>0)
	((MultiLayer*)w)->exportPage();

else if(plot3DWindows.contains(w->caption())>0)
	((Graph3D*)w)->saveImage();
}

void ApplicationWindow::saveProject()
{
if (projectname=="untitled")
	{
	saveProjectAs();
	return;
	}

QFile f( projectname );
if ( !f.open( IO_WriteOnly ) )
	{
	QMessageBox::about(0, tr("QtiPlot - File Save Error"),
	tr("The file: <br><h4>"+projectname+ "</h4> is opened in read-only mode").arg(projectname));
    return;
	}

QApplication::setOverrideCursor(waitCursor);

QWidgetList windows = ws->windowList(QWorkspace::StackingOrder);
int i,j,c=(int)windows.count();
QPoint p;
WksheetWindow *w=0;
MultiLayer *g=0;
Graph3D *plot=0;

QString text="QtiPlot 0.6.4 project file\n";
text+="<listViewOn>\t"+QString::number(listViewOn)+"\n";
text+="<widgets>\t"+QString::number(plot3DWindows.count()+plotWindows.count()+tableWindows.count())+"\n";
for (i=0;i<c;i++)
	{
	QString s="";
	QString caption=windows.at(i)->caption();
	plot=(Graph3D*)windows.at(i);
	if (plot3DWindows.contains(caption)>0 && plot->plotStyle() != Qwt3D::NOPLOT)
		{
		QString windowGeometry="sgeometry\t";

		if (minimized(windows.at(i)))
			windowGeometry+="minimized\n";
		else if (maximized(windows.at(i)))
			windowGeometry+="maximized\n";
		else
			{
			p =plot->parentWidget()->pos();// store position
			windowGeometry+=QString::number(p.x())+"\t";
			windowGeometry+=QString::number(p.y())+"\t";
			windowGeometry+=QString::number(plot->parentWidget()->frameGeometry().width())+"\t";
			windowGeometry+=QString::number(plot->parentWidget()->frameGeometry().height())+"\t";

			if (windows.at(i)==ws->activeWindow() && !hidden(windows.at(i)))
				windowGeometry+="active\n";
			else if(hidden(windows.at(i)))
				windowGeometry+="hidden\n";
			else
				windowGeometry+="\n";
			}
		text+=plot->saveToString(windowGeometry);
		}

	else if (plotWindows.contains(caption)>0)
		{
		g=(MultiLayer*)windows.at(i);
		s="<multiLayer>\n";
		s+=caption+"\t";
		s+=QString::number(g->getCols())+"\t";
		s+=QString::number(g->getRows())+"\t";
		s+=listViewDate(caption)+"\n";

		s+="mlgeometry\t";
		if (minimized(windows.at(i)))
				{
				s+="minimized\t";
				p =g->parentWidget()->pos();// store position
				s+=QString::number(p.x())+"\t";
				s+=QString::number(p.y())+"\t";
				s+=QString::number(g->parentWidget()->frameGeometry().width())+"\t";
				s+=QString::number(g->parentWidget()->frameGeometry().height())+"\n";
				}
		else if (maximized(windows.at(i)))
				s+="maximized\n";
		else
				{
				p =g->parentWidget()->pos();// store position
				s+=QString::number(p.x())+"\t";
				s+=QString::number(p.y())+"\t";
				s+=QString::number(g->parentWidget()->frameGeometry().width())+"\t";
				s+=QString::number(g->parentWidget()->frameGeometry().height())+"\t";
			
				if (windows.at(i)==ws->activeWindow() && !hidden(windows.at(i)))
					s+="active\n";
				else if(hidden(windows.at(i)))
					s+="hidden\n";
				else
					s+="\n";
				}

		QWidgetList *graphsList=g->graphPtrs();
		for (int j=0;j<(int)graphsList->count();j++)
				{
				Graph* ag=(Graph*)graphsList->at(j);
				s+=ag->saveToString();
				}
			text+=s;
			text+="</multiLayer>\n";
			setListViewSize(caption,QString::number(s.length()*sizeof(QChar)/1024)+" kB");
		}
	else if (tableWindows.contains(caption)>0)
		{
		w=(WksheetWindow*)windows.at(i);
		s="<table>\n";
		s+=caption+"\t";
		s+=QString::number(w->tableRows())+"\t";
		s+=QString::number(w->tableCols())+"\t";
		s+=listViewDate(w->caption())+"\t";
		s+=QString::number(w->colX())+"\t";
		s+=":\t";
		s+=QString::number(w->colY())+"\n";	//starting from 0.5.0 version
		s+="tgeometry\t";

			if (minimized(windows.at(i)))
				s+="minimized\n";
			else if (maximized(windows.at(i)))
				s+="maximized\n";
			else
			{
			p =w->parentWidget()->pos();// store position
			s+=QString::number(p.x())+"\t";
			s+=QString::number(p.y())+"\t";
			s+=QString::number(w->parentWidget()->frameGeometry().width())+"\t";
			s+=QString::number(w->parentWidget()->frameGeometry().height())+"\t";

			if (windows.at(i)==ws->activeWindow() && !hidden(windows.at(i)))
				s+="active\n";
			else if(hidden(windows.at(i)))
				s+="hidden\n";
			else
				s+="\n";
			}

			s+="header\t";
			QStringList header=w->colNames();
			int col=w->tableCols();
			for (j=0;j<col;j++)
					s+=header[j]+"\t";
			s+="\n";
			s+=w->saveColumnWidths();
			s+=w->saveCommandes();
			s+=w->saveText();
			s+="</table>\n";
			text+=s;
			w->setSpecifications(s);
			setListViewSize(caption,QString::number(s.length()*sizeof(QString)/1024)+" kB");
		}
	}

text+="<log>\n"+logInfo+"</log>";

QTextStream t( &f );
t << text;
f.close();
setCaption("QtiPlot - "+projectname);

saved=TRUE;
fileSave->setDisabled(TRUE);
btnUndo->setDisabled(TRUE);
btnRedo->setDisabled(TRUE);
edit->setItemEnabled (undoID,FALSE);
edit->setItemEnabled (redoID,FALSE);

if (autoSave)
	{	
	if (savingTimerId)
		killTimer(savingTimerId);
	
	savingTimerId=startTimer(autoSaveTime*60000);
	}
else
	savingTimerId=0;

QApplication::restoreOverrideCursor();
}

void ApplicationWindow::saveProjectAs()
{
QString fn = QFileDialog::getSaveFileName("",".qti", this );
if ( !fn.isEmpty() )
	{
	QFileInfo fi(fn);
	QString baseName = fi.fileName();
	
	QString name=fn;
	if (baseName.contains(".")==0)
		name.append(".qti");	

	if ( QFile::exists(name) &&
        QMessageBox::question(
            this,
            tr("QtiPlot -- Overwrite File? "),
            tr("A file called: <p><b>%1</b><p>already exists.\n"
                "Do you want to overwrite it?")
                .arg(name),
            tr("&Yes"), tr("&No"),
            QString::null, 0, 1 ) )
        return ;
	else
		{
		projectname=name;
		saveProject();

		QFile f(projectname);
		QFileInfo fi(f);
		recentProjects<<(fi.absFilePath());

		recent->clear();
		for (int i=0;i<(int)recentProjects.count();i++)

			recent->insertItem("&"+QString::number(i+1)+" "+recentProjects[i]);
		}
    }
}

void ApplicationWindow::rename()
{
QWidget* m = (QWidget*)ws->activeWindow();
QString caption=m->caption();

bool ok;
QString text = QInputDialog::getText( tr("Rename Window"), tr("Please choose a name" ),QLineEdit::Normal,caption, &ok, this );

if (! ok)
	return;

while(plotWindows.contains(text)>0 || tableWindows.contains(text)>0 ||
	  plot3DWindows.contains(text)>0 || text.isEmpty())
	{
	if (text.isEmpty())
		QMessageBox::critical(this,"QtiPlot - Error",
			   "The name you chose is not valid!\nPlease choose another name!");
	else
		QMessageBox::critical(this,"QtiPlot - Error",
				"Name already exists!\n Please choose another name!");

	text = QInputDialog::getText( tr("Rename Window"), tr("Please choose a name" ),QLineEdit::Normal,caption, &ok, this );
	if (! ok)
		return;
	}

text=text.replace("_","-");

int id;
if (plotWindows.contains(caption)>0)
	{
	id=plotWindows.findIndex(caption);
	plotWindows[id]=text;
	}
else if (plot3DWindows.contains(caption)>0)
	{
	id=plot3DWindows.findIndex(caption);
	plot3DWindows[id]=text;
	}
else if (tableWindows.contains(caption)>0)
	{
	QStringList labels=((WksheetWindow *)m)->colNames();
	if (labels.contains(text)>0)
			{
			QMessageBox::critical(0,tr("QtiPlot - Error"),
			"The table caption must be different from the names of its columns!<p>Please choose another name!");
			return;
			}

	id=tableWindows.findIndex(caption);
	tableWindows[id]=text;
	updateCurvesDialogs(caption,text);

	edit->setItemEnabled (redoID,FALSE);
	btnRedo->setEnabled(FALSE);

	edit->setItemEnabled (undoID,TRUE);
	btnUndo->setEnabled(TRUE);
	}

m->setCaption(text);
lastModified=m;
renameListViewItem(caption,text);
emit modified();
}

void ApplicationWindow::renameWindow()
{
int id;
QListViewItem *lbi=lv->currentItem();
QWidget *w=window(lbi);

QString caption=w->caption();

bool ok;
QString text = QInputDialog::getText( tr("Rename Window"), tr("Please choose a name" ),QLineEdit::Normal,caption, &ok, this );

if (! ok)
	return;

while(plotWindows.contains(text)>0 || tableWindows.contains(text)>0 ||
	  plot3DWindows.contains(text)>0 || text.isEmpty())
	{
	if (text.isEmpty())
		QMessageBox::critical(this,"QtiPlot - Error",
			   "The name you chose is not valid!\nPlease choose another name!");
	else
		QMessageBox::critical(this,"QtiPlot - Error",
				"Name already exists!\n Please choose another name!");

	text = QInputDialog::getText( tr("Rename Window"), tr("Please choose a name" ),QLineEdit::Normal,caption, &ok, this );
	if (! ok)
		return;
	}

text=text.replace("_","-");

if (plotWindows.contains(caption)>0)
	{
	id=plotWindows.findIndex(caption);
	plotWindows[id]=text;
	}
else if (plot3DWindows.contains(caption)>0)
	{
	id=plot3DWindows.findIndex(caption);
	plot3DWindows[id]=text;
	}
else if (tableWindows.contains(caption)>0)
	{
	QStringList labels=((WksheetWindow *)w)->colNames();
	if (labels.contains(text)>0)
			{
			QMessageBox::critical(this,tr("QtiPlot - Error"),
			"The table caption must be different from the names of its columns!<p>Please choose another name!");
			return;
			}

	id=tableWindows.findIndex(caption);
	tableWindows[id]=text;
	updateCurvesDialogs(caption,text);
	edit->setItemEnabled (redoID,FALSE);
	btnRedo->setEnabled(FALSE);

	edit->setItemEnabled (undoID,TRUE);
	btnUndo->setEnabled(TRUE);
	}

w->setCaption(text);
lastModified=w;
lbi->setText(0,text);
emit modified();
}

QStringList ApplicationWindow::columnsList()
{
QWidgetList windows = ws->windowList();
int i,j,n;
int c=windows.count();
QStringList list,header;
WksheetWindow* w;

for (i=0;i<c;i++)
	{
	w=(WksheetWindow*)windows.at(i);
	if (w && tableWindows.contains(w->caption())>=1)
		{
		header=w->colNames();
		n=w->tableCols();
		for (j=0;j<n;j++)
			list<<w->caption()+"_"+header[j];
		}
	}
return list;
}

QStringList ApplicationWindow::columnsListXY()
{
QWidgetList windows = ws->windowList();
int i,j,n;
int c=windows.count();
QStringList list,header;
WksheetWindow* w;

for (i=0;i<c;i++)
	{
	w=(WksheetWindow*)windows.at(i);
	if (w && tableWindows.contains(w->caption())>=1)
		{
		header=w->colNames();
		n=w->tableCols();
		for (j=0;j<n;j++)
			{
			if (j==w->colX())
				list<<w->caption()+"_"+header[j]+"(X)";
			else
				list<<w->caption()+"_"+header[j]+"(Y)";
			}
		}
	}
return list;
}

void ApplicationWindow::showCurvesDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
		return;

Graph* g = (Graph*)plot->activeGraph();
if (!g)
		return;

if (g->isPiePlot())
	{
	QMessageBox::warning(this,"QtiPlot - Warning",
				"This functionality is not available for pie plots!");
	}
else
	{
	curvesDialog* crvDialog=new curvesDialog(this,"curves",TRUE,WStyle_StaysOnTop|WStyle_Tool|WDestructiveClose);

	connect (crvDialog,SIGNAL(curveToRemove(int)),g,SLOT(removeCurve(int)));
	connect (crvDialog,SIGNAL(curveToPlot(const QString&)),g,SLOT(addCurve(const QString&)));

	QStringList list=columnsList();
	crvDialog->insertCurvesToDialog(list);
	crvDialog->insertPlottedCurves(g->plotCurvesList());
	crvDialog->showNormal();
	crvDialog->setActiveWindow();
	}
}

void ApplicationWindow::showTitleDialog()
{
QWidget *w = ws->activeWindow();
if (!w)
	return;
QString caption=w->caption();

if (plotWindows.contains(caption)>0)
{
MultiLayer* plot = (MultiLayer*)w;
if (!plot)
	return;
Graph* g = (Graph*)plot->activeGraph();
if (g)
	{
	textDialog* td= new textDialog(this,"axesDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (td,SIGNAL(changeFont(const QFont &)),g,SLOT(setTitleFont(const QFont &)));
	connect (td,SIGNAL(changeText(const QString &)),g,SLOT(setTitle(const QString &)));
	connect (td,SIGNAL(changeColor(const QColor &)),g,SLOT(setTitleColor(const QColor &)));
	connect (td,SIGNAL(changeAlignment(int)),g,SLOT(setTitleAlignment(int)));

	td->setText(g->title());
	td->setFont(g->titleFont());
	td->setColor(g->titleColor());
	td->setAlignment(g->titleAlignment());
	td->showNormal();
	td->setActiveWindow();
	g->setTitleSelected(FALSE);
	}
}
else if (plot3DWindows.contains(caption)>0)
	{
	plot3DDialog* pd=showPlot3dDialog();
	if (pd)
		pd->showTitleTab();
	}
}

void ApplicationWindow::showXAxisTitleDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (g)
	{
	textDialog* td= new textDialog(this,"axesDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (td,SIGNAL(changeFont(const QFont &)),g,SLOT(setXAxisTitleFont(const QFont &)));
	connect (td,SIGNAL(changeText(const QString &)),g,SLOT(setXAxisTitle(const QString &)));
	connect (td,SIGNAL(changeColor(const QColor &)),g,SLOT(setXAxisTitleColor(const QColor &)));
	connect (td,SIGNAL(changeAlignment(int)),g,SLOT(setXAxisTitleAlignment(int)));

	QStringList t=g->scalesTitles();
	td->setText(t[0]);
	td->setFont(g->axisTitleFont(2));
	td->setColor(g->axisTitleColor(2));
	td->setAlignment(g->axisTitleAlignment(2));
	td->setCaption("QtiPlot - X Axis Title");
	td->showNormal();
	td->setActiveWindow();
	}
}

void ApplicationWindow::showYAxisTitleDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (g)
	{
	textDialog* td= new textDialog(this,"axesDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (td,SIGNAL(changeFont(const QFont &)),g,SLOT(setYAxisTitleFont(const QFont &)));
	connect (td,SIGNAL(changeText(const QString &)),g,SLOT(setYAxisTitle(const QString &)));
	connect (td,SIGNAL(changeColor(const QColor &)),g,SLOT(setYAxisTitleColor(const QColor &)));
	connect (td,SIGNAL(changeAlignment(int)),g,SLOT(setYAxisTitleAlignment(int)));

	QStringList t=g->scalesTitles();
	td->setText(t[1]);
	td->setFont(g->axisTitleFont(0));
	td->setColor(g->axisTitleColor(0));
	td->setAlignment(g->axisTitleAlignment(0));
	td->setCaption("QtiPlot - Y Axis Title");
	td->showNormal();
	td->setActiveWindow();
	}
}

void ApplicationWindow::showRightAxisTitleDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (g)
	{
	textDialog* td= new textDialog(this,"axesDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (td,SIGNAL(changeFont(const QFont &)),g,SLOT(setRightAxisTitleFont(const QFont &)));
	connect (td,SIGNAL(changeText(const QString &)),g,SLOT(setRightAxisTitle(const QString &)));
	connect (td,SIGNAL(changeColor(const QColor &)),g,SLOT(setRightAxisTitleColor(const QColor &)));
	connect (td,SIGNAL(changeAlignment(int)),g,SLOT(setRightAxisTitleAlignment(int)));

	QStringList t=g->scalesTitles();
	td->setText(t[3]);
	td->setFont(g->axisTitleFont(1));
	td->setColor(g->axisTitleColor(1));
	td->setAlignment(g->axisTitleAlignment(1));
	td->setCaption("QtiPlot - Right Axis Title");
	td->showNormal();
	td->setActiveWindow();
	}
}

void ApplicationWindow::showTopAxisTitleDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (g)
	{
	textDialog* td= new textDialog(this,"axesDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (td,SIGNAL(changeFont(const QFont &)),g,SLOT(setTopAxisTitleFont(const QFont &)));
	connect (td,SIGNAL(changeText(const QString &)),g,SLOT(setTopAxisTitle(const QString &)));
	connect (td,SIGNAL(changeColor(const QColor &)),g,SLOT(setTopAxisTitleColor(const QColor &)));
	connect (td,SIGNAL(changeAlignment(int)),g,SLOT(setTopAxisTitleAlignment(int)));

	QStringList t=g->scalesTitles();
	td->setText(t[2]);
	td->setFont(g->axisTitleFont(3));
	td->setColor(g->axisTitleColor(3));
	td->setAlignment(g->axisTitleAlignment(3));
	td->setCaption("QtiPLot - Top Axis Title");
	td->showNormal();
	td->setActiveWindow();
	}
}

void ApplicationWindow::showExportASCIIDialog()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();

if ( w && tableWindows.contains(w->caption())>0)
	{
	exportDialog* ed= new exportDialog(this,"exportDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (ed,SIGNAL(options(const QString&,bool,bool)),w,SLOT(exportAscii(const QString&,bool,bool)));
	ed->showNormal();
	ed->setActiveWindow();
	}
}

void ApplicationWindow::showRowsDialog()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if ( w && tableWindows.contains(w->caption())>0)
	{
	rowsDialog* rd= new rowsDialog(this,"rowsDialog",TRUE,WDestructiveClose);
	connect(rd, SIGNAL(updateRows(int)),w, SLOT(updateNumRows(int)));
	connect(rd, SIGNAL(updateCols(int)),w, SLOT(updateNumCols(int)));

	rd->init(w->tableRows(), w->tableCols(), w->colName(w->colX()), w->colName(w->colY()));
	rd->showNormal();
	rd->setActiveWindow();
	}
}

void ApplicationWindow::showColumnValuesDialog()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();

if ( w && tableWindows.contains(w->caption())>0)
	{
	if (int(w->selectedColumns().count())>0)
		{
		setColValuesDialog* vd= new setColValuesDialog(this,"valuesDialog",TRUE,WDestructiveClose);
		connect(vd, SIGNAL(setValues(const QString&, const QString&, const QStringList&, const QStringList&, int, int)),
			w, SLOT(setColValues(const QString&, const QString&, const QStringList&, const QStringList&, int, int)));

		QStringList cols=w->colNames();
		for (int i=0;i<w->tableCols();i++)
			vd->insertColItem("col("+cols[i]+")",i);

		vd->setTable(w);
		vd->setText(w->selectedColComand());
		vd->setEndRow(w->tableRows());
		vd->showNormal();
		vd->setActiveWindow();
		}
	else
		QMessageBox::warning(this, "QtiPlot - Column selection error","Please select a column first!");
	}
}

void ApplicationWindow::showColStatistics()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if ( w && tableWindows.contains(w->caption())>0)
	{
	if (int(w->selectedColumns().count())>0)
		w->showColStatistics();
	else
		QMessageBox::warning(this, "QtiPlot - Column selection error","Please select a column first!");
	}
}

void ApplicationWindow::showRowStatistics()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if ( w && tableWindows.contains(w->caption())>0)
	{
	if (w->selectedRows()>0)
		w->showRowStatistics();
	else
		QMessageBox::warning(this ,"QtiPlot - Row selection error","Please select a row first!");
	}
}

void ApplicationWindow::showColMenu(int c)
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();
if ( w && tableWindows.contains(w->caption())>0)
	{
	QPopupMenu Legend(this);
	QPopupMenu plot(this);
	QPopupMenu specialPlot(this);
	QPopupMenu fill(this);
	QPopupMenu sorting(this);
	QPopupMenu colType(this);
	colType.setCheckable ( true );
	QPopupMenu panels(this);
	QPopupMenu stat(this);
	QPopupMenu norm(this);
		
	if ((int)w->selectedColumns().count()==1)
		{
		w->setSelectedCol(c);
		plot.insertItem(QPixmap(lPlot_xpm),"&Line",w, SLOT(plotL()));
		plot.insertItem(QPixmap(pPlot_xpm),"&Scatter",w, SLOT(plotP()));
		plot.insertItem(QPixmap(lpPlot_xpm),"Line + s&ymbol",w,SLOT(plotLP()));

		specialPlot.insertItem(QPixmap(dropLines_xpm),"Vertical &drop lines",w,SLOT(plotVerticalDropLines()));
		specialPlot.insertItem(QPixmap(spline_xpm),"&Spline",w,SLOT(plotSpline()));
		specialPlot.insertItem(QPixmap(steps_xpm),"&Vertical Steps",w,SLOT(plotSteps()));
		plot.insertItem("Special Line/Symb&ol",&specialPlot);
		plot.insertSeparator();

		plot.insertItem(QPixmap(vertBars_xpm),"&Columns",w,SLOT(plotVB()));
		plot.insertItem(QPixmap(hBars_xpm),"&Rows",w,SLOT(plotHB()));
		plot.insertItem(QPixmap(area_xpm),"&Area",w,SLOT(plotArea()));

		plot.insertItem(QPixmap(pie_xpm),"&Pie",w,SLOT(plotPie()));
		plot.insertSeparator();

		plot.insertItem(QPixmap(ribbon_xpm),"3D Ribbo&n",w,SLOT(plot3DRibbon()));
		plot.insertItem(QPixmap(bars_xpm),"3D &Bars",w,SLOT(plot3DBars()));
		plot.insertItem(QPixmap(scatter_xpm),"3&D Scatter",w,SLOT(plot3DScatter()));
		plot.insertItem(QPixmap(trajectory_xpm),"3D &Trajectory",w,SLOT(plot3DTrajectory()));

		plot.insertSeparator();

		stat.insertItem(QPixmap(histogram_xpm),"&Histogram",w,SLOT(plotHistogram()));
		stat.insertItem(QPixmap(stacked_hist_xpm),"&Stacked Histograms",this,SLOT(plotStackedHistograms()));
		plot.insertItem("Statistical &Graphs",&stat);

		Legend.insertItem("&Plot",&plot);
		Legend.insertSeparator();

		Legend.insertItem(QPixmap(cut_xpm),"Cu&t", w, SLOT(cutSelection()));
		Legend.insertItem(QPixmap(copy_xpm),"&Copy", w, SLOT(copySelection()));
		Legend.insertItem(QPixmap(paste_xpm),"Past&e", w, SLOT(pasteSelection()));
		Legend.insertSeparator();

		int xColID=colType.insertItem("X",w,SLOT(setXCol()));
		int yColID=colType.insertItem("Y",w,SLOT(setYCol()));

		if (c==w->colX())
			colType.setItemChecked (xColID,true);
		else if (c==w->colY())
			colType.setItemChecked (yColID,true);

		Legend.insertItem("Set as...",&colType);
		Legend.insertSeparator();

		Legend.insertItem("Set column &values",w,SIGNAL(colValuesDialog()));
		fill.insertItem(QPixmap(rowNumbers_xpm),"Ro&w Numbers",w, SLOT(setAscValues()));
		fill.insertItem(QPixmap(randomNumbers_xpm),"&Random values",w, SLOT(setRandomValues()));
		Legend.insertItem("&Fill column with",&fill);
		
		norm.insertItem("&Column", w, SLOT(normalizeSelection()));
		norm.insertItem("&Table", w, SLOT(normalizeTable()));
		Legend.insertItem("&Normalize", &norm);

		Legend.insertSeparator();
		Legend.insertItem(QPixmap(col_stat_xpm),"&Statistics on Column",w, SLOT(showColStatistics()));

		Legend.insertSeparator();

		Legend.insertItem("Clea&r", w, SLOT(clearCol()));
		Legend.insertItem("&Delete", w, SLOT(removeCol()));
		Legend.insertItem("&Insert", w, SLOT(insertCol()));
		Legend.insertItem("&Add column",w, SLOT(addCol()));
		Legend.insertSeparator();

		sorting.insertItem("&Ascending",w, SLOT(sortColAsc()));
		sorting.insertItem("&Descending",w, SLOT(sortColDesc()));
		Legend.insertItem("Sort Colu&mn",&sorting);

		Legend.insertItem("Sort Ta&ble",w, SLOT(sortTableDialog()));

		Legend.insertSeparator();
		Legend.insertItem("Pr&operties",w, SIGNAL(optionsDialog()));
		}
	else if ((int)w->selectedColumns().count()>1)
		{
		plot.insertItem(QPixmap(lPlot_xpm),"&Line",w, SLOT(plotL()));
		plot.insertItem(QPixmap(pPlot_xpm),"&Scatter",w, SLOT(plotP()));
		plot.insertItem(QPixmap(lpPlot_xpm),"Line + s&ymbol",w,SLOT(plotLP()));

		specialPlot.insertItem(QPixmap(dropLines_xpm),"Vertical &drop lines",w,SLOT(plotVerticalDropLines()));
		specialPlot.insertItem(QPixmap(spline_xpm),"&Spline",w,SLOT(plotSpline()));
		specialPlot.insertItem(QPixmap(steps_xpm),"&Vertical Steps",w,SLOT(plotSteps()));
		plot.insertItem("Special Line/Symb&ol",&specialPlot);
		plot.insertSeparator();

		plot.insertItem(QPixmap(vertBars_xpm),"&Columns",w,SLOT(plotVB()));
		plot.insertItem(QPixmap(hBars_xpm),"&Rows",w,SLOT(plotHB()));
		plot.insertItem(QPixmap(area_xpm),"&Area",w,SLOT(plotArea()));
		plot.insertItem(QPixmap(vectXYXY_xpm),"Vectors &XYXY", w, SLOT(plotVectXYXY()));
		plot.insertSeparator();

		stat.insertItem(QPixmap(histogram_xpm),"&Histogram",w,SLOT(plotHistogram()));
		stat.insertItem(QPixmap(stacked_hist_xpm),"&Stacked Histograms",this,SLOT(plotStackedHistograms()));
		plot.insertItem("Statistical &Graphs",&stat);

		panels.insertItem(QPixmap(panel_v2_xpm),"&Vertical 2 Layers",this, SLOT(plot2VerticalLayers()));
		panels.insertItem(QPixmap(panel_h2_xpm),"&Horizontal 2 Layers",this, SLOT(plot2HorizontalLayers()));
		panels.insertItem(QPixmap(panel_4_xpm),"&4 Layers",this, SLOT(plot4Layers()));
		panels.insertItem(QPixmap(stacked_xpm),"&Stacked Layers",this, SLOT(plotStackedLayers()));
		plot.insertItem("Pa&nel",&panels);
		
		Legend.insertItem("&Plot",&plot);
		Legend.insertSeparator();
		Legend.insertItem(QPixmap(cut_xpm),"Cu&t", w, SLOT(cutSelection()));
		Legend.insertItem(QPixmap(copy_xpm),"&Copy", w, SLOT(copySelection()));
		Legend.insertItem(QPixmap(paste_xpm),"Past&e", w, SLOT(pasteSelection()));
		Legend.insertSeparator();

		Legend.insertItem(QPixmap(erase_xpm),"Clea&r", w, SLOT(clearSelection()));
		Legend.insertItem("&Delete", w, SLOT(removeCol()));
		Legend.insertSeparator();
		Legend.insertItem("&Insert", w, SLOT(insertCol()));
		Legend.insertItem("&Add column",w, SLOT(addCol()));
		Legend.insertSeparator();

		fill.insertItem(QPixmap(rowNumbers_xpm),"Ro&w Numbers",w, SLOT(setAscValues()));
		fill.insertItem(QPixmap(randomNumbers_xpm),"&Random values",w, SLOT(setRandomValues()));
		Legend.insertItem("&Fill columns with",&fill);

		Legend.insertItem("&Normalize", w, SLOT(normalizeSelection()));
		
		Legend.insertSeparator();
		Legend.insertItem("&Sort",w, SLOT(sortManyColDialog()));
		Legend.insertSeparator();
		Legend.insertItem(QPixmap(col_stat_xpm),"Statistics on Colu&mns",w, SLOT(showColStatistics()));
		}

	QPoint posMouse=QCursor::pos();
    Legend.exec(posMouse);
	}
}

void ApplicationWindow::plot2VerticalLayers()
{
	plotMultiLayers(1,2, Graph::Line);
}

void ApplicationWindow::plot2HorizontalLayers()
{
plotMultiLayers(2,1, Graph::Line);
}

void ApplicationWindow::plot4Layers()
{
plotMultiLayers(2,2, Graph::Line);
}

void ApplicationWindow::plotStackedLayers()
{
plotMultiLayers(1,-1, Graph::Line);
}

void ApplicationWindow::plotStackedHistograms()
{
plotMultiLayers(1,-1, Graph::Histogram);
}

void ApplicationWindow::showRowMenu(int r)
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();

if ( w && tableWindows.contains(w->caption())>0)
	{
	w->setSelectedRow(r);

	QPopupMenu select(this);
	select.insertItem(QPixmap(cut_xpm),"Cu&t", w, SLOT(cutSelection()));
	select.insertItem(QPixmap(copy_xpm),"&Copy", w, SLOT(copySelection()));
	select.insertItem(QPixmap(paste_xpm),"&Paste", w, SLOT(pasteSelection()));
	select.insertSeparator();
	select.insertItem(QPixmap(erase_xpm),"Clea&r", w, SLOT(clearSelection()));
	select.insertItem("&Delete", w, SLOT(deleteRow()));
	select.insertItem("&Insert", w, SLOT(insertRow()));
	select.insertSeparator();
	select.insertItem(QPixmap(stat_rows_xpm),"&Statistics on Rows",w, SLOT(showRowStatistics()));

	QPoint posMouse=QCursor::pos();
    select.exec(posMouse);
	}
}

void ApplicationWindow::showTableOptionsDialog()
{
WksheetWindow* w = (WksheetWindow*)ws->activeWindow();

if ( w && tableWindows.contains(w->caption())>0)
	{
	if(	int(w->selectedColumns().count())>0)
		{
	w->setMatrix();
	tableDialog* td= new tableDialog(this,"optionsDialog",TRUE,WDestructiveClose);
	connect(td, SIGNAL(nameChanged(const QString&)),w, SLOT(changeColName(const QString&)));
	connect(td, SIGNAL(enumRightCols(bool)),w, SLOT(enumerateRightCols(bool)));
	connect(td, SIGNAL(changeWidth(const QString&,bool)),w, SLOT(changeColWidth(const QString&,bool)));
	connect(td, SIGNAL(changeFormat(int,int,const QString&,bool)),w,SLOT(setNumericFormat(int,int,const QString&,bool)));
	connect(td, SIGNAL(closedTableDialog()),w,SLOT(freeMatrix()));

	int sc=w->selectedColumn();
	if (w->colX()== sc)
		td->setPlotDesignation(0);
	else if (w->colY()== sc)
		td->setPlotDesignation(1);
	else
		td->setPlotDesignation(2);

	td->setColName(w->selectedColName());
	td->setColWidth(w->selectedColWidth());
	td->showNormal();
	}
	else
		QMessageBox::warning(this, "QtiPlot","Please select a column first!");
	}
}

void ApplicationWindow::showAxis(int axis, int type, const QString& labelsColName, bool axisOn, int ticksType,
								 bool labelsOn, const QColor& c, int format, int prec, int rotation)
{
WksheetWindow *w = table(labelsColName);
if (!w)
	return;

activeGraph->showAxis(axis,type, labelsColName, w, axisOn, ticksType, labelsOn, 
					  c, format, prec, rotation);

}

void ApplicationWindow::showXAxesDialog()
{
QWidget *w = ws->activeWindow();
if (!w)
	return;
QString caption=w->caption();

if (plotWindows.contains(caption)>0)
{
MultiLayer* plot = (MultiLayer*)w;
if (!plot)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (!g)
	return;

if (!g->isPiePlot())
	{
	activeGraph = g;

genDialog* axesDialog= new genDialog(this,"axesDialog",TRUE,WStyle_Tool|WDestructiveClose);
connect (axesDialog,SIGNAL(updatePlotLimits(int,const QStringList&)),g,SLOT(setPlotLimits(int,const QStringList&)));
connect (axesDialog,SIGNAL(updateAxisTitle(int,const QString&)),g,SLOT(setAxisTitle(int,const QString&)));
connect (axesDialog,SIGNAL(changeGridOptions(const gridOptions &)),g,SLOT(setGridOptions(const gridOptions &)));
connect (axesDialog,SIGNAL(changeAxisFont(int, const QFont &)),g,SLOT(setAxisFont(int,const QFont &)));
connect (axesDialog,SIGNAL(showAxis(int, int, const QString&, bool,int, bool,const QColor&, int, int, int)),
							this,SLOT(showAxis(int,int, const QString&, bool, int,bool,const QColor&, int, int, int)));
connect (axesDialog,SIGNAL(updateSecondaryLimits(int,const QStringList&)),g,SLOT(setSecondaryPlotLimits(int,const QStringList&)));
connect (axesDialog,SIGNAL(drawFrame(bool,int,const QColor&)),g,SLOT(drawCanvasFrame(bool,int,const QColor& )));

	axesDialog->insertColList(columnsList());
	axesDialog->setAxesLabelsColumns(g->axesLabelsDatasheets());
	axesDialog->setEnabledAxes(g->enabledAxes());
	axesDialog->setAxesType(g->axesType());
	QStringList lim=g->plotLimits();
	axesDialog->setScaleLimits(lim);
	axesDialog->initAxisFonts(g->axisFont(2), g->axisFont(0),g->axisFont(3),g->axisFont(1));
	axesDialog->setAxisTitles(g->scalesTitles());
	axesDialog->updateTitleBox(0);
	axesDialog->putGridOptions(g->getGridOptions());
	axesDialog->setFramed(g->framed());
	axesDialog->setSecondaryScales(g->secondaryPlotLimits());
	axesDialog->setAxesColors(g->axesColors());
	axesDialog->setFrameColor(g->canvasFrameColor());
	axesDialog->setFrameWidth(g->canvasFrameWidth());
	axesDialog->setTicksType(g->ticksType());
	axesDialog->setEnabledTickLabels(g->enabledTickLabels());
	axesDialog->setLabelsFormat(g->labelsFormat());
	axesDialog->initLabelsRotation(g->labelsRotation(QwtPlot::xBottom), 
														g->labelsRotation(QwtPlot::xTop));
	axesDialog->setCurrentScale(0);
	axesDialog->showNormal();
	axesDialog->setActiveWindow();
	}
else
	QMessageBox::warning(this,"QtiPlot - Warning",
				"This functionality is not available for pie plots!");
}
else if (plot3DWindows.contains(caption)>0)
	{
	plot3DDialog*pd = showPlot3dDialog();
	if (pd)
		pd->showAxisTab();
	}
}

void ApplicationWindow::showYAxesDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0 )
	return;

Graph* g = (Graph*)plot->activeGraph();
if (!g)
	return;

if (!g->isPiePlot())
	{
	activeGraph = g;
		
genDialog* axesDialog= new genDialog(this,"axesDialog",TRUE,WStyle_Tool|WDestructiveClose);
connect (axesDialog,SIGNAL(updatePlotLimits(int,const QStringList&)),g,SLOT(setPlotLimits(int,const QStringList&)));
connect (axesDialog,SIGNAL(updateAxisTitle(int,const QString&)),g,SLOT(setAxisTitle(int,const QString&)));
connect (axesDialog,SIGNAL(changeGridOptions(const gridOptions &)),g,SLOT(setGridOptions(const gridOptions &)));
connect (axesDialog,SIGNAL(changeAxisFont(int, const QFont &)),g,SLOT(setAxisFont(int,const QFont &)));
connect (axesDialog,SIGNAL(showAxis(int, int, const QString&, bool,int, bool,const QColor&, int, int, int)),
							this,SLOT(showAxis(int,int, const QString&, bool, int,bool,const QColor&, int, int, int)));
connect (axesDialog,SIGNAL(updateSecondaryLimits(int,const QStringList&)),g,SLOT(setSecondaryPlotLimits(int,const QStringList&)));
connect (axesDialog,SIGNAL(drawFrame(bool,int,const QColor&)),g,SLOT(drawCanvasFrame(bool,int,const QColor& )));

	axesDialog->insertColList(columnsList());
	axesDialog->setAxesLabelsColumns(g->axesLabelsDatasheets());
	axesDialog->setEnabledAxes(g->enabledAxes());
	axesDialog->setAxesType(g->axesType());

	QStringList lim=g->plotLimits();
	axesDialog->setScaleLimits(lim);
	axesDialog->initAxisFonts(g->axisFont(2), g->axisFont(0),g->axisFont(3),g->axisFont(1));
	axesDialog->setAxisTitles(g->scalesTitles());
	axesDialog->updateTitleBox(0);
	axesDialog->putGridOptions(g->getGridOptions());
	axesDialog->setFramed(g->framed());
	axesDialog->setSecondaryScales(g->secondaryPlotLimits());
	axesDialog->setAxesColors(g->axesColors());
	axesDialog->setFrameColor(g->canvasFrameColor());
	axesDialog->setFrameWidth(g->canvasFrameWidth());
	axesDialog->setTicksType(g->ticksType());
	axesDialog->setEnabledTickLabels(g->enabledTickLabels());
	axesDialog->setLabelsFormat(g->labelsFormat());
	axesDialog->initLabelsRotation(g->labelsRotation(QwtPlot::xBottom), 
														g->labelsRotation(QwtPlot::xTop));
	axesDialog->showNormal();
	axesDialog->setActiveWindow();
	axesDialog->setCurrentScale(1);
	}
else if (plot3DWindows.contains(g->caption())>0)
	showPlot3dDialog();
else
	QMessageBox::warning(this,"QtiPlot - Warning",
				"This functionality is not available for pie plots!");
}

plot3DDialog* ApplicationWindow::showPlot3dDialog()
{
Graph3D* g = (Graph3D*)ws->activeWindow();

if ( g && plot3DWindows.contains(g->caption())>0)
	{
	if (g->plotStyle() == Qwt3D::NOPLOT)
		{
		QApplication::restoreOverrideCursor();
		QMessageBox::warning(this, tr("QtiPlot - Warning"),
				      tr("Not available for empty 3D surface plots!"));
		return 0;
		}

	plot3DDialog* pd= new plot3DDialog(this,"plot3DDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (pd,SIGNAL(updateColors(const QColor&,const QColor&,const QColor&,const QColor&,const QColor&,const QColor&)),
		g,SLOT(updateColors(const QColor&,const QColor&,const QColor&,const QColor&,const QColor&,const QColor&)));

	connect (pd,SIGNAL(updateDataColors(const QColor&,const QColor&)),
		g,SLOT(setDataColors(const QColor&,const QColor&)));

	connect (pd,SIGNAL(updateTitle(const QString&,const QColor&,const QFont&)),
		g,SLOT(updateTitle(const QString&,const QColor&,const QFont&)));
	connect (pd,SIGNAL(updateResolution(int)),g,SLOT(setResolution(int)));
	connect (pd,SIGNAL(showColorLegend(bool)),g,SLOT(showColorLegend(bool)));
	connect (pd,SIGNAL(updateLabel(int,const QString&, const QFont&)),
		g,SLOT(updateLabel(int,const QString&, const QFont&)));
	connect (pd,SIGNAL(updateScale(int,const QStringList&)),
		g,SLOT(updateScale(int,const QStringList&)));
	connect (pd,SIGNAL(adjustLabels(int)),
		g,SLOT(adjustLabels(int)));
	connect (pd,SIGNAL(updateTickLength(int, double, double)),
		g,SLOT(updateTickLength(int, double, double)));
	connect (pd,SIGNAL(setNumbersFont(const QFont&)),
		g,SLOT(setNumbersFont(const QFont&)));
	connect (pd,SIGNAL(updateMeshLineWidth(double)),
		g,SLOT(setMeshLineWidth(double)));
	connect (pd,SIGNAL(updateBars(double)),g,SLOT(updateBars(double)));
	connect (pd,SIGNAL(updatePoints(double, bool)),g,SLOT(updatePoints(double, bool)));
	connect (pd,SIGNAL(updateTransparency(double)),g,SLOT(setTransparency(double)));
	connect (pd,SIGNAL(showWorksheet()),g,SLOT(showWorksheet()));
	connect (pd,SIGNAL(updateZoom(double)),g,SLOT(updateZoom(double)));
	connect (pd,SIGNAL(updateScaling(double,double,double)),
			g,SLOT(updateScaling(double,double,double)));
	connect (pd,SIGNAL(updateCones(double, int)),g,SLOT(updateCones(double, int)));
	connect (pd,SIGNAL(updateCross(double, double, bool, bool)),
					g,SLOT(updateCross(double, double, bool, bool)));

	pd->setMeshLineWidth(g->meshLineWidth());
	pd->setTransparency(g->transparency());
	pd->setDataColors(g->minDataColor(),g->maxDataColor());
	pd->setColors(g->titleColor(),g->meshColor(),g->axesColor(),g->numColor(),
				g->labelColor(), g->bgColor(),g->gridColor());

	pd->setTitle(g->plotTitle());
	pd->setTitleFont(g->titleFont());

	pd->setZoom(g->zoom());
	pd->setScaling(g->xScale(),g->yScale(),g->zScale());
	pd->setResolution(g->resolution());
	pd->showLegend(g->isLegendOn());
	pd->setAxesLabels(g->axesLabels());
	pd->setAxesTickLengths(g->axisTickLengths());
	pd->setAxesFonts(g->xAxisLabelFont(),g->yAxisLabelFont(),g->zAxisLabelFont());
	pd->setScales(g->scaleLimits());
	pd->setLabelsDistance(g->labelsDistance());
	pd->setNumbersFonts(g->numbersFont());

	if (g->coordStyle() == Qwt3D::NOCOORD)
		pd->disableAxesOptions();

	Qwt3D::PLOTSTYLE style= g->plotStyle();
	Graph3D::PointStyle pt=g->pointType();

	if ( style == Qwt3D::USER )
		{
		switch (pt)
			{
			case Graph3D::None :
			break;

			case Graph3D::Dots :
				pd->disableMeshOptions();
				pd->initPointsOptionsStack();
				pd->showPointsTab (g->pointsSize(), g->smoothPoints());
			break;

			case Graph3D::VerticalBars :
				pd->showBarsTab(g->barsRadius());
			break;

			case Graph3D::HairCross :
				pd->disableMeshOptions();
				pd->initPointsOptionsStack();
				pd->showCrossHairTab (g->crossHairRadius(), g->crossHairLinewidth(),
															g->smoothCrossHair(), g->boxedCrossHair());
			break;

			case Graph3D::Cones :
				pd->disableMeshOptions();
				pd->initPointsOptionsStack();
				pd->showConesTab(g->coneRadius(), g->coneQuality());
			break;
			}
		}
	else if ( style == Qwt3D::FILLED )
		pd->disableMeshOptions();
	else if (style == Qwt3D::HIDDENLINE || style == Qwt3D::WIREFRAME)
		pd->disableLegend();

	if (g->grids() == 0)
		pd->disableGridOptions();
	if (!g->userFunction())
		pd->showWorksheetBtn();

	pd->showNormal();
	pd->setActiveWindow();
	return pd;
	}
else return 0;
}

void ApplicationWindow::showPieDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (g)
	{
	pieDialog* pd= new pieDialog(this,"plotDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (pd,SIGNAL(drawFrame(bool,int,const QColor&)),g,SLOT(drawCanvasFrame(bool,int,const QColor& )));
	connect (pd,SIGNAL(toggleCurve()),g,SLOT(removePie()));
	connect (pd,SIGNAL(updatePie(const QPen&, const Qt::BrushStyle &,int,int)),g,SLOT(updatePie(const QPen&, const Qt::BrushStyle &,int,int)));
	connect (pd,SIGNAL(worksheet(const QString&)),this,SLOT(showWorksheet(const QString&)));

	QString curve=(g->plotCurvesList())[0];
	pd->insertCurveName(curve);
	QPen piePen=g->pieCurvePen();

	pd->setBorderWidth(piePen.width());
	pd->setBorderColor(piePen.color());
	pd->setBorderStyle(piePen.style());
	pd->setFirstColor(g->pieFirstColor());
	pd->setPattern(g->pieBrushStyle());

	pd->setPieSize(g->pieSize());
	pd->setFramed(g->framed());
	pd->setFrameColor(g->canvasFrameColor());
	pd->setFrameWidth(g->canvasFrameWidth());
	pd->showNormal();
	pd->setActiveWindow();
	}
}

void ApplicationWindow::showLayoutDialog()
{
QWidget *w = ws->activeWindow();
if (!w)
	return;
QString caption=w->caption();

if (plotWindows.contains(caption)>0)
{
MultiLayer* plot = (MultiLayer*)w;
if (!plot)
	return;
Graph *g=plot->activeGraph();
if (!g)
	return;

if (g->curves()>0)
	{
	if (!g->isPiePlot())
		{
		MyDialog* tabDialog= new MyDialog(this,"plotDialog",FALSE,WStyle_Tool|WDestructiveClose);
		tabDialog->setCaption( tr( "QtiPlot - Custom curves" ) );
			
		connect (tabDialog,SIGNAL(createWorksheet(const QString&)),g,SLOT(createWorksheet(const QString&)));
		connect (tabDialog,SIGNAL(worksheet(const QString&)),this,SLOT(showWorksheet(const QString&)));
		connect (tabDialog,SIGNAL(setLayout(int,const curveLayout&)),g,SLOT(updateCurveLayout(int,const curveLayout&)));
		connect (tabDialog,SIGNAL(updateErrorBars(int,bool,int,int, const QColor&,bool,bool,bool)),g,SLOT(updateErrorBars(int,bool,int,int,const QColor&,bool,bool,bool)));
		connect (tabDialog,SIGNAL(curveToRemove(int)),g,SLOT(removeCurve(int)));
		connect (tabDialog,SIGNAL(updateHistogram(const QString&,int, bool, double, double, double)),
					this,SLOT(updateHistogram(const QString&,int, bool, double, double, double)));
		connect (tabDialog,SIGNAL(updateBarsGap(int, int)),g,SLOT(updateBarsGap(int, int)));
		connect (tabDialog,SIGNAL(showHistogramStats(const QString&, int)),
					this,SLOT(showHistogramStats(const QString&, int)));
		connect (tabDialog,SIGNAL(updateVectorsLayout(int, int, int, int, int, bool, const QString&, const QString&)),
					this,SLOT(updateVectorsLayout(int, int, int, int, int, bool, const QString&, const QString&)));
			
		tabDialog->insertCurvesList(g->plotAssociations());
		tabDialog->insertColumnsList(columnsList());
		tabDialog->setAllCurvesLayout(g->allCurvesLayout());
		tabDialog->setErrorBarsLayout(g->erorsLayout());
		tabDialog->setSelectedCurve(0);
		tabDialog->showNormal();
		tabDialog->setActiveWindow();
		}
	else
		showPieDialog();
	}
else if (g->curves() == 0)
			QMessageBox::warning(this, "QtiPlot - Empty plot","There are actually  no curves on the active layer!");
}
else if (plot3DWindows.contains(caption)>0)
	showPlot3dDialog();
}

void ApplicationWindow::updateHistogram(const QString& curveName, int curveIndex, 
										bool automatic, double binSize, double begin, double end)
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (!g)
	return;

WksheetWindow* w=table(curveName);
if (!w)
	return;

g->updateHistogram(w, curveName, curveIndex, automatic, binSize, begin, end);
g->setFocus();
g->setActiveWindow();
}

void ApplicationWindow::showHistogramStats(const QString& curveName, int curveIndex)
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;
Graph* g = (Graph*)plot->activeGraph();
if (!g)
	return;
WksheetWindow* w=table(curveName);
if (!w)
	return;
QString result=g->showHistogramStats(w, curveName, curveIndex);
if (!result.isEmpty())
	{
	logInfo+=result;
	if (resultsOn)
		resultsOn = !resultsOn;
	showResults();
	}
}

void ApplicationWindow::zoom()
{
	MultiLayer* plot = (MultiLayer*)ws->activeWindow();
	if (!plot || plotWindows.contains(plot->caption())<=0)
		return;

    Graph* g = (Graph*)plot->activeGraph();
	if (!g)
		return;

	if (g->isPiePlot())
		{
		if (btnZoom->isOn())
			QMessageBox::warning(this,"QtiPlot - Warning",
				"This functionality is not available for pie plots!");
		btnZoom->setOn(FALSE);
		return;
		}
	else
		{
		zoomOn=!zoomOn;
		g->zoom(zoomOn);
		}
}

void ApplicationWindow::removePoints()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (g)
{
if (g->isPiePlot())
	{
	QMessageBox::warning(this,"QtiPlot - Warning",
				"This functionality is not available for pie plots!");
	removePointsOn=FALSE;
	btnRemovePoints->setOn(removePointsOn);
	dataMenu->setItemChecked(removeID,removePointsOn);
	return;
	}
else
	{
	if (! removePointsOn)
			{
			switch(QMessageBox::warning (this,"QtiPlot",
				"This will modify the data in the worksheets!\nAre you sure you want to continue?",
			"Continue","Cancel",0,1))
							{
							case 0:
								    removePointsOn=!removePointsOn;
									g->removePoints(removePointsOn);
									info->setText("Select point and double click to remove it!");
									displayBar->show();
									btnRemovePoints->setOn(removePointsOn);
									dataMenu->setItemChecked(removeID,removePointsOn);
							break;
							case 1:
									removePointsOn = FALSE;
									btnPointer->setOn(TRUE);
							break;
							}// end switch
			}
	else
			{
			g->removePoints(FALSE);
			displayBar->hide();
			btnRemovePoints->setOn(FALSE);
			removePointsOn = FALSE;
			btnMovePoints->setOn(movePointsOn);
			dataMenu->setItemChecked(removeID,removePointsOn);
		    }
	}
}
}

void ApplicationWindow::movePoints()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (!g)
	return;

if (g->isPiePlot())
{
	QMessageBox::warning(this,"QtiPlot - Warning",
				"This functionality is not available for pie plots!");

	btnMovePoints->setOn(movePointsOn);
	dataMenu->setItemChecked(moveID,movePointsOn);
	return;
}
else
{
if (!movePointsOn)
{
switch(QMessageBox::warning (this,"QtiPlot",
	"This will modify the data in the worksheets! \
	\nAre you sure you want to continue?",
	"Continue","Cancel",0,1))
	{
	case 0:
	movePointsOn =TRUE;
	btnMovePoints->setOn(movePointsOn);
	dataMenu->setItemChecked(moveID,movePointsOn);

	if (g)
		{
		g->movePoints(movePointsOn);
		info->setText("Please, click on plot and move cursor!");
		displayBar->show();
		}
		break;

	case 1:
		movePointsOn = FALSE;
		btnPointer->setOn(TRUE);
		break;
	}
}
else
	{
	displayBar->hide();
	movePointsOn = FALSE;
	btnMovePoints->setOn(movePointsOn);
	dataMenu->setItemChecked(moveID,movePointsOn);
	g->movePoints(movePointsOn);
	}
}
}

//print active window
void ApplicationWindow::print()
{
QWidget* w = ws->activeWindow();
if (!w)
	return;

if (plotWindows.contains(w->caption())>0)
	{
	((MultiLayer*)w)->printAllLayers();
	}
else if (plot3DWindows.contains(w->caption())>0)
	{
	((Graph3D*)w)->print();
	}
else if (tableWindows.contains(w->caption())>0)
	{
	((WksheetWindow*)w)->print();
	}
}

// print window from project explorer
void ApplicationWindow::printWindow()
{
QListViewItem *lbi=lv->currentItem();
QWidget *w=window(lbi);

if (!w)
	return;

if (plotWindows.contains(w->caption())>0)
	((MultiLayer*)w)->printAllLayers();

else if (plot3DWindows.contains(w->caption())>0)
	((Graph3D*)w)->print();

else if (tableWindows.contains(w->caption())>0)
	((WksheetWindow*)w)->print();
}

void ApplicationWindow::showExpDecayDialog()
{
    MultiLayer* plot = (MultiLayer*)ws->activeWindow();
	if (!plot || plotWindows.contains(plot->caption())<=0)
		return;

Graph* g = (Graph*)plot->activeGraph();
if ( g )
	{
	if (g->curves()==0)
		{
		QMessageBox::warning(this,"QtiPlot - Warning",
				"There are no curves available on this plot!");
		return;
		}

	activeGraph=g;
	aw= (QWidget *)plot;

	expDecayDialog *edd=new expDecayDialog(1, this,"polyDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (edd,SIGNAL(options(const QString&, double, double, double)),
		 this,SLOT(fitExpDecay(const QString&, double, double, double)));

	QStringList curvesOnPlot=g->plotCurvesList();
	edd->setCurveNames(curvesOnPlot);
	edd->showNormal();
	edd->setActiveWindow();
	}
}

void ApplicationWindow::showTwoExpDecayDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
		return;

Graph* g = (Graph*)plot->activeGraph();
if ( g )
	{
	if (g->curves()==0)
		{
		QMessageBox::warning(this,"QtiPlot - Warning",
				"There are no curves available on this plot!");
		return;
		}

	activeGraph=g;
	aw= (QWidget *)plot;

	expDecayDialog *edd=new expDecayDialog(2, this,"polyDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (edd,SIGNAL(options(const QString&, double, double, double, double)),
		 this,SLOT(fitExpDecay2(const QString&, double, double, double, double)));

	QStringList curvesOnPlot=g->plotCurvesList();
	edd->setCurveNames(curvesOnPlot);
	edd->showNormal();
	edd->setActiveWindow();
	}
}

void ApplicationWindow::showExpDecay3Dialog()
{
      MultiLayer* plot = (MultiLayer*)ws->activeWindow();
	if (!plot || plotWindows.contains(plot->caption())<=0)
		return;

Graph* g = (Graph*)plot->activeGraph();
if ( g )
	{
	if (g->curves()==0)
		{
		QMessageBox::warning(this,"QtiPlot - Warning",
				"There are no curves available on this plot!");
		return;
		}

	activeGraph=g;
	aw= (QWidget *)plot;

	expDecayDialog *edd=new expDecayDialog(3, this,"polyDialog",TRUE,WStyle_Tool|WDestructiveClose);

	connect (edd,SIGNAL(options3(const QString&,double,double,double,double,double)),
		 this,SLOT(fitExpDecay3(const QString&,double,double,double,double,double)));

	QStringList curvesOnPlot=g->plotCurvesList();
	edd->setCurveNames(curvesOnPlot);
	edd->show3Times();
	edd->showNormal();
	edd->setActiveWindow();
	}
}

void ApplicationWindow::showFitDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if ( g )
	{
	if (g->curves()==0)
		{
		QMessageBox::warning(this,"QtiPlot - Warning",
				"There are no curves available on this plot!");
		return;
		}

	activeGraph=g;
	aw=(QWidget*) plot;

	fd=new fitDialog(0,"fitDialog", WDestructiveClose);
	fd->	setIcon(QPixmap(logo_xpm));
	connect (fd,SIGNAL(options(const QString&,const QString&,const QStringList&,
								const QStringList&,double,double,int, int,double)),
		 this,SLOT(fitCurve(const QString&,const QString&,const QStringList&,
							const QStringList&,double,double,int, int,double)));
	connect (fd,SIGNAL(clearFunctionsList()),this,SLOT(clearFitFunctionsList()));

	QStringList curvesOnPlot=g->plotCurvesList();

	fd->setLimits(g->xBottomAxisLimits());
	fd->insertFunctionsList(fitFunctions);
	fd->setCurveNames(curvesOnPlot);
	fd->showNormal();
	fd->setActiveWindow();
	}
}

void ApplicationWindow::fitCurve(const QString& curve,const QString& formula,
								 const QStringList& params,const QStringList& paramsInit,
								 double from,double to,int points, int solver, double tolerance)
{
activeGraph->setFitID(++fitNumber);
QString result=activeGraph->fitNonlinearCurve(curve,formula,params,paramsInit,
											from,to,points, solver, tolerance);

if(!result.isEmpty())
	{
	logInfo+=result;
	if (resultsOn)
		resultsOn = !resultsOn;
	showResults();
	activeGraph->setFocus();
	emit modified();

	if (!aw)
		return;

	((MultiLayer*)aw)->updateTransparency();
	aw->setActiveWindow();
	}

if (fitFunctions.contains(formula)<=0)
	fitFunctions<<formula;
}

void ApplicationWindow::showFitPolynomDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
		return;

Graph* g = (Graph*)plot->activeGraph();
if ( g )
	{
	int curves=g->curves();
	if (curves == 0)
		{
		QMessageBox::warning(this,"QtiPlot - Warning",
				"There are no curves available on this plot!");
		return;
		}
	else if(curves==1 && g->curveDataSize(0)<2)
			{
			QMessageBox::warning(this,"QtiPlot - Fitting error","You need at least 2 points to fit. Operation aborted!");
			return;
			}
	else if(curves > 1)
			{
			int check=0;
			while(g->curveDataSize(check)<2){
				if(check==curves)
					{
					QMessageBox::warning(this,"QtiPlot - Fitting error","You need at least 2 points to fit. Operation aborted!");
					return;
					}
				check++;
				}
			}

	activeGraph=g;
	aw=(QWidget*)plot;

	// Find the minimum and the maximum x for each graph
	double *Xmin,*Xmax;
	Xmin=vector(0,curves-1);
	Xmax=vector(0,curves-1);
	int *points;
	points=intvector(0,curves-1);

	for(int i=0;i<curves;i++)
		{ // For each curve find the minimum and the maximum value of x
		int n=g->curveDataSize(i);
		if (n>=2)
			{
			Xmax[i]=g->maxXValue(i);
			Xmin[i]=g->minXValue(i);
			}
		else
			{
			Xmax[i]=0;
			Xmin[i]=0;
			}
		points[i]=n;
		}

	polynomFitDialog *pfd=new polynomFitDialog(this,"polyDialog",TRUE,WStyle_Tool|WDestructiveClose);	
	connect (pfd,SIGNAL(options(const QString&,int,int,double,double,bool)),
		 this,SLOT(fitPolynomial(const QString&,int,int,double,double,bool)));

	pfd->setCurveNames(g->plotCurvesList());
	pfd->setXLimits(&Xmin[0],&Xmax[0],curves,&points[0]);
	free_vector(Xmin,0,curves-1); free_vector(Xmax,0,curves-1); free_intvector(points,0,curves-1);

	long sc=g->selectedCurveID();
	if (sc> 0) // a curve is selected
		pfd->setCurveLimits(g->curveIndex(sc),g->selectedXStartValue(),g->selectedXEndValue());//use that curve

	pfd->showNormal();
	pfd->setActiveWindow();
	}
}

void ApplicationWindow::fitPolynomial(const QString& name,int order,int points,
				double start, double end ,bool showFormula)
{
activeGraph->setFitID(++fitNumber);
QString result=activeGraph->fitPolynomial(name,order, points, start, end, showFormula);
if ( !result.isEmpty() )
	{
	logInfo+=result;
	if (resultsOn)
		resultsOn = !resultsOn;
	showResults();
	activeGraph->setFocus();
	emit modified();

	if (! aw)
		return;

	((MultiLayer*)aw)->updateTransparency();
	aw->setActiveWindow();
	}
}

void ApplicationWindow::fft_for()
{
analysis("fft_for");
}

void ApplicationWindow::fft_inv()
{
analysis("fft_inv");
}

void ApplicationWindow::interlinear()
{
analysis("interlinear");
}

void ApplicationWindow::akima()
{
analysis("akima");
}

void ApplicationWindow::intercubic()
{
analysis("intercubic");
}

void ApplicationWindow::fitLinear()
{
analysis("fitLinear");
}

void ApplicationWindow::fitExpDecay2(const QString& name, double firstTime, double secondTime,
						double start, double yOffset)
{
activeGraph->setFitID(++fitNumber);
QString result=activeGraph->fitExpDecay2(name, firstTime, secondTime, start, yOffset);
if ( !result.isEmpty() )
	{
	logInfo+=result;
	if (resultsOn)
		resultsOn = !resultsOn;
	showResults();
	activeGraph->setFocus();
	emit modified();

	if (! aw)
		return;

	((MultiLayer*)aw)->updateTransparency();
	aw->setActiveWindow();
	}
}

void ApplicationWindow::fitExpDecay3(const QString& name, double firstTime, double secondTime,
						double thirdTime, double start, double yOffset)
{
activeGraph->setFitID(++fitNumber);
QString result=activeGraph->fitExpDecay3(name, firstTime, secondTime, thirdTime, start, yOffset);
if ( !result.isEmpty() )
	{
	logInfo+=result;
	if (resultsOn)
		resultsOn = !resultsOn;
	showResults();
	emit modified();

	if (! aw)
		return;

	((MultiLayer*)aw)->updateTransparency();
	aw->setFocus();
	}
}

void ApplicationWindow::showIntDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
		return;

Graph* g = (Graph*)plot->activeGraph();
if ( g )
	{
	int curves=g->curves();
	if (curves==0)
		{
		QMessageBox::warning(this,"QtiPlot - Warning",
				"There are no curves available on this plot!");
		return;
		}
	else if(curves==1 && g->curveDataSize(0)<2)
			{
			QMessageBox::warning(this,"QtiPlot - Integration error","You need at least 2 points to integrate. Integration can not proceed!");
			return;
			}
	else if(curves > 1)
			{
			int check=0;
			while(g->curveDataSize(check)<2){
				if(check==curves)
					{
					QMessageBox::warning(this,"QtiPlot - Integration error","I could not find a curve with more than two points. Integration can not proceed!");
					return;
					}
				check++;
				}
			}

	activeGraph=g;

	// Find the minimum and the maximum x for each graph
	double *Xmin,*Xmax;
	Xmin=vector(0,curves-1);
	Xmax=vector(0,curves-1);
	int *points;
	points=intvector(0,curves-1);

	for(int i=0;i<curves;i++)
		{ // For each curve find the minimum and the maximum value of x
		int n=g->curveDataSize(i);
		if (n>=2)
			{
			Xmax[i]=g->maxXValue(i);
			Xmin[i]=g->minXValue(i);
			}
		else
			{
			Xmax[i]=0;
			Xmin[i]=0;
			}
		points[i]=n;
		}

	intDialog *intd=new intDialog(this,"intDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (intd,SIGNAL(options(int,int,int,double,double,double)),
		 this,SLOT(integrate(int,int,int,double,double,double)));

	intd->setCurveNames(g->plotCurvesList());
	intd->getMinMax(&Xmin[0],&Xmax[0],curves,&points[0]);
	free_vector(Xmin,0,curves-1); free_vector(Xmax,0,curves-1); free_intvector(points,0,curves-1);

	long sc=g->selectedCurveID();
	if (sc> 0) // a curve is selected
			intd->setCurveLimits(g->curveIndex(sc),g->selectedXStartValue(),g->selectedXEndValue());//use that curve
	intd->showNormal();
	intd->setActiveWindow();
	}
}

void ApplicationWindow::integrate(int curveID, int order, int iter, double tol, double low, double up)
{
long curveKey=activeGraph->curveKey(curveID);
QString result=activeGraph->integrateCurve(curveKey,order,iter,tol,low,up);
if ( !result.isEmpty() )
	{
	logInfo+=result;
	if (resultsOn)
		resultsOn = !resultsOn;
	showResults();
	emit modified();
	}
activeGraph->setFocus();
}

void ApplicationWindow::fitExpDecay()
{
analysis("fitExpDecay");
}

void ApplicationWindow::fitExpDecay(const QString& name, double damping, double amplitude, double yOffset)
{
activeGraph->setFitID(++fitNumber);
QString result=activeGraph->fitExpDecay(name, damping, amplitude, yOffset);
if ( !result.isEmpty() )
	{
	logInfo+=result;
	if (resultsOn)
		resultsOn = !resultsOn;
	showResults();
	activeGraph->setFocus();
	emit modified();

	if (! aw)
		return;

	((MultiLayer*)aw)->updateTransparency();
	aw->setActiveWindow();
	}
}

void ApplicationWindow::fitExpGrowth()
{
      analysis("fitExpGrowth");
}

void ApplicationWindow::fitGauss()
{
  analysis("fitGauss");
}

void ApplicationWindow::fitLorentz()

{
  analysis("fitLorentz");
}

void ApplicationWindow::differentiate()
{
analysis("differentiate");
}

void ApplicationWindow::showResults()
{
resultsOn = !resultsOn;
btnResults->setOn(resultsOn);
view->setItemChecked(logID,resultsOn);

if (resultsOn)
	{
	if (!logInfo.isEmpty())
			results->setText(logInfo);
	else
			results->setText("Sorry, there are no results to display!");

	results->show();
	results->scrollToBottom ();
	}
else
	results->hide();
}

void ApplicationWindow::showPicker()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (!g)
	{
	pickerOn=!pickerOn;
	activeGraph->showPlotPicker(pickerOn);
	btnPicker->setOn(pickerOn);
	return;
	}

if (g->isPiePlot())
	{
	QMessageBox::warning(this,"QtiPlot - Warning",
				"This functionality is not available for pie plots!");

	btnPicker->setOn(pickerOn);
	return;
	}
else
{
pickerOn = !pickerOn;
btnPicker->setOn(pickerOn);

if (pickerOn)
	{
    if (g)
		{
		activeGraph=g;
		g->showPlotPicker(pickerOn);
		info->setText("Please, click on plot and move cursor!");
		displayBar->show();
		}
	}
else
	{
	displayBar->hide();
	activeGraph->showPlotPicker(FALSE);
	}
}
}

void ApplicationWindow::showRangeSelectors()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;
	
Graph* g = (Graph*)plot->activeGraph();
if (!g)
	return;
	
if (g->isPiePlot())
{
	QMessageBox::warning(this,"QtiPlot - Warning",
				"This functionality is not available for pie plots!");
	btnSelect->setOn(selectOn);
	return;
}

if (g->curves()==0)
	{
	QMessageBox::warning(this,"QtiPlot - Warning",
				"There are no curves available on this plot!");	
	btnSelect->setOn(FALSE);
	return;
	}
	
selectOn = !selectOn;
if (selectOn)
	{	
	selectorsActiveGraph=g;	
	if (g->enableRangeSelectors(selectOn))	
		{
		info->setText("Click or use Ctrl+arrow key to select range (arrows select active cursor)!");
		displayBar->show();
		}
	}
else 
	{
	displayBar->hide();
	if (selectorsActiveGraph)
		selectorsActiveGraph->enableRangeSelectors(FALSE);
	btnSelect->setOn(FALSE);
	}
}

void ApplicationWindow::showCursor()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (!g)
	return;

if (g->isPiePlot())
	{
	QMessageBox::warning(this,"QtiPlot - Warning",
				"This functionality is not available for pie plots!");

	btnCursor->setOn(cursorOn);
	return;
	}
else
	{
	cursorOn = !cursorOn;
	btnCursor->setOn(cursorOn);

	if (cursorOn)
		{
		activeGraph=g;
		g->enableCursor(cursorOn);
		info->setText("Click on plot to display information!");
		displayBar->show();
		}
	else
		{
		displayBar->hide();
		activeGraph->enableCursor(FALSE);
		activeGraph->replot();
		}
	}
}

void ApplicationWindow::unzoom()
{
    MultiLayer* plot = (MultiLayer*)ws->activeWindow();
	if (!plot || plotWindows.contains(plot->caption())<=0)
		return;

    Graph* g = (Graph*)plot->activeGraph();
    if ( g )
		{
		g->setAutoScale();
		emit modified();
		}
}

void ApplicationWindow::newLegend()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if ( g )
	g->newLegend(plotLegendFont, legendFrameStyle);
}

void ApplicationWindow::disableAddText()
{
textOn = !textOn;
btnText->setOn(textOn);
showTextDialog();
}

void ApplicationWindow::addText()
{
if (drawLineOn || cursorOn || selectOn || pickerOn || movePointsOn || removePointsOn || zoomOn)
	{
	pickDataTool(btnPointer);
	btnPointer->setOn(TRUE);
	}

textOn = !textOn;
btnText->setOn(textOn);

MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
		return;

Graph* g=0;

switch(QMessageBox::information(this,
            tr("QtiPlot - Add new layer?"),
            tr("Do you want to add the text on a new layer or on the active layer?"),
            tr("On &New Layer"), tr("On &Active Layer"), tr("&Cancel"),
            0, 2 ) )
	{
	case 0:
		plot->addTextLayer();
	break;

	case 1:
		g = (Graph*)plot->activeGraph();
		if (g)
			g->drawText(textOn);
	break;

	case 2:
		textOn = FALSE;
		btnText->setOn(textOn);
		return;
	break;
	}
}

void ApplicationWindow::addImage()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (g)
	{
	QStringList list=QImage::inputFormatList ();
	QString filter="Images (*.jpg *JPG ",aux;
	int i;
	for (i=0;i<(int)list.count();i++)
		{
		aux="*."+(list[i]).lower()+" *. "+list[i] + " ";
		filter+=aux;
		}
	filter+=");;";
	for (i=0;i<(int)list.count();i++)
		{
		aux="*."+(list[i]).lower()+" *. "+list[i] +";;";
		filter+=aux;
		}

	QString fn = QFileDialog::getOpenFileName(QString::null, filter, 0, 0,
			"QtiPlot - Insert image from file", 0, TRUE);
	if ( !fn.isEmpty() )
		{
		QApplication::setOverrideCursor(waitCursor);
		QPixmap photo;
		for (i=0;i<(int)list.count();i++)
				{
				if (fn.contains("."+(list[i]).lower())>0 || fn.contains("."+list[i])>0)
					{
					photo.load(fn,list[i],QPixmap::Color);
					break;
					}
				}

		if (fn.contains(".jpg")>0 || fn.contains(".JPG")>0)
			photo.load(fn,"JPEG",QPixmap::Color);

		g->insertImageMarker(photo,fn);
		QApplication::restoreOverrideCursor();
		}
	}
}

void ApplicationWindow::drawLine()
{
drawLineOn = !drawLineOn;

MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
    if (g)
	{
	g->drawLine(drawLineOn);
	emit modified();
	}
}

void ApplicationWindow::showImageDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (g)
	{
	ImageMarker *im= (ImageMarker *) g->selectedMarkerPtr();
	if (!im)
		return;
	
	imageDialog *id=new imageDialog(0,"imageDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (id,SIGNAL(options(int,int,int,int)),g,SLOT(updateImageMarker(int,int,int,int)));
	id->setIcon(QPixmap(logo_xpm));
	id->setOrigin(im->getOrigin());
	id->setSize(im->size());
	id->showNormal();
	id->setFocus();
	}
}

void ApplicationWindow::showLayerDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

layerDialog *id=new layerDialog(this,"layerDialog",TRUE,WStyle_Tool|WDestructiveClose);

connect (id, SIGNAL(options(int, int, int, int, bool)), plot, SLOT(arrangeLayers(int, int, int, int, bool)));
connect (id, SIGNAL(setFonts(const QFont&, const QFont&, const QFont&, const QFont&)),
						plot,SLOT(setFonts(const QFont&, const QFont&, const QFont&, const QFont&)));

	id->setCaption(tr("QtiPlot - Define Layout"));
	id->initFonts(plotTitleFont, plotAxesFont, plotNumbersFont, plotLegendFont);
	id->setColumns(plot->getCols());
	id->setRows(plot->getRows());
	id->setGraphsNumber(plot->graphsNumber());
	id->setSpacing(plot->colsSpacing(), plot->rowsSpacing());
	id->showNormal();
	id->setActiveWindow();
}

void ApplicationWindow::showPlotGeometryDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (g)
	{
	imageDialog *id=new imageDialog(0,"imageDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (id,SIGNAL(options(int,int,int,int)),plot,SLOT(setGraphGeometry(int,int,int,int)));
	id->setIcon(QPixmap(logo_xpm));
	id->setCaption(tr("QtiPlot - Layer Geometry"));
	id->setOrigin(g->pos());
	id->setSize(g->plotWidget()->size());
	id->showNormal();
	id->setActiveWindow();
	}
}

void ApplicationWindow::showTextDialog()
{
MultiLayer* plot= (MultiLayer*)ws->activeWindow();
if (plotWindows.contains(plot->caption())<=0)
		return;

Graph *g=(Graph *)plot->activeGraph();
if ( g )
	{
	LegendMarker *m= (LegendMarker *) g->selectedMarkerPtr();
	if (!m)
		return;
	
	textDlg *td=new textDlg(0,"txtDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (td,SIGNAL(values(const QString&,int,int,const QFont&, const QColor&)),
		 g,SLOT(updateTextMarker(const QString&,int,int,const QFont&, const QColor&)));

	td->setIcon(QPixmap(logo_xpm));
	td->setText(m->getText());
	td->setFont(m->getFont());
	td->setColor(m->getTextColor());
	td->setBackgroundType(m->getBkgType());
	td->setAngle(m->getAngle());
	td->showNormal();
	td->setFocus();
	}
}

void ApplicationWindow::showLineDialog()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (g)
	{
	LineMarker* lm=(LineMarker*)g->selectedMarkerPtr();
	if (!lm)
		return;

	lineDialog *ld=new lineDialog(0,"lineDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (ld,SIGNAL(values(const QColor&,int,Qt::PenStyle,bool,bool)),
		 g,SLOT(updateLineMarker(const QColor&,int,Qt::PenStyle,bool, bool)));

	connect (ld,SIGNAL(setLineGeometry(const QPoint&,const QPoint&)),
		 g,SLOT(updateLineMarkerGeometry(const QPoint&,const QPoint&)));

	connect (ld,SIGNAL(setHeadGeometry(int, int, bool)),
		 g,SLOT(setArrowHeadGeometry(int, int, bool)));

	ld->setIcon(QPixmap(logo_xpm));
	ld->setStartPoint(lm->startPoint());
	ld->setEndPoint(lm->endPoint());
	ld->setColor(lm->color());
	ld->setWidth(lm->width());
	ld->setStyle(lm->style());
	ld->setEndArrow(lm->getEndArrow());
	ld->setStartArrow(lm->getStartArrow());
	ld->initHeadGeometry(lm->headLength(), lm->headAngle(),
											lm->filledArrowHead());
	ld->enableHeadTab();
	ld->showNormal();
	ld->setFocus();
	}
}

void ApplicationWindow::addColToTable()
{
    WksheetWindow* m = (WksheetWindow*)ws->activeWindow();
    if ( m )
		m->addCol();
}

void ApplicationWindow::clearSelection()
{
QWidget* m = (QWidget*)ws->activeWindow();
    if (tableWindows.contains(m->caption())>0)
		{
		WksheetWindow* t= (WksheetWindow*)ws->activeWindow();
		t->clearSelection();
		emit modified();
		}
	else  if (plotWindows.contains(m->caption())>0)
		{
		MultiLayer* plot = (MultiLayer*)m;
		if (!plot)
			return;

		Graph* g = (Graph*)plot->activeGraph();

		if (g->titleSelected())
			g->removeTitle();
		else if (g->markerSelected())
			g->removeMarker();
		}
}

void ApplicationWindow::copySelection()
{
    QWidget* m = (QWidget*)ws->activeWindow();
	if (!m)
		return;

    if (tableWindows.contains(m->caption())>0)
		{
		WksheetWindow* t= (WksheetWindow*)ws->activeWindow();
		t->copySelection();
		}
	else if (plotWindows.contains(m->caption())>0)
		{
		MultiLayer* plot = (MultiLayer*)m;
		if (!plot || plot->graphsNumber() == 0)
			return;

		plot->copyAllLayers();
		Graph* g = (Graph*)plot->activeGraph();
		if (g && g->markerSelected())
			copyMarker();
		else
			copyActiveLayer();
		}
}

void ApplicationWindow::cutSelection()
{
QWidget* m = (QWidget*)ws->activeWindow();
    if (tableWindows.contains(m->caption())>0)
		{
		WksheetWindow* t= (WksheetWindow*)ws->activeWindow();
		t->cutSelection();
		}
	else if (plotWindows.contains(m->caption())>0)
		{
		MultiLayer* plot = (MultiLayer*)m;
		if (!plot || plot->graphsNumber() == 0)
			return;

		Graph* g = (Graph*)plot->activeGraph();
		copyMarker();
		g->removeMarker();
		}
emit modified();
}

void ApplicationWindow::copyMarker()
{
QWidget* m = (QWidget*)ws->activeWindow();
MultiLayer* plot = (MultiLayer*)m;
Graph* g = (Graph*)plot->activeGraph();
if (g && g->markerSelected())
			{
				g->copyMarker();
				copiedMarkerType=g->copiedMarkerType();
				QRect rect=g->copiedMarkerRect();
				auxMrkStart=rect.topLeft();
				auxMrkEnd=rect.bottomRight();

				if (copiedMarkerType == Graph::Text)
					{
					LegendMarker *m= (LegendMarker *) g->selectedMarkerPtr();
					auxMrkText=m->getText();
					auxMrkColor=m->getTextColor();
					auxMrkFont=m->getFont();
					auxMrkBkg=m->getBkgType();
					}
				else if (copiedMarkerType == Graph::Arrow)
					{
					LineMarker *m=(LineMarker *) g->selectedMarkerPtr();
					auxMrkWidth=m->width();
					auxMrkColor=m->color();
					auxMrkStyle=m->style();
					startArrowOn=m->getStartArrow();
					endArrowOn=m->getEndArrow();
					arrowHeadLength=m->headLength();
					arrowHeadAngle=m->headAngle();
					fillArrowHead=m->filledArrowHead();
					}
				 else if (copiedMarkerType == Graph::Image)
					{
					ImageMarker *im= (ImageMarker *) g->selectedMarkerPtr();
					auxMrkFileName=im->getFileName();
					}
				}
copiedLayer=FALSE;
}

void ApplicationWindow::pasteSelection()
{
    QWidget* m = (QWidget*)ws->activeWindow();
    if (tableWindows.contains(m->caption())>0)
		{
		WksheetWindow* t= (WksheetWindow*)ws->activeWindow();
		t->pasteSelection();
		}
	else if (plotWindows.contains(m->caption())>0)
		{
		MultiLayer* plot = (MultiLayer*)m;
		if (!plot)
			return;

		if (copiedLayer)
			{
			QApplication::setOverrideCursor(waitCursor);

			Graph* g=plot->loadLayer();
			clone (lastCopiedLayer,g);
			QPoint pos=plot->mapFromGlobal(QCursor::pos());
			plot->setGraphGeometry(pos.x(),pos.y()-20,lastCopiedLayer->width(),lastCopiedLayer->height());
			plot->connectLayer(g);

			QApplication::restoreOverrideCursor();
			}
		else
			{
			if (plot->graphsNumber() == 0)
				return;

			Graph* g = (Graph*)plot->activeGraph();
			if (!g)
				return;
	
			g->setCopiedMarkerType(copiedMarkerType);
			g->setCopiedMarkerEnds(auxMrkStart,auxMrkEnd);

			if (copiedMarkerType == Graph::Text)
					g->setCopiedTextOptions(auxMrkBkg,auxMrkText,auxMrkFont,auxMrkColor);
			if (copiedMarkerType == Graph::Arrow)
					g->setCopiedArrowOptions(auxMrkWidth,auxMrkStyle,
																auxMrkColor,startArrowOn,
																endArrowOn, arrowHeadLength,
																arrowHeadAngle, fillArrowHead);
			if (copiedMarkerType == Graph::Image)
					g->setCopiedImageName(auxMrkFileName);
			g->pasteMarker();
			}
		}
emit modified();
}

WksheetWindow* ApplicationWindow::copyTable()
{
int i,j;
QString s;
WksheetWindow* w=0;

WksheetWindow* m = (WksheetWindow*)ws->activeWindow();
if (m)
	{
	tables++;
	QString caption="table"+QString::number(tables);
	while(tableWindows.contains(caption)>0 || plotWindows.contains(caption)>0 || plot3DWindows.contains(caption)>0)
		{
		tables++;
		caption="table"+QString::number(tables);
		}
	int c=m->tableCols();
	int r=m->tableRows();
	w=newWrksheet(caption,r,c);
	for (i=0;i<r;i++)
		for (j=0;j<c;j++)
			{
			s=m->text(i,j);
			w->setText(i,j,s);
			}

	w->setColWidths(m->columnWidths());
	w->setXCol(m->colX());
	w->setYCol(m->colY());
	w->setHeader(m->colNames());
	w->setCommandes(m->getCommandes());

	QString spec=m->makeSpecifications();
	w->setSpecifications(spec.replace(m->caption(),caption));
	w->showNormal();
	setListViewSize(caption, w->sizeToString());
	emit modified();
	}
return w;
}

Graph3D* ApplicationWindow::copySurfacePlot()
{
Graph3D* g = (Graph3D*)ws->activeWindow();

if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	if (g->plotStyle() == Qwt3D::NOPLOT)
		{
		QApplication::restoreOverrideCursor();
		QMessageBox::warning(this, tr("QtiPlot - Duplicate error"),
				      tr("Empty 3D surface plots can not be duplicated!"));
		return 0;
		}

	graphs++;
	QString caption="graph"+QString::number(graphs);
	while(plotWindows.contains(caption)>0||plot3DWindows.contains(caption)>0)
		{
		graphs++;
		caption="graph"+QString::number(graphs);
		}

	Graph3D *g2=0;
	QString colNames=g->formula();
	if (g->userFunction())
		{
		g2=newPlot3D(caption,g->formula(),g->xStart(),g->xStop(),
								g->yStart(),g->yStop(),
								g->zStart(),g->zStop());
		}
	else if (colNames.contains("(Z)",TRUE) > 0)
		g2=dataPlotXYZ(caption,colNames,g->xStart(),g->xStop(),
									  g->yStart(),g->yStop(),g->zStart(),g->zStop());
	else
		g2=dataPlot3D(caption,colNames,g->xStart(),g->xStop(),
									g->yStart(),g->yStop(),g->zStart(),g->zStop());//Ribbon plot
	if (!g2)
		return 0;

	Graph3D::PointStyle pt=g->pointType();
	if (g->plotStyle() == Qwt3D::USER )
		{
		switch (pt)
				{
				case Graph3D::None :
				break;

				case Graph3D::Dots :
					g2->setPointOptions(g->pointsSize(), g->smoothPoints());
				break;

				case Graph3D::VerticalBars :
					g2->setBarsRadius(g->barsRadius());
				break;

				case Graph3D::HairCross :
					g2->setCrossOptions(g->crossHairRadius(), g->crossHairLinewidth(),
										g->smoothCrossHair(), g->boxedCrossHair());
				break;

				case Graph3D::Cones :
					g2->setConesOptions(g->coneRadius(), g->coneQuality());
				break;
				}
			}
	g2->setStyle(g->coordStyle(),g->floorStyle(),g->plotStyle(),pt);
	g2->setGrid(g->grids());
	g2->setTitle(g->plotTitle(),g->titleColor(),g->titleFont());
	g2->setTransparency(g->transparency());
	g2->setDataColors(g->minDataColor(),g->maxDataColor());
	g2->setColors(g->meshColor(),g->axesColor(),g->numColor(),
				g->labelColor(), g->bgColor(),g->gridColor());
	g2->setAxesLabels(g->axesLabels());
	g2->setTicks(g->scaleTicks());
	g2->setTickLengths(g->axisTickLengths());
	g2->setOptions(g->isLegendOn(), g->resolution(),g->labelsDistance());
	g2->setNumbersFont(g->numbersFont());
	g2->setXAxisLabelFont(g->xAxisLabelFont());
	g2->setYAxisLabelFont(g->yAxisLabelFont());
	g2->setZAxisLabelFont(g->zAxisLabelFont());
	g2->setRotation(g->xRotation(),g->yRotation(),g->zRotation());
	g2->setZoom(g->zoom());
	g2->setScale(g->xScale(),g->yScale(),g->zScale());
	g2->setShift(g->xShift(),g->yShift(),g->zShift());
	g2->setMeshLineWidth(g->meshLineWidth());
	g2->update();
	customToolBars((QWidget*)g2);

	setListViewSize(caption, g2->sizeToString());
	return g2;
	}
else
	return 0;
}

MultiLayer* ApplicationWindow::copyGraph()
{
MultiLayer* plot2=0;
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (plot &&  plotWindows.contains(plot->caption())>0)
	{
	graphs++;
	QString caption="graph"+QString::number(graphs);
	while(plotWindows.contains(caption)>0)
		{
		graphs++;
		caption="graph"+QString::number(graphs);
		}
	plot2=multilayerPlot(caption);
	plot2->showNormal();
	plot2->resize(plot->size());
		
	QWidgetList *graphsList=plot->graphPtrs();
	for (int j=0;j<(int)graphsList->count();j++)
			{
			Graph* g=(Graph*)graphsList->at(j);
			Graph* g2=plot2->loadLayer(g->pos().x(), g->pos().y(), g->width(), g->height());
			g2->setIgnoreResizeEvents(true);
			clone(g, g2);
			g2->updateScale();
			plot2->connectLayer(g2);
			g2->setIgnoreResizeEvents(false);
			}

	setListViewSize(caption,plot2->sizeToString());
	}
return plot2;
}

QWidget* ApplicationWindow::copyWindow()
{
QWidget* w=0;
QWidget* g = (QWidget*)ws->activeWindow();
if (!g)
	{
	QMessageBox::critical(this,tr("QtiPlot - Duplicate window error"),
		tr("There are no windows available in this project!"));
	return w;
	}

QApplication::setOverrideCursor(waitCursor);

if (plotWindows.contains(g->caption())>0)
	w=copyGraph();
else if (tableWindows.contains(g->caption())>0)
	w=copyTable();
else if (plot3DWindows.contains(g->caption())>0)
	w=copySurfacePlot();

if (w)
	{
    if (plotWindows.contains(g->caption())>0)
 		((MultiLayer*)w)->updateTransparency();
	else
		w->resize(g->size());
	}

QApplication::restoreOverrideCursor();
return w;
}

void ApplicationWindow::undo()
{
if (!lastModified)
	return;

QApplication::setOverrideCursor(waitCursor);

QString caption=lastModified->caption();
WksheetWindow *t=0;
if (tableWindows.contains(caption)>0)
	{
	t= (WksheetWindow *)lastModified;
	t->setNewSpecifications();
	QString newCaption=t->oldCaption();

	if (newCaption != caption)
		{
		int id=tableWindows.findIndex(caption);
		tableWindows[id]=newCaption;
		updateCurvesDialogs(caption,newCaption);
		renameListViewItem(caption,newCaption);
		}

	t->restore(t->getSpecifications());

	edit->setItemEnabled (undoID,FALSE);
	btnUndo->setEnabled(FALSE);

	edit->setItemEnabled (redoID,TRUE);
	btnRedo->setEnabled(TRUE);
	}
QApplication::restoreOverrideCursor();
}

void ApplicationWindow::redo()
{
if (!lastModified)
	return;

QApplication::setOverrideCursor(waitCursor);
QString caption=lastModified->caption();
WksheetWindow *t;
if (tableWindows.contains(caption)>0)
	{
	t= (WksheetWindow *)lastModified;
	QString newCaption=t->newCaption();

	if (newCaption != caption && tableWindows.contains(caption)>0)
		{
		int id=tableWindows.findIndex(caption);
		tableWindows[id]=newCaption;
		updateCurvesDialogs(caption,newCaption);
		renameListViewItem(caption,newCaption);
		}
	t->restore(t->getNewSpecifications());

	edit->setItemEnabled (undoID,TRUE);
	btnUndo->setEnabled(TRUE);

	edit->setItemEnabled (redoID,FALSE);
	btnRedo->setEnabled(FALSE);
	}
QApplication::restoreOverrideCursor();
}

bool ApplicationWindow::hidden(QWidget* window)
{
for (int i=0;i<(int)hiddenWindows->count();i++)
	{
	if (hiddenWindows->at(i) == window)
		return TRUE;
	}

return FALSE;
}

bool ApplicationWindow::maximized(QWidget* window)
{
bool aux=FALSE;
int w=window->visibleRect().width();
int h=window->visibleRect().height();
int W=ws->width();
int H=ws->height();
if (w==W && h==H)
	aux=TRUE;
return aux;
}

bool ApplicationWindow::minimized(QWidget* window)
{
bool aux=FALSE;
if (window->visibleRect().isEmpty() && !hidden(window) )
	aux=TRUE;
return aux;
}

void ApplicationWindow::checkMinimizedWindows()
{
QWidgetList windows = ws->windowList(QWorkspace::StackingOrder);


for (int i=0;i<(int)windows.count();i++)
	{
	QString caption=windows.at(i)->caption();
	QListViewItem *it=lv->findItem ( caption,0, Qt::ExactMatch | Qt::CaseSensitive );

	bool min = minimized(windows.at(i));
	if (min)
		setListView(caption, "Minimized");
	else if (!min && it->text(2) == "Minimized")
		{
		if (maximized(windows.at(i)))
			setListView(caption, "Maximized");
		else
			setListView(caption, "Normal");
		}
	}
}

void ApplicationWindow::resizedWindow(QWidget* window)
{
QString view,caption=window->caption();

if (minimized(window))
	{
	view="Minimized";
	//return;
	}
else if (maximized(window))
	view="Maximized";
else if (hidden(window))
	view="Hidden";
else 
	view="Normal";

setListView(caption,view);
}

void ApplicationWindow::resizeActiveWindow()
{
QWidget *w=(QWidget *)ws->activeWindow();
if (!w)
	return;

imageDialog *id=new imageDialog(this,"imageDialog",TRUE,WStyle_Tool|WDestructiveClose);
connect (id,SIGNAL(options(int,int,int,int)),w->parentWidget(),SLOT(setGeometry(int,int,int,int)));

id->setCaption(tr("QtiPlot - Window Geometry"));
id->setOrigin(w->parentWidget()->pos());
id->setSize(w->parentWidget()->size());
id->showNormal();
id->setActiveWindow();
}

void ApplicationWindow::hideActiveWindow()
{
QWidget *w=(QWidget *)ws->activeWindow();
if (!w)
	return;

hiddenWindows->append(w);
setListView(w->caption(),"Hidden");
w->hide();
emit modified();
}

void ApplicationWindow::hideWindow(QWidget* window)
{
hiddenWindows->append(window);
window->hide();
setListView(window->caption(),"Hidden");
emit modified();
}

void ApplicationWindow::hideWindow()
{
QListViewItem *lbi=lv->currentItem();
QWidget *w=window(lbi);

hiddenWindows->append(w);
w->hide();
lbi->setText(2,"Hidden");
emit modified();
}

void ApplicationWindow::resizeWindow()
{
QListViewItem *lbi=lv->currentItem();
QWidget *w=window(lbi);

if (!w)
	return;

imageDialog *id=new imageDialog(this,"imageDialog",TRUE,WStyle_Tool|WDestructiveClose);
connect (id,SIGNAL(options(int,int,int,int)),w->parentWidget(),SLOT(setGeometry(int,int,int,int)));

id->setCaption(tr("QtiPlot - Window Geometry"));
id->setOrigin(w->parentWidget()->pos());
id->setSize(w->parentWidget()->size());
id->showNormal();
id->setActiveWindow();
}

void ApplicationWindow::activateWindow()
{
QListViewItem *lbi=lv->currentItem();
QWidget *w=window(lbi);
w->showNormal();
w->setFocus();
lbi->setText(2,"Normal");

if (hidden(w))
	hiddenWindows->take(hiddenWindows->find (w));

emit modified();
}

void ApplicationWindow::maximizeWindow()
{
QListViewItem *lbi=lv->currentItem();
QWidget *w=window(lbi);
w->showMaximized();
lbi->setText(2,"Maximized");

if (hidden(w))
	hiddenWindows->take(hiddenWindows->find (w));

emit modified();
}

void ApplicationWindow::minimizeWindow()
{
QListViewItem *lbi=lv->currentItem();
QWidget *w=window(lbi);
w->showMinimized();
lbi->setText(2,"Minimized");

if (hidden(w))
	hiddenWindows->take(hiddenWindows->find (w));

emit modified();
}

void ApplicationWindow::closeActiveWindow()
{
QWidget *w=(QWidget *)ws->activeWindow();
if (w)
	w->close();
}

void ApplicationWindow::closeWindow(QWidget* window)
{
QWidgetList windows = ws->windowList();
int i,n;

if (!window)
	return;
QString name,caption=window->caption();

WksheetWindow* m=(WksheetWindow*)window;
if (m && tableWindows.contains(caption)>0)
	{
	n=m->tableCols();
	for (i=0;i<n;i++)
		{
		name=m->colName(i);
		removeColFromGraphs(name);
		}
	tableWindows.remove(caption);
	if (window == lastModified)
		{
		edit->setItemEnabled (undoID, FALSE);
		btnUndo->setEnabled(FALSE);

		edit->setItemEnabled (redoID, FALSE);
		btnRedo->setEnabled(FALSE);
		}
	}

else if (plotWindows.contains(window->caption())>0)
	plotWindows.remove(caption);

else if (plot3DWindows.contains(window->caption())>0)
	plot3DWindows.remove(caption);

emit modified();
emit windowClosed(caption);
window->close(TRUE);
}

void ApplicationWindow::about()
{
QMessageBox::about(this,tr("About QtiPlot"),
				   tr("<h2> QtiPlot 0.6.4</h2>"
					  "<p><h3>Author: Ion Vasilief</h3>"
					  "<p><h3>Released: 23/07/2005</h3>"));
}

void ApplicationWindow::windowsMenuAboutToShow()
{
	windowsMenu->clear();
    windowsMenu->insertItem("&Cascade", ws, SLOT(cascade() ) );
    windowsMenu->insertItem("&Tile", ws, SLOT(tile() ) );
    windowsMenu->insertSeparator();
	windowsMenu->insertItem(QPixmap(next_xpm),"&Next",
			   ws, SLOT(activateNextWindow()), Key_F5);
	windowsMenu->insertItem(QPixmap(prev_xpm),"&Previous",
			   ws, SLOT(activatePrevWindow()), Key_F6);
	windowsMenu->insertSeparator();
	windowsMenu->insertItem("&Rename", this, SLOT(rename()));
	windowsMenu->insertItem(QPixmap(duplicate_xpm),"&Duplicate",
			   this, SLOT(copyWindow()));
	windowsMenu->insertSeparator();
	windowsMenu->insertItem("Window &Geometry...",
			   this, SLOT(resizeActiveWindow()));
	windowsMenu->insertItem("&Hide Window",
			   this, SLOT(hideActiveWindow()));
 	windowsMenu->insertItem(QPixmap(close_xpm), "Close &Window",
			   this, SLOT(closeActiveWindow()), CTRL+Key_W );
	
	QWidgetList windows = ws->windowList();
	int n=int(windows.count());
	if (n == 0 || n == (int)hiddenWindows->count())
		{
		for (int i=0; i<12; i++)
			windowsMenu->setItemEnabled (windowsMenu->idAt(i),false);
		}

	if (n>0 && n<10)
		{
		windowsMenu->insertSeparator();
		for (int i = 0; i<n; ++i )
			{
			int id = windowsMenu->insertItem(windows.at(i)->caption(),
					 this, SLOT( windowsMenuActivated( int ) ) );
			windowsMenu->setItemParameter( id, i );
			windowsMenu->setItemChecked( id, ws->activeWindow() == windows.at(i) );
			}
		}
	else if (n>=10)
		{
		windowsMenu->insertSeparator();
		for ( int i = 0; i<9; ++i )
			{
			int id = windowsMenu->insertItem(windows.at(i)->caption(),
					 this, SLOT( windowsMenuActivated( int ) ) );
			windowsMenu->setItemParameter( id, i );
			windowsMenu->setItemChecked( id, ws->activeWindow() == windows.at(i) );
			}
		windowsMenu->insertSeparator();
		windowsMenu->insertItem("More windows...",this, SLOT(showMoreWindows()));
		}
}

void ApplicationWindow::showMarkerPopupMenu()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();

QPopupMenu markerMenu(this);

if (g->imageMarkerSelected())
	{
	markerMenu.insertItem(QPixmap(pixelProfile_xpm),"&View pixel line profile",this, SLOT(pixelLineProfile()));
	markerMenu.insertItem("&Intensity table",this, SLOT(intensityTable()));
	markerMenu.insertSeparator();
	}

markerMenu.insertItem(QPixmap(cut_xpm),"&Cut",this, SLOT(cutSelection()));
markerMenu.insertItem(QPixmap(copy_xpm), "&Copy",this, SLOT(copySelection()));
markerMenu.insertItem(QPixmap(erase_xpm), "&Delete",this, SLOT(clearSelection()));
markerMenu.insertSeparator();
if (g->arrowMarkerSelected())
	markerMenu.insertItem("&Properties",this, SLOT(showLineDialog()));
else if (g->imageMarkerSelected())
	markerMenu.insertItem("&Properties",this, SLOT(showImageDialog()));
else
	markerMenu.insertItem("&Properties",this, SLOT(showTextDialog()));

markerMenu.exec(QCursor::pos());
}

void ApplicationWindow::showMoreWindows()
{
	if (listViewOn)
		QMessageBox::information(this, "QtiPlot","Please use the project explorer to select a window!");
	else
		showList();
}

void ApplicationWindow::windowsMenuActivated( int id )
{
QWidgetList windows = ws->windowList();
QWidget* w = windows.at( id );
    if ( w )
	{
	w->showNormal();
	w->setFocus();
	if(hidden(w))
		{
		hiddenWindows->take(hiddenWindows->find (w));
		setListView(w->caption(),"Normal");
		}
    }
}

void ApplicationWindow::newProject()
{
	ApplicationWindow *ed = new ApplicationWindow;
	ed->setCaption("QtiPlot-untitled");

	if (this->isMaximized())
		ed->showMaximized();
	else
		ed->show();

	this->close();
}

void ApplicationWindow::modifiedProject()
{
fileSave->setEnabled(TRUE);
saved=FALSE;
}

void ApplicationWindow::modifiedProject(QWidget *w)
{
fileSave->setEnabled(TRUE);
saved=FALSE;

btnUndo->setEnabled(TRUE);
edit->setItemEnabled (undoID,TRUE);

lastModified=w;
}

void ApplicationWindow::timerEvent ( QTimerEvent *e)
{
if (e->timerId() == timerId)
	checkMinimizedWindows();
else if (e->timerId() == savingTimerId)
	saveProject();
else
	QWidget::timerEvent(e);
}

void ApplicationWindow::showEvent ( QShowEvent *)
{
timerId= startTimer(100);
}

void ApplicationWindow::hideEvent ( QHideEvent *)
{
killTimer(timerId);
}

void ApplicationWindow::closeEvent( QCloseEvent* ce )
{
	if (projectname.isEmpty()) projectname="untitled";
	if (!saved)
	{
	QString s="Save changes to project: ";
	s+="<p><b>'"+projectname+"'</b> ?";
    switch( QMessageBox::information(this,"QtiPlot",s,"Yes", "No", "Cancel", 0, 1 ) )
	{
    case 0:
	saveProject();
	saveSettings();

	if (fd) // non-linear fit dialog opened
		fd->close();

	ce->accept();
	break;
    case 1:
    default:
	saveSettings();

	if (fd) // non-linear fit dialog opened
		fd->close();

	ce->accept();
	break;
    case 2:
	ce->ignore();
	break;
    }
	}
	else
		{
		saveSettings();
		if (fd) // non-linear fit dialog opened
			fd->close();
		ce->accept();
		}
}

void ApplicationWindow::showWindowPopupMenu(QListViewItem *it, const QPoint &p, int)
{
QWidget *w = window(it);
if (w)
	{
	QPopupMenu cm(this);
	QPopupMenu plots(this);
	
	cm.insertItem("&Activate Window", this, SLOT(activateWindow()));
	cm.insertItem("Mi&nimize Window", this, SLOT(minimizeWindow()));
	cm.insertItem("Ma&ximize Window", this, SLOT(maximizeWindow()));
	cm.insertSeparator();
	cm.insertItem("&Hide Window", this, SLOT(hideWindow()));
	cm.insertItem("&Delete Window", w, SLOT(close()));
	cm.insertSeparator();
	cm.insertItem("&Rename Window", this, SLOT(renameWindow()));
	cm.insertItem("Re&size Window...", this, SLOT(resizeWindow()));
	cm.insertSeparator();
	cm.insertItem(QPixmap(fileprint_xpm),"&Print Window", this, SLOT(printWindow()));
	if (tableWindows.contains(w->caption())>0)
		{
		QStringList graphs=dependingPlots(w->caption());
		if (int(graphs.count())>0)
			{
			cm.insertSeparator();
			for (int i=0;i<int(graphs.count());i++)
				{
				if (plotWindows.contains(graphs[i])>0)
					plots.insertItem(graphs[i],plot(graphs[i]), SLOT(showNormal()));
				else
					plots.insertItem(graphs[i],surfacePlot(graphs[i]), SLOT(showNormal()));
				}
			cm.insertItem("D&ependents",&plots);
			}
		}
	cm.exec(p);
	}
}

QStringList ApplicationWindow::dependingPlots(const QString& caption)
{
QWidgetList windows = ws->windowList();
int c=int(windows.count());
QStringList onPlot, plots;

for (int i=0;i<c;i++)
	{
	if (plotWindows.contains(windows.at(i)->caption())>0)
		{
		MultiLayer *g=(MultiLayer*)windows.at(i);

		QWidgetList *graphsList=g->graphPtrs();
		for (int j=0;j<(int)graphsList->count();j++)
			{
			Graph* ag=(Graph*)graphsList->at(j);
			onPlot=ag->plotCurvesList();
			onPlot=onPlot.grep (caption,TRUE);
			if (int(onPlot.count())>0 && plots.contains(g->caption())<=0)
		  		 plots<<g->caption();
			}
		}
	else if (plot3DWindows.contains(windows.at(i)->caption())>0)
		{
			Graph3D *g=(Graph3D*)windows.at(i);
			if ((g->formula()).contains(caption,TRUE)>0 && plots.contains(g->caption())<=0)
		  		 plots<<g->caption();
		}
	}
return plots;
}

void ApplicationWindow::showGraphContextMenu()
{
QWidget* w = (QWidget*)ws->activeWindow();
if (!w)
	return;

if (plotWindows.contains(w->caption())>0)
	{
	MultiLayer *plot=(MultiLayer*)w;
	QPopupMenu cm(this);
	QPopupMenu exports(this);
	QPopupMenu copy(this);
	QPopupMenu prints(this);	
	QPopupMenu calcul(this);
	QPopupMenu fft(this);
	QPopupMenu inter(this);
	QPopupMenu decay(this);
		
	Graph* ag = (Graph*)plot->activeGraph();

	if (ag->isPiePlot())
		cm.insertItem("Re&move pie curve",ag, SLOT(removePie()));
	else
		{
		cm.insertItem(QPixmap(curves_xpm),"&Add/Remove curve",plot, SIGNAL(showCurvesDialog()));
		
		calcul.insertItem( "&Integrate ...", this, SLOT(showIntDialog()));
		inter.insertItem( "Li&near", this, SLOT(interlinear()));
		inter.insertItem( "Cu&bic spline", this, SLOT(intercubic()));
		inter.insertItem( "Non-&rounded Akime spline", this, SLOT(akima()));
		calcul.insertItem( "Inter&polate", &inter);
		calcul.insertSeparator();
		fft.insertItem("&Forward",this,SLOT(fft_for()));
		fft.insertItem("&Inverse",this,SLOT(fft_inv()));
		calcul.insertItem("&FFT", &fft);
		calcul.insertSeparator();
		calcul.insertItem( "&Differentiate", this, SLOT(differentiate()));
		calcul.insertSeparator();
    	calcul.insertItem( "Fit &Linear", this, SLOT(fitLinear()));
		calcul.insertItem( "Fit &Polynomial ...", this, SLOT(showFitPolynomDialog()));
		calcul.insertSeparator();
		decay.insertItem( "&First Order ...", this, SLOT(showExpDecayDialog()));
		decay.insertItem( "&Second Order ...", this, SLOT(showTwoExpDecayDialog()));
		decay.insertItem( "&Third Order ...", this, SLOT(showExpDecay3Dialog()));
		calcul.insertItem( "Fit E&xponential Decay",  &decay);
		calcul.insertItem( "Fit Exponential Gro&wth", this, SLOT(fitExpGrowth()));
		calcul.insertItem( "Fit &Gaussian",this, SLOT(fitGauss()));
		calcul.insertItem( "Fit Lorent&zian",this, SLOT(fitLorentz()));
		calcul.insertSeparator();
		calcul.insertItem( "&Non-linear Curve Fit ...",this, SLOT(showFitDialog()),CTRL+Key_Y);
		cm.insertItem("Anal&yse curve", &calcul);
		}
	
	if (copiedLayer)
		{
		cm.insertSeparator();
		cm.insertItem(QPixmap(paste_xpm),"&Paste layer",this, SLOT(pasteSelection()));
		}
	else if (copiedMarkerType >=0 )
		{
		cm.insertSeparator();
		if (copiedMarkerType == Graph::Text )
			cm.insertItem(QPixmap(paste_xpm),"&Paste text",plot, SIGNAL(pasteMarker()));
		else if (copiedMarkerType == Graph::Arrow )
			cm.insertItem(QPixmap(paste_xpm),"&Paste line/arrow",plot, SIGNAL(pasteMarker()));
		else if (copiedMarkerType == Graph::Image )
			cm.insertItem(QPixmap(paste_xpm),"&Paste image",plot, SIGNAL(pasteMarker()));
		}
	cm.insertSeparator();
	copy.insertItem("&Layer", this, SLOT(copyActiveLayer()));
	copy.insertItem("&Window",plot, SLOT(copyAllLayers()));
	cm.insertItem(QPixmap(copy_xpm),"&Copy", &copy);

	exports.insertItem("&Layer", plot, SLOT(exportActiveLayer()));
	exports.insertItem("&Window",plot, SLOT(exportPage()));
	cm.insertItem("E&xport",&exports);

	prints.insertItem("&Layer", plot, SLOT(printActiveLayer()));
	prints.insertItem("&Window",plot, SLOT(printAllLayers()));
	cm.insertItem(QPixmap(fileprint_xpm),"&Print",&prints);
	cm.insertSeparator();
	cm.insertItem("Layer &geometry", plot, SIGNAL(showGeometryDialog()));
	cm.insertSeparator();
	cm.insertItem(QPixmap(close_xpm), "&Delete Layer", plot, SLOT(confirmRemoveLayer()));
	cm.exec(QCursor::pos());
	}
}

void ApplicationWindow::showWindowContextMenu()
{
QWidget* w = (QWidget*)ws->activeWindow();
if (!w)
	return;

QPopupMenu cm(this);
if (plotWindows.contains(w->caption())>0)
	{
	MultiLayer *g=(MultiLayer*)w;

	if (copiedLayer)
		{
		cm.insertItem(QPixmap(paste_xpm),"&Paste Layer",this, SLOT(pasteSelection()));
		cm.insertSeparator();
		}
	cm.insertItem(QPixmap(newLayer_xpm),"&New Layer", this, SLOT(addLayer()));
	cm.insertSeparator();
	if (g->graphsNumber() != 0)
		{
		cm.insertItem(QPixmap(erase_xpm),"Dele&te Layer", g, SLOT(confirmRemoveLayer()));
		cm.insertSeparator();
		cm.insertItem("&Layer geometry", this, SLOT(showPlotGeometryDialog()));
		cm.insertItem(QPixmap(arrangeLayers_xpm),"&Arrange/Custom layers", this, SLOT(showLayerDialog()));
		cm.insertSeparator();
		}
	cm.insertItem("&Rename Window", this, SLOT(rename()));
	cm.insertItem(QPixmap(duplicate_xpm),"&Duplicate", this, SLOT(copyWindow()));
	cm.insertSeparator();
	cm.insertItem(QPixmap(copy_xpm),"&Copy Page", g, SLOT(copyAllLayers()));
	cm.insertItem("E&xport Page",g, SLOT(exportPage()));
	cm.insertItem(QPixmap(fileprint_xpm ),"&Print Page", this, SLOT(print()));
	cm.insertSeparator();
	cm.insertItem(QPixmap(close_xpm),"Close &Window", w, SLOT(close()));
	}
else if (tableWindows.contains(w->caption())>0)
	{
	WksheetWindow *t=(WksheetWindow *)w;
	if (! t->singleCellSelected())
		{
		cm.insertItem(QPixmap(cut_xpm),"Cu&t", w, SLOT(cutSelection()));
		cm.insertItem(QPixmap(copy_xpm),"&Copy", w, SLOT(copySelection()));
		cm.insertItem(QPixmap(paste_xpm),"&Paste", w, SLOT(pasteSelection()));
		cm.insertSeparator();
		cm.insertItem(QPixmap(erase_xpm),"Clea&r", w, SLOT(clearSelection()));
		}
	else
		{
		cm.insertItem("&Rename Window", this, SLOT(rename()));
		cm.insertItem(QPixmap(duplicate_xpm),"&Duplicate", this, SLOT(copyWindow()));
		cm.insertSeparator();
		cm.insertItem("&Export ASCII", this, SLOT(showExportASCIIDialog()));
		cm.insertItem(QPixmap(fileprint_xpm ),"&Print", this, SLOT(print()));
		cm.insertSeparator();
		cm.insertItem(QPixmap(close_xpm),"Close &Window", w, SLOT(close()));
		}
	}
else if (plot3DWindows.contains(w->caption())>0)
	{
	Graph3D *g=(Graph3D*)w;
	if (g->plotStyle() != Qwt3D::NOPLOT)
		cm.insertItem("E&mpty 3D plot", this, SLOT(remove3DPlot()));
	else
		{
		if (g->userFunction())
			cm.insertItem("&Add 3D Function", this, SLOT(addSurfacePlot()));
		else
			cm.insertItem("&Add 3D Data", this, SLOT(add3DData()));
		}
	cm.insertSeparator();
	cm.insertItem("&Rename Window", this, SLOT(rename()));
	cm.insertItem(QPixmap(duplicate_xpm),"&Duplicate", this, SLOT(copyWindow()));
	cm.insertSeparator();
	cm.insertItem("&Copy Graph", g, SLOT(copyImage()));
	cm.insertItem("&Export", g, SLOT(saveImage()));
	cm.insertItem(QPixmap(fileprint_xpm ),"&Print", this, SLOT(print()));
	cm.insertSeparator();
	cm.insertItem(QPixmap(close_xpm),"Close &Window", w, SLOT(close()));
	}

cm.exec(QCursor::pos());
}

void ApplicationWindow::showHelp()
{
	QMainWindow *helpWindow= new QMainWindow(0, "browser",WDestructiveClose);
	browser=new QTextBrowser (helpWindow,"helpBrowse");
	helpWindow->setFocus();
	helpWindow->setCentralWidget(browser);

	QToolBar* toolbar = new QToolBar( helpWindow );
    helpWindow->addToolBar( toolbar, "Toolbar");
    QToolButton* button;

	button = new QToolButton(QPixmap(fileprint_xpm), tr("Print"), "", this, SLOT(printHelp()), toolbar );
    button = new QToolButton(QPixmap(back_xpm), tr("Backward"), "", browser, SLOT(backward()), toolbar );
    connect( browser, SIGNAL( backwardAvailable(bool) ), button, SLOT( setEnabled(bool) ) );
    button->setEnabled( FALSE );
	button = new QToolButton(QPixmap(forward_xpm), tr("Forward"), "", browser, SLOT(forward()), toolbar );
    connect( browser, SIGNAL( forwardAvailable(bool) ), button, SLOT( setEnabled(bool) ) );
    button->setEnabled( FALSE );
    button = new QToolButton(QPixmap(home_xpm), tr("Home"), "", browser, SLOT(home()), toolbar );

	QString s=QDir::currentDirPath();
    browser->mimeSourceFactory()->setFilePath(s);
    browser->setFrameStyle( QFrame::Panel | QFrame::Sunken );

	QFile helpFile(helpFilePath);
	if (!helpFile.exists())
		{
		QMessageBox::critical(this,"QtiPlot - Help File Not Found!",
			   "Please indicate the location of the help file!");
		QString fn = QFileDialog::getOpenFileName( QString::null,QString::null, this );
		if ( !fn.isEmpty() )
			{
			browser->setSource (fn);
			QFileInfo fi(fn);
			helpFilePath=fi.absFilePath();
			}
		}
	else
		browser->setSource (helpFilePath);

	helpWindow->setCaption("QtiPlot - Help Browser");
    helpWindow->showMaximized();
}

void ApplicationWindow::printHelp()
{
#ifndef QT_NO_PRINTER
    QPrinter printer( QPrinter::HighResolution );
    printer.setFullPage(TRUE);
    if ( printer.setup( this ) ) {
	QPainter p( &printer );
	if( !p.isActive() ) // starting printing failed
	    return;
	QPaintDeviceMetrics metrics(p.device());
	int dpiy = metrics.logicalDpiY();
	int margin = (int) ( (2/2.54)*dpiy ); // 2 cm margins
	QRect body( margin, margin, metrics.width() - 2*margin, metrics.height() - 2*margin );
	QSimpleRichText richText( browser->text(),
				  QFont(),
				  browser->context(),
				  browser->styleSheet(),
				  browser->mimeSourceFactory(),
				  body.height() );
	richText.setWidth( &p, body.width() );
	QRect view( body );
	int page = 1;
	do {
	    richText.draw( &p, body.left(), body.top(), view, colorGroup() );
	    view.moveBy( 0, body.height() );
	    p.translate( 0 , -body.height() );
	    p.drawText( view.right() - p.fontMetrics().width( QString::number(page) ),
			view.bottom() + p.fontMetrics().ascent() + 5, QString::number(page) );
	    if ( view.top()  >= richText.height() )
		break;
	    printer.newPage();
	    page++;
	} while (TRUE);
    }
#endif
}

void ApplicationWindow::showPlotWizard()
{
if (tableWindows.count()>0)
	{
	plotWizard* pw = new plotWizard(this,"plotWizard",TRUE,WStyle_Tool|WDestructiveClose);
	connect (pw,SIGNAL(plot(const QStringList&)),this,SLOT(multilayerPlot(const QStringList&)));
	connect (pw,SIGNAL(plot3D(const QString&)),this,SLOT(dataPlotXYZ(const QString&)));
	connect (pw,SIGNAL(plot3DRibbon(const QString&)),this,SLOT(dataPlot3D(const QString&)));

	pw->insertTablesList(tableWindows);
	QStringList cols=columnsListXY();
	pw->setColumnsList(cols);
	pw->changeColumnsList(tableWindows[0]);
	pw->show();
	pw->setActiveWindow();
	}
else
	QMessageBox::warning(this,"QtiPlot - Warning",
				tr("<h4>There are no tables available in this project.</h4>"
					  "<p><h4>Please create a table and try again!</h4>"));
}

void ApplicationWindow::addFunctionCurve()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
		return;

Graph* g = (Graph*)plot->activeGraph();
if ( g )
	{
	activeGraph=g;
	fDialog* fd= new fDialog(this,"fDialog",TRUE,WStyle_Tool|WDestructiveClose);
	connect (fd,SIGNAL(options(QString&,QStringList &,QStringList &,QValueList<double> &,QValueList<int> &)),
			this,SLOT(addFunctionCurve(QString&,QStringList &,QStringList &,QValueList<double> &,QValueList<int> &)));
	connect (fd,SIGNAL(clearFunctionsList()),this,SLOT(clearFunctionsList()));
	connect (fd,SIGNAL(clearParamFunctionsList()),this,SLOT(clearParamFunctionsList()));
	connect (fd,SIGNAL(clearPolarFunctionsList()),this,SLOT(clearPolarFunctionsList()));

	fd->insertFunctionsList(functions);
	fd->insertParamFunctionsList(xFunctions, yFunctions);
	fd->insertPolarFunctionsList(rFunctions, tetaFunctions);
	fd->show();
	fd->setActiveWindow();
	}
}

void ApplicationWindow::addFunctionCurve(QString& type,QStringList &formulas,QStringList &vars,QValueList<double> &ranges,QValueList<int> &points)
{
if (type == "Polar plot")
	{
	if (rFunctions.contains(formulas[0])<=0)
		rFunctions<<formulas[0];
	if (tetaFunctions.contains(formulas[1])<=0)
		tetaFunctions<<formulas[1];
	}
else if  (type == "Parametric plot")
	{
	if (xFunctions.contains(formulas[0])<=0)
		xFunctions<<formulas[0];
	if (yFunctions.contains(formulas[1])<=0)
		yFunctions<<formulas[1];
	}
else if (type == "Function")
	{
	if (functions.contains(formulas[0])<=0)
		functions<<formulas[0];
	}

activeGraph->addFunctionCurve(type,formulas,vars,ranges,points);
}

void ApplicationWindow::newFunctionPlot()
{
fDialog* fd= new fDialog(this,"fDialog",TRUE,WStyle_Tool|WDestructiveClose);
connect (fd,SIGNAL(options(QString&,QStringList &,QStringList &,QValueList<double> &,QValueList<int> &)),
		this,SLOT(newFunctionPlot(QString&,QStringList &,QStringList &,QValueList<double> &,QValueList<int> &)));
connect (fd,SIGNAL(clearFunctionsList()),this,SLOT(clearFunctionsList()));
connect (fd,SIGNAL(clearParamFunctionsList()),this,SLOT(clearParamFunctionsList()));
connect (fd,SIGNAL(clearPolarFunctionsList()),this,SLOT(clearPolarFunctionsList()));

fd->insertFunctionsList(functions);
fd->insertParamFunctionsList(xFunctions, yFunctions);
fd->insertPolarFunctionsList(rFunctions, tetaFunctions);
fd->show();
fd->setActiveWindow();
}
void ApplicationWindow::newFunctionPlot(QString& type,QStringList &formulas,QStringList &vars,QValueList<double> &ranges,QValueList<int> &points)
{
graphs++;
QString label="graph"+QString::number(graphs);
while(plotWindows.contains(label)>0 || plot3DWindows.contains(label)>0
	  || tableWindows.contains(label)>0)
	{
	graphs++;
	label="graph"+QString::number(graphs);
	}

MultiLayer* plot = multilayerPlot(label);
Graph* g=plot->loadLayer();
customGraph(g);
g->addFunctionCurve(type,formulas,vars,ranges,points);
g->newLegend(plotLegendFont, legendFrameStyle);

plot->connectLayer(g);
plot->showNormal();
setListViewSize(plot->caption(),plot->sizeToString());

for(int i=0;i<(int)formulas.size();i++)
	{
	if (functions.contains(formulas[i])<=0)
		functions<<formulas[i];
	}
}

void ApplicationWindow::clearLogInfo()
{
if (!logInfo.isEmpty())
	{
	logInfo="";
	results->setText(logInfo);
	emit modified();
	}
}

void ApplicationWindow::clearParamFunctionsList()
{
xFunctions.clear();
yFunctions.clear();
}

void ApplicationWindow::clearPolarFunctionsList()
{
rFunctions.clear();
tetaFunctions.clear();
}

void ApplicationWindow::clearFunctionsList()
{
functions.clear();
}

void ApplicationWindow::clearFitFunctionsList()
{
fitFunctions.clear();
}

void ApplicationWindow::clearSurfaceFunctionsList()
{
surfaceFunc.clear();
}

void ApplicationWindow::setFramed3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setFramed();
	format->setItemEnabled (axesID,TRUE);
	}
}

void ApplicationWindow::setBoxed3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setBoxed();
	format->setItemEnabled (axesID,TRUE);
	}
}

void ApplicationWindow::removeAxes3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setNoAxes();
	format->setItemEnabled (axesID,FALSE);
	}
}

void ApplicationWindow::removeGrid3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setNoGrid();
	}
}

void ApplicationWindow::setHiddenLineGrid3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setHiddenLineGrid();
	}
}

void ApplicationWindow::setPoints3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setPointsMesh();
	}
}

void ApplicationWindow::setCones3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setConesMesh();
	}
}

void ApplicationWindow::setCrosses3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setCrossMesh();
	}
}

void ApplicationWindow::setBars3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setBarsPlot();
	}
}

void ApplicationWindow::setLineGrid3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setLineGrid();
	}
}

void ApplicationWindow::setFilledMesh3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setFilledMesh();
	}
}

void ApplicationWindow::setFloorData3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setFloorData();
	}
}

void ApplicationWindow::setFloorIso3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setFloorIsolines();
	}
}

void ApplicationWindow::setEmptyFloor3DPlot()
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setEmptyFloor();
	}
}

void ApplicationWindow::setFrontGrid3DPlot(bool on)
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setFrontGrid(on);

	}
}

void ApplicationWindow::setBackGrid3DPlot(bool on)
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setBackGrid(on);
	}
}

void ApplicationWindow::setFloorGrid3DPlot(bool on)
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setFloorGrid(on);
	}
}

void ApplicationWindow::setCeilGrid3DPlot(bool on)
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setCeilGrid(on);
	}
}

void ApplicationWindow::setRightGrid3DPlot(bool on)
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setRightGrid(on);
	}
}

void ApplicationWindow::setLeftGrid3DPlot(bool on)
{
QWidget* g = (QWidget*)ws->activeWindow();
if (g &&  plot3DWindows.contains(g->caption())>0)
	{
	Graph3D* plot= (Graph3D*)g;
	plot->setLeftGrid(on);
	}
}

void ApplicationWindow::pickPlotStyle( QAction* action )
{
	if (!action )
		return;

	if (action == polygon)
	{
		removeGrid3DPlot();
	}
	else if (action == filledmesh)
	{
		setFilledMesh3DPlot();
	}
	else if (action == wireframe)
	{
		setLineGrid3DPlot();
	}
	else if (action == hiddenline)
	{
		setHiddenLineGrid3DPlot();
	}
	else if (action == pointstyle)
	{
		setPoints3DPlot();
	}
	else if (action == conestyle)
	{
		setCones3DPlot();
	}
	else if (action == crossHairStyle)
	{
		setCrosses3DPlot();
	}
	else if (action == barstyle)
	{
		setBars3DPlot();
	}
	emit modified();
}


void ApplicationWindow::pickCoordSystem( QAction* action)
{
	if (!action)
		return;

	if (action == Box || action == Frame)
	{
		if (action == Box)
			setBoxed3DPlot();
		if (action == Frame)
			setFramed3DPlot();
		grids->setEnabled(true);
	}
	else if (action == None)
	{
		removeAxes3DPlot();
		grids->setEnabled(false);
	}

	emit modified();
}

void ApplicationWindow::pickFloorStyle( QAction* action )
{
	if (!action)
		return;

	if (action == floordata)
	{
	setFloorData3DPlot();
	}
	else if (action == flooriso)
	{
	setFloorIso3DPlot();
	}
	else
	{
	setEmptyFloor3DPlot();
	}

	emit modified();
}

void ApplicationWindow::custom3DActions(QWidget *w)
{
	if (w &&  plot3DWindows.contains(w->caption())>0)
	{
	Graph3D* plot= (Graph3D*)w;
	if (plot->plotStyle() == Qwt3D::NOPLOT)
		{
		plot3DTools->setDisabled(TRUE);
		return;
		}

	switch(plot->plotStyle())
		{
		case FILLEDMESH:
			wireframe->setOn( FALSE );
			hiddenline->setOn( FALSE );
			polygon->setOn( FALSE );
			filledmesh->setOn( TRUE );
			pointstyle->setOn( FALSE );
			barstyle->setOn( FALSE );
			conestyle->setOn( FALSE );
			crossHairStyle->setOn( FALSE );
		break;

		case FILLED:
			wireframe->setOn( FALSE );
			hiddenline->setOn( FALSE );
			polygon->setOn( TRUE );
			filledmesh->setOn( FALSE );
			pointstyle->setOn( FALSE );
			barstyle->setOn( FALSE );
			conestyle->setOn( FALSE );
			crossHairStyle->setOn( FALSE );
		break;

		case Qwt3D::USER:
			wireframe->setOn( FALSE );
			hiddenline->setOn( FALSE );
			polygon->setOn( FALSE );
			filledmesh->setOn( FALSE );

			if (plot->pointType() == Graph3D::VerticalBars)
				{
				pointstyle->setOn( FALSE );
				conestyle->setOn( FALSE );
				crossHairStyle->setOn( FALSE );
				barstyle->setOn( TRUE );
				}
			else if (plot->pointType() == Graph3D::Dots)
				{
				pointstyle->setOn( TRUE );
				barstyle->setOn( FALSE );
				conestyle->setOn( FALSE );
				crossHairStyle->setOn( FALSE );
				}
			else if (plot->pointType() == Graph3D::HairCross)
				{
				pointstyle->setOn( FALSE );
				barstyle->setOn( FALSE );
				conestyle->setOn( FALSE );
				crossHairStyle->setOn( true );
				}
			else if (plot->pointType() == Graph3D::Cones)
				{
				pointstyle->setOn( FALSE );
				barstyle->setOn( FALSE );
				conestyle->setOn( true );
				crossHairStyle->setOn( FALSE );
				}
		break;

		case WIREFRAME:
			wireframe->setOn( TRUE );
			hiddenline->setOn( FALSE );
			polygon->setOn( FALSE );
			filledmesh->setOn( FALSE );
			pointstyle->setOn( FALSE );
			barstyle->setOn( FALSE );
			conestyle->setOn( FALSE );
			crossHairStyle->setOn( FALSE );
		break;

		case HIDDENLINE:
			wireframe->setOn( FALSE );
			hiddenline->setOn( TRUE );
			polygon->setOn( FALSE );
			filledmesh->setOn( FALSE );
			pointstyle->setOn( FALSE );
			barstyle->setOn( FALSE );
			conestyle->setOn( FALSE );
			crossHairStyle->setOn( FALSE );
		break;

		default:
			;
		}

	switch(plot->coordStyle())
		{
		case Qwt3D::NOCOORD:
			None->setOn( TRUE );
			Box->setOn( FALSE );
			Frame->setOn( FALSE );
		break;

		case Qwt3D::BOX:
			None->setOn( FALSE );
			Box->setOn( TRUE );
			Frame->setOn( FALSE );
		break;

		case Qwt3D::FRAME:
			None->setOn(FALSE );
			Box->setOn( FALSE );
			Frame->setOn(TRUE );
		break;
		}

	switch(plot->floorStyle())
		{
		case NOFLOOR:
			floornone->setOn( TRUE );
			flooriso->setOn( FALSE );
			floordata->setOn( FALSE );
		break;

		case FLOORISO:
			floornone->setOn( FALSE );
			flooriso->setOn( TRUE );
			floordata->setOn( FALSE );
		break;

		case FLOORDATA:
			floornone->setOn(FALSE );
			flooriso->setOn( FALSE );
			floordata->setOn(TRUE );
		break;
		}
	custom3DGrids(plot->grids());
	}
}

void ApplicationWindow::custom3DGrids(int grids)
{
	if (Qwt3D::BACK & grids)
		back->setOn(TRUE);
	else
		back->setOn(FALSE);

	if (Qwt3D::FRONT & grids)
		front->setOn(TRUE);
	else
		front->setOn(FALSE);

	if (Qwt3D::CEIL & grids)
		ceil->setOn(TRUE);
	else
		ceil->setOn(FALSE);

	if (Qwt3D::FLOOR & grids)
		floor->setOn(TRUE);
	else
		floor->setOn(FALSE);

	if (Qwt3D::RIGHT & grids)
		right->setOn(TRUE);
	else
		right->setOn(FALSE);

	if (Qwt3D::LEFT & grids)
		left->setOn(TRUE);
	else
		left->setOn(FALSE);
}

void ApplicationWindow::initPlot3DToolBar()
{
	plot3DTools = new QToolBar( this, "plot3d operations" );
    addToolBar( plot3DTools, tr( "Surface 3D" ),Top, FALSE );

	coord = new QActionGroup( this, "coord" );
    coord->setUsesDropDown( FALSE );
    Box = new QAction( coord, "Box" );
    Box->setToggleAction( TRUE );
    Box->setIconSet(QPixmap(box_xpm));
	Box->setOn( TRUE );

    Frame = new QAction( coord, "Frame" );
    Frame->setToggleAction( TRUE );
    Frame->setIconSet(QPixmap(free_axes_xpm) );
    None = new QAction( coord, "None" );
    None->setToggleAction( TRUE );
    None->setIconSet( QPixmap(no_axes_xpm) );

	coord->setText( tr( "Coordinates" ) );
    coord->setMenuText( tr( "&Coord" ) );
    coord->setStatusTip( tr( "Coordinates" ) );
    Box->setText( tr( "Box" ) );
    Box->setMenuText( tr( "Box" ) );
    Box->setToolTip( tr( "Box" ) );
    Box->setStatusTip( tr( "Box" ) );
    Box->setAccel( tr( "A" ) );
    Frame->setText( tr( "Frame" ) );
    Frame->setMenuText( tr( "&Frame" ) );
    Frame->setToolTip( tr( "Frame" ) );
    Frame->setStatusTip( tr( "Frame" ) );
    Frame->setAccel( tr( "F" ) );
    None->setText( tr( "No Axes" ) );
    None->setMenuText( tr( "No Axes" ) );
    None->setToolTip( tr( "No Axes" ) );
    None->setStatusTip( tr( "No Axes" ) );

	Frame->addTo( plot3DTools );
    Box->addTo( plot3DTools );
    None->addTo( plot3DTools );

	plot3DTools->addSeparator();

	// grid actions
	grids = new QActionGroup( this, "grids" );
    grids->setEnabled( TRUE );
    grids->setExclusive( FALSE );
    grids->setUsesDropDown( FALSE );
    front = new QAction( grids, "front" );
    front->setToggleAction( TRUE );
    front->setIconSet(QPixmap(frontGrid_xpm) );
    back = new QAction( grids, "back" );
    back->setToggleAction( TRUE );
    back->setIconSet(QPixmap(backGrid_xpm));
    right = new QAction( grids, "right" );
    right->setToggleAction( TRUE );
    right->setIconSet(QPixmap(leftGrid_xpm) );
    left = new QAction( grids, "left" );
    left->setToggleAction( TRUE );
    left->setIconSet( QPixmap(rightGrid_xpm));
    ceil = new QAction( grids, "ceil" );
    ceil->setToggleAction( TRUE );
    ceil->setIconSet( QPixmap(ceilGrid_xpm) );
    floor = new QAction( grids, "floor" );
    floor->setToggleAction( TRUE );
    floor->setIconSet( QPixmap(floorGrid_xpm) );

	grids->setText( tr( "grid" ) );
    grids->setMenuText( tr( "grid" ) );
    grids->setStatusTip( tr( "grid" ) );
    front->setText( tr( "Action" ) );
    front->setMenuText( tr( "Action" ) );
    front->setToolTip( tr( "Front Grid" ) );
    front->setAccel( QString::null );
    back->setText( tr( "Action_2" ) );
    back->setMenuText( tr( "Action_2" ) );
    back->setToolTip( tr( "Back Grid" ) );
    right->setText( tr( "Action_3" ) );
    right->setMenuText( tr( "Action_3" ) );
    right->setToolTip( tr( "Right Grid" ) );
    left->setText( tr( "Action_4" ) );
    left->setMenuText( tr( "Action_4" ) );
    left->setToolTip( tr( "Left Grid" ) );
    ceil->setText( tr( "Action_5" ) );
    ceil->setMenuText( tr( "Action_5" ) );
    ceil->setToolTip( tr( "Ceiling Grid" ) );
    floor->setText( tr( "Action_6" ) );
    floor->setMenuText( tr( "Action_6" ) );
    floor->setToolTip( tr( "Floor Grid" ) );

	front->addTo(plot3DTools );
    back->addTo(plot3DTools );
    right->addTo(plot3DTools );
    left->addTo(plot3DTools );
    ceil->addTo(plot3DTools );
    floor->addTo(plot3DTools );

	plot3DTools->addSeparator();

	//plot style actions

	plotstyle = new QActionGroup( this, "plotstyle" );
    plotstyle->setUsesDropDown( FALSE );
    wireframe = new QAction( plotstyle, "wireframe" );
    wireframe->setToggleAction( TRUE );
    wireframe->setEnabled( TRUE );
    wireframe->setIconSet(QPixmap(lineMesh_xpm) );
    hiddenline = new QAction( plotstyle, "hiddenline" );
    hiddenline->setToggleAction( TRUE );
    hiddenline->setEnabled( TRUE );
    hiddenline->setIconSet(QPixmap(grid_only_xpm) );
    polygon = new QAction( plotstyle, "polygon" );
    polygon->setToggleAction( TRUE );
    polygon->setEnabled( TRUE );
    polygon->setIconSet(QPixmap(no_grid_xpm));
    filledmesh = new QAction( plotstyle, "filledmesh" );
    filledmesh->setToggleAction( TRUE );
    filledmesh->setOn( TRUE );
    filledmesh->setIconSet(QPixmap(grid_poly_xpm) );
    pointstyle = new QAction( plotstyle, "pointstyle" );
    pointstyle->setToggleAction( TRUE );
    pointstyle->setIconSet( QPixmap(pointsMesh_xpm) );

 	conestyle = new QAction( plotstyle, "conestyle" );
    conestyle->setToggleAction( TRUE );
    conestyle->setIconSet( QPixmap(cones_xpm) );

	crossHairStyle = new QAction( plotstyle, "crossHairStyle" );
    crossHairStyle->setToggleAction( TRUE );
    crossHairStyle->setIconSet( QPixmap(crosses_xpm) );

	barstyle = new QAction( plotstyle, "barstyle" );
    barstyle->setToggleAction( TRUE );
    barstyle->setIconSet( QPixmap(plot_bars_xpm) );

	plotstyle->setText( tr( "Plot Style" ) );
    plotstyle->setMenuText( tr( "Plot Style" ) );
    plotstyle->setStatusTip( tr( "Plot Style" ) );
    wireframe->setText( tr( "Wireframe" ) );
    wireframe->setMenuText( tr( "Wireframe" ) );
    wireframe->setToolTip( tr( "Wireframe" ) );
    wireframe->setStatusTip( tr( "Wireframe" ) );
    hiddenline->setText( tr( "Hidden Line" ) );
    hiddenline->setMenuText( tr( "Hidden Line" ) );
    hiddenline->setToolTip( tr( "Hidden Line" ) );
    hiddenline->setStatusTip( tr( "Hidden Line" ) );
    polygon->setText( tr( "Polygon only" ) );
    polygon->setMenuText( tr( "Polygon only" ) );
    polygon->setToolTip( tr( "Polygon only" ) );
    polygon->setStatusTip( tr( "Polygon only" ) );
    filledmesh->setText( tr( "Mesh & filled Polygons" ) );
    filledmesh->setMenuText( tr( "Mesh & filled Polygons" ) );
    filledmesh->setToolTip( tr( "Mesh & filled Polygons" ) );
    filledmesh->setStatusTip( tr( "Mesh & filled Polygons" ) );
	pointstyle->setText( tr( "Dots" ) );
    pointstyle->setMenuText( tr( "Dots" ) );
    pointstyle->setToolTip( tr( "Dots" ) );
    pointstyle->setStatusTip( tr( "Dots" ) );
	barstyle->setText( tr( "Bars" ) );
    barstyle->setMenuText( tr( "Bars" ) );
    barstyle->setToolTip( tr( "Bars" ) );
    barstyle->setStatusTip( tr( "Bars" ) );
	conestyle->setText( tr( "Cones" ) );
    conestyle->setMenuText( tr( "Cones" ) );
    conestyle->setToolTip( tr( "Cones" ) );
    conestyle->setStatusTip( tr( "Cones" ) );
	crossHairStyle->setText( tr( "Cross Hairs" ) );
    crossHairStyle->setMenuText( tr( "Cross Hairs" ) );
    crossHairStyle->setToolTip( tr( "Cross Hairs" ) );
    crossHairStyle->setStatusTip( tr( "Cross Hairs" ) );

	barstyle->addTo( plot3DTools );
	plot3DTools->addSeparator();
	pointstyle->addTo( plot3DTools );

	conestyle->addTo( plot3DTools );
	crossHairStyle->addTo( plot3DTools );
	plot3DTools->addSeparator();

    wireframe->addTo( plot3DTools );
    hiddenline->addTo( plot3DTools );
    polygon->addTo( plot3DTools );
    filledmesh->addTo( plot3DTools );

	plot3DTools->addSeparator();

	//floor actions

	floorstyle = new QActionGroup( this, "floorstyle" );
    floorstyle->setUsesDropDown( FALSE );
    floordata = new QAction( floorstyle, "floordata" );
    floordata->setToggleAction( TRUE );
    floordata->setIconSet(QPixmap(floor_xpm) );
    flooriso = new QAction( floorstyle, "flooriso" );
    flooriso->setToggleAction( TRUE );
    flooriso->setIconSet(QPixmap(isolines_xpm) );
    floornone = new QAction( floorstyle, "floornone" );
    floornone->setToggleAction( TRUE );
    floornone->setOn( TRUE );
    floornone->setIconSet(QPixmap(no_floor_xpm));

	floorstyle->setText( tr( "Floor Style" ) );
    floorstyle->setMenuText( tr( "Floor Style" ) );
    floorstyle->setStatusTip( tr( "Floor Style" ) );
    floordata->setText( tr( "Floor Data Projection" ) );
    floordata->setMenuText( tr( "Floor Data Projection" ) );
    floordata->setToolTip( tr( "Floor Data Projection" ) );
    floordata->setStatusTip( tr( "Floor Data Projection" ) );
    flooriso->setText( tr( "Floor Isolines" ) );
    flooriso->setMenuText( tr( "Floor Isolines" ) );
    flooriso->setToolTip( tr( "Floor Isolines" ) );
    flooriso->setStatusTip( tr( "Floor Isolines" ) );
    floornone->setText( tr( "Empty Floor" ) );
    floornone->setMenuText( tr( "Empty Floor" ) );
    floornone->setToolTip( tr( "Empty Floor" ) );
    floornone->setStatusTip( tr( "Empty Floor" ) );

	floordata->addTo( plot3DTools );
    flooriso->addTo( plot3DTools );
    floornone->addTo( plot3DTools );

	plot3DTools->hide();

connect( coord, SIGNAL( selected( QAction* ) ), this, SLOT( pickCoordSystem( QAction* ) ) );
connect( floorstyle, SIGNAL( selected( QAction* ) ), this, SLOT( pickFloorStyle( QAction* ) ) );
connect( plotstyle, SIGNAL( selected( QAction* ) ), this, SLOT( pickPlotStyle( QAction* ) ) );

connect( left, SIGNAL( toggled( bool ) ), this, SLOT( setLeftGrid3DPlot(bool) ));
connect( right, SIGNAL( toggled( bool ) ), this, SLOT( setRightGrid3DPlot( bool ) ) );
connect( ceil, SIGNAL( toggled( bool ) ), this, SLOT( setCeilGrid3DPlot( bool ) ) );
connect( floor, SIGNAL( toggled( bool ) ), this, SLOT(setFloorGrid3DPlot( bool ) ) );
connect( back, SIGNAL( toggled( bool ) ), this, SLOT(setBackGrid3DPlot( bool ) ) );
connect( front, SIGNAL( toggled( bool ) ), this, SLOT( setFrontGrid3DPlot( bool ) ) );
}

void ApplicationWindow::pixelLineProfile()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (g)
	{
	 bool ok;
    int res = QInputDialog::getInteger(
            "QtiPlot - Set number of pixels to average", "Number of average pixels",1, 1, 20, 2,
            &ok, this );
    if ( ok )
		{
		QApplication::setOverrideCursor(waitCursor);
		g->calculateProfile(res,TRUE);
		QApplication::restoreOverrideCursor();
		}
	else
       return;
	}
}

void ApplicationWindow::intensityTable()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (g)
	g->showIntensityTable();
}

WksheetWindow* ApplicationWindow::createIntensityTable(const QPixmap& pic)
{
QImage image=pic.convertToImage();
QSize size=pic.size();
int width=size.width();
int n=width*size.height();
if (n>1e6)
	{
	QMessageBox::about(this, tr("QtiPlot - Error"), "The image size is too big to be analysed (max size=1.000.000 pixels)!");
	return 0;
	}

QApplication::setOverrideCursor(waitCursor);
WksheetWindow* w = newWrksheet("table1", n, 3);
QRgb pixel;
int l;
for (int j=0;j<size.height();j++)
 	{
	for (int i=0;i<size.width();i++)
 		{
		l=width*j+i;
		w->setText (l, 0, QString::number(i));
		w->setText (l, 1, QString::number(j));
		pixel= image.pixel(i,j);
		w->setText (l , 2, QString::number(qGray(pixel)));
		}
	}
QStringList colList;
colList<<"x"<<"y"<<"intensity";
w->setHeader(colList);
w->show();

QApplication::restoreOverrideCursor();
return w;
}

void ApplicationWindow::clone(Graph* g, Graph* g2)
{
	int i,errorID=0;

		g2->enableAxes(g->enabledAxes());
		g2->setAxesColors(g->axesColors());
		g2->setEnabledTickLabels(g->enabledTickLabels());
		g2->setTicksType(g->ticksType());
		
		int axis=QwtPlot::xBottom;
		g2->setAxisLabelRotation(axis, g->labelsRotation(axis));
		axis=QwtPlot::xTop;
		g2->setAxisLabelRotation(axis, g->labelsRotation(axis));
		
		g2->setGridOptions(g->getGridOptions());
		g2->setTitle(g->title());
		g2->setTitleFont(g->titleFont());
		g2->setTitleColor(g->titleColor());
		g2->setTitleAlignment(g->titleAlignment());
		g2->drawCanvasFrame(g->framed(),g->canvasFrameWidth(),g->canvasFrameColor());

		QStringList axesTitles=g->scalesTitles();
		for (i=0;i<(int)axesTitles.count();i++)
			g2->setAxisTitle(i,axesTitles[i]);

		for (i=0;i<4;i++)
			g2->setAxisFont(i,g->axisFont(i));
		
		QStringList format=g->labelsFormat();
		format.prepend("Format");
		g2->setLabelsFormat(format);

		g2->setXAxisTitleColor(g->axisTitleColor(2));
		g2->setXAxisTitleFont(g->axisTitleFont(2));
		g2->setXAxisTitleAlignment(g->axisTitleAlignment(2));

		g2->setYAxisTitleColor(g->axisTitleColor(0));
		g2->setYAxisTitleFont(g->axisTitleFont(0));
		g2->setYAxisTitleAlignment(g->axisTitleAlignment(0));

		g2->setTopAxisTitleColor(g->axisTitleColor(3));
		g2->setTopAxisTitleFont(g->axisTitleFont(3));
		g2->setTopAxisTitleAlignment(g->axisTitleAlignment(3));

		g2->setRightAxisTitleColor(g->axisTitleColor(1));
		g2->setRightAxisTitleFont(g->axisTitleFont(1));
		g2->setRightAxisTitleAlignment(g->axisTitleAlignment(1));

		g2->removeLegend();
		
	if (g->isPiePlot())
		g2->plotPie(g->pieCurve());
	else
		{		
		int c=g->curves();
		QStringList ass=g->plotAssociations();
		QStringList el=g->erorsLayout();
		QArray<curveLayout> cvsLayout=g->allCurvesLayout();

		for (i=0;i<c;i++)
			{
			if (ass[i].contains("=")>0)
				{
				double from=g->curveXValue(i,0);
				double step=g->curveXValue(i,1)-from;

				g2->insertFunctionCurve(ass[i],from,step,g->curveDataSize(i));
				curveLayout cl=cvsLayout[i];
				g2->updateCurveLayout(i,cl);
				}
			else
				{
				QStringList cv=QStringList::split(",",ass[i],FALSE);

				int aux=cv[0].find("_",0);
				QString name=cv[0].left(aux);
				WksheetWindow* w=table(name);

				int pos=cv[0].find("(",0);
				QString xColName=cv[0].left(pos);
				int xCol=w->colIndex(xColName);

				pos=cv[1].find("(",0);
				QString yColName=cv[1].left(pos);

				if (ass[i].contains("(yErr)",TRUE)==0 && ass[i].contains("(xErr)",TRUE)==0)
					{
					curveLayout cl=cvsLayout[i];
					if (cl.plotType == Graph::VectXYXY)
						{
						QStringList colsList=cv.gres ("(X)","",TRUE );
						colsList=cv.gres ("(Y)","",TRUE );
						g2->plotVectXYXY(w, colsList);
						}
					else
						g2->plot(w,xCol,yColName,cl.plotType);
					
					g2->updateCurveLayout(i,cl);

					bool autoBinning = false;
					if (cl.autoBinning)
						autoBinning=true;

					if (cl.plotType == Graph::Histogram && !autoBinning)
						g2->updateHistogram(w,yColName,i, autoBinning,
												cl.binSize,cl.histBegin, cl.histEnd);
					}
				else
					{
					pos=cv[2].find("(",0);
					QString errColName=cv[2].left(pos);

					bool ok,plus=TRUE,minus=TRUE,through=TRUE,xErrors=FALSE;
					QString ebl=el[errorID];
					errorID++;
					QStringList curve=QStringList::split("\t",ebl,FALSE);

					int type=curve[0].toInt(&ok);
					if (type==1) xErrors=TRUE;

					int width=curve[1].toInt(&ok);
					int cap=curve[2].toInt(&ok);
					QColor color=QColor(curve[3]);

					int aux=curve[4].toInt(&ok);
					if (aux==0) through=FALSE;

					aux=curve[5].toInt(&ok);
					if (aux==0) plus=FALSE;

					aux=curve[6].toInt(&ok);
					if (aux==0) minus=FALSE;

					WksheetWindow *errTable=table(errColName);
					g2->addErrorBars(w,xErrors,xColName,yColName, errTable, errColName,
							width,cap,color,through,minus,plus);
					}
				}
			}
		}

	g2->definePlotLimits(g->plotLimits());
	//define top and right scales
	g2->setSecondaryPlotLimits(g->secondaryPlotLimits());
		
	QStringList tickLabelsColNames = g->axesLabelsDatasheets();
	QValueList<int> axesTypes = g->axesType();
	for (i=0; i<4; i++)
			{
			WksheetWindow *w = table(tickLabelsColNames[i]);
			g2->setAxisTickLabels(i, axesTypes[i], tickLabelsColNames[i], w);
			}


	QwtArray<long> imageMrkKeys=g->imageMarkerKeys();
	for (i=0;i<(int)imageMrkKeys.size();i++)
		{
		ImageMarker* imrk=(ImageMarker*)g->imageMarker(imageMrkKeys[i]);
		if (imrk)
			g2->insertImageMarker(imrk);
		}

	QwtArray<long> txtMrkKeys=g->textMarkerKeys();
	for (i=0;i<(int)txtMrkKeys.size();i++)
		{
		LegendMarker* mrk=(LegendMarker*)g->textMarker(txtMrkKeys[i]);
		if (mrk)
			g2->insertTextMarker(mrk);
		}

	QwtArray<long> lineMrkKeys=g->lineMarkerKeys();
	for (i=0;i<(int)lineMrkKeys.size();i++)
		{
		LineMarker* lmrk=(LineMarker*)g->lineMarker(lineMrkKeys[i]);
		if (lmrk)
			g2->insertLineMarker(lmrk);
		}
		
g2->replot();
}


void ApplicationWindow::addLayer()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (plot &&  plotWindows.contains(plot->caption())>0)
	{
	switch(QMessageBox::information(this,
            tr("QtiPlot - Guess best origin for the new layer?"),
            tr("Do you want QtiPlot to guess the best position for the new layer?\n Warning: this will rearrange existing layers!"),
            tr("&Guess"), tr("&Top-left corner"), tr("&Cancel"),
            0, 2 ) )
			{
			case 0:
				customGraph(plot->addLayer());
				plot->updateTransparency();
			break;

			case 1:
				customGraph(plot->addLayerToOrigin());
				plot->updateTransparency();
			break;

			case 2:
		   		return;
			break;
			}
	}
}

void ApplicationWindow::deleteLayer()
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (plot &&  plotWindows.contains(plot->caption())>0)
	plot->confirmRemoveLayer();
}

void ApplicationWindow::openTable(ApplicationWindow* app, const QStringList &flist)
{
QStringList list=QStringList::split ("\t",flist[0],TRUE);
QString caption=list[0];
QString rows=list[1];
QString cols=list[2];

WksheetWindow* w = app->newWrksheet(caption, rows.toInt(),cols.toInt());
w->setXCol(list[4].toInt());

if ((int)list.count()>=7)
	w->setYCol(list[6].toInt());//starting with 0.5.0 version

QString date=list[3];
app->setListViewDate(caption,date);

int tb=0;
if (caption.contains ("table",TRUE)>0)
			{
			tb=caption.mid(5,caption.length()-5).toInt();
			if (tb>maxt) maxt=tb;
			}
else if (caption.contains ("Fit",TRUE)>0)
			{
			int pos=caption.find("Fit",0);
			tb=caption.right(caption.length()-pos-3).toInt();
			if (tb>maxFit) maxFit=tb;
			}

QString s=flist[1];
if (s.contains ("minimized",TRUE)>0)
			{
			w->showMinimized();
			app->setListView(w->caption(),"Minimized");
			}
else if (s.contains ("maximized",TRUE)>0)
			{
			w->show();
			app->wmax=(QWidget*)w;
			}
else
			{
			 list=QStringList::split ("\t",s,TRUE);
			 int x=list[1].toInt();
			 int y=list[2].toInt();
			 int width=list[3].toInt();
			 int height=list[4].toInt();
			 w->showNormal();
			 w->parentWidget()->setGeometry(x,y,width,height);

			if (list[5]=="active")
					app->aw=(QWidget*)w;

			if (list[5]=="hidden")
					{
					app->hiddenWindows->append(w);
					w->hide();
					app->setListView(w->caption(),"Hidden");
					}
			else
					app->setListView(w->caption(),"Normal");
				}

s=flist[2].right(flist[2].length()-7);
list=QStringList::split ("\t",s,FALSE );
w->setHeader(list);

s=flist[3].right(flist[3].length()-9);
list=QStringList::split ("\t",s,FALSE );

w->setColWidths(list);
w->setCommandes(flist[4]);

//read and set table values
int c=cols.toInt();
for (int k=5;k<(int)flist.count()-1;k++)
	{
	list=QStringList::split("\t",flist[k],TRUE);
	int line=list[0].toInt();
	for (int i=0;i<c;i++)
			w->setText(line,i,list[i+1]);
	}
}

void ApplicationWindow::openGraph(ApplicationWindow* app, MultiLayer *plot,
																	const QStringList &list)
{
Graph* ag=0;
int curveID=0,xcol=0,i;
WksheetWindow* w=0;
QStringList fList,curve;
QString s,caption;
QArray<curveLayout> layout;
curveLayout cl;

int x,y,width,height;
for (int j=0;j<(int)list.count()-1;j++)
	{
	s=list[j];
	if (s.contains ("ggeometry",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			x=fList[1].toInt();
			y=fList[2].toInt();
			width=fList[3].toInt();
			height=fList[4].toInt();
			ag=(Graph*)plot->loadLayer(x,y,width,height);
			ag->setIgnoreResizeEvents(true);
			}
		else if (s.contains ("EnabledAxes",TRUE)>0)
			{
				fList=QStringList::split ("\t",s,TRUE);
				ag->enableAxes(fList);
			}
		else if (s.contains ("EnabledTicks",TRUE)>0)
			{
				fList=QStringList::split ("\t",s,TRUE);
				ag->setTicksType(fList);
			}
		else if (s.contains ("EnabledTickLabels",TRUE)>0)
			{
				fList=QStringList::split ("\t",s,TRUE);
				for (i=0;i<(int)fList.count();i++)
					fList[i]=fList[i+1];
				ag->setEnabledTickLabels(fList);
			}
		else if (s.contains ("AxesColors",TRUE)>0)
			{
				fList=QStringList::split ("\t",s,TRUE);
				for (i=0;i<(int)fList.count();i++)
					fList[i]=fList[i+1];
				ag->setAxesColors(fList);
			}
		else if (s.contains ("SecondaryAxes",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			for (i=0;i<(int)fList.count();i++)
				fList[i]=fList[i+1];
			ag->setSecondaryPlotLimits(fList);
			}
		else if (s.left(5)=="grid\t")
			{
				QStringList grid=QStringList::split ("\t",s,TRUE);
				gridOptions gr;
				gr.majorOnX=grid[1].toInt();
				gr.minorOnX=grid[2].toInt();
				gr.majorOnY=grid[3].toInt();
				gr.minorOnY=grid[4].toInt();
				gr.majorCol=grid[5].toInt();
				gr.majorStyle=grid[6].toInt();
				gr.majorWidth=grid[7].toInt();
				gr.minorCol=grid[8].toInt();
				gr.minorStyle=grid[9].toInt();
				gr.minorWidth=grid[10].toInt();
				gr.xZeroOn=grid[11].toInt();
				gr.yZeroOn=grid[12].toInt();

				ag->setGridOptions(gr);
			}
		else if (s.contains ("PieCurve",TRUE)>0)
			{
			curve=QStringList::split ("\t",s,TRUE);
			QString name=curve[1];
			int pos=name.find("_");

			caption=name.left(pos);

			QWidgetList windows = app->ws->windowList();
			int c=windows.count();
			for (i=0;i<c;i++)
				{
				w=(WksheetWindow *)windows.at(i);
				if (w && app->tableWindows.contains(w->caption())>0)
					{
					if (w->caption()==caption)
						{
						Qt::PenStyle style;
						if (curve[4]=="SolidLine")
								style=Qt::SolidLine;
						else if (curve[4]=="DashLine")
								style=Qt::DashLine;
						else if (curve[4]=="DotLine")
								style=Qt::DotLine;
						else if (curve[4]=="DashDotLine")
								style=Qt::DashDotLine;
						else if (curve[4]=="DashDotDotLine")
								style=Qt::DashDotDotLine;

						QPen pen=QPen(QColor(curve[3]),curve[2].toInt(),style);
						ag->plotPie(w,name,pen,curve[5].toInt(),curve[6].toInt(),curve[7].toInt());
						break;
						}
					}
				}
			}
		else if (s.left(6)=="curve\t")
			{
			curve=QStringList::split ("\t",s,TRUE);
			QString name=curve[2];
			int pos=name.find("_");
			caption=name.left(pos);
			xcol=curve[1].toInt();

			cl.plotType=curve[3].toInt();

			if (fileVersion <= 6)
				cl.connectType=curve[4].toInt()+1;
			else
				cl.connectType=curve[4].toInt();

			cl.lCol=curve[5].toInt();
			cl.lStyle=curve[6].toInt();
			cl.lWidth=curve[7].toInt();
			cl.sSize=curve[8].toInt();
			cl.sType=curve[9].toInt();
			cl.symCol=curve[10].toInt();
			cl.fillCol=curve[11].toInt();
			cl.filledArea=curve[12].toInt();
			cl.aCol=curve[13].toInt();
			cl.aStyle=curve[14].toInt();

			if(cl.plotType == Graph::VerticalBars || cl.plotType == Graph::HorizontalBars || cl.plotType == Graph::Histogram)
				cl.barGap=curve[15].toInt();

			if(cl.plotType == Graph::Histogram)
				{
				cl.autoBinning=curve[16].toInt();
				cl.binSize=curve[17].toDouble();
				cl.histBegin=curve[18].toDouble();
				cl.histEnd=curve[19].toDouble();
				}
				
			QStringList colsList;
			colsList<<name;
			if(cl.plotType == Graph::VectXYXY)
				{
				cl.vectColor=curve[15].toInt();
				cl.vectWidth=curve[16].toInt();
				cl.headLength=curve[17].toInt();
				cl.headAngle=curve[18].toInt();
				cl.filledHead=curve[19].toInt();
				colsList<<curve[20];
				colsList<<curve[21];
				}

			layout.resize(curveID+1);
			layout[curveID]=cl;

			QWidgetList windows = app->ws->windowList();
			int c=windows.count();
			for (i=0;i<c;i++)
				{
				w=(WksheetWindow *)windows.at(i);
				if (w && app->tableWindows.contains(w->caption())>0)
					{
					if (w->caption()==caption)
						{
						if(cl.plotType == Graph::VectXYXY)
							{
							colsList.prepend(w->colName(xcol));
							ag->plotVectXYXY(w, colsList);
							}
						else
							ag->plot(w,xcol,name,cl.plotType);
						ag->updateCurveLayout(curveID,cl);

						bool autoBinning = false;
						if (cl.autoBinning)
							autoBinning=true;

						if (cl.plotType == Graph::Histogram && !autoBinning)
							ag->updateHistogram(w,name,curveID, autoBinning,
												cl.binSize,cl.histBegin, cl.histEnd);

						long key=ag->curveKey(curveID);
						ag->setCurveLegend(name,key);
						}
					}
				}
			curveID++;
			}
		else if (s.contains ("FunctionCurve",TRUE)>0)
			{
			curve=QStringList::split ("\t",s,TRUE);
			QString name=curve[1];

			cl.plotType=curve[5].toInt();
			cl.connectType=curve[6].toInt();
			cl.lCol=curve[7].toInt();
			cl.lStyle=curve[8].toInt();
			cl.lWidth=curve[9].toInt();
			cl.sSize=curve[10].toInt();
			cl.sType=curve[11].toInt();
			cl.symCol=curve[12].toInt();
			cl.fillCol=curve[13].toInt();
			cl.filledArea=curve[14].toInt();
			cl.aCol=curve[15].toInt();
			cl.aStyle=curve[16].toInt();

			layout.resize(curveID+1);
			layout[curveID]=cl;

			ag->insertFunctionCurve(name, curve[3].toDouble(),curve[4].toDouble(),curve[2].toInt());
			ag->updateCurveLayout(curveID,cl);
			curveID++;
			}
		else if (s.contains ("ErrorBars",TRUE)>0)
			{
			curve=QStringList::split ("\t",s,FALSE);
			QString xColName=curve[2];
			QString yColName=curve[3];
			QString errColName=curve[4];

			bool plus=TRUE,minus=TRUE,through=TRUE,xErrors=FALSE;
			int type=curve[1].toInt();
			if (type==0) xErrors=TRUE;

			int width=curve[5].toInt();
			int cap=curve[6].toInt();
			QColor color=QColor(curve[7]);

			int aux=curve[8].toInt();
			if (aux==0) through=FALSE;

			aux=curve[9].toInt();
			if (aux==0) plus=FALSE;

			aux=curve[10].toInt();
			if (aux==0) minus=FALSE;

			w=app->table(yColName);
			WksheetWindow *errTable=app->table(errColName);
			ag->addErrorBars(w,xErrors,xColName,yColName,errTable, errColName,
							width,cap,color,through,minus,plus);
			}
		else if (s.left(6)=="scale\t")
			{
			QStringList scale=QStringList::split ("\t",s,TRUE);
			
			if (fileVersion < 6.4)
				{//insure backwards compatibility	
				QStringList newScale; 
				for (i=1; i<7; i++)
					newScale<<scale[i];
				
				QString sclType = scale[7];	
				if ( sclType == "2")
					{
					newScale<<"0";//linear scale
					newScale<<"1";//inverted scale
					}
				else if ( sclType == "3")
					{
					newScale<<"1";//log scale
					newScale<<"1";//inverted scale
					}
				else
					{
					newScale<<sclType;
					newScale<<"0";//not inverted scale
					}
					
				for (i=8; i<14; i++)
					newScale<<scale[i];	
				
				sclType = scale[14];	
				if ( sclType == "2")
					{
					newScale<<"0";//linear scale
					newScale<<"1";//inverted scale
					}
				else if ( sclType == "3")
					{
					newScale<<"1";//log scale
					newScale<<"1";//inverted scale
					}
				else
					{
					newScale<<sclType;
					newScale<<"0";//not inverted scale
					}
				ag->definePlotLimits(newScale);
				}
			else
				{
				for (i=0; i<(int)scale.count(); i++)
					scale[i]=scale[i+1];	
				
				ag->definePlotLimits(scale);
				}
			}
		else if (s.contains ("PlotTitle",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->setTitle(fList[1]);
			ag->setTitleColor(QColor(fList[2]));
			ag->setTitleAlignment(fList[3].toInt());
			}
		else if (s.contains ("TitleFont",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			QFont fnt=QFont (fList[1],fList[2].toInt(),fList[3].toInt(),fList[4].toInt());
			fnt.setUnderline(fList[5].toInt());
			fnt.setStrikeOut(fList[6].toInt());
			ag->setTitleFont(fnt);
			}
		else if (s.contains ("AxesTitles",TRUE)>0)
			{
				QStringList legend=QStringList::split ("\t",s,TRUE);
				for (i=0;i<4;i++)
					ag->setAxisTitle(i,legend[i+1]);
			}
		else if (s.contains ("AxesTitleColors",TRUE)>0)
			{
				QStringList colors=QStringList::split ("\t",s,FALSE);
				ag->setAxesTitleColor(colors);
			}
		else if (s.contains ("AxesTitleAlignment",TRUE)>0)
			{
				QStringList align=QStringList::split ("\t",s,FALSE);
				ag->setAxesTitlesAlignment(align);
			}
		else if (s.contains ("ScaleFont",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			QFont fnt=QFont (fList[1],fList[2].toInt(),fList[3].toInt(),fList[4].toInt());
			fnt.setUnderline(fList[5].toInt());
			fnt.setStrikeOut(fList[6].toInt());

			int axis=(fList[0].right(1)).toInt();
			ag->setAxisTitleFont(axis,fnt);
			}
		else if (s.contains ("AxisFont",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			QFont fnt=QFont (fList[1],fList[2].toInt(),fList[3].toInt(),fList[4].toInt());
			fnt.setUnderline(fList[5].toInt());
			fnt.setStrikeOut(fList[6].toInt());

			int axis=(fList[0].right(1)).toInt();
			ag->setAxisFont(axis,fnt);
			}
		else if (s.contains ("LabelsFormat",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
				
			if (fileVersion < 6.4)
				{//insure backwards compatibility
				for (i=0; i<4; i++)
					{			
					QString fmt = fList[2*i + 1];	
					if ( fmt == "0")
						fList[2*i + 1] = "1";
					else if ( fmt == "1")
						fList[2*i  + 1] = "2";
					else if ( fmt == "2")
						fList[2*i + 1] = "0";
					}					
				}
			
			ag->setLabelsFormat(fList);
			}
		else if (s.contains ("LabelsRotation",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->setAxisLabelRotation(QwtPlot::xBottom, fList[1].toInt());
			ag->setAxisLabelRotation(QwtPlot::xTop, fList[2].toInt());
			}
		else if (s.contains ("CanvasFrame",TRUE)>0)
			{
			QStringList list=QStringList::split ("\t",s,TRUE);
			ag->drawCanvasFrame(list);
			}
		else if (s.contains ("Legend",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->insertLegend(fList);
			}
		else if (s.contains ("textMarker",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->insertTextMarker(fList);
			}
		else if (s.contains ("lineMarker",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->insertLineMarker(fList);
			}
		else if (s.contains ("ImageMarker",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->insertImageMarker(fList);
			}
		else if (s.contains ("FitID",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->setFitID(fList[1].toInt());
			}
		else if (s.contains ("AxisType",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			ag->setAxesType(fList);
			}
		else if (s.contains ("AxesTickLabelsCol",TRUE)>0)
			{
			fList=QStringList::split ("\t",s,TRUE);
			QValueList<int> axesTypes = ag->axesType();
			for (i=0; i<4; i++)
				{
				QString colName = fList[i+1];
				WksheetWindow *nw = app->table(colName);
				ag->setAxisTickLabels(i, axesTypes[i], colName, nw);
				}
			}
	}

if (plot->overlapsLayers(ag))
	plot->makeTransparentLayer(ag);

ag->setIgnoreResizeEvents(false);
ag->updateScale();
plot->connectLayer(ag);
}

void ApplicationWindow::openSurfacePlot(ApplicationWindow* app, const QStringList &list)
{
			QStringList fList=QStringList::split ("\t",list[0],TRUE);
			QString caption=fList[0];
			QString date=fList[1];

			fList=QStringList::split ("\t",list[2],FALSE );

			Graph3D *plot=0;

			if (fList[1].contains("(X),",TRUE) == 0)
				plot=app->newPlot3D(caption, fList[1],fList[2].toDouble(),fList[3].toDouble(),
								fList[4].toDouble(),fList[5].toDouble(),
								fList[6].toDouble(),fList[7].toDouble());
			else if (fList[1].contains("(Z)",TRUE) > 0)
				plot=app->dataPlotXYZ(caption, fList[1],fList[2].toDouble(),fList[3].toDouble(),
								fList[4].toDouble(),fList[5].toDouble(),fList[6].toDouble(),fList[7].toDouble());
			else //Ribbon plot
				plot=app->dataPlot3D(caption, fList[1],fList[2].toDouble(),fList[3].toDouble(),
								fList[4].toDouble(),fList[5].toDouble(),fList[6].toDouble(),fList[7].toDouble());

			if (!plot)
				return;

			QString sg = list[1];
			if (sg.contains ("minimized",TRUE)>0)
				{
				plot->showMinimized();
				app->setListView(caption,"Minimized");
				}
			else if (sg.contains ("maximized",TRUE)>0)
				{
				plot->showNormal();
				app->wmax=(QWidget*)plot;
				}
			else
				{
				QStringList geometry=QStringList::split ("\t",sg,FALSE );
				int x=geometry[1].toInt();
				int y=geometry[2].toInt();
				int width=geometry[3].toInt();
				int height=geometry[4].toInt();
				plot->showNormal();
				plot->parentWidget()->setGeometry(x,y,width,height);

				if (geometry[5]=="active")
					app->aw=(QWidget*)plot;

				if (geometry[5]=="hidden")
					{
					app->hiddenWindows->append(plot);
					plot->hide();
					app->setListView(caption,"Hidden");
					}
				else
					app->setListView(caption,"Normal");
				}

			fList=QStringList::split ("\t",list[3],FALSE );
			plot->setStyle(fList);

			fList=QStringList::split ("\t", list[4],FALSE );
			plot->setGrid(fList[1].toInt());

			fList=QStringList::split ("\t",list[5],FALSE );
			plot->setTitle(fList);

			fList=QStringList::split ("\t",list[6],FALSE );
			plot->setColors(fList);

			fList=QStringList::split ("\t",list[7],FALSE );
			plot->setAxesLabels(fList);

			fList=QStringList::split ("\t",list[8],FALSE );
			plot->setTicks(fList);

			fList=QStringList::split ("\t",list[9],FALSE );
			plot->setTickLengths(fList);

			fList=QStringList::split ("\t",list[10],FALSE );
			plot->setOptions(fList);

			fList=QStringList::split ("\t",list[11],FALSE );
			plot->setNumbersFont(fList);

			fList=QStringList::split ("\t",list[12],FALSE );
			plot->setXAxisLabelFont(fList);

			fList=QStringList::split ("\t",list[13],FALSE );
			plot->setYAxisLabelFont(fList);

			fList=QStringList::split ("\t",list[14],FALSE );
			plot->setZAxisLabelFont(fList);

			fList=QStringList::split ("\t",list[15],FALSE );
			plot->setRotation(fList[1].toDouble(),fList[2].toDouble(),fList[3].toDouble());

			fList=QStringList::split ("\t",list[16],FALSE );
			plot->setZoom(fList[1].toDouble());

			fList=QStringList::split ("\t",list[17],FALSE );
			plot->setScale(fList[1].toDouble(),fList[2].toDouble(),fList[3].toDouble());

			fList=QStringList::split ("\t",list[18],FALSE );
			plot->setShift(fList[1].toDouble(),fList[2].toDouble(),fList[3].toDouble());

			if (list.count()>19)
				{//added beginning with version 0.5.0
				fList=QStringList::split ("\t",list[19],FALSE );
				plot->setMeshLineWidth(fList[1].toDouble());
				}

			plot->update();
			app->setListViewDate(caption,date);
			app->setListViewSize(caption,plot->sizeToString());
}

void ApplicationWindow::copyActiveLayer()
{
copiedLayer=TRUE;

MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
		return;

Graph *g= (Graph*)plot->activeGraph();
delete lastCopiedLayer;
lastCopiedLayer = new Graph (0, 0, WDestructiveClose);
clone(g, lastCopiedLayer);
lastCopiedLayer->setGeometry(0,0,g->width(), g->height());
g->copyImage();
}


void ApplicationWindow::showAnalysisDialog()
{
analysisDialog *ad=new analysisDialog(0,"Curve :", "analysisDialog",TRUE, WDestructiveClose);
connect (ad,SIGNAL(options(const QString&)),
			 this,SLOT(analyzeCurve(const QString&)));

QStringList curvesOnPlot=activeGraph->plotCurvesList();
ad->setCurveNames(curvesOnPlot);
ad->showNormal();
ad->setActiveWindow();
}

void ApplicationWindow::analyzeCurve(const QString& curve)
{
QStringList curves=activeGraph->plotCurvesList();
int curveID=curves.findIndex(curve);
long curveKey=activeGraph->curveKey(curveID);
int points=activeGraph->selectedPoints(curveKey);
analyzeCurve(analysisType, curveKey, points);
}

void ApplicationWindow::analyzeCurve(const QString& whichFit, long curveKey, int points)
{
if(points == 0)
	{
	QMessageBox::warning(this,"QtiPlot - Warning","The selected curve is empty!");
	return;
	}

QString result="";

	if (whichFit=="fitLinear") // Linear fit
		{
		if(points<2)
			{
			QMessageBox::warning(this,"QtiPlot - Warning","You need at least 2 points on the curve for linear fitting!");
			return;
			}
		result=activeGraph->fitCurveLinear(curveKey);
		}
	else if(whichFit=="fitExpDecay")
		{
		if(points<3)
			{
			QMessageBox::warning(this,"QtiPlot - Warning","You need at least 3 points on the curve for exponential fitting!");
			return;
			}
		result=activeGraph->fitExpDecay(curveKey);
		}
	else if(whichFit=="fitExpGrowth")
		{
		if(points<3)
		{
		QMessageBox::warning(this,"QtiPlot - Warning","You need at least 3 points on the curve for exponential fitting!");
		return;
		}
		result=activeGraph->fitExpGrowth(curveKey);
		}
	else if(whichFit=="fitGauss")
		{
		if(points<4)
		{
		QMessageBox::warning(this,"QtiPlot - Warning","You need at least 4 points on the curve for Gauss fitting!");
		return;
		}
		result=activeGraph->fitGauss(curveKey);
		}
	else if(whichFit=="fitLorentz")
		{
		if(points<4)
		{
		QMessageBox::warning(this,"QtiPlot - Warning","You need at least 4 points on the curve for Lorentz fitting!");
		return;
		}
		result=activeGraph->fitLorentz(curveKey);
		}
	else if(whichFit=="interlinear")
		{
		if(points<3)
			{
			QMessageBox::warning(this,"QtiPlot - Warning","You need at least 3 points to interpolate!");
			return;
			}
		activeGraph->interpolation(curveKey,whichFit);
		}
	else if(whichFit=="intercubic")
		{
		if(points<4)
			{
			QMessageBox::warning(this,"QtiPlot - Warning","You need at least 4 points to interpolate!");
			return;
			}
		activeGraph->interpolation(curveKey,whichFit);
		}
	else if(whichFit=="akima")
		{
		if(points<5)
			{
			QMessageBox::warning(this,"QtiPlot - Warning","You need at least 5 points to interpolate!");
			return;
			}
		activeGraph->interpolation(curveKey,whichFit);
		}
	else if(whichFit=="differentiate")
		{
		activeGraph->diffCurve(curveKey);
		WksheetWindow* w=table(tableWindows.last());
		QStringList list;
		list<<w->caption()+"_derivative";
		MultiLayer* d=multilayerPlot(w,list,0);
		d->setFocus();
		}
	else if(whichFit=="fft_for"|| whichFit=="fft_inv")
		{
		activeGraph->setFitID(++fitNumber);
		activeGraph->fft(curveKey,whichFit);
		}

	if (whichFit=="fitLinear" || whichFit=="fitExpDecay" || whichFit=="fitExpGrowth" || whichFit=="fitGauss" || whichFit=="fitLorentz")
		{
		if (!result.isEmpty())
			{
			logInfo+=result;
			if (resultsOn)
				resultsOn = !resultsOn;
			showResults();

			activeGraph->setFitID(++fitNumber);
			activeGraph->setFocus();
			emit modified();

			if (!aw)
				return;

			aw->setActiveWindow();
			((MultiLayer*)aw)->updateTransparency();
			}
		}

	if (whichFit=="interlinear" || whichFit=="intercubic" || whichFit=="akima")
		{
		activeGraph->setFitID(++fitNumber);
		activeGraph->setFocus();
		emit modified();

		if (!aw)
				return;

		aw->setActiveWindow();
		((MultiLayer*)aw)->updateTransparency();
		}
}

void ApplicationWindow::analysis(const QString& whichFit)
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
		return;

Graph* g = (Graph*)plot->activeGraph();
if ( g )
	{
	analysisType=whichFit;
	if (g->curves()==0)
		{
		QMessageBox::warning(this,"QtiPlot - Warning",
				"There are no curves available on this plot!");
		return;
		}
	else if(g->curves()>1)
		{
		activeGraph=g;
		aw=(QWidget*)plot;
		long selectedCurve=g->selectedCurveID();
		if (selectedCurve > 0) // a curve is selected
				analyzeCurve(whichFit, selectedCurve, g->selectedPoints(selectedCurve));
		else
				showAnalysisDialog();
		}
	else if (g->curves()==1)
		{
		activeGraph=g;
		aw=(QWidget*)plot;
		long selectedCurve=g->selectedCurveID();
		if (selectedCurve > 0) // a curve is selected
			analyzeCurve(whichFit, selectedCurve,g->selectedPoints(selectedCurve));
		else
			analyzeCurve(whichFit,g->curveKey(0),g->curveDataSize(0));
		}
	}
}

void ApplicationWindow::pickDataTool( QAction* action )
{
	if (!action)
		return;

	if (action == btnCursor)
	{
		if (selectOn)
			showRangeSelectors();
		else if (pickerOn)
			showPicker();
		else if (movePointsOn)
			movePoints();
		else if (removePointsOn)
			removePoints();
		else if (zoomOn)
			zoom();
		else if (drawLineOn)
			drawLine();
		else if (textOn)
			addText();

		showCursor();
	}

	else if (action == btnSelect)
	{
		if (cursorOn)
			showCursor();
		else if (pickerOn)
			showPicker();
		else if (movePointsOn)
			movePoints();
		else if (removePointsOn)
			removePoints();
		else if (zoomOn)
			zoom();
		else if (drawLineOn)
			drawLine();
		else if (textOn)
			addText();
	
		showRangeSelectors();
	}

	else if (action == btnPicker)
	{
		if (cursorOn)
			showCursor();
		else if (selectOn)
			showRangeSelectors();
		else if (movePointsOn)
			movePoints();
		else if (removePointsOn)
			removePoints();
		else if (zoomOn)
			zoom();
		else if (drawLineOn)
			drawLine();
		else if (textOn)
			addText();

		showPicker();
	}

	else if (action == btnMovePoints)
	{
		if (cursorOn)
			showCursor();
		else if (selectOn)
			showRangeSelectors();
		else if (pickerOn)
			showPicker();
		else if (removePointsOn)
			removePoints();
		else if (zoomOn)
			zoom();
		else if (drawLineOn)
			drawLine();
		else if (textOn)
			addText();

		movePoints();
	}

	else if (action == btnRemovePoints)
	{
		if (cursorOn)
			showCursor();
		else if (selectOn)
			showRangeSelectors();
		else if (pickerOn)
			showPicker();
		else if (movePointsOn)
			movePoints();
		else if (zoomOn)
			zoom();
		else if (drawLineOn)
			drawLine();
		else if (textOn)
			addText();

		removePoints();
	}

	else if (action == btnZoom)
	{
		if (cursorOn)
			showCursor();
		else if (selectOn)
			showRangeSelectors();
		else if (pickerOn)
			showPicker();
		else if (movePointsOn)
			movePoints();
		else if (removePointsOn)
					removePoints();
		else if (drawLineOn)
			drawLine();
		else if (textOn)
			addText();

		zoom();
	}

	else if (action == btnPointer)
	{
		if (cursorOn)
			showCursor();
		else if (selectOn)
			showRangeSelectors();
		else if (pickerOn)
			showPicker();
		else if (movePointsOn)
			movePoints();
		else if (removePointsOn)
			removePoints();
		else if (zoomOn)
			zoom();
		else if (drawLineOn)
			drawLine();
		else if (textOn)
			addText();
	}
	else if (action == btnLine)
	{
		if (cursorOn)
			showCursor();
		else if (selectOn)
			showRangeSelectors();
		else if (pickerOn)
			showPicker();
		else if (movePointsOn)
			movePoints();
		else if (removePointsOn)
					removePoints();
		else if (zoomOn)
			zoom();
		else if (textOn)
			addText();

		drawLine();
	}
}

void ApplicationWindow::connectSurfacePlot(Graph3D *plot)
{
connect (plot,SIGNAL(showContextMenu()),this,SLOT(showWindowContextMenu()));
connect (plot,SIGNAL(showOptionsDialog()),this,SLOT(showPlot3dDialog()));
connect (plot,SIGNAL(closedGraph(QWidget*)),this, SLOT(closeWindow(QWidget*)));
connect (plot,SIGNAL(hiddenPlot(QWidget*)),this, SLOT(hideWindow(QWidget*)));
connect (plot,SIGNAL(resizedPlot(QWidget*)),this, SLOT(resizedWindow(QWidget*)));
connect (plot,SIGNAL(modified()),this, SIGNAL(modified()));
connect (plot,SIGNAL(showWorksheet(const QString&)),this, SLOT(showWorksheet(const QString&)));
connect (plot,SIGNAL(custom3DActions(QWidget*)),this, SLOT(custom3DActions(QWidget*)));

plot->askOnCloseEvent(confirmClosePlot3D);
}

void ApplicationWindow::connectMultilayerPlot(MultiLayer *g)
{
connect (g,SIGNAL(showTextDialog()),this,SLOT(showTextDialog()));
connect (g,SIGNAL(showPlotDialog()),this,SLOT(showLayoutDialog()));
connect (g,SIGNAL(showXAxesDialog()),this,SLOT(showXAxesDialog()));
connect (g,SIGNAL(showYAxesDialog()),this,SLOT(showYAxesDialog()));
connect (g,SIGNAL(showWindowContextMenu()),this,SLOT(showWindowContextMenu()));
connect (g,SIGNAL(showCurvesDialog()),this,SLOT(showCurvesDialog()));
connect (g,SIGNAL(addCurveToGraph(Graph*,const QString&)),this,SLOT(addCurve(Graph*,const QString&)));
connect (g,SIGNAL(drawTextOff()),this, SLOT(disableAddText()));
connect (g,SIGNAL(showXAxisTitleDialog()),this,SLOT(showXAxisTitleDialog()));
connect (g,SIGNAL(showYAxisTitleDialog()),this,SLOT(showYAxisTitleDialog()));
connect (g,SIGNAL(showRightAxisTitleDialog()),this,SLOT(showRightAxisTitleDialog()));
connect (g,SIGNAL(showTopAxisTitleDialog()),this,SLOT(showTopAxisTitleDialog()));
connect (g,SIGNAL(showMarkerPopupMenu()),this,SLOT(showMarkerPopupMenu()));
connect (g,SIGNAL(closedPlot(QWidget*)),this, SLOT(closeWindow(QWidget*)));
connect (g,SIGNAL(hidePlot(QWidget*)),this, SLOT(hideWindow(QWidget*)));
connect (g,SIGNAL(resizedPlot(QWidget*)),this, SLOT(resizedWindow(QWidget*)));
connect (g,SIGNAL(cursorInfo(const QString&)),info,SLOT(setText(const QString&)));
connect (g,SIGNAL(showImageDialog()),this,SLOT(showImageDialog()));
connect (g,SIGNAL(createTablePlot(const QString&,int,int,const QString&)),this,SLOT(newWrksheetPlot(const QString&,int,int,const QString&)));
connect (g,SIGNAL(createTable(const QString&,int,int,const QString&)),this,SLOT(newWrksheet(const QString&,int,int,const QString&)));
connect (g,SIGNAL(showPieDialog()),this,SLOT(showPieDialog()));
connect (g,SIGNAL(viewTitleDialog()),this,SLOT(showTitleDialog()));
connect (g,SIGNAL(modifiedPlot()),this,SLOT(modifiedProject()));
connect (g,SIGNAL(showLineDialog()),this,SLOT(showLineDialog()));
connect (g,SIGNAL(updateTable(const QString&,int,const QString&)),this,SLOT(updateTable(const QString&,int,const QString&)));
connect (g,SIGNAL(clearCell(const QString&,double)),this,SLOT(clearCellFromTable(const QString&,double)));
connect (g,SIGNAL(showGeometryDialog()),this,SLOT(showPlotGeometryDialog()));
connect (g,SIGNAL(pasteMarker()),this,SLOT(pasteSelection()));
connect (g,SIGNAL(showGraphContextMenu()),this,SLOT(showGraphContextMenu()));

connect (g,SIGNAL(createIntensityTable(const QPixmap&)),
			this,SLOT(createIntensityTable(const QPixmap&)));
connect (g,SIGNAL(createHistogramTable(const QString&,int,int,const QString&)),
		this,SLOT(showHistogramTable(const QString&,int,int,const QString&)));

g->askOnCloseEvent(confirmClosePlot2D);
}

void ApplicationWindow::connectTable(WksheetWindow* w)
{
connect (w,SIGNAL(hiddenTable(QWidget*)),this, SLOT(hideWindow(QWidget*)));
connect (w,SIGNAL(resizedTable(QWidget*)),this, SLOT(resizedWindow(QWidget*)));
connect( w,SIGNAL(closedTable(QWidget*)),this, SLOT(closeWindow(QWidget*)));
connect (w,SIGNAL(removedCol(QString&)),this,SLOT(removeColFromGraphs(QString&)));
connect (w,SIGNAL(modifiedData(const QString&)),this,SLOT(updateCurveInGraphs(const QString&)));
connect( w,SIGNAL(plotCol(WksheetWindow*,const QStringList&, int)),this, SLOT(multilayerPlot(WksheetWindow*,const QStringList&, int)));
connect (w,SIGNAL(modifiedTable(QWidget*)),this,SLOT(modifiedProject(QWidget*)));
connect (w,SIGNAL(optionsDialog()),this,SLOT(showTableOptionsDialog()));
connect (w,SIGNAL(colValuesDialog()),this,SLOT(showColumnValuesDialog()));
connect (w,SIGNAL(rowMenu(int)),this,SLOT(showRowMenu(int)));
connect (w,SIGNAL(colMenu(int)),this,SLOT(showColMenu(int)));
connect (w,SIGNAL(showContextMenu()),this,SLOT(showWindowContextMenu()));
connect (w,SIGNAL(changedColHeader(const QString&,const QString&)),this,SLOT(updateCurvesDialogs(const QString&,const QString&)));
connect (w,SIGNAL(createTable(const QString&,int,int,const QString&)),this,SLOT(newWrksheet(const QString&,int,int,const QString&)));

//3d plots
connect( w,SIGNAL(plot3DRibbon(WksheetWindow*,const QString&)),this, SLOT(dataPlot3D(WksheetWindow*,const QString&)));
connect( w,SIGNAL(plotXYZ(WksheetWindow*,const QString&, int)),this, SLOT(dataPlotXYZ(WksheetWindow*,const QString&, int)));

w->askOnCloseEvent(confirmCloseTable);
}

void ApplicationWindow::setAppColors(const QColor& wc,const QColor& pc,const QColor& tpc)
{
if (workspaceColor != wc)
	{
	workspaceColor = wc;
	ws->setPaletteBackgroundColor (wc);
	}
	
if (panelsColor != pc)
	{
	panelsColor = pc;
	lv->setPaletteBackgroundColor (pc);
	results->setPaper ( QBrush (pc, SolidPattern ) );
	}
	
if (panelsTextColor != tpc)
	{
	panelsTextColor = tpc;
	lv->setPaletteForegroundColor (tpc);
	
	results->setPaletteForegroundColor (tpc);
	if (results->isVisible())
		{
		results->hide();
		results->show();
		}
	}
}

void ApplicationWindow::updateVectorsLayout(int curve, int colorIndex, int width, 
						int arrowLength, int arrowAngle, bool filled, 
						const QString& xEndColName, const QString& yEndColName)
{
MultiLayer* plot = (MultiLayer*)ws->activeWindow();
if (!plot || plotWindows.contains(plot->caption())<=0)
	return;

Graph* g = (Graph*)plot->activeGraph();
if (!g)
	return;

WksheetWindow* w=table(xEndColName);
if (!w)
	return;

g->updateVectorsLayout(w, curve, colorIndex, width, arrowLength, arrowAngle,
								filled, xEndColName, yEndColName);
g->setFocus();
g->setActiveWindow();
}


ApplicationWindow::~ApplicationWindow()
{
if (lastCopiedLayer)
	delete lastCopiedLayer;

if (fd && fd->isVisible())
	fd->close();
}
